<?php

 $gtClassMap = array(
 
    'gtCodeSnippet'                 => 'gtCodeSnippet.php',
    'gtTestSubject'                 => 'gtTestSubject.php',
    'gtFunction'                    => 'gtFunction.php',
    'gtMethod'                      => 'gtMethod.php',
    'gtTestCaseWriter'              => 'gtTestCaseWriter.php',
    'gtText'                        => 'gtText.php',
 
 
 
    'gtCommandLineOptions'          => 'setup/gtCommandLineOptions.php',
    'gtOptionalSections'            => 'setup/gtOptionalSections.php',
    'gtMissingArgumentException'    => 'setup/exceptions/gtMissingArgumentException.php',
    'gtUnknownOptionException'      => 'setup/exceptions/gtUnknownOptionException.php',
    'gtUnknownSectionException'      => 'setup/exceptions/gtUnknownSectionException.php',
    'gtMissingOptionsException'     => 'setup/exceptions/gtMissingOptionsException.php',
 
    'gtPreCondition'                => 'setup/gtPreCondition.php',
    'gtPreConditionList'            => 'setup/gtPreConditionList.php',
    'gtIsSpecifiedTestType'         => 'setup/preconditions/gtIsSpecifiedTestType.php',
    'gtIfClassHasMethod'            => 'setup/preconditions/gtIfClassHasMethod.php',
    'gtIsSpecifiedFunctionOrMethod' => 'setup/preconditions/gtIsSpecifiedFunctionOrMethod.php',
    'gtIsValidClass'                => 'setup/preconditions/gtIsValidClass.php',
    'gtIsValidMethod'               => 'setup/preconditions/gtIsValidMethod.php',
    'gtIsValidFunction'             => 'setup/preconditions/gtIsValidFunction.php',
 
 
     'gtTestCase'                   => 'testcase/gtTestCase.php',
     'gtVariationTestCase'          => 'testcase/gtVariationTestCase.php',
     'gtVariationTestCaseFunction'  => 'testcase/gtVariationTestCaseFunction.php',
     'gtVariationTestCaseMethod'    => 'testcase/gtVariationTestCaseMethod.php',
 
     'gtBasicTestCase'              => 'testcase/gtBasicTestCase.php',
     'gtBasicTestCaseFunction'      => 'testcase/gtBasicTestCaseFunction.php',
     'gtBasicTestCaseMethod'        => 'testcase/gtBasicTestCaseMethod.php',
 
     'gtErrorTestCase'              => 'testcase/gtErrorTestCase.php',
     'gtErrorTestCaseFunction'      => 'testcase/gtErrorTestCaseFunction.php',
     'gtErrorTestCaseMethod'        => 'testcase/gtErrorTestCaseMethod.php',
 
     'gtVariationContainer'         => 'testcase/gtVariationContainer.php',
     'gtVariationContainerMethod'   => 'testcase/gtVariationContainerMethod.php',
     'gtVariationContainerFunction' => 'testcase/gtVariationContainerFunction.php',
 );
?>