EXPORTS
tidyBufInit
tidyBufAlloc
tidyBufCheckAlloc
tidyBufFree
tidyBufClear
tidyBufAttach
tidyBufDetach
tidyBufAppend
tidyBufPutByte
tidyBufPopByte
tidyBufGetByte
tidyBufEndOfInput
tidyBufUngetByte
tidyCreate
tidyRelease
tidySetAppData
tidyGetAppData
tidyReleaseDate
tidyStatus
tidyDetectedHtmlVersion
tidyDetectedXhtml
tidyDetectedGenericXml
tidyErrorCount
tidyWarningCount
tidyAccessWarningCount
tidyConfigErrorCount
tidyLoadConfig
tidyLoadConfigEnc
tidyFileExists
tidySetCharEncoding
tidySetOptionCallback
tidyOptGetIdForName
tidyGetOptionList
tidyGetNextOption
tidyGetOption
tidyGetOptionByName
tidyOptGetId
tidyOptGetName
tidyOptGetType
tidyOptIsReadOnly
tidyOptGetCategory
tidyOptGetDefault
tidyOptGetDefaultInt
tidyOptGetDefaultBool
tidyOptGetPickList
tidyOptGetNextPick
tidyOptGetValue
tidyOptSetValue
tidyOptParseValue
tidyOptGetInt
tidyOptSetInt
tidyOptGetBool
tidyOptSetBool
tidyOptResetToDefault
tidyOptResetAllToDefault
tidyOptSnapshot
tidyOptResetToSnapshot
tidyOptDiffThanDefault
tidyOptDiffThanSnapshot
tidyOptCopyConfig
tidyOptGetEncName
tidyOptGetCurrPick
tidyOptGetDeclTagList
tidyOptGetNextDeclTag
tidyInitSource
tidyGetByte
tidyUngetByte
tidyIsEOF
tidyInitSink
tidyPutByte
tidySetReportFilter
tidySetErrorFile
tidySetErrorBuffer
tidySetErrorSink
tidySetMallocCall
tidySetReallocCall
tidySetFreeCall
tidySetPanicCall
tidyParseFile
tidyParseStdin
tidyParseString
tidyParseBuffer
tidyParseSource
tidyCleanAndRepair
tidyRunDiagnostics
tidySaveFile
tidySaveStdout
tidySaveBuffer
tidySaveString
tidySaveSink
tidyOptSaveFile
tidyOptSaveSink
tidyErrorSummary
tidyGeneralInfo
tidyGetRoot
tidyGetHtml
tidyGetHead
tidyGetBody
tidyGetParent
tidyGetChild
tidyGetNext
tidyGetPrev
tidyAttrFirst
tidyAttrNext
tidyAttrName
tidyAttrValue
tidyNodeGetType
tidyNodeGetName
tidyNodeIsText
tidyNodeIsProp
tidyNodeIsHeader
tidyNodeHasText
tidyNodeGetText
tidyNodeGetId
tidyNodeLine
tidyNodeColumn
tidyNodeIsHTML
tidyNodeIsHEAD
tidyNodeIsTITLE
tidyNodeIsBASE
tidyNodeIsMETA
tidyNodeIsBODY
tidyNodeIsFRAMESET
tidyNodeIsFRAME
tidyNodeIsIFRAME
tidyNodeIsNOFRAMES
tidyNodeIsHR
tidyNodeIsH1
tidyNodeIsH2
tidyNodeIsPRE
tidyNodeIsLISTING
tidyNodeIsP
tidyNodeIsUL
tidyNodeIsOL
tidyNodeIsDL
tidyNodeIsDIR
tidyNodeIsLI
tidyNodeIsDT
tidyNodeIsDD
tidyNodeIsTABLE
tidyNodeIsCAPTION
tidyNodeIsTD
tidyNodeIsTH
tidyNodeIsTR
tidyNodeIsCOL
tidyNodeIsCOLGROUP
tidyNodeIsBR
tidyNodeIsA
tidyNodeIsLINK
tidyNodeIsB
tidyNodeIsI
tidyNodeIsSTRONG
tidyNodeIsEM
tidyNodeIsBIG
tidyNodeIsSMALL
tidyNodeIsPARAM
tidyNodeIsOPTION
tidyNodeIsOPTGROUP
tidyNodeIsIMG
tidyNodeIsMAP
tidyNodeIsAREA
tidyNodeIsNOBR
tidyNodeIsWBR
tidyNodeIsFONT
tidyNodeIsLAYER
tidyNodeIsSPACER
tidyNodeIsCENTER
tidyNodeIsSTYLE
tidyNodeIsSCRIPT
tidyNodeIsNOSCRIPT
tidyNodeIsFORM
tidyNodeIsTEXTAREA
tidyNodeIsBLOCKQUOTE
tidyNodeIsAPPLET
tidyNodeIsOBJECT
tidyNodeIsDIV
tidyNodeIsSPAN
tidyNodeIsINPUT
tidyNodeIsQ
tidyNodeIsLABEL
tidyNodeIsH3
tidyNodeIsH4
tidyNodeIsH5
tidyNodeIsH6
tidyNodeIsADDRESS
tidyNodeIsXMP
tidyNodeIsSELECT
tidyNodeIsBLINK
tidyNodeIsMARQUEE
tidyNodeIsEMBED
tidyNodeIsBASEFONT
tidyNodeIsISINDEX
tidyNodeIsS
tidyNodeIsSTRIKE
tidyNodeIsU
tidyNodeIsMENU
tidyAttrGetId
tidyAttrIsEvent
tidyAttrIsProp
tidyAttrIsHREF
tidyAttrIsSRC
tidyAttrIsID
tidyAttrIsNAME
tidyAttrIsSUMMARY
tidyAttrIsALT
tidyAttrIsLONGDESC
tidyAttrIsUSEMAP
tidyAttrIsISMAP
tidyAttrIsLANGUAGE
tidyAttrIsTYPE
tidyAttrIsVALUE
tidyAttrIsCONTENT
tidyAttrIsTITLE
tidyAttrIsXMLNS
tidyAttrIsDATAFLD
tidyAttrIsWIDTH
tidyAttrIsHEIGHT
tidyAttrIsFOR
tidyAttrIsSELECTED
tidyAttrIsCHECKED
tidyAttrIsLANG
tidyAttrIsTARGET
tidyAttrIsHTTP_EQUIV
tidyAttrIsREL
tidyAttrIsOnMOUSEMOVE
tidyAttrIsOnMOUSEDOWN
tidyAttrIsOnMOUSEUP
tidyAttrIsOnCLICK
tidyAttrIsOnMOUSEOVER
tidyAttrIsOnMOUSEOUT
tidyAttrIsOnKEYDOWN
tidyAttrIsOnKEYUP
tidyAttrIsOnKEYPRESS
tidyAttrIsOnFOCUS
tidyAttrIsOnBLUR
tidyAttrIsBGCOLOR
tidyAttrIsLINK
tidyAttrIsALINK
tidyAttrIsVLINK
tidyAttrIsTEXT
tidyAttrIsSTYLE
tidyAttrIsABBR
tidyAttrIsCOLSPAN
tidyAttrIsROWSPAN
tidyAttrGetHREF
tidyAttrGetSRC
tidyAttrGetID
tidyAttrGetNAME
tidyAttrGetSUMMARY
tidyAttrGetALT
tidyAttrGetLONGDESC
tidyAttrGetUSEMAP
tidyAttrGetISMAP
tidyAttrGetLANGUAGE
tidyAttrGetTYPE
tidyAttrGetVALUE
tidyAttrGetCONTENT
tidyAttrGetTITLE
tidyAttrGetXMLNS
tidyAttrGetDATAFLD
tidyAttrGetWIDTH
tidyAttrGetHEIGHT
tidyAttrGetFOR
tidyAttrGetSELECTED
tidyAttrGetCHECKED
tidyAttrGetLANG
tidyAttrGetTARGET
tidyAttrGetHTTP_EQUIV
tidyAttrGetREL
tidyAttrGetOnMOUSEMOVE
tidyAttrGetOnMOUSEDOWN
tidyAttrGetOnMOUSEUP
tidyAttrGetOnCLICK
tidyAttrGetOnMOUSEOVER
tidyAttrGetOnMOUSEOUT
tidyAttrGetOnKEYDOWN
tidyAttrGetOnKEYUP
tidyAttrGetOnKEYPRESS
tidyAttrGetOnFOCUS
tidyAttrGetOnBLUR
tidyAttrGetBGCOLOR
tidyAttrGetLINK
tidyAttrGetALINK
tidyAttrGetVLINK
tidyAttrGetTEXT
tidyAttrGetSTYLE
tidyAttrGetABBR
tidyAttrGetCOLSPAN
tidyAttrGetROWSPAN
