struct _IO_FILE;
struct _IO_FILE *_coverage_fout ;
typedef struct _IO_FILE FILE;
extern int fflush(FILE *__stream ) ;
extern int fprintf(FILE * __restrict  __stream ,
                   char const   * __restrict  __format  , ...) ;
struct _zend_class_entry;
typedef struct _zend_class_entry zend_class_entry;
zend_class_entry __attribute__((__visibility__("default")))  *spl_ce_RecursiveArrayIterator ;
typedef unsigned int size_t;
extern int __attribute__((__visibility__("default")))  zend_declare_class_constant_long(zend_class_entry *ce ,
                                                                                        char const   *name ,
                                                                                        size_t name_length ,
                                                                                        long value ) ;
zend_class_entry __attribute__((__visibility__("default")))  *spl_ce_ArrayIterator ;
zend_class_entry __attribute__((__visibility__("default")))  *spl_ce_ArrayObject ;
extern zend_class_entry __attribute__((__visibility__("default")))  *spl_ce_Countable ;
extern void __attribute__((__visibility__("default")))  zend_class_implements(zend_class_entry *class_entry ,
                                                                              int num_interfaces 
                                                                              , ...) ;
struct _zval_struct;
typedef struct _zval_struct zval;
struct _zend_object_iterator;
typedef struct _zend_object_iterator zend_object_iterator;
zend_object_iterator *spl_array_get_iterator(zend_class_entry *ce ,
                                             zval *object , int by_ref ) ;
struct _zend_module_entry;
struct _zend_function_entry;
struct __anonstruct_internal_42 {
   struct _zend_function_entry  const  *builtin_functions ;
   struct _zend_module_entry *module ;
};
typedef unsigned int zend_uint;
struct __anonstruct_user_41 {
   char const   *filename ;
   zend_uint line_start ;
   zend_uint line_end ;
   char const   *doc_comment ;
   zend_uint doc_comment_len ;
};
union __anonunion_info_40 {
   struct __anonstruct_user_41 user ;
   struct __anonstruct_internal_42 internal ;
};
struct _zend_trait_precedence;
typedef struct _zend_trait_precedence zend_trait_precedence;
struct _zend_trait_alias;
typedef struct _zend_trait_alias zend_trait_alias;
struct _zend_unserialize_data;
typedef struct _zend_unserialize_data zend_unserialize_data;
struct _zend_serialize_data;
typedef struct _zend_serialize_data zend_serialize_data;
union _zend_function;
struct _zend_object_handlers;
typedef struct _zend_object_handlers zend_object_handlers;
typedef unsigned int zend_object_handle;
struct _zend_object_value {
   zend_object_handle handle ;
   zend_object_handlers const   *handlers ;
};
typedef struct _zend_object_value zend_object_value;
struct _zend_object_iterator_funcs;
typedef struct _zend_object_iterator_funcs zend_object_iterator_funcs;
struct _zend_class_iterator_funcs {
   zend_object_iterator_funcs *funcs ;
   union _zend_function *zf_new_iterator ;
   union _zend_function *zf_valid ;
   union _zend_function *zf_current ;
   union _zend_function *zf_key ;
   union _zend_function *zf_next ;
   union _zend_function *zf_rewind ;
};
typedef struct _zend_class_iterator_funcs zend_class_iterator_funcs;
typedef unsigned char zend_bool;
struct bucket;
typedef struct bucket Bucket;
typedef unsigned long ulong;
typedef unsigned int uint;
struct _hashtable {
   uint nTableSize ;
   uint nTableMask ;
   uint nNumOfElements ;
   ulong nNextFreeElement ;
   Bucket *pInternalPointer ;
   Bucket *pListHead ;
   Bucket *pListTail ;
   Bucket **arBuckets ;
   void (*pDestructor)(void *pDest ) ;
   zend_bool persistent ;
   unsigned char nApplyCount ;
   zend_bool bApplyProtection ;
};
typedef struct _hashtable HashTable;
struct _zend_class_entry {
   char type ;
   char const   *name ;
   zend_uint name_length ;
   struct _zend_class_entry *parent ;
   int refcount ;
   zend_uint ce_flags ;
   HashTable function_table ;
   HashTable properties_info ;
   zval **default_properties_table ;
   zval **default_static_members_table ;
   zval **static_members_table ;
   HashTable constants_table ;
   int default_properties_count ;
   int default_static_members_count ;
   union _zend_function *constructor ;
   union _zend_function *destructor ;
   union _zend_function *clone ;
   union _zend_function *__get ;
   union _zend_function *__set ;
   union _zend_function *__unset ;
   union _zend_function *__isset ;
   union _zend_function *__call ;
   union _zend_function *__callstatic ;
   union _zend_function *__tostring ;
   union _zend_function *serialize_func ;
   union _zend_function *unserialize_func ;
   zend_class_iterator_funcs iterator_funcs ;
   zend_object_value (*create_object)(zend_class_entry *class_type ) ;
   zend_object_iterator *(*get_iterator)(zend_class_entry *ce , zval *object ,
                                         int by_ref ) ;
   int (*interface_gets_implemented)(zend_class_entry *iface ,
                                     zend_class_entry *class_type ) ;
   union _zend_function *(*get_static_method)(zend_class_entry *ce ,
                                              char *method , int method_len ) ;
   int (*serialize)(zval *object , unsigned char **buffer , zend_uint *buf_len ,
                    zend_serialize_data *data ) ;
   int (*unserialize)(zval **object , zend_class_entry *ce ,
                      unsigned char const   *buf , zend_uint buf_len ,
                      zend_unserialize_data *data ) ;
   zend_class_entry **interfaces ;
   zend_uint num_interfaces ;
   zend_class_entry **traits ;
   zend_uint num_traits ;
   zend_trait_alias **trait_aliases ;
   zend_trait_precedence **trait_precedences ;
   union __anonunion_info_40 info ;
};
extern zend_class_entry __attribute__((__visibility__("default")))  *spl_ce_RecursiveIterator ;
struct _zend_arg_info;
struct _zend_function_entry {
   char const   *fname ;
   void (*handler)(int ht , zval *return_value , zval **return_value_ptr ,
                   zval *this_ptr , int return_value_used ) ;
   struct _zend_arg_info  const  *arg_info ;
   zend_uint num_args ;
   zend_uint flags ;
};
typedef struct _zend_function_entry zend_function_entry;
static zend_function_entry const   spl_funcs_RecursiveArrayIterator[3] ;
struct _zend_object_value;
static zend_object_value spl_array_object_new(zend_class_entry *class_type ) ;
extern void spl_register_sub_class(zend_class_entry **ppce ,
                                   zend_class_entry *parent_ce ,
                                   char *class_name ,
                                   zend_object_value (*ctor)(zend_class_entry *class_type ) ,
                                   zend_function_entry const   *function_list ) ;
struct _hashtable;
struct _zend_literal;
struct _zend_object_handlers {
   void (*add_ref)(zval *object ) ;
   void (*del_ref)(zval *object ) ;
   zend_object_value (*clone_obj)(zval *object ) ;
   zval *(*read_property)(zval *object , zval *member , int type ,
                          struct _zend_literal  const  *key ) ;
   void (*write_property)(zval *object , zval *member , zval *value ,
                          struct _zend_literal  const  *key ) ;
   zval *(*read_dimension)(zval *object , zval *offset , int type ) ;
   void (*write_dimension)(zval *object , zval *offset , zval *value ) ;
   zval **(*get_property_ptr_ptr)(zval *object , zval *member ,
                                  struct _zend_literal  const  *key ) ;
   zval *(*get)(zval *object ) ;
   void (*set)(zval **object , zval *value ) ;
   int (*has_property)(zval *object , zval *member , int has_set_exists ,
                       struct _zend_literal  const  *key ) ;
   void (*unset_property)(zval *object , zval *member ,
                          struct _zend_literal  const  *key ) ;
   int (*has_dimension)(zval *object , zval *member , int check_empty ) ;
   void (*unset_dimension)(zval *object , zval *offset ) ;
   HashTable *(*get_properties)(zval *object ) ;
   union _zend_function *(*get_method)(zval **object_ptr , char *method ,
                                       int method_len ,
                                       struct _zend_literal  const  *key ) ;
   int (*call_method)(char const   *method , int ht , zval *return_value ,
                      zval **return_value_ptr , zval *this_ptr ,
                      int return_value_used ) ;
   union _zend_function *(*get_constructor)(zval *object ) ;
   zend_class_entry *(*get_class_entry)(zval const   *object ) ;
   int (*get_class_name)(zval const   *object , char const   **class_name ,
                         zend_uint *class_name_len , int parent ) ;
   int (*compare_objects)(zval *object1 , zval *object2 ) ;
   int (*cast_object)(zval *readobj , zval *retval , int type ) ;
   int (*count_elements)(zval *object , long *count ) ;
   HashTable *(*get_debug_info)(zval *object , int *is_temp ) ;
   int (*get_closure)(zval *obj , zend_class_entry **ce_ptr ,
                      union _zend_function **fptr_ptr , zval **zobj_ptr ) ;
   HashTable *(*get_gc)(zval *object , zval ***table , int *n ) ;
};
zend_object_handlers spl_handler_ArrayObject ;
zend_object_handlers spl_handler_ArrayIterator ;
extern  __attribute__((__nothrow__)) void *memcpy(void * __restrict  __dest ,
                                                  void const   * __restrict  __src ,
                                                  size_t __n )  __attribute__((__nonnull__(1,2))) ;
extern zend_class_entry __attribute__((__visibility__("default")))  *zend_ce_serializable ;
extern zend_class_entry __attribute__((__visibility__("default")))  *spl_ce_SeekableIterator ;
extern zend_class_entry __attribute__((__visibility__("default")))  *zend_ce_arrayaccess ;
extern zend_class_entry __attribute__((__visibility__("default")))  *zend_ce_iterator ;
static zend_function_entry const   spl_funcs_ArrayIterator[25] ;
extern void spl_register_std_class(zend_class_entry **ppce , char *class_name ,
                                   zend_object_value (*ctor)(zend_class_entry *class_type ) ,
                                   zend_function_entry const   *function_list ) ;
typedef struct _zend_literal zend_literal;
static void spl_array_unset_property(zval *object , zval *member ,
                                     zend_literal const   *key ) ;
static int spl_array_has_property(zval *object , zval *member ,
                                  int has_set_exists ,
                                  zend_literal const   *key ) ;
static zval **spl_array_get_property_ptr_ptr(zval *object , zval *member ,
                                             zend_literal const   *key ) ;
static void spl_array_write_property(zval *object , zval *member , zval *value ,
                                     zend_literal const   *key ) ;
static zval *spl_array_read_property(zval *object , zval *member , int type ,
                                     zend_literal const   *key ) ;
static HashTable *spl_array_get_debug_info(zval *obj , int *is_temp ) ;
static HashTable *spl_array_get_properties(zval *object ) ;
int spl_array_object_count_elements(zval *object , long *count ) ;
static int spl_array_has_dimension(zval *object , zval *offset ,
                                   int check_empty ) ;
static void spl_array_unset_dimension(zval *object , zval *offset ) ;
static void spl_array_write_dimension(zval *object , zval *offset , zval *value ) ;
static zval *spl_array_read_dimension(zval *object , zval *offset , int type ) ;
static zend_object_value spl_array_object_clone(zval *zobject ) ;
extern zend_object_handlers __attribute__((__visibility__("default")))  *zend_get_std_object_handlers(void) ;
extern zend_class_entry __attribute__((__visibility__("default")))  *zend_ce_aggregate ;
static zend_function_entry const   spl_funcs_ArrayObject[23] ;
extern FILE *fopen(char const   * __restrict  __filename ,
                   char const   * __restrict  __modes ) ;
int zm_startup_spl_array(int type , int module_number ) 
{ zend_object_handlers __attribute__((__visibility__("default")))  *tmp ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "3162\n");
  fflush(_coverage_fout);
  }
  spl_register_std_class((zend_class_entry **)(& spl_ce_ArrayObject),
                         (char *)"ArrayObject", & spl_array_object_new,
                         spl_funcs_ArrayObject);
  {
  fprintf(_coverage_fout, "3163\n");
  fflush(_coverage_fout);
  }
  zend_class_implements((zend_class_entry *)spl_ce_ArrayObject, 1,
                        zend_ce_aggregate);
  {
  fprintf(_coverage_fout, "3164\n");
  fflush(_coverage_fout);
  }
  zend_class_implements((zend_class_entry *)spl_ce_ArrayObject, 1,
                        zend_ce_arrayaccess);
  {
  fprintf(_coverage_fout, "3165\n");
  fflush(_coverage_fout);
  }
  zend_class_implements((zend_class_entry *)spl_ce_ArrayObject, 1,
                        zend_ce_serializable);
  {
  fprintf(_coverage_fout, "3166\n");
  fflush(_coverage_fout);
  }
  tmp = zend_get_std_object_handlers();
  {
  fprintf(_coverage_fout, "3167\n");
  fflush(_coverage_fout);
  }
  memcpy((void */* __restrict  */)(& spl_handler_ArrayObject),
         (void const   */* __restrict  */)tmp, sizeof(zend_object_handlers ));
  {
  fprintf(_coverage_fout, "3168\n");
  fflush(_coverage_fout);
  }
  spl_handler_ArrayObject.clone_obj = & spl_array_object_clone;
  {
  fprintf(_coverage_fout, "3169\n");
  fflush(_coverage_fout);
  }
  spl_handler_ArrayObject.read_dimension = & spl_array_read_dimension;
  {
  fprintf(_coverage_fout, "3170\n");
  fflush(_coverage_fout);
  }
  spl_handler_ArrayObject.write_dimension = & spl_array_write_dimension;
  {
  fprintf(_coverage_fout, "3171\n");
  fflush(_coverage_fout);
  }
  spl_handler_ArrayObject.unset_dimension = & spl_array_unset_dimension;
  {
  fprintf(_coverage_fout, "3172\n");
  fflush(_coverage_fout);
  }
  spl_handler_ArrayObject.has_dimension = & spl_array_has_dimension;
  {
  fprintf(_coverage_fout, "3173\n");
  fflush(_coverage_fout);
  }
  spl_handler_ArrayObject.count_elements = & spl_array_object_count_elements;
  {
  fprintf(_coverage_fout, "3174\n");
  fflush(_coverage_fout);
  }
  spl_handler_ArrayObject.get_properties = & spl_array_get_properties;
  {
  fprintf(_coverage_fout, "3175\n");
  fflush(_coverage_fout);
  }
  spl_handler_ArrayObject.get_debug_info = & spl_array_get_debug_info;
  {
  fprintf(_coverage_fout, "3176\n");
  fflush(_coverage_fout);
  }
  spl_handler_ArrayObject.read_property = & spl_array_read_property;
  {
  fprintf(_coverage_fout, "3177\n");
  fflush(_coverage_fout);
  }
  spl_handler_ArrayObject.write_property = & spl_array_write_property;
  {
  fprintf(_coverage_fout, "3178\n");
  fflush(_coverage_fout);
  }
  spl_handler_ArrayObject.get_property_ptr_ptr = & spl_array_get_property_ptr_ptr;
  {
  fprintf(_coverage_fout, "3179\n");
  fflush(_coverage_fout);
  }
  spl_handler_ArrayObject.has_property = & spl_array_has_property;
  {
  fprintf(_coverage_fout, "3180\n");
  fflush(_coverage_fout);
  }
  spl_handler_ArrayObject.unset_property = & spl_array_unset_property;
  {
  fprintf(_coverage_fout, "3181\n");
  fflush(_coverage_fout);
  }
  spl_register_std_class((zend_class_entry **)(& spl_ce_ArrayIterator),
                         (char *)"ArrayIterator", & spl_array_object_new,
                         spl_funcs_ArrayIterator);
  {
  fprintf(_coverage_fout, "3182\n");
  fflush(_coverage_fout);
  }
  zend_class_implements((zend_class_entry *)spl_ce_ArrayIterator, 1,
                        zend_ce_iterator);
  {
  fprintf(_coverage_fout, "3183\n");
  fflush(_coverage_fout);
  }
  zend_class_implements((zend_class_entry *)spl_ce_ArrayIterator, 1,
                        zend_ce_arrayaccess);
  {
  fprintf(_coverage_fout, "3184\n");
  fflush(_coverage_fout);
  }
  zend_class_implements((zend_class_entry *)spl_ce_ArrayIterator, 1,
                        spl_ce_SeekableIterator);
  {
  fprintf(_coverage_fout, "3185\n");
  fflush(_coverage_fout);
  }
  zend_class_implements((zend_class_entry *)spl_ce_ArrayIterator, 1,
                        zend_ce_serializable);
  {
  fprintf(_coverage_fout, "3186\n");
  fflush(_coverage_fout);
  }
  memcpy((void */* __restrict  */)(& spl_handler_ArrayIterator),
         (void const   */* __restrict  */)(& spl_handler_ArrayObject),
         sizeof(zend_object_handlers ));
  {
  fprintf(_coverage_fout, "3187\n");
  fflush(_coverage_fout);
  }
  spl_ce_ArrayIterator->get_iterator = & spl_array_get_iterator;
  {
  fprintf(_coverage_fout, "3188\n");
  fflush(_coverage_fout);
  }
  spl_register_sub_class((zend_class_entry **)(& spl_ce_RecursiveArrayIterator),
                         (zend_class_entry *)spl_ce_ArrayIterator,
                         (char *)"RecursiveArrayIterator",
                         & spl_array_object_new,
                         spl_funcs_RecursiveArrayIterator);
  {
  fprintf(_coverage_fout, "3189\n");
  fflush(_coverage_fout);
  }
  zend_class_implements((zend_class_entry *)spl_ce_RecursiveArrayIterator, 1,
                        spl_ce_RecursiveIterator);
  {
  fprintf(_coverage_fout, "3190\n");
  fflush(_coverage_fout);
  }
  spl_ce_RecursiveArrayIterator->get_iterator = & spl_array_get_iterator;
  {
  fprintf(_coverage_fout, "3191\n");
  fflush(_coverage_fout);
  }
  zend_class_implements((zend_class_entry *)spl_ce_ArrayObject, 1,
                        spl_ce_Countable);
  {
  fprintf(_coverage_fout, "3192\n");
  fflush(_coverage_fout);
  }
  zend_class_implements((zend_class_entry *)spl_ce_ArrayIterator, 1,
                        spl_ce_Countable);
  {
  fprintf(_coverage_fout, "3193\n");
  fflush(_coverage_fout);
  }
  zend_declare_class_constant_long((zend_class_entry *)spl_ce_ArrayObject,
                                   "STD_PROP_LIST",
                                   sizeof("STD_PROP_LIST") - 1U, 1L);
  {
  fprintf(_coverage_fout, "3194\n");
  fflush(_coverage_fout);
  }
  zend_declare_class_constant_long((zend_class_entry *)spl_ce_ArrayObject,
                                   "ARRAY_AS_PROPS",
                                   sizeof("ARRAY_AS_PROPS") - 1U, 2L);
  {
  fprintf(_coverage_fout, "3195\n");
  fflush(_coverage_fout);
  }
  zend_declare_class_constant_long((zend_class_entry *)spl_ce_ArrayIterator,
                                   "STD_PROP_LIST",
                                   sizeof("STD_PROP_LIST") - 1U, 1L);
  {
  fprintf(_coverage_fout, "3196\n");
  fflush(_coverage_fout);
  }
  zend_declare_class_constant_long((zend_class_entry *)spl_ce_ArrayIterator,
                                   "ARRAY_AS_PROPS",
                                   sizeof("ARRAY_AS_PROPS") - 1U, 2L);
  {
  fprintf(_coverage_fout, "3197\n");
  fflush(_coverage_fout);
  }
  zend_declare_class_constant_long((zend_class_entry *)spl_ce_RecursiveArrayIterator,
                                   "CHILD_ARRAYS_ONLY",
                                   sizeof("CHILD_ARRAYS_ONLY") - 1U, 4L);
  {
  fprintf(_coverage_fout, "3198\n");
  fflush(_coverage_fout);
  }
  return (0);
}
}
int zm_startup_spl_array(int type , int module_number ) ;
typedef unsigned char zend_uchar;
struct _zend_arg_info {
   char const   *name ;
   zend_uint name_len ;
   char const   *class_name ;
   zend_uint class_name_len ;
   zend_uchar type_hint ;
   zend_bool allow_null ;
   zend_bool pass_by_reference ;
};
typedef struct _zend_arg_info zend_arg_info;
static zend_arg_info const   arginfo_array_void[1] ;
void zim_spl_Array_getChildren(int ht , zval *return_value ,
                               zval **return_value_ptr , zval *this_ptr ,
                               int return_value_used ) ;
void zim_spl_Array_hasChildren(int ht , zval *return_value ,
                               zval **return_value_ptr , zval *this_ptr ,
                               int return_value_used ) ;
static zend_function_entry const   spl_funcs_RecursiveArrayIterator[3]  = {      {"hasChildren", & zim_spl_Array_hasChildren, arginfo_array_void,
      sizeof(arginfo_array_void) / sizeof(struct _zend_arg_info ) - 1U,
      (zend_uint )0x100}, 
        {"getChildren", & zim_spl_Array_getChildren, arginfo_array_void,
      sizeof(arginfo_array_void) / sizeof(struct _zend_arg_info ) - 1U,
      (zend_uint )0x100}, 
        {(char const   *)((void *)0),
      (void (*)(int ht , zval *return_value ,
                zval **return_value_ptr , zval *this_ptr ,
                int return_value_used ))((void *)0),
      (struct _zend_arg_info  const  *)((void *)0), (zend_uint )0, (zend_uint )0}};
static zend_arg_info const   arginfo_array_seek[2] ;
void zim_spl_Array_seek(int ht , zval *return_value , zval **return_value_ptr ,
                        zval *this_ptr , int return_value_used ) ;
void zim_spl_Array_valid(int ht , zval *return_value , zval **return_value_ptr ,
                         zval *this_ptr , int return_value_used ) ;
void zim_spl_Array_next(int ht , zval *return_value , zval **return_value_ptr ,
                        zval *this_ptr , int return_value_used ) ;
void zim_spl_Array_key(int ht , zval *return_value , zval **return_value_ptr ,
                       zval *this_ptr , int return_value_used ) ;
void zim_spl_Array_current(int ht , zval *return_value ,
                           zval **return_value_ptr , zval *this_ptr ,
                           int return_value_used ) ;
void zim_spl_Array_rewind(int ht , zval *return_value ,
                          zval **return_value_ptr , zval *this_ptr ,
                          int return_value_used ) ;
void zim_spl_Array_serialize(int ht , zval *return_value ,
                             zval **return_value_ptr , zval *this_ptr ,
                             int return_value_used ) ;
static zend_arg_info const   arginfo_array_unserialize[2] ;
void zim_spl_Array_unserialize(int ht , zval *return_value ,
                               zval **return_value_ptr , zval *this_ptr ,
                               int return_value_used ) ;
void zim_spl_Array_natcasesort(int ht , zval *return_value ,
                               zval **return_value_ptr , zval *this_ptr ,
                               int return_value_used ) ;
void zim_spl_Array_natsort(int ht , zval *return_value ,
                           zval **return_value_ptr , zval *this_ptr ,
                           int return_value_used ) ;
static zend_arg_info const   arginfo_array_uXsort[2] ;
void zim_spl_Array_uksort(int ht , zval *return_value ,
                          zval **return_value_ptr , zval *this_ptr ,
                          int return_value_used ) ;
void zim_spl_Array_uasort(int ht , zval *return_value ,
                          zval **return_value_ptr , zval *this_ptr ,
                          int return_value_used ) ;
void zim_spl_Array_ksort(int ht , zval *return_value , zval **return_value_ptr ,
                         zval *this_ptr , int return_value_used ) ;
void zim_spl_Array_asort(int ht , zval *return_value , zval **return_value_ptr ,
                         zval *this_ptr , int return_value_used ) ;
static zend_arg_info const   arginfo_array_setFlags[2] ;
void zim_spl_Array_setFlags(int ht , zval *return_value ,
                            zval **return_value_ptr , zval *this_ptr ,
                            int return_value_used ) ;
void zim_spl_Array_getFlags(int ht , zval *return_value ,
                            zval **return_value_ptr , zval *this_ptr ,
                            int return_value_used ) ;
void zim_spl_Array_count(int ht , zval *return_value , zval **return_value_ptr ,
                         zval *this_ptr , int return_value_used ) ;
void zim_spl_Array_getArrayCopy(int ht , zval *return_value ,
                                zval **return_value_ptr , zval *this_ptr ,
                                int return_value_used ) ;
static zend_arg_info const   arginfo_array_append[2] ;
void zim_spl_Array_append(int ht , zval *return_value ,
                          zval **return_value_ptr , zval *this_ptr ,
                          int return_value_used ) ;
static zend_arg_info const   arginfo_array_offsetGet[2] ;
void zim_spl_Array_offsetUnset(int ht , zval *return_value ,
                               zval **return_value_ptr , zval *this_ptr ,
                               int return_value_used ) ;
static zend_arg_info const   arginfo_array_offsetSet[3] ;
void zim_spl_Array_offsetSet(int ht , zval *return_value ,
                             zval **return_value_ptr , zval *this_ptr ,
                             int return_value_used ) ;
void zim_spl_Array_offsetGet(int ht , zval *return_value ,
                             zval **return_value_ptr , zval *this_ptr ,
                             int return_value_used ) ;
void zim_spl_Array_offsetExists(int ht , zval *return_value ,
                                zval **return_value_ptr , zval *this_ptr ,
                                int return_value_used ) ;
static zend_arg_info const   arginfo_array___construct[2] ;
void zim_spl_Array___construct(int ht , zval *return_value ,
                               zval **return_value_ptr , zval *this_ptr ,
                               int return_value_used ) ;
static zend_function_entry const   spl_funcs_ArrayIterator[25]  = 
  {      {"__construct", & zim_spl_Array___construct, arginfo_array___construct,
      sizeof(arginfo_array___construct) / sizeof(struct _zend_arg_info ) - 1U,
      (zend_uint )0x100}, 
        {"offsetExists", & zim_spl_Array_offsetExists, arginfo_array_offsetGet,
      sizeof(arginfo_array_offsetGet) / sizeof(struct _zend_arg_info ) - 1U,
      (zend_uint )0x100}, 
        {"offsetGet", & zim_spl_Array_offsetGet, arginfo_array_offsetGet,
      sizeof(arginfo_array_offsetGet) / sizeof(struct _zend_arg_info ) - 1U,
      (zend_uint )0x100}, 
        {"offsetSet", & zim_spl_Array_offsetSet, arginfo_array_offsetSet,
      sizeof(arginfo_array_offsetSet) / sizeof(struct _zend_arg_info ) - 1U,
      (zend_uint )0x100}, 
        {"offsetUnset", & zim_spl_Array_offsetUnset, arginfo_array_offsetGet,
      sizeof(arginfo_array_offsetGet) / sizeof(struct _zend_arg_info ) - 1U,
      (zend_uint )0x100}, 
        {"append", & zim_spl_Array_append, arginfo_array_append,
      sizeof(arginfo_array_append) / sizeof(struct _zend_arg_info ) - 1U,
      (zend_uint )0x100}, 
        {"getArrayCopy", & zim_spl_Array_getArrayCopy, arginfo_array_void,
      sizeof(arginfo_array_void) / sizeof(struct _zend_arg_info ) - 1U,
      (zend_uint )0x100}, 
        {"count", & zim_spl_Array_count, arginfo_array_void,
      sizeof(arginfo_array_void) / sizeof(struct _zend_arg_info ) - 1U,
      (zend_uint )0x100}, 
        {"getFlags", & zim_spl_Array_getFlags, arginfo_array_void,
      sizeof(arginfo_array_void) / sizeof(struct _zend_arg_info ) - 1U,
      (zend_uint )0x100}, 
        {"setFlags", & zim_spl_Array_setFlags, arginfo_array_setFlags,
      sizeof(arginfo_array_setFlags) / sizeof(struct _zend_arg_info ) - 1U,
      (zend_uint )0x100}, 
        {"asort", & zim_spl_Array_asort, arginfo_array_void,
      sizeof(arginfo_array_void) / sizeof(struct _zend_arg_info ) - 1U,
      (zend_uint )0x100}, 
        {"ksort", & zim_spl_Array_ksort, arginfo_array_void,
      sizeof(arginfo_array_void) / sizeof(struct _zend_arg_info ) - 1U,
      (zend_uint )0x100}, 
        {"uasort", & zim_spl_Array_uasort, arginfo_array_uXsort,
      sizeof(arginfo_array_uXsort) / sizeof(struct _zend_arg_info ) - 1U,
      (zend_uint )0x100}, 
        {"uksort", & zim_spl_Array_uksort, arginfo_array_uXsort,
      sizeof(arginfo_array_uXsort) / sizeof(struct _zend_arg_info ) - 1U,
      (zend_uint )0x100}, 
        {"natsort", & zim_spl_Array_natsort, arginfo_array_void,
      sizeof(arginfo_array_void) / sizeof(struct _zend_arg_info ) - 1U,
      (zend_uint )0x100}, 
        {"natcasesort", & zim_spl_Array_natcasesort, arginfo_array_void,
      sizeof(arginfo_array_void) / sizeof(struct _zend_arg_info ) - 1U,
      (zend_uint )0x100}, 
        {"unserialize", & zim_spl_Array_unserialize, arginfo_array_unserialize,
      sizeof(arginfo_array_unserialize) / sizeof(struct _zend_arg_info ) - 1U,
      (zend_uint )0x100}, 
        {"serialize", & zim_spl_Array_serialize, arginfo_array_void,
      sizeof(arginfo_array_void) / sizeof(struct _zend_arg_info ) - 1U,
      (zend_uint )0x100}, 
        {"rewind", & zim_spl_Array_rewind, arginfo_array_void,
      sizeof(arginfo_array_void) / sizeof(struct _zend_arg_info ) - 1U,
      (zend_uint )0x100}, 
        {"current", & zim_spl_Array_current, arginfo_array_void,
      sizeof(arginfo_array_void) / sizeof(struct _zend_arg_info ) - 1U,
      (zend_uint )0x100}, 
        {"key", & zim_spl_Array_key, arginfo_array_void,
      sizeof(arginfo_array_void) / sizeof(struct _zend_arg_info ) - 1U,
      (zend_uint )0x100}, 
        {"next", & zim_spl_Array_next, arginfo_array_void,
      sizeof(arginfo_array_void) / sizeof(struct _zend_arg_info ) - 1U,
      (zend_uint )0x100}, 
        {"valid", & zim_spl_Array_valid, arginfo_array_void,
      sizeof(arginfo_array_void) / sizeof(struct _zend_arg_info ) - 1U,
      (zend_uint )0x100}, 
        {"seek", & zim_spl_Array_seek, arginfo_array_seek,
      sizeof(arginfo_array_seek) / sizeof(struct _zend_arg_info ) - 1U,
      (zend_uint )0x100}, 
        {(char const   *)((void *)0),
      (void (*)(int ht , zval *return_value ,
                zval **return_value_ptr , zval *this_ptr ,
                int return_value_used ))((void *)0),
      (struct _zend_arg_info  const  *)((void *)0), (zend_uint )0, (zend_uint )0}};
void zim_spl_Array_getIteratorClass(int ht ,
                                    zval *return_value ,
                                    zval **return_value_ptr , zval *this_ptr ,
                                    int return_value_used ) ;
static zend_arg_info const   arginfo_array_setIteratorClass[2] ;
void zim_spl_Array_setIteratorClass(int ht , zval *return_value ,
                                    zval **return_value_ptr , zval *this_ptr ,
                                    int return_value_used ) ;
static zend_arg_info const   arginfo_array_exchangeArray[2] ;
void zim_spl_Array_exchangeArray(int ht , zval *return_value ,
                                 zval **return_value_ptr , zval *this_ptr ,
                                 int return_value_used ) ;
void zim_spl_Array_getIterator(int ht , zval *return_value ,
                               zval **return_value_ptr , zval *this_ptr ,
                               int return_value_used ) ;
static zend_function_entry const   spl_funcs_ArrayObject[23]  = 
  {      {"__construct", & zim_spl_Array___construct, arginfo_array___construct,
      sizeof(arginfo_array___construct) / sizeof(struct _zend_arg_info ) - 1U,
      (zend_uint )0x100}, 
        {"offsetExists", & zim_spl_Array_offsetExists, arginfo_array_offsetGet,
      sizeof(arginfo_array_offsetGet) / sizeof(struct _zend_arg_info ) - 1U,
      (zend_uint )0x100}, 
        {"offsetGet", & zim_spl_Array_offsetGet, arginfo_array_offsetGet,
      sizeof(arginfo_array_offsetGet) / sizeof(struct _zend_arg_info ) - 1U,
      (zend_uint )0x100}, 
        {"offsetSet", & zim_spl_Array_offsetSet, arginfo_array_offsetSet,
      sizeof(arginfo_array_offsetSet) / sizeof(struct _zend_arg_info ) - 1U,
      (zend_uint )0x100}, 
        {"offsetUnset", & zim_spl_Array_offsetUnset, arginfo_array_offsetGet,
      sizeof(arginfo_array_offsetGet) / sizeof(struct _zend_arg_info ) - 1U,
      (zend_uint )0x100}, 
        {"append", & zim_spl_Array_append, arginfo_array_append,
      sizeof(arginfo_array_append) / sizeof(struct _zend_arg_info ) - 1U,
      (zend_uint )0x100}, 
        {"getArrayCopy", & zim_spl_Array_getArrayCopy, arginfo_array_void,
      sizeof(arginfo_array_void) / sizeof(struct _zend_arg_info ) - 1U,
      (zend_uint )0x100}, 
        {"count", & zim_spl_Array_count, arginfo_array_void,
      sizeof(arginfo_array_void) / sizeof(struct _zend_arg_info ) - 1U,
      (zend_uint )0x100}, 
        {"getFlags", & zim_spl_Array_getFlags, arginfo_array_void,
      sizeof(arginfo_array_void) / sizeof(struct _zend_arg_info ) - 1U,
      (zend_uint )0x100}, 
        {"setFlags", & zim_spl_Array_setFlags, arginfo_array_setFlags,
      sizeof(arginfo_array_setFlags) / sizeof(struct _zend_arg_info ) - 1U,
      (zend_uint )0x100}, 
        {"asort", & zim_spl_Array_asort, arginfo_array_void,
      sizeof(arginfo_array_void) / sizeof(struct _zend_arg_info ) - 1U,
      (zend_uint )0x100}, 
        {"ksort", & zim_spl_Array_ksort, arginfo_array_void,
      sizeof(arginfo_array_void) / sizeof(struct _zend_arg_info ) - 1U,
      (zend_uint )0x100}, 
        {"uasort", & zim_spl_Array_uasort, arginfo_array_uXsort,
      sizeof(arginfo_array_uXsort) / sizeof(struct _zend_arg_info ) - 1U,
      (zend_uint )0x100}, 
        {"uksort", & zim_spl_Array_uksort, arginfo_array_uXsort,
      sizeof(arginfo_array_uXsort) / sizeof(struct _zend_arg_info ) - 1U,
      (zend_uint )0x100}, 
        {"natsort", & zim_spl_Array_natsort, arginfo_array_void,
      sizeof(arginfo_array_void) / sizeof(struct _zend_arg_info ) - 1U,
      (zend_uint )0x100}, 
        {"natcasesort", & zim_spl_Array_natcasesort, arginfo_array_void,
      sizeof(arginfo_array_void) / sizeof(struct _zend_arg_info ) - 1U,
      (zend_uint )0x100}, 
        {"unserialize", & zim_spl_Array_unserialize, arginfo_array_unserialize,
      sizeof(arginfo_array_unserialize) / sizeof(struct _zend_arg_info ) - 1U,
      (zend_uint )0x100}, 
        {"serialize", & zim_spl_Array_serialize, arginfo_array_void,
      sizeof(arginfo_array_void) / sizeof(struct _zend_arg_info ) - 1U,
      (zend_uint )0x100}, 
        {"getIterator", & zim_spl_Array_getIterator, arginfo_array_void,
      sizeof(arginfo_array_void) / sizeof(struct _zend_arg_info ) - 1U,
      (zend_uint )0x100}, 
        {"exchangeArray", & zim_spl_Array_exchangeArray,
      arginfo_array_exchangeArray,
      sizeof(arginfo_array_exchangeArray) / sizeof(struct _zend_arg_info ) - 1U,
      (zend_uint )0x100}, 
        {"setIteratorClass", & zim_spl_Array_setIteratorClass,
      arginfo_array_setIteratorClass,
      sizeof(arginfo_array_setIteratorClass) / sizeof(struct _zend_arg_info ) - 1U,
      (zend_uint )0x100}, 
        {"getIteratorClass", & zim_spl_Array_getIteratorClass, arginfo_array_void,
      sizeof(arginfo_array_void) / sizeof(struct _zend_arg_info ) - 1U,
      (zend_uint )0x100}, 
        {(char const   *)((void *)0),
      (void (*)(int ht , zval *return_value ,
                zval **return_value_ptr , zval *this_ptr ,
                int return_value_used ))((void *)0),
      (struct _zend_arg_info  const  *)((void *)0), (zend_uint )0, (zend_uint )0}};
static zend_arg_info const   arginfo_array_void[1]  = {      {(char const   *)((void *)0),
      (zend_uint )0, (char const   *)((void *)0), (zend_uint )-1,
      (zend_uchar )0, (zend_bool )0, (zend_bool )0}};
static zend_arg_info const   arginfo_array_unserialize[2]  = {      {(char const   *)((void *)0), (zend_uint )0, (char const   *)((void *)0),
      (zend_uint )-1, (zend_uchar )0, (zend_bool )0, (zend_bool )0}, 
        {"serialized", sizeof("serialized") - 1U, (char const   *)((void *)0),
      (zend_uint )0, (zend_uchar )0, (zend_bool )0, (zend_bool )0}};
static zend_arg_info const   arginfo_array_uXsort[2]  = {      {(char const   *)((void *)0), (zend_uint )0, (char const   *)((void *)0),
      (zend_uint )-1, (zend_uchar )0, (zend_bool )0, (zend_bool )0}, 
        {"cmp_function", sizeof("cmp_function") - 1U, (char const   *)((void *)0),
      (zend_uint )0, (zend_uchar )0, (zend_bool )0, (zend_bool )0}};
static zend_arg_info const   arginfo_array_setIteratorClass[2]  = {      {(char const   *)((void *)0), (zend_uint )0, (char const   *)((void *)0),
      (zend_uint )-1, (zend_uchar )0, (zend_bool )0, (zend_bool )0}, 
        {"iteratorClass", sizeof("iteratorClass") - 1U,
      (char const   *)((void *)0), (zend_uint )0, (zend_uchar )0, (zend_bool )0,
      (zend_bool )0}};
static zend_arg_info const   arginfo_array_setFlags[2]  = {      {(char const   *)((void *)0), (zend_uint )0, (char const   *)((void *)0),
      (zend_uint )-1, (zend_uchar )0, (zend_bool )0, (zend_bool )0}, 
        {"flags", sizeof("flags") - 1U, (char const   *)((void *)0), (zend_uint )0,
      (zend_uchar )0, (zend_bool )0, (zend_bool )0}};
static zend_arg_info const   arginfo_array_exchangeArray[2]  = {      {(char const   *)((void *)0), (zend_uint )0, (char const   *)((void *)0),
      (zend_uint )-1, (zend_uchar )0, (zend_bool )0, (zend_bool )0}, 
        {"array", sizeof("array") - 1U, (char const   *)((void *)0), (zend_uint )0,
      (zend_uchar )0, (zend_bool )0, (zend_bool )0}};
static zend_arg_info const   arginfo_array_seek[2]  = {      {(char const   *)((void *)0), (zend_uint )0, (char const   *)((void *)0),
      (zend_uint )-1, (zend_uchar )0, (zend_bool )0, (zend_bool )0}, 
        {"position", sizeof("position") - 1U, (char const   *)((void *)0),
      (zend_uint )0, (zend_uchar )0, (zend_bool )0, (zend_bool )0}};
static zend_arg_info const   arginfo_array_append[2]  = {      {(char const   *)((void *)0), (zend_uint )0, (char const   *)((void *)0),
      (zend_uint )-1, (zend_uchar )0, (zend_bool )0, (zend_bool )0}, 
        {"value", sizeof("value") - 1U, (char const   *)((void *)0), (zend_uint )0,
      (zend_uchar )0, (zend_bool )0, (zend_bool )0}};
static zend_arg_info const   arginfo_array_offsetSet[3]  = {      {(char const   *)((void *)0), (zend_uint )0, (char const   *)((void *)0),
      (zend_uint )2, (zend_uchar )0, (zend_bool )0, (zend_bool )0}, 
        {"index", sizeof("index") - 1U, (char const   *)((void *)0), (zend_uint )0,
      (zend_uchar )0, (zend_bool )0, (zend_bool )0}, 
        {"newval", sizeof("newval") - 1U, (char const   *)((void *)0),
      (zend_uint )0, (zend_uchar )0, (zend_bool )0, (zend_bool )0}};
static zend_arg_info const   arginfo_array_offsetGet[2]  = {      {(char const   *)((void *)0), (zend_uint )0, (char const   *)((void *)0),
      (zend_uint )1, (zend_uchar )0, (zend_bool )0, (zend_bool )0}, 
        {"index", sizeof("index") - 1U, (char const   *)((void *)0), (zend_uint )0,
      (zend_uchar )0, (zend_bool )0, (zend_bool )0}};
static zend_arg_info const   arginfo_array___construct[2]  = {      {(char const   *)((void *)0), (zend_uint )0, (char const   *)((void *)0),
      (zend_uint )-1, (zend_uchar )0, (zend_bool )0, (zend_bool )0}, 
        {"array", sizeof("array") - 1U, (char const   *)((void *)0), (zend_uint )0,
      (zend_uchar )0, (zend_bool )0, (zend_bool )0}};
extern zend_class_entry __attribute__((__visibility__("default")))  *spl_ce_UnexpectedValueException ;
extern zval __attribute__((__visibility__("default")))  *zend_throw_exception_ex(zend_class_entry *exception_ce ,
                                                                                 long code ,
                                                                                 char *format 
                                                                                 , ...) ;
struct _php_basic_globals;
typedef struct _php_basic_globals php_basic_globals;
extern php_basic_globals __attribute__((__visibility__("default")))  basic_globals ;
struct __anonstruct_smart_str_80 {
   char *c ;
   size_t len ;
   size_t a ;
};
typedef struct __anonstruct_smart_str_80 smart_str;
struct __anonstruct_url_adapt_state_ex_t_81 {
   smart_str tag ;
   smart_str arg ;
   smart_str val ;
   smart_str buf ;
   smart_str result ;
   smart_str form_app ;
   smart_str url_app ;
   int active ;
   char *lookup_data ;
   int state ;
   HashTable *tags ;
};
typedef struct __anonstruct_url_adapt_state_ex_t_81 url_adapt_state_ex_t;
struct __anonstruct_unserialize_83 {
   void *var_hash ;
   unsigned int level ;
};
struct __anonstruct_serialize_82 {
   void *var_hash ;
   unsigned int level ;
};
typedef unsigned int php_uint32;
typedef long __time_t;
struct timespec {
   __time_t tv_sec ;
   long tv_nsec ;
};
typedef long __blkcnt_t;
typedef long __blksize_t;
typedef long __off_t;
typedef unsigned long long __u_quad_t;
typedef __u_quad_t __dev_t;
typedef unsigned int __gid_t;
typedef unsigned int __uid_t;
typedef unsigned int __nlink_t;
typedef unsigned int __mode_t;
typedef unsigned long __ino_t;
struct stat {
   __dev_t st_dev ;
   unsigned short __pad1 ;
   __ino_t st_ino ;
   __mode_t st_mode ;
   __nlink_t st_nlink ;
   __uid_t st_uid ;
   __gid_t st_gid ;
   __dev_t st_rdev ;
   unsigned short __pad2 ;
   __off_t st_size ;
   __blksize_t st_blksize ;
   __blkcnt_t st_blocks ;
   struct timespec st_atim ;
   struct timespec st_mtim ;
   struct timespec st_ctim ;
   unsigned long __unused4 ;
   unsigned long __unused5 ;
};
struct _php_stream_statbuf {
   struct stat sb ;
};
typedef struct _php_stream_statbuf php_stream_statbuf;
typedef __time_t time_t;
struct _zend_llist;
typedef struct _zend_llist zend_llist;
typedef union _zend_function zend_function;
struct _zend_fcall_info_cache {
   zend_bool initialized ;
   zend_function *function_handler ;
   zend_class_entry *calling_scope ;
   zend_class_entry *called_scope ;
   zval *object_ptr ;
};
typedef struct _zend_fcall_info_cache zend_fcall_info_cache;
struct _zend_fcall_info {
   size_t size ;
   HashTable *function_table ;
   zval *function_name ;
   HashTable *symbol_table ;
   zval **retval_ptr_ptr ;
   zend_uint param_count ;
   zval ***params ;
   zval *object_ptr ;
   zend_bool no_separation ;
};
typedef struct _zend_fcall_info zend_fcall_info;
struct _php_basic_globals {
   HashTable *user_shutdown_function_names ;
   HashTable putenv_ht ;
   zval *strtok_zval ;
   char *strtok_string ;
   char *locale_string ;
   char *strtok_last ;
   char strtok_table[256] ;
   ulong strtok_len ;
   char str_ebuf[40] ;
   zend_fcall_info array_walk_fci ;
   zend_fcall_info_cache array_walk_fci_cache ;
   zend_fcall_info user_compare_fci ;
   zend_fcall_info_cache user_compare_fci_cache ;
   zend_llist *user_tick_functions ;
   zval *active_ini_file_section ;
   long page_uid ;
   long page_gid ;
   long page_inode ;
   time_t page_mtime ;
   char *CurrentStatFile ;
   char *CurrentLStatFile ;
   php_stream_statbuf ssb ;
   php_stream_statbuf lssb ;
   php_uint32 state[625] ;
   php_uint32 *next ;
   int left ;
   unsigned int rand_seed ;
   zend_bool rand_is_seeded ;
   zend_bool mt_rand_is_seeded ;
   char *syslog_device ;
   zend_class_entry *incomplete_class ;
   unsigned int serialize_lock ;
   struct __anonstruct_serialize_82 serialize ;
   struct __anonstruct_unserialize_83 unserialize ;
   url_adapt_state_ex_t url_adapt_state_ex ;
   void *mmap_file ;
   size_t mmap_len ;
   HashTable *user_filter_map ;
   int umask ;
};
struct php_unserialize_data;
typedef struct php_unserialize_data *php_unserialize_data_t;
extern void __attribute__((__visibility__("default")))  _efree(void *ptr ) ;
extern void __attribute__((__visibility__("default")))  var_destroy(php_unserialize_data_t *var_hash ) ;
extern void __attribute__((__visibility__("default")))  _zval_ptr_dtor(zval **zval_ptr ) ;
extern void __attribute__((__visibility__("default")))  zval_add_ref(zval **p ) ;
struct __anonstruct_str_39 {
   char *val ;
   int len ;
};
union _zvalue_value {
   long lval ;
   double dval ;
   struct __anonstruct_str_39 str ;
   HashTable *ht ;
   zend_object_value obj ;
};
typedef union _zvalue_value zvalue_value;
struct _zval_struct {
   zvalue_value value ;
   zend_uint refcount__gc ;
   zend_uchar type ;
   zend_uchar is_ref__gc ;
};
struct _spl_array_object;
typedef struct _spl_array_object spl_array_object;
typedef Bucket *HashPosition;
struct _zend_object {
   zend_class_entry *ce ;
   HashTable *properties ;
   zval **properties_table ;
   HashTable *guards ;
};
typedef struct _zend_object zend_object;
struct _spl_array_object {
   zend_object std ;
   zval *array ;
   zval *retval ;
   HashPosition pos ;
   ulong pos_h ;
   int ar_flags ;
   int is_self ;
   zend_function *fptr_offset_get ;
   zend_function *fptr_offset_set ;
   zend_function *fptr_offset_has ;
   zend_function *fptr_offset_del ;
   zend_function *fptr_count ;
   zend_class_entry *ce_get_iterator ;
   HashTable *debug_info ;
   unsigned char nApplyCount ;
};
extern void __attribute__((__visibility__("default")))  zend_hash_copy(HashTable *target ,
                                                                       HashTable *source ,
                                                                       void (*pCopyConstructor)(void *pElement ) ,
                                                                       void *tmp ,
                                                                       uint size ) ;
struct _zend_object;
extern void __attribute__((__visibility__("default")))  rebuild_object_properties(zend_object *zobj ) ;
extern int __attribute__((__visibility__("default")))  php_var_unserialize(zval **rval ,
                                                                           unsigned char const   **p ,
                                                                           unsigned char const   *max ,
                                                                           php_unserialize_data_t *var_hash ) ;
extern zval __attribute__((__visibility__("default")))  zval_used_for_init ;
struct _gc_root_buffer;
typedef struct _gc_root_buffer gc_root_buffer;
struct _zval_gc_info;
typedef struct _zval_gc_info zval_gc_info;
union __anonunion_u_46 {
   gc_root_buffer *buffered ;
   struct _zval_gc_info *next ;
};
struct _zval_gc_info {
   zval z ;
   union __anonunion_u_46 u ;
};
extern void __attribute__((__visibility__("default")))  *_emalloc(size_t size )  __attribute__((__malloc__)) ;
struct php_unserialize_data {
   void *first ;
   void *last ;
   void *first_dtor ;
   void *last_dtor ;
};
extern void __attribute__((__visibility__("default")))  *_ecalloc(size_t nmemb ,
                                                                  size_t size )  __attribute__((__malloc__)) ;
extern int __attribute__((__visibility__("default")))  zend_parse_parameters(int num_args ,
                                                                             char const   *type_spec 
                                                                             , ...) ;
extern void __attribute__((__visibility__("default")))  *zend_object_store_get_object(zval const   *object ) ;
void zim_spl_Array_unserialize(int ht , zval *return_value ,
                               zval **return_value_ptr , zval *this_ptr ,
                               int return_value_used ) 
{ spl_array_object *intern ;
  void __attribute__((__visibility__("default")))  *tmp ;
  char *buf ;
  int buf_len ;
  unsigned char const   *p ;
  unsigned char const   *s ;
  php_unserialize_data_t var_hash ;
  zval *pmembers ;
  zval *pflags ;
  long flags ;
  int __attribute__((__visibility__("default")))  tmp___0 ;
  void __attribute__((__visibility__("default")))  *tmp___1 ;
  void __attribute__((__visibility__("default")))  *tmp___2 ;
  int __attribute__((__visibility__("default")))  tmp___3 ;
  void __attribute__((__visibility__("default")))  *tmp___4 ;
  int __attribute__((__visibility__("default")))  tmp___5 ;
  void __attribute__((__visibility__("default")))  *tmp___6 ;
  int __attribute__((__visibility__("default")))  tmp___7 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "3125\n");
  fflush(_coverage_fout);
  }
  tmp = zend_object_store_get_object((zval const   *)this_ptr);
  {
  fprintf(_coverage_fout, "3126\n");
  fflush(_coverage_fout);
  }
  intern = (spl_array_object *)tmp;
  {
  fprintf(_coverage_fout, "3127\n");
  fflush(_coverage_fout);
  }
  pflags = (zval *)((void *)0);
  {
  fprintf(_coverage_fout, "3128\n");
  fflush(_coverage_fout);
  }
  tmp___0 = zend_parse_parameters(ht, "s", & buf, & buf_len);
  {
  fprintf(_coverage_fout, "3129\n");
  fflush(_coverage_fout);
  }
  if (tmp___0 == (int __attribute__((__visibility__("default")))  )-1) {
    {
    fprintf(_coverage_fout, "3046\n");
    fflush(_coverage_fout);
    }
    return;
  } else {
    {
    fprintf(_coverage_fout, "3047\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "3130\n");
  fflush(_coverage_fout);
  }
  if (buf_len == 0) {
    {
    fprintf(_coverage_fout, "3048\n");
    fflush(_coverage_fout);
    }
    zend_throw_exception_ex((zend_class_entry *)spl_ce_UnexpectedValueException,
                            0L,
                            (char *)"Empty serialized string cannot be empty");
    {
    fprintf(_coverage_fout, "3049\n");
    fflush(_coverage_fout);
    }
    return;
  } else {
    {
    fprintf(_coverage_fout, "3050\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "3131\n");
  fflush(_coverage_fout);
  }
  p = (unsigned char const   *)buf;
  {
  fprintf(_coverage_fout, "3132\n");
  fflush(_coverage_fout);
  }
  s = p;
  {
  fprintf(_coverage_fout, "3133\n");
  fflush(_coverage_fout);
  }
  while (1) {
    {
    fprintf(_coverage_fout, "3060\n");
    fflush(_coverage_fout);
    }
    if (basic_globals.serialize_lock) {
      goto _L;
    } else {
      {
      fprintf(_coverage_fout, "3059\n");
      fflush(_coverage_fout);
      }
      if (! basic_globals.unserialize.level) {
        {
        fprintf(_coverage_fout, "3054\n");
        fflush(_coverage_fout);
        }
        _L: /* CIL Label */ 
        tmp___1 = _ecalloc(1U, sizeof(struct php_unserialize_data ));
        {
        fprintf(_coverage_fout, "3055\n");
        fflush(_coverage_fout);
        }
        var_hash = (struct php_unserialize_data *)tmp___1;
        {
        fprintf(_coverage_fout, "3056\n");
        fflush(_coverage_fout);
        }
        if (! basic_globals.serialize_lock) {
          {
          fprintf(_coverage_fout, "3051\n");
          fflush(_coverage_fout);
          }
          basic_globals.unserialize.var_hash = (void *)var_hash;
          {
          fprintf(_coverage_fout, "3052\n");
          fflush(_coverage_fout);
          }
          basic_globals.unserialize.level = 1U;
        } else {
          {
          fprintf(_coverage_fout, "3053\n");
          fflush(_coverage_fout);
          }

        }
      } else {
        {
        fprintf(_coverage_fout, "3057\n");
        fflush(_coverage_fout);
        }
        var_hash = (struct php_unserialize_data *)basic_globals.unserialize.var_hash;
        {
        fprintf(_coverage_fout, "3058\n");
        fflush(_coverage_fout);
        }
        (basic_globals.unserialize.level) ++;
      }
    }
    break;
  }
  {
  fprintf(_coverage_fout, "3134\n");
  fflush(_coverage_fout);
  }
  if ((int const   )*p != 120) {
    goto outexcept;
  } else {
    {
    fprintf(_coverage_fout, "3062\n");
    fflush(_coverage_fout);
    }
    p ++;
    {
    fprintf(_coverage_fout, "3063\n");
    fflush(_coverage_fout);
    }
    if ((int const   )*p != 58) {
      goto outexcept;
    } else {
      {
      fprintf(_coverage_fout, "3061\n");
      fflush(_coverage_fout);
      }

    }
  }
  {
  fprintf(_coverage_fout, "3135\n");
  fflush(_coverage_fout);
  }
  p ++;
  {
  fprintf(_coverage_fout, "3136\n");
  fflush(_coverage_fout);
  }
  while (1) {
    {
    fprintf(_coverage_fout, "3064\n");
    fflush(_coverage_fout);
    }
    tmp___2 = _emalloc(sizeof(zval_gc_info ));
    {
    fprintf(_coverage_fout, "3065\n");
    fflush(_coverage_fout);
    }
    pflags = (zval *)tmp___2;
    {
    fprintf(_coverage_fout, "3066\n");
    fflush(_coverage_fout);
    }
    ((zval_gc_info *)pflags)->u.buffered = (gc_root_buffer *)((void *)0);
    break;
  }
  {
  fprintf(_coverage_fout, "3137\n");
  fflush(_coverage_fout);
  }
  *pflags = (struct _zval_struct )zval_used_for_init;
  {
  fprintf(_coverage_fout, "3138\n");
  fflush(_coverage_fout);
  }
  tmp___3 = php_var_unserialize(& pflags, & p, s + buf_len, & var_hash);
  {
  fprintf(_coverage_fout, "3139\n");
  fflush(_coverage_fout);
  }
  if (tmp___3) {
    {
    fprintf(_coverage_fout, "3069\n");
    fflush(_coverage_fout);
    }
    if ((int )pflags->type != 1) {
      {
      fprintf(_coverage_fout, "3067\n");
      fflush(_coverage_fout);
      }
      _zval_ptr_dtor(& pflags);
      goto outexcept;
    } else {
      {
      fprintf(_coverage_fout, "3068\n");
      fflush(_coverage_fout);
      }

    }
  } else {
    {
    fprintf(_coverage_fout, "3070\n");
    fflush(_coverage_fout);
    }
    _zval_ptr_dtor(& pflags);
    goto outexcept;
  }
  {
  fprintf(_coverage_fout, "3140\n");
  fflush(_coverage_fout);
  }
  p --;
  {
  fprintf(_coverage_fout, "3141\n");
  fflush(_coverage_fout);
  }
  flags = pflags->value.lval;
  {
  fprintf(_coverage_fout, "3142\n");
  fflush(_coverage_fout);
  }
  _zval_ptr_dtor(& pflags);
  {
  fprintf(_coverage_fout, "3143\n");
  fflush(_coverage_fout);
  }
  if ((int const   )*p != 59) {
    goto outexcept;
  } else {
    {
    fprintf(_coverage_fout, "3071\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "3144\n");
  fflush(_coverage_fout);
  }
  p ++;
  {
  fprintf(_coverage_fout, "3145\n");
  fflush(_coverage_fout);
  }
  if ((int const   )*p != 109) {
    {
    fprintf(_coverage_fout, "3081\n");
    fflush(_coverage_fout);
    }
    if ((int const   )*p != 97) {
      {
      fprintf(_coverage_fout, "3075\n");
      fflush(_coverage_fout);
      }
      if ((int const   )*p != 79) {
        {
        fprintf(_coverage_fout, "3073\n");
        fflush(_coverage_fout);
        }
        if ((int const   )*p != 67) {
          goto outexcept;
        } else {
          {
          fprintf(_coverage_fout, "3072\n");
          fflush(_coverage_fout);
          }

        }
      } else {
        {
        fprintf(_coverage_fout, "3074\n");
        fflush(_coverage_fout);
        }

      }
    } else {
      {
      fprintf(_coverage_fout, "3076\n");
      fflush(_coverage_fout);
      }

    }
    {
    fprintf(_coverage_fout, "3082\n");
    fflush(_coverage_fout);
    }
    intern->ar_flags &= -50397184;
    {
    fprintf(_coverage_fout, "3083\n");
    fflush(_coverage_fout);
    }
    intern->ar_flags = (int )((long )intern->ar_flags | (flags & 50397183L));
    {
    fprintf(_coverage_fout, "3084\n");
    fflush(_coverage_fout);
    }
    _zval_ptr_dtor(& intern->array);
    {
    fprintf(_coverage_fout, "3085\n");
    fflush(_coverage_fout);
    }
    while (1) {
      {
      fprintf(_coverage_fout, "3077\n");
      fflush(_coverage_fout);
      }
      tmp___4 = _emalloc(sizeof(zval_gc_info ));
      {
      fprintf(_coverage_fout, "3078\n");
      fflush(_coverage_fout);
      }
      intern->array = (zval *)tmp___4;
      {
      fprintf(_coverage_fout, "3079\n");
      fflush(_coverage_fout);
      }
      ((zval_gc_info *)intern->array)->u.buffered = (gc_root_buffer *)((void *)0);
      break;
    }
    {
    fprintf(_coverage_fout, "3086\n");
    fflush(_coverage_fout);
    }
    *(intern->array) = (struct _zval_struct )zval_used_for_init;
    {
    fprintf(_coverage_fout, "3087\n");
    fflush(_coverage_fout);
    }
    tmp___5 = php_var_unserialize(& intern->array, & p, s + buf_len, & var_hash);
    {
    fprintf(_coverage_fout, "3088\n");
    fflush(_coverage_fout);
    }
    if (tmp___5) {
      {
      fprintf(_coverage_fout, "3080\n");
      fflush(_coverage_fout);
      }

    } else {
      goto outexcept;
    }
  } else {
    {
    fprintf(_coverage_fout, "3089\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "3146\n");
  fflush(_coverage_fout);
  }
  if ((int const   )*p != 59) {
    goto outexcept;
  } else {
    {
    fprintf(_coverage_fout, "3090\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "3147\n");
  fflush(_coverage_fout);
  }
  p ++;
  {
  fprintf(_coverage_fout, "3148\n");
  fflush(_coverage_fout);
  }
  if ((int const   )*p != 109) {
    goto outexcept;
  } else {
    {
    fprintf(_coverage_fout, "3092\n");
    fflush(_coverage_fout);
    }
    p ++;
    {
    fprintf(_coverage_fout, "3093\n");
    fflush(_coverage_fout);
    }
    if ((int const   )*p != 58) {
      goto outexcept;
    } else {
      {
      fprintf(_coverage_fout, "3091\n");
      fflush(_coverage_fout);
      }

    }
  }
  {
  fprintf(_coverage_fout, "3149\n");
  fflush(_coverage_fout);
  }
  p ++;
  {
  fprintf(_coverage_fout, "3150\n");
  fflush(_coverage_fout);
  }
  while (1) {
    {
    fprintf(_coverage_fout, "3094\n");
    fflush(_coverage_fout);
    }
    tmp___6 = _emalloc(sizeof(zval_gc_info ));
    {
    fprintf(_coverage_fout, "3095\n");
    fflush(_coverage_fout);
    }
    pmembers = (zval *)tmp___6;
    {
    fprintf(_coverage_fout, "3096\n");
    fflush(_coverage_fout);
    }
    ((zval_gc_info *)pmembers)->u.buffered = (gc_root_buffer *)((void *)0);
    break;
  }
  {
  fprintf(_coverage_fout, "3151\n");
  fflush(_coverage_fout);
  }
  *pmembers = (struct _zval_struct )zval_used_for_init;
  {
  fprintf(_coverage_fout, "3152\n");
  fflush(_coverage_fout);
  }
  tmp___7 = php_var_unserialize(& pmembers, & p, s + buf_len, & var_hash);
  {
  fprintf(_coverage_fout, "3153\n");
  fflush(_coverage_fout);
  }
  if (tmp___7) {
    {
    fprintf(_coverage_fout, "3097\n");
    fflush(_coverage_fout);
    }

  } else {
    {
    fprintf(_coverage_fout, "3098\n");
    fflush(_coverage_fout);
    }
    _zval_ptr_dtor(& pmembers);
    goto outexcept;
  }
  {
  fprintf(_coverage_fout, "3154\n");
  fflush(_coverage_fout);
  }
  if (! intern->std.properties) {
    {
    fprintf(_coverage_fout, "3099\n");
    fflush(_coverage_fout);
    }
    rebuild_object_properties(& intern->std);
  } else {
    {
    fprintf(_coverage_fout, "3100\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "3155\n");
  fflush(_coverage_fout);
  }
  zend_hash_copy(intern->std.properties, pmembers->value.ht,
                 (void (*)(void *pElement ))(& zval_add_ref), (void *)0,
                 sizeof(zval *));
  {
  fprintf(_coverage_fout, "3156\n");
  fflush(_coverage_fout);
  }
  _zval_ptr_dtor(& pmembers);
  {
  fprintf(_coverage_fout, "3157\n");
  fflush(_coverage_fout);
  }
  while (1) {
    {
    fprintf(_coverage_fout, "3112\n");
    fflush(_coverage_fout);
    }
    if (basic_globals.serialize_lock) {
      {
      fprintf(_coverage_fout, "3101\n");
      fflush(_coverage_fout);
      }
      var_destroy(& var_hash);
      {
      fprintf(_coverage_fout, "3102\n");
      fflush(_coverage_fout);
      }
      _efree((void *)var_hash);
    } else {
      {
      fprintf(_coverage_fout, "3111\n");
      fflush(_coverage_fout);
      }
      if (! basic_globals.unserialize.level) {
        {
        fprintf(_coverage_fout, "3103\n");
        fflush(_coverage_fout);
        }
        var_destroy(& var_hash);
        {
        fprintf(_coverage_fout, "3104\n");
        fflush(_coverage_fout);
        }
        _efree((void *)var_hash);
      } else {
        {
        fprintf(_coverage_fout, "3109\n");
        fflush(_coverage_fout);
        }
        (basic_globals.unserialize.level) --;
        {
        fprintf(_coverage_fout, "3110\n");
        fflush(_coverage_fout);
        }
        if (basic_globals.unserialize.level) {
          {
          fprintf(_coverage_fout, "3105\n");
          fflush(_coverage_fout);
          }

        } else {
          {
          fprintf(_coverage_fout, "3106\n");
          fflush(_coverage_fout);
          }
          var_destroy(& var_hash);
          {
          fprintf(_coverage_fout, "3107\n");
          fflush(_coverage_fout);
          }
          _efree((void *)var_hash);
          {
          fprintf(_coverage_fout, "3108\n");
          fflush(_coverage_fout);
          }
          basic_globals.unserialize.var_hash = (void *)0;
        }
      }
    }
    break;
  }
  {
  fprintf(_coverage_fout, "3158\n");
  fflush(_coverage_fout);
  }
  return;
  {
  fprintf(_coverage_fout, "3159\n");
  fflush(_coverage_fout);
  }
  outexcept: 
  while (1) {
    {
    fprintf(_coverage_fout, "3124\n");
    fflush(_coverage_fout);
    }
    if (basic_globals.serialize_lock) {
      {
      fprintf(_coverage_fout, "3113\n");
      fflush(_coverage_fout);
      }
      var_destroy(& var_hash);
      {
      fprintf(_coverage_fout, "3114\n");
      fflush(_coverage_fout);
      }
      _efree((void *)var_hash);
    } else {
      {
      fprintf(_coverage_fout, "3123\n");
      fflush(_coverage_fout);
      }
      if (! basic_globals.unserialize.level) {
        {
        fprintf(_coverage_fout, "3115\n");
        fflush(_coverage_fout);
        }
        var_destroy(& var_hash);
        {
        fprintf(_coverage_fout, "3116\n");
        fflush(_coverage_fout);
        }
        _efree((void *)var_hash);
      } else {
        {
        fprintf(_coverage_fout, "3121\n");
        fflush(_coverage_fout);
        }
        (basic_globals.unserialize.level) --;
        {
        fprintf(_coverage_fout, "3122\n");
        fflush(_coverage_fout);
        }
        if (basic_globals.unserialize.level) {
          {
          fprintf(_coverage_fout, "3117\n");
          fflush(_coverage_fout);
          }

        } else {
          {
          fprintf(_coverage_fout, "3118\n");
          fflush(_coverage_fout);
          }
          var_destroy(& var_hash);
          {
          fprintf(_coverage_fout, "3119\n");
          fflush(_coverage_fout);
          }
          _efree((void *)var_hash);
          {
          fprintf(_coverage_fout, "3120\n");
          fflush(_coverage_fout);
          }
          basic_globals.unserialize.var_hash = (void *)0;
        }
      }
    }
    break;
  }
  {
  fprintf(_coverage_fout, "3160\n");
  fflush(_coverage_fout);
  }
  zend_throw_exception_ex((zend_class_entry *)spl_ce_UnexpectedValueException,
                          0L, (char *)"Error at offset %ld of %d bytes",
                          (long )((char *)p - buf), buf_len);
  {
  fprintf(_coverage_fout, "3161\n");
  fflush(_coverage_fout);
  }
  return;
}
}
extern void __attribute__((__visibility__("default")))  zend_hash_destroy(HashTable *ht ) ;
typedef HashTable *php_serialize_data_t;
struct __anonstruct_smart_str_80;
extern void __attribute__((__visibility__("default")))  php_var_serialize(smart_str *buf ,
                                                                          zval **struc ,
                                                                          php_serialize_data_t *var_hash ) ;
extern void __attribute__((__visibility__("default")))  *_erealloc(void *ptr ,
                                                                   size_t size ,
                                                                   int allow_failure ) ;
extern int __attribute__((__visibility__("default")))  _zend_hash_init(HashTable *ht ,
                                                                       uint nSize ,
                                                                       ulong (*pHashFunction)(char const   *arKey ,
                                                                                              uint nKeyLength ) ,
                                                                       void (*pDestructor)(void *pDest ) ,
                                                                       zend_bool persistent ) ;
extern void __attribute__((__visibility__("default")))  ( /* format attribute */  php_error_docref0)(char const   *docref ,
                                                                                                     int type ,
                                                                                                     char const   *format 
                                                                                                     , ...) ;
__inline static HashTable *spl_array_get_hash_table(spl_array_object *intern ,
                                                    int check_std_props ) ;
void zim_spl_Array_serialize(int ht , zval *return_value ,
                             zval **return_value_ptr , zval *this_ptr ,
                             int return_value_used ) 
{ zval *object ;
  spl_array_object *intern ;
  void __attribute__((__visibility__("default")))  *tmp ;
  HashTable *aht ;
  HashTable *tmp___0 ;
  zval members ;
  zval *pmembers ;
  php_serialize_data_t var_hash ;
  smart_str buf ;
  zval *flags ;
  int __attribute__((__visibility__("default")))  tmp___1 ;
  void __attribute__((__visibility__("default")))  *tmp___2 ;
  void __attribute__((__visibility__("default")))  *tmp___3 ;
  zval *__z ;
  register size_t __nl ;
  smart_str *__dest ;
  void *tmp___4 ;
  void __attribute__((__visibility__("default")))  *tmp___5 ;
  void *tmp___6 ;
  void __attribute__((__visibility__("default")))  *tmp___7 ;
  register size_t __nl___0 ;
  void *tmp___8 ;
  void __attribute__((__visibility__("default")))  *tmp___9 ;
  void *tmp___10 ;
  void __attribute__((__visibility__("default")))  *tmp___11 ;
  register size_t __nl___1 ;
  smart_str *__dest___0 ;
  void *tmp___12 ;
  void __attribute__((__visibility__("default")))  *tmp___13 ;
  void *tmp___14 ;
  void __attribute__((__visibility__("default")))  *tmp___15 ;
  char const   *__s ;
  int __l ;
  zval *__z___0 ;
  char __attribute__((__visibility__("default")))  *tmp___16 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "3012\n");
  fflush(_coverage_fout);
  }
  object = this_ptr;
  {
  fprintf(_coverage_fout, "3013\n");
  fflush(_coverage_fout);
  }
  tmp = zend_object_store_get_object((zval const   *)object);
  {
  fprintf(_coverage_fout, "3014\n");
  fflush(_coverage_fout);
  }
  intern = (spl_array_object *)tmp;
  {
  fprintf(_coverage_fout, "3015\n");
  fflush(_coverage_fout);
  }
  tmp___0 = spl_array_get_hash_table(intern, 0);
  {
  fprintf(_coverage_fout, "3016\n");
  fflush(_coverage_fout);
  }
  aht = tmp___0;
  {
  fprintf(_coverage_fout, "3017\n");
  fflush(_coverage_fout);
  }
  buf.c = (char *)0;
  {
  fprintf(_coverage_fout, "3018\n");
  fflush(_coverage_fout);
  }
  buf.len = 0U;
  {
  fprintf(_coverage_fout, "3019\n");
  fflush(_coverage_fout);
  }
  buf.a = 0U;
  {
  fprintf(_coverage_fout, "3020\n");
  fflush(_coverage_fout);
  }
  tmp___1 = zend_parse_parameters(ht, "");
  {
  fprintf(_coverage_fout, "3021\n");
  fflush(_coverage_fout);
  }
  if (tmp___1 == (int __attribute__((__visibility__("default")))  )-1) {
    {
    fprintf(_coverage_fout, "2914\n");
    fflush(_coverage_fout);
    }
    return;
  } else {
    {
    fprintf(_coverage_fout, "2915\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "3022\n");
  fflush(_coverage_fout);
  }
  if (! aht) {
    {
    fprintf(_coverage_fout, "2916\n");
    fflush(_coverage_fout);
    }
    php_error_docref0((char const   *)((void *)0), 1 << 3L,
                      "Array was modified outside object and is no longer an array");
    {
    fprintf(_coverage_fout, "2917\n");
    fflush(_coverage_fout);
    }
    return;
  } else {
    {
    fprintf(_coverage_fout, "2918\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "3023\n");
  fflush(_coverage_fout);
  }
  while (1) {
    {
    fprintf(_coverage_fout, "2929\n");
    fflush(_coverage_fout);
    }
    if (basic_globals.serialize_lock) {
      goto _L;
    } else {
      {
      fprintf(_coverage_fout, "2928\n");
      fflush(_coverage_fout);
      }
      if (! basic_globals.serialize.level) {
        {
        fprintf(_coverage_fout, "2922\n");
        fflush(_coverage_fout);
        }
        _L: /* CIL Label */ 
        tmp___2 = _emalloc(sizeof(HashTable ));
        {
        fprintf(_coverage_fout, "2923\n");
        fflush(_coverage_fout);
        }
        var_hash = (HashTable *)tmp___2;
        {
        fprintf(_coverage_fout, "2924\n");
        fflush(_coverage_fout);
        }
        _zend_hash_init(var_hash, 10U, (ulong (*)(char const   *arKey ,
                                                  uint nKeyLength ))((void *)0),
                        (void (*)(void *pDest ))((void *)0), (unsigned char)0);
        {
        fprintf(_coverage_fout, "2925\n");
        fflush(_coverage_fout);
        }
        if (! basic_globals.serialize_lock) {
          {
          fprintf(_coverage_fout, "2919\n");
          fflush(_coverage_fout);
          }
          basic_globals.serialize.var_hash = (void *)var_hash;
          {
          fprintf(_coverage_fout, "2920\n");
          fflush(_coverage_fout);
          }
          basic_globals.serialize.level = 1U;
        } else {
          {
          fprintf(_coverage_fout, "2921\n");
          fflush(_coverage_fout);
          }

        }
      } else {
        {
        fprintf(_coverage_fout, "2926\n");
        fflush(_coverage_fout);
        }
        var_hash = (HashTable *)basic_globals.serialize.var_hash;
        {
        fprintf(_coverage_fout, "2927\n");
        fflush(_coverage_fout);
        }
        (basic_globals.serialize.level) ++;
      }
    }
    break;
  }
  {
  fprintf(_coverage_fout, "3024\n");
  fflush(_coverage_fout);
  }
  while (1) {
    {
    fprintf(_coverage_fout, "2930\n");
    fflush(_coverage_fout);
    }
    tmp___3 = _emalloc(sizeof(zval_gc_info ));
    {
    fprintf(_coverage_fout, "2931\n");
    fflush(_coverage_fout);
    }
    flags = (zval *)tmp___3;
    {
    fprintf(_coverage_fout, "2932\n");
    fflush(_coverage_fout);
    }
    ((zval_gc_info *)flags)->u.buffered = (gc_root_buffer *)((void *)0);
    break;
  }
  {
  fprintf(_coverage_fout, "3025\n");
  fflush(_coverage_fout);
  }
  flags->refcount__gc = 1U;
  {
  fprintf(_coverage_fout, "3026\n");
  fflush(_coverage_fout);
  }
  flags->is_ref__gc = (unsigned char)0;
  {
  fprintf(_coverage_fout, "3027\n");
  fflush(_coverage_fout);
  }
  __z = flags;
  {
  fprintf(_coverage_fout, "3028\n");
  fflush(_coverage_fout);
  }
  __z->value.lval = (long )(intern->ar_flags & 0x0300FFFF);
  {
  fprintf(_coverage_fout, "3029\n");
  fflush(_coverage_fout);
  }
  __z->type = (unsigned char)1;
  {
  fprintf(_coverage_fout, "3030\n");
  fflush(_coverage_fout);
  }
  while (1) {
    {
    fprintf(_coverage_fout, "2947\n");
    fflush(_coverage_fout);
    }
    __dest = & buf;
    {
    fprintf(_coverage_fout, "2948\n");
    fflush(_coverage_fout);
    }
    while (1) {
      {
      fprintf(_coverage_fout, "2946\n");
      fflush(_coverage_fout);
      }
      if (! __dest->c) {
        {
        fprintf(_coverage_fout, "2935\n");
        fflush(_coverage_fout);
        }
        __dest->len = 0U;
        {
        fprintf(_coverage_fout, "2936\n");
        fflush(_coverage_fout);
        }
        __nl = 2U;
        {
        fprintf(_coverage_fout, "2937\n");
        fflush(_coverage_fout);
        }
        if (__nl < 78U) {
          {
          fprintf(_coverage_fout, "2933\n");
          fflush(_coverage_fout);
          }
          __dest->a = 78U;
        } else {
          {
          fprintf(_coverage_fout, "2934\n");
          fflush(_coverage_fout);
          }
          __dest->a = __nl + 128U;
        }
        {
        fprintf(_coverage_fout, "2938\n");
        fflush(_coverage_fout);
        }
        tmp___5 = _erealloc((void *)__dest->c, __dest->a + 1U, 0);
        {
        fprintf(_coverage_fout, "2939\n");
        fflush(_coverage_fout);
        }
        __dest->c = (char *)((void *)tmp___5);
      } else {
        {
        fprintf(_coverage_fout, "2944\n");
        fflush(_coverage_fout);
        }
        __nl = __dest->len + 2U;
        {
        fprintf(_coverage_fout, "2945\n");
        fflush(_coverage_fout);
        }
        if (__nl >= __dest->a) {
          {
          fprintf(_coverage_fout, "2940\n");
          fflush(_coverage_fout);
          }
          __dest->a = __nl + 128U;
          {
          fprintf(_coverage_fout, "2941\n");
          fflush(_coverage_fout);
          }
          tmp___7 = _erealloc((void *)__dest->c, __dest->a + 1U, 0);
          {
          fprintf(_coverage_fout, "2942\n");
          fflush(_coverage_fout);
          }
          __dest->c = (char *)((void *)tmp___7);
        } else {
          {
          fprintf(_coverage_fout, "2943\n");
          fflush(_coverage_fout);
          }

        }
      }
      break;
    }
    {
    fprintf(_coverage_fout, "2949\n");
    fflush(_coverage_fout);
    }
    memcpy((void */* __restrict  */)(__dest->c + __dest->len),
           (void const   */* __restrict  */)"x:", 2U);
    {
    fprintf(_coverage_fout, "2950\n");
    fflush(_coverage_fout);
    }
    __dest->len = __nl;
    break;
  }
  {
  fprintf(_coverage_fout, "3031\n");
  fflush(_coverage_fout);
  }
  php_var_serialize(& buf, & flags, & var_hash);
  {
  fprintf(_coverage_fout, "3032\n");
  fflush(_coverage_fout);
  }
  _zval_ptr_dtor(& flags);
  {
  fprintf(_coverage_fout, "3033\n");
  fflush(_coverage_fout);
  }
  if (! (intern->ar_flags & 0x02000000)) {
    {
    fprintf(_coverage_fout, "2968\n");
    fflush(_coverage_fout);
    }
    php_var_serialize(& buf, & intern->array, & var_hash);
    {
    fprintf(_coverage_fout, "2969\n");
    fflush(_coverage_fout);
    }
    while (1) {
      {
      fprintf(_coverage_fout, "2965\n");
      fflush(_coverage_fout);
      }
      while (1) {
        {
        fprintf(_coverage_fout, "2964\n");
        fflush(_coverage_fout);
        }
        if (! buf.c) {
          {
          fprintf(_coverage_fout, "2953\n");
          fflush(_coverage_fout);
          }
          buf.len = 0U;
          {
          fprintf(_coverage_fout, "2954\n");
          fflush(_coverage_fout);
          }
          __nl___0 = 1U;
          {
          fprintf(_coverage_fout, "2955\n");
          fflush(_coverage_fout);
          }
          if (__nl___0 < 78U) {
            {
            fprintf(_coverage_fout, "2951\n");
            fflush(_coverage_fout);
            }
            buf.a = 78U;
          } else {
            {
            fprintf(_coverage_fout, "2952\n");
            fflush(_coverage_fout);
            }
            buf.a = __nl___0 + 128U;
          }
          {
          fprintf(_coverage_fout, "2956\n");
          fflush(_coverage_fout);
          }
          tmp___9 = _erealloc((void *)buf.c, buf.a + 1U, 0);
          {
          fprintf(_coverage_fout, "2957\n");
          fflush(_coverage_fout);
          }
          buf.c = (char *)((void *)tmp___9);
        } else {
          {
          fprintf(_coverage_fout, "2962\n");
          fflush(_coverage_fout);
          }
          __nl___0 = buf.len + 1U;
          {
          fprintf(_coverage_fout, "2963\n");
          fflush(_coverage_fout);
          }
          if (__nl___0 >= buf.a) {
            {
            fprintf(_coverage_fout, "2958\n");
            fflush(_coverage_fout);
            }
            buf.a = __nl___0 + 128U;
            {
            fprintf(_coverage_fout, "2959\n");
            fflush(_coverage_fout);
            }
            tmp___11 = _erealloc((void *)buf.c, buf.a + 1U, 0);
            {
            fprintf(_coverage_fout, "2960\n");
            fflush(_coverage_fout);
            }
            buf.c = (char *)((void *)tmp___11);
          } else {
            {
            fprintf(_coverage_fout, "2961\n");
            fflush(_coverage_fout);
            }

          }
        }
        break;
      }
      {
      fprintf(_coverage_fout, "2966\n");
      fflush(_coverage_fout);
      }
      buf.len = __nl___0;
      {
      fprintf(_coverage_fout, "2967\n");
      fflush(_coverage_fout);
      }
      *((unsigned char *)buf.c + (buf.len - 1U)) = (unsigned char )';';
      break;
    }
  } else {
    {
    fprintf(_coverage_fout, "2970\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "3034\n");
  fflush(_coverage_fout);
  }
  while (1) {
    {
    fprintf(_coverage_fout, "2985\n");
    fflush(_coverage_fout);
    }
    __dest___0 = & buf;
    {
    fprintf(_coverage_fout, "2986\n");
    fflush(_coverage_fout);
    }
    while (1) {
      {
      fprintf(_coverage_fout, "2984\n");
      fflush(_coverage_fout);
      }
      if (! __dest___0->c) {
        {
        fprintf(_coverage_fout, "2973\n");
        fflush(_coverage_fout);
        }
        __dest___0->len = 0U;
        {
        fprintf(_coverage_fout, "2974\n");
        fflush(_coverage_fout);
        }
        __nl___1 = 2U;
        {
        fprintf(_coverage_fout, "2975\n");
        fflush(_coverage_fout);
        }
        if (__nl___1 < 78U) {
          {
          fprintf(_coverage_fout, "2971\n");
          fflush(_coverage_fout);
          }
          __dest___0->a = 78U;
        } else {
          {
          fprintf(_coverage_fout, "2972\n");
          fflush(_coverage_fout);
          }
          __dest___0->a = __nl___1 + 128U;
        }
        {
        fprintf(_coverage_fout, "2976\n");
        fflush(_coverage_fout);
        }
        tmp___13 = _erealloc((void *)__dest___0->c, __dest___0->a + 1U, 0);
        {
        fprintf(_coverage_fout, "2977\n");
        fflush(_coverage_fout);
        }
        __dest___0->c = (char *)((void *)tmp___13);
      } else {
        {
        fprintf(_coverage_fout, "2982\n");
        fflush(_coverage_fout);
        }
        __nl___1 = __dest___0->len + 2U;
        {
        fprintf(_coverage_fout, "2983\n");
        fflush(_coverage_fout);
        }
        if (__nl___1 >= __dest___0->a) {
          {
          fprintf(_coverage_fout, "2978\n");
          fflush(_coverage_fout);
          }
          __dest___0->a = __nl___1 + 128U;
          {
          fprintf(_coverage_fout, "2979\n");
          fflush(_coverage_fout);
          }
          tmp___15 = _erealloc((void *)__dest___0->c, __dest___0->a + 1U, 0);
          {
          fprintf(_coverage_fout, "2980\n");
          fflush(_coverage_fout);
          }
          __dest___0->c = (char *)((void *)tmp___15);
        } else {
          {
          fprintf(_coverage_fout, "2981\n");
          fflush(_coverage_fout);
          }

        }
      }
      break;
    }
    {
    fprintf(_coverage_fout, "2987\n");
    fflush(_coverage_fout);
    }
    memcpy((void */* __restrict  */)(__dest___0->c + __dest___0->len),
           (void const   */* __restrict  */)"m:", 2U);
    {
    fprintf(_coverage_fout, "2988\n");
    fflush(_coverage_fout);
    }
    __dest___0->len = __nl___1;
    break;
  }
  {
  fprintf(_coverage_fout, "3035\n");
  fflush(_coverage_fout);
  }
  members.refcount__gc = 1U;
  {
  fprintf(_coverage_fout, "3036\n");
  fflush(_coverage_fout);
  }
  members.is_ref__gc = (unsigned char)0;
  {
  fprintf(_coverage_fout, "3037\n");
  fflush(_coverage_fout);
  }
  if (! intern->std.properties) {
    {
    fprintf(_coverage_fout, "2989\n");
    fflush(_coverage_fout);
    }
    rebuild_object_properties(& intern->std);
  } else {
    {
    fprintf(_coverage_fout, "2990\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "3038\n");
  fflush(_coverage_fout);
  }
  members.value.ht = intern->std.properties;
  {
  fprintf(_coverage_fout, "3039\n");
  fflush(_coverage_fout);
  }
  members.type = (unsigned char)4;
  {
  fprintf(_coverage_fout, "3040\n");
  fflush(_coverage_fout);
  }
  pmembers = & members;
  {
  fprintf(_coverage_fout, "3041\n");
  fflush(_coverage_fout);
  }
  php_var_serialize(& buf, & pmembers, & var_hash);
  {
  fprintf(_coverage_fout, "3042\n");
  fflush(_coverage_fout);
  }
  while (1) {
    {
    fprintf(_coverage_fout, "3002\n");
    fflush(_coverage_fout);
    }
    if (basic_globals.serialize_lock) {
      {
      fprintf(_coverage_fout, "2991\n");
      fflush(_coverage_fout);
      }
      zend_hash_destroy(var_hash);
      {
      fprintf(_coverage_fout, "2992\n");
      fflush(_coverage_fout);
      }
      _efree((void *)var_hash);
    } else {
      {
      fprintf(_coverage_fout, "3001\n");
      fflush(_coverage_fout);
      }
      if (! basic_globals.serialize.level) {
        {
        fprintf(_coverage_fout, "2993\n");
        fflush(_coverage_fout);
        }
        zend_hash_destroy(var_hash);
        {
        fprintf(_coverage_fout, "2994\n");
        fflush(_coverage_fout);
        }
        _efree((void *)var_hash);
      } else {
        {
        fprintf(_coverage_fout, "2999\n");
        fflush(_coverage_fout);
        }
        (basic_globals.serialize.level) --;
        {
        fprintf(_coverage_fout, "3000\n");
        fflush(_coverage_fout);
        }
        if (basic_globals.serialize.level) {
          {
          fprintf(_coverage_fout, "2995\n");
          fflush(_coverage_fout);
          }

        } else {
          {
          fprintf(_coverage_fout, "2996\n");
          fflush(_coverage_fout);
          }
          zend_hash_destroy((HashTable *)basic_globals.serialize.var_hash);
          {
          fprintf(_coverage_fout, "2997\n");
          fflush(_coverage_fout);
          }
          _efree((void *)((HashTable *)basic_globals.serialize.var_hash));
          {
          fprintf(_coverage_fout, "2998\n");
          fflush(_coverage_fout);
          }
          basic_globals.serialize.var_hash = (void *)0;
        }
      }
    }
    break;
  }
  {
  fprintf(_coverage_fout, "3043\n");
  fflush(_coverage_fout);
  }
  if (buf.c) {
    {
    fprintf(_coverage_fout, "3009\n");
    fflush(_coverage_fout);
    }
    while (1) {
      {
      fprintf(_coverage_fout, "3003\n");
      fflush(_coverage_fout);
      }
      __s = (char const   *)buf.c;
      {
      fprintf(_coverage_fout, "3004\n");
      fflush(_coverage_fout);
      }
      __l = (int )buf.len;
      {
      fprintf(_coverage_fout, "3005\n");
      fflush(_coverage_fout);
      }
      __z___0 = return_value;
      {
      fprintf(_coverage_fout, "3006\n");
      fflush(_coverage_fout);
      }
      __z___0->value.str.len = __l;
      {
      fprintf(_coverage_fout, "3007\n");
      fflush(_coverage_fout);
      }
      __z___0->value.str.val = (char *)((char __attribute__((__visibility__("default")))  *)((char *)__s));
      {
      fprintf(_coverage_fout, "3008\n");
      fflush(_coverage_fout);
      }
      __z___0->type = (unsigned char)6;
      break;
    }
    {
    fprintf(_coverage_fout, "3010\n");
    fflush(_coverage_fout);
    }
    return;
  } else {
    {
    fprintf(_coverage_fout, "3011\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "3044\n");
  fflush(_coverage_fout);
  }
  return_value->type = (unsigned char)0;
  {
  fprintf(_coverage_fout, "3045\n");
  fflush(_coverage_fout);
  }
  return;
}
}
__inline static int spl_instantiate_arg_ex2(zend_class_entry *pce ,
                                            zval **retval , int alloc ,
                                            zval *arg1 , zval *arg2 ) ;
extern zend_class_entry __attribute__((__visibility__("default")))  *zend_get_class_entry(zval const   *zobject ) ;
__inline static zend_uint ( __attribute__((__always_inline__)) zval_set_refcount_p)(zval *pz ,
                                                                                    zend_uint rc ) ;
__inline static zend_bool ( __attribute__((__always_inline__)) zval_set_isref_to_p)(zval *pz ,
                                                                                    zend_bool isref ) ;
__inline static zend_uint ( __attribute__((__always_inline__)) zval_refcount_p)(zval *pz ) ;
__inline static zend_bool ( __attribute__((__always_inline__)) zval_isref_p)(zval *pz ) ;
extern zend_bool __attribute__((__visibility__("default")))  instanceof_function(zend_class_entry const   *instance_ce ,
                                                                                 zend_class_entry const   *ce ) ;
extern int __attribute__((__visibility__("default")))  zend_hash_get_current_data_ex(HashTable *ht ,
                                                                                     void **pData ,
                                                                                     HashPosition *pos ) ;
int __attribute__((__visibility__("default")))  spl_hash_verify_pos_ex(spl_array_object *intern ,
                                                                       HashTable *ht ) ;
void zim_spl_Array_getChildren(int ht , zval *return_value ,
                               zval **return_value_ptr , zval *this_ptr ,
                               int return_value_used ) 
{ zval *object ;
  zval **entry ;
  zval *flags ;
  spl_array_object *intern ;
  void __attribute__((__visibility__("default")))  *tmp ;
  HashTable *aht ;
  HashTable *tmp___0 ;
  int __attribute__((__visibility__("default")))  tmp___1 ;
  int __attribute__((__visibility__("default")))  tmp___2 ;
  int __attribute__((__visibility__("default")))  tmp___3 ;
  zend_uchar is_ref ;
  zend_bool tmp___4 ;
  zend_uint refcount ;
  zend_uint tmp___5 ;
  zend_class_entry __attribute__((__visibility__("default")))  *tmp___6 ;
  zend_class_entry __attribute__((__visibility__("default")))  *tmp___7 ;
  zend_bool __attribute__((__visibility__("default")))  tmp___8 ;
  void __attribute__((__visibility__("default")))  *tmp___9 ;
  zval *__z ;
  zend_class_entry __attribute__((__visibility__("default")))  *tmp___10 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "2892\n");
  fflush(_coverage_fout);
  }
  object = this_ptr;
  {
  fprintf(_coverage_fout, "2893\n");
  fflush(_coverage_fout);
  }
  tmp = zend_object_store_get_object((zval const   *)object);
  {
  fprintf(_coverage_fout, "2894\n");
  fflush(_coverage_fout);
  }
  intern = (spl_array_object *)tmp;
  {
  fprintf(_coverage_fout, "2895\n");
  fflush(_coverage_fout);
  }
  tmp___0 = spl_array_get_hash_table(intern, 0);
  {
  fprintf(_coverage_fout, "2896\n");
  fflush(_coverage_fout);
  }
  aht = tmp___0;
  {
  fprintf(_coverage_fout, "2897\n");
  fflush(_coverage_fout);
  }
  tmp___1 = zend_parse_parameters(ht, "");
  {
  fprintf(_coverage_fout, "2898\n");
  fflush(_coverage_fout);
  }
  if (tmp___1 == (int __attribute__((__visibility__("default")))  )-1) {
    {
    fprintf(_coverage_fout, "2857\n");
    fflush(_coverage_fout);
    }
    return;
  } else {
    {
    fprintf(_coverage_fout, "2858\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2899\n");
  fflush(_coverage_fout);
  }
  if (! aht) {
    {
    fprintf(_coverage_fout, "2859\n");
    fflush(_coverage_fout);
    }
    php_error_docref0((char const   *)((void *)0), 1 << 3L,
                      "Array was modified outside object and is no longer an array");
    {
    fprintf(_coverage_fout, "2860\n");
    fflush(_coverage_fout);
    }
    return;
  } else {
    {
    fprintf(_coverage_fout, "2861\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2900\n");
  fflush(_coverage_fout);
  }
  if (intern->ar_flags & 0x01000000) {
    {
    fprintf(_coverage_fout, "2865\n");
    fflush(_coverage_fout);
    }
    tmp___2 = spl_hash_verify_pos_ex(intern, aht);
    {
    fprintf(_coverage_fout, "2866\n");
    fflush(_coverage_fout);
    }
    if (tmp___2 == (int __attribute__((__visibility__("default")))  )-1) {
      {
      fprintf(_coverage_fout, "2862\n");
      fflush(_coverage_fout);
      }
      php_error_docref0((char const   *)((void *)0), 1 << 3L,
                        "Array was modified outside object and internal position is no longer valid");
      {
      fprintf(_coverage_fout, "2863\n");
      fflush(_coverage_fout);
      }
      return;
    } else {
      {
      fprintf(_coverage_fout, "2864\n");
      fflush(_coverage_fout);
      }

    }
  } else {
    {
    fprintf(_coverage_fout, "2867\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2901\n");
  fflush(_coverage_fout);
  }
  tmp___3 = zend_hash_get_current_data_ex(aht, (void **)(& entry), & intern->pos);
  {
  fprintf(_coverage_fout, "2902\n");
  fflush(_coverage_fout);
  }
  if (tmp___3 == (int __attribute__((__visibility__("default")))  )-1) {
    {
    fprintf(_coverage_fout, "2868\n");
    fflush(_coverage_fout);
    }
    return;
  } else {
    {
    fprintf(_coverage_fout, "2869\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2903\n");
  fflush(_coverage_fout);
  }
  if ((int )(*entry)->type == 5) {
    {
    fprintf(_coverage_fout, "2883\n");
    fflush(_coverage_fout);
    }
    if ((intern->ar_flags & 0x00000004) != 0) {
      {
      fprintf(_coverage_fout, "2870\n");
      fflush(_coverage_fout);
      }
      return;
    } else {
      {
      fprintf(_coverage_fout, "2871\n");
      fflush(_coverage_fout);
      }

    }
    {
    fprintf(_coverage_fout, "2884\n");
    fflush(_coverage_fout);
    }
    tmp___6 = zend_get_class_entry((zval const   *)this_ptr);
    {
    fprintf(_coverage_fout, "2885\n");
    fflush(_coverage_fout);
    }
    tmp___7 = zend_get_class_entry((zval const   *)*entry);
    {
    fprintf(_coverage_fout, "2886\n");
    fflush(_coverage_fout);
    }
    tmp___8 = instanceof_function((zend_class_entry const   *)tmp___7,
                                  (zend_class_entry const   *)tmp___6);
    {
    fprintf(_coverage_fout, "2887\n");
    fflush(_coverage_fout);
    }
    if (tmp___8) {
      {
      fprintf(_coverage_fout, "2874\n");
      fflush(_coverage_fout);
      }
      tmp___4 = zval_isref_p(return_value);
      {
      fprintf(_coverage_fout, "2875\n");
      fflush(_coverage_fout);
      }
      is_ref = tmp___4;
      {
      fprintf(_coverage_fout, "2876\n");
      fflush(_coverage_fout);
      }
      tmp___5 = zval_refcount_p(return_value);
      {
      fprintf(_coverage_fout, "2877\n");
      fflush(_coverage_fout);
      }
      refcount = tmp___5;
      {
      fprintf(_coverage_fout, "2878\n");
      fflush(_coverage_fout);
      }
      while (1) {
        {
        fprintf(_coverage_fout, "2872\n");
        fflush(_coverage_fout);
        }
        return_value->value = (*entry)->value;
        {
        fprintf(_coverage_fout, "2873\n");
        fflush(_coverage_fout);
        }
        return_value->type = (*entry)->type;
        break;
      }
      {
      fprintf(_coverage_fout, "2879\n");
      fflush(_coverage_fout);
      }
      zval_set_isref_to_p(return_value, is_ref);
      {
      fprintf(_coverage_fout, "2880\n");
      fflush(_coverage_fout);
      }
      zval_set_refcount_p(return_value, refcount);
      {
      fprintf(_coverage_fout, "2881\n");
      fflush(_coverage_fout);
      }
      return;
    } else {
      {
      fprintf(_coverage_fout, "2882\n");
      fflush(_coverage_fout);
      }

    }
  } else {
    {
    fprintf(_coverage_fout, "2888\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2904\n");
  fflush(_coverage_fout);
  }
  while (1) {
    {
    fprintf(_coverage_fout, "2889\n");
    fflush(_coverage_fout);
    }
    tmp___9 = _emalloc(sizeof(zval_gc_info ));
    {
    fprintf(_coverage_fout, "2890\n");
    fflush(_coverage_fout);
    }
    flags = (zval *)tmp___9;
    {
    fprintf(_coverage_fout, "2891\n");
    fflush(_coverage_fout);
    }
    ((zval_gc_info *)flags)->u.buffered = (gc_root_buffer *)((void *)0);
    break;
  }
  {
  fprintf(_coverage_fout, "2905\n");
  fflush(_coverage_fout);
  }
  flags->refcount__gc = 1U;
  {
  fprintf(_coverage_fout, "2906\n");
  fflush(_coverage_fout);
  }
  flags->is_ref__gc = (unsigned char)0;
  {
  fprintf(_coverage_fout, "2907\n");
  fflush(_coverage_fout);
  }
  __z = flags;
  {
  fprintf(_coverage_fout, "2908\n");
  fflush(_coverage_fout);
  }
  __z->value.lval = (long )(0x04000000 | intern->ar_flags);
  {
  fprintf(_coverage_fout, "2909\n");
  fflush(_coverage_fout);
  }
  __z->type = (unsigned char)1;
  {
  fprintf(_coverage_fout, "2910\n");
  fflush(_coverage_fout);
  }
  tmp___10 = zend_get_class_entry((zval const   *)this_ptr);
  {
  fprintf(_coverage_fout, "2911\n");
  fflush(_coverage_fout);
  }
  spl_instantiate_arg_ex2((zend_class_entry *)tmp___10, & return_value, 0,
                          *entry, flags);
  {
  fprintf(_coverage_fout, "2912\n");
  fflush(_coverage_fout);
  }
  _zval_ptr_dtor(& flags);
  {
  fprintf(_coverage_fout, "2913\n");
  fflush(_coverage_fout);
  }
  return;
}
}
void zim_spl_Array_hasChildren(int ht , zval *return_value ,
                               zval **return_value_ptr , zval *this_ptr ,
                               int return_value_used ) 
{ zval *object ;
  zval **entry ;
  spl_array_object *intern ;
  void __attribute__((__visibility__("default")))  *tmp ;
  HashTable *aht ;
  HashTable *tmp___0 ;
  int __attribute__((__visibility__("default")))  tmp___1 ;
  zval *__z ;
  zval *__z___0 ;
  int __attribute__((__visibility__("default")))  tmp___2 ;
  zval *__z___1 ;
  int __attribute__((__visibility__("default")))  tmp___3 ;
  zval *__z___2 ;
  int tmp___4 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "2844\n");
  fflush(_coverage_fout);
  }
  object = this_ptr;
  {
  fprintf(_coverage_fout, "2845\n");
  fflush(_coverage_fout);
  }
  tmp = zend_object_store_get_object((zval const   *)object);
  {
  fprintf(_coverage_fout, "2846\n");
  fflush(_coverage_fout);
  }
  intern = (spl_array_object *)tmp;
  {
  fprintf(_coverage_fout, "2847\n");
  fflush(_coverage_fout);
  }
  tmp___0 = spl_array_get_hash_table(intern, 0);
  {
  fprintf(_coverage_fout, "2848\n");
  fflush(_coverage_fout);
  }
  aht = tmp___0;
  {
  fprintf(_coverage_fout, "2849\n");
  fflush(_coverage_fout);
  }
  tmp___1 = zend_parse_parameters(ht, "");
  {
  fprintf(_coverage_fout, "2850\n");
  fflush(_coverage_fout);
  }
  if (tmp___1 == (int __attribute__((__visibility__("default")))  )-1) {
    {
    fprintf(_coverage_fout, "2809\n");
    fflush(_coverage_fout);
    }
    return;
  } else {
    {
    fprintf(_coverage_fout, "2810\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2851\n");
  fflush(_coverage_fout);
  }
  if (! aht) {
    {
    fprintf(_coverage_fout, "2814\n");
    fflush(_coverage_fout);
    }
    php_error_docref0((char const   *)((void *)0), 1 << 3L,
                      "Array was modified outside object and is no longer an array");
    {
    fprintf(_coverage_fout, "2815\n");
    fflush(_coverage_fout);
    }
    while (1) {
      {
      fprintf(_coverage_fout, "2811\n");
      fflush(_coverage_fout);
      }
      __z = return_value;
      {
      fprintf(_coverage_fout, "2812\n");
      fflush(_coverage_fout);
      }
      __z->value.lval = 0L;
      {
      fprintf(_coverage_fout, "2813\n");
      fflush(_coverage_fout);
      }
      __z->type = (unsigned char)3;
      break;
    }
    {
    fprintf(_coverage_fout, "2816\n");
    fflush(_coverage_fout);
    }
    return;
  } else {
    {
    fprintf(_coverage_fout, "2817\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2852\n");
  fflush(_coverage_fout);
  }
  if (intern->ar_flags & 0x01000000) {
    {
    fprintf(_coverage_fout, "2825\n");
    fflush(_coverage_fout);
    }
    tmp___2 = spl_hash_verify_pos_ex(intern, aht);
    {
    fprintf(_coverage_fout, "2826\n");
    fflush(_coverage_fout);
    }
    if (tmp___2 == (int __attribute__((__visibility__("default")))  )-1) {
      {
      fprintf(_coverage_fout, "2821\n");
      fflush(_coverage_fout);
      }
      php_error_docref0((char const   *)((void *)0), 1 << 3L,
                        "Array was modified outside object and internal position is no longer valid");
      {
      fprintf(_coverage_fout, "2822\n");
      fflush(_coverage_fout);
      }
      while (1) {
        {
        fprintf(_coverage_fout, "2818\n");
        fflush(_coverage_fout);
        }
        __z___0 = return_value;
        {
        fprintf(_coverage_fout, "2819\n");
        fflush(_coverage_fout);
        }
        __z___0->value.lval = 0L;
        {
        fprintf(_coverage_fout, "2820\n");
        fflush(_coverage_fout);
        }
        __z___0->type = (unsigned char)3;
        break;
      }
      {
      fprintf(_coverage_fout, "2823\n");
      fflush(_coverage_fout);
      }
      return;
    } else {
      {
      fprintf(_coverage_fout, "2824\n");
      fflush(_coverage_fout);
      }

    }
  } else {
    {
    fprintf(_coverage_fout, "2827\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2853\n");
  fflush(_coverage_fout);
  }
  tmp___3 = zend_hash_get_current_data_ex(aht, (void **)(& entry), & intern->pos);
  {
  fprintf(_coverage_fout, "2854\n");
  fflush(_coverage_fout);
  }
  if (tmp___3 == (int __attribute__((__visibility__("default")))  )-1) {
    {
    fprintf(_coverage_fout, "2831\n");
    fflush(_coverage_fout);
    }
    while (1) {
      {
      fprintf(_coverage_fout, "2828\n");
      fflush(_coverage_fout);
      }
      __z___1 = return_value;
      {
      fprintf(_coverage_fout, "2829\n");
      fflush(_coverage_fout);
      }
      __z___1->value.lval = 0L;
      {
      fprintf(_coverage_fout, "2830\n");
      fflush(_coverage_fout);
      }
      __z___1->type = (unsigned char)3;
      break;
    }
    {
    fprintf(_coverage_fout, "2832\n");
    fflush(_coverage_fout);
    }
    return;
  } else {
    {
    fprintf(_coverage_fout, "2833\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2855\n");
  fflush(_coverage_fout);
  }
  while (1) {
    {
    fprintf(_coverage_fout, "2840\n");
    fflush(_coverage_fout);
    }
    __z___2 = return_value;
    {
    fprintf(_coverage_fout, "2841\n");
    fflush(_coverage_fout);
    }
    if ((int )(*entry)->type == 4) {
      {
      fprintf(_coverage_fout, "2834\n");
      fflush(_coverage_fout);
      }
      tmp___4 = 1;
    } else {
      {
      fprintf(_coverage_fout, "2839\n");
      fflush(_coverage_fout);
      }
      if ((int )(*entry)->type == 5) {
        {
        fprintf(_coverage_fout, "2837\n");
        fflush(_coverage_fout);
        }
        if ((intern->ar_flags & 0x00000004) == 0) {
          {
          fprintf(_coverage_fout, "2835\n");
          fflush(_coverage_fout);
          }
          tmp___4 = 1;
        } else {
          {
          fprintf(_coverage_fout, "2836\n");
          fflush(_coverage_fout);
          }
          tmp___4 = 0;
        }
      } else {
        {
        fprintf(_coverage_fout, "2838\n");
        fflush(_coverage_fout);
        }
        tmp___4 = 0;
      }
    }
    {
    fprintf(_coverage_fout, "2842\n");
    fflush(_coverage_fout);
    }
    __z___2->value.lval = (long )(tmp___4 != 0);
    {
    fprintf(_coverage_fout, "2843\n");
    fflush(_coverage_fout);
    }
    __z___2->type = (unsigned char)3;
    break;
  }
  {
  fprintf(_coverage_fout, "2856\n");
  fflush(_coverage_fout);
  }
  return;
}
}
extern int __attribute__((__visibility__("default")))  zend_hash_get_current_key_type_ex(HashTable *ht ,
                                                                                         HashPosition *pos ) ;
void zim_spl_Array_valid(int ht , zval *return_value , zval **return_value_ptr ,
                         zval *this_ptr , int return_value_used ) 
{ zval *object ;
  spl_array_object *intern ;
  void __attribute__((__visibility__("default")))  *tmp ;
  HashTable *aht ;
  HashTable *tmp___0 ;
  int __attribute__((__visibility__("default")))  tmp___1 ;
  zval *__z ;
  zval *__z___0 ;
  int __attribute__((__visibility__("default")))  tmp___2 ;
  int tmp___3 ;
  int __attribute__((__visibility__("default")))  tmp___4 ;
  int __attribute__((__visibility__("default")))  tmp___5 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "2800\n");
  fflush(_coverage_fout);
  }
  object = this_ptr;
  {
  fprintf(_coverage_fout, "2801\n");
  fflush(_coverage_fout);
  }
  tmp = zend_object_store_get_object((zval const   *)object);
  {
  fprintf(_coverage_fout, "2802\n");
  fflush(_coverage_fout);
  }
  intern = (spl_array_object *)tmp;
  {
  fprintf(_coverage_fout, "2803\n");
  fflush(_coverage_fout);
  }
  tmp___0 = spl_array_get_hash_table(intern, 0);
  {
  fprintf(_coverage_fout, "2804\n");
  fflush(_coverage_fout);
  }
  aht = tmp___0;
  {
  fprintf(_coverage_fout, "2805\n");
  fflush(_coverage_fout);
  }
  tmp___1 = zend_parse_parameters(ht, "");
  {
  fprintf(_coverage_fout, "2806\n");
  fflush(_coverage_fout);
  }
  if (tmp___1 == (int __attribute__((__visibility__("default")))  )-1) {
    {
    fprintf(_coverage_fout, "2777\n");
    fflush(_coverage_fout);
    }
    return;
  } else {
    {
    fprintf(_coverage_fout, "2778\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2807\n");
  fflush(_coverage_fout);
  }
  if (! aht) {
    {
    fprintf(_coverage_fout, "2779\n");
    fflush(_coverage_fout);
    }
    php_error_docref0((char const   *)((void *)0), 1 << 3L,
                      "Array was modified outside object and is no longer an array");
    {
    fprintf(_coverage_fout, "2780\n");
    fflush(_coverage_fout);
    }
    return;
  } else {
    {
    fprintf(_coverage_fout, "2781\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2808\n");
  fflush(_coverage_fout);
  }
  if (intern->pos) {
    {
    fprintf(_coverage_fout, "2790\n");
    fflush(_coverage_fout);
    }
    if (intern->ar_flags & 0x01000000) {
      {
      fprintf(_coverage_fout, "2788\n");
      fflush(_coverage_fout);
      }
      tmp___5 = spl_hash_verify_pos_ex(intern, aht);
      {
      fprintf(_coverage_fout, "2789\n");
      fflush(_coverage_fout);
      }
      if (tmp___5 == (int __attribute__((__visibility__("default")))  )-1) {
        {
        fprintf(_coverage_fout, "2785\n");
        fflush(_coverage_fout);
        }
        php_error_docref0((char const   *)((void *)0), 1 << 3L,
                          "Array was modified outside object and internal position is no longer valid");
        {
        fprintf(_coverage_fout, "2786\n");
        fflush(_coverage_fout);
        }
        while (1) {
          {
          fprintf(_coverage_fout, "2782\n");
          fflush(_coverage_fout);
          }
          __z = return_value;
          {
          fprintf(_coverage_fout, "2783\n");
          fflush(_coverage_fout);
          }
          __z->value.lval = 0L;
          {
          fprintf(_coverage_fout, "2784\n");
          fflush(_coverage_fout);
          }
          __z->type = (unsigned char)3;
          break;
        }
        {
        fprintf(_coverage_fout, "2787\n");
        fflush(_coverage_fout);
        }
        return;
      } else {
        goto _L___0;
      }
    } else {
      goto _L___0;
    }
  } else {
    {
    fprintf(_coverage_fout, "2798\n");
    fflush(_coverage_fout);
    }
    _L___0: /* CIL Label */ 
    _L: /* CIL Label */ 
    while (1) {
      {
      fprintf(_coverage_fout, "2793\n");
      fflush(_coverage_fout);
      }
      __z___0 = return_value;
      {
      fprintf(_coverage_fout, "2794\n");
      fflush(_coverage_fout);
      }
      tmp___4 = zend_hash_get_current_key_type_ex(aht, & intern->pos);
      {
      fprintf(_coverage_fout, "2795\n");
      fflush(_coverage_fout);
      }
      if (tmp___4 == (int __attribute__((__visibility__("default")))  )3) {
        {
        fprintf(_coverage_fout, "2791\n");
        fflush(_coverage_fout);
        }
        tmp___3 = -1;
      } else {
        {
        fprintf(_coverage_fout, "2792\n");
        fflush(_coverage_fout);
        }
        tmp___3 = 0;
      }
      {
      fprintf(_coverage_fout, "2796\n");
      fflush(_coverage_fout);
      }
      __z___0->value.lval = (long )((tmp___3 == 0) != 0);
      {
      fprintf(_coverage_fout, "2797\n");
      fflush(_coverage_fout);
      }
      __z___0->type = (unsigned char)3;
      break;
    }
    {
    fprintf(_coverage_fout, "2799\n");
    fflush(_coverage_fout);
    }
    return;
  }
}
}
static int spl_array_next_ex(spl_array_object *intern , HashTable *aht ) ;
void zim_spl_Array_next(int ht , zval *return_value , zval **return_value_ptr ,
                        zval *this_ptr , int return_value_used ) 
{ zval *object ;
  spl_array_object *intern ;
  void __attribute__((__visibility__("default")))  *tmp ;
  HashTable *aht ;
  HashTable *tmp___0 ;
  int __attribute__((__visibility__("default")))  tmp___1 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "2767\n");
  fflush(_coverage_fout);
  }
  object = this_ptr;
  {
  fprintf(_coverage_fout, "2768\n");
  fflush(_coverage_fout);
  }
  tmp = zend_object_store_get_object((zval const   *)object);
  {
  fprintf(_coverage_fout, "2769\n");
  fflush(_coverage_fout);
  }
  intern = (spl_array_object *)tmp;
  {
  fprintf(_coverage_fout, "2770\n");
  fflush(_coverage_fout);
  }
  tmp___0 = spl_array_get_hash_table(intern, 0);
  {
  fprintf(_coverage_fout, "2771\n");
  fflush(_coverage_fout);
  }
  aht = tmp___0;
  {
  fprintf(_coverage_fout, "2772\n");
  fflush(_coverage_fout);
  }
  tmp___1 = zend_parse_parameters(ht, "");
  {
  fprintf(_coverage_fout, "2773\n");
  fflush(_coverage_fout);
  }
  if (tmp___1 == (int __attribute__((__visibility__("default")))  )-1) {
    {
    fprintf(_coverage_fout, "2762\n");
    fflush(_coverage_fout);
    }
    return;
  } else {
    {
    fprintf(_coverage_fout, "2763\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2774\n");
  fflush(_coverage_fout);
  }
  if (! aht) {
    {
    fprintf(_coverage_fout, "2764\n");
    fflush(_coverage_fout);
    }
    php_error_docref0((char const   *)((void *)0), 1 << 3L,
                      "Array was modified outside object and is no longer an array");
    {
    fprintf(_coverage_fout, "2765\n");
    fflush(_coverage_fout);
    }
    return;
  } else {
    {
    fprintf(_coverage_fout, "2766\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2775\n");
  fflush(_coverage_fout);
  }
  spl_array_next_ex(intern, aht);
  {
  fprintf(_coverage_fout, "2776\n");
  fflush(_coverage_fout);
  }
  return;
}
}
extern int __attribute__((__visibility__("default")))  zend_hash_get_current_key_ex(HashTable const   *ht ,
                                                                                    char **str_index ,
                                                                                    uint *str_length ,
                                                                                    ulong *num_index ,
                                                                                    zend_bool duplicate ,
                                                                                    HashPosition *pos ) ;
void spl_array_iterator_key(zval *object , zval *return_value ) 
{ spl_array_object *intern ;
  void __attribute__((__visibility__("default")))  *tmp ;
  char *string_key ;
  uint string_length ;
  ulong num_key ;
  HashTable *aht ;
  HashTable *tmp___0 ;
  int __attribute__((__visibility__("default")))  tmp___1 ;
  int __attribute__((__visibility__("default")))  tmp___2 ;
  char const   *__s ;
  int __l ;
  zval *__z ;
  char __attribute__((__visibility__("default")))  *tmp___3 ;
  zval *__z___0 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "2754\n");
  fflush(_coverage_fout);
  }
  tmp = zend_object_store_get_object((zval const   *)object);
  {
  fprintf(_coverage_fout, "2755\n");
  fflush(_coverage_fout);
  }
  intern = (spl_array_object *)tmp;
  {
  fprintf(_coverage_fout, "2756\n");
  fflush(_coverage_fout);
  }
  tmp___0 = spl_array_get_hash_table(intern, 0);
  {
  fprintf(_coverage_fout, "2757\n");
  fflush(_coverage_fout);
  }
  aht = tmp___0;
  {
  fprintf(_coverage_fout, "2758\n");
  fflush(_coverage_fout);
  }
  if (! aht) {
    {
    fprintf(_coverage_fout, "2734\n");
    fflush(_coverage_fout);
    }
    php_error_docref0((char const   *)((void *)0), 1 << 3L,
                      "Array was modified outside object and is no longer an array");
    {
    fprintf(_coverage_fout, "2735\n");
    fflush(_coverage_fout);
    }
    return;
  } else {
    {
    fprintf(_coverage_fout, "2736\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2759\n");
  fflush(_coverage_fout);
  }
  if (intern->ar_flags & 0x01000000) {
    {
    fprintf(_coverage_fout, "2740\n");
    fflush(_coverage_fout);
    }
    tmp___1 = spl_hash_verify_pos_ex(intern, aht);
    {
    fprintf(_coverage_fout, "2741\n");
    fflush(_coverage_fout);
    }
    if (tmp___1 == (int __attribute__((__visibility__("default")))  )-1) {
      {
      fprintf(_coverage_fout, "2737\n");
      fflush(_coverage_fout);
      }
      php_error_docref0((char const   *)((void *)0), 1 << 3L,
                        "Array was modified outside object and internal position is no longer valid");
      {
      fprintf(_coverage_fout, "2738\n");
      fflush(_coverage_fout);
      }
      return;
    } else {
      {
      fprintf(_coverage_fout, "2739\n");
      fflush(_coverage_fout);
      }

    }
  } else {
    {
    fprintf(_coverage_fout, "2742\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2760\n");
  fflush(_coverage_fout);
  }
  tmp___2 = zend_hash_get_current_key_ex((HashTable const   *)aht, & string_key,
                                         & string_length, & num_key,
                                         (unsigned char)1, & intern->pos);
  switch ((int )tmp___2) {
  {
  fprintf(_coverage_fout, "2749\n");
  fflush(_coverage_fout);
  }
  case 1: 
  while (1) {
    {
    fprintf(_coverage_fout, "2743\n");
    fflush(_coverage_fout);
    }
    __s = (char const   *)string_key;
    {
    fprintf(_coverage_fout, "2744\n");
    fflush(_coverage_fout);
    }
    __l = (int )(string_length - 1U);
    {
    fprintf(_coverage_fout, "2745\n");
    fflush(_coverage_fout);
    }
    __z = return_value;
    {
    fprintf(_coverage_fout, "2746\n");
    fflush(_coverage_fout);
    }
    __z->value.str.len = __l;
    {
    fprintf(_coverage_fout, "2747\n");
    fflush(_coverage_fout);
    }
    __z->value.str.val = (char *)((char __attribute__((__visibility__("default")))  *)((char *)__s));
    {
    fprintf(_coverage_fout, "2748\n");
    fflush(_coverage_fout);
    }
    __z->type = (unsigned char)6;
    break;
  }
  break;
  {
  fprintf(_coverage_fout, "2750\n");
  fflush(_coverage_fout);
  }
  case 2: 
  __z___0 = return_value;
  {
  fprintf(_coverage_fout, "2751\n");
  fflush(_coverage_fout);
  }
  __z___0->value.lval = (long )num_key;
  {
  fprintf(_coverage_fout, "2752\n");
  fflush(_coverage_fout);
  }
  __z___0->type = (unsigned char)1;
  break;
  {
  fprintf(_coverage_fout, "2753\n");
  fflush(_coverage_fout);
  }
  case 3: 
  return;
  }
  {
  fprintf(_coverage_fout, "2761\n");
  fflush(_coverage_fout);
  }
  return;
}
}
void spl_array_iterator_key(zval *object , zval *return_value ) ;
void zim_spl_Array_key(int ht , zval *return_value , zval **return_value_ptr ,
                       zval *this_ptr , int return_value_used ) 
{ int __attribute__((__visibility__("default")))  tmp ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "2730\n");
  fflush(_coverage_fout);
  }
  tmp = zend_parse_parameters(ht, "");
  {
  fprintf(_coverage_fout, "2731\n");
  fflush(_coverage_fout);
  }
  if (tmp == (int __attribute__((__visibility__("default")))  )-1) {
    {
    fprintf(_coverage_fout, "2728\n");
    fflush(_coverage_fout);
    }
    return;
  } else {
    {
    fprintf(_coverage_fout, "2729\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2732\n");
  fflush(_coverage_fout);
  }
  spl_array_iterator_key(this_ptr, return_value);
  {
  fprintf(_coverage_fout, "2733\n");
  fflush(_coverage_fout);
  }
  return;
}
}
__inline static void ( __attribute__((__always_inline__)) _zval_copy_ctor)(zval *zvalue ) ;
void zim_spl_Array_current(int ht , zval *return_value ,
                           zval **return_value_ptr , zval *this_ptr ,
                           int return_value_used ) 
{ zval *object ;
  spl_array_object *intern ;
  void __attribute__((__visibility__("default")))  *tmp ;
  zval **entry ;
  HashTable *aht ;
  HashTable *tmp___0 ;
  int __attribute__((__visibility__("default")))  tmp___1 ;
  int __attribute__((__visibility__("default")))  tmp___2 ;
  int __attribute__((__visibility__("default")))  tmp___3 ;
  zend_uchar is_ref ;
  zend_bool tmp___4 ;
  zend_uint refcount ;
  zend_uint tmp___5 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "2708\n");
  fflush(_coverage_fout);
  }
  object = this_ptr;
  {
  fprintf(_coverage_fout, "2709\n");
  fflush(_coverage_fout);
  }
  tmp = zend_object_store_get_object((zval const   *)object);
  {
  fprintf(_coverage_fout, "2710\n");
  fflush(_coverage_fout);
  }
  intern = (spl_array_object *)tmp;
  {
  fprintf(_coverage_fout, "2711\n");
  fflush(_coverage_fout);
  }
  tmp___0 = spl_array_get_hash_table(intern, 0);
  {
  fprintf(_coverage_fout, "2712\n");
  fflush(_coverage_fout);
  }
  aht = tmp___0;
  {
  fprintf(_coverage_fout, "2713\n");
  fflush(_coverage_fout);
  }
  tmp___1 = zend_parse_parameters(ht, "");
  {
  fprintf(_coverage_fout, "2714\n");
  fflush(_coverage_fout);
  }
  if (tmp___1 == (int __attribute__((__visibility__("default")))  )-1) {
    {
    fprintf(_coverage_fout, "2693\n");
    fflush(_coverage_fout);
    }
    return;
  } else {
    {
    fprintf(_coverage_fout, "2694\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2715\n");
  fflush(_coverage_fout);
  }
  if (! aht) {
    {
    fprintf(_coverage_fout, "2695\n");
    fflush(_coverage_fout);
    }
    php_error_docref0((char const   *)((void *)0), 1 << 3L,
                      "Array was modified outside object and is no longer an array");
    {
    fprintf(_coverage_fout, "2696\n");
    fflush(_coverage_fout);
    }
    return;
  } else {
    {
    fprintf(_coverage_fout, "2697\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2716\n");
  fflush(_coverage_fout);
  }
  if (intern->ar_flags & 0x01000000) {
    {
    fprintf(_coverage_fout, "2701\n");
    fflush(_coverage_fout);
    }
    tmp___2 = spl_hash_verify_pos_ex(intern, aht);
    {
    fprintf(_coverage_fout, "2702\n");
    fflush(_coverage_fout);
    }
    if (tmp___2 == (int __attribute__((__visibility__("default")))  )-1) {
      {
      fprintf(_coverage_fout, "2698\n");
      fflush(_coverage_fout);
      }
      php_error_docref0((char const   *)((void *)0), 1 << 3L,
                        "Array was modified outside object and internal position is no longer valid");
      {
      fprintf(_coverage_fout, "2699\n");
      fflush(_coverage_fout);
      }
      return;
    } else {
      {
      fprintf(_coverage_fout, "2700\n");
      fflush(_coverage_fout);
      }

    }
  } else {
    {
    fprintf(_coverage_fout, "2703\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2717\n");
  fflush(_coverage_fout);
  }
  tmp___3 = zend_hash_get_current_data_ex(aht, (void **)(& entry), & intern->pos);
  {
  fprintf(_coverage_fout, "2718\n");
  fflush(_coverage_fout);
  }
  if (tmp___3 == (int __attribute__((__visibility__("default")))  )-1) {
    {
    fprintf(_coverage_fout, "2704\n");
    fflush(_coverage_fout);
    }
    return;
  } else {
    {
    fprintf(_coverage_fout, "2705\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2719\n");
  fflush(_coverage_fout);
  }
  tmp___4 = zval_isref_p(return_value);
  {
  fprintf(_coverage_fout, "2720\n");
  fflush(_coverage_fout);
  }
  is_ref = tmp___4;
  {
  fprintf(_coverage_fout, "2721\n");
  fflush(_coverage_fout);
  }
  tmp___5 = zval_refcount_p(return_value);
  {
  fprintf(_coverage_fout, "2722\n");
  fflush(_coverage_fout);
  }
  refcount = tmp___5;
  {
  fprintf(_coverage_fout, "2723\n");
  fflush(_coverage_fout);
  }
  while (1) {
    {
    fprintf(_coverage_fout, "2706\n");
    fflush(_coverage_fout);
    }
    return_value->value = (*entry)->value;
    {
    fprintf(_coverage_fout, "2707\n");
    fflush(_coverage_fout);
    }
    return_value->type = (*entry)->type;
    break;
  }
  {
  fprintf(_coverage_fout, "2724\n");
  fflush(_coverage_fout);
  }
  _zval_copy_ctor(return_value);
  {
  fprintf(_coverage_fout, "2725\n");
  fflush(_coverage_fout);
  }
  zval_set_isref_to_p(return_value, is_ref);
  {
  fprintf(_coverage_fout, "2726\n");
  fflush(_coverage_fout);
  }
  zval_set_refcount_p(return_value, refcount);
  {
  fprintf(_coverage_fout, "2727\n");
  fflush(_coverage_fout);
  }
  return;
}
}
static void spl_array_method(int ht , zval *return_value ,
                             zval **return_value_ptr , zval *this_ptr ,
                             int return_value_used , char *fname ,
                             int fname_len , int use_arg ) ;
void zim_spl_Array_natcasesort(int ht , zval *return_value ,
                               zval **return_value_ptr , zval *this_ptr ,
                               int return_value_used ) 
{ 

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "2691\n");
  fflush(_coverage_fout);
  }
  spl_array_method(ht, return_value, return_value_ptr, this_ptr,
                   return_value_used, (char *)"natcasesort",
                   (int )(sizeof("natcasesort") - 1U), 0);
  {
  fprintf(_coverage_fout, "2692\n");
  fflush(_coverage_fout);
  }
  return;
}
}
void zim_spl_Array_natsort(int ht , zval *return_value ,
                           zval **return_value_ptr , zval *this_ptr ,
                           int return_value_used ) 
{ 

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "2689\n");
  fflush(_coverage_fout);
  }
  spl_array_method(ht, return_value, return_value_ptr, this_ptr,
                   return_value_used, (char *)"natsort",
                   (int )(sizeof("natsort") - 1U), 0);
  {
  fprintf(_coverage_fout, "2690\n");
  fflush(_coverage_fout);
  }
  return;
}
}
void zim_spl_Array_uksort(int ht , zval *return_value ,
                          zval **return_value_ptr , zval *this_ptr ,
                          int return_value_used ) 
{ 

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "2687\n");
  fflush(_coverage_fout);
  }
  spl_array_method(ht, return_value, return_value_ptr, this_ptr,
                   return_value_used, (char *)"uksort",
                   (int )(sizeof("uksort") - 1U), 1);
  {
  fprintf(_coverage_fout, "2688\n");
  fflush(_coverage_fout);
  }
  return;
}
}
void zim_spl_Array_uasort(int ht , zval *return_value ,
                          zval **return_value_ptr , zval *this_ptr ,
                          int return_value_used ) 
{ 

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "2685\n");
  fflush(_coverage_fout);
  }
  spl_array_method(ht, return_value, return_value_ptr, this_ptr,
                   return_value_used, (char *)"uasort",
                   (int )(sizeof("uasort") - 1U), 1);
  {
  fprintf(_coverage_fout, "2686\n");
  fflush(_coverage_fout);
  }
  return;
}
}
void zim_spl_Array_ksort(int ht , zval *return_value , zval **return_value_ptr ,
                         zval *this_ptr , int return_value_used ) 
{ 

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "2683\n");
  fflush(_coverage_fout);
  }
  spl_array_method(ht, return_value, return_value_ptr, this_ptr,
                   return_value_used, (char *)"ksort",
                   (int )(sizeof("ksort") - 1U), 0);
  {
  fprintf(_coverage_fout, "2684\n");
  fflush(_coverage_fout);
  }
  return;
}
}
void zim_spl_Array_asort(int ht , zval *return_value , zval **return_value_ptr ,
                         zval *this_ptr , int return_value_used ) 
{ 

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "2681\n");
  fflush(_coverage_fout);
  }
  spl_array_method(ht, return_value, return_value_ptr, this_ptr,
                   return_value_used, (char *)"asort",
                   (int )(sizeof("asort") - 1U), 0);
  {
  fprintf(_coverage_fout, "2682\n");
  fflush(_coverage_fout);
  }
  return;
}
}
extern void __attribute__((__visibility__("default")))  gc_remove_zval_from_buffer(zval *zv ) ;
__inline static zend_uint ( __attribute__((__always_inline__)) zval_delref_p)(zval *pz ) ;
extern zval __attribute__((__visibility__("default")))  *zend_call_method(zval **object_pp ,
                                                                          zend_class_entry *obj_ce ,
                                                                          zend_function **fn_proxy ,
                                                                          char const   *function_name ,
                                                                          int function_name_len ,
                                                                          zval **retval_ptr_ptr ,
                                                                          int param_count ,
                                                                          zval *arg1 ,
                                                                          zval *arg2 ) ;
extern zend_class_entry __attribute__((__visibility__("default")))  *spl_ce_BadMethodCallException ;
extern zval __attribute__((__visibility__("default")))  *zend_throw_exception(zend_class_entry *exception_ce ,
                                                                              char *message ,
                                                                              long code ) ;
extern int __attribute__((__visibility__("default")))  zend_parse_parameters_ex(int flags ,
                                                                                int num_args ,
                                                                                char const   *type_spec 
                                                                                , ...) ;
static void spl_array_method(int ht , zval *return_value ,
                             zval **return_value_ptr , zval *this_ptr ,
                             int return_value_used , char *fname ,
                             int fname_len , int use_arg ) 
{ spl_array_object *intern ;
  void __attribute__((__visibility__("default")))  *tmp ;
  HashTable *aht ;
  HashTable *tmp___0 ;
  zval *tmp___1 ;
  zval *arg ;
  zval *retval_ptr ;
  void __attribute__((__visibility__("default")))  *tmp___2 ;
  int __attribute__((__visibility__("default")))  tmp___3 ;
  zend_uint tmp___4 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "2666\n");
  fflush(_coverage_fout);
  }
  tmp = zend_object_store_get_object((zval const   *)this_ptr);
  {
  fprintf(_coverage_fout, "2667\n");
  fflush(_coverage_fout);
  }
  intern = (spl_array_object *)tmp;
  {
  fprintf(_coverage_fout, "2668\n");
  fflush(_coverage_fout);
  }
  tmp___0 = spl_array_get_hash_table(intern, 0);
  {
  fprintf(_coverage_fout, "2669\n");
  fflush(_coverage_fout);
  }
  aht = tmp___0;
  {
  fprintf(_coverage_fout, "2670\n");
  fflush(_coverage_fout);
  }
  retval_ptr = (zval *)((void *)0);
  {
  fprintf(_coverage_fout, "2671\n");
  fflush(_coverage_fout);
  }
  while (1) {
    {
    fprintf(_coverage_fout, "2632\n");
    fflush(_coverage_fout);
    }
    tmp___2 = _emalloc(sizeof(zval_gc_info ));
    {
    fprintf(_coverage_fout, "2633\n");
    fflush(_coverage_fout);
    }
    tmp___1 = (zval *)tmp___2;
    {
    fprintf(_coverage_fout, "2634\n");
    fflush(_coverage_fout);
    }
    ((zval_gc_info *)tmp___1)->u.buffered = (gc_root_buffer *)((void *)0);
    break;
  }
  {
  fprintf(_coverage_fout, "2672\n");
  fflush(_coverage_fout);
  }
  tmp___1->refcount__gc = 1U;
  {
  fprintf(_coverage_fout, "2673\n");
  fflush(_coverage_fout);
  }
  tmp___1->is_ref__gc = (unsigned char)0;
  {
  fprintf(_coverage_fout, "2674\n");
  fflush(_coverage_fout);
  }
  tmp___1->type = (unsigned char)4;
  {
  fprintf(_coverage_fout, "2675\n");
  fflush(_coverage_fout);
  }
  tmp___1->value.ht = aht;
  {
  fprintf(_coverage_fout, "2676\n");
  fflush(_coverage_fout);
  }
  if (use_arg) {
    {
    fprintf(_coverage_fout, "2646\n");
    fflush(_coverage_fout);
    }
    if (ht != 1) {
      {
      fprintf(_coverage_fout, "2635\n");
      fflush(_coverage_fout);
      }
      tmp___1->type = (unsigned char)0;
      {
      fprintf(_coverage_fout, "2636\n");
      fflush(_coverage_fout);
      }
      _zval_ptr_dtor(& tmp___1);
      {
      fprintf(_coverage_fout, "2637\n");
      fflush(_coverage_fout);
      }
      zend_throw_exception((zend_class_entry *)spl_ce_BadMethodCallException,
                           (char *)"Function expects exactly one argument", 0L);
      {
      fprintf(_coverage_fout, "2638\n");
      fflush(_coverage_fout);
      }
      return;
    } else {
      {
      fprintf(_coverage_fout, "2644\n");
      fflush(_coverage_fout);
      }
      tmp___3 = zend_parse_parameters_ex(1 << 1, ht, "z", & arg);
      {
      fprintf(_coverage_fout, "2645\n");
      fflush(_coverage_fout);
      }
      if (tmp___3 == (int __attribute__((__visibility__("default")))  )-1) {
        {
        fprintf(_coverage_fout, "2639\n");
        fflush(_coverage_fout);
        }
        tmp___1->type = (unsigned char)0;
        {
        fprintf(_coverage_fout, "2640\n");
        fflush(_coverage_fout);
        }
        _zval_ptr_dtor(& tmp___1);
        {
        fprintf(_coverage_fout, "2641\n");
        fflush(_coverage_fout);
        }
        zend_throw_exception((zend_class_entry *)spl_ce_BadMethodCallException,
                             (char *)"Function expects exactly one argument", 0L);
        {
        fprintf(_coverage_fout, "2642\n");
        fflush(_coverage_fout);
        }
        return;
      } else {
        {
        fprintf(_coverage_fout, "2643\n");
        fflush(_coverage_fout);
        }

      }
    }
    {
    fprintf(_coverage_fout, "2647\n");
    fflush(_coverage_fout);
    }
    aht->nApplyCount = (unsigned char )((int )aht->nApplyCount + 1);
    {
    fprintf(_coverage_fout, "2648\n");
    fflush(_coverage_fout);
    }
    zend_call_method((zval **)((void *)0), (zend_class_entry *)((void *)0),
                     (zend_function **)((void *)0), (char const   *)fname,
                     fname_len, & retval_ptr, 2, tmp___1, arg);
    {
    fprintf(_coverage_fout, "2649\n");
    fflush(_coverage_fout);
    }
    aht->nApplyCount = (unsigned char )((int )aht->nApplyCount - 1);
  } else {
    {
    fprintf(_coverage_fout, "2650\n");
    fflush(_coverage_fout);
    }
    aht->nApplyCount = (unsigned char )((int )aht->nApplyCount + 1);
    {
    fprintf(_coverage_fout, "2651\n");
    fflush(_coverage_fout);
    }
    zend_call_method((zval **)((void *)0), (zend_class_entry *)((void *)0),
                     (zend_function **)((void *)0), (char const   *)fname,
                     fname_len, & retval_ptr, 1, tmp___1, (zval *)((void *)0));
    {
    fprintf(_coverage_fout, "2652\n");
    fflush(_coverage_fout);
    }
    aht->nApplyCount = (unsigned char )((int )aht->nApplyCount - 1);
  }
  {
  fprintf(_coverage_fout, "2677\n");
  fflush(_coverage_fout);
  }
  tmp___1->type = (unsigned char)0;
  {
  fprintf(_coverage_fout, "2678\n");
  fflush(_coverage_fout);
  }
  _zval_ptr_dtor(& tmp___1);
  {
  fprintf(_coverage_fout, "2679\n");
  fflush(_coverage_fout);
  }
  if (retval_ptr) {
    {
    fprintf(_coverage_fout, "2660\n");
    fflush(_coverage_fout);
    }
    *return_value = *retval_ptr;
    {
    fprintf(_coverage_fout, "2661\n");
    fflush(_coverage_fout);
    }
    tmp___4 = zval_refcount_p(retval_ptr);
    {
    fprintf(_coverage_fout, "2662\n");
    fflush(_coverage_fout);
    }
    if (tmp___4 > 1U) {
      {
      fprintf(_coverage_fout, "2653\n");
      fflush(_coverage_fout);
      }
      _zval_copy_ctor(return_value);
      {
      fprintf(_coverage_fout, "2654\n");
      fflush(_coverage_fout);
      }
      zval_delref_p(retval_ptr);
    } else {
      {
      fprintf(_coverage_fout, "2659\n");
      fflush(_coverage_fout);
      }
      while (1) {
        {
        fprintf(_coverage_fout, "2657\n");
        fflush(_coverage_fout);
        }
        if ((gc_root_buffer *)((unsigned long )((zval_gc_info *)retval_ptr)->u.buffered & 4294967292UL)) {
          {
          fprintf(_coverage_fout, "2655\n");
          fflush(_coverage_fout);
          }
          gc_remove_zval_from_buffer(retval_ptr);
        } else {
          {
          fprintf(_coverage_fout, "2656\n");
          fflush(_coverage_fout);
          }

        }
        {
        fprintf(_coverage_fout, "2658\n");
        fflush(_coverage_fout);
        }
        _efree((void *)retval_ptr);
        break;
      }
    }
    {
    fprintf(_coverage_fout, "2663\n");
    fflush(_coverage_fout);
    }
    return_value->refcount__gc = 1U;
    {
    fprintf(_coverage_fout, "2664\n");
    fflush(_coverage_fout);
    }
    return_value->is_ref__gc = (unsigned char)0;
  } else {
    {
    fprintf(_coverage_fout, "2665\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2680\n");
  fflush(_coverage_fout);
  }
  return;
}
}
static int spl_array_object_count_elements_helper(spl_array_object *intern ,
                                                  long *count ) ;
void zim_spl_Array_count(int ht , zval *return_value , zval **return_value_ptr ,
                         zval *this_ptr , int return_value_used ) 
{ long count ;
  spl_array_object *intern ;
  void __attribute__((__visibility__("default")))  *tmp ;
  int __attribute__((__visibility__("default")))  tmp___0 ;
  zval *__z ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "2623\n");
  fflush(_coverage_fout);
  }
  tmp = zend_object_store_get_object((zval const   *)this_ptr);
  {
  fprintf(_coverage_fout, "2624\n");
  fflush(_coverage_fout);
  }
  intern = (spl_array_object *)tmp;
  {
  fprintf(_coverage_fout, "2625\n");
  fflush(_coverage_fout);
  }
  tmp___0 = zend_parse_parameters(ht, "");
  {
  fprintf(_coverage_fout, "2626\n");
  fflush(_coverage_fout);
  }
  if (tmp___0 == (int __attribute__((__visibility__("default")))  )-1) {
    {
    fprintf(_coverage_fout, "2621\n");
    fflush(_coverage_fout);
    }
    return;
  } else {
    {
    fprintf(_coverage_fout, "2622\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2627\n");
  fflush(_coverage_fout);
  }
  spl_array_object_count_elements_helper(intern, & count);
  {
  fprintf(_coverage_fout, "2628\n");
  fflush(_coverage_fout);
  }
  __z = return_value;
  {
  fprintf(_coverage_fout, "2629\n");
  fflush(_coverage_fout);
  }
  __z->value.lval = count;
  {
  fprintf(_coverage_fout, "2630\n");
  fflush(_coverage_fout);
  }
  __z->type = (unsigned char)1;
  {
  fprintf(_coverage_fout, "2631\n");
  fflush(_coverage_fout);
  }
  return;
}
}
extern void __attribute__((__visibility__("default")))  convert_to_long(zval *op ) ;
int spl_array_object_count_elements(zval *object , long *count ) 
{ spl_array_object *intern ;
  void __attribute__((__visibility__("default")))  *tmp ;
  zval *rv ;
  void __attribute__((__visibility__("default")))  *tmp___0 ;
  zend_uchar is_ref ;
  zend_bool tmp___1 ;
  zend_uint refcount ;
  zend_uint tmp___2 ;
  int tmp___3 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "2616\n");
  fflush(_coverage_fout);
  }
  tmp = zend_object_store_get_object((zval const   *)object);
  {
  fprintf(_coverage_fout, "2617\n");
  fflush(_coverage_fout);
  }
  intern = (spl_array_object *)tmp;
  {
  fprintf(_coverage_fout, "2618\n");
  fflush(_coverage_fout);
  }
  if (intern->fptr_count) {
    {
    fprintf(_coverage_fout, "2611\n");
    fflush(_coverage_fout);
    }
    zend_call_method(& object, intern->std.ce, & intern->fptr_count, "count",
                     (int )(sizeof("count") - 1U), & rv, 0, (zval *)((void *)0),
                     (zval *)((void *)0));
    {
    fprintf(_coverage_fout, "2612\n");
    fflush(_coverage_fout);
    }
    if (rv) {
      {
      fprintf(_coverage_fout, "2594\n");
      fflush(_coverage_fout);
      }
      _zval_ptr_dtor(& intern->retval);
      {
      fprintf(_coverage_fout, "2595\n");
      fflush(_coverage_fout);
      }
      while (1) {
        {
        fprintf(_coverage_fout, "2589\n");
        fflush(_coverage_fout);
        }
        tmp___0 = _emalloc(sizeof(zval_gc_info ));
        {
        fprintf(_coverage_fout, "2590\n");
        fflush(_coverage_fout);
        }
        intern->retval = (zval *)tmp___0;
        {
        fprintf(_coverage_fout, "2591\n");
        fflush(_coverage_fout);
        }
        ((zval_gc_info *)intern->retval)->u.buffered = (gc_root_buffer *)((void *)0);
        break;
      }
      {
      fprintf(_coverage_fout, "2596\n");
      fflush(_coverage_fout);
      }
      (intern->retval)->refcount__gc = 1U;
      {
      fprintf(_coverage_fout, "2597\n");
      fflush(_coverage_fout);
      }
      (intern->retval)->is_ref__gc = (unsigned char)0;
      {
      fprintf(_coverage_fout, "2598\n");
      fflush(_coverage_fout);
      }
      tmp___1 = zval_isref_p(intern->retval);
      {
      fprintf(_coverage_fout, "2599\n");
      fflush(_coverage_fout);
      }
      is_ref = tmp___1;
      {
      fprintf(_coverage_fout, "2600\n");
      fflush(_coverage_fout);
      }
      tmp___2 = zval_refcount_p(intern->retval);
      {
      fprintf(_coverage_fout, "2601\n");
      fflush(_coverage_fout);
      }
      refcount = tmp___2;
      {
      fprintf(_coverage_fout, "2602\n");
      fflush(_coverage_fout);
      }
      while (1) {
        {
        fprintf(_coverage_fout, "2592\n");
        fflush(_coverage_fout);
        }
        (intern->retval)->value = rv->value;
        {
        fprintf(_coverage_fout, "2593\n");
        fflush(_coverage_fout);
        }
        (intern->retval)->type = rv->type;
        break;
      }
      {
      fprintf(_coverage_fout, "2603\n");
      fflush(_coverage_fout);
      }
      _zval_copy_ctor(intern->retval);
      {
      fprintf(_coverage_fout, "2604\n");
      fflush(_coverage_fout);
      }
      _zval_ptr_dtor(& rv);
      {
      fprintf(_coverage_fout, "2605\n");
      fflush(_coverage_fout);
      }
      zval_set_isref_to_p(intern->retval, is_ref);
      {
      fprintf(_coverage_fout, "2606\n");
      fflush(_coverage_fout);
      }
      zval_set_refcount_p(intern->retval, refcount);
      {
      fprintf(_coverage_fout, "2607\n");
      fflush(_coverage_fout);
      }
      convert_to_long(intern->retval);
      {
      fprintf(_coverage_fout, "2608\n");
      fflush(_coverage_fout);
      }
      *count = (intern->retval)->value.lval;
      {
      fprintf(_coverage_fout, "2609\n");
      fflush(_coverage_fout);
      }
      return (0);
    } else {
      {
      fprintf(_coverage_fout, "2610\n");
      fflush(_coverage_fout);
      }

    }
    {
    fprintf(_coverage_fout, "2613\n");
    fflush(_coverage_fout);
    }
    *count = 0L;
    {
    fprintf(_coverage_fout, "2614\n");
    fflush(_coverage_fout);
    }
    return (-1);
  } else {
    {
    fprintf(_coverage_fout, "2615\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2619\n");
  fflush(_coverage_fout);
  }
  tmp___3 = spl_array_object_count_elements_helper(intern, count);
  {
  fprintf(_coverage_fout, "2620\n");
  fflush(_coverage_fout);
  }
  return (tmp___3);
}
}
extern int __attribute__((__visibility__("default")))  zend_hash_num_elements(HashTable const   *ht ) ;
static void spl_array_set_pos(spl_array_object *intern , HashPosition pos ) ;
static int spl_array_next(spl_array_object *intern ) ;
static void spl_array_rewind(spl_array_object *intern ) ;
static int spl_array_object_count_elements_helper(spl_array_object *intern ,
                                                  long *count ) 
{ HashTable *aht ;
  HashTable *tmp ;
  HashPosition pos ;
  int tmp___0 ;
  int __attribute__((__visibility__("default")))  tmp___1 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "2585\n");
  fflush(_coverage_fout);
  }
  tmp = spl_array_get_hash_table(intern, 0);
  {
  fprintf(_coverage_fout, "2586\n");
  fflush(_coverage_fout);
  }
  aht = tmp;
  {
  fprintf(_coverage_fout, "2587\n");
  fflush(_coverage_fout);
  }
  if (! aht) {
    {
    fprintf(_coverage_fout, "2567\n");
    fflush(_coverage_fout);
    }
    php_error_docref0((char const   *)((void *)0), 1 << 3L,
                      "Array was modified outside object and is no longer an array");
    {
    fprintf(_coverage_fout, "2568\n");
    fflush(_coverage_fout);
    }
    *count = 0L;
    {
    fprintf(_coverage_fout, "2569\n");
    fflush(_coverage_fout);
    }
    return (-1);
  } else {
    {
    fprintf(_coverage_fout, "2570\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2588\n");
  fflush(_coverage_fout);
  }
  if ((int )(intern->array)->type == 5) {
    {
    fprintf(_coverage_fout, "2576\n");
    fflush(_coverage_fout);
    }
    pos = intern->pos;
    {
    fprintf(_coverage_fout, "2577\n");
    fflush(_coverage_fout);
    }
    *count = 0L;
    {
    fprintf(_coverage_fout, "2578\n");
    fflush(_coverage_fout);
    }
    spl_array_rewind(intern);
    {
    fprintf(_coverage_fout, "2579\n");
    fflush(_coverage_fout);
    }
    while (1) {
      {
      fprintf(_coverage_fout, "2574\n");
      fflush(_coverage_fout);
      }
      if (intern->pos) {
        {
        fprintf(_coverage_fout, "2572\n");
        fflush(_coverage_fout);
        }
        tmp___0 = spl_array_next(intern);
        {
        fprintf(_coverage_fout, "2573\n");
        fflush(_coverage_fout);
        }
        if (tmp___0 == 0) {
          {
          fprintf(_coverage_fout, "2571\n");
          fflush(_coverage_fout);
          }

        } else {
          break;
        }
      } else {
        break;
      }
      {
      fprintf(_coverage_fout, "2575\n");
      fflush(_coverage_fout);
      }
      (*count) ++;
    }
    {
    fprintf(_coverage_fout, "2580\n");
    fflush(_coverage_fout);
    }
    spl_array_set_pos(intern, pos);
    {
    fprintf(_coverage_fout, "2581\n");
    fflush(_coverage_fout);
    }
    return (0);
  } else {
    {
    fprintf(_coverage_fout, "2582\n");
    fflush(_coverage_fout);
    }
    tmp___1 = zend_hash_num_elements((HashTable const   *)aht);
    {
    fprintf(_coverage_fout, "2583\n");
    fflush(_coverage_fout);
    }
    *count = (long )tmp___1;
    {
    fprintf(_coverage_fout, "2584\n");
    fflush(_coverage_fout);
    }
    return (0);
  }
}
}
extern zend_class_entry __attribute__((__visibility__("default")))  *spl_ce_OutOfBoundsException ;
void zim_spl_Array_seek(int ht , zval *return_value , zval **return_value_ptr ,
                        zval *this_ptr , int return_value_used ) 
{ long opos ;
  long position ;
  zval *object ;
  spl_array_object *intern ;
  void __attribute__((__visibility__("default")))  *tmp ;
  HashTable *aht ;
  HashTable *tmp___0 ;
  int result ;
  int __attribute__((__visibility__("default")))  tmp___1 ;
  long tmp___2 ;
  int __attribute__((__visibility__("default")))  tmp___3 ;
  int tmp___4 ;
  int __attribute__((__visibility__("default")))  tmp___5 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "2555\n");
  fflush(_coverage_fout);
  }
  object = this_ptr;
  {
  fprintf(_coverage_fout, "2556\n");
  fflush(_coverage_fout);
  }
  tmp = zend_object_store_get_object((zval const   *)object);
  {
  fprintf(_coverage_fout, "2557\n");
  fflush(_coverage_fout);
  }
  intern = (spl_array_object *)tmp;
  {
  fprintf(_coverage_fout, "2558\n");
  fflush(_coverage_fout);
  }
  tmp___0 = spl_array_get_hash_table(intern, 0);
  {
  fprintf(_coverage_fout, "2559\n");
  fflush(_coverage_fout);
  }
  aht = tmp___0;
  {
  fprintf(_coverage_fout, "2560\n");
  fflush(_coverage_fout);
  }
  tmp___1 = zend_parse_parameters(ht, "l", & position);
  {
  fprintf(_coverage_fout, "2561\n");
  fflush(_coverage_fout);
  }
  if (tmp___1 == (int __attribute__((__visibility__("default")))  )-1) {
    {
    fprintf(_coverage_fout, "2531\n");
    fflush(_coverage_fout);
    }
    return;
  } else {
    {
    fprintf(_coverage_fout, "2532\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2562\n");
  fflush(_coverage_fout);
  }
  if (! aht) {
    {
    fprintf(_coverage_fout, "2533\n");
    fflush(_coverage_fout);
    }
    php_error_docref0((char const   *)((void *)0), 1 << 3L,
                      "Array was modified outside object and is no longer an array");
    {
    fprintf(_coverage_fout, "2534\n");
    fflush(_coverage_fout);
    }
    return;
  } else {
    {
    fprintf(_coverage_fout, "2535\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2563\n");
  fflush(_coverage_fout);
  }
  opos = position;
  {
  fprintf(_coverage_fout, "2564\n");
  fflush(_coverage_fout);
  }
  if (position >= 0L) {
    {
    fprintf(_coverage_fout, "2550\n");
    fflush(_coverage_fout);
    }
    spl_array_rewind(intern);
    {
    fprintf(_coverage_fout, "2551\n");
    fflush(_coverage_fout);
    }
    result = 0;
    {
    fprintf(_coverage_fout, "2552\n");
    fflush(_coverage_fout);
    }
    while (1) {
      {
      fprintf(_coverage_fout, "2539\n");
      fflush(_coverage_fout);
      }
      tmp___2 = position;
      {
      fprintf(_coverage_fout, "2540\n");
      fflush(_coverage_fout);
      }
      position --;
      {
      fprintf(_coverage_fout, "2541\n");
      fflush(_coverage_fout);
      }
      if (tmp___2 > 0L) {
        {
        fprintf(_coverage_fout, "2537\n");
        fflush(_coverage_fout);
        }
        result = spl_array_next(intern);
        {
        fprintf(_coverage_fout, "2538\n");
        fflush(_coverage_fout);
        }
        if (result == 0) {
          {
          fprintf(_coverage_fout, "2536\n");
          fflush(_coverage_fout);
          }

        } else {
          break;
        }
      } else {
        break;
      }
    }
    {
    fprintf(_coverage_fout, "2553\n");
    fflush(_coverage_fout);
    }
    if (result == 0) {
      {
      fprintf(_coverage_fout, "2546\n");
      fflush(_coverage_fout);
      }
      tmp___5 = zend_hash_get_current_key_type_ex(aht, & intern->pos);
      {
      fprintf(_coverage_fout, "2547\n");
      fflush(_coverage_fout);
      }
      if (tmp___5 == (int __attribute__((__visibility__("default")))  )3) {
        {
        fprintf(_coverage_fout, "2542\n");
        fflush(_coverage_fout);
        }
        tmp___4 = -1;
      } else {
        {
        fprintf(_coverage_fout, "2543\n");
        fflush(_coverage_fout);
        }
        tmp___4 = 0;
      }
      {
      fprintf(_coverage_fout, "2548\n");
      fflush(_coverage_fout);
      }
      if (tmp___4 == 0) {
        {
        fprintf(_coverage_fout, "2544\n");
        fflush(_coverage_fout);
        }
        return;
      } else {
        {
        fprintf(_coverage_fout, "2545\n");
        fflush(_coverage_fout);
        }

      }
    } else {
      {
      fprintf(_coverage_fout, "2549\n");
      fflush(_coverage_fout);
      }

    }
  } else {
    {
    fprintf(_coverage_fout, "2554\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2565\n");
  fflush(_coverage_fout);
  }
  zend_throw_exception_ex((zend_class_entry *)spl_ce_OutOfBoundsException, 0L,
                          (char *)"Seek position %ld is out of range", opos);
  {
  fprintf(_coverage_fout, "2566\n");
  fflush(_coverage_fout);
  }
  return;
}
}
void zim_spl_Array_rewind(int ht , zval *return_value ,
                          zval **return_value_ptr , zval *this_ptr ,
                          int return_value_used ) 
{ zval *object ;
  spl_array_object *intern ;
  void __attribute__((__visibility__("default")))  *tmp ;
  int __attribute__((__visibility__("default")))  tmp___0 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "2524\n");
  fflush(_coverage_fout);
  }
  object = this_ptr;
  {
  fprintf(_coverage_fout, "2525\n");
  fflush(_coverage_fout);
  }
  tmp = zend_object_store_get_object((zval const   *)object);
  {
  fprintf(_coverage_fout, "2526\n");
  fflush(_coverage_fout);
  }
  intern = (spl_array_object *)tmp;
  {
  fprintf(_coverage_fout, "2527\n");
  fflush(_coverage_fout);
  }
  tmp___0 = zend_parse_parameters(ht, "");
  {
  fprintf(_coverage_fout, "2528\n");
  fflush(_coverage_fout);
  }
  if (tmp___0 == (int __attribute__((__visibility__("default")))  )-1) {
    {
    fprintf(_coverage_fout, "2522\n");
    fflush(_coverage_fout);
    }
    return;
  } else {
    {
    fprintf(_coverage_fout, "2523\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2529\n");
  fflush(_coverage_fout);
  }
  spl_array_rewind(intern);
  {
  fprintf(_coverage_fout, "2530\n");
  fflush(_coverage_fout);
  }
  return;
}
}
__inline static zend_bool ( __attribute__((__always_inline__)) zval_set_isref_p)(zval *pz ) ;
static zend_object_value spl_array_object_new_ex(zend_class_entry *class_type ,
                                                 spl_array_object **obj ,
                                                 zval *orig , int clone_orig ) ;
void zim_spl_Array_getIterator(int ht , zval *return_value ,
                               zval **return_value_ptr , zval *this_ptr ,
                               int return_value_used ) 
{ zval *object ;
  spl_array_object *intern ;
  void __attribute__((__visibility__("default")))  *tmp ;
  spl_array_object *iterator ;
  HashTable *aht ;
  HashTable *tmp___0 ;
  int __attribute__((__visibility__("default")))  tmp___1 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "2509\n");
  fflush(_coverage_fout);
  }
  object = this_ptr;
  {
  fprintf(_coverage_fout, "2510\n");
  fflush(_coverage_fout);
  }
  tmp = zend_object_store_get_object((zval const   *)object);
  {
  fprintf(_coverage_fout, "2511\n");
  fflush(_coverage_fout);
  }
  intern = (spl_array_object *)tmp;
  {
  fprintf(_coverage_fout, "2512\n");
  fflush(_coverage_fout);
  }
  tmp___0 = spl_array_get_hash_table(intern, 0);
  {
  fprintf(_coverage_fout, "2513\n");
  fflush(_coverage_fout);
  }
  aht = tmp___0;
  {
  fprintf(_coverage_fout, "2514\n");
  fflush(_coverage_fout);
  }
  tmp___1 = zend_parse_parameters(ht, "");
  {
  fprintf(_coverage_fout, "2515\n");
  fflush(_coverage_fout);
  }
  if (tmp___1 == (int __attribute__((__visibility__("default")))  )-1) {
    {
    fprintf(_coverage_fout, "2504\n");
    fflush(_coverage_fout);
    }
    return;
  } else {
    {
    fprintf(_coverage_fout, "2505\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2516\n");
  fflush(_coverage_fout);
  }
  if (! aht) {
    {
    fprintf(_coverage_fout, "2506\n");
    fflush(_coverage_fout);
    }
    php_error_docref0((char const   *)((void *)0), 1 << 3L,
                      "Array was modified outside object and is no longer an array");
    {
    fprintf(_coverage_fout, "2507\n");
    fflush(_coverage_fout);
    }
    return;
  } else {
    {
    fprintf(_coverage_fout, "2508\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2517\n");
  fflush(_coverage_fout);
  }
  return_value->type = (unsigned char)5;
  {
  fprintf(_coverage_fout, "2518\n");
  fflush(_coverage_fout);
  }
  return_value->value.obj = spl_array_object_new_ex(intern->ce_get_iterator,
                                                    & iterator, object, 0);
  {
  fprintf(_coverage_fout, "2519\n");
  fflush(_coverage_fout);
  }
  zval_set_refcount_p(return_value, 1U);
  {
  fprintf(_coverage_fout, "2520\n");
  fflush(_coverage_fout);
  }
  zval_set_isref_p(return_value);
  {
  fprintf(_coverage_fout, "2521\n");
  fflush(_coverage_fout);
  }
  return;
}
}
static void spl_array_set_array(zval *object , spl_array_object *intern ,
                                zval **array , long ar_flags , int just_array ) ;
extern int __attribute__((__visibility__("default")))  _array_init(zval *arg ,
                                                                   uint size ) ;
void zim_spl_Array_exchangeArray(int ht , zval *return_value ,
                                 zval **return_value_ptr , zval *this_ptr ,
                                 int return_value_used ) 
{ zval *object ;
  zval *tmp ;
  zval **array ;
  spl_array_object *intern ;
  void __attribute__((__visibility__("default")))  *tmp___0 ;
  HashTable *tmp___1 ;
  HashTable *tmp___2 ;
  HashTable *tmp___3 ;
  HashTable *tmp___4 ;
  int __attribute__((__visibility__("default")))  tmp___5 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "2493\n");
  fflush(_coverage_fout);
  }
  object = this_ptr;
  {
  fprintf(_coverage_fout, "2494\n");
  fflush(_coverage_fout);
  }
  tmp___0 = zend_object_store_get_object((zval const   *)object);
  {
  fprintf(_coverage_fout, "2495\n");
  fflush(_coverage_fout);
  }
  intern = (spl_array_object *)tmp___0;
  {
  fprintf(_coverage_fout, "2496\n");
  fflush(_coverage_fout);
  }
  _array_init(return_value, 0U);
  {
  fprintf(_coverage_fout, "2497\n");
  fflush(_coverage_fout);
  }
  tmp___1 = spl_array_get_hash_table(intern, 0);
  {
  fprintf(_coverage_fout, "2498\n");
  fflush(_coverage_fout);
  }
  if ((int )return_value->type == 4) {
    {
    fprintf(_coverage_fout, "2485\n");
    fflush(_coverage_fout);
    }
    tmp___4 = return_value->value.ht;
  } else {
    {
    fprintf(_coverage_fout, "2489\n");
    fflush(_coverage_fout);
    }
    if ((int )return_value->type == 5) {
      {
      fprintf(_coverage_fout, "2486\n");
      fflush(_coverage_fout);
      }
      tmp___2 = (*((return_value->value.obj.handlers)->get_properties))(return_value);
      {
      fprintf(_coverage_fout, "2487\n");
      fflush(_coverage_fout);
      }
      tmp___3 = tmp___2;
    } else {
      {
      fprintf(_coverage_fout, "2488\n");
      fflush(_coverage_fout);
      }
      tmp___3 = (HashTable *)((void *)0);
    }
    {
    fprintf(_coverage_fout, "2490\n");
    fflush(_coverage_fout);
    }
    tmp___4 = tmp___3;
  }
  {
  fprintf(_coverage_fout, "2499\n");
  fflush(_coverage_fout);
  }
  zend_hash_copy(tmp___4, tmp___1, (void (*)(void *pElement ))(& zval_add_ref),
                 (void *)(& tmp), sizeof(zval *));
  {
  fprintf(_coverage_fout, "2500\n");
  fflush(_coverage_fout);
  }
  tmp___5 = zend_parse_parameters(ht, "Z", & array);
  {
  fprintf(_coverage_fout, "2501\n");
  fflush(_coverage_fout);
  }
  if (tmp___5 == (int __attribute__((__visibility__("default")))  )-1) {
    {
    fprintf(_coverage_fout, "2491\n");
    fflush(_coverage_fout);
    }
    return;
  } else {
    {
    fprintf(_coverage_fout, "2492\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2502\n");
  fflush(_coverage_fout);
  }
  spl_array_set_array(object, intern, array, 0L, 1);
  {
  fprintf(_coverage_fout, "2503\n");
  fflush(_coverage_fout);
  }
  return;
}
}
void zim_spl_Array_setFlags(int ht , zval *return_value ,
                            zval **return_value_ptr , zval *this_ptr ,
                            int return_value_used ) 
{ zval *object ;
  spl_array_object *intern ;
  void __attribute__((__visibility__("default")))  *tmp ;
  long ar_flags ;
  int __attribute__((__visibility__("default")))  tmp___0 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "2477\n");
  fflush(_coverage_fout);
  }
  object = this_ptr;
  {
  fprintf(_coverage_fout, "2478\n");
  fflush(_coverage_fout);
  }
  tmp = zend_object_store_get_object((zval const   *)object);
  {
  fprintf(_coverage_fout, "2479\n");
  fflush(_coverage_fout);
  }
  intern = (spl_array_object *)tmp;
  {
  fprintf(_coverage_fout, "2480\n");
  fflush(_coverage_fout);
  }
  ar_flags = 0L;
  {
  fprintf(_coverage_fout, "2481\n");
  fflush(_coverage_fout);
  }
  tmp___0 = zend_parse_parameters(ht, "l", & ar_flags);
  {
  fprintf(_coverage_fout, "2482\n");
  fflush(_coverage_fout);
  }
  if (tmp___0 == (int __attribute__((__visibility__("default")))  )-1) {
    {
    fprintf(_coverage_fout, "2475\n");
    fflush(_coverage_fout);
    }
    return;
  } else {
    {
    fprintf(_coverage_fout, "2476\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2483\n");
  fflush(_coverage_fout);
  }
  intern->ar_flags = (int )((unsigned long )((unsigned int )intern->ar_flags & 0xFFFF0000) | ((unsigned long )ar_flags & 65535UL));
  {
  fprintf(_coverage_fout, "2484\n");
  fflush(_coverage_fout);
  }
  return;
}
}
void zim_spl_Array_getFlags(int ht , zval *return_value ,
                            zval **return_value_ptr , zval *this_ptr ,
                            int return_value_used ) 
{ zval *object ;
  spl_array_object *intern ;
  void __attribute__((__visibility__("default")))  *tmp ;
  int __attribute__((__visibility__("default")))  tmp___0 ;
  zval *__z ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "2466\n");
  fflush(_coverage_fout);
  }
  object = this_ptr;
  {
  fprintf(_coverage_fout, "2467\n");
  fflush(_coverage_fout);
  }
  tmp = zend_object_store_get_object((zval const   *)object);
  {
  fprintf(_coverage_fout, "2468\n");
  fflush(_coverage_fout);
  }
  intern = (spl_array_object *)tmp;
  {
  fprintf(_coverage_fout, "2469\n");
  fflush(_coverage_fout);
  }
  tmp___0 = zend_parse_parameters(ht, "");
  {
  fprintf(_coverage_fout, "2470\n");
  fflush(_coverage_fout);
  }
  if (tmp___0 == (int __attribute__((__visibility__("default")))  )-1) {
    {
    fprintf(_coverage_fout, "2464\n");
    fflush(_coverage_fout);
    }
    return;
  } else {
    {
    fprintf(_coverage_fout, "2465\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2471\n");
  fflush(_coverage_fout);
  }
  __z = return_value;
  {
  fprintf(_coverage_fout, "2472\n");
  fflush(_coverage_fout);
  }
  __z->value.lval = (long )((unsigned int )intern->ar_flags & 65535U);
  {
  fprintf(_coverage_fout, "2473\n");
  fflush(_coverage_fout);
  }
  __z->type = (unsigned char)1;
  {
  fprintf(_coverage_fout, "2474\n");
  fflush(_coverage_fout);
  }
  return;
}
}
extern char __attribute__((__visibility__("default")))  *_estrndup(char const   *s ,
                                                                   unsigned int length )  __attribute__((__malloc__)) ;
extern  __attribute__((__nothrow__)) size_t strlen(char const   *__s )  __attribute__((__pure__,
__nonnull__(1))) ;
void zim_spl_Array_getIteratorClass(int ht , zval *return_value ,
                                    zval **return_value_ptr , zval *this_ptr ,
                                    int return_value_used ) 
{ zval *object ;
  spl_array_object *intern ;
  void __attribute__((__visibility__("default")))  *tmp ;
  int __attribute__((__visibility__("default")))  tmp___0 ;
  char const   *__s ;
  zval *__z ;
  size_t tmp___1 ;
  char __attribute__((__visibility__("default")))  *tmp___2 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "2457\n");
  fflush(_coverage_fout);
  }
  object = this_ptr;
  {
  fprintf(_coverage_fout, "2458\n");
  fflush(_coverage_fout);
  }
  tmp = zend_object_store_get_object((zval const   *)object);
  {
  fprintf(_coverage_fout, "2459\n");
  fflush(_coverage_fout);
  }
  intern = (spl_array_object *)tmp;
  {
  fprintf(_coverage_fout, "2460\n");
  fflush(_coverage_fout);
  }
  tmp___0 = zend_parse_parameters(ht, "");
  {
  fprintf(_coverage_fout, "2461\n");
  fflush(_coverage_fout);
  }
  if (tmp___0 == (int __attribute__((__visibility__("default")))  )-1) {
    {
    fprintf(_coverage_fout, "2448\n");
    fflush(_coverage_fout);
    }
    return;
  } else {
    {
    fprintf(_coverage_fout, "2449\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2462\n");
  fflush(_coverage_fout);
  }
  while (1) {
    {
    fprintf(_coverage_fout, "2450\n");
    fflush(_coverage_fout);
    }
    __s = (intern->ce_get_iterator)->name;
    {
    fprintf(_coverage_fout, "2451\n");
    fflush(_coverage_fout);
    }
    __z = return_value;
    {
    fprintf(_coverage_fout, "2452\n");
    fflush(_coverage_fout);
    }
    tmp___1 = strlen(__s);
    {
    fprintf(_coverage_fout, "2453\n");
    fflush(_coverage_fout);
    }
    __z->value.str.len = (int )tmp___1;
    {
    fprintf(_coverage_fout, "2454\n");
    fflush(_coverage_fout);
    }
    tmp___2 = _estrndup(__s, (unsigned int )__z->value.str.len);
    {
    fprintf(_coverage_fout, "2455\n");
    fflush(_coverage_fout);
    }
    __z->value.str.val = (char *)tmp___2;
    {
    fprintf(_coverage_fout, "2456\n");
    fflush(_coverage_fout);
    }
    __z->type = (unsigned char)6;
    break;
  }
  {
  fprintf(_coverage_fout, "2463\n");
  fflush(_coverage_fout);
  }
  return;
}
}
void zim_spl_Array_setIteratorClass(int ht , zval *return_value ,
                                    zval **return_value_ptr , zval *this_ptr ,
                                    int return_value_used ) 
{ zval *object ;
  spl_array_object *intern ;
  void __attribute__((__visibility__("default")))  *tmp ;
  zend_class_entry *ce_get_iterator ;
  int __attribute__((__visibility__("default")))  tmp___0 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "2440\n");
  fflush(_coverage_fout);
  }
  object = this_ptr;
  {
  fprintf(_coverage_fout, "2441\n");
  fflush(_coverage_fout);
  }
  tmp = zend_object_store_get_object((zval const   *)object);
  {
  fprintf(_coverage_fout, "2442\n");
  fflush(_coverage_fout);
  }
  intern = (spl_array_object *)tmp;
  {
  fprintf(_coverage_fout, "2443\n");
  fflush(_coverage_fout);
  }
  ce_get_iterator = (zend_class_entry *)zend_ce_iterator;
  {
  fprintf(_coverage_fout, "2444\n");
  fflush(_coverage_fout);
  }
  tmp___0 = zend_parse_parameters(ht, "C", & ce_get_iterator);
  {
  fprintf(_coverage_fout, "2445\n");
  fflush(_coverage_fout);
  }
  if (tmp___0 == (int __attribute__((__visibility__("default")))  )-1) {
    {
    fprintf(_coverage_fout, "2438\n");
    fflush(_coverage_fout);
    }
    return;
  } else {
    {
    fprintf(_coverage_fout, "2439\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2446\n");
  fflush(_coverage_fout);
  }
  intern->ce_get_iterator = ce_get_iterator;
  {
  fprintf(_coverage_fout, "2447\n");
  fflush(_coverage_fout);
  }
  return;
}
}
enum __anonenum_zend_error_handling_t_47 {
    EH_NORMAL = 0,
    EH_SUPPRESS = 1,
    EH_THROW = 2
} ;
typedef enum __anonenum_zend_error_handling_t_47 zend_error_handling_t;
struct __anonstruct_zend_error_handling_48 {
   zend_error_handling_t handling ;
   zend_class_entry *exception ;
   zval *user_handler ;
};
typedef struct __anonstruct_zend_error_handling_48 zend_error_handling;
struct __anonstruct_zend_error_handling_48;
extern void __attribute__((__visibility__("default")))  zend_restore_error_handling(zend_error_handling *saved ) ;
extern zend_class_entry __attribute__((__visibility__("default")))  *spl_ce_InvalidArgumentException ;
enum __anonenum_zend_error_handling_t_47;
extern void __attribute__((__visibility__("default")))  zend_replace_error_handling(zend_error_handling_t error_handling ,
                                                                                    zend_class_entry *exception_class ,
                                                                                    zend_error_handling *current ) ;
void zim_spl_Array___construct(int ht , zval *return_value ,
                               zval **return_value_ptr , zval *this_ptr ,
                               int return_value_used ) 
{ zval *object ;
  spl_array_object *intern ;
  zval **array ;
  long ar_flags ;
  zend_class_entry *ce_get_iterator ;
  zend_error_handling error_handling ;
  void __attribute__((__visibility__("default")))  *tmp ;
  int __attribute__((__visibility__("default")))  tmp___0 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "2424\n");
  fflush(_coverage_fout);
  }
  object = this_ptr;
  {
  fprintf(_coverage_fout, "2425\n");
  fflush(_coverage_fout);
  }
  ar_flags = 0L;
  {
  fprintf(_coverage_fout, "2426\n");
  fflush(_coverage_fout);
  }
  ce_get_iterator = (zend_class_entry *)zend_ce_iterator;
  {
  fprintf(_coverage_fout, "2427\n");
  fflush(_coverage_fout);
  }
  if (ht == 0) {
    {
    fprintf(_coverage_fout, "2417\n");
    fflush(_coverage_fout);
    }
    return;
  } else {
    {
    fprintf(_coverage_fout, "2418\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2428\n");
  fflush(_coverage_fout);
  }
  zend_replace_error_handling((enum __anonenum_zend_error_handling_t_47 )2,
                              (zend_class_entry *)spl_ce_InvalidArgumentException,
                              & error_handling);
  {
  fprintf(_coverage_fout, "2429\n");
  fflush(_coverage_fout);
  }
  tmp = zend_object_store_get_object((zval const   *)object);
  {
  fprintf(_coverage_fout, "2430\n");
  fflush(_coverage_fout);
  }
  intern = (spl_array_object *)tmp;
  {
  fprintf(_coverage_fout, "2431\n");
  fflush(_coverage_fout);
  }
  tmp___0 = zend_parse_parameters(ht, "Z|lC", & array, & ar_flags,
                                  & ce_get_iterator);
  {
  fprintf(_coverage_fout, "2432\n");
  fflush(_coverage_fout);
  }
  if (tmp___0 == (int __attribute__((__visibility__("default")))  )-1) {
    {
    fprintf(_coverage_fout, "2419\n");
    fflush(_coverage_fout);
    }
    zend_restore_error_handling(& error_handling);
    {
    fprintf(_coverage_fout, "2420\n");
    fflush(_coverage_fout);
    }
    return;
  } else {
    {
    fprintf(_coverage_fout, "2421\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2433\n");
  fflush(_coverage_fout);
  }
  if (ht > 2) {
    {
    fprintf(_coverage_fout, "2422\n");
    fflush(_coverage_fout);
    }
    intern->ce_get_iterator = ce_get_iterator;
  } else {
    {
    fprintf(_coverage_fout, "2423\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2434\n");
  fflush(_coverage_fout);
  }
  ar_flags = (long )((unsigned long )ar_flags & 65535UL);
  {
  fprintf(_coverage_fout, "2435\n");
  fflush(_coverage_fout);
  }
  spl_array_set_array(object, intern, array, ar_flags, ht == 1);
  {
  fprintf(_coverage_fout, "2436\n");
  fflush(_coverage_fout);
  }
  zend_restore_error_handling(& error_handling);
  {
  fprintf(_coverage_fout, "2437\n");
  fflush(_coverage_fout);
  }
  return;
}
}
struct __anonstruct_spl_array_it_99;
typedef struct __anonstruct_spl_array_it_99 spl_array_it;
struct _zend_object_iterator {
   void *data ;
   zend_object_iterator_funcs *funcs ;
   ulong index ;
};
struct _zend_user_iterator {
   zend_object_iterator it ;
   zend_class_entry *ce ;
   zval *value ;
};
typedef struct _zend_user_iterator zend_user_iterator;
struct __anonstruct_spl_array_it_99 {
   zend_user_iterator intern ;
   spl_array_object *object ;
};
zend_object_iterator_funcs spl_array_it_funcs ;
struct _zend_object_iterator_funcs {
   void (*dtor)(zend_object_iterator *iter ) ;
   int (*valid)(zend_object_iterator *iter ) ;
   void (*get_current_data)(zend_object_iterator *iter , zval ***data ) ;
   int (*get_current_key)(zend_object_iterator *iter , char **str_key ,
                          uint *str_key_len , ulong *int_key ) ;
   void (*move_forward)(zend_object_iterator *iter ) ;
   void (*rewind)(zend_object_iterator *iter ) ;
   void (*invalidate_current)(zend_object_iterator *iter ) ;
};
__inline static zend_uint ( __attribute__((__always_inline__)) zval_addref_p)(zval *pz ) ;
extern void __attribute__((__visibility__("default")))  ( /* format attribute */  zend_error)(int type ,
                                                                                              char const   *format 
                                                                                              , ...) ;
zend_object_iterator *spl_array_get_iterator(zend_class_entry *ce ,
                                             zval *object , int by_ref ) 
{ spl_array_it *iterator ;
  spl_array_object *array_object ;
  void __attribute__((__visibility__("default")))  *tmp ;
  void __attribute__((__visibility__("default")))  *tmp___0 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "2405\n");
  fflush(_coverage_fout);
  }
  tmp = zend_object_store_get_object((zval const   *)object);
  {
  fprintf(_coverage_fout, "2406\n");
  fflush(_coverage_fout);
  }
  array_object = (spl_array_object *)tmp;
  {
  fprintf(_coverage_fout, "2407\n");
  fflush(_coverage_fout);
  }
  if (by_ref) {
    {
    fprintf(_coverage_fout, "2403\n");
    fflush(_coverage_fout);
    }
    if (array_object->ar_flags & 0x00080000) {
      {
      fprintf(_coverage_fout, "2401\n");
      fflush(_coverage_fout);
      }
      zend_error(1, "An iterator cannot be used with foreach by reference");
    } else {
      {
      fprintf(_coverage_fout, "2402\n");
      fflush(_coverage_fout);
      }

    }
  } else {
    {
    fprintf(_coverage_fout, "2404\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2408\n");
  fflush(_coverage_fout);
  }
  tmp___0 = _emalloc(sizeof(spl_array_it ));
  {
  fprintf(_coverage_fout, "2409\n");
  fflush(_coverage_fout);
  }
  iterator = (spl_array_it *)tmp___0;
  {
  fprintf(_coverage_fout, "2410\n");
  fflush(_coverage_fout);
  }
  zval_addref_p(object);
  {
  fprintf(_coverage_fout, "2411\n");
  fflush(_coverage_fout);
  }
  iterator->intern.it.data = (void *)object;
  {
  fprintf(_coverage_fout, "2412\n");
  fflush(_coverage_fout);
  }
  iterator->intern.it.funcs = & spl_array_it_funcs;
  {
  fprintf(_coverage_fout, "2413\n");
  fflush(_coverage_fout);
  }
  iterator->intern.ce = ce;
  {
  fprintf(_coverage_fout, "2414\n");
  fflush(_coverage_fout);
  }
  iterator->intern.value = (zval *)((void *)0);
  {
  fprintf(_coverage_fout, "2415\n");
  fflush(_coverage_fout);
  }
  iterator->object = array_object;
  {
  fprintf(_coverage_fout, "2416\n");
  fflush(_coverage_fout);
  }
  return ((zend_object_iterator *)iterator);
}
}
static void spl_array_it_rewind(zend_object_iterator *iter ) ;
static void spl_array_it_move_forward(zend_object_iterator *iter ) ;
static int spl_array_it_get_current_key(zend_object_iterator *iter ,
                                        char **str_key , uint *str_key_len ,
                                        ulong *int_key ) ;
static void spl_array_it_get_current_data(zend_object_iterator *iter ,
                                          zval ***data ) ;
static int spl_array_it_valid(zend_object_iterator *iter ) ;
static void spl_array_it_dtor(zend_object_iterator *iter ) ;
zend_object_iterator_funcs spl_array_it_funcs  =    {& spl_array_it_dtor, & spl_array_it_valid, & spl_array_it_get_current_data,
    & spl_array_it_get_current_key, & spl_array_it_move_forward,
    & spl_array_it_rewind, (void (*)(zend_object_iterator *iter ))0};
extern zend_object_handlers __attribute__((__visibility__("default")))  std_object_handlers ;
__inline static zend_bool ( __attribute__((__always_inline__)) zval_unset_isref_p)(zval *pz ) ;
static void spl_array_set_array(zval *object , spl_array_object *intern ,
                                zval **array , long ar_flags , int just_array ) 
{ zval *new_zv ;
  void __attribute__((__visibility__("default")))  *tmp ;
  zend_uint tmp___0 ;
  zend_bool tmp___1 ;
  spl_array_object *other ;
  void __attribute__((__visibility__("default")))  *tmp___2 ;
  HashTable *(*handler)(zval *object ) ;
  zend_class_entry __attribute__((__visibility__("default")))  *tmp___3 ;
  HashTable *tmp___4 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "2393\n");
  fflush(_coverage_fout);
  }
  if ((int )(*array)->type == 4) {
    {
    fprintf(_coverage_fout, "2358\n");
    fflush(_coverage_fout);
    }
    tmp___1 = zval_isref_p(*array);
    {
    fprintf(_coverage_fout, "2359\n");
    fflush(_coverage_fout);
    }
    if (tmp___1) {
      {
      fprintf(_coverage_fout, "2340\n");
      fflush(_coverage_fout);
      }

    } else {
      {
      fprintf(_coverage_fout, "2357\n");
      fflush(_coverage_fout);
      }
      while (1) {
        {
        fprintf(_coverage_fout, "2355\n");
        fflush(_coverage_fout);
        }
        tmp___0 = zval_refcount_p(*array);
        {
        fprintf(_coverage_fout, "2356\n");
        fflush(_coverage_fout);
        }
        if (tmp___0 > 1U) {
          {
          fprintf(_coverage_fout, "2349\n");
          fflush(_coverage_fout);
          }
          zval_delref_p(*array);
          {
          fprintf(_coverage_fout, "2350\n");
          fflush(_coverage_fout);
          }
          while (1) {
            {
            fprintf(_coverage_fout, "2341\n");
            fflush(_coverage_fout);
            }
            tmp = _emalloc(sizeof(zval_gc_info ));
            {
            fprintf(_coverage_fout, "2342\n");
            fflush(_coverage_fout);
            }
            new_zv = (zval *)tmp;
            {
            fprintf(_coverage_fout, "2343\n");
            fflush(_coverage_fout);
            }
            ((zval_gc_info *)new_zv)->u.buffered = (gc_root_buffer *)((void *)0);
            break;
          }
          {
          fprintf(_coverage_fout, "2351\n");
          fflush(_coverage_fout);
          }
          while (1) {
            {
            fprintf(_coverage_fout, "2346\n");
            fflush(_coverage_fout);
            }
            while (1) {
              {
              fprintf(_coverage_fout, "2344\n");
              fflush(_coverage_fout);
              }
              new_zv->value = (*array)->value;
              {
              fprintf(_coverage_fout, "2345\n");
              fflush(_coverage_fout);
              }
              new_zv->type = (*array)->type;
              break;
            }
            {
            fprintf(_coverage_fout, "2347\n");
            fflush(_coverage_fout);
            }
            zval_set_refcount_p(new_zv, 1U);
            {
            fprintf(_coverage_fout, "2348\n");
            fflush(_coverage_fout);
            }
            zval_unset_isref_p(new_zv);
            break;
          }
          {
          fprintf(_coverage_fout, "2352\n");
          fflush(_coverage_fout);
          }
          *array = new_zv;
          {
          fprintf(_coverage_fout, "2353\n");
          fflush(_coverage_fout);
          }
          _zval_copy_ctor(new_zv);
        } else {
          {
          fprintf(_coverage_fout, "2354\n");
          fflush(_coverage_fout);
          }

        }
        break;
      }
    }
  } else {
    {
    fprintf(_coverage_fout, "2360\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2394\n");
  fflush(_coverage_fout);
  }
  if ((int )(*array)->type == 5) {
    {
    fprintf(_coverage_fout, "2370\n");
    fflush(_coverage_fout);
    }
    if ((unsigned int )(*array)->value.obj.handlers == (unsigned int )(& spl_handler_ArrayObject)) {
      goto _L___0;
    } else {
      {
      fprintf(_coverage_fout, "2369\n");
      fflush(_coverage_fout);
      }
      if ((unsigned int )(*array)->value.obj.handlers == (unsigned int )(& spl_handler_ArrayIterator)) {
        {
        fprintf(_coverage_fout, "2365\n");
        fflush(_coverage_fout);
        }
        _L___0: /* CIL Label */ 
        _zval_ptr_dtor(& intern->array);
        {
        fprintf(_coverage_fout, "2366\n");
        fflush(_coverage_fout);
        }
        if (just_array) {
          {
          fprintf(_coverage_fout, "2361\n");
          fflush(_coverage_fout);
          }
          tmp___2 = zend_object_store_get_object((zval const   *)*array);
          {
          fprintf(_coverage_fout, "2362\n");
          fflush(_coverage_fout);
          }
          other = (spl_array_object *)tmp___2;
          {
          fprintf(_coverage_fout, "2363\n");
          fflush(_coverage_fout);
          }
          ar_flags = (long )((unsigned int )other->ar_flags & 65535U);
        } else {
          {
          fprintf(_coverage_fout, "2364\n");
          fflush(_coverage_fout);
          }

        }
        {
        fprintf(_coverage_fout, "2367\n");
        fflush(_coverage_fout);
        }
        ar_flags |= 67108864L;
        {
        fprintf(_coverage_fout, "2368\n");
        fflush(_coverage_fout);
        }
        intern->array = *array;
      } else {
        goto _L;
      }
    }
  } else {
    {
    fprintf(_coverage_fout, "2376\n");
    fflush(_coverage_fout);
    }
    _L: /* CIL Label */ 
    if ((int )(*array)->type != 5) {
      {
      fprintf(_coverage_fout, "2374\n");
      fflush(_coverage_fout);
      }
      if ((int )(*array)->type != 4) {
        {
        fprintf(_coverage_fout, "2371\n");
        fflush(_coverage_fout);
        }
        zend_throw_exception((zend_class_entry *)spl_ce_InvalidArgumentException,
                             (char *)"Passed variable is not an array or object, using empty array instead",
                             0L);
        {
        fprintf(_coverage_fout, "2372\n");
        fflush(_coverage_fout);
        }
        return;
      } else {
        {
        fprintf(_coverage_fout, "2373\n");
        fflush(_coverage_fout);
        }

      }
    } else {
      {
      fprintf(_coverage_fout, "2375\n");
      fflush(_coverage_fout);
      }

    }
    {
    fprintf(_coverage_fout, "2377\n");
    fflush(_coverage_fout);
    }
    _zval_ptr_dtor(& intern->array);
    {
    fprintf(_coverage_fout, "2378\n");
    fflush(_coverage_fout);
    }
    intern->array = *array;
  }
  {
  fprintf(_coverage_fout, "2395\n");
  fflush(_coverage_fout);
  }
  if ((unsigned int )object == (unsigned int )*array) {
    {
    fprintf(_coverage_fout, "2379\n");
    fflush(_coverage_fout);
    }
    intern->ar_flags |= 0x02000000;
    {
    fprintf(_coverage_fout, "2380\n");
    fflush(_coverage_fout);
    }
    intern->ar_flags &= -67108865;
  } else {
    {
    fprintf(_coverage_fout, "2381\n");
    fflush(_coverage_fout);
    }
    intern->ar_flags &= -33554433;
  }
  {
  fprintf(_coverage_fout, "2396\n");
  fflush(_coverage_fout);
  }
  intern->ar_flags = (int )((long )intern->ar_flags | ar_flags);
  {
  fprintf(_coverage_fout, "2397\n");
  fflush(_coverage_fout);
  }
  zval_addref_p(intern->array);
  {
  fprintf(_coverage_fout, "2398\n");
  fflush(_coverage_fout);
  }
  if ((int )(*array)->type == 5) {
    {
    fprintf(_coverage_fout, "2390\n");
    fflush(_coverage_fout);
    }
    handler = (HashTable *(*)(zval *object ))((*array)->value.obj.handlers)->get_properties;
    {
    fprintf(_coverage_fout, "2391\n");
    fflush(_coverage_fout);
    }
    if ((unsigned int )handler != (unsigned int )std_object_handlers.get_properties) {
      {
      fprintf(_coverage_fout, "2384\n");
      fflush(_coverage_fout);
      }
      if ((unsigned int )handler != (unsigned int )(& spl_array_get_properties)) {
        {
        fprintf(_coverage_fout, "2382\n");
        fflush(_coverage_fout);
        }
        tmp___3 = zend_get_class_entry((zval const   *)*array);
        {
        fprintf(_coverage_fout, "2383\n");
        fflush(_coverage_fout);
        }
        zend_throw_exception_ex((zend_class_entry *)spl_ce_InvalidArgumentException,
                                0L,
                                (char *)"Overloaded object of type %s is not compatible with %s",
                                tmp___3->name, (intern->std.ce)->name);
      } else {
        goto _L___1;
      }
    } else {
      {
      fprintf(_coverage_fout, "2388\n");
      fflush(_coverage_fout);
      }
      _L___1: /* CIL Label */ 
      tmp___4 = spl_array_get_hash_table(intern, 0);
      {
      fprintf(_coverage_fout, "2389\n");
      fflush(_coverage_fout);
      }
      if (tmp___4) {
        {
        fprintf(_coverage_fout, "2385\n");
        fflush(_coverage_fout);
        }

      } else {
        {
        fprintf(_coverage_fout, "2386\n");
        fflush(_coverage_fout);
        }
        tmp___3 = zend_get_class_entry((zval const   *)*array);
        {
        fprintf(_coverage_fout, "2387\n");
        fflush(_coverage_fout);
        }
        zend_throw_exception_ex((zend_class_entry *)spl_ce_InvalidArgumentException,
                                0L,
                                (char *)"Overloaded object of type %s is not compatible with %s",
                                tmp___3->name, (intern->std.ce)->name);
      }
    }
  } else {
    {
    fprintf(_coverage_fout, "2392\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2399\n");
  fflush(_coverage_fout);
  }
  spl_array_rewind(intern);
  {
  fprintf(_coverage_fout, "2400\n");
  fflush(_coverage_fout);
  }
  return;
}
}
extern void __attribute__((__visibility__("default")))  zend_user_it_invalidate_current(zend_object_iterator *_iter ) ;
extern void __attribute__((__visibility__("default")))  zend_user_it_rewind(zend_object_iterator *_iter ) ;
static void spl_array_it_rewind(zend_object_iterator *iter ) 
{ spl_array_it *iterator ;
  spl_array_object *object ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "2336\n");
  fflush(_coverage_fout);
  }
  iterator = (spl_array_it *)iter;
  {
  fprintf(_coverage_fout, "2337\n");
  fflush(_coverage_fout);
  }
  object = iterator->object;
  {
  fprintf(_coverage_fout, "2338\n");
  fflush(_coverage_fout);
  }
  if (object->ar_flags & 0x00010000) {
    {
    fprintf(_coverage_fout, "2333\n");
    fflush(_coverage_fout);
    }
    zend_user_it_rewind(iter);
  } else {
    {
    fprintf(_coverage_fout, "2334\n");
    fflush(_coverage_fout);
    }
    zend_user_it_invalidate_current(iter);
    {
    fprintf(_coverage_fout, "2335\n");
    fflush(_coverage_fout);
    }
    spl_array_rewind(object);
  }
  {
  fprintf(_coverage_fout, "2339\n");
  fflush(_coverage_fout);
  }
  return;
}
}
static void spl_array_rewind_ex(spl_array_object *intern , HashTable *aht ) ;
static void spl_array_rewind(spl_array_object *intern ) 
{ HashTable *aht ;
  HashTable *tmp ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "2328\n");
  fflush(_coverage_fout);
  }
  tmp = spl_array_get_hash_table(intern, 0);
  {
  fprintf(_coverage_fout, "2329\n");
  fflush(_coverage_fout);
  }
  aht = tmp;
  {
  fprintf(_coverage_fout, "2330\n");
  fflush(_coverage_fout);
  }
  if (! aht) {
    {
    fprintf(_coverage_fout, "2325\n");
    fflush(_coverage_fout);
    }
    php_error_docref0((char const   *)((void *)0), 1 << 3L,
                      "ArrayIterator::rewind(): Array was modified outside object and is no longer an array");
    {
    fprintf(_coverage_fout, "2326\n");
    fflush(_coverage_fout);
    }
    return;
  } else {
    {
    fprintf(_coverage_fout, "2327\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2331\n");
  fflush(_coverage_fout);
  }
  spl_array_rewind_ex(intern, aht);
  {
  fprintf(_coverage_fout, "2332\n");
  fflush(_coverage_fout);
  }
  return;
}
}
static int spl_array_skip_protected(spl_array_object *intern , HashTable *aht ) ;
static void spl_array_update_pos(spl_array_object *intern ) ;
extern void __attribute__((__visibility__("default")))  zend_hash_internal_pointer_reset_ex(HashTable *ht ,
                                                                                            HashPosition *pos ) ;
static void spl_array_rewind_ex(spl_array_object *intern , HashTable *aht ) 
{ 

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "2321\n");
  fflush(_coverage_fout);
  }
  zend_hash_internal_pointer_reset_ex(aht, & intern->pos);
  {
  fprintf(_coverage_fout, "2322\n");
  fflush(_coverage_fout);
  }
  spl_array_update_pos(intern);
  {
  fprintf(_coverage_fout, "2323\n");
  fflush(_coverage_fout);
  }
  spl_array_skip_protected(intern, aht);
  {
  fprintf(_coverage_fout, "2324\n");
  fflush(_coverage_fout);
  }
  return;
}
}
static int spl_array_next_no_verify(spl_array_object *intern , HashTable *aht ) ;
extern void __attribute__((__visibility__("default")))  zend_user_it_move_forward(zend_object_iterator *_iter ) ;
static void spl_array_it_move_forward(zend_object_iterator *iter ) 
{ spl_array_it *iterator ;
  spl_array_object *object ;
  HashTable *aht ;
  HashTable *tmp ;
  int __attribute__((__visibility__("default")))  tmp___0 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "2315\n");
  fflush(_coverage_fout);
  }
  iterator = (spl_array_it *)iter;
  {
  fprintf(_coverage_fout, "2316\n");
  fflush(_coverage_fout);
  }
  object = iterator->object;
  {
  fprintf(_coverage_fout, "2317\n");
  fflush(_coverage_fout);
  }
  tmp = spl_array_get_hash_table(object, 0);
  {
  fprintf(_coverage_fout, "2318\n");
  fflush(_coverage_fout);
  }
  aht = tmp;
  {
  fprintf(_coverage_fout, "2319\n");
  fflush(_coverage_fout);
  }
  if (object->ar_flags & 0x00100000) {
    {
    fprintf(_coverage_fout, "2303\n");
    fflush(_coverage_fout);
    }
    zend_user_it_move_forward(iter);
  } else {
    {
    fprintf(_coverage_fout, "2312\n");
    fflush(_coverage_fout);
    }
    zend_user_it_invalidate_current(iter);
    {
    fprintf(_coverage_fout, "2313\n");
    fflush(_coverage_fout);
    }
    if (! aht) {
      {
      fprintf(_coverage_fout, "2304\n");
      fflush(_coverage_fout);
      }
      php_error_docref0((char const   *)((void *)0), 1 << 3L,
                        "ArrayIterator::current(): Array was modified outside object and is no longer an array");
      {
      fprintf(_coverage_fout, "2305\n");
      fflush(_coverage_fout);
      }
      return;
    } else {
      {
      fprintf(_coverage_fout, "2306\n");
      fflush(_coverage_fout);
      }

    }
    {
    fprintf(_coverage_fout, "2314\n");
    fflush(_coverage_fout);
    }
    if (object->ar_flags & 0x01000000) {
      {
      fprintf(_coverage_fout, "2309\n");
      fflush(_coverage_fout);
      }
      tmp___0 = spl_hash_verify_pos_ex(object, aht);
      {
      fprintf(_coverage_fout, "2310\n");
      fflush(_coverage_fout);
      }
      if (tmp___0 == (int __attribute__((__visibility__("default")))  )-1) {
        {
        fprintf(_coverage_fout, "2307\n");
        fflush(_coverage_fout);
        }
        php_error_docref0((char const   *)((void *)0), 1 << 3L,
                          "ArrayIterator::next(): Array was modified outside object and internal position is no longer valid");
      } else {
        {
        fprintf(_coverage_fout, "2308\n");
        fflush(_coverage_fout);
        }
        spl_array_next_no_verify(object, aht);
      }
    } else {
      {
      fprintf(_coverage_fout, "2311\n");
      fflush(_coverage_fout);
      }
      spl_array_next_no_verify(object, aht);
    }
  }
  {
  fprintf(_coverage_fout, "2320\n");
  fflush(_coverage_fout);
  }
  return;
}
}
extern int __attribute__((__visibility__("default")))  zend_user_it_get_current_key(zend_object_iterator *_iter ,
                                                                                    char **str_key ,
                                                                                    uint *str_key_len ,
                                                                                    ulong *int_key ) ;
static int spl_array_it_get_current_key(zend_object_iterator *iter ,
                                        char **str_key , uint *str_key_len ,
                                        ulong *int_key ) 
{ spl_array_it *iterator ;
  spl_array_object *object ;
  HashTable *aht ;
  HashTable *tmp ;
  int __attribute__((__visibility__("default")))  tmp___0 ;
  int __attribute__((__visibility__("default")))  tmp___1 ;
  int __attribute__((__visibility__("default")))  tmp___2 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "2298\n");
  fflush(_coverage_fout);
  }
  iterator = (spl_array_it *)iter;
  {
  fprintf(_coverage_fout, "2299\n");
  fflush(_coverage_fout);
  }
  object = iterator->object;
  {
  fprintf(_coverage_fout, "2300\n");
  fflush(_coverage_fout);
  }
  tmp = spl_array_get_hash_table(object, 0);
  {
  fprintf(_coverage_fout, "2301\n");
  fflush(_coverage_fout);
  }
  aht = tmp;
  {
  fprintf(_coverage_fout, "2302\n");
  fflush(_coverage_fout);
  }
  if (object->ar_flags & 0x00040000) {
    {
    fprintf(_coverage_fout, "2283\n");
    fflush(_coverage_fout);
    }
    tmp___0 = zend_user_it_get_current_key(iter, str_key, str_key_len, int_key);
    {
    fprintf(_coverage_fout, "2284\n");
    fflush(_coverage_fout);
    }
    return ((int )tmp___0);
  } else {
    {
    fprintf(_coverage_fout, "2294\n");
    fflush(_coverage_fout);
    }
    if (! aht) {
      {
      fprintf(_coverage_fout, "2285\n");
      fflush(_coverage_fout);
      }
      php_error_docref0((char const   *)((void *)0), 1 << 3L,
                        "ArrayIterator::current(): Array was modified outside object and is no longer an array");
      {
      fprintf(_coverage_fout, "2286\n");
      fflush(_coverage_fout);
      }
      return (3);
    } else {
      {
      fprintf(_coverage_fout, "2287\n");
      fflush(_coverage_fout);
      }

    }
    {
    fprintf(_coverage_fout, "2295\n");
    fflush(_coverage_fout);
    }
    if (object->ar_flags & 0x01000000) {
      {
      fprintf(_coverage_fout, "2291\n");
      fflush(_coverage_fout);
      }
      tmp___1 = spl_hash_verify_pos_ex(object, aht);
      {
      fprintf(_coverage_fout, "2292\n");
      fflush(_coverage_fout);
      }
      if (tmp___1 == (int __attribute__((__visibility__("default")))  )-1) {
        {
        fprintf(_coverage_fout, "2288\n");
        fflush(_coverage_fout);
        }
        php_error_docref0((char const   *)((void *)0), 1 << 3L,
                          "ArrayIterator::current(): Array was modified outside object and internal position is no longer valid");
        {
        fprintf(_coverage_fout, "2289\n");
        fflush(_coverage_fout);
        }
        return (3);
      } else {
        {
        fprintf(_coverage_fout, "2290\n");
        fflush(_coverage_fout);
        }

      }
    } else {
      {
      fprintf(_coverage_fout, "2293\n");
      fflush(_coverage_fout);
      }

    }
    {
    fprintf(_coverage_fout, "2296\n");
    fflush(_coverage_fout);
    }
    tmp___2 = zend_hash_get_current_key_ex((HashTable const   *)aht, str_key,
                                           str_key_len, int_key,
                                           (unsigned char)1, & object->pos);
    {
    fprintf(_coverage_fout, "2297\n");
    fflush(_coverage_fout);
    }
    return ((int )tmp___2);
  }
}
}
extern void __attribute__((__visibility__("default")))  zend_user_it_get_current_data(zend_object_iterator *_iter ,
                                                                                      zval ***data ) ;
static void spl_array_it_get_current_data(zend_object_iterator *iter ,
                                          zval ***data ) 
{ spl_array_it *iterator ;
  spl_array_object *object ;
  HashTable *aht ;
  HashTable *tmp ;
  int __attribute__((__visibility__("default")))  tmp___0 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "2277\n");
  fflush(_coverage_fout);
  }
  iterator = (spl_array_it *)iter;
  {
  fprintf(_coverage_fout, "2278\n");
  fflush(_coverage_fout);
  }
  object = iterator->object;
  {
  fprintf(_coverage_fout, "2279\n");
  fflush(_coverage_fout);
  }
  tmp = spl_array_get_hash_table(object, 0);
  {
  fprintf(_coverage_fout, "2280\n");
  fflush(_coverage_fout);
  }
  aht = tmp;
  {
  fprintf(_coverage_fout, "2281\n");
  fflush(_coverage_fout);
  }
  if (object->ar_flags & 0x00080000) {
    {
    fprintf(_coverage_fout, "2272\n");
    fflush(_coverage_fout);
    }
    zend_user_it_get_current_data(iter, data);
  } else {
    {
    fprintf(_coverage_fout, "2275\n");
    fflush(_coverage_fout);
    }
    tmp___0 = zend_hash_get_current_data_ex(aht, (void **)data, & object->pos);
    {
    fprintf(_coverage_fout, "2276\n");
    fflush(_coverage_fout);
    }
    if (tmp___0 == (int __attribute__((__visibility__("default")))  )-1) {
      {
      fprintf(_coverage_fout, "2273\n");
      fflush(_coverage_fout);
      }
      *data = (zval **)((void *)0);
    } else {
      {
      fprintf(_coverage_fout, "2274\n");
      fflush(_coverage_fout);
      }

    }
  }
  {
  fprintf(_coverage_fout, "2282\n");
  fflush(_coverage_fout);
  }
  return;
}
}
extern int __attribute__((__visibility__("default")))  zend_user_it_valid(zend_object_iterator *_iter ) ;
static int spl_array_it_valid(zend_object_iterator *iter ) 
{ spl_array_it *iterator ;
  spl_array_object *object ;
  HashTable *aht ;
  HashTable *tmp ;
  int __attribute__((__visibility__("default")))  tmp___0 ;
  int __attribute__((__visibility__("default")))  tmp___1 ;
  int tmp___2 ;
  int __attribute__((__visibility__("default")))  tmp___3 ;
  int __attribute__((__visibility__("default")))  tmp___4 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "2267\n");
  fflush(_coverage_fout);
  }
  iterator = (spl_array_it *)iter;
  {
  fprintf(_coverage_fout, "2268\n");
  fflush(_coverage_fout);
  }
  object = iterator->object;
  {
  fprintf(_coverage_fout, "2269\n");
  fflush(_coverage_fout);
  }
  tmp = spl_array_get_hash_table(object, 0);
  {
  fprintf(_coverage_fout, "2270\n");
  fflush(_coverage_fout);
  }
  aht = tmp;
  {
  fprintf(_coverage_fout, "2271\n");
  fflush(_coverage_fout);
  }
  if (object->ar_flags & 0x00020000) {
    {
    fprintf(_coverage_fout, "2250\n");
    fflush(_coverage_fout);
    }
    tmp___0 = zend_user_it_valid(iter);
    {
    fprintf(_coverage_fout, "2251\n");
    fflush(_coverage_fout);
    }
    return ((int )tmp___0);
  } else {
    {
    fprintf(_coverage_fout, "2265\n");
    fflush(_coverage_fout);
    }
    if (! aht) {
      {
      fprintf(_coverage_fout, "2252\n");
      fflush(_coverage_fout);
      }
      php_error_docref0((char const   *)((void *)0), 1 << 3L,
                        "ArrayIterator::valid(): Array was modified outside object and is no longer an array");
      {
      fprintf(_coverage_fout, "2253\n");
      fflush(_coverage_fout);
      }
      return (-1);
    } else {
      {
      fprintf(_coverage_fout, "2254\n");
      fflush(_coverage_fout);
      }

    }
    {
    fprintf(_coverage_fout, "2266\n");
    fflush(_coverage_fout);
    }
    if (object->pos) {
      {
      fprintf(_coverage_fout, "2259\n");
      fflush(_coverage_fout);
      }
      if (object->ar_flags & 0x01000000) {
        {
        fprintf(_coverage_fout, "2257\n");
        fflush(_coverage_fout);
        }
        tmp___4 = spl_hash_verify_pos_ex(object, aht);
        {
        fprintf(_coverage_fout, "2258\n");
        fflush(_coverage_fout);
        }
        if (tmp___4 == (int __attribute__((__visibility__("default")))  )-1) {
          {
          fprintf(_coverage_fout, "2255\n");
          fflush(_coverage_fout);
          }
          php_error_docref0((char const   *)((void *)0), 1 << 3L,
                            "ArrayIterator::valid(): Array was modified outside object and internal position is no longer valid");
          {
          fprintf(_coverage_fout, "2256\n");
          fflush(_coverage_fout);
          }
          return (-1);
        } else {
          goto _L___0;
        }
      } else {
        goto _L___0;
      }
    } else {
      {
      fprintf(_coverage_fout, "2262\n");
      fflush(_coverage_fout);
      }
      _L___0: /* CIL Label */ 
      _L: /* CIL Label */ 
      tmp___3 = zend_hash_get_current_key_type_ex(aht, & object->pos);
      {
      fprintf(_coverage_fout, "2263\n");
      fflush(_coverage_fout);
      }
      if (tmp___3 == (int __attribute__((__visibility__("default")))  )3) {
        {
        fprintf(_coverage_fout, "2260\n");
        fflush(_coverage_fout);
        }
        tmp___2 = -1;
      } else {
        {
        fprintf(_coverage_fout, "2261\n");
        fflush(_coverage_fout);
        }
        tmp___2 = 0;
      }
      {
      fprintf(_coverage_fout, "2264\n");
      fflush(_coverage_fout);
      }
      return (tmp___2);
    }
  }
}
}
static void spl_array_it_dtor(zend_object_iterator *iter ) 
{ spl_array_it *iterator ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "2245\n");
  fflush(_coverage_fout);
  }
  iterator = (spl_array_it *)iter;
  {
  fprintf(_coverage_fout, "2246\n");
  fflush(_coverage_fout);
  }
  zend_user_it_invalidate_current(iter);
  {
  fprintf(_coverage_fout, "2247\n");
  fflush(_coverage_fout);
  }
  _zval_ptr_dtor((zval **)(& iterator->intern.it.data));
  {
  fprintf(_coverage_fout, "2248\n");
  fflush(_coverage_fout);
  }
  _efree((void *)iterator);
  {
  fprintf(_coverage_fout, "2249\n");
  fflush(_coverage_fout);
  }
  return;
}
}
static int spl_array_next(spl_array_object *intern ) 
{ HashTable *aht ;
  HashTable *tmp ;
  int tmp___0 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "2241\n");
  fflush(_coverage_fout);
  }
  tmp = spl_array_get_hash_table(intern, 0);
  {
  fprintf(_coverage_fout, "2242\n");
  fflush(_coverage_fout);
  }
  aht = tmp;
  {
  fprintf(_coverage_fout, "2243\n");
  fflush(_coverage_fout);
  }
  tmp___0 = spl_array_next_ex(intern, aht);
  {
  fprintf(_coverage_fout, "2244\n");
  fflush(_coverage_fout);
  }
  return (tmp___0);
}
}
static int spl_array_next_ex(spl_array_object *intern , HashTable *aht ) 
{ int __attribute__((__visibility__("default")))  tmp ;
  int tmp___0 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "2238\n");
  fflush(_coverage_fout);
  }
  if (intern->ar_flags & 0x01000000) {
    {
    fprintf(_coverage_fout, "2235\n");
    fflush(_coverage_fout);
    }
    tmp = spl_hash_verify_pos_ex(intern, aht);
    {
    fprintf(_coverage_fout, "2236\n");
    fflush(_coverage_fout);
    }
    if (tmp == (int __attribute__((__visibility__("default")))  )-1) {
      {
      fprintf(_coverage_fout, "2232\n");
      fflush(_coverage_fout);
      }
      php_error_docref0((char const   *)((void *)0), 1 << 3L,
                        "Array was modified outside object and internal position is no longer valid");
      {
      fprintf(_coverage_fout, "2233\n");
      fflush(_coverage_fout);
      }
      return (-1);
    } else {
      {
      fprintf(_coverage_fout, "2234\n");
      fflush(_coverage_fout);
      }

    }
  } else {
    {
    fprintf(_coverage_fout, "2237\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2239\n");
  fflush(_coverage_fout);
  }
  tmp___0 = spl_array_next_no_verify(intern, aht);
  {
  fprintf(_coverage_fout, "2240\n");
  fflush(_coverage_fout);
  }
  return (tmp___0);
}
}
extern int __attribute__((__visibility__("default")))  zend_hash_move_forward_ex(HashTable *ht ,
                                                                                 HashPosition *pos ) ;
static int spl_array_next_no_verify(spl_array_object *intern , HashTable *aht ) 
{ int tmp ;
  int __attribute__((__visibility__("default")))  tmp___0 ;
  int tmp___1 ;
  int __attribute__((__visibility__("default")))  tmp___2 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "2229\n");
  fflush(_coverage_fout);
  }
  zend_hash_move_forward_ex(aht, & intern->pos);
  {
  fprintf(_coverage_fout, "2230\n");
  fflush(_coverage_fout);
  }
  spl_array_update_pos(intern);
  {
  fprintf(_coverage_fout, "2231\n");
  fflush(_coverage_fout);
  }
  if ((int )(intern->array)->type == 5) {
    {
    fprintf(_coverage_fout, "2222\n");
    fflush(_coverage_fout);
    }
    tmp = spl_array_skip_protected(intern, aht);
    {
    fprintf(_coverage_fout, "2223\n");
    fflush(_coverage_fout);
    }
    return (tmp);
  } else {
    {
    fprintf(_coverage_fout, "2226\n");
    fflush(_coverage_fout);
    }
    tmp___2 = zend_hash_get_current_key_type_ex(aht, & intern->pos);
    {
    fprintf(_coverage_fout, "2227\n");
    fflush(_coverage_fout);
    }
    if (tmp___2 == (int __attribute__((__visibility__("default")))  )3) {
      {
      fprintf(_coverage_fout, "2224\n");
      fflush(_coverage_fout);
      }
      tmp___1 = -1;
    } else {
      {
      fprintf(_coverage_fout, "2225\n");
      fflush(_coverage_fout);
      }
      tmp___1 = 0;
    }
    {
    fprintf(_coverage_fout, "2228\n");
    fflush(_coverage_fout);
    }
    return (tmp___1);
  }
}
}
static int spl_array_skip_protected(spl_array_object *intern , HashTable *aht ) 
{ char *string_key ;
  uint string_length ;
  ulong num_key ;
  int __attribute__((__visibility__("default")))  tmp ;
  int __attribute__((__visibility__("default")))  tmp___0 ;
  int tmp___1 ;
  int __attribute__((__visibility__("default")))  tmp___2 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "2220\n");
  fflush(_coverage_fout);
  }
  if ((int )(intern->array)->type == 5) {
    {
    fprintf(_coverage_fout, "2218\n");
    fflush(_coverage_fout);
    }
    while (1) {
      {
      fprintf(_coverage_fout, "2211\n");
      fflush(_coverage_fout);
      }
      tmp = zend_hash_get_current_key_ex((HashTable const   *)aht, & string_key,
                                         & string_length, & num_key,
                                         (unsigned char)0, & intern->pos);
      {
      fprintf(_coverage_fout, "2212\n");
      fflush(_coverage_fout);
      }
      if (tmp == (int __attribute__((__visibility__("default")))  )1) {
        {
        fprintf(_coverage_fout, "2205\n");
        fflush(_coverage_fout);
        }
        if (! string_length) {
          {
          fprintf(_coverage_fout, "2201\n");
          fflush(_coverage_fout);
          }
          return (0);
        } else {
          {
          fprintf(_coverage_fout, "2204\n");
          fflush(_coverage_fout);
          }
          if (*(string_key + 0)) {
            {
            fprintf(_coverage_fout, "2202\n");
            fflush(_coverage_fout);
            }
            return (0);
          } else {
            {
            fprintf(_coverage_fout, "2203\n");
            fflush(_coverage_fout);
            }

          }
        }
      } else {
        {
        fprintf(_coverage_fout, "2206\n");
        fflush(_coverage_fout);
        }
        return (0);
      }
      {
      fprintf(_coverage_fout, "2213\n");
      fflush(_coverage_fout);
      }
      tmp___2 = zend_hash_get_current_key_type_ex(aht, & intern->pos);
      {
      fprintf(_coverage_fout, "2214\n");
      fflush(_coverage_fout);
      }
      if (tmp___2 == (int __attribute__((__visibility__("default")))  )3) {
        {
        fprintf(_coverage_fout, "2207\n");
        fflush(_coverage_fout);
        }
        tmp___1 = -1;
      } else {
        {
        fprintf(_coverage_fout, "2208\n");
        fflush(_coverage_fout);
        }
        tmp___1 = 0;
      }
      {
      fprintf(_coverage_fout, "2215\n");
      fflush(_coverage_fout);
      }
      if (tmp___1 != 0) {
        {
        fprintf(_coverage_fout, "2209\n");
        fflush(_coverage_fout);
        }
        return (-1);
      } else {
        {
        fprintf(_coverage_fout, "2210\n");
        fflush(_coverage_fout);
        }

      }
      {
      fprintf(_coverage_fout, "2216\n");
      fflush(_coverage_fout);
      }
      zend_hash_move_forward_ex(aht, & intern->pos);
      {
      fprintf(_coverage_fout, "2217\n");
      fflush(_coverage_fout);
      }
      spl_array_update_pos(intern);
    }
  } else {
    {
    fprintf(_coverage_fout, "2219\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2221\n");
  fflush(_coverage_fout);
  }
  return (-1);
}
}
static void spl_array_unset_property(zval *object , zval *member ,
                                     zend_literal const   *key ) 
{ spl_array_object *intern ;
  void __attribute__((__visibility__("default")))  *tmp ;
  int tmp___0 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "2196\n");
  fflush(_coverage_fout);
  }
  tmp = zend_object_store_get_object((zval const   *)object);
  {
  fprintf(_coverage_fout, "2197\n");
  fflush(_coverage_fout);
  }
  intern = (spl_array_object *)tmp;
  {
  fprintf(_coverage_fout, "2198\n");
  fflush(_coverage_fout);
  }
  if ((intern->ar_flags & 0x00000002) != 0) {
    {
    fprintf(_coverage_fout, "2193\n");
    fflush(_coverage_fout);
    }
    tmp___0 = (*(std_object_handlers.has_property))(object, member, 2, key);
    {
    fprintf(_coverage_fout, "2194\n");
    fflush(_coverage_fout);
    }
    if (tmp___0) {
      {
      fprintf(_coverage_fout, "2189\n");
      fflush(_coverage_fout);
      }

    } else {
      {
      fprintf(_coverage_fout, "2190\n");
      fflush(_coverage_fout);
      }
      spl_array_unset_dimension(object, member);
      {
      fprintf(_coverage_fout, "2191\n");
      fflush(_coverage_fout);
      }
      spl_array_rewind(intern);
      {
      fprintf(_coverage_fout, "2192\n");
      fflush(_coverage_fout);
      }
      return;
    }
  } else {
    {
    fprintf(_coverage_fout, "2195\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2199\n");
  fflush(_coverage_fout);
  }
  (*(std_object_handlers.unset_property))(object, member, key);
  {
  fprintf(_coverage_fout, "2200\n");
  fflush(_coverage_fout);
  }
  return;
}
}
static int spl_array_has_property(zval *object , zval *member ,
                                  int has_set_exists ,
                                  zend_literal const   *key ) 
{ spl_array_object *intern ;
  void __attribute__((__visibility__("default")))  *tmp ;
  int tmp___0 ;
  int tmp___1 ;
  int tmp___2 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "2184\n");
  fflush(_coverage_fout);
  }
  tmp = zend_object_store_get_object((zval const   *)object);
  {
  fprintf(_coverage_fout, "2185\n");
  fflush(_coverage_fout);
  }
  intern = (spl_array_object *)tmp;
  {
  fprintf(_coverage_fout, "2186\n");
  fflush(_coverage_fout);
  }
  if ((intern->ar_flags & 0x00000002) != 0) {
    {
    fprintf(_coverage_fout, "2181\n");
    fflush(_coverage_fout);
    }
    tmp___1 = (*(std_object_handlers.has_property))(object, member, 2, key);
    {
    fprintf(_coverage_fout, "2182\n");
    fflush(_coverage_fout);
    }
    if (tmp___1) {
      {
      fprintf(_coverage_fout, "2178\n");
      fflush(_coverage_fout);
      }

    } else {
      {
      fprintf(_coverage_fout, "2179\n");
      fflush(_coverage_fout);
      }
      tmp___0 = spl_array_has_dimension(object, member, has_set_exists);
      {
      fprintf(_coverage_fout, "2180\n");
      fflush(_coverage_fout);
      }
      return (tmp___0);
    }
  } else {
    {
    fprintf(_coverage_fout, "2183\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2187\n");
  fflush(_coverage_fout);
  }
  tmp___2 = (*(std_object_handlers.has_property))(object, member,
                                                  has_set_exists, key);
  {
  fprintf(_coverage_fout, "2188\n");
  fflush(_coverage_fout);
  }
  return (tmp___2);
}
}
static zval **spl_array_get_dimension_ptr_ptr(int check_inherited ,
                                              zval *object , zval *offset ,
                                              int type ) ;
static zval **spl_array_get_property_ptr_ptr(zval *object , zval *member ,
                                             zend_literal const   *key ) 
{ spl_array_object *intern ;
  void __attribute__((__visibility__("default")))  *tmp ;
  zval **tmp___0 ;
  int tmp___1 ;
  zval **tmp___2 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "2173\n");
  fflush(_coverage_fout);
  }
  tmp = zend_object_store_get_object((zval const   *)object);
  {
  fprintf(_coverage_fout, "2174\n");
  fflush(_coverage_fout);
  }
  intern = (spl_array_object *)tmp;
  {
  fprintf(_coverage_fout, "2175\n");
  fflush(_coverage_fout);
  }
  if ((intern->ar_flags & 0x00000002) != 0) {
    {
    fprintf(_coverage_fout, "2170\n");
    fflush(_coverage_fout);
    }
    tmp___1 = (*(std_object_handlers.has_property))(object, member, 2, key);
    {
    fprintf(_coverage_fout, "2171\n");
    fflush(_coverage_fout);
    }
    if (tmp___1) {
      {
      fprintf(_coverage_fout, "2167\n");
      fflush(_coverage_fout);
      }

    } else {
      {
      fprintf(_coverage_fout, "2168\n");
      fflush(_coverage_fout);
      }
      tmp___0 = spl_array_get_dimension_ptr_ptr(1, object, member, 2);
      {
      fprintf(_coverage_fout, "2169\n");
      fflush(_coverage_fout);
      }
      return (tmp___0);
    }
  } else {
    {
    fprintf(_coverage_fout, "2172\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2176\n");
  fflush(_coverage_fout);
  }
  tmp___2 = (*(std_object_handlers.get_property_ptr_ptr))(object, member, key);
  {
  fprintf(_coverage_fout, "2177\n");
  fflush(_coverage_fout);
  }
  return (tmp___2);
}
}
static void spl_array_write_property(zval *object , zval *member , zval *value ,
                                     zend_literal const   *key ) 
{ spl_array_object *intern ;
  void __attribute__((__visibility__("default")))  *tmp ;
  int tmp___0 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "2162\n");
  fflush(_coverage_fout);
  }
  tmp = zend_object_store_get_object((zval const   *)object);
  {
  fprintf(_coverage_fout, "2163\n");
  fflush(_coverage_fout);
  }
  intern = (spl_array_object *)tmp;
  {
  fprintf(_coverage_fout, "2164\n");
  fflush(_coverage_fout);
  }
  if ((intern->ar_flags & 0x00000002) != 0) {
    {
    fprintf(_coverage_fout, "2159\n");
    fflush(_coverage_fout);
    }
    tmp___0 = (*(std_object_handlers.has_property))(object, member, 2, key);
    {
    fprintf(_coverage_fout, "2160\n");
    fflush(_coverage_fout);
    }
    if (tmp___0) {
      {
      fprintf(_coverage_fout, "2156\n");
      fflush(_coverage_fout);
      }

    } else {
      {
      fprintf(_coverage_fout, "2157\n");
      fflush(_coverage_fout);
      }
      spl_array_write_dimension(object, member, value);
      {
      fprintf(_coverage_fout, "2158\n");
      fflush(_coverage_fout);
      }
      return;
    }
  } else {
    {
    fprintf(_coverage_fout, "2161\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2165\n");
  fflush(_coverage_fout);
  }
  (*(std_object_handlers.write_property))(object, member, value, key);
  {
  fprintf(_coverage_fout, "2166\n");
  fflush(_coverage_fout);
  }
  return;
}
}
static zval *spl_array_read_property(zval *object , zval *member , int type ,
                                     zend_literal const   *key ) 
{ spl_array_object *intern ;
  void __attribute__((__visibility__("default")))  *tmp ;
  zval *tmp___0 ;
  int tmp___1 ;
  zval *tmp___2 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "2151\n");
  fflush(_coverage_fout);
  }
  tmp = zend_object_store_get_object((zval const   *)object);
  {
  fprintf(_coverage_fout, "2152\n");
  fflush(_coverage_fout);
  }
  intern = (spl_array_object *)tmp;
  {
  fprintf(_coverage_fout, "2153\n");
  fflush(_coverage_fout);
  }
  if ((intern->ar_flags & 0x00000002) != 0) {
    {
    fprintf(_coverage_fout, "2148\n");
    fflush(_coverage_fout);
    }
    tmp___1 = (*(std_object_handlers.has_property))(object, member, 2, key);
    {
    fprintf(_coverage_fout, "2149\n");
    fflush(_coverage_fout);
    }
    if (tmp___1) {
      {
      fprintf(_coverage_fout, "2145\n");
      fflush(_coverage_fout);
      }

    } else {
      {
      fprintf(_coverage_fout, "2146\n");
      fflush(_coverage_fout);
      }
      tmp___0 = spl_array_read_dimension(object, member, type);
      {
      fprintf(_coverage_fout, "2147\n");
      fflush(_coverage_fout);
      }
      return (tmp___0);
    }
  } else {
    {
    fprintf(_coverage_fout, "2150\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2154\n");
  fflush(_coverage_fout);
  }
  tmp___2 = (*(std_object_handlers.read_property))(object, member, type, key);
  {
  fprintf(_coverage_fout, "2155\n");
  fflush(_coverage_fout);
  }
  return (tmp___2);
}
}
__inline static int zend_symtable_update(HashTable *ht , char const   *arKey ,
                                         uint nKeyLength , void *pData ,
                                         uint nDataSize , void **pDest ) ;
extern char *spl_gen_private_prop_name(zend_class_entry *ce , char *prop_name ,
                                       int prop_len , int *name_len ) ;
extern void __attribute__((__visibility__("default")))  zend_hash_clean(HashTable *ht ) ;
static HashTable *spl_array_get_debug_info(zval *obj , int *is_temp ) 
{ spl_array_object *intern ;
  void __attribute__((__visibility__("default")))  *tmp ;
  zval *tmp___0 ;
  zval *storage ;
  int name_len ;
  char *zname ;
  zend_class_entry *base ;
  void __attribute__((__visibility__("default")))  *tmp___1 ;
  int __attribute__((__visibility__("default")))  tmp___2 ;
  HashTable *tmp___3 ;
  HashTable *tmp___4 ;
  HashTable *tmp___5 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "2139\n");
  fflush(_coverage_fout);
  }
  tmp = zend_object_store_get_object((zval const   *)obj);
  {
  fprintf(_coverage_fout, "2140\n");
  fflush(_coverage_fout);
  }
  intern = (spl_array_object *)tmp;
  {
  fprintf(_coverage_fout, "2141\n");
  fflush(_coverage_fout);
  }
  *is_temp = 0;
  {
  fprintf(_coverage_fout, "2142\n");
  fflush(_coverage_fout);
  }
  if (! intern->std.properties) {
    {
    fprintf(_coverage_fout, "2111\n");
    fflush(_coverage_fout);
    }
    rebuild_object_properties(& intern->std);
  } else {
    {
    fprintf(_coverage_fout, "2112\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2143\n");
  fflush(_coverage_fout);
  }
  if ((int )(intern->array)->type == 4) {
    {
    fprintf(_coverage_fout, "2113\n");
    fflush(_coverage_fout);
    }
    tmp___5 = (intern->array)->value.ht;
  } else {
    {
    fprintf(_coverage_fout, "2117\n");
    fflush(_coverage_fout);
    }
    if ((int )(intern->array)->type == 5) {
      {
      fprintf(_coverage_fout, "2114\n");
      fflush(_coverage_fout);
      }
      tmp___3 = (*(((intern->array)->value.obj.handlers)->get_properties))(intern->array);
      {
      fprintf(_coverage_fout, "2115\n");
      fflush(_coverage_fout);
      }
      tmp___4 = tmp___3;
    } else {
      {
      fprintf(_coverage_fout, "2116\n");
      fflush(_coverage_fout);
      }
      tmp___4 = (HashTable *)((void *)0);
    }
    {
    fprintf(_coverage_fout, "2118\n");
    fflush(_coverage_fout);
    }
    tmp___5 = tmp___4;
  }
  {
  fprintf(_coverage_fout, "2144\n");
  fflush(_coverage_fout);
  }
  if ((unsigned int )tmp___5 == (unsigned int )intern->std.properties) {
    {
    fprintf(_coverage_fout, "2119\n");
    fflush(_coverage_fout);
    }
    return (intern->std.properties);
  } else {
    {
    fprintf(_coverage_fout, "2136\n");
    fflush(_coverage_fout);
    }
    if ((unsigned int )intern->debug_info == (unsigned int )((void *)0)) {
      {
      fprintf(_coverage_fout, "2120\n");
      fflush(_coverage_fout);
      }
      tmp___1 = _emalloc(sizeof(HashTable ));
      {
      fprintf(_coverage_fout, "2121\n");
      fflush(_coverage_fout);
      }
      intern->debug_info = (HashTable *)tmp___1;
      {
      fprintf(_coverage_fout, "2122\n");
      fflush(_coverage_fout);
      }
      tmp___2 = zend_hash_num_elements((HashTable const   *)intern->std.properties);
      {
      fprintf(_coverage_fout, "2123\n");
      fflush(_coverage_fout);
      }
      _zend_hash_init(intern->debug_info,
                      (unsigned int )(tmp___2 + (int __attribute__((__visibility__("default")))  )1),
                      (ulong (*)(char const   *arKey , uint nKeyLength ))((void *)0),
                      (void (*)(void * ))(& _zval_ptr_dtor), (unsigned char)0);
    } else {
      {
      fprintf(_coverage_fout, "2124\n");
      fflush(_coverage_fout);
      }

    }
    {
    fprintf(_coverage_fout, "2137\n");
    fflush(_coverage_fout);
    }
    if ((int )(intern->debug_info)->nApplyCount == 0) {
      {
      fprintf(_coverage_fout, "2127\n");
      fflush(_coverage_fout);
      }
      zend_hash_clean(intern->debug_info);
      {
      fprintf(_coverage_fout, "2128\n");
      fflush(_coverage_fout);
      }
      zend_hash_copy(intern->debug_info, intern->std.properties,
                     (void (*)(void *pElement ))(& zval_add_ref),
                     (void *)(& tmp___0), sizeof(zval *));
      {
      fprintf(_coverage_fout, "2129\n");
      fflush(_coverage_fout);
      }
      storage = intern->array;
      {
      fprintf(_coverage_fout, "2130\n");
      fflush(_coverage_fout);
      }
      zval_add_ref(& storage);
      {
      fprintf(_coverage_fout, "2131\n");
      fflush(_coverage_fout);
      }
      if ((unsigned int )obj->value.obj.handlers == (unsigned int )(& spl_handler_ArrayIterator)) {
        {
        fprintf(_coverage_fout, "2125\n");
        fflush(_coverage_fout);
        }
        base = (zend_class_entry *)spl_ce_ArrayIterator;
      } else {
        {
        fprintf(_coverage_fout, "2126\n");
        fflush(_coverage_fout);
        }
        base = (zend_class_entry *)spl_ce_ArrayObject;
      }
      {
      fprintf(_coverage_fout, "2132\n");
      fflush(_coverage_fout);
      }
      zname = spl_gen_private_prop_name(base, (char *)"storage",
                                        (int )(sizeof("storage") - 1U),
                                        & name_len);
      {
      fprintf(_coverage_fout, "2133\n");
      fflush(_coverage_fout);
      }
      zend_symtable_update(intern->debug_info, (char const   *)zname,
                           (unsigned int )(name_len + 1), (void *)(& storage),
                           sizeof(zval *), (void **)((void *)0));
      {
      fprintf(_coverage_fout, "2134\n");
      fflush(_coverage_fout);
      }
      _efree((void *)zname);
    } else {
      {
      fprintf(_coverage_fout, "2135\n");
      fflush(_coverage_fout);
      }

    }
    {
    fprintf(_coverage_fout, "2138\n");
    fflush(_coverage_fout);
    }
    return (intern->debug_info);
  }
}
}
static HashTable *spl_array_get_properties(zval *object ) 
{ spl_array_object *intern ;
  void __attribute__((__visibility__("default")))  *tmp ;
  HashTable *result ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "2104\n");
  fflush(_coverage_fout);
  }
  tmp = zend_object_store_get_object((zval const   *)object);
  {
  fprintf(_coverage_fout, "2105\n");
  fflush(_coverage_fout);
  }
  intern = (spl_array_object *)tmp;
  {
  fprintf(_coverage_fout, "2106\n");
  fflush(_coverage_fout);
  }
  if ((int )intern->nApplyCount > 1) {
    {
    fprintf(_coverage_fout, "2102\n");
    fflush(_coverage_fout);
    }
    php_error_docref0((char const   *)((void *)0), 1,
                      "Nesting level too deep - recursive dependency?");
  } else {
    {
    fprintf(_coverage_fout, "2103\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2107\n");
  fflush(_coverage_fout);
  }
  intern->nApplyCount = (unsigned char )((int )intern->nApplyCount + 1);
  {
  fprintf(_coverage_fout, "2108\n");
  fflush(_coverage_fout);
  }
  result = spl_array_get_hash_table(intern, 1);
  {
  fprintf(_coverage_fout, "2109\n");
  fflush(_coverage_fout);
  }
  intern->nApplyCount = (unsigned char )((int )intern->nApplyCount - 1);
  {
  fprintf(_coverage_fout, "2110\n");
  fflush(_coverage_fout);
  }
  return (result);
}
}
void zim_spl_Array_getArrayCopy(int ht , zval *return_value ,
                                zval **return_value_ptr , zval *this_ptr ,
                                int return_value_used ) 
{ zval *object ;
  zval *tmp ;
  spl_array_object *intern ;
  void __attribute__((__visibility__("default")))  *tmp___0 ;
  HashTable *tmp___1 ;
  HashTable *tmp___2 ;
  HashTable *tmp___3 ;
  HashTable *tmp___4 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "2094\n");
  fflush(_coverage_fout);
  }
  object = this_ptr;
  {
  fprintf(_coverage_fout, "2095\n");
  fflush(_coverage_fout);
  }
  tmp___0 = zend_object_store_get_object((zval const   *)object);
  {
  fprintf(_coverage_fout, "2096\n");
  fflush(_coverage_fout);
  }
  intern = (spl_array_object *)tmp___0;
  {
  fprintf(_coverage_fout, "2097\n");
  fflush(_coverage_fout);
  }
  _array_init(return_value, 0U);
  {
  fprintf(_coverage_fout, "2098\n");
  fflush(_coverage_fout);
  }
  tmp___1 = spl_array_get_hash_table(intern, 0);
  {
  fprintf(_coverage_fout, "2099\n");
  fflush(_coverage_fout);
  }
  if ((int )return_value->type == 4) {
    {
    fprintf(_coverage_fout, "2088\n");
    fflush(_coverage_fout);
    }
    tmp___4 = return_value->value.ht;
  } else {
    {
    fprintf(_coverage_fout, "2092\n");
    fflush(_coverage_fout);
    }
    if ((int )return_value->type == 5) {
      {
      fprintf(_coverage_fout, "2089\n");
      fflush(_coverage_fout);
      }
      tmp___2 = (*((return_value->value.obj.handlers)->get_properties))(return_value);
      {
      fprintf(_coverage_fout, "2090\n");
      fflush(_coverage_fout);
      }
      tmp___3 = tmp___2;
    } else {
      {
      fprintf(_coverage_fout, "2091\n");
      fflush(_coverage_fout);
      }
      tmp___3 = (HashTable *)((void *)0);
    }
    {
    fprintf(_coverage_fout, "2093\n");
    fflush(_coverage_fout);
    }
    tmp___4 = tmp___3;
  }
  {
  fprintf(_coverage_fout, "2100\n");
  fflush(_coverage_fout);
  }
  zend_hash_copy(tmp___4, tmp___1, (void (*)(void *pElement ))(& zval_add_ref),
                 (void *)(& tmp), sizeof(zval *));
  {
  fprintf(_coverage_fout, "2101\n");
  fflush(_coverage_fout);
  }
  return;
}
}
static void spl_array_unset_dimension_ex(int check_inherited , zval *object ,
                                         zval *offset ) ;
void zim_spl_Array_offsetUnset(int ht , zval *return_value ,
                               zval **return_value_ptr , zval *this_ptr ,
                               int return_value_used ) 
{ zval *index___0 ;
  int __attribute__((__visibility__("default")))  tmp ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "2084\n");
  fflush(_coverage_fout);
  }
  tmp = zend_parse_parameters(ht, "z", & index___0);
  {
  fprintf(_coverage_fout, "2085\n");
  fflush(_coverage_fout);
  }
  if (tmp == (int __attribute__((__visibility__("default")))  )-1) {
    {
    fprintf(_coverage_fout, "2082\n");
    fflush(_coverage_fout);
    }
    return;
  } else {
    {
    fprintf(_coverage_fout, "2083\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2086\n");
  fflush(_coverage_fout);
  }
  spl_array_unset_dimension_ex(0, this_ptr, index___0);
  {
  fprintf(_coverage_fout, "2087\n");
  fflush(_coverage_fout);
  }
  return;
}
}
void spl_array_iterator_append(zval *object , zval *append_value ) ;
void zim_spl_Array_append(int ht , zval *return_value ,
                          zval **return_value_ptr , zval *this_ptr ,
                          int return_value_used ) 
{ zval *value ;
  int __attribute__((__visibility__("default")))  tmp ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "2078\n");
  fflush(_coverage_fout);
  }
  tmp = zend_parse_parameters(ht, "z", & value);
  {
  fprintf(_coverage_fout, "2079\n");
  fflush(_coverage_fout);
  }
  if (tmp == (int __attribute__((__visibility__("default")))  )-1) {
    {
    fprintf(_coverage_fout, "2076\n");
    fflush(_coverage_fout);
    }
    return;
  } else {
    {
    fprintf(_coverage_fout, "2077\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2080\n");
  fflush(_coverage_fout);
  }
  spl_array_iterator_append(this_ptr, value);
  {
  fprintf(_coverage_fout, "2081\n");
  fflush(_coverage_fout);
  }
  return;
}
}
void spl_array_iterator_append(zval *object , zval *append_value ) 
{ spl_array_object *intern ;
  void __attribute__((__visibility__("default")))  *tmp ;
  HashTable *aht ;
  HashTable *tmp___0 ;
  zend_class_entry __attribute__((__visibility__("default")))  *tmp___1 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "2067\n");
  fflush(_coverage_fout);
  }
  tmp = zend_object_store_get_object((zval const   *)object);
  {
  fprintf(_coverage_fout, "2068\n");
  fflush(_coverage_fout);
  }
  intern = (spl_array_object *)tmp;
  {
  fprintf(_coverage_fout, "2069\n");
  fflush(_coverage_fout);
  }
  tmp___0 = spl_array_get_hash_table(intern, 0);
  {
  fprintf(_coverage_fout, "2070\n");
  fflush(_coverage_fout);
  }
  aht = tmp___0;
  {
  fprintf(_coverage_fout, "2071\n");
  fflush(_coverage_fout);
  }
  if (! aht) {
    {
    fprintf(_coverage_fout, "2058\n");
    fflush(_coverage_fout);
    }
    php_error_docref0((char const   *)((void *)0), 1 << 3L,
                      "Array was modified outside object and is no longer an array");
    {
    fprintf(_coverage_fout, "2059\n");
    fflush(_coverage_fout);
    }
    return;
  } else {
    {
    fprintf(_coverage_fout, "2060\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2072\n");
  fflush(_coverage_fout);
  }
  if ((int )(intern->array)->type == 5) {
    {
    fprintf(_coverage_fout, "2061\n");
    fflush(_coverage_fout);
    }
    tmp___1 = zend_get_class_entry((zval const   *)object);
    {
    fprintf(_coverage_fout, "2062\n");
    fflush(_coverage_fout);
    }
    php_error_docref0((char const   *)((void *)0), 1 << 12L,
                      "Cannot append properties to objects, use %s::offsetSet() instead",
                      tmp___1->name);
    {
    fprintf(_coverage_fout, "2063\n");
    fflush(_coverage_fout);
    }
    return;
  } else {
    {
    fprintf(_coverage_fout, "2064\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2073\n");
  fflush(_coverage_fout);
  }
  spl_array_write_dimension(object, (zval *)((void *)0), append_value);
  {
  fprintf(_coverage_fout, "2074\n");
  fflush(_coverage_fout);
  }
  if (! intern->pos) {
    {
    fprintf(_coverage_fout, "2065\n");
    fflush(_coverage_fout);
    }
    spl_array_set_pos(intern, aht->pListTail);
  } else {
    {
    fprintf(_coverage_fout, "2066\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2075\n");
  fflush(_coverage_fout);
  }
  return;
}
}
static void spl_array_write_dimension_ex(int check_inherited , zval *object ,
                                         zval *offset , zval *value ) ;
void zim_spl_Array_offsetSet(int ht , zval *return_value ,
                             zval **return_value_ptr , zval *this_ptr ,
                             int return_value_used ) 
{ zval *index___0 ;
  zval *value ;
  int __attribute__((__visibility__("default")))  tmp ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "2054\n");
  fflush(_coverage_fout);
  }
  tmp = zend_parse_parameters(ht, "zz", & index___0, & value);
  {
  fprintf(_coverage_fout, "2055\n");
  fflush(_coverage_fout);
  }
  if (tmp == (int __attribute__((__visibility__("default")))  )-1) {
    {
    fprintf(_coverage_fout, "2052\n");
    fflush(_coverage_fout);
    }
    return;
  } else {
    {
    fprintf(_coverage_fout, "2053\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2056\n");
  fflush(_coverage_fout);
  }
  spl_array_write_dimension_ex(0, this_ptr, index___0, value);
  {
  fprintf(_coverage_fout, "2057\n");
  fflush(_coverage_fout);
  }
  return;
}
}
static zval *spl_array_read_dimension_ex(int check_inherited , zval *object ,
                                         zval *offset , int type ) ;
void zim_spl_Array_offsetGet(int ht , zval *return_value ,
                             zval **return_value_ptr , zval *this_ptr ,
                             int return_value_used ) 
{ zval *index___0 ;
  zval *value ;
  int __attribute__((__visibility__("default")))  tmp ;
  zend_uchar is_ref ;
  zend_bool tmp___0 ;
  zend_uint refcount ;
  zend_uint tmp___1 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "2040\n");
  fflush(_coverage_fout);
  }
  tmp = zend_parse_parameters(ht, "z", & index___0);
  {
  fprintf(_coverage_fout, "2041\n");
  fflush(_coverage_fout);
  }
  if (tmp == (int __attribute__((__visibility__("default")))  )-1) {
    {
    fprintf(_coverage_fout, "2036\n");
    fflush(_coverage_fout);
    }
    return;
  } else {
    {
    fprintf(_coverage_fout, "2037\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2042\n");
  fflush(_coverage_fout);
  }
  value = spl_array_read_dimension_ex(0, this_ptr, index___0, 0);
  {
  fprintf(_coverage_fout, "2043\n");
  fflush(_coverage_fout);
  }
  tmp___0 = zval_isref_p(return_value);
  {
  fprintf(_coverage_fout, "2044\n");
  fflush(_coverage_fout);
  }
  is_ref = tmp___0;
  {
  fprintf(_coverage_fout, "2045\n");
  fflush(_coverage_fout);
  }
  tmp___1 = zval_refcount_p(return_value);
  {
  fprintf(_coverage_fout, "2046\n");
  fflush(_coverage_fout);
  }
  refcount = tmp___1;
  {
  fprintf(_coverage_fout, "2047\n");
  fflush(_coverage_fout);
  }
  while (1) {
    {
    fprintf(_coverage_fout, "2038\n");
    fflush(_coverage_fout);
    }
    return_value->value = value->value;
    {
    fprintf(_coverage_fout, "2039\n");
    fflush(_coverage_fout);
    }
    return_value->type = value->type;
    break;
  }
  {
  fprintf(_coverage_fout, "2048\n");
  fflush(_coverage_fout);
  }
  _zval_copy_ctor(return_value);
  {
  fprintf(_coverage_fout, "2049\n");
  fflush(_coverage_fout);
  }
  zval_set_isref_to_p(return_value, is_ref);
  {
  fprintf(_coverage_fout, "2050\n");
  fflush(_coverage_fout);
  }
  zval_set_refcount_p(return_value, refcount);
  {
  fprintf(_coverage_fout, "2051\n");
  fflush(_coverage_fout);
  }
  return;
}
}
static int spl_array_has_dimension_ex(int check_inherited , zval *object ,
                                      zval *offset , int check_empty ) ;
void zim_spl_Array_offsetExists(int ht , zval *return_value ,
                                zval **return_value_ptr , zval *this_ptr ,
                                int return_value_used ) 
{ zval *index___0 ;
  int __attribute__((__visibility__("default")))  tmp ;
  zval *__z ;
  int tmp___0 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "2032\n");
  fflush(_coverage_fout);
  }
  tmp = zend_parse_parameters(ht, "z", & index___0);
  {
  fprintf(_coverage_fout, "2033\n");
  fflush(_coverage_fout);
  }
  if (tmp == (int __attribute__((__visibility__("default")))  )-1) {
    {
    fprintf(_coverage_fout, "2026\n");
    fflush(_coverage_fout);
    }
    return;
  } else {
    {
    fprintf(_coverage_fout, "2027\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "2034\n");
  fflush(_coverage_fout);
  }
  while (1) {
    {
    fprintf(_coverage_fout, "2028\n");
    fflush(_coverage_fout);
    }
    __z = return_value;
    {
    fprintf(_coverage_fout, "2029\n");
    fflush(_coverage_fout);
    }
    tmp___0 = spl_array_has_dimension_ex(0, this_ptr, index___0, 0);
    {
    fprintf(_coverage_fout, "2030\n");
    fflush(_coverage_fout);
    }
    __z->value.lval = (long )(tmp___0 != 0);
    {
    fprintf(_coverage_fout, "2031\n");
    fflush(_coverage_fout);
    }
    __z->type = (unsigned char)3;
    break;
  }
  {
  fprintf(_coverage_fout, "2035\n");
  fflush(_coverage_fout);
  }
  return;
}
}
static int spl_array_has_dimension(zval *object , zval *offset ,
                                   int check_empty ) 
{ int tmp ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "2024\n");
  fflush(_coverage_fout);
  }
  tmp = spl_array_has_dimension_ex(1, object, offset, check_empty);
  {
  fprintf(_coverage_fout, "2025\n");
  fflush(_coverage_fout);
  }
  return (tmp);
}
}
extern int __attribute__((__visibility__("default")))  zend_hash_index_exists(HashTable const   *ht ,
                                                                              ulong h ) ;
extern int __attribute__((__visibility__("default")))  zend_is_true(zval *op ) ;
extern int __attribute__((__visibility__("default")))  zend_hash_index_find(HashTable const   *ht ,
                                                                            ulong h ,
                                                                            void **pData ) ;
__inline static int zend_symtable_exists(HashTable *ht , char const   *arKey ,
                                         uint nKeyLength ) ;
__inline static int zend_symtable_find(HashTable *ht , char const   *arKey ,
                                       uint nKeyLength , void **pData ) ;
static int spl_array_has_dimension_ex(int check_inherited , zval *object ,
                                      zval *offset , int check_empty ) 
{ spl_array_object *intern ;
  void __attribute__((__visibility__("default")))  *tmp ;
  long index___0 ;
  zval *rv ;
  zval **tmp___0 ;
  zval *original_var ;
  void __attribute__((__visibility__("default")))  *tmp___1 ;
  zend_bool tmp___2 ;
  zend_class_entry __attribute__((__visibility__("default")))  *tmp___3 ;
  int __attribute__((__visibility__("default")))  tmp___4 ;
  int __attribute__((__visibility__("default")))  tmp___5 ;
  HashTable *tmp___6 ;
  int tmp___7 ;
  HashTable *tmp___8 ;
  int tmp___9 ;
  HashTable *ht ;
  HashTable *tmp___10 ;
  int __attribute__((__visibility__("default")))  tmp___11 ;
  int __attribute__((__visibility__("default")))  tmp___12 ;
  HashTable *tmp___13 ;
  int __attribute__((__visibility__("default")))  tmp___14 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "2020\n");
  fflush(_coverage_fout);
  }
  tmp = zend_object_store_get_object((zval const   *)object);
  {
  fprintf(_coverage_fout, "2021\n");
  fflush(_coverage_fout);
  }
  intern = (spl_array_object *)tmp;
  {
  fprintf(_coverage_fout, "2022\n");
  fflush(_coverage_fout);
  }
  if (check_inherited) {
    {
    fprintf(_coverage_fout, "1987\n");
    fflush(_coverage_fout);
    }
    if (intern->fptr_offset_has) {
      {
      fprintf(_coverage_fout, "1978\n");
      fflush(_coverage_fout);
      }
      tmp___2 = zval_isref_p(offset);
      {
      fprintf(_coverage_fout, "1979\n");
      fflush(_coverage_fout);
      }
      if (tmp___2) {
        {
        fprintf(_coverage_fout, "1965\n");
        fflush(_coverage_fout);
        }
        original_var = offset;
        {
        fprintf(_coverage_fout, "1966\n");
        fflush(_coverage_fout);
        }
        while (1) {
          {
          fprintf(_coverage_fout, "1957\n");
          fflush(_coverage_fout);
          }
          tmp___1 = _emalloc(sizeof(zval_gc_info ));
          {
          fprintf(_coverage_fout, "1958\n");
          fflush(_coverage_fout);
          }
          offset = (zval *)tmp___1;
          {
          fprintf(_coverage_fout, "1959\n");
          fflush(_coverage_fout);
          }
          ((zval_gc_info *)offset)->u.buffered = (gc_root_buffer *)((void *)0);
          break;
        }
        {
        fprintf(_coverage_fout, "1967\n");
        fflush(_coverage_fout);
        }
        while (1) {
          {
          fprintf(_coverage_fout, "1962\n");
          fflush(_coverage_fout);
          }
          while (1) {
            {
            fprintf(_coverage_fout, "1960\n");
            fflush(_coverage_fout);
            }
            offset->value = original_var->value;
            {
            fprintf(_coverage_fout, "1961\n");
            fflush(_coverage_fout);
            }
            offset->type = original_var->type;
            break;
          }
          {
          fprintf(_coverage_fout, "1963\n");
          fflush(_coverage_fout);
          }
          zval_set_refcount_p(offset, 1U);
          {
          fprintf(_coverage_fout, "1964\n");
          fflush(_coverage_fout);
          }
          zval_unset_isref_p(offset);
          break;
        }
        {
        fprintf(_coverage_fout, "1968\n");
        fflush(_coverage_fout);
        }
        _zval_copy_ctor(offset);
      } else {
        {
        fprintf(_coverage_fout, "1969\n");
        fflush(_coverage_fout);
        }
        zval_addref_p(offset);
      }
      {
      fprintf(_coverage_fout, "1980\n");
      fflush(_coverage_fout);
      }
      tmp___3 = zend_get_class_entry((zval const   *)object);
      {
      fprintf(_coverage_fout, "1981\n");
      fflush(_coverage_fout);
      }
      zend_call_method(& object, (zend_class_entry *)tmp___3,
                       & intern->fptr_offset_has, "offsetExists",
                       (int )(sizeof("offsetExists") - 1U), & rv, 1, offset,
                       (zval *)((void *)0));
      {
      fprintf(_coverage_fout, "1982\n");
      fflush(_coverage_fout);
      }
      _zval_ptr_dtor(& offset);
      {
      fprintf(_coverage_fout, "1983\n");
      fflush(_coverage_fout);
      }
      if (rv) {
        {
        fprintf(_coverage_fout, "1973\n");
        fflush(_coverage_fout);
        }
        tmp___4 = zend_is_true(rv);
        {
        fprintf(_coverage_fout, "1974\n");
        fflush(_coverage_fout);
        }
        if (tmp___4) {
          {
          fprintf(_coverage_fout, "1970\n");
          fflush(_coverage_fout);
          }
          _zval_ptr_dtor(& rv);
          {
          fprintf(_coverage_fout, "1971\n");
          fflush(_coverage_fout);
          }
          return (1);
        } else {
          {
          fprintf(_coverage_fout, "1972\n");
          fflush(_coverage_fout);
          }

        }
      } else {
        {
        fprintf(_coverage_fout, "1975\n");
        fflush(_coverage_fout);
        }

      }
      {
      fprintf(_coverage_fout, "1984\n");
      fflush(_coverage_fout);
      }
      if (rv) {
        {
        fprintf(_coverage_fout, "1976\n");
        fflush(_coverage_fout);
        }
        _zval_ptr_dtor(& rv);
      } else {
        {
        fprintf(_coverage_fout, "1977\n");
        fflush(_coverage_fout);
        }

      }
      {
      fprintf(_coverage_fout, "1985\n");
      fflush(_coverage_fout);
      }
      return (0);
    } else {
      {
      fprintf(_coverage_fout, "1986\n");
      fflush(_coverage_fout);
      }

    }
  } else {
    {
    fprintf(_coverage_fout, "1988\n");
    fflush(_coverage_fout);
    }

  }
  switch ((int )offset->type) {
  {
  fprintf(_coverage_fout, "2016\n");
  fflush(_coverage_fout);
  }
  case 6: 
  if (check_empty) {
    {
    fprintf(_coverage_fout, "1994\n");
    fflush(_coverage_fout);
    }
    tmp___6 = spl_array_get_hash_table(intern, 0);
    {
    fprintf(_coverage_fout, "1995\n");
    fflush(_coverage_fout);
    }
    tmp___7 = zend_symtable_find(tmp___6, (char const   *)offset->value.str.val,
                                 (unsigned int )(offset->value.str.len + 1),
                                 (void **)(& tmp___0));
    {
    fprintf(_coverage_fout, "1996\n");
    fflush(_coverage_fout);
    }
    if (tmp___7 != -1) {
      switch (check_empty) {
      {
      fprintf(_coverage_fout, "1989\n");
      fflush(_coverage_fout);
      }
      case 0: 
      return ((int )(*tmp___0)->type != 0);
      {
      fprintf(_coverage_fout, "1990\n");
      fflush(_coverage_fout);
      }
      case 2: 
      return (1);
      {
      fprintf(_coverage_fout, "1991\n");
      fflush(_coverage_fout);
      }
      default: 
      tmp___5 = zend_is_true(*tmp___0);
      {
      fprintf(_coverage_fout, "1992\n");
      fflush(_coverage_fout);
      }
      return ((int )tmp___5);
      }
    } else {
      {
      fprintf(_coverage_fout, "1993\n");
      fflush(_coverage_fout);
      }

    }
    {
    fprintf(_coverage_fout, "1997\n");
    fflush(_coverage_fout);
    }
    return (0);
  } else {
    {
    fprintf(_coverage_fout, "1998\n");
    fflush(_coverage_fout);
    }
    tmp___8 = spl_array_get_hash_table(intern, 0);
    {
    fprintf(_coverage_fout, "1999\n");
    fflush(_coverage_fout);
    }
    tmp___9 = zend_symtable_exists(tmp___8,
                                   (char const   *)offset->value.str.val,
                                   (unsigned int )(offset->value.str.len + 1));
    {
    fprintf(_coverage_fout, "2000\n");
    fflush(_coverage_fout);
    }
    return (tmp___9);
  }
  {
  fprintf(_coverage_fout, "2017\n");
  fflush(_coverage_fout);
  }
  case 2: 
  case 7: 
  case 3: 
  case 1: 
  if ((int )offset->type == 2) {
    {
    fprintf(_coverage_fout, "2001\n");
    fflush(_coverage_fout);
    }
    index___0 = (long )offset->value.dval;
  } else {
    {
    fprintf(_coverage_fout, "2002\n");
    fflush(_coverage_fout);
    }
    index___0 = offset->value.lval;
  }
  {
  fprintf(_coverage_fout, "2018\n");
  fflush(_coverage_fout);
  }
  if (check_empty) {
    {
    fprintf(_coverage_fout, "2008\n");
    fflush(_coverage_fout);
    }
    tmp___10 = spl_array_get_hash_table(intern, 0);
    {
    fprintf(_coverage_fout, "2009\n");
    fflush(_coverage_fout);
    }
    ht = tmp___10;
    {
    fprintf(_coverage_fout, "2010\n");
    fflush(_coverage_fout);
    }
    tmp___12 = zend_hash_index_find((HashTable const   *)ht,
                                    (unsigned long )index___0,
                                    (void **)(& tmp___0));
    {
    fprintf(_coverage_fout, "2011\n");
    fflush(_coverage_fout);
    }
    if (tmp___12 != (int __attribute__((__visibility__("default")))  )-1) {
      switch (check_empty) {
      {
      fprintf(_coverage_fout, "2003\n");
      fflush(_coverage_fout);
      }
      case 0: 
      return ((int )(*tmp___0)->type != 0);
      {
      fprintf(_coverage_fout, "2004\n");
      fflush(_coverage_fout);
      }
      case 2: 
      return (1);
      {
      fprintf(_coverage_fout, "2005\n");
      fflush(_coverage_fout);
      }
      default: 
      tmp___11 = zend_is_true(*tmp___0);
      {
      fprintf(_coverage_fout, "2006\n");
      fflush(_coverage_fout);
      }
      return ((int )tmp___11);
      }
    } else {
      {
      fprintf(_coverage_fout, "2007\n");
      fflush(_coverage_fout);
      }

    }
    {
    fprintf(_coverage_fout, "2012\n");
    fflush(_coverage_fout);
    }
    return (0);
  } else {
    {
    fprintf(_coverage_fout, "2013\n");
    fflush(_coverage_fout);
    }
    tmp___13 = spl_array_get_hash_table(intern, 0);
    {
    fprintf(_coverage_fout, "2014\n");
    fflush(_coverage_fout);
    }
    tmp___14 = zend_hash_index_exists((HashTable const   *)tmp___13,
                                      (unsigned long )index___0);
    {
    fprintf(_coverage_fout, "2015\n");
    fflush(_coverage_fout);
    }
    return ((int )tmp___14);
  }
  {
  fprintf(_coverage_fout, "2019\n");
  fflush(_coverage_fout);
  }
  default: 
  zend_error(1 << 1L, "Illegal offset type");
  }
  {
  fprintf(_coverage_fout, "2023\n");
  fflush(_coverage_fout);
  }
  return (0);
}
}
static void spl_array_unset_dimension(zval *object , zval *offset ) 
{ 

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1955\n");
  fflush(_coverage_fout);
  }
  spl_array_unset_dimension_ex(1, object, offset);
  {
  fprintf(_coverage_fout, "1956\n");
  fflush(_coverage_fout);
  }
  return;
}
}
int __attribute__((__visibility__("default")))  spl_hash_verify_pos(spl_array_object *intern ) ;
extern int __attribute__((__visibility__("default")))  zend_hash_del_key_or_index(HashTable *ht ,
                                                                                  char const   *arKey ,
                                                                                  uint nKeyLength ,
                                                                                  ulong h ,
                                                                                  int flag ) ;
struct _zend_property_info;
typedef struct _zend_property_info zend_property_info;
struct _zend_property_info {
   zend_uint flags ;
   char const   *name ;
   int name_length ;
   ulong h ;
   int offset ;
   char const   *doc_comment ;
   int doc_comment_len ;
   zend_class_entry *ce ;
};
extern struct _zend_property_info  __attribute__((__visibility__("default"))) *zend_get_property_info(zend_class_entry *ce ,
                                                                                                      zval *member ,
                                                                                                      int silent ) ;
__inline static int zend_symtable_del(HashTable *ht , char const   *arKey ,
                                      uint nKeyLength ) ;
extern int __attribute__((__visibility__("default")))  zend_delete_global_variable(char const   *name ,
                                                                                   int name_len ) ;
struct _zend_executor_globals;
typedef struct _zend_executor_globals zend_executor_globals;
extern zend_executor_globals __attribute__((__visibility__("default")))  executor_globals ;
typedef unsigned short fpu_control_t;
struct _zend_op;
typedef struct _zend_op zend_op;
struct _zend_execute_data;
typedef unsigned long zend_ulong;
union _znode_op {
   zend_uint constant ;
   zend_uint var ;
   zend_uint num ;
   zend_ulong hash ;
   zend_uint opline_num ;
   zend_op *jmp_addr ;
   zval *zv ;
   zend_literal *literal ;
   void *ptr ;
};
typedef union _znode_op znode_op;
typedef struct _zend_execute_data zend_execute_data;
struct _zend_op {
   int ( __attribute__((__fastcall__)) (*handler))(zend_execute_data *execute_data ) ;
   znode_op op1 ;
   znode_op op2 ;
   znode_op result ;
   ulong extended_value ;
   uint lineno ;
   zend_uchar opcode ;
   zend_uchar op1_type ;
   zend_uchar op2_type ;
   zend_uchar result_type ;
};
struct _zend_object_store_bucket;
typedef struct _zend_object_store_bucket zend_object_store_bucket;
struct _zend_objects_store {
   zend_object_store_bucket *object_buckets ;
   zend_uint top ;
   zend_uint size ;
   int free_list_head ;
};
typedef struct _zend_objects_store zend_objects_store;
struct _zend_ini_entry;
typedef struct _zend_ini_entry zend_ini_entry;
struct _zend_ptr_stack {
   int top ;
   int max ;
   void **elements ;
   void **top_element ;
   zend_bool persistent ;
};
typedef struct _zend_ptr_stack zend_ptr_stack;
struct _zend_stack {
   int top ;
   int max ;
   void **elements ;
};
typedef struct _zend_stack zend_stack;
struct _zend_vm_stack;
typedef struct _zend_vm_stack *zend_vm_stack;
struct _zend_op_array;
typedef struct _zend_op_array zend_op_array;
struct __anonstruct___sigset_t_8 {
   unsigned long __val[1024U / (8U * sizeof(unsigned long ))] ;
};
typedef struct __anonstruct___sigset_t_8 __sigset_t;
typedef int __jmp_buf[6];
struct __jmp_buf_tag {
   __jmp_buf __jmpbuf ;
   int __mask_was_saved ;
   __sigset_t __saved_mask ;
};
typedef struct __jmp_buf_tag jmp_buf[1];
struct _zend_executor_globals {
   zval **return_value_ptr_ptr ;
   zval uninitialized_zval ;
   zval *uninitialized_zval_ptr ;
   zval error_zval ;
   zval *error_zval_ptr ;
   zend_ptr_stack arg_types_stack ;
   HashTable *symtable_cache[32] ;
   HashTable **symtable_cache_limit ;
   HashTable **symtable_cache_ptr ;
   zend_op **opline_ptr ;
   HashTable *active_symbol_table ;
   HashTable symbol_table ;
   HashTable included_files ;
   jmp_buf *bailout ;
   int error_reporting ;
   int orig_error_reporting ;
   int exit_status ;
   zend_op_array *active_op_array ;
   HashTable *function_table ;
   HashTable *class_table ;
   HashTable *zend_constants ;
   zend_class_entry *scope ;
   zend_class_entry *called_scope ;
   zval *This ;
   long precision ;
   int ticks_count ;
   zend_bool in_execution ;
   HashTable *in_autoload ;
   zend_function *autoload_func ;
   zend_bool full_tables_cleanup ;
   zend_bool no_extensions ;
   HashTable regular_list ;
   HashTable persistent_list ;
   zend_vm_stack argument_stack ;
   int user_error_handler_error_reporting ;
   zval *user_error_handler ;
   zval *user_exception_handler ;
   zend_stack user_error_handlers_error_reporting ;
   zend_ptr_stack user_error_handlers ;
   zend_ptr_stack user_exception_handlers ;
   zend_error_handling_t error_handling ;
   zend_class_entry *exception_class ;
   int timeout_seconds ;
   int lambda_count ;
   HashTable *ini_directives ;
   HashTable *modified_ini_directives ;
   zend_ini_entry *error_reporting_ini_entry ;
   zend_objects_store objects_store ;
   zval *exception ;
   zval *prev_exception ;
   zend_op *opline_before_exception ;
   zend_op exception_op[3] ;
   struct _zend_execute_data *current_execute_data ;
   struct _zend_module_entry *current_module ;
   zend_property_info std_property_info ;
   zend_bool active ;
   zend_op *start_op ;
   void *saved_fpu_cw_ptr ;
   fpu_control_t saved_fpu_cw ;
   void *reserved[4] ;
};
static void spl_array_unset_dimension_ex(int check_inherited , zval *object ,
                                         zval *offset ) 
{ spl_array_object *intern ;
  void __attribute__((__visibility__("default")))  *tmp ;
  long index___0 ;
  HashTable *ht ;
  zval *original_var ;
  void __attribute__((__visibility__("default")))  *tmp___0 ;
  zend_bool tmp___1 ;
  zend_class_entry __attribute__((__visibility__("default")))  *tmp___2 ;
  int __attribute__((__visibility__("default")))  tmp___3 ;
  spl_array_object *obj ;
  void __attribute__((__visibility__("default")))  *tmp___4 ;
  void __attribute__((__visibility__("default")))  *tmp___5 ;
  zend_property_info *property_info ;
  struct _zend_property_info  __attribute__((__visibility__("default"))) *tmp___6 ;
  int tmp___7 ;
  int __attribute__((__visibility__("default")))  tmp___8 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1950\n");
  fflush(_coverage_fout);
  }
  tmp = zend_object_store_get_object((zval const   *)object);
  {
  fprintf(_coverage_fout, "1951\n");
  fflush(_coverage_fout);
  }
  intern = (spl_array_object *)tmp;
  {
  fprintf(_coverage_fout, "1952\n");
  fflush(_coverage_fout);
  }
  if (check_inherited) {
    {
    fprintf(_coverage_fout, "1900\n");
    fflush(_coverage_fout);
    }
    if (intern->fptr_offset_del) {
      {
      fprintf(_coverage_fout, "1893\n");
      fflush(_coverage_fout);
      }
      tmp___1 = zval_isref_p(offset);
      {
      fprintf(_coverage_fout, "1894\n");
      fflush(_coverage_fout);
      }
      if (tmp___1) {
        {
        fprintf(_coverage_fout, "1888\n");
        fflush(_coverage_fout);
        }
        original_var = offset;
        {
        fprintf(_coverage_fout, "1889\n");
        fflush(_coverage_fout);
        }
        while (1) {
          {
          fprintf(_coverage_fout, "1880\n");
          fflush(_coverage_fout);
          }
          tmp___0 = _emalloc(sizeof(zval_gc_info ));
          {
          fprintf(_coverage_fout, "1881\n");
          fflush(_coverage_fout);
          }
          offset = (zval *)tmp___0;
          {
          fprintf(_coverage_fout, "1882\n");
          fflush(_coverage_fout);
          }
          ((zval_gc_info *)offset)->u.buffered = (gc_root_buffer *)((void *)0);
          break;
        }
        {
        fprintf(_coverage_fout, "1890\n");
        fflush(_coverage_fout);
        }
        while (1) {
          {
          fprintf(_coverage_fout, "1885\n");
          fflush(_coverage_fout);
          }
          while (1) {
            {
            fprintf(_coverage_fout, "1883\n");
            fflush(_coverage_fout);
            }
            offset->value = original_var->value;
            {
            fprintf(_coverage_fout, "1884\n");
            fflush(_coverage_fout);
            }
            offset->type = original_var->type;
            break;
          }
          {
          fprintf(_coverage_fout, "1886\n");
          fflush(_coverage_fout);
          }
          zval_set_refcount_p(offset, 1U);
          {
          fprintf(_coverage_fout, "1887\n");
          fflush(_coverage_fout);
          }
          zval_unset_isref_p(offset);
          break;
        }
        {
        fprintf(_coverage_fout, "1891\n");
        fflush(_coverage_fout);
        }
        _zval_copy_ctor(offset);
      } else {
        {
        fprintf(_coverage_fout, "1892\n");
        fflush(_coverage_fout);
        }
        zval_addref_p(offset);
      }
      {
      fprintf(_coverage_fout, "1895\n");
      fflush(_coverage_fout);
      }
      tmp___2 = zend_get_class_entry((zval const   *)object);
      {
      fprintf(_coverage_fout, "1896\n");
      fflush(_coverage_fout);
      }
      zend_call_method(& object, (zend_class_entry *)tmp___2,
                       & intern->fptr_offset_del, "offsetUnset",
                       (int )(sizeof("offsetUnset") - 1U), (zval **)((void *)0),
                       1, offset, (zval *)((void *)0));
      {
      fprintf(_coverage_fout, "1897\n");
      fflush(_coverage_fout);
      }
      _zval_ptr_dtor(& offset);
      {
      fprintf(_coverage_fout, "1898\n");
      fflush(_coverage_fout);
      }
      return;
    } else {
      {
      fprintf(_coverage_fout, "1899\n");
      fflush(_coverage_fout);
      }

    }
  } else {
    {
    fprintf(_coverage_fout, "1901\n");
    fflush(_coverage_fout);
    }

  }
  switch ((int )offset->type) {
  {
  fprintf(_coverage_fout, "1940\n");
  fflush(_coverage_fout);
  }
  case 6: 
  ht = spl_array_get_hash_table(intern, 0);
  {
  fprintf(_coverage_fout, "1941\n");
  fflush(_coverage_fout);
  }
  if ((int )ht->nApplyCount > 0) {
    {
    fprintf(_coverage_fout, "1902\n");
    fflush(_coverage_fout);
    }
    zend_error(1 << 1L,
               "Modification of ArrayObject during sorting is prohibited");
    {
    fprintf(_coverage_fout, "1903\n");
    fflush(_coverage_fout);
    }
    return;
  } else {
    {
    fprintf(_coverage_fout, "1904\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "1942\n");
  fflush(_coverage_fout);
  }
  if ((unsigned int )ht == (unsigned int )(& executor_globals.symbol_table)) {
    {
    fprintf(_coverage_fout, "1907\n");
    fflush(_coverage_fout);
    }
    tmp___3 = zend_delete_global_variable((char const   *)offset->value.str.val,
                                          offset->value.str.len);
    {
    fprintf(_coverage_fout, "1908\n");
    fflush(_coverage_fout);
    }
    if (tmp___3) {
      {
      fprintf(_coverage_fout, "1905\n");
      fflush(_coverage_fout);
      }
      zend_error(1 << 3L, "Undefined index:  %s", offset->value.str.val);
    } else {
      {
      fprintf(_coverage_fout, "1906\n");
      fflush(_coverage_fout);
      }

    }
  } else {
    {
    fprintf(_coverage_fout, "1931\n");
    fflush(_coverage_fout);
    }
    tmp___7 = zend_symtable_del(ht, (char const   *)offset->value.str.val,
                                (unsigned int )(offset->value.str.len + 1));
    {
    fprintf(_coverage_fout, "1932\n");
    fflush(_coverage_fout);
    }
    if (tmp___7 == -1) {
      {
      fprintf(_coverage_fout, "1909\n");
      fflush(_coverage_fout);
      }
      zend_error(1 << 3L, "Undefined index:  %s", offset->value.str.val);
    } else {
      {
      fprintf(_coverage_fout, "1928\n");
      fflush(_coverage_fout);
      }
      obj = intern;
      {
      fprintf(_coverage_fout, "1929\n");
      fflush(_coverage_fout);
      }
      while (1) {
        {
        fprintf(_coverage_fout, "1917\n");
        fflush(_coverage_fout);
        }
        if ((obj->ar_flags & 0x02000000) != 0) {
          break;
        } else {
          {
          fprintf(_coverage_fout, "1916\n");
          fflush(_coverage_fout);
          }
          if ((int )(obj->array)->type == 5) {
            {
            fprintf(_coverage_fout, "1914\n");
            fflush(_coverage_fout);
            }
            if ((obj->ar_flags & 0x04000000) == 0) {
              {
              fprintf(_coverage_fout, "1910\n");
              fflush(_coverage_fout);
              }
              tmp___4 = zend_object_store_get_object((zval const   *)obj->array);
              {
              fprintf(_coverage_fout, "1911\n");
              fflush(_coverage_fout);
              }
              obj = (spl_array_object *)tmp___4;
              break;
            } else {
              {
              fprintf(_coverage_fout, "1912\n");
              fflush(_coverage_fout);
              }
              tmp___5 = zend_object_store_get_object((zval const   *)obj->array);
              {
              fprintf(_coverage_fout, "1913\n");
              fflush(_coverage_fout);
              }
              obj = (spl_array_object *)tmp___5;
            }
          } else {
            {
            fprintf(_coverage_fout, "1915\n");
            fflush(_coverage_fout);
            }
            obj = (spl_array_object *)((void *)0);
            break;
          }
        }
      }
      {
      fprintf(_coverage_fout, "1930\n");
      fflush(_coverage_fout);
      }
      if (obj) {
        {
        fprintf(_coverage_fout, "1924\n");
        fflush(_coverage_fout);
        }
        tmp___6 = zend_get_property_info(obj->std.ce, offset, 1);
        {
        fprintf(_coverage_fout, "1925\n");
        fflush(_coverage_fout);
        }
        property_info = (zend_property_info *)tmp___6;
        {
        fprintf(_coverage_fout, "1926\n");
        fflush(_coverage_fout);
        }
        if (property_info) {
          {
          fprintf(_coverage_fout, "1922\n");
          fflush(_coverage_fout);
          }
          if ((property_info->flags & 1U) == 0U) {
            {
            fprintf(_coverage_fout, "1920\n");
            fflush(_coverage_fout);
            }
            if (property_info->offset >= 0) {
              {
              fprintf(_coverage_fout, "1918\n");
              fflush(_coverage_fout);
              }
              *(obj->std.properties_table + property_info->offset) = (zval *)((void *)0);
            } else {
              {
              fprintf(_coverage_fout, "1919\n");
              fflush(_coverage_fout);
              }

            }
          } else {
            {
            fprintf(_coverage_fout, "1921\n");
            fflush(_coverage_fout);
            }

          }
        } else {
          {
          fprintf(_coverage_fout, "1923\n");
          fflush(_coverage_fout);
          }

        }
      } else {
        {
        fprintf(_coverage_fout, "1927\n");
        fflush(_coverage_fout);
        }

      }
    }
  }
  break;
  {
  fprintf(_coverage_fout, "1943\n");
  fflush(_coverage_fout);
  }
  case 2: 
  case 7: 
  case 3: 
  case 1: 
  if ((int )offset->type == 2) {
    {
    fprintf(_coverage_fout, "1933\n");
    fflush(_coverage_fout);
    }
    index___0 = (long )offset->value.dval;
  } else {
    {
    fprintf(_coverage_fout, "1934\n");
    fflush(_coverage_fout);
    }
    index___0 = offset->value.lval;
  }
  {
  fprintf(_coverage_fout, "1944\n");
  fflush(_coverage_fout);
  }
  ht = spl_array_get_hash_table(intern, 0);
  {
  fprintf(_coverage_fout, "1945\n");
  fflush(_coverage_fout);
  }
  if ((int )ht->nApplyCount > 0) {
    {
    fprintf(_coverage_fout, "1935\n");
    fflush(_coverage_fout);
    }
    zend_error(1 << 1L,
               "Modification of ArrayObject during sorting is prohibited");
    {
    fprintf(_coverage_fout, "1936\n");
    fflush(_coverage_fout);
    }
    return;
  } else {
    {
    fprintf(_coverage_fout, "1937\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "1946\n");
  fflush(_coverage_fout);
  }
  tmp___8 = zend_hash_del_key_or_index(ht, (char const   *)((void *)0), 0U,
                                       (unsigned long )index___0, 1);
  {
  fprintf(_coverage_fout, "1947\n");
  fflush(_coverage_fout);
  }
  if (tmp___8 == (int __attribute__((__visibility__("default")))  )-1) {
    {
    fprintf(_coverage_fout, "1938\n");
    fflush(_coverage_fout);
    }
    zend_error(1 << 3L, "Undefined offset:  %ld", offset->value.lval);
  } else {
    {
    fprintf(_coverage_fout, "1939\n");
    fflush(_coverage_fout);
    }

  }
  break;
  {
  fprintf(_coverage_fout, "1948\n");
  fflush(_coverage_fout);
  }
  default: 
  zend_error(1 << 1L, "Illegal offset type");
  {
  fprintf(_coverage_fout, "1949\n");
  fflush(_coverage_fout);
  }
  return;
  }
  {
  fprintf(_coverage_fout, "1953\n");
  fflush(_coverage_fout);
  }
  spl_hash_verify_pos(intern);
  {
  fprintf(_coverage_fout, "1954\n");
  fflush(_coverage_fout);
  }
  return;
}
}
static void spl_array_write_dimension(zval *object , zval *offset , zval *value ) 
{ 

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1878\n");
  fflush(_coverage_fout);
  }
  spl_array_write_dimension_ex(1, object, offset, value);
  {
  fprintf(_coverage_fout, "1879\n");
  fflush(_coverage_fout);
  }
  return;
}
}
extern int __attribute__((__visibility__("default")))  _zend_hash_index_update_or_next_insert(HashTable *ht ,
                                                                                              ulong h ,
                                                                                              void *pData ,
                                                                                              uint nDataSize ,
                                                                                              void **pDest ,
                                                                                              int flag ) ;
static void spl_array_write_dimension_ex(int check_inherited , zval *object ,
                                         zval *offset , zval *value ) 
{ spl_array_object *intern ;
  void __attribute__((__visibility__("default")))  *tmp ;
  long index___0 ;
  HashTable *ht ;
  void __attribute__((__visibility__("default")))  *tmp___0 ;
  zval *original_var ;
  void __attribute__((__visibility__("default")))  *tmp___1 ;
  zend_bool tmp___2 ;
  zend_class_entry __attribute__((__visibility__("default")))  *tmp___3 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1874\n");
  fflush(_coverage_fout);
  }
  tmp = zend_object_store_get_object((zval const   *)object);
  {
  fprintf(_coverage_fout, "1875\n");
  fflush(_coverage_fout);
  }
  intern = (spl_array_object *)tmp;
  {
  fprintf(_coverage_fout, "1876\n");
  fflush(_coverage_fout);
  }
  if (check_inherited) {
    {
    fprintf(_coverage_fout, "1834\n");
    fflush(_coverage_fout);
    }
    if (intern->fptr_offset_set) {
      {
      fprintf(_coverage_fout, "1828\n");
      fflush(_coverage_fout);
      }
      if (! offset) {
        {
        fprintf(_coverage_fout, "1811\n");
        fflush(_coverage_fout);
        }
        while (1) {
          {
          fprintf(_coverage_fout, "1808\n");
          fflush(_coverage_fout);
          }
          tmp___0 = _emalloc(sizeof(zval_gc_info ));
          {
          fprintf(_coverage_fout, "1809\n");
          fflush(_coverage_fout);
          }
          offset = (zval *)tmp___0;
          {
          fprintf(_coverage_fout, "1810\n");
          fflush(_coverage_fout);
          }
          ((zval_gc_info *)offset)->u.buffered = (gc_root_buffer *)((void *)0);
          break;
        }
        {
        fprintf(_coverage_fout, "1812\n");
        fflush(_coverage_fout);
        }
        *offset = (struct _zval_struct )zval_used_for_init;
      } else {
        {
        fprintf(_coverage_fout, "1826\n");
        fflush(_coverage_fout);
        }
        tmp___2 = zval_isref_p(offset);
        {
        fprintf(_coverage_fout, "1827\n");
        fflush(_coverage_fout);
        }
        if (tmp___2) {
          {
          fprintf(_coverage_fout, "1821\n");
          fflush(_coverage_fout);
          }
          original_var = offset;
          {
          fprintf(_coverage_fout, "1822\n");
          fflush(_coverage_fout);
          }
          while (1) {
            {
            fprintf(_coverage_fout, "1813\n");
            fflush(_coverage_fout);
            }
            tmp___1 = _emalloc(sizeof(zval_gc_info ));
            {
            fprintf(_coverage_fout, "1814\n");
            fflush(_coverage_fout);
            }
            offset = (zval *)tmp___1;
            {
            fprintf(_coverage_fout, "1815\n");
            fflush(_coverage_fout);
            }
            ((zval_gc_info *)offset)->u.buffered = (gc_root_buffer *)((void *)0);
            break;
          }
          {
          fprintf(_coverage_fout, "1823\n");
          fflush(_coverage_fout);
          }
          while (1) {
            {
            fprintf(_coverage_fout, "1818\n");
            fflush(_coverage_fout);
            }
            while (1) {
              {
              fprintf(_coverage_fout, "1816\n");
              fflush(_coverage_fout);
              }
              offset->value = original_var->value;
              {
              fprintf(_coverage_fout, "1817\n");
              fflush(_coverage_fout);
              }
              offset->type = original_var->type;
              break;
            }
            {
            fprintf(_coverage_fout, "1819\n");
            fflush(_coverage_fout);
            }
            zval_set_refcount_p(offset, 1U);
            {
            fprintf(_coverage_fout, "1820\n");
            fflush(_coverage_fout);
            }
            zval_unset_isref_p(offset);
            break;
          }
          {
          fprintf(_coverage_fout, "1824\n");
          fflush(_coverage_fout);
          }
          _zval_copy_ctor(offset);
        } else {
          {
          fprintf(_coverage_fout, "1825\n");
          fflush(_coverage_fout);
          }
          zval_addref_p(offset);
        }
      }
      {
      fprintf(_coverage_fout, "1829\n");
      fflush(_coverage_fout);
      }
      tmp___3 = zend_get_class_entry((zval const   *)object);
      {
      fprintf(_coverage_fout, "1830\n");
      fflush(_coverage_fout);
      }
      zend_call_method(& object, (zend_class_entry *)tmp___3,
                       & intern->fptr_offset_set, "offsetSet",
                       (int )(sizeof("offsetSet") - 1U), (zval **)((void *)0),
                       2, offset, value);
      {
      fprintf(_coverage_fout, "1831\n");
      fflush(_coverage_fout);
      }
      _zval_ptr_dtor(& offset);
      {
      fprintf(_coverage_fout, "1832\n");
      fflush(_coverage_fout);
      }
      return;
    } else {
      {
      fprintf(_coverage_fout, "1833\n");
      fflush(_coverage_fout);
      }

    }
  } else {
    {
    fprintf(_coverage_fout, "1835\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "1877\n");
  fflush(_coverage_fout);
  }
  if (! offset) {
    {
    fprintf(_coverage_fout, "1839\n");
    fflush(_coverage_fout);
    }
    ht = spl_array_get_hash_table(intern, 0);
    {
    fprintf(_coverage_fout, "1840\n");
    fflush(_coverage_fout);
    }
    if ((int )ht->nApplyCount > 0) {
      {
      fprintf(_coverage_fout, "1836\n");
      fflush(_coverage_fout);
      }
      zend_error(1 << 1L,
                 "Modification of ArrayObject during sorting is prohibited");
      {
      fprintf(_coverage_fout, "1837\n");
      fflush(_coverage_fout);
      }
      return;
    } else {
      {
      fprintf(_coverage_fout, "1838\n");
      fflush(_coverage_fout);
      }

    }
    {
    fprintf(_coverage_fout, "1841\n");
    fflush(_coverage_fout);
    }
    zval_addref_p(value);
    {
    fprintf(_coverage_fout, "1842\n");
    fflush(_coverage_fout);
    }
    _zend_hash_index_update_or_next_insert(ht, 0UL,
                                           (void *)((void **)(& value)),
                                           sizeof(void *), (void **)((void *)0),
                                           1 << 2);
    {
    fprintf(_coverage_fout, "1843\n");
    fflush(_coverage_fout);
    }
    return;
  } else {
    {
    fprintf(_coverage_fout, "1844\n");
    fflush(_coverage_fout);
    }

  }
  switch ((int )offset->type) {
  {
  fprintf(_coverage_fout, "1856\n");
  fflush(_coverage_fout);
  }
  case 6: 
  ht = spl_array_get_hash_table(intern, 0);
  {
  fprintf(_coverage_fout, "1857\n");
  fflush(_coverage_fout);
  }
  if ((int )ht->nApplyCount > 0) {
    {
    fprintf(_coverage_fout, "1845\n");
    fflush(_coverage_fout);
    }
    zend_error(1 << 1L,
               "Modification of ArrayObject during sorting is prohibited");
    {
    fprintf(_coverage_fout, "1846\n");
    fflush(_coverage_fout);
    }
    return;
  } else {
    {
    fprintf(_coverage_fout, "1847\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "1858\n");
  fflush(_coverage_fout);
  }
  zval_addref_p(value);
  {
  fprintf(_coverage_fout, "1859\n");
  fflush(_coverage_fout);
  }
  zend_symtable_update(ht, (char const   *)offset->value.str.val,
                       (unsigned int )(offset->value.str.len + 1),
                       (void *)((void **)(& value)), sizeof(void *),
                       (void **)((void *)0));
  {
  fprintf(_coverage_fout, "1860\n");
  fflush(_coverage_fout);
  }
  return;
  {
  fprintf(_coverage_fout, "1861\n");
  fflush(_coverage_fout);
  }
  case 2: 
  case 7: 
  case 3: 
  case 1: 
  ht = spl_array_get_hash_table(intern, 0);
  {
  fprintf(_coverage_fout, "1862\n");
  fflush(_coverage_fout);
  }
  if ((int )ht->nApplyCount > 0) {
    {
    fprintf(_coverage_fout, "1848\n");
    fflush(_coverage_fout);
    }
    zend_error(1 << 1L,
               "Modification of ArrayObject during sorting is prohibited");
    {
    fprintf(_coverage_fout, "1849\n");
    fflush(_coverage_fout);
    }
    return;
  } else {
    {
    fprintf(_coverage_fout, "1850\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "1863\n");
  fflush(_coverage_fout);
  }
  if ((int )offset->type == 2) {
    {
    fprintf(_coverage_fout, "1851\n");
    fflush(_coverage_fout);
    }
    index___0 = (long )offset->value.dval;
  } else {
    {
    fprintf(_coverage_fout, "1852\n");
    fflush(_coverage_fout);
    }
    index___0 = offset->value.lval;
  }
  {
  fprintf(_coverage_fout, "1864\n");
  fflush(_coverage_fout);
  }
  zval_addref_p(value);
  {
  fprintf(_coverage_fout, "1865\n");
  fflush(_coverage_fout);
  }
  _zend_hash_index_update_or_next_insert(ht, (unsigned long )index___0,
                                         (void *)((void **)(& value)),
                                         sizeof(void *), (void **)((void *)0), 1);
  {
  fprintf(_coverage_fout, "1866\n");
  fflush(_coverage_fout);
  }
  return;
  {
  fprintf(_coverage_fout, "1867\n");
  fflush(_coverage_fout);
  }
  case 0: 
  ht = spl_array_get_hash_table(intern, 0);
  {
  fprintf(_coverage_fout, "1868\n");
  fflush(_coverage_fout);
  }
  if ((int )ht->nApplyCount > 0) {
    {
    fprintf(_coverage_fout, "1853\n");
    fflush(_coverage_fout);
    }
    zend_error(1 << 1L,
               "Modification of ArrayObject during sorting is prohibited");
    {
    fprintf(_coverage_fout, "1854\n");
    fflush(_coverage_fout);
    }
    return;
  } else {
    {
    fprintf(_coverage_fout, "1855\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "1869\n");
  fflush(_coverage_fout);
  }
  zval_addref_p(value);
  {
  fprintf(_coverage_fout, "1870\n");
  fflush(_coverage_fout);
  }
  _zend_hash_index_update_or_next_insert(ht, 0UL, (void *)((void **)(& value)),
                                         sizeof(void *), (void **)((void *)0),
                                         1 << 2);
  {
  fprintf(_coverage_fout, "1871\n");
  fflush(_coverage_fout);
  }
  return;
  {
  fprintf(_coverage_fout, "1872\n");
  fflush(_coverage_fout);
  }
  default: 
  zend_error(1 << 1L, "Illegal offset type");
  {
  fprintf(_coverage_fout, "1873\n");
  fflush(_coverage_fout);
  }
  return;
  }
}
}
static zval *spl_array_read_dimension(zval *object , zval *offset , int type ) 
{ zval *tmp ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1806\n");
  fflush(_coverage_fout);
  }
  tmp = spl_array_read_dimension_ex(1, object, offset, type);
  {
  fprintf(_coverage_fout, "1807\n");
  fflush(_coverage_fout);
  }
  return (tmp);
}
}
static zval *spl_array_read_dimension_ex(int check_inherited , zval *object ,
                                         zval *offset , int type ) 
{ zval **ret ;
  spl_array_object *intern ;
  void __attribute__((__visibility__("default")))  *tmp ;
  zval *rv ;
  zval *original_var ;
  void __attribute__((__visibility__("default")))  *tmp___0 ;
  zend_bool tmp___1 ;
  zend_class_entry __attribute__((__visibility__("default")))  *tmp___2 ;
  void __attribute__((__visibility__("default")))  *tmp___3 ;
  zend_uchar is_ref ;
  zend_bool tmp___4 ;
  zend_uint refcount ;
  zend_uint tmp___5 ;
  zval *newval ;
  void __attribute__((__visibility__("default")))  *tmp___6 ;
  zend_uint tmp___7 ;
  zend_bool tmp___8 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1802\n");
  fflush(_coverage_fout);
  }
  if (check_inherited) {
    {
    fprintf(_coverage_fout, "1778\n");
    fflush(_coverage_fout);
    }
    tmp = zend_object_store_get_object((zval const   *)object);
    {
    fprintf(_coverage_fout, "1779\n");
    fflush(_coverage_fout);
    }
    intern = (spl_array_object *)tmp;
    {
    fprintf(_coverage_fout, "1780\n");
    fflush(_coverage_fout);
    }
    if (intern->fptr_offset_get) {
      {
      fprintf(_coverage_fout, "1770\n");
      fflush(_coverage_fout);
      }
      tmp___1 = zval_isref_p(offset);
      {
      fprintf(_coverage_fout, "1771\n");
      fflush(_coverage_fout);
      }
      if (tmp___1) {
        {
        fprintf(_coverage_fout, "1745\n");
        fflush(_coverage_fout);
        }
        original_var = offset;
        {
        fprintf(_coverage_fout, "1746\n");
        fflush(_coverage_fout);
        }
        while (1) {
          {
          fprintf(_coverage_fout, "1737\n");
          fflush(_coverage_fout);
          }
          tmp___0 = _emalloc(sizeof(zval_gc_info ));
          {
          fprintf(_coverage_fout, "1738\n");
          fflush(_coverage_fout);
          }
          offset = (zval *)tmp___0;
          {
          fprintf(_coverage_fout, "1739\n");
          fflush(_coverage_fout);
          }
          ((zval_gc_info *)offset)->u.buffered = (gc_root_buffer *)((void *)0);
          break;
        }
        {
        fprintf(_coverage_fout, "1747\n");
        fflush(_coverage_fout);
        }
        while (1) {
          {
          fprintf(_coverage_fout, "1742\n");
          fflush(_coverage_fout);
          }
          while (1) {
            {
            fprintf(_coverage_fout, "1740\n");
            fflush(_coverage_fout);
            }
            offset->value = original_var->value;
            {
            fprintf(_coverage_fout, "1741\n");
            fflush(_coverage_fout);
            }
            offset->type = original_var->type;
            break;
          }
          {
          fprintf(_coverage_fout, "1743\n");
          fflush(_coverage_fout);
          }
          zval_set_refcount_p(offset, 1U);
          {
          fprintf(_coverage_fout, "1744\n");
          fflush(_coverage_fout);
          }
          zval_unset_isref_p(offset);
          break;
        }
        {
        fprintf(_coverage_fout, "1748\n");
        fflush(_coverage_fout);
        }
        _zval_copy_ctor(offset);
      } else {
        {
        fprintf(_coverage_fout, "1749\n");
        fflush(_coverage_fout);
        }
        zval_addref_p(offset);
      }
      {
      fprintf(_coverage_fout, "1772\n");
      fflush(_coverage_fout);
      }
      tmp___2 = zend_get_class_entry((zval const   *)object);
      {
      fprintf(_coverage_fout, "1773\n");
      fflush(_coverage_fout);
      }
      zend_call_method(& object, (zend_class_entry *)tmp___2,
                       & intern->fptr_offset_get, "offsetGet",
                       (int )(sizeof("offsetGet") - 1U), & rv, 1, offset,
                       (zval *)((void *)0));
      {
      fprintf(_coverage_fout, "1774\n");
      fflush(_coverage_fout);
      }
      _zval_ptr_dtor(& offset);
      {
      fprintf(_coverage_fout, "1775\n");
      fflush(_coverage_fout);
      }
      if (rv) {
        {
        fprintf(_coverage_fout, "1755\n");
        fflush(_coverage_fout);
        }
        _zval_ptr_dtor(& intern->retval);
        {
        fprintf(_coverage_fout, "1756\n");
        fflush(_coverage_fout);
        }
        while (1) {
          {
          fprintf(_coverage_fout, "1750\n");
          fflush(_coverage_fout);
          }
          tmp___3 = _emalloc(sizeof(zval_gc_info ));
          {
          fprintf(_coverage_fout, "1751\n");
          fflush(_coverage_fout);
          }
          intern->retval = (zval *)tmp___3;
          {
          fprintf(_coverage_fout, "1752\n");
          fflush(_coverage_fout);
          }
          ((zval_gc_info *)intern->retval)->u.buffered = (gc_root_buffer *)((void *)0);
          break;
        }
        {
        fprintf(_coverage_fout, "1757\n");
        fflush(_coverage_fout);
        }
        (intern->retval)->refcount__gc = 1U;
        {
        fprintf(_coverage_fout, "1758\n");
        fflush(_coverage_fout);
        }
        (intern->retval)->is_ref__gc = (unsigned char)0;
        {
        fprintf(_coverage_fout, "1759\n");
        fflush(_coverage_fout);
        }
        tmp___4 = zval_isref_p(intern->retval);
        {
        fprintf(_coverage_fout, "1760\n");
        fflush(_coverage_fout);
        }
        is_ref = tmp___4;
        {
        fprintf(_coverage_fout, "1761\n");
        fflush(_coverage_fout);
        }
        tmp___5 = zval_refcount_p(intern->retval);
        {
        fprintf(_coverage_fout, "1762\n");
        fflush(_coverage_fout);
        }
        refcount = tmp___5;
        {
        fprintf(_coverage_fout, "1763\n");
        fflush(_coverage_fout);
        }
        while (1) {
          {
          fprintf(_coverage_fout, "1753\n");
          fflush(_coverage_fout);
          }
          (intern->retval)->value = rv->value;
          {
          fprintf(_coverage_fout, "1754\n");
          fflush(_coverage_fout);
          }
          (intern->retval)->type = rv->type;
          break;
        }
        {
        fprintf(_coverage_fout, "1764\n");
        fflush(_coverage_fout);
        }
        _zval_copy_ctor(intern->retval);
        {
        fprintf(_coverage_fout, "1765\n");
        fflush(_coverage_fout);
        }
        _zval_ptr_dtor(& rv);
        {
        fprintf(_coverage_fout, "1766\n");
        fflush(_coverage_fout);
        }
        zval_set_isref_to_p(intern->retval, is_ref);
        {
        fprintf(_coverage_fout, "1767\n");
        fflush(_coverage_fout);
        }
        zval_set_refcount_p(intern->retval, refcount);
        {
        fprintf(_coverage_fout, "1768\n");
        fflush(_coverage_fout);
        }
        return (intern->retval);
      } else {
        {
        fprintf(_coverage_fout, "1769\n");
        fflush(_coverage_fout);
        }

      }
      {
      fprintf(_coverage_fout, "1776\n");
      fflush(_coverage_fout);
      }
      return (executor_globals.uninitialized_zval_ptr);
    } else {
      {
      fprintf(_coverage_fout, "1777\n");
      fflush(_coverage_fout);
      }

    }
  } else {
    {
    fprintf(_coverage_fout, "1781\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "1803\n");
  fflush(_coverage_fout);
  }
  ret = spl_array_get_dimension_ptr_ptr(check_inherited, object, offset, type);
  {
  fprintf(_coverage_fout, "1804\n");
  fflush(_coverage_fout);
  }
  if (type == 1) {
    goto _L;
  } else {
    {
    fprintf(_coverage_fout, "1801\n");
    fflush(_coverage_fout);
    }
    if (type == 2) {
      {
      fprintf(_coverage_fout, "1798\n");
      fflush(_coverage_fout);
      }
      _L: /* CIL Label */ 
      tmp___8 = zval_isref_p(*ret);
      {
      fprintf(_coverage_fout, "1799\n");
      fflush(_coverage_fout);
      }
      if (tmp___8) {
        {
        fprintf(_coverage_fout, "1782\n");
        fflush(_coverage_fout);
        }

      } else {
        {
        fprintf(_coverage_fout, "1795\n");
        fflush(_coverage_fout);
        }
        tmp___7 = zval_refcount_p(*ret);
        {
        fprintf(_coverage_fout, "1796\n");
        fflush(_coverage_fout);
        }
        if (tmp___7 > 1U) {
          {
          fprintf(_coverage_fout, "1786\n");
          fflush(_coverage_fout);
          }
          while (1) {
            {
            fprintf(_coverage_fout, "1783\n");
            fflush(_coverage_fout);
            }
            tmp___6 = _emalloc(sizeof(zval_gc_info ));
            {
            fprintf(_coverage_fout, "1784\n");
            fflush(_coverage_fout);
            }
            newval = (zval *)tmp___6;
            {
            fprintf(_coverage_fout, "1785\n");
            fflush(_coverage_fout);
            }
            ((zval_gc_info *)newval)->u.buffered = (gc_root_buffer *)((void *)0);
            break;
          }
          {
          fprintf(_coverage_fout, "1787\n");
          fflush(_coverage_fout);
          }
          newval->refcount__gc = 1U;
          {
          fprintf(_coverage_fout, "1788\n");
          fflush(_coverage_fout);
          }
          newval->is_ref__gc = (unsigned char)0;
          {
          fprintf(_coverage_fout, "1789\n");
          fflush(_coverage_fout);
          }
          *newval = *(*ret);
          {
          fprintf(_coverage_fout, "1790\n");
          fflush(_coverage_fout);
          }
          _zval_copy_ctor(newval);
          {
          fprintf(_coverage_fout, "1791\n");
          fflush(_coverage_fout);
          }
          zval_set_refcount_p(newval, 1U);
          {
          fprintf(_coverage_fout, "1792\n");
          fflush(_coverage_fout);
          }
          zval_delref_p(*ret);
          {
          fprintf(_coverage_fout, "1793\n");
          fflush(_coverage_fout);
          }
          *ret = newval;
        } else {
          {
          fprintf(_coverage_fout, "1794\n");
          fflush(_coverage_fout);
          }

        }
        {
        fprintf(_coverage_fout, "1797\n");
        fflush(_coverage_fout);
        }
        zval_set_isref_p(*ret);
      }
    } else {
      {
      fprintf(_coverage_fout, "1800\n");
      fflush(_coverage_fout);
      }

    }
  }
  {
  fprintf(_coverage_fout, "1805\n");
  fflush(_coverage_fout);
  }
  return (*ret);
}
}
static zval **spl_array_get_dimension_ptr_ptr(int check_inherited ,
                                              zval *object , zval *offset ,
                                              int type ) 
{ spl_array_object *intern ;
  void __attribute__((__visibility__("default")))  *tmp ;
  zval **retval ;
  long index___0 ;
  HashTable *ht ;
  HashTable *tmp___0 ;
  zval *value ;
  void __attribute__((__visibility__("default")))  *tmp___1 ;
  int tmp___2 ;
  int tmp___3 ;
  zval *value___0 ;
  void __attribute__((__visibility__("default")))  *tmp___4 ;
  int __attribute__((__visibility__("default")))  tmp___5 ;
  int __attribute__((__visibility__("default")))  tmp___6 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1730\n");
  fflush(_coverage_fout);
  }
  tmp = zend_object_store_get_object((zval const   *)object);
  {
  fprintf(_coverage_fout, "1731\n");
  fflush(_coverage_fout);
  }
  intern = (spl_array_object *)tmp;
  {
  fprintf(_coverage_fout, "1732\n");
  fflush(_coverage_fout);
  }
  tmp___0 = spl_array_get_hash_table(intern, 0);
  {
  fprintf(_coverage_fout, "1733\n");
  fflush(_coverage_fout);
  }
  ht = tmp___0;
  {
  fprintf(_coverage_fout, "1734\n");
  fflush(_coverage_fout);
  }
  if (! offset) {
    {
    fprintf(_coverage_fout, "1683\n");
    fflush(_coverage_fout);
    }
    return (& executor_globals.uninitialized_zval_ptr);
  } else {
    {
    fprintf(_coverage_fout, "1684\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "1735\n");
  fflush(_coverage_fout);
  }
  if (type == 1) {
    goto _L;
  } else {
    {
    fprintf(_coverage_fout, "1690\n");
    fflush(_coverage_fout);
    }
    if (type == 2) {
      {
      fprintf(_coverage_fout, "1688\n");
      fflush(_coverage_fout);
      }
      _L: /* CIL Label */ 
      if ((int )ht->nApplyCount > 0) {
        {
        fprintf(_coverage_fout, "1685\n");
        fflush(_coverage_fout);
        }
        zend_error(1 << 1L,
                   "Modification of ArrayObject during sorting is prohibited");
        {
        fprintf(_coverage_fout, "1686\n");
        fflush(_coverage_fout);
        }
        return (& executor_globals.uninitialized_zval_ptr);
      } else {
        {
        fprintf(_coverage_fout, "1687\n");
        fflush(_coverage_fout);
        }

      }
    } else {
      {
      fprintf(_coverage_fout, "1689\n");
      fflush(_coverage_fout);
      }

    }
  }
  switch ((int )offset->type) {
  {
  fprintf(_coverage_fout, "1723\n");
  fflush(_coverage_fout);
  }
  case 6: 
  tmp___3 = zend_symtable_find(ht, (char const   *)offset->value.str.val,
                               (unsigned int )(offset->value.str.len + 1),
                               (void **)(& retval));
  {
  fprintf(_coverage_fout, "1724\n");
  fflush(_coverage_fout);
  }
  if (tmp___3 == -1) {
    {
    fprintf(_coverage_fout, "1704\n");
    fflush(_coverage_fout);
    }
    if (type == 1) {
      goto _L___0;
    } else {
      {
      fprintf(_coverage_fout, "1703\n");
      fflush(_coverage_fout);
      }
      if (type == 2) {
        {
        fprintf(_coverage_fout, "1696\n");
        fflush(_coverage_fout);
        }
        _L___0: /* CIL Label */ 
        while (1) {
          {
          fprintf(_coverage_fout, "1691\n");
          fflush(_coverage_fout);
          }
          tmp___1 = _emalloc(sizeof(zval_gc_info ));
          {
          fprintf(_coverage_fout, "1692\n");
          fflush(_coverage_fout);
          }
          value = (zval *)tmp___1;
          {
          fprintf(_coverage_fout, "1693\n");
          fflush(_coverage_fout);
          }
          ((zval_gc_info *)value)->u.buffered = (gc_root_buffer *)((void *)0);
          break;
        }
        {
        fprintf(_coverage_fout, "1697\n");
        fflush(_coverage_fout);
        }
        *value = (struct _zval_struct )zval_used_for_init;
        {
        fprintf(_coverage_fout, "1698\n");
        fflush(_coverage_fout);
        }
        zend_symtable_update(ht, (char const   *)offset->value.str.val,
                             (unsigned int )(offset->value.str.len + 1),
                             (void *)((void **)(& value)), sizeof(void *),
                             (void **)((void *)0));
        {
        fprintf(_coverage_fout, "1699\n");
        fflush(_coverage_fout);
        }
        tmp___2 = zend_symtable_find(ht, (char const   *)offset->value.str.val,
                                     (unsigned int )(offset->value.str.len + 1),
                                     (void **)(& retval));
        {
        fprintf(_coverage_fout, "1700\n");
        fflush(_coverage_fout);
        }
        if (tmp___2 != -1) {
          {
          fprintf(_coverage_fout, "1694\n");
          fflush(_coverage_fout);
          }
          return (retval);
        } else {
          {
          fprintf(_coverage_fout, "1695\n");
          fflush(_coverage_fout);
          }
          return (& executor_globals.uninitialized_zval_ptr);
        }
      } else {
        {
        fprintf(_coverage_fout, "1701\n");
        fflush(_coverage_fout);
        }
        zend_error(1 << 3L, "Undefined index:  %s", offset->value.str.val);
        {
        fprintf(_coverage_fout, "1702\n");
        fflush(_coverage_fout);
        }
        return (& executor_globals.uninitialized_zval_ptr);
      }
    }
  } else {
    {
    fprintf(_coverage_fout, "1705\n");
    fflush(_coverage_fout);
    }
    return (retval);
  }
  {
  fprintf(_coverage_fout, "1725\n");
  fflush(_coverage_fout);
  }
  case 2: 
  case 7: 
  case 3: 
  case 1: 
  if ((int )offset->type == 2) {
    {
    fprintf(_coverage_fout, "1706\n");
    fflush(_coverage_fout);
    }
    index___0 = (long )offset->value.dval;
  } else {
    {
    fprintf(_coverage_fout, "1707\n");
    fflush(_coverage_fout);
    }
    index___0 = offset->value.lval;
  }
  {
  fprintf(_coverage_fout, "1726\n");
  fflush(_coverage_fout);
  }
  tmp___6 = zend_hash_index_find((HashTable const   *)ht,
                                 (unsigned long )index___0, (void **)(& retval));
  {
  fprintf(_coverage_fout, "1727\n");
  fflush(_coverage_fout);
  }
  if (tmp___6 == (int __attribute__((__visibility__("default")))  )-1) {
    {
    fprintf(_coverage_fout, "1721\n");
    fflush(_coverage_fout);
    }
    if (type == 1) {
      goto _L___1;
    } else {
      {
      fprintf(_coverage_fout, "1720\n");
      fflush(_coverage_fout);
      }
      if (type == 2) {
        {
        fprintf(_coverage_fout, "1713\n");
        fflush(_coverage_fout);
        }
        _L___1: /* CIL Label */ 
        while (1) {
          {
          fprintf(_coverage_fout, "1708\n");
          fflush(_coverage_fout);
          }
          tmp___4 = _emalloc(sizeof(zval_gc_info ));
          {
          fprintf(_coverage_fout, "1709\n");
          fflush(_coverage_fout);
          }
          value___0 = (zval *)tmp___4;
          {
          fprintf(_coverage_fout, "1710\n");
          fflush(_coverage_fout);
          }
          ((zval_gc_info *)value___0)->u.buffered = (gc_root_buffer *)((void *)0);
          break;
        }
        {
        fprintf(_coverage_fout, "1714\n");
        fflush(_coverage_fout);
        }
        *value___0 = (struct _zval_struct )zval_used_for_init;
        {
        fprintf(_coverage_fout, "1715\n");
        fflush(_coverage_fout);
        }
        _zend_hash_index_update_or_next_insert(ht, (unsigned long )index___0,
                                               (void *)((void **)(& value___0)),
                                               sizeof(void *),
                                               (void **)((void *)0), 1);
        {
        fprintf(_coverage_fout, "1716\n");
        fflush(_coverage_fout);
        }
        tmp___5 = zend_hash_index_find((HashTable const   *)ht,
                                       (unsigned long )index___0,
                                       (void **)(& retval));
        {
        fprintf(_coverage_fout, "1717\n");
        fflush(_coverage_fout);
        }
        if (tmp___5 != (int __attribute__((__visibility__("default")))  )-1) {
          {
          fprintf(_coverage_fout, "1711\n");
          fflush(_coverage_fout);
          }
          return (retval);
        } else {
          {
          fprintf(_coverage_fout, "1712\n");
          fflush(_coverage_fout);
          }
          return (& executor_globals.uninitialized_zval_ptr);
        }
      } else {
        {
        fprintf(_coverage_fout, "1718\n");
        fflush(_coverage_fout);
        }
        zend_error(1 << 3L, "Undefined offset:  %ld", index___0);
        {
        fprintf(_coverage_fout, "1719\n");
        fflush(_coverage_fout);
        }
        return (& executor_globals.uninitialized_zval_ptr);
      }
    }
  } else {
    {
    fprintf(_coverage_fout, "1722\n");
    fflush(_coverage_fout);
    }
    return (retval);
  }
  break;
  {
  fprintf(_coverage_fout, "1728\n");
  fflush(_coverage_fout);
  }
  default: 
  zend_error(1 << 1L, "Illegal offset type");
  {
  fprintf(_coverage_fout, "1729\n");
  fflush(_coverage_fout);
  }
  return (& executor_globals.uninitialized_zval_ptr);
  }
  {
  fprintf(_coverage_fout, "1736\n");
  fflush(_coverage_fout);
  }
  return ((zval **)0);
}
}
extern void __attribute__((__visibility__("default")))  zend_objects_clone_members(zend_object *new_object ,
                                                                                   zend_object_value new_obj_val ,
                                                                                   zend_object *old_object ,
                                                                                   zend_object_handle handle ) ;
extern zend_object __attribute__((__visibility__("default")))  *zend_objects_get_address(zval const   *object ) ;
static zend_object_value spl_array_object_clone(zval *zobject ) 
{ zend_object_value new_obj_val ;
  zend_object *old_object ;
  zend_object *new_object ;
  zend_object_handle handle ;
  spl_array_object *intern ;
  zend_object __attribute__((__visibility__("default")))  *tmp ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1676\n");
  fflush(_coverage_fout);
  }
  handle = zobject->value.obj.handle;
  {
  fprintf(_coverage_fout, "1677\n");
  fflush(_coverage_fout);
  }
  tmp = zend_objects_get_address((zval const   *)zobject);
  {
  fprintf(_coverage_fout, "1678\n");
  fflush(_coverage_fout);
  }
  old_object = (zend_object *)tmp;
  {
  fprintf(_coverage_fout, "1679\n");
  fflush(_coverage_fout);
  }
  new_obj_val = spl_array_object_new_ex(old_object->ce, & intern, zobject, 1);
  {
  fprintf(_coverage_fout, "1680\n");
  fflush(_coverage_fout);
  }
  new_object = & intern->std;
  {
  fprintf(_coverage_fout, "1681\n");
  fflush(_coverage_fout);
  }
  zend_objects_clone_members(new_object, new_obj_val, old_object, handle);
  {
  fprintf(_coverage_fout, "1682\n");
  fflush(_coverage_fout);
  }
  return (new_obj_val);
}
}
static zend_object_value spl_array_object_new(zend_class_entry *class_type ) 
{ spl_array_object *tmp ;
  zend_object_value tmp___0 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1674\n");
  fflush(_coverage_fout);
  }
  tmp___0 = spl_array_object_new_ex(class_type, & tmp, (zval *)((void *)0), 0);
  {
  fprintf(_coverage_fout, "1675\n");
  fflush(_coverage_fout);
  }
  return (tmp___0);
}
}
struct _zend_internal_function {
   zend_uchar type ;
   char const   *function_name ;
   zend_class_entry *scope ;
   zend_uint fn_flags ;
   union _zend_function *prototype ;
   zend_uint num_args ;
   zend_uint required_num_args ;
   zend_arg_info *arg_info ;
   void (*handler)(int ht , zval *return_value , zval **return_value_ptr ,
                   zval *this_ptr , int return_value_used ) ;
   struct _zend_module_entry *module ;
};
typedef struct _zend_internal_function zend_internal_function;
struct _zend_try_catch_element;
typedef struct _zend_try_catch_element zend_try_catch_element;
struct _zend_brk_cont_element;
typedef struct _zend_brk_cont_element zend_brk_cont_element;
struct _zend_compiled_variable;
typedef struct _zend_compiled_variable zend_compiled_variable;
struct _zend_op_array {
   zend_uchar type ;
   char const   *function_name ;
   zend_class_entry *scope ;
   zend_uint fn_flags ;
   union _zend_function *prototype ;
   zend_uint num_args ;
   zend_uint required_num_args ;
   zend_arg_info *arg_info ;
   zend_uint *refcount ;
   zend_op *opcodes ;
   zend_uint last ;
   zend_compiled_variable *vars ;
   int last_var ;
   zend_uint T ;
   zend_brk_cont_element *brk_cont_array ;
   int last_brk_cont ;
   zend_try_catch_element *try_catch_array ;
   int last_try_catch ;
   HashTable *static_variables ;
   zend_uint this_var ;
   char const   *filename ;
   zend_uint line_start ;
   zend_uint line_end ;
   char const   *doc_comment ;
   zend_uint doc_comment_len ;
   zend_uint early_binding ;
   zend_literal *literals ;
   int last_literal ;
   void **run_time_cache ;
   int last_cache_slot ;
   void *reserved[4] ;
};
struct __anonstruct_common_50 {
   zend_uchar type ;
   char const   *function_name ;
   zend_class_entry *scope ;
   zend_uint fn_flags ;
   union _zend_function *prototype ;
   zend_uint num_args ;
   zend_uint required_num_args ;
   zend_arg_info *arg_info ;
};
union _zend_function {
   zend_uchar type ;
   struct __anonstruct_common_50 common ;
   zend_op_array op_array ;
   zend_internal_function internal_function ;
};
extern int __attribute__((__visibility__("default")))  zend_hash_find(HashTable const   *ht ,
                                                                      char const   *arKey ,
                                                                      uint nKeyLength ,
                                                                      void **pData ) ;
static void spl_array_object_free_storage(void *object ) ;
extern void __attribute__((__visibility__("default")))  zend_objects_destroy_object(zend_object *object ,
                                                                                    zend_object_handle handle ) ;
extern zend_object_handle __attribute__((__visibility__("default")))  zend_objects_store_put(void *object ,
                                                                                             void (*dtor)(void *object ,
                                                                                                          zend_object_handle handle ) ,
                                                                                             void (*storage)(void *object ) ,
                                                                                             void (*clone)(void *object ,
                                                                                                           void **object_clone ) ) ;
extern void __attribute__((__visibility__("default")))  object_properties_init(zend_object *object ,
                                                                               zend_class_entry *class_type ) ;
extern void __attribute__((__visibility__("default")))  zend_object_std_init(zend_object *object ,
                                                                             zend_class_entry *ce ) ;
extern  __attribute__((__nothrow__)) void *memset(void *__s , int __c ,
                                                  size_t __n )  __attribute__((__nonnull__(1))) ;
static zend_object_value spl_array_object_new_ex(zend_class_entry *class_type ,
                                                 spl_array_object **obj ,
                                                 zval *orig , int clone_orig ) 
{ zend_object_value retval ;
  spl_array_object *intern ;
  zval *tmp ;
  zend_class_entry *parent ;
  int inherited ;
  void __attribute__((__visibility__("default")))  *tmp___0 ;
  void __attribute__((__visibility__("default")))  *tmp___1 ;
  spl_array_object *other ;
  void __attribute__((__visibility__("default")))  *tmp___2 ;
  void __attribute__((__visibility__("default")))  *tmp___3 ;
  HashTable *tmp___4 ;
  HashTable *tmp___5 ;
  HashTable *tmp___6 ;
  HashTable *tmp___7 ;
  HashTable *tmp___8 ;
  HashTable *tmp___9 ;
  void __attribute__((__visibility__("default")))  *tmp___10 ;
  zend_object_handle __attribute__((__visibility__("default")))  tmp___11 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1652\n");
  fflush(_coverage_fout);
  }
  parent = class_type;
  {
  fprintf(_coverage_fout, "1653\n");
  fflush(_coverage_fout);
  }
  inherited = 0;
  {
  fprintf(_coverage_fout, "1654\n");
  fflush(_coverage_fout);
  }
  tmp___0 = _emalloc(sizeof(spl_array_object ));
  {
  fprintf(_coverage_fout, "1655\n");
  fflush(_coverage_fout);
  }
  intern = (spl_array_object *)tmp___0;
  {
  fprintf(_coverage_fout, "1656\n");
  fflush(_coverage_fout);
  }
  memset((void *)intern, 0, sizeof(spl_array_object ));
  {
  fprintf(_coverage_fout, "1657\n");
  fflush(_coverage_fout);
  }
  *obj = intern;
  {
  fprintf(_coverage_fout, "1658\n");
  fflush(_coverage_fout);
  }
  while (1) {
    {
    fprintf(_coverage_fout, "1543\n");
    fflush(_coverage_fout);
    }
    tmp___1 = _emalloc(sizeof(zval_gc_info ));
    {
    fprintf(_coverage_fout, "1544\n");
    fflush(_coverage_fout);
    }
    intern->retval = (zval *)tmp___1;
    {
    fprintf(_coverage_fout, "1545\n");
    fflush(_coverage_fout);
    }
    ((zval_gc_info *)intern->retval)->u.buffered = (gc_root_buffer *)((void *)0);
    break;
  }
  {
  fprintf(_coverage_fout, "1659\n");
  fflush(_coverage_fout);
  }
  *(intern->retval) = (struct _zval_struct )zval_used_for_init;
  {
  fprintf(_coverage_fout, "1660\n");
  fflush(_coverage_fout);
  }
  zend_object_std_init(& intern->std, class_type);
  {
  fprintf(_coverage_fout, "1661\n");
  fflush(_coverage_fout);
  }
  object_properties_init(& intern->std, class_type);
  {
  fprintf(_coverage_fout, "1662\n");
  fflush(_coverage_fout);
  }
  intern->ar_flags = 0;
  {
  fprintf(_coverage_fout, "1663\n");
  fflush(_coverage_fout);
  }
  intern->debug_info = (HashTable *)((void *)0);
  {
  fprintf(_coverage_fout, "1664\n");
  fflush(_coverage_fout);
  }
  intern->ce_get_iterator = (zend_class_entry *)spl_ce_ArrayIterator;
  {
  fprintf(_coverage_fout, "1665\n");
  fflush(_coverage_fout);
  }
  if (orig) {
    {
    fprintf(_coverage_fout, "1577\n");
    fflush(_coverage_fout);
    }
    tmp___2 = zend_object_store_get_object((zval const   *)orig);
    {
    fprintf(_coverage_fout, "1578\n");
    fflush(_coverage_fout);
    }
    other = (spl_array_object *)tmp___2;
    {
    fprintf(_coverage_fout, "1579\n");
    fflush(_coverage_fout);
    }
    intern->ar_flags &= -50397184;
    {
    fprintf(_coverage_fout, "1580\n");
    fflush(_coverage_fout);
    }
    intern->ar_flags |= other->ar_flags & 0x0300FFFF;
    {
    fprintf(_coverage_fout, "1581\n");
    fflush(_coverage_fout);
    }
    intern->ce_get_iterator = other->ce_get_iterator;
    {
    fprintf(_coverage_fout, "1582\n");
    fflush(_coverage_fout);
    }
    if (clone_orig) {
      {
      fprintf(_coverage_fout, "1571\n");
      fflush(_coverage_fout);
      }
      intern->array = other->array;
      {
      fprintf(_coverage_fout, "1572\n");
      fflush(_coverage_fout);
      }
      if ((unsigned int )orig->value.obj.handlers == (unsigned int )(& spl_handler_ArrayObject)) {
        {
        fprintf(_coverage_fout, "1561\n");
        fflush(_coverage_fout);
        }
        while (1) {
          {
          fprintf(_coverage_fout, "1546\n");
          fflush(_coverage_fout);
          }
          tmp___3 = _emalloc(sizeof(zval_gc_info ));
          {
          fprintf(_coverage_fout, "1547\n");
          fflush(_coverage_fout);
          }
          intern->array = (zval *)tmp___3;
          {
          fprintf(_coverage_fout, "1548\n");
          fflush(_coverage_fout);
          }
          ((zval_gc_info *)intern->array)->u.buffered = (gc_root_buffer *)((void *)0);
          break;
        }
        {
        fprintf(_coverage_fout, "1562\n");
        fflush(_coverage_fout);
        }
        (intern->array)->refcount__gc = 1U;
        {
        fprintf(_coverage_fout, "1563\n");
        fflush(_coverage_fout);
        }
        (intern->array)->is_ref__gc = (unsigned char)0;
        {
        fprintf(_coverage_fout, "1564\n");
        fflush(_coverage_fout);
        }
        _array_init(intern->array, 0U);
        {
        fprintf(_coverage_fout, "1565\n");
        fflush(_coverage_fout);
        }
        if ((int )(other->array)->type == 4) {
          {
          fprintf(_coverage_fout, "1549\n");
          fflush(_coverage_fout);
          }
          tmp___6 = (other->array)->value.ht;
        } else {
          {
          fprintf(_coverage_fout, "1553\n");
          fflush(_coverage_fout);
          }
          if ((int )(other->array)->type == 5) {
            {
            fprintf(_coverage_fout, "1550\n");
            fflush(_coverage_fout);
            }
            tmp___4 = (*(((other->array)->value.obj.handlers)->get_properties))(other->array);
            {
            fprintf(_coverage_fout, "1551\n");
            fflush(_coverage_fout);
            }
            tmp___5 = tmp___4;
          } else {
            {
            fprintf(_coverage_fout, "1552\n");
            fflush(_coverage_fout);
            }
            tmp___5 = (HashTable *)((void *)0);
          }
          {
          fprintf(_coverage_fout, "1554\n");
          fflush(_coverage_fout);
          }
          tmp___6 = tmp___5;
        }
        {
        fprintf(_coverage_fout, "1566\n");
        fflush(_coverage_fout);
        }
        if ((int )(intern->array)->type == 4) {
          {
          fprintf(_coverage_fout, "1555\n");
          fflush(_coverage_fout);
          }
          tmp___9 = (intern->array)->value.ht;
        } else {
          {
          fprintf(_coverage_fout, "1559\n");
          fflush(_coverage_fout);
          }
          if ((int )(intern->array)->type == 5) {
            {
            fprintf(_coverage_fout, "1556\n");
            fflush(_coverage_fout);
            }
            tmp___7 = (*(((intern->array)->value.obj.handlers)->get_properties))(intern->array);
            {
            fprintf(_coverage_fout, "1557\n");
            fflush(_coverage_fout);
            }
            tmp___8 = tmp___7;
          } else {
            {
            fprintf(_coverage_fout, "1558\n");
            fflush(_coverage_fout);
            }
            tmp___8 = (HashTable *)((void *)0);
          }
          {
          fprintf(_coverage_fout, "1560\n");
          fflush(_coverage_fout);
          }
          tmp___9 = tmp___8;
        }
        {
        fprintf(_coverage_fout, "1567\n");
        fflush(_coverage_fout);
        }
        zend_hash_copy(tmp___9, tmp___6,
                       (void (*)(void *pElement ))(& zval_add_ref),
                       (void *)(& tmp), sizeof(zval *));
      } else {
        {
        fprintf(_coverage_fout, "1568\n");
        fflush(_coverage_fout);
        }

      }
      {
      fprintf(_coverage_fout, "1573\n");
      fflush(_coverage_fout);
      }
      if ((unsigned int )orig->value.obj.handlers == (unsigned int )(& spl_handler_ArrayIterator)) {
        {
        fprintf(_coverage_fout, "1569\n");
        fflush(_coverage_fout);
        }
        zval_addref_p(other->array);
      } else {
        {
        fprintf(_coverage_fout, "1570\n");
        fflush(_coverage_fout);
        }

      }
    } else {
      {
      fprintf(_coverage_fout, "1574\n");
      fflush(_coverage_fout);
      }
      intern->array = orig;
      {
      fprintf(_coverage_fout, "1575\n");
      fflush(_coverage_fout);
      }
      zval_addref_p(intern->array);
      {
      fprintf(_coverage_fout, "1576\n");
      fflush(_coverage_fout);
      }
      intern->ar_flags |= 83886080;
    }
  } else {
    {
    fprintf(_coverage_fout, "1586\n");
    fflush(_coverage_fout);
    }
    while (1) {
      {
      fprintf(_coverage_fout, "1583\n");
      fflush(_coverage_fout);
      }
      tmp___10 = _emalloc(sizeof(zval_gc_info ));
      {
      fprintf(_coverage_fout, "1584\n");
      fflush(_coverage_fout);
      }
      intern->array = (zval *)tmp___10;
      {
      fprintf(_coverage_fout, "1585\n");
      fflush(_coverage_fout);
      }
      ((zval_gc_info *)intern->array)->u.buffered = (gc_root_buffer *)((void *)0);
      break;
    }
    {
    fprintf(_coverage_fout, "1587\n");
    fflush(_coverage_fout);
    }
    (intern->array)->refcount__gc = 1U;
    {
    fprintf(_coverage_fout, "1588\n");
    fflush(_coverage_fout);
    }
    (intern->array)->is_ref__gc = (unsigned char)0;
    {
    fprintf(_coverage_fout, "1589\n");
    fflush(_coverage_fout);
    }
    _array_init(intern->array, 0U);
    {
    fprintf(_coverage_fout, "1590\n");
    fflush(_coverage_fout);
    }
    intern->ar_flags &= -16777217;
  }
  {
  fprintf(_coverage_fout, "1666\n");
  fflush(_coverage_fout);
  }
  tmp___11 = zend_objects_store_put((void *)intern,
                                    (void (*)(void *object ,
                                              zend_object_handle handle ))(& zend_objects_destroy_object),
                                    & spl_array_object_free_storage,
                                    (void (*)(void *object ,
                                              void **object_clone ))((void *)0));
  {
  fprintf(_coverage_fout, "1667\n");
  fflush(_coverage_fout);
  }
  retval.handle = (unsigned int )tmp___11;
  {
  fprintf(_coverage_fout, "1668\n");
  fflush(_coverage_fout);
  }
  while (1) {
    {
    fprintf(_coverage_fout, "1600\n");
    fflush(_coverage_fout);
    }
    if (parent) {
      {
      fprintf(_coverage_fout, "1591\n");
      fflush(_coverage_fout);
      }

    } else {
      break;
    }
    {
    fprintf(_coverage_fout, "1601\n");
    fflush(_coverage_fout);
    }
    if ((unsigned int )parent == (unsigned int )spl_ce_ArrayIterator) {
      {
      fprintf(_coverage_fout, "1592\n");
      fflush(_coverage_fout);
      }
      retval.handlers = (zend_object_handlers const   *)(& spl_handler_ArrayIterator);
      {
      fprintf(_coverage_fout, "1593\n");
      fflush(_coverage_fout);
      }
      class_type->get_iterator = & spl_array_get_iterator;
      break;
    } else {
      {
      fprintf(_coverage_fout, "1599\n");
      fflush(_coverage_fout);
      }
      if ((unsigned int )parent == (unsigned int )spl_ce_RecursiveArrayIterator) {
        {
        fprintf(_coverage_fout, "1594\n");
        fflush(_coverage_fout);
        }
        retval.handlers = (zend_object_handlers const   *)(& spl_handler_ArrayIterator);
        {
        fprintf(_coverage_fout, "1595\n");
        fflush(_coverage_fout);
        }
        class_type->get_iterator = & spl_array_get_iterator;
        break;
      } else {
        {
        fprintf(_coverage_fout, "1598\n");
        fflush(_coverage_fout);
        }
        if ((unsigned int )parent == (unsigned int )spl_ce_ArrayObject) {
          {
          fprintf(_coverage_fout, "1596\n");
          fflush(_coverage_fout);
          }
          retval.handlers = (zend_object_handlers const   *)(& spl_handler_ArrayObject);
          break;
        } else {
          {
          fprintf(_coverage_fout, "1597\n");
          fflush(_coverage_fout);
          }

        }
      }
    }
    {
    fprintf(_coverage_fout, "1602\n");
    fflush(_coverage_fout);
    }
    parent = parent->parent;
    {
    fprintf(_coverage_fout, "1603\n");
    fflush(_coverage_fout);
    }
    inherited = 1;
  }
  {
  fprintf(_coverage_fout, "1669\n");
  fflush(_coverage_fout);
  }
  if (! parent) {
    {
    fprintf(_coverage_fout, "1604\n");
    fflush(_coverage_fout);
    }
    php_error_docref0((char const   *)((void *)0), 1 << 6L,
                      "Internal compiler error, Class is not child of ArrayObject or ArrayIterator");
  } else {
    {
    fprintf(_coverage_fout, "1605\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "1670\n");
  fflush(_coverage_fout);
  }
  if (inherited) {
    {
    fprintf(_coverage_fout, "1616\n");
    fflush(_coverage_fout);
    }
    zend_hash_find((HashTable const   *)(& class_type->function_table),
                   "offsetget", sizeof("offsetget"),
                   (void **)(& intern->fptr_offset_get));
    {
    fprintf(_coverage_fout, "1617\n");
    fflush(_coverage_fout);
    }
    if ((unsigned int )(intern->fptr_offset_get)->common.scope == (unsigned int )parent) {
      {
      fprintf(_coverage_fout, "1606\n");
      fflush(_coverage_fout);
      }
      intern->fptr_offset_get = (zend_function *)((void *)0);
    } else {
      {
      fprintf(_coverage_fout, "1607\n");
      fflush(_coverage_fout);
      }

    }
    {
    fprintf(_coverage_fout, "1618\n");
    fflush(_coverage_fout);
    }
    zend_hash_find((HashTable const   *)(& class_type->function_table),
                   "offsetset", sizeof("offsetset"),
                   (void **)(& intern->fptr_offset_set));
    {
    fprintf(_coverage_fout, "1619\n");
    fflush(_coverage_fout);
    }
    if ((unsigned int )(intern->fptr_offset_set)->common.scope == (unsigned int )parent) {
      {
      fprintf(_coverage_fout, "1608\n");
      fflush(_coverage_fout);
      }
      intern->fptr_offset_set = (zend_function *)((void *)0);
    } else {
      {
      fprintf(_coverage_fout, "1609\n");
      fflush(_coverage_fout);
      }

    }
    {
    fprintf(_coverage_fout, "1620\n");
    fflush(_coverage_fout);
    }
    zend_hash_find((HashTable const   *)(& class_type->function_table),
                   "offsetexists", sizeof("offsetexists"),
                   (void **)(& intern->fptr_offset_has));
    {
    fprintf(_coverage_fout, "1621\n");
    fflush(_coverage_fout);
    }
    if ((unsigned int )(intern->fptr_offset_has)->common.scope == (unsigned int )parent) {
      {
      fprintf(_coverage_fout, "1610\n");
      fflush(_coverage_fout);
      }
      intern->fptr_offset_has = (zend_function *)((void *)0);
    } else {
      {
      fprintf(_coverage_fout, "1611\n");
      fflush(_coverage_fout);
      }

    }
    {
    fprintf(_coverage_fout, "1622\n");
    fflush(_coverage_fout);
    }
    zend_hash_find((HashTable const   *)(& class_type->function_table),
                   "offsetunset", sizeof("offsetunset"),
                   (void **)(& intern->fptr_offset_del));
    {
    fprintf(_coverage_fout, "1623\n");
    fflush(_coverage_fout);
    }
    if ((unsigned int )(intern->fptr_offset_del)->common.scope == (unsigned int )parent) {
      {
      fprintf(_coverage_fout, "1612\n");
      fflush(_coverage_fout);
      }
      intern->fptr_offset_del = (zend_function *)((void *)0);
    } else {
      {
      fprintf(_coverage_fout, "1613\n");
      fflush(_coverage_fout);
      }

    }
    {
    fprintf(_coverage_fout, "1624\n");
    fflush(_coverage_fout);
    }
    zend_hash_find((HashTable const   *)(& class_type->function_table), "count",
                   sizeof("count"), (void **)(& intern->fptr_count));
    {
    fprintf(_coverage_fout, "1625\n");
    fflush(_coverage_fout);
    }
    if ((unsigned int )(intern->fptr_count)->common.scope == (unsigned int )parent) {
      {
      fprintf(_coverage_fout, "1614\n");
      fflush(_coverage_fout);
      }
      intern->fptr_count = (zend_function *)((void *)0);
    } else {
      {
      fprintf(_coverage_fout, "1615\n");
      fflush(_coverage_fout);
      }

    }
  } else {
    {
    fprintf(_coverage_fout, "1626\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "1671\n");
  fflush(_coverage_fout);
  }
  if ((unsigned int )retval.handlers == (unsigned int )(& spl_handler_ArrayIterator)) {
    {
    fprintf(_coverage_fout, "1649\n");
    fflush(_coverage_fout);
    }
    if (! class_type->iterator_funcs.zf_current) {
      {
      fprintf(_coverage_fout, "1627\n");
      fflush(_coverage_fout);
      }
      zend_hash_find((HashTable const   *)(& class_type->function_table),
                     "rewind", sizeof("rewind"),
                     (void **)(& class_type->iterator_funcs.zf_rewind));
      {
      fprintf(_coverage_fout, "1628\n");
      fflush(_coverage_fout);
      }
      zend_hash_find((HashTable const   *)(& class_type->function_table),
                     "valid", sizeof("valid"),
                     (void **)(& class_type->iterator_funcs.zf_valid));
      {
      fprintf(_coverage_fout, "1629\n");
      fflush(_coverage_fout);
      }
      zend_hash_find((HashTable const   *)(& class_type->function_table), "key",
                     sizeof("key"),
                     (void **)(& class_type->iterator_funcs.zf_key));
      {
      fprintf(_coverage_fout, "1630\n");
      fflush(_coverage_fout);
      }
      zend_hash_find((HashTable const   *)(& class_type->function_table),
                     "current", sizeof("current"),
                     (void **)(& class_type->iterator_funcs.zf_current));
      {
      fprintf(_coverage_fout, "1631\n");
      fflush(_coverage_fout);
      }
      zend_hash_find((HashTable const   *)(& class_type->function_table),
                     "next", sizeof("next"),
                     (void **)(& class_type->iterator_funcs.zf_next));
    } else {
      {
      fprintf(_coverage_fout, "1632\n");
      fflush(_coverage_fout);
      }

    }
    {
    fprintf(_coverage_fout, "1650\n");
    fflush(_coverage_fout);
    }
    if (inherited) {
      {
      fprintf(_coverage_fout, "1643\n");
      fflush(_coverage_fout);
      }
      if ((unsigned int )(class_type->iterator_funcs.zf_rewind)->common.scope != (unsigned int )parent) {
        {
        fprintf(_coverage_fout, "1633\n");
        fflush(_coverage_fout);
        }
        intern->ar_flags |= 0x00010000;
      } else {
        {
        fprintf(_coverage_fout, "1634\n");
        fflush(_coverage_fout);
        }

      }
      {
      fprintf(_coverage_fout, "1644\n");
      fflush(_coverage_fout);
      }
      if ((unsigned int )(class_type->iterator_funcs.zf_valid)->common.scope != (unsigned int )parent) {
        {
        fprintf(_coverage_fout, "1635\n");
        fflush(_coverage_fout);
        }
        intern->ar_flags |= 0x00020000;
      } else {
        {
        fprintf(_coverage_fout, "1636\n");
        fflush(_coverage_fout);
        }

      }
      {
      fprintf(_coverage_fout, "1645\n");
      fflush(_coverage_fout);
      }
      if ((unsigned int )(class_type->iterator_funcs.zf_key)->common.scope != (unsigned int )parent) {
        {
        fprintf(_coverage_fout, "1637\n");
        fflush(_coverage_fout);
        }
        intern->ar_flags |= 0x00040000;
      } else {
        {
        fprintf(_coverage_fout, "1638\n");
        fflush(_coverage_fout);
        }

      }
      {
      fprintf(_coverage_fout, "1646\n");
      fflush(_coverage_fout);
      }
      if ((unsigned int )(class_type->iterator_funcs.zf_current)->common.scope != (unsigned int )parent) {
        {
        fprintf(_coverage_fout, "1639\n");
        fflush(_coverage_fout);
        }
        intern->ar_flags |= 0x00080000;
      } else {
        {
        fprintf(_coverage_fout, "1640\n");
        fflush(_coverage_fout);
        }

      }
      {
      fprintf(_coverage_fout, "1647\n");
      fflush(_coverage_fout);
      }
      if ((unsigned int )(class_type->iterator_funcs.zf_next)->common.scope != (unsigned int )parent) {
        {
        fprintf(_coverage_fout, "1641\n");
        fflush(_coverage_fout);
        }
        intern->ar_flags |= 0x00100000;
      } else {
        {
        fprintf(_coverage_fout, "1642\n");
        fflush(_coverage_fout);
        }

      }
    } else {
      {
      fprintf(_coverage_fout, "1648\n");
      fflush(_coverage_fout);
      }

    }
  } else {
    {
    fprintf(_coverage_fout, "1651\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "1672\n");
  fflush(_coverage_fout);
  }
  spl_array_rewind(intern);
  {
  fprintf(_coverage_fout, "1673\n");
  fflush(_coverage_fout);
  }
  return (retval);
}
}
extern void __attribute__((__visibility__("default")))  zend_object_std_dtor(zend_object *object ) ;
static void spl_array_object_free_storage(void *object ) 
{ spl_array_object *intern ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1536\n");
  fflush(_coverage_fout);
  }
  intern = (spl_array_object *)object;
  {
  fprintf(_coverage_fout, "1537\n");
  fflush(_coverage_fout);
  }
  zend_object_std_dtor(& intern->std);
  {
  fprintf(_coverage_fout, "1538\n");
  fflush(_coverage_fout);
  }
  _zval_ptr_dtor(& intern->array);
  {
  fprintf(_coverage_fout, "1539\n");
  fflush(_coverage_fout);
  }
  _zval_ptr_dtor(& intern->retval);
  {
  fprintf(_coverage_fout, "1540\n");
  fflush(_coverage_fout);
  }
  if ((unsigned int )intern->debug_info != (unsigned int )((void *)0)) {
    {
    fprintf(_coverage_fout, "1533\n");
    fflush(_coverage_fout);
    }
    zend_hash_destroy(intern->debug_info);
    {
    fprintf(_coverage_fout, "1534\n");
    fflush(_coverage_fout);
    }
    _efree((void *)intern->debug_info);
  } else {
    {
    fprintf(_coverage_fout, "1535\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "1541\n");
  fflush(_coverage_fout);
  }
  _efree(object);
  {
  fprintf(_coverage_fout, "1542\n");
  fflush(_coverage_fout);
  }
  return;
}
}
int __attribute__((__visibility__("default")))  spl_hash_verify_pos(spl_array_object *intern ) 
{ HashTable *ht ;
  HashTable *tmp ;
  int __attribute__((__visibility__("default")))  tmp___0 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1529\n");
  fflush(_coverage_fout);
  }
  tmp = spl_array_get_hash_table(intern, 0);
  {
  fprintf(_coverage_fout, "1530\n");
  fflush(_coverage_fout);
  }
  ht = tmp;
  {
  fprintf(_coverage_fout, "1531\n");
  fflush(_coverage_fout);
  }
  tmp___0 = spl_hash_verify_pos_ex(intern, ht);
  {
  fprintf(_coverage_fout, "1532\n");
  fflush(_coverage_fout);
  }
  return (tmp___0);
}
}
struct bucket {
   ulong h ;
   uint nKeyLength ;
   void *pData ;
   void *pDataPtr ;
   struct bucket *pListNext ;
   struct bucket *pListLast ;
   struct bucket *pNext ;
   struct bucket *pLast ;
   char const   *arKey ;
};
int __attribute__((__visibility__("default")))  spl_hash_verify_pos_ex(spl_array_object *intern ,
                                                                       HashTable *ht ) 
{ Bucket *p ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1525\n");
  fflush(_coverage_fout);
  }
  p = *(ht->arBuckets + (intern->pos_h & (unsigned long )ht->nTableMask));
  {
  fprintf(_coverage_fout, "1526\n");
  fflush(_coverage_fout);
  }
  while (1) {
    {
    fprintf(_coverage_fout, "1522\n");
    fflush(_coverage_fout);
    }
    if ((unsigned int )p != (unsigned int )((void *)0)) {
      {
      fprintf(_coverage_fout, "1519\n");
      fflush(_coverage_fout);
      }

    } else {
      break;
    }
    {
    fprintf(_coverage_fout, "1523\n");
    fflush(_coverage_fout);
    }
    if ((unsigned int )p == (unsigned int )intern->pos) {
      {
      fprintf(_coverage_fout, "1520\n");
      fflush(_coverage_fout);
      }
      return ((int __attribute__((__visibility__("default")))  )0);
    } else {
      {
      fprintf(_coverage_fout, "1521\n");
      fflush(_coverage_fout);
      }

    }
    {
    fprintf(_coverage_fout, "1524\n");
    fflush(_coverage_fout);
    }
    p = p->pNext;
  }
  {
  fprintf(_coverage_fout, "1527\n");
  fflush(_coverage_fout);
  }
  spl_array_rewind(intern);
  {
  fprintf(_coverage_fout, "1528\n");
  fflush(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )-1);
}
}
static void spl_array_set_pos(spl_array_object *intern , HashPosition pos ) 
{ 

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1516\n");
  fflush(_coverage_fout);
  }
  intern->pos = pos;
  {
  fprintf(_coverage_fout, "1517\n");
  fflush(_coverage_fout);
  }
  spl_array_update_pos(intern);
  {
  fprintf(_coverage_fout, "1518\n");
  fflush(_coverage_fout);
  }
  return;
}
}
static void spl_array_update_pos(spl_array_object *intern ) 
{ Bucket *pos ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1513\n");
  fflush(_coverage_fout);
  }
  pos = intern->pos;
  {
  fprintf(_coverage_fout, "1514\n");
  fflush(_coverage_fout);
  }
  if ((unsigned int )pos != (unsigned int )((void *)0)) {
    {
    fprintf(_coverage_fout, "1511\n");
    fflush(_coverage_fout);
    }
    intern->pos_h = pos->h;
  } else {
    {
    fprintf(_coverage_fout, "1512\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "1515\n");
  fflush(_coverage_fout);
  }
  return;
}
}
__inline static HashTable *spl_array_get_hash_table(spl_array_object *intern ,
                                                    int check_std_props ) 
{ spl_array_object *other ;
  void __attribute__((__visibility__("default")))  *tmp ;
  HashTable *tmp___0 ;
  HashTable *tmp___1 ;
  HashTable *tmp___2 ;
  HashTable *tmp___3 ;
  int tmp___4 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1510\n");
  fflush(_coverage_fout);
  }
  if ((intern->ar_flags & 0x02000000) != 0) {
    {
    fprintf(_coverage_fout, "1484\n");
    fflush(_coverage_fout);
    }
    if (! intern->std.properties) {
      {
      fprintf(_coverage_fout, "1482\n");
      fflush(_coverage_fout);
      }
      rebuild_object_properties(& intern->std);
    } else {
      {
      fprintf(_coverage_fout, "1483\n");
      fflush(_coverage_fout);
      }

    }
    {
    fprintf(_coverage_fout, "1485\n");
    fflush(_coverage_fout);
    }
    return (intern->std.properties);
  } else {
    {
    fprintf(_coverage_fout, "1509\n");
    fflush(_coverage_fout);
    }
    if (intern->ar_flags & 0x04000000) {
      {
      fprintf(_coverage_fout, "1492\n");
      fflush(_coverage_fout);
      }
      if (check_std_props == 0) {
        goto _L___1;
      } else {
        {
        fprintf(_coverage_fout, "1491\n");
        fflush(_coverage_fout);
        }
        if ((intern->ar_flags & 0x00000001) == 0) {
          {
          fprintf(_coverage_fout, "1490\n");
          fflush(_coverage_fout);
          }
          _L___1: /* CIL Label */ 
          if ((int )(intern->array)->type == 5) {
            {
            fprintf(_coverage_fout, "1486\n");
            fflush(_coverage_fout);
            }
            tmp = zend_object_store_get_object((zval const   *)intern->array);
            {
            fprintf(_coverage_fout, "1487\n");
            fflush(_coverage_fout);
            }
            other = (spl_array_object *)tmp;
            {
            fprintf(_coverage_fout, "1488\n");
            fflush(_coverage_fout);
            }
            tmp___0 = spl_array_get_hash_table(other, check_std_props);
            {
            fprintf(_coverage_fout, "1489\n");
            fflush(_coverage_fout);
            }
            return (tmp___0);
          } else {
            goto _L___0;
          }
        } else {
          goto _L___0;
        }
      }
    } else {
      {
      fprintf(_coverage_fout, "1507\n");
      fflush(_coverage_fout);
      }
      _L___0: /* CIL Label */ 
      _L: /* CIL Label */ 
      if (check_std_props) {
        {
        fprintf(_coverage_fout, "1493\n");
        fflush(_coverage_fout);
        }
        tmp___4 = 0x00000001;
      } else {
        {
        fprintf(_coverage_fout, "1494\n");
        fflush(_coverage_fout);
        }
        tmp___4 = 0;
      }
      {
      fprintf(_coverage_fout, "1508\n");
      fflush(_coverage_fout);
      }
      if ((intern->ar_flags & (tmp___4 | 0x02000000)) != 0) {
        {
        fprintf(_coverage_fout, "1497\n");
        fflush(_coverage_fout);
        }
        if (! intern->std.properties) {
          {
          fprintf(_coverage_fout, "1495\n");
          fflush(_coverage_fout);
          }
          rebuild_object_properties(& intern->std);
        } else {
          {
          fprintf(_coverage_fout, "1496\n");
          fflush(_coverage_fout);
          }

        }
        {
        fprintf(_coverage_fout, "1498\n");
        fflush(_coverage_fout);
        }
        return (intern->std.properties);
      } else {
        {
        fprintf(_coverage_fout, "1505\n");
        fflush(_coverage_fout);
        }
        if ((int )(intern->array)->type == 4) {
          {
          fprintf(_coverage_fout, "1499\n");
          fflush(_coverage_fout);
          }
          tmp___3 = (intern->array)->value.ht;
        } else {
          {
          fprintf(_coverage_fout, "1503\n");
          fflush(_coverage_fout);
          }
          if ((int )(intern->array)->type == 5) {
            {
            fprintf(_coverage_fout, "1500\n");
            fflush(_coverage_fout);
            }
            tmp___1 = (*(((intern->array)->value.obj.handlers)->get_properties))(intern->array);
            {
            fprintf(_coverage_fout, "1501\n");
            fflush(_coverage_fout);
            }
            tmp___2 = tmp___1;
          } else {
            {
            fprintf(_coverage_fout, "1502\n");
            fflush(_coverage_fout);
            }
            tmp___2 = (HashTable *)((void *)0);
          }
          {
          fprintf(_coverage_fout, "1504\n");
          fflush(_coverage_fout);
          }
          tmp___3 = tmp___2;
        }
        {
        fprintf(_coverage_fout, "1506\n");
        fflush(_coverage_fout);
        }
        return (tmp___3);
      }
    }
  }
}
}
zend_object_handlers spl_handler_ArrayIterator  ;
zend_object_handlers spl_handler_ArrayObject  ;
extern int zm_startup_spl_exceptions(int type , int module_number ) ;
extern zend_class_entry __attribute__((__visibility__("default")))  *spl_ce_UnderflowException ;
extern zend_class_entry __attribute__((__visibility__("default")))  *spl_ce_RangeException ;
extern zend_class_entry __attribute__((__visibility__("default")))  *spl_ce_OverflowException ;
extern zend_class_entry __attribute__((__visibility__("default")))  *spl_ce_RuntimeException ;
extern zend_class_entry __attribute__((__visibility__("default")))  *spl_ce_OutOfRangeException ;
extern zend_class_entry __attribute__((__visibility__("default")))  *spl_ce_LengthException ;
extern zend_class_entry __attribute__((__visibility__("default")))  *spl_ce_DomainException ;
extern zend_class_entry __attribute__((__visibility__("default")))  *spl_ce_BadFunctionCallException ;
extern zend_class_entry __attribute__((__visibility__("default")))  *spl_ce_LogicException ;
zend_class_entry __attribute__((__visibility__("default")))  *spl_ce_RecursiveArrayIterator  ;
zend_class_entry __attribute__((__visibility__("default")))  *spl_ce_ArrayIterator  ;
zend_class_entry __attribute__((__visibility__("default")))  *spl_ce_ArrayObject  ;
extern int __attribute__((__visibility__("default")))  spl_iterator_apply(zval *obj ,
                                                                          int (*apply_func)(zend_object_iterator *iter ,
                                                                                            void *puser ) ,
                                                                          void *puser ) ;
extern void zif_iterator_apply(int ht , zval *return_value ,
                               zval **return_value_ptr , zval *this_ptr ,
                               int return_value_used ) ;
extern void zif_iterator_count(int ht , zval *return_value ,
                               zval **return_value_ptr , zval *this_ptr ,
                               int return_value_used ) ;
extern void zif_iterator_to_array(int ht , zval *return_value ,
                                  zval **return_value_ptr , zval *this_ptr ,
                                  int return_value_used ) ;
extern int zm_startup_spl_iterators(int type , int module_number ) ;
extern zend_class_entry __attribute__((__visibility__("default")))  *spl_ce_RecursiveCallbackFilterIterator ;
extern zend_class_entry __attribute__((__visibility__("default")))  *spl_ce_CallbackFilterIterator ;
extern zend_class_entry __attribute__((__visibility__("default")))  *spl_ce_RecursiveRegexIterator ;
extern zend_class_entry __attribute__((__visibility__("default")))  *spl_ce_RegexIterator ;
extern zend_class_entry __attribute__((__visibility__("default")))  *spl_ce_AppendIterator ;
extern zend_class_entry __attribute__((__visibility__("default")))  *spl_ce_EmptyIterator ;
extern zend_class_entry __attribute__((__visibility__("default")))  *spl_ce_InfiniteIterator ;
extern zend_class_entry __attribute__((__visibility__("default")))  *spl_ce_NoRewindIterator ;
extern zend_class_entry __attribute__((__visibility__("default")))  *spl_ce_IteratorIterator ;
extern zend_class_entry __attribute__((__visibility__("default")))  *spl_ce_OuterIterator ;
extern zend_class_entry __attribute__((__visibility__("default")))  *spl_ce_RecursiveCachingIterator ;
extern zend_class_entry __attribute__((__visibility__("default")))  *spl_ce_CachingIterator ;
extern zend_class_entry __attribute__((__visibility__("default")))  *spl_ce_LimitIterator ;
extern zend_class_entry __attribute__((__visibility__("default")))  *spl_ce_ParentIterator ;
extern zend_class_entry __attribute__((__visibility__("default")))  *spl_ce_RecursiveFilterIterator ;
extern zend_class_entry __attribute__((__visibility__("default")))  *spl_ce_FilterIterator ;
extern zend_class_entry __attribute__((__visibility__("default")))  *spl_ce_RecursiveTreeIterator ;
extern zend_class_entry __attribute__((__visibility__("default")))  *spl_ce_RecursiveIteratorIterator ;
struct __anonstruct_pcre_cache_entry_86;
typedef struct __anonstruct_pcre_cache_entry_86 pcre_cache_entry;
extern void __attribute__((__visibility__("default")))  php_pcre_grep_impl(pcre_cache_entry *pce ,
                                                                           zval *input ,
                                                                           zval *return_value ,
                                                                           long flags ) ;
extern void __attribute__((__visibility__("default")))  php_pcre_split_impl(pcre_cache_entry *pce ,
                                                                            char *subject ,
                                                                            int subject_len ,
                                                                            zval *return_value ,
                                                                            long limit_val ,
                                                                            long flags ) ;
extern char __attribute__((__visibility__("default")))  *php_pcre_replace_impl(pcre_cache_entry *pce ,
                                                                               char *subject ,
                                                                               int subject_len ,
                                                                               zval *return_value ,
                                                                               int is_callable_replace ,
                                                                               int *result_len ,
                                                                               int limit ,
                                                                               int *replace_count ) ;
extern void __attribute__((__visibility__("default")))  php_pcre_match_impl(pcre_cache_entry *pce ,
                                                                            char *subject ,
                                                                            int subject_len ,
                                                                            zval *return_value ,
                                                                            zval *subpats ,
                                                                            int global ,
                                                                            int use_flags ,
                                                                            long flags ,
                                                                            long start_offset ) ;
extern pcre_cache_entry __attribute__((__visibility__("default")))  *pcre_get_compiled_regex_cache(char *regex ,
                                                                                                   int regex_len ) ;
typedef struct _zend_module_entry zend_module_entry;
extern zend_module_entry pcre_module_entry ;
struct pcre_extra;
typedef struct pcre_extra pcre_extra;
struct real_pcre;
typedef struct real_pcre pcre;
extern pcre __attribute__((__visibility__("default")))  *pcre_get_compiled_regex_ex(char *regex ,
                                                                                    pcre_extra **extra ,
                                                                                    int *preg_options ,
                                                                                    int *coptions ) ;
extern pcre __attribute__((__visibility__("default")))  *pcre_get_compiled_regex(char *regex ,
                                                                                 pcre_extra **extra ,
                                                                                 int *options ) ;
extern char __attribute__((__visibility__("default")))  *php_pcre_replace(char *regex ,
                                                                          int regex_len ,
                                                                          char *subject ,
                                                                          int subject_len ,
                                                                          zval *replace_val ,
                                                                          int is_callable_replace ,
                                                                          int *result_len ,
                                                                          int limit ,
                                                                          int *replace_count ) ;
struct __locale_struct;
typedef struct __locale_struct *__locale_t;
extern  __attribute__((__nothrow__)) __locale_t uselocale(__locale_t __dataset ) ;
extern  __attribute__((__nothrow__)) void freelocale(__locale_t __dataset ) ;
extern  __attribute__((__nothrow__)) __locale_t duplocale(__locale_t __dataset ) ;
extern  __attribute__((__nothrow__)) __locale_t newlocale(int __category_mask ,
                                                          char const   *__locale ,
                                                          __locale_t __base ) ;
struct lconv;
extern  __attribute__((__nothrow__)) struct lconv *localeconv(void) ;
extern  __attribute__((__nothrow__)) char *setlocale(int __category ,
                                                     char const   *__locale ) ;
extern char const   *php_pcre_version(void) ;
extern pcre_extra *php_pcre_study(pcre const   * , int  , char const   ** ) ;
extern int php_pcre_refcount(pcre * , int  ) ;
extern unsigned char const   *php_pcre_maketables(void) ;
extern int pcre_info(pcre const   * , int * , int * ) ;
extern int php_pcre_get_substring_list(char const   * , int * , int  ,
                                       char const   *** ) ;
extern int php_pcre_get_substring(char const   * , int * , int  , int  ,
                                  char const   ** ) ;
extern int php_pcre_get_stringtable_entries(pcre const   * , char const   * ,
                                            char ** , char ** ) ;
extern int php_pcre_get_stringnumber(pcre const   * , char const   * ) ;
extern int php_pcre_get_named_substring(pcre const   * , char const   * ,
                                        int * , int  , char const   * ,
                                        char const   ** ) ;
extern int php_pcre_fullinfo(pcre const   * , pcre_extra const   * , int  ,
                             void * ) ;
extern void php_pcre_free_substring_list(char const   ** ) ;
extern void php_pcre_free_substring(char const   * ) ;
extern int php_pcre_exec(pcre const   * , pcre_extra const   * ,
                         char const   * , int  , int  , int  , int * , int  ) ;
extern int pcre_dfa_exec(pcre const   * , pcre_extra const   * ,
                         char const   * , int  , int  , int  , int * , int  ,
                         int * , int  ) ;
extern int php_pcre_copy_substring(char const   * , int * , int  , int  ,
                                   char * , int  ) ;
extern int php_pcre_copy_named_substring(pcre const   * , char const   * ,
                                         int * , int  , char const   * ,
                                         char * , int  ) ;
extern int php_pcre_config(int  , void * ) ;
extern pcre *php_pcre_compile2(char const   * , int  , int * , char const   ** ,
                               int * , unsigned char const   * ) ;
extern pcre *php_pcre_compile(char const   * , int  , char const   ** , int * ,
                              unsigned char const   * ) ;
struct pcre_callout_block;
typedef struct pcre_callout_block pcre_callout_block;
extern int (*php_pcre_callout)(pcre_callout_block * ) ;
extern void (*php_pcre_stack_free)(void * ) ;
extern void *(*php_pcre_stack_malloc)(size_t  ) ;
extern void (*php_pcre_free)(void * ) ;
extern void *(*php_pcre_malloc)(size_t  ) ;
extern void __attribute__((__visibility__("default")))  spl_instantiate(zend_class_entry *pce ,
                                                                        zval **object ,
                                                                        int alloc ) ;
__inline static int spl_instantiate_arg_ex2(zend_class_entry *pce ,
                                            zval **retval , int alloc ,
                                            zval *arg1 , zval *arg2 ) 
{ size_t tmp ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1478\n");
  fflush(_coverage_fout);
  }
  spl_instantiate(pce, retval, alloc);
  {
  fprintf(_coverage_fout, "1479\n");
  fflush(_coverage_fout);
  }
  tmp = strlen((pce->constructor)->common.function_name);
  {
  fprintf(_coverage_fout, "1480\n");
  fflush(_coverage_fout);
  }
  zend_call_method(retval, pce, & pce->constructor,
                   (pce->constructor)->common.function_name, (int )tmp,
                   (zval **)((void *)0), 2, arg1, arg2);
  {
  fprintf(_coverage_fout, "1481\n");
  fflush(_coverage_fout);
  }
  return (0);
}
}
__inline static int spl_instantiate_arg_ex1(zend_class_entry *pce ,
                                            zval **retval , int alloc ,
                                            zval *arg1 ) 
{ size_t tmp ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1474\n");
  fflush(_coverage_fout);
  }
  spl_instantiate(pce, retval, alloc);
  {
  fprintf(_coverage_fout, "1475\n");
  fflush(_coverage_fout);
  }
  tmp = strlen((pce->constructor)->common.function_name);
  {
  fprintf(_coverage_fout, "1476\n");
  fflush(_coverage_fout);
  }
  zend_call_method(retval, pce, & pce->constructor,
                   (pce->constructor)->common.function_name, (int )tmp,
                   (zval **)((void *)0), 1, arg1, (zval *)((void *)0));
  {
  fprintf(_coverage_fout, "1477\n");
  fflush(_coverage_fout);
  }
  return (0);
}
}
__inline static int spl_instantiate_arg_ex1(zend_class_entry *pce ,
                                            zval **retval , int alloc ,
                                            zval *arg1 ) ;
extern long __attribute__((__visibility__("default")))  spl_offset_convert_to_long(zval *offset ) ;
extern int spl_add_classes(zend_class_entry *pce , zval *list , int sub ,
                           int allow , int ce_flags ) ;
extern void spl_add_traits(zval *list , zend_class_entry *pce , int allow ,
                           int ce_flags ) ;
extern void spl_add_interfaces(zval *list , zend_class_entry *pce , int allow ,
                               int ce_flags ) ;
extern void spl_add_class_name(zval *list , zend_class_entry *pce , int allow ,
                               int ce_flags ) ;
extern void spl_register_property(zend_class_entry *class_entry ,
                                  char *prop_name , int prop_name_len ,
                                  int prop_flags ) ;
extern void spl_register_interface(zend_class_entry **ppce , char *class_name ,
                                   zend_function_entry const   *functions ) ;
extern void __attribute__((__visibility__("default")))  php_spl_object_hash(zval *obj ,
                                                                            char *md5str ) ;
extern void zif_class_uses(int ht , zval *return_value ,
                           zval **return_value_ptr , zval *this_ptr ,
                           int return_value_used ) ;
extern void zif_class_implements(int ht , zval *return_value ,
                                 zval **return_value_ptr , zval *this_ptr ,
                                 int return_value_used ) ;
extern void zif_class_parents(int ht , zval *return_value ,
                              zval **return_value_ptr , zval *this_ptr ,
                              int return_value_used ) ;
extern void zif_spl_classes(int ht , zval *return_value ,
                            zval **return_value_ptr , zval *this_ptr ,
                            int return_value_used ) ;
struct _zend_spl_globals;
typedef struct _zend_spl_globals zend_spl_globals;
extern zend_spl_globals spl_globals ;
extern void zm_info_spl(zend_module_entry *zend_module ) ;
extern int zm_deactivate_spl(int type , int module_number ) ;
extern int zm_activate_spl(int type , int module_number ) ;
extern int zm_shutdown_spl(int type , int module_number ) ;
extern int zm_startup_spl(int type , int module_number ) ;
extern zend_module_entry spl_module_entry ;
extern int zend_spprintf(char **message , int max_len , char *format  , ...) ;
extern void __attribute__((__visibility__("default")))  zend_exception_error(zval *exception ,
                                                                             int severity ) ;
extern void __attribute__((__visibility__("default")))  (*zend_throw_exception_hook)(zval *ex ) ;
extern zval __attribute__((__visibility__("default")))  *zend_throw_error_exception(zend_class_entry *exception_ce ,
                                                                                    char *message ,
                                                                                    long code ,
                                                                                    int severity ) ;
extern void __attribute__((__visibility__("default")))  zend_clear_exception(void) ;
extern void __attribute__((__visibility__("default")))  zend_throw_exception_object(zval *exception ) ;
extern void __attribute__((__visibility__("default")))  zend_register_default_classes(void) ;
extern zend_class_entry __attribute__((__visibility__("default")))  *zend_get_error_exception(void) ;
extern zend_class_entry __attribute__((__visibility__("default")))  *zend_exception_get_default(void) ;
extern void zend_register_default_exception(void) ;
extern void zend_throw_exception_internal(zval *exception ) ;
extern void __attribute__((__visibility__("default")))  zend_exception_restore(void) ;
extern void __attribute__((__visibility__("default")))  zend_exception_save(void) ;
extern void __attribute__((__visibility__("default")))  zend_exception_set_previous(zval *exception ,
                                                                                    zval *add_previous ) ;
extern int __attribute__((__visibility__("default")))  zend_class_unserialize_deny(zval **object ,
                                                                                   zend_class_entry *ce ,
                                                                                   unsigned char const   *buf ,
                                                                                   zend_uint buf_len ,
                                                                                   zend_unserialize_data *data ) ;
extern int __attribute__((__visibility__("default")))  zend_class_serialize_deny(zval *object ,
                                                                                 unsigned char **buffer ,
                                                                                 zend_uint *buf_len ,
                                                                                 zend_serialize_data *data ) ;
extern int __attribute__((__visibility__("default")))  zend_user_unserialize(zval **object ,
                                                                             zend_class_entry *ce ,
                                                                             unsigned char const   *buf ,
                                                                             zend_uint buf_len ,
                                                                             zend_unserialize_data *data ) ;
extern int __attribute__((__visibility__("default")))  zend_user_serialize(zval *object ,
                                                                           unsigned char **buffer ,
                                                                           zend_uint *buf_len ,
                                                                           zend_serialize_data *data ) ;
extern void __attribute__((__visibility__("default")))  zend_register_interfaces(void) ;
extern zend_object_iterator __attribute__((__visibility__("default")))  *zend_user_it_get_new_iterator(zend_class_entry *ce ,
                                                                                                       zval *object ,
                                                                                                       int by_ref ) ;
extern zval __attribute__((__visibility__("default")))  *zend_user_it_new_iterator(zend_class_entry *ce ,
                                                                                   zval *object ) ;
extern zend_class_entry __attribute__((__visibility__("default")))  *zend_ce_traversable ;
__inline static char *smart_str_print_unsigned(char *buf , long num ) 
{ char *r ;
  char *__p ;
  unsigned long __num ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1472\n");
  fflush(_coverage_fout);
  }
  while (1) {
    {
    fprintf(_coverage_fout, "1467\n");
    fflush(_coverage_fout);
    }
    __p = buf;
    {
    fprintf(_coverage_fout, "1468\n");
    fflush(_coverage_fout);
    }
    __num = (unsigned long )num;
    {
    fprintf(_coverage_fout, "1469\n");
    fflush(_coverage_fout);
    }
    *__p = (char )'\000';
    {
    fprintf(_coverage_fout, "1470\n");
    fflush(_coverage_fout);
    }
    while (1) {
      {
      fprintf(_coverage_fout, "1463\n");
      fflush(_coverage_fout);
      }
      __p --;
      {
      fprintf(_coverage_fout, "1464\n");
      fflush(_coverage_fout);
      }
      *__p = (char )((int )((char )(__num % 10UL)) + 48);
      {
      fprintf(_coverage_fout, "1465\n");
      fflush(_coverage_fout);
      }
      __num /= 10UL;
      {
      fprintf(_coverage_fout, "1466\n");
      fflush(_coverage_fout);
      }
      if (__num > 0UL) {
        {
        fprintf(_coverage_fout, "1462\n");
        fflush(_coverage_fout);
        }

      } else {
        break;
      }
    }
    {
    fprintf(_coverage_fout, "1471\n");
    fflush(_coverage_fout);
    }
    r = __p;
    break;
  }
  {
  fprintf(_coverage_fout, "1473\n");
  fflush(_coverage_fout);
  }
  return (r);
}
}
__inline static char *smart_str_print_unsigned(char *buf , long num ) ;
__inline static char *smart_str_print_long(char *buf , long num ) 
{ char *r ;
  char *__p ;
  unsigned long __num ;
  char *__p___0 ;
  unsigned long __num___0 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1460\n");
  fflush(_coverage_fout);
  }
  while (1) {
    {
    fprintf(_coverage_fout, "1459\n");
    fflush(_coverage_fout);
    }
    if (num < 0L) {
      {
      fprintf(_coverage_fout, "1445\n");
      fflush(_coverage_fout);
      }
      while (1) {
        {
        fprintf(_coverage_fout, "1440\n");
        fflush(_coverage_fout);
        }
        __p = buf;
        {
        fprintf(_coverage_fout, "1441\n");
        fflush(_coverage_fout);
        }
        __num = (unsigned long )(- num);
        {
        fprintf(_coverage_fout, "1442\n");
        fflush(_coverage_fout);
        }
        *__p = (char )'\000';
        {
        fprintf(_coverage_fout, "1443\n");
        fflush(_coverage_fout);
        }
        while (1) {
          {
          fprintf(_coverage_fout, "1436\n");
          fflush(_coverage_fout);
          }
          __p --;
          {
          fprintf(_coverage_fout, "1437\n");
          fflush(_coverage_fout);
          }
          *__p = (char )((int )((char )(__num % 10UL)) + 48);
          {
          fprintf(_coverage_fout, "1438\n");
          fflush(_coverage_fout);
          }
          __num /= 10UL;
          {
          fprintf(_coverage_fout, "1439\n");
          fflush(_coverage_fout);
          }
          if (__num > 0UL) {
            {
            fprintf(_coverage_fout, "1435\n");
            fflush(_coverage_fout);
            }

          } else {
            break;
          }
        }
        {
        fprintf(_coverage_fout, "1444\n");
        fflush(_coverage_fout);
        }
        r = __p;
        break;
      }
      {
      fprintf(_coverage_fout, "1446\n");
      fflush(_coverage_fout);
      }
      r --;
      {
      fprintf(_coverage_fout, "1447\n");
      fflush(_coverage_fout);
      }
      *r = (char )'-';
    } else {
      {
      fprintf(_coverage_fout, "1458\n");
      fflush(_coverage_fout);
      }
      while (1) {
        {
        fprintf(_coverage_fout, "1453\n");
        fflush(_coverage_fout);
        }
        __p___0 = buf;
        {
        fprintf(_coverage_fout, "1454\n");
        fflush(_coverage_fout);
        }
        __num___0 = (unsigned long )num;
        {
        fprintf(_coverage_fout, "1455\n");
        fflush(_coverage_fout);
        }
        *__p___0 = (char )'\000';
        {
        fprintf(_coverage_fout, "1456\n");
        fflush(_coverage_fout);
        }
        while (1) {
          {
          fprintf(_coverage_fout, "1449\n");
          fflush(_coverage_fout);
          }
          __p___0 --;
          {
          fprintf(_coverage_fout, "1450\n");
          fflush(_coverage_fout);
          }
          *__p___0 = (char )((int )((char )(__num___0 % 10UL)) + 48);
          {
          fprintf(_coverage_fout, "1451\n");
          fflush(_coverage_fout);
          }
          __num___0 /= 10UL;
          {
          fprintf(_coverage_fout, "1452\n");
          fflush(_coverage_fout);
          }
          if (__num___0 > 0UL) {
            {
            fprintf(_coverage_fout, "1448\n");
            fflush(_coverage_fout);
            }

          } else {
            break;
          }
        }
        {
        fprintf(_coverage_fout, "1457\n");
        fflush(_coverage_fout);
        }
        r = __p___0;
        break;
      }
    }
    break;
  }
  {
  fprintf(_coverage_fout, "1461\n");
  fflush(_coverage_fout);
  }
  return (r);
}
}
__inline static char *smart_str_print_long(char *buf , long num ) ;
extern  __attribute__((__nothrow__)) int memcmp(void const   *__s1 ,
                                                void const   *__s2 , size_t __n )  __attribute__((__pure__,
__nonnull__(1,2))) ;
__inline static int php_varname_check(char *name , int name_len ,
                                      zend_bool silent ) 
{ int tmp ;
  int tmp___0 ;
  int tmp___1 ;
  int tmp___2 ;
  int tmp___3 ;
  int tmp___4 ;
  int tmp___5 ;
  int tmp___6 ;
  int tmp___7 ;
  int tmp___8 ;
  int tmp___9 ;
  int tmp___10 ;
  int tmp___11 ;
  int tmp___12 ;
  int tmp___13 ;
  int tmp___14 ;
  int tmp___15 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1433\n");
  fflush(_coverage_fout);
  }
  if ((unsigned int )name_len == sizeof("GLOBALS") - 1U) {
    {
    fprintf(_coverage_fout, "1370\n");
    fflush(_coverage_fout);
    }
    tmp___15 = memcmp((void const   *)name, (void const   *)"GLOBALS",
                      sizeof("GLOBALS") - 1U);
    {
    fprintf(_coverage_fout, "1371\n");
    fflush(_coverage_fout);
    }
    if (tmp___15) {
      goto _L___16;
    } else {
      {
      fprintf(_coverage_fout, "1368\n");
      fflush(_coverage_fout);
      }
      if (! silent) {
        {
        fprintf(_coverage_fout, "1366\n");
        fflush(_coverage_fout);
        }
        php_error_docref0((char const   *)((void *)0), 1 << 1L,
                          "Attempted GLOBALS variable overwrite");
      } else {
        {
        fprintf(_coverage_fout, "1367\n");
        fflush(_coverage_fout);
        }

      }
      {
      fprintf(_coverage_fout, "1369\n");
      fflush(_coverage_fout);
      }
      return (-1);
    }
  } else {
    {
    fprintf(_coverage_fout, "1432\n");
    fflush(_coverage_fout);
    }
    _L___16: /* CIL Label */ 
    if ((int )*(name + 0) == 95) {
      {
      fprintf(_coverage_fout, "1399\n");
      fflush(_coverage_fout);
      }
      if ((unsigned int )name_len == sizeof("_GET") - 1U) {
        {
        fprintf(_coverage_fout, "1372\n");
        fflush(_coverage_fout);
        }
        tmp___7 = memcmp((void const   *)name, (void const   *)"_GET",
                         sizeof("_GET") - 1U);
        {
        fprintf(_coverage_fout, "1373\n");
        fflush(_coverage_fout);
        }
        if (tmp___7) {
          goto _L___15;
        } else {
          goto _L___8;
        }
      } else {
        {
        fprintf(_coverage_fout, "1398\n");
        fflush(_coverage_fout);
        }
        _L___15: /* CIL Label */ 
        if ((unsigned int )name_len == sizeof("_POST") - 1U) {
          {
          fprintf(_coverage_fout, "1374\n");
          fflush(_coverage_fout);
          }
          tmp___8 = memcmp((void const   *)name, (void const   *)"_POST",
                           sizeof("_POST") - 1U);
          {
          fprintf(_coverage_fout, "1375\n");
          fflush(_coverage_fout);
          }
          if (tmp___8) {
            goto _L___14;
          } else {
            goto _L___8;
          }
        } else {
          {
          fprintf(_coverage_fout, "1397\n");
          fflush(_coverage_fout);
          }
          _L___14: /* CIL Label */ 
          if ((unsigned int )name_len == sizeof("_COOKIE") - 1U) {
            {
            fprintf(_coverage_fout, "1376\n");
            fflush(_coverage_fout);
            }
            tmp___9 = memcmp((void const   *)name, (void const   *)"_COOKIE",
                             sizeof("_COOKIE") - 1U);
            {
            fprintf(_coverage_fout, "1377\n");
            fflush(_coverage_fout);
            }
            if (tmp___9) {
              goto _L___13;
            } else {
              goto _L___8;
            }
          } else {
            {
            fprintf(_coverage_fout, "1396\n");
            fflush(_coverage_fout);
            }
            _L___13: /* CIL Label */ 
            if ((unsigned int )name_len == sizeof("_ENV") - 1U) {
              {
              fprintf(_coverage_fout, "1378\n");
              fflush(_coverage_fout);
              }
              tmp___10 = memcmp((void const   *)name, (void const   *)"_ENV",
                                sizeof("_ENV") - 1U);
              {
              fprintf(_coverage_fout, "1379\n");
              fflush(_coverage_fout);
              }
              if (tmp___10) {
                goto _L___12;
              } else {
                goto _L___8;
              }
            } else {
              {
              fprintf(_coverage_fout, "1395\n");
              fflush(_coverage_fout);
              }
              _L___12: /* CIL Label */ 
              if ((unsigned int )name_len == sizeof("_SERVER") - 1U) {
                {
                fprintf(_coverage_fout, "1380\n");
                fflush(_coverage_fout);
                }
                tmp___11 = memcmp((void const   *)name,
                                  (void const   *)"_SERVER",
                                  sizeof("_SERVER") - 1U);
                {
                fprintf(_coverage_fout, "1381\n");
                fflush(_coverage_fout);
                }
                if (tmp___11) {
                  goto _L___11;
                } else {
                  goto _L___8;
                }
              } else {
                {
                fprintf(_coverage_fout, "1394\n");
                fflush(_coverage_fout);
                }
                _L___11: /* CIL Label */ 
                if ((unsigned int )name_len == sizeof("_SESSION") - 1U) {
                  {
                  fprintf(_coverage_fout, "1382\n");
                  fflush(_coverage_fout);
                  }
                  tmp___12 = memcmp((void const   *)name,
                                    (void const   *)"_SESSION",
                                    sizeof("_SESSION") - 1U);
                  {
                  fprintf(_coverage_fout, "1383\n");
                  fflush(_coverage_fout);
                  }
                  if (tmp___12) {
                    goto _L___10;
                  } else {
                    goto _L___8;
                  }
                } else {
                  {
                  fprintf(_coverage_fout, "1393\n");
                  fflush(_coverage_fout);
                  }
                  _L___10: /* CIL Label */ 
                  if ((unsigned int )name_len == sizeof("_FILES") - 1U) {
                    {
                    fprintf(_coverage_fout, "1384\n");
                    fflush(_coverage_fout);
                    }
                    tmp___13 = memcmp((void const   *)name,
                                      (void const   *)"_FILES",
                                      sizeof("_FILES") - 1U);
                    {
                    fprintf(_coverage_fout, "1385\n");
                    fflush(_coverage_fout);
                    }
                    if (tmp___13) {
                      goto _L___9;
                    } else {
                      goto _L___8;
                    }
                  } else {
                    {
                    fprintf(_coverage_fout, "1392\n");
                    fflush(_coverage_fout);
                    }
                    _L___9: /* CIL Label */ 
                    if ((unsigned int )name_len == sizeof("_REQUEST") - 1U) {
                      {
                      fprintf(_coverage_fout, "1390\n");
                      fflush(_coverage_fout);
                      }
                      tmp___14 = memcmp((void const   *)name,
                                        (void const   *)"_REQUEST",
                                        sizeof("_REQUEST") - 1U);
                      {
                      fprintf(_coverage_fout, "1391\n");
                      fflush(_coverage_fout);
                      }
                      if (tmp___14) {
                        goto _L___7;
                      } else {
                        {
                        fprintf(_coverage_fout, "1388\n");
                        fflush(_coverage_fout);
                        }
                        _L___8: /* CIL Label */ 
                        if (! silent) {
                          {
                          fprintf(_coverage_fout, "1386\n");
                          fflush(_coverage_fout);
                          }
                          php_error_docref0((char const   *)((void *)0),
                                            1 << 1L,
                                            "Attempted super-global (%s) variable overwrite",
                                            name);
                        } else {
                          {
                          fprintf(_coverage_fout, "1387\n");
                          fflush(_coverage_fout);
                          }

                        }
                        {
                        fprintf(_coverage_fout, "1389\n");
                        fflush(_coverage_fout);
                        }
                        return (-1);
                      }
                    } else {
                      goto _L___7;
                    }
                  }
                }
              }
            }
          }
        }
      }
    } else {
      {
      fprintf(_coverage_fout, "1431\n");
      fflush(_coverage_fout);
      }
      _L___7: /* CIL Label */ 
      if ((int )*(name + 0) == 72) {
        {
        fprintf(_coverage_fout, "1429\n");
        fflush(_coverage_fout);
        }
        if ((unsigned int )name_len == sizeof("HTTP_POST_VARS") - 1U) {
          {
          fprintf(_coverage_fout, "1400\n");
          fflush(_coverage_fout);
          }
          tmp = memcmp((void const   *)name, (void const   *)"HTTP_POST_VARS",
                       sizeof("HTTP_POST_VARS") - 1U);
          {
          fprintf(_coverage_fout, "1401\n");
          fflush(_coverage_fout);
          }
          if (tmp) {
            goto _L___6;
          } else {
            goto _L;
          }
        } else {
          {
          fprintf(_coverage_fout, "1428\n");
          fflush(_coverage_fout);
          }
          _L___6: /* CIL Label */ 
          if ((unsigned int )name_len == sizeof("HTTP_GET_VARS") - 1U) {
            {
            fprintf(_coverage_fout, "1402\n");
            fflush(_coverage_fout);
            }
            tmp___0 = memcmp((void const   *)name,
                             (void const   *)"HTTP_GET_VARS",
                             sizeof("HTTP_GET_VARS") - 1U);
            {
            fprintf(_coverage_fout, "1403\n");
            fflush(_coverage_fout);
            }
            if (tmp___0) {
              goto _L___5;
            } else {
              goto _L;
            }
          } else {
            {
            fprintf(_coverage_fout, "1427\n");
            fflush(_coverage_fout);
            }
            _L___5: /* CIL Label */ 
            if ((unsigned int )name_len == sizeof("HTTP_COOKIE_VARS") - 1U) {
              {
              fprintf(_coverage_fout, "1404\n");
              fflush(_coverage_fout);
              }
              tmp___1 = memcmp((void const   *)name,
                               (void const   *)"HTTP_COOKIE_VARS",
                               sizeof("HTTP_COOKIE_VARS") - 1U);
              {
              fprintf(_coverage_fout, "1405\n");
              fflush(_coverage_fout);
              }
              if (tmp___1) {
                goto _L___4;
              } else {
                goto _L;
              }
            } else {
              {
              fprintf(_coverage_fout, "1426\n");
              fflush(_coverage_fout);
              }
              _L___4: /* CIL Label */ 
              if ((unsigned int )name_len == sizeof("HTTP_ENV_VARS") - 1U) {
                {
                fprintf(_coverage_fout, "1406\n");
                fflush(_coverage_fout);
                }
                tmp___2 = memcmp((void const   *)name,
                                 (void const   *)"HTTP_ENV_VARS",
                                 sizeof("HTTP_ENV_VARS") - 1U);
                {
                fprintf(_coverage_fout, "1407\n");
                fflush(_coverage_fout);
                }
                if (tmp___2) {
                  goto _L___3;
                } else {
                  goto _L;
                }
              } else {
                {
                fprintf(_coverage_fout, "1425\n");
                fflush(_coverage_fout);
                }
                _L___3: /* CIL Label */ 
                if ((unsigned int )name_len == sizeof("HTTP_SERVER_VARS") - 1U) {
                  {
                  fprintf(_coverage_fout, "1408\n");
                  fflush(_coverage_fout);
                  }
                  tmp___3 = memcmp((void const   *)name,
                                   (void const   *)"HTTP_SERVER_VARS",
                                   sizeof("HTTP_SERVER_VARS") - 1U);
                  {
                  fprintf(_coverage_fout, "1409\n");
                  fflush(_coverage_fout);
                  }
                  if (tmp___3) {
                    goto _L___2;
                  } else {
                    goto _L;
                  }
                } else {
                  {
                  fprintf(_coverage_fout, "1424\n");
                  fflush(_coverage_fout);
                  }
                  _L___2: /* CIL Label */ 
                  if ((unsigned int )name_len == sizeof("HTTP_SESSION_VARS") - 1U) {
                    {
                    fprintf(_coverage_fout, "1410\n");
                    fflush(_coverage_fout);
                    }
                    tmp___4 = memcmp((void const   *)name,
                                     (void const   *)"HTTP_SESSION_VARS",
                                     sizeof("HTTP_SESSION_VARS") - 1U);
                    {
                    fprintf(_coverage_fout, "1411\n");
                    fflush(_coverage_fout);
                    }
                    if (tmp___4) {
                      goto _L___1;
                    } else {
                      goto _L;
                    }
                  } else {
                    {
                    fprintf(_coverage_fout, "1423\n");
                    fflush(_coverage_fout);
                    }
                    _L___1: /* CIL Label */ 
                    if ((unsigned int )name_len == sizeof("HTTP_RAW_POST_DATA") - 1U) {
                      {
                      fprintf(_coverage_fout, "1412\n");
                      fflush(_coverage_fout);
                      }
                      tmp___5 = memcmp((void const   *)name,
                                       (void const   *)"HTTP_RAW_POST_DATA",
                                       sizeof("HTTP_RAW_POST_DATA") - 1U);
                      {
                      fprintf(_coverage_fout, "1413\n");
                      fflush(_coverage_fout);
                      }
                      if (tmp___5) {
                        goto _L___0;
                      } else {
                        goto _L;
                      }
                    } else {
                      {
                      fprintf(_coverage_fout, "1422\n");
                      fflush(_coverage_fout);
                      }
                      _L___0: /* CIL Label */ 
                      if ((unsigned int )name_len == sizeof("HTTP_POST_FILES") - 1U) {
                        {
                        fprintf(_coverage_fout, "1419\n");
                        fflush(_coverage_fout);
                        }
                        tmp___6 = memcmp((void const   *)name,
                                         (void const   *)"HTTP_POST_FILES",
                                         sizeof("HTTP_POST_FILES") - 1U);
                        {
                        fprintf(_coverage_fout, "1420\n");
                        fflush(_coverage_fout);
                        }
                        if (tmp___6) {
                          {
                          fprintf(_coverage_fout, "1414\n");
                          fflush(_coverage_fout);
                          }

                        } else {
                          {
                          fprintf(_coverage_fout, "1417\n");
                          fflush(_coverage_fout);
                          }
                          _L: /* CIL Label */ 
                          if (! silent) {
                            {
                            fprintf(_coverage_fout, "1415\n");
                            fflush(_coverage_fout);
                            }
                            php_error_docref0((char const   *)((void *)0),
                                              1 << 1L,
                                              "Attempted long input array (%s) overwrite",
                                              name);
                          } else {
                            {
                            fprintf(_coverage_fout, "1416\n");
                            fflush(_coverage_fout);
                            }

                          }
                          {
                          fprintf(_coverage_fout, "1418\n");
                          fflush(_coverage_fout);
                          }
                          return (-1);
                        }
                      } else {
                        {
                        fprintf(_coverage_fout, "1421\n");
                        fflush(_coverage_fout);
                        }

                      }
                    }
                  }
                }
              }
            }
          }
        }
      } else {
        {
        fprintf(_coverage_fout, "1430\n");
        fflush(_coverage_fout);
        }

      }
    }
  }
  {
  fprintf(_coverage_fout, "1434\n");
  fflush(_coverage_fout);
  }
  return (0);
}
}
__inline static int php_varname_check(char *name , int name_len ,
                                      zend_bool silent ) ;
extern zend_class_entry __attribute__((__visibility__("default")))  *php_create_empty_class(char *class_name ,
                                                                                            int len ) ;
extern void __attribute__((__visibility__("default")))  var_push_dtor(php_unserialize_data_t *var_hash ,
                                                                      zval **val ) ;
extern void __attribute__((__visibility__("default")))  var_replace(php_unserialize_data_t *var_hash ,
                                                                    zval *ozval ,
                                                                    zval **nzval ) ;
extern void __attribute__((__visibility__("default")))  php_debug_zval_dump(zval **struc ,
                                                                            int level ) ;
extern void __attribute__((__visibility__("default")))  php_var_export_ex(zval **struc ,
                                                                          int level ,
                                                                          smart_str *buf ) ;
extern void __attribute__((__visibility__("default")))  php_var_export(zval **struc ,
                                                                       int level ) ;
extern void __attribute__((__visibility__("default")))  php_var_dump(zval **struc ,
                                                                     int level ) ;
extern void zif_memory_get_peak_usage(int ht , zval *return_value ,
                                      zval **return_value_ptr , zval *this_ptr ,
                                      int return_value_used ) ;
extern void zif_memory_get_usage(int ht , zval *return_value ,
                                 zval **return_value_ptr , zval *this_ptr ,
                                 int return_value_used ) ;
extern void zif_unserialize(int ht , zval *return_value ,
                            zval **return_value_ptr , zval *this_ptr ,
                            int return_value_used ) ;
extern void zif_serialize(int ht , zval *return_value ,
                          zval **return_value_ptr , zval *this_ptr ,
                          int return_value_used ) ;
extern void zif_debug_zval_dump(int ht , zval *return_value ,
                                zval **return_value_ptr , zval *this_ptr ,
                                int return_value_used ) ;
extern void zif_var_export(int ht , zval *return_value ,
                           zval **return_value_ptr , zval *this_ptr ,
                           int return_value_used ) ;
extern void zif_var_dump(int ht , zval *return_value , zval **return_value_ptr ,
                         zval *this_ptr , int return_value_used ) ;
struct _php_shutdown_function_entry;
typedef struct _php_shutdown_function_entry php_shutdown_function_entry;
extern zend_bool __attribute__((__visibility__("default")))  append_user_shutdown_function(php_shutdown_function_entry shutdown_function_entry ) ;
extern zend_bool __attribute__((__visibility__("default")))  remove_user_shutdown_function(char *function_name ,
                                                                                           size_t function_len ) ;
extern zend_bool __attribute__((__visibility__("default")))  register_user_shutdown_function(char *function_name ,
                                                                                             size_t function_len ,
                                                                                             php_shutdown_function_entry *shutdown_function_entry ) ;
extern double __attribute__((__visibility__("default")))  php_get_inf(void) ;
extern double __attribute__((__visibility__("default")))  php_get_nan(void) ;
extern int __attribute__((__visibility__("default")))  php_prefix_varname(zval *result ,
                                                                          zval *prefix ,
                                                                          char *var_name ,
                                                                          int var_name_len ,
                                                                          zend_bool add_underscore ) ;
extern int __attribute__((__visibility__("default")))  _php_error_log_ex(int opt_err ,
                                                                         char *message ,
                                                                         int message_len ,
                                                                         char *opt ,
                                                                         char *headers ) ;
extern int __attribute__((__visibility__("default")))  _php_error_log(int opt_err ,
                                                                      char *message ,
                                                                      char *opt ,
                                                                      char *headers ) ;
extern int zm_deactivate_browscap(int type , int module_number ) ;
extern int zm_deactivate_user_filters(int type , int module_number ) ;
extern int zm_startup_user_filters(int type , int module_number ) ;
extern void zif_stream_bucket_new(int ht , zval *return_value ,
                                  zval **return_value_ptr , zval *this_ptr ,
                                  int return_value_used ) ;
extern void zif_stream_bucket_append(int ht , zval *return_value ,
                                     zval **return_value_ptr , zval *this_ptr ,
                                     int return_value_used ) ;
extern void zif_stream_bucket_prepend(int ht , zval *return_value ,
                                      zval **return_value_ptr , zval *this_ptr ,
                                      int return_value_used ) ;
extern void zif_stream_bucket_make_writeable(int ht , zval *return_value ,
                                             zval **return_value_ptr ,
                                             zval *this_ptr ,
                                             int return_value_used ) ;
extern void zif_stream_filter_register(int ht , zval *return_value ,
                                       zval **return_value_ptr ,
                                       zval *this_ptr , int return_value_used ) ;
extern void zif_stream_get_filters(int ht , zval *return_value ,
                                   zval **return_value_ptr , zval *this_ptr ,
                                   int return_value_used ) ;
extern void zif_str_rot13(int ht , zval *return_value ,
                          zval **return_value_ptr , zval *this_ptr ,
                          int return_value_used ) ;
extern void zif_parse_ini_string(int ht , zval *return_value ,
                                 zval **return_value_ptr , zval *this_ptr ,
                                 int return_value_used ) ;
extern void zif_parse_ini_file(int ht , zval *return_value ,
                               zval **return_value_ptr , zval *this_ptr ,
                               int return_value_used ) ;
extern void zif_move_uploaded_file(int ht , zval *return_value ,
                                   zval **return_value_ptr , zval *this_ptr ,
                                   int return_value_used ) ;
extern void zif_is_uploaded_file(int ht , zval *return_value ,
                                 zval **return_value_ptr , zval *this_ptr ,
                                 int return_value_used ) ;
extern void zif_sys_getloadavg(int ht , zval *return_value ,
                               zval **return_value_ptr , zval *this_ptr ,
                               int return_value_used ) ;
extern void zif_unregister_tick_function(int ht , zval *return_value ,
                                         zval **return_value_ptr ,
                                         zval *this_ptr , int return_value_used ) ;
extern void zif_register_tick_function(int ht , zval *return_value ,
                                       zval **return_value_ptr ,
                                       zval *this_ptr , int return_value_used ) ;
extern void php_if_crc32(int ht , zval *return_value , zval **return_value_ptr ,
                         zval *this_ptr , int return_value_used ) ;
extern void zif_getprotobynumber(int ht , zval *return_value ,
                                 zval **return_value_ptr , zval *this_ptr ,
                                 int return_value_used ) ;
extern void zif_getprotobyname(int ht , zval *return_value ,
                               zval **return_value_ptr , zval *this_ptr ,
                               int return_value_used ) ;
extern void zif_getservbyport(int ht , zval *return_value ,
                              zval **return_value_ptr , zval *this_ptr ,
                              int return_value_used ) ;
extern void zif_getservbyname(int ht , zval *return_value ,
                              zval **return_value_ptr , zval *this_ptr ,
                              int return_value_used ) ;
extern void zif_ignore_user_abort(int ht , zval *return_value ,
                                  zval **return_value_ptr , zval *this_ptr ,
                                  int return_value_used ) ;
extern void zif_connection_status(int ht , zval *return_value ,
                                  zval **return_value_ptr , zval *this_ptr ,
                                  int return_value_used ) ;
extern void zif_connection_aborted(int ht , zval *return_value ,
                                   zval **return_value_ptr , zval *this_ptr ,
                                   int return_value_used ) ;
extern void zif_vfprintf(int ht , zval *return_value , zval **return_value_ptr ,
                         zval *this_ptr , int return_value_used ) ;
extern void zif_fprintf(int ht , zval *return_value , zval **return_value_ptr ,
                        zval *this_ptr , int return_value_used ) ;
extern void zif_print_r(int ht , zval *return_value , zval **return_value_ptr ,
                        zval *this_ptr , int return_value_used ) ;
extern void zif_restore_include_path(int ht , zval *return_value ,
                                     zval **return_value_ptr , zval *this_ptr ,
                                     int return_value_used ) ;
extern void zif_set_include_path(int ht , zval *return_value ,
                                 zval **return_value_ptr , zval *this_ptr ,
                                 int return_value_used ) ;
extern void zif_get_include_path(int ht , zval *return_value ,
                                 zval **return_value_ptr , zval *this_ptr ,
                                 int return_value_used ) ;
extern void zif_ini_restore(int ht , zval *return_value ,
                            zval **return_value_ptr , zval *this_ptr ,
                            int return_value_used ) ;
extern void zif_ini_set(int ht , zval *return_value , zval **return_value_ptr ,
                        zval *this_ptr , int return_value_used ) ;
extern void zif_ini_get_all(int ht , zval *return_value ,
                            zval **return_value_ptr , zval *this_ptr ,
                            int return_value_used ) ;
extern void zif_ini_get(int ht , zval *return_value , zval **return_value_ptr ,
                        zval *this_ptr , int return_value_used ) ;
struct _zend_syntax_highlighter_ini;
typedef struct _zend_syntax_highlighter_ini zend_syntax_highlighter_ini;
extern void __attribute__((__visibility__("default")))  php_get_highlight_struct(zend_syntax_highlighter_ini *syntax_highlighter_ini ) ;
extern void zif_php_strip_whitespace(int ht , zval *return_value ,
                                     zval **return_value_ptr , zval *this_ptr ,
                                     int return_value_used ) ;
extern void zif_highlight_string(int ht , zval *return_value ,
                                 zval **return_value_ptr , zval *this_ptr ,
                                 int return_value_used ) ;
extern void zif_highlight_file(int ht , zval *return_value ,
                               zval **return_value_ptr , zval *this_ptr ,
                               int return_value_used ) ;
extern void zif_register_shutdown_function(int ht , zval *return_value ,
                                           zval **return_value_ptr ,
                                           zval *this_ptr ,
                                           int return_value_used ) ;
extern void zif_forward_static_call_array(int ht , zval *return_value ,
                                          zval **return_value_ptr ,
                                          zval *this_ptr ,
                                          int return_value_used ) ;
extern void zif_forward_static_call(int ht , zval *return_value ,
                                    zval **return_value_ptr , zval *this_ptr ,
                                    int return_value_used ) ;
extern void zif_call_user_method_array(int ht , zval *return_value ,
                                       zval **return_value_ptr ,
                                       zval *this_ptr , int return_value_used ) ;
extern void zif_call_user_method(int ht , zval *return_value ,
                                 zval **return_value_ptr , zval *this_ptr ,
                                 int return_value_used ) ;
extern void zif_call_user_func_array(int ht , zval *return_value ,
                                     zval **return_value_ptr , zval *this_ptr ,
                                     int return_value_used ) ;
extern void zif_call_user_func(int ht , zval *return_value ,
                               zval **return_value_ptr , zval *this_ptr ,
                               int return_value_used ) ;
extern void zif_error_get_last(int ht , zval *return_value ,
                               zval **return_value_ptr , zval *this_ptr ,
                               int return_value_used ) ;
extern void zif_error_log(int ht , zval *return_value ,
                          zval **return_value_ptr , zval *this_ptr ,
                          int return_value_used ) ;
extern void zif_get_magic_quotes_gpc(int ht , zval *return_value ,
                                     zval **return_value_ptr , zval *this_ptr ,
                                     int return_value_used ) ;
extern void zif_get_magic_quotes_runtime(int ht , zval *return_value ,
                                         zval **return_value_ptr ,
                                         zval *this_ptr , int return_value_used ) ;
extern void zif_set_magic_quotes_runtime(int ht , zval *return_value ,
                                         zval **return_value_ptr ,
                                         zval *this_ptr , int return_value_used ) ;
extern void zif_get_cfg_var(int ht , zval *return_value ,
                            zval **return_value_ptr , zval *this_ptr ,
                            int return_value_used ) ;
extern void zif_header_register_callback(int ht , zval *return_value ,
                                         zval **return_value_ptr ,
                                         zval *this_ptr , int return_value_used ) ;
extern void zif_set_time_limit(int ht , zval *return_value ,
                               zval **return_value_ptr , zval *this_ptr ,
                               int return_value_used ) ;
extern void zif_get_current_user(int ht , zval *return_value ,
                                 zval **return_value_ptr , zval *this_ptr ,
                                 int return_value_used ) ;
extern void zif_getopt(int ht , zval *return_value , zval **return_value_ptr ,
                       zval *this_ptr , int return_value_used ) ;
extern void zif_putenv(int ht , zval *return_value , zval **return_value_ptr ,
                       zval *this_ptr , int return_value_used ) ;
extern void zif_getenv(int ht , zval *return_value , zval **return_value_ptr ,
                       zval *this_ptr , int return_value_used ) ;
extern void zif_long2ip(int ht , zval *return_value , zval **return_value_ptr ,
                        zval *this_ptr , int return_value_used ) ;
extern void zif_ip2long(int ht , zval *return_value , zval **return_value_ptr ,
                        zval *this_ptr , int return_value_used ) ;
extern void php_inet_pton(int ht , zval *return_value ,
                          zval **return_value_ptr , zval *this_ptr ,
                          int return_value_used ) ;
extern void php_inet_ntop(int ht , zval *return_value ,
                          zval **return_value_ptr , zval *this_ptr ,
                          int return_value_used ) ;
extern void zif_flush(int ht , zval *return_value , zval **return_value_ptr ,
                      zval *this_ptr , int return_value_used ) ;
extern void zif_time_sleep_until(int ht , zval *return_value ,
                                 zval **return_value_ptr , zval *this_ptr ,
                                 int return_value_used ) ;
extern void zif_time_nanosleep(int ht , zval *return_value ,
                               zval **return_value_ptr , zval *this_ptr ,
                               int return_value_used ) ;
extern void zif_usleep(int ht , zval *return_value , zval **return_value_ptr ,
                       zval *this_ptr , int return_value_used ) ;
extern void zif_sleep(int ht , zval *return_value , zval **return_value_ptr ,
                      zval *this_ptr , int return_value_used ) ;
extern void zif_constant(int ht , zval *return_value , zval **return_value_ptr ,
                         zval *this_ptr , int return_value_used ) ;
extern void zm_info_basic(zend_module_entry *zend_module ) ;
extern int zm_deactivate_basic(int type , int module_number ) ;
extern int zm_activate_basic(int type , int module_number ) ;
extern int zm_shutdown_basic(int type , int module_number ) ;
extern int zm_startup_basic(int type , int module_number ) ;
extern zend_module_entry basic_functions_module ;
extern int __attribute__((__visibility__("default")))  php_url_scanner_reset_vars(void) ;
extern int __attribute__((__visibility__("default")))  php_url_scanner_add_var(char *name ,
                                                                               int name_len ,
                                                                               char *value ,
                                                                               int value_len ,
                                                                               int urlencode ) ;
extern char __attribute__((__visibility__("default")))  *php_url_scanner_adapt_single_url(char const   *url ,
                                                                                          size_t urllen ,
                                                                                          char const   *name ,
                                                                                          char const   *value ,
                                                                                          size_t *newlen ) ;
extern int zm_deactivate_url_scanner_ex(int type , int module_number ) ;
extern int zm_activate_url_scanner_ex(int type , int module_number ) ;
extern int zm_shutdown_url_scanner_ex(int type , int module_number ) ;
extern int zm_startup_url_scanner_ex(int type , int module_number ) ;
extern zend_syntax_highlighter_ini syntax_highlighter_ini ;
extern void __attribute__((__visibility__("default")))  zend_html_puts(char const   *s ,
                                                                       uint len ) ;
extern void __attribute__((__visibility__("default")))  zend_html_putc(char c ) ;
extern int __attribute__((__visibility__("default")))  highlight_string(zval *str ,
                                                                        zend_syntax_highlighter_ini *syntax_highlighter_ini ,
                                                                        char *str_name ) ;
extern int __attribute__((__visibility__("default")))  highlight_file(char *filename ,
                                                                      zend_syntax_highlighter_ini *syntax_highlighter_ini ) ;
extern void __attribute__((__visibility__("default")))  zend_strip(void) ;
extern void __attribute__((__visibility__("default")))  zend_highlight(zend_syntax_highlighter_ini *syntax_highlighter_ini ) ;
typedef int php_stat_len;
extern void __attribute__((__visibility__("default")))  php_stat(char const   *filename ,
                                                                 php_stat_len filename_length ,
                                                                 int type ,
                                                                 zval *return_value ) ;
extern void __attribute__((__visibility__("default")))  php_clear_stat_cache(zend_bool clear_realpath_cache ,
                                                                             char const   *filename ,
                                                                             int filename_len ) ;
extern void zif_touch(int ht , zval *return_value , zval **return_value_ptr ,
                      zval *this_ptr , int return_value_used ) ;
extern void zif_chmod(int ht , zval *return_value , zval **return_value_ptr ,
                      zval *this_ptr , int return_value_used ) ;
extern void zif_lchgrp(int ht , zval *return_value , zval **return_value_ptr ,
                       zval *this_ptr , int return_value_used ) ;
extern void zif_lchown(int ht , zval *return_value , zval **return_value_ptr ,
                       zval *this_ptr , int return_value_used ) ;
extern void zif_chgrp(int ht , zval *return_value , zval **return_value_ptr ,
                      zval *this_ptr , int return_value_used ) ;
extern void zif_chown(int ht , zval *return_value , zval **return_value_ptr ,
                      zval *this_ptr , int return_value_used ) ;
extern void zif_disk_free_space(int ht , zval *return_value ,
                                zval **return_value_ptr , zval *this_ptr ,
                                int return_value_used ) ;
extern void zif_disk_total_space(int ht , zval *return_value ,
                                 zval **return_value_ptr , zval *this_ptr ,
                                 int return_value_used ) ;
extern void php_if_lstat(int ht , zval *return_value , zval **return_value_ptr ,
                         zval *this_ptr , int return_value_used ) ;
extern void php_if_stat(int ht , zval *return_value , zval **return_value_ptr ,
                        zval *this_ptr , int return_value_used ) ;
extern void zif_file_exists(int ht , zval *return_value ,
                            zval **return_value_ptr , zval *this_ptr ,
                            int return_value_used ) ;
extern void zif_is_link(int ht , zval *return_value , zval **return_value_ptr ,
                        zval *this_ptr , int return_value_used ) ;
extern void zif_is_dir(int ht , zval *return_value , zval **return_value_ptr ,
                       zval *this_ptr , int return_value_used ) ;
extern void zif_is_file(int ht , zval *return_value , zval **return_value_ptr ,
                        zval *this_ptr , int return_value_used ) ;
extern void zif_is_executable(int ht , zval *return_value ,
                              zval **return_value_ptr , zval *this_ptr ,
                              int return_value_used ) ;
extern void zif_is_readable(int ht , zval *return_value ,
                            zval **return_value_ptr , zval *this_ptr ,
                            int return_value_used ) ;
extern void zif_is_writable(int ht , zval *return_value ,
                            zval **return_value_ptr , zval *this_ptr ,
                            int return_value_used ) ;
extern void zif_filetype(int ht , zval *return_value , zval **return_value_ptr ,
                         zval *this_ptr , int return_value_used ) ;
extern void zif_filesize(int ht , zval *return_value , zval **return_value_ptr ,
                         zval *this_ptr , int return_value_used ) ;
extern void zif_fileperms(int ht , zval *return_value ,
                          zval **return_value_ptr , zval *this_ptr ,
                          int return_value_used ) ;
extern void zif_fileowner(int ht , zval *return_value ,
                          zval **return_value_ptr , zval *this_ptr ,
                          int return_value_used ) ;
extern void zif_filemtime(int ht , zval *return_value ,
                          zval **return_value_ptr , zval *this_ptr ,
                          int return_value_used ) ;
extern void zif_fileinode(int ht , zval *return_value ,
                          zval **return_value_ptr , zval *this_ptr ,
                          int return_value_used ) ;
extern void zif_filegroup(int ht , zval *return_value ,
                          zval **return_value_ptr , zval *this_ptr ,
                          int return_value_used ) ;
extern void zif_filectime(int ht , zval *return_value ,
                          zval **return_value_ptr , zval *this_ptr ,
                          int return_value_used ) ;
extern void zif_fileatime(int ht , zval *return_value ,
                          zval **return_value_ptr , zval *this_ptr ,
                          int return_value_used ) ;
extern void zif_clearstatcache(int ht , zval *return_value ,
                               zval **return_value_ptr , zval *this_ptr ,
                               int return_value_used ) ;
extern void zif_realpath_cache_get(int ht , zval *return_value ,
                                   zval **return_value_ptr , zval *this_ptr ,
                                   int return_value_used ) ;
extern void zif_realpath_cache_size(int ht , zval *return_value ,
                                    zval **return_value_ptr , zval *this_ptr ,
                                    int return_value_used ) ;
extern int zm_deactivate_filestat(int type , int module_number ) ;
extern int zm_activate_filestat(int type , int module_number ) ;
struct tm;
typedef long wchar_t;
extern  __attribute__((__nothrow__)) size_t wcsftime_l(wchar_t * __restrict  __s ,
                                                       size_t __maxsize ,
                                                       wchar_t const   * __restrict  __format ,
                                                       struct tm  const  * __restrict  __tp ,
                                                       __locale_t __loc ) ;
extern  __attribute__((__nothrow__)) size_t wcsftime(wchar_t * __restrict  __s ,
                                                     size_t __maxsize ,
                                                     wchar_t const   * __restrict  __format ,
                                                     struct tm  const  * __restrict  __tp ) ;
typedef struct _IO_FILE __FILE;
extern int fputws_unlocked(wchar_t const   * __restrict  __ws ,
                           __FILE * __restrict  __stream ) ;
extern wchar_t *fgetws_unlocked(wchar_t * __restrict  __ws , int __n ,
                                __FILE * __restrict  __stream ) ;
typedef unsigned int wint_t;
extern wint_t putwchar_unlocked(wchar_t __wc ) ;
extern wint_t putwc_unlocked(wchar_t __wc , __FILE *__stream ) ;
extern wint_t fputwc_unlocked(wchar_t __wc , __FILE *__stream ) ;
extern wint_t fgetwc_unlocked(__FILE *__stream ) ;
extern wint_t getwchar_unlocked(void) ;
extern wint_t getwc_unlocked(__FILE *__stream ) ;
extern wint_t ungetwc(wint_t __wc , __FILE *__stream ) ;
extern int fputws(wchar_t const   * __restrict  __ws ,
                  __FILE * __restrict  __stream ) ;
extern wchar_t *fgetws(wchar_t * __restrict  __ws , int __n ,
                       __FILE * __restrict  __stream ) ;
extern wint_t putwchar(wchar_t __wc ) ;
extern wint_t putwc(wchar_t __wc , __FILE *__stream ) ;
extern wint_t fputwc(wchar_t __wc , __FILE *__stream ) ;
extern wint_t getwchar(void) ;
extern wint_t getwc(__FILE *__stream ) ;
extern wint_t fgetwc(__FILE *__stream ) ;
typedef __builtin_va_list __gnuc_va_list;
extern  __attribute__((__nothrow__)) int vswscanf(wchar_t const   * __restrict  __s ,
                                                  wchar_t const   * __restrict  __format ,
                                                  __gnuc_va_list __arg ) ;
extern int vwscanf(wchar_t const   * __restrict  __format ,
                   __gnuc_va_list __arg ) ;
extern int vfwscanf(__FILE * __restrict  __s ,
                    wchar_t const   * __restrict  __format ,
                    __gnuc_va_list __arg ) ;
extern  __attribute__((__nothrow__)) int swscanf(wchar_t const   * __restrict  __s ,
                                                 wchar_t const   * __restrict  __format 
                                                 , ...) ;
extern int wscanf(wchar_t const   * __restrict  __format  , ...) ;
extern int fwscanf(__FILE * __restrict  __stream ,
                   wchar_t const   * __restrict  __format  , ...) ;
extern  __attribute__((__nothrow__)) int vswprintf(wchar_t * __restrict  __s ,
                                                   size_t __n ,
                                                   wchar_t const   * __restrict  __format ,
                                                   __gnuc_va_list __arg ) ;
extern int vwprintf(wchar_t const   * __restrict  __format ,
                    __gnuc_va_list __arg ) ;
extern int vfwprintf(__FILE * __restrict  __s ,
                     wchar_t const   * __restrict  __format ,
                     __gnuc_va_list __arg ) ;
extern  __attribute__((__nothrow__)) int swprintf(wchar_t * __restrict  __s ,
                                                  size_t __n ,
                                                  wchar_t const   * __restrict  __format 
                                                  , ...) ;
extern int wprintf(wchar_t const   * __restrict  __format  , ...) ;
extern int fwprintf(__FILE * __restrict  __stream ,
                    wchar_t const   * __restrict  __format  , ...) ;
extern  __attribute__((__nothrow__)) int fwide(__FILE *__fp , int __mode ) ;
extern  __attribute__((__nothrow__)) __FILE *open_wmemstream(wchar_t **__bufloc ,
                                                             size_t *__sizeloc ) ;
extern  __attribute__((__nothrow__)) wchar_t *wcpncpy(wchar_t * __restrict  __dest ,
                                                      wchar_t const   * __restrict  __src ,
                                                      size_t __n ) ;
extern  __attribute__((__nothrow__)) wchar_t *wcpcpy(wchar_t * __restrict  __dest ,
                                                     wchar_t const   * __restrict  __src ) ;
extern  __attribute__((__nothrow__)) long double wcstold_l(wchar_t const   * __restrict  __nptr ,
                                                           wchar_t ** __restrict  __endptr ,
                                                           __locale_t __loc ) ;
extern  __attribute__((__nothrow__)) float wcstof_l(wchar_t const   * __restrict  __nptr ,
                                                    wchar_t ** __restrict  __endptr ,
                                                    __locale_t __loc ) ;
extern  __attribute__((__nothrow__)) double wcstod_l(wchar_t const   * __restrict  __nptr ,
                                                     wchar_t ** __restrict  __endptr ,
                                                     __locale_t __loc ) ;
extern  __attribute__((__nothrow__)) unsigned long long wcstoull_l(wchar_t const   * __restrict  __nptr ,
                                                                   wchar_t ** __restrict  __endptr ,
                                                                   int __base ,
                                                                   __locale_t __loc ) ;
extern  __attribute__((__nothrow__)) long long wcstoll_l(wchar_t const   * __restrict  __nptr ,
                                                         wchar_t ** __restrict  __endptr ,
                                                         int __base ,
                                                         __locale_t __loc ) ;
extern  __attribute__((__nothrow__)) unsigned long wcstoul_l(wchar_t const   * __restrict  __nptr ,
                                                             wchar_t ** __restrict  __endptr ,
                                                             int __base ,
                                                             __locale_t __loc ) ;
extern  __attribute__((__nothrow__)) long wcstol_l(wchar_t const   * __restrict  __nptr ,
                                                   wchar_t ** __restrict  __endptr ,
                                                   int __base ,
                                                   __locale_t __loc ) ;
extern  __attribute__((__nothrow__)) unsigned long long wcstouq(wchar_t const   * __restrict  __nptr ,
                                                                wchar_t ** __restrict  __endptr ,
                                                                int __base ) ;
extern  __attribute__((__nothrow__)) long long wcstoq(wchar_t const   * __restrict  __nptr ,
                                                      wchar_t ** __restrict  __endptr ,
                                                      int __base ) ;
extern  __attribute__((__nothrow__)) unsigned long long wcstoull(wchar_t const   * __restrict  __nptr ,
                                                                 wchar_t ** __restrict  __endptr ,
                                                                 int __base ) ;
extern  __attribute__((__nothrow__)) long long wcstoll(wchar_t const   * __restrict  __nptr ,
                                                       wchar_t ** __restrict  __endptr ,
                                                       int __base ) ;
extern  __attribute__((__nothrow__)) unsigned long wcstoul(wchar_t const   * __restrict  __nptr ,
                                                           wchar_t ** __restrict  __endptr ,
                                                           int __base ) ;
extern  __attribute__((__nothrow__)) long wcstol(wchar_t const   * __restrict  __nptr ,
                                                 wchar_t ** __restrict  __endptr ,
                                                 int __base ) ;
extern  __attribute__((__nothrow__)) long double wcstold(wchar_t const   * __restrict  __nptr ,
                                                         wchar_t ** __restrict  __endptr ) ;
extern  __attribute__((__nothrow__)) float wcstof(wchar_t const   * __restrict  __nptr ,
                                                  wchar_t ** __restrict  __endptr ) ;
extern  __attribute__((__nothrow__)) double wcstod(wchar_t const   * __restrict  __nptr ,
                                                   wchar_t ** __restrict  __endptr ) ;
extern  __attribute__((__nothrow__)) int wcswidth(wchar_t const   *__s ,
                                                  size_t __n ) ;
extern  __attribute__((__nothrow__)) int wcwidth(wchar_t __c ) ;
struct __anonstruct___mbstate_t_29;
typedef struct __anonstruct___mbstate_t_29 __mbstate_t;
typedef __mbstate_t mbstate_t;
extern  __attribute__((__nothrow__)) size_t wcsnrtombs(char * __restrict  __dst ,
                                                       wchar_t const   ** __restrict  __src ,
                                                       size_t __nwc ,
                                                       size_t __len ,
                                                       mbstate_t * __restrict  __ps ) ;
extern  __attribute__((__nothrow__)) size_t mbsnrtowcs(wchar_t * __restrict  __dst ,
                                                       char const   ** __restrict  __src ,
                                                       size_t __nmc ,
                                                       size_t __len ,
                                                       mbstate_t * __restrict  __ps ) ;
extern  __attribute__((__nothrow__)) size_t wcsrtombs(char * __restrict  __dst ,
                                                      wchar_t const   ** __restrict  __src ,
                                                      size_t __len ,
                                                      mbstate_t * __restrict  __ps ) ;
extern  __attribute__((__nothrow__)) size_t mbsrtowcs(wchar_t * __restrict  __dst ,
                                                      char const   ** __restrict  __src ,
                                                      size_t __len ,
                                                      mbstate_t * __restrict  __ps ) ;
extern  __attribute__((__nothrow__)) size_t __mbrlen(char const   * __restrict  __s ,
                                                     size_t __n ,
                                                     mbstate_t * __restrict  __ps ) ;
extern  __attribute__((__nothrow__)) size_t mbrtowc(wchar_t * __restrict  __pwc ,
                                                    char const   * __restrict  __s ,
                                                    size_t __n , mbstate_t *__p ) ;
__inline extern  __attribute__((__nothrow__)) size_t mbrlen(char const   * __restrict  __s ,
                                                            size_t __n ,
                                                            mbstate_t * __restrict  __ps ) ;
__inline extern size_t mbrlen(char const   * __restrict  __s , size_t __n ,
                              mbstate_t * __restrict  __ps ) 
{ size_t tmp ;
  size_t tmp___0 ;
  size_t tmp___1 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1364\n");
  fflush(_coverage_fout);
  }
  if ((unsigned int )__ps != (unsigned int )((void *)0)) {
    {
    fprintf(_coverage_fout, "1360\n");
    fflush(_coverage_fout);
    }
    tmp = mbrtowc((wchar_t */* __restrict  */)((void *)0), __s, __n,
                  (mbstate_t *)__ps);
    {
    fprintf(_coverage_fout, "1361\n");
    fflush(_coverage_fout);
    }
    tmp___1 = tmp;
  } else {
    {
    fprintf(_coverage_fout, "1362\n");
    fflush(_coverage_fout);
    }
    tmp___0 = __mbrlen(__s, __n, (mbstate_t */* __restrict  */)((void *)0));
    {
    fprintf(_coverage_fout, "1363\n");
    fflush(_coverage_fout);
    }
    tmp___1 = tmp___0;
  }
  {
  fprintf(_coverage_fout, "1365\n");
  fflush(_coverage_fout);
  }
  return (tmp___1);
}
}
__inline extern  __attribute__((__nothrow__)) size_t mbrlen(char const   * __restrict  __s ,
                                                            size_t __n ,
                                                            mbstate_t * __restrict  __ps ) ;
extern int __wctob_alias(wint_t __c )  __asm__("wctob")  ;
__inline extern  __attribute__((__nothrow__)) int wctob(wint_t __wc ) ;
__inline extern int wctob(wint_t __wc ) 
{ int tmp ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1358\n");
  fflush(_coverage_fout);
  }
  tmp = __wctob_alias(__wc);
  {
  fprintf(_coverage_fout, "1359\n");
  fflush(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) int wctob(wint_t __wc ) ;
extern wint_t __btowc_alias(int __c )  __asm__("btowc")  ;
__inline extern  __attribute__((__nothrow__)) wint_t btowc(int __c ) ;
__inline extern wint_t btowc(int __c ) 
{ wint_t tmp ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1356\n");
  fflush(_coverage_fout);
  }
  tmp = __btowc_alias(__c);
  {
  fprintf(_coverage_fout, "1357\n");
  fflush(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) wint_t btowc(int __c ) ;
extern  __attribute__((__nothrow__)) size_t wcrtomb(char * __restrict  __s ,
                                                    wchar_t __wc ,
                                                    mbstate_t * __restrict  __ps ) ;
extern  __attribute__((__nothrow__)) int mbsinit(mbstate_t const   *__ps )  __attribute__((__pure__)) ;
extern  __attribute__((__nothrow__)) wchar_t *wmempcpy(wchar_t * __restrict  __s1 ,
                                                       wchar_t const   * __restrict  __s2 ,
                                                       size_t __n ) ;
extern  __attribute__((__nothrow__)) wchar_t *wmemset(wchar_t *__s ,
                                                      wchar_t __c , size_t __n ) ;
extern  __attribute__((__nothrow__)) wchar_t *wmemmove(wchar_t *__s1 ,
                                                       wchar_t const   *__s2 ,
                                                       size_t __n ) ;
extern  __attribute__((__nothrow__)) wchar_t *wmemcpy(wchar_t * __restrict  __s1 ,
                                                      wchar_t const   * __restrict  __s2 ,
                                                      size_t __n ) ;
extern  __attribute__((__nothrow__)) int wmemcmp(wchar_t const   * __restrict  __s1 ,
                                                 wchar_t const   * __restrict  __s2 ,
                                                 size_t __n )  __attribute__((__pure__)) ;
extern  __attribute__((__nothrow__)) wchar_t *wmemchr(wchar_t const   *__s ,
                                                      wchar_t __c , size_t __n )  __attribute__((__pure__)) ;
extern  __attribute__((__nothrow__)) size_t wcsnlen(wchar_t const   *__s ,
                                                    size_t __maxlen )  __attribute__((__pure__)) ;
extern  __attribute__((__nothrow__)) wchar_t *wcswcs(wchar_t const   *__haystack ,
                                                     wchar_t const   *__needle )  __attribute__((__pure__)) ;
extern  __attribute__((__nothrow__)) size_t wcslen(wchar_t const   *__s )  __attribute__((__pure__)) ;
extern  __attribute__((__nothrow__)) wchar_t *wcstok(wchar_t * __restrict  __s ,
                                                     wchar_t const   * __restrict  __delim ,
                                                     wchar_t ** __restrict  __ptr ) ;
extern  __attribute__((__nothrow__)) wchar_t *wcsstr(wchar_t const   *__haystack ,
                                                     wchar_t const   *__needle )  __attribute__((__pure__)) ;
extern  __attribute__((__nothrow__)) wchar_t *wcspbrk(wchar_t const   *__wcs ,
                                                      wchar_t const   *__accept )  __attribute__((__pure__)) ;
extern  __attribute__((__nothrow__)) size_t wcsspn(wchar_t const   *__wcs ,
                                                   wchar_t const   *__accept )  __attribute__((__pure__)) ;
extern  __attribute__((__nothrow__)) size_t wcscspn(wchar_t const   *__wcs ,
                                                    wchar_t const   *__reject )  __attribute__((__pure__)) ;
extern  __attribute__((__nothrow__)) wchar_t *wcschrnul(wchar_t const   *__s ,
                                                        wchar_t __wc )  __attribute__((__pure__)) ;
extern  __attribute__((__nothrow__)) wchar_t *wcsrchr(wchar_t const   *__wcs ,
                                                      wchar_t __wc )  __attribute__((__pure__)) ;
extern  __attribute__((__nothrow__)) wchar_t *wcschr(wchar_t const   *__wcs ,
                                                     wchar_t __wc )  __attribute__((__pure__)) ;
extern  __attribute__((__nothrow__)) wchar_t *wcsdup(wchar_t const   *__s )  __attribute__((__malloc__)) ;
extern  __attribute__((__nothrow__)) size_t wcsxfrm_l(wchar_t *__s1 ,
                                                      wchar_t const   *__s2 ,
                                                      size_t __n ,
                                                      __locale_t __loc ) ;
extern  __attribute__((__nothrow__)) int wcscoll_l(wchar_t const   *__s1 ,
                                                   wchar_t const   *__s2 ,
                                                   __locale_t __loc ) ;
extern  __attribute__((__nothrow__)) size_t wcsxfrm(wchar_t * __restrict  __s1 ,
                                                    wchar_t const   * __restrict  __s2 ,
                                                    size_t __n ) ;
extern  __attribute__((__nothrow__)) int wcscoll(wchar_t const   *__s1 ,
                                                 wchar_t const   *__s2 ) ;
extern  __attribute__((__nothrow__)) int wcsncasecmp_l(wchar_t const   *__s1 ,
                                                       wchar_t const   *__s2 ,
                                                       size_t __n ,
                                                       __locale_t __loc ) ;
extern  __attribute__((__nothrow__)) int wcscasecmp_l(wchar_t const   *__s1 ,
                                                      wchar_t const   *__s2 ,
                                                      __locale_t __loc ) ;
extern  __attribute__((__nothrow__)) int wcsncasecmp(wchar_t const   *__s1 ,
                                                     wchar_t const   *__s2 ,
                                                     size_t __n ) ;
extern  __attribute__((__nothrow__)) int wcscasecmp(wchar_t const   *__s1 ,
                                                    wchar_t const   *__s2 ) ;
extern  __attribute__((__nothrow__)) int wcsncmp(wchar_t const   *__s1 ,
                                                 wchar_t const   *__s2 ,
                                                 size_t __n )  __attribute__((__pure__)) ;
extern  __attribute__((__nothrow__)) int wcscmp(wchar_t const   *__s1 ,
                                                wchar_t const   *__s2 )  __attribute__((__pure__)) ;
extern  __attribute__((__nothrow__)) wchar_t *wcsncat(wchar_t * __restrict  __dest ,
                                                      wchar_t const   * __restrict  __src ,
                                                      size_t __n ) ;
extern  __attribute__((__nothrow__)) wchar_t *wcscat(wchar_t * __restrict  __dest ,
                                                     wchar_t const   * __restrict  __src ) ;
extern  __attribute__((__nothrow__)) wchar_t *wcsncpy(wchar_t * __restrict  __dest ,
                                                      wchar_t const   * __restrict  __src ,
                                                      size_t __n ) ;
extern  __attribute__((__nothrow__)) wchar_t *wcscpy(wchar_t * __restrict  __dest ,
                                                     wchar_t const   * __restrict  __src ) ;
extern void register_phpinfo_constants(int type , int module_number ) ;
extern char __attribute__((__visibility__("default")))  *php_get_uname(char mode ) ;
extern char __attribute__((__visibility__("default")))  *php_logo_guid(void) ;
extern void __attribute__((__visibility__("default")))  php_info_print_module(zend_module_entry *module ) ;
extern void __attribute__((__visibility__("default")))  php_info_print_hr(void) ;
extern void __attribute__((__visibility__("default")))  php_info_print_box_end(void) ;
extern void __attribute__((__visibility__("default")))  php_info_print_box_start(int bg ) ;
extern void __attribute__((__visibility__("default")))  php_info_print_table_end(void) ;
extern void __attribute__((__visibility__("default")))  php_info_print_table_start(void) ;
extern void __attribute__((__visibility__("default")))  php_info_print_table_row_ex(int num_cols ,
                                                                                    char const   * 
                                                                                    , ...) ;
extern void __attribute__((__visibility__("default")))  php_info_print_table_row(int num_cols 
                                                                                 , ...) ;
extern void __attribute__((__visibility__("default")))  php_info_print_table_header(int num_cols 
                                                                                    , ...) ;
extern void __attribute__((__visibility__("default")))  php_info_print_table_colspan_header(int num_cols ,
                                                                                            char *header ) ;
extern void __attribute__((__visibility__("default")))  php_info_print_style(void) ;
extern void __attribute__((__visibility__("default")))  php_print_style(void) ;
extern void __attribute__((__visibility__("default")))  php_print_info(int flag ) ;
extern void __attribute__((__visibility__("default")))  php_print_info_htmlhead(void) ;
extern void __attribute__((__visibility__("default")))  php_info_html_esc_write(char *string ,
                                                                                int str_len ) ;
extern char __attribute__((__visibility__("default")))  *php_info_html_esc(char *string ) ;
extern void zif_php_ini_loaded_file(int ht , zval *return_value ,
                                    zval **return_value_ptr , zval *this_ptr ,
                                    int return_value_used ) ;
extern void zif_php_ini_scanned_files(int ht , zval *return_value ,
                                      zval **return_value_ptr , zval *this_ptr ,
                                      int return_value_used ) ;
extern void zif_php_uname(int ht , zval *return_value ,
                          zval **return_value_ptr , zval *this_ptr ,
                          int return_value_used ) ;
extern void zif_php_sapi_name(int ht , zval *return_value ,
                              zval **return_value_ptr , zval *this_ptr ,
                              int return_value_used ) ;
extern void zif_php_egg_logo_guid(int ht , zval *return_value ,
                                  zval **return_value_ptr , zval *this_ptr ,
                                  int return_value_used ) ;
extern void zif_zend_logo_guid(int ht , zval *return_value ,
                               zval **return_value_ptr , zval *this_ptr ,
                               int return_value_used ) ;
extern void zif_php_real_logo_guid(int ht , zval *return_value ,
                                   zval **return_value_ptr , zval *this_ptr ,
                                   int return_value_used ) ;
extern void zif_php_logo_guid(int ht , zval *return_value ,
                              zval **return_value_ptr , zval *this_ptr ,
                              int return_value_used ) ;
extern void zif_phpcredits(int ht , zval *return_value ,
                           zval **return_value_ptr , zval *this_ptr ,
                           int return_value_used ) ;
extern void zif_phpinfo(int ht , zval *return_value , zval **return_value_ptr ,
                        zval *this_ptr , int return_value_used ) ;
extern void zif_phpversion(int ht , zval *return_value ,
                           zval **return_value_ptr , zval *this_ptr ,
                           int return_value_used ) ;
extern int getdate_r(char const   * __restrict  __string ,
                     struct tm * __restrict  __resbufp ) ;
extern struct tm *getdate(char const   *__string ) ;
extern int getdate_err ;
typedef void *__timer_t;
typedef __timer_t timer_t;
extern  __attribute__((__nothrow__)) int timer_getoverrun(timer_t __timerid ) ;
struct itimerspec;
extern  __attribute__((__nothrow__)) int timer_gettime(timer_t __timerid ,
                                                       struct itimerspec *__value ) ;
extern  __attribute__((__nothrow__)) int timer_settime(timer_t __timerid ,
                                                       int __flags ,
                                                       struct itimerspec  const  * __restrict  __value ,
                                                       struct itimerspec * __restrict  __ovalue ) ;
extern  __attribute__((__nothrow__)) int timer_delete(timer_t __timerid ) ;
struct sigevent;
typedef int __clockid_t;
typedef __clockid_t clockid_t;
extern  __attribute__((__nothrow__)) int timer_create(clockid_t __clock_id ,
                                                      struct sigevent * __restrict  __evp ,
                                                      timer_t * __restrict  __timerid ) ;
typedef int __pid_t;
typedef __pid_t pid_t;
extern  __attribute__((__nothrow__)) int clock_getcpuclockid(pid_t __pid ,
                                                             clockid_t *__clock_id ) ;
struct timespec;
extern int clock_nanosleep(clockid_t __clock_id , int __flags ,
                           struct timespec  const  *__req ,
                           struct timespec *__rem ) ;
extern  __attribute__((__nothrow__)) int clock_settime(clockid_t __clock_id ,
                                                       struct timespec  const  *__tp ) ;
extern  __attribute__((__nothrow__)) int clock_gettime(clockid_t __clock_id ,
                                                       struct timespec *__tp ) ;
extern  __attribute__((__nothrow__)) int clock_getres(clockid_t __clock_id ,
                                                      struct timespec *__res ) ;
extern int nanosleep(struct timespec  const  *__requested_time ,
                     struct timespec *__remaining ) ;
extern  __attribute__((__nothrow__)) int dysize(int __year )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) time_t timelocal(struct tm *__tp ) ;
extern  __attribute__((__nothrow__)) time_t timegm(struct tm *__tp ) ;
extern  __attribute__((__nothrow__)) int stime(time_t const   *__when ) ;
extern long timezone ;
extern int daylight ;
extern  __attribute__((__nothrow__)) void tzset(void) ;
extern char *tzname[2] ;
extern long __timezone ;
extern int __daylight ;
extern char *__tzname[2] ;
extern  __attribute__((__nothrow__)) char *ctime_r(time_t const   * __restrict  __timer ,
                                                   char * __restrict  __buf ) ;
extern  __attribute__((__nothrow__)) char *asctime_r(struct tm  const  * __restrict  __tp ,
                                                     char * __restrict  __buf ) ;
extern  __attribute__((__nothrow__)) char *ctime(time_t const   *__timer ) ;
extern  __attribute__((__nothrow__)) char *asctime(struct tm  const  *__tp ) ;
extern  __attribute__((__nothrow__)) struct tm *localtime_r(time_t const   * __restrict  __timer ,
                                                            struct tm * __restrict  __tp ) ;
extern  __attribute__((__nothrow__)) struct tm *gmtime_r(time_t const   * __restrict  __timer ,
                                                         struct tm * __restrict  __tp ) ;
extern  __attribute__((__nothrow__)) struct tm *localtime(time_t const   *__timer ) ;
extern  __attribute__((__nothrow__)) struct tm *gmtime(time_t const   *__timer ) ;
extern  __attribute__((__nothrow__)) char *strptime_l(char const   * __restrict  __s ,
                                                      char const   * __restrict  __fmt ,
                                                      struct tm *__tp ,
                                                      __locale_t __loc ) ;
extern  __attribute__((__nothrow__)) size_t strftime_l(char * __restrict  __s ,
                                                       size_t __maxsize ,
                                                       char const   * __restrict  __format ,
                                                       struct tm  const  * __restrict  __tp ,
                                                       __locale_t __loc ) ;
extern  __attribute__((__nothrow__)) char *strptime(char const   * __restrict  __s ,
                                                    char const   * __restrict  __fmt ,
                                                    struct tm *__tp ) ;
extern  __attribute__((__nothrow__)) size_t strftime(char * __restrict  __s ,
                                                     size_t __maxsize ,
                                                     char const   * __restrict  __format ,
                                                     struct tm  const  * __restrict  __tp ) ;
extern  __attribute__((__nothrow__)) time_t mktime(struct tm *__tp ) ;
extern  __attribute__((__nothrow__)) double difftime(time_t __time1 ,
                                                     time_t __time0 )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) time_t time(time_t *__timer ) ;
typedef long __clock_t;
typedef __clock_t clock_t;
extern  __attribute__((__nothrow__)) clock_t clock(void) ;
struct _zend_constant;
typedef struct _zend_constant zend_constant;
extern zend_constant *zend_quick_get_constant(zend_literal const   *key ,
                                              ulong flags ) ;
extern void copy_zend_constant(zend_constant *c ) ;
extern void zend_copy_constants(HashTable *target , HashTable *sourc ) ;
extern int __attribute__((__visibility__("default")))  zend_register_constant(zend_constant *c ) ;
extern void __attribute__((__visibility__("default")))  zend_register_stringl_constant(char const   *name ,
                                                                                       uint name_len ,
                                                                                       char *strval ,
                                                                                       uint strlen ,
                                                                                       int flags ,
                                                                                       int module_number ) ;
extern void __attribute__((__visibility__("default")))  zend_register_string_constant(char const   *name ,
                                                                                      uint name_len ,
                                                                                      char *strval ,
                                                                                      int flags ,
                                                                                      int module_number ) ;
extern void __attribute__((__visibility__("default")))  zend_register_double_constant(char const   *name ,
                                                                                      uint name_len ,
                                                                                      double dval ,
                                                                                      int flags ,
                                                                                      int module_number ) ;
extern void __attribute__((__visibility__("default")))  zend_register_long_constant(char const   *name ,
                                                                                    uint name_len ,
                                                                                    long lval ,
                                                                                    int flags ,
                                                                                    int module_number ) ;
extern int __attribute__((__visibility__("default")))  zend_get_constant_ex(char const   *name ,
                                                                            uint name_len ,
                                                                            zval *result ,
                                                                            zend_class_entry *scope ,
                                                                            ulong flags ) ;
extern int __attribute__((__visibility__("default")))  zend_get_constant(char const   *name ,
                                                                         uint name_len ,
                                                                         zval *result ) ;
extern void clean_non_persistent_constants(void) ;
extern void zend_register_standard_constants(void) ;
extern int zend_shutdown_constants(void) ;
extern int zend_startup_constants(void) ;
extern void free_zend_constant(zend_constant *c ) ;
extern void clean_module_constants(int module_number ) ;
struct _realpath_cache_bucket;
typedef struct _realpath_cache_bucket realpath_cache_bucket;
extern realpath_cache_bucket __attribute__((__visibility__("default")))  **realpath_cache_get_buckets(void) ;
extern int __attribute__((__visibility__("default")))  realpath_cache_max_buckets(void) ;
extern int __attribute__((__visibility__("default")))  realpath_cache_size(void) ;
extern realpath_cache_bucket __attribute__((__visibility__("default")))  *realpath_cache_lookup(char const   *path ,
                                                                                                int path_len ,
                                                                                                time_t t ) ;
extern void __attribute__((__visibility__("default")))  realpath_cache_del(char const   *path ,
                                                                           int path_len ) ;
extern void __attribute__((__visibility__("default")))  realpath_cache_clean(void) ;
struct _virtual_cwd_globals;
typedef struct _virtual_cwd_globals virtual_cwd_globals;
extern virtual_cwd_globals cwd_globals ;
extern char __attribute__((__visibility__("default")))  *tsrm_realpath(char const   *path ,
                                                                       char *real_path ) ;
struct _cwd_state;
typedef struct _cwd_state cwd_state;
extern int __attribute__((__visibility__("default")))  virtual_file_ex(cwd_state *state ,
                                                                       char const   *path ,
                                                                       int (*verify_path)(cwd_state const   * ) ,
                                                                       int use_realpath ) ;
typedef __gid_t gid_t;
typedef __uid_t uid_t;
extern int __attribute__((__visibility__("default")))  virtual_chown(char const   *filename ,
                                                                     uid_t owner ,
                                                                     gid_t group ,
                                                                     int link ) ;
typedef __mode_t mode_t;
extern int __attribute__((__visibility__("default")))  virtual_chmod(char const   *filename ,
                                                                     mode_t mode ) ;
struct utimbuf;
extern int __attribute__((__visibility__("default")))  virtual_utime(char const   *filename ,
                                                                     struct utimbuf *buf ) ;
extern int __attribute__((__visibility__("default")))  virtual_access(char const   *pathname ,
                                                                      int mode ) ;
extern FILE __attribute__((__visibility__("default")))  *virtual_popen(char const   *command ,
                                                                       char const   *type ) ;
struct __dirstream;
typedef struct __dirstream DIR;
extern DIR __attribute__((__visibility__("default")))  *virtual_opendir(char const   *pathname ) ;
extern int __attribute__((__visibility__("default")))  virtual_rmdir(char const   *pathname ) ;
extern int __attribute__((__visibility__("default")))  virtual_mkdir(char const   *pathname ,
                                                                     mode_t mode ) ;
extern int __attribute__((__visibility__("default")))  virtual_unlink(char const   *path ) ;
struct stat;
extern int __attribute__((__visibility__("default")))  virtual_lstat(char const   *path ,
                                                                     struct stat *buf ) ;
extern int __attribute__((__visibility__("default")))  virtual_stat(char const   *path ,
                                                                    struct stat *buf ) ;
extern int __attribute__((__visibility__("default")))  virtual_rename(char *oldname ,
                                                                      char *newname ) ;
extern int __attribute__((__visibility__("default")))  virtual_creat(char const   *path ,
                                                                     mode_t mode ) ;
extern int __attribute__((__visibility__("default")))  virtual_open(char const   *path ,
                                                                    int flags 
                                                                    , ...) ;
extern FILE __attribute__((__visibility__("default")))  *virtual_fopen(char const   *path ,
                                                                       char const   *mode ) ;
extern char __attribute__((__visibility__("default")))  *virtual_realpath(char const   *path ,
                                                                          char *real_path ) ;
extern int __attribute__((__visibility__("default")))  virtual_filepath_ex(char const   *path ,
                                                                           char **filepath ,
                                                                           int (*verify_path)(cwd_state const   * ) ) ;
extern int __attribute__((__visibility__("default")))  virtual_filepath(char const   *path ,
                                                                        char **filepath ) ;
extern int __attribute__((__visibility__("default")))  virtual_chdir_file(char const   *path ,
                                                                          int (*p_chdir)(char const   *path ) ) ;
extern int __attribute__((__visibility__("default")))  virtual_chdir(char const   *path ) ;
extern char __attribute__((__visibility__("default")))  *virtual_getcwd(char *buf ,
                                                                        size_t size ) ;
extern char __attribute__((__visibility__("default")))  *virtual_getcwd_ex(size_t *length ) ;
extern void __attribute__((__visibility__("default")))  virtual_cwd_shutdown(void) ;
extern void __attribute__((__visibility__("default")))  virtual_cwd_startup(void) ;
struct dirent64;
extern  __attribute__((__nothrow__)) int versionsort64(struct dirent64  const  **__e1 ,
                                                       struct dirent64  const  **__e2 )  __attribute__((__pure__,
__nonnull__(1,2))) ;
struct dirent;
extern  __attribute__((__nothrow__)) int versionsort(struct dirent  const  **__e1 ,
                                                     struct dirent  const  **__e2 )  __attribute__((__pure__,
__nonnull__(1,2))) ;
typedef long long __quad_t;
typedef __quad_t __off64_t;
typedef int __ssize_t;
extern  __attribute__((__nothrow__)) __ssize_t getdirentries64(int __fd ,
                                                               char * __restrict  __buf ,
                                                               size_t __nbytes ,
                                                               __off64_t * __restrict  __basep )  __attribute__((__nonnull__(2,4))) ;
extern  __attribute__((__nothrow__)) __ssize_t getdirentries(int __fd ,
                                                             char * __restrict  __buf ,
                                                             size_t __nbytes ,
                                                             __off_t * __restrict  __basep )  __attribute__((__nonnull__(2,4))) ;
extern  __attribute__((__nothrow__)) int alphasort64(struct dirent64  const  **__e1 ,
                                                     struct dirent64  const  **__e2 )  __attribute__((__pure__,
__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int alphasort(struct dirent  const  **__e1 ,
                                                   struct dirent  const  **__e2 )  __attribute__((__pure__,
__nonnull__(1,2))) ;
extern int scandir64(char const   * __restrict  __dir ,
                     struct dirent64 *** __restrict  __namelist ,
                     int (*__selector)(struct dirent64  const  * ) ,
                     int (*__cmp)(struct dirent64  const  ** ,
                                  struct dirent64  const  ** ) )  __attribute__((__nonnull__(1,2))) ;
extern int scandir(char const   * __restrict  __dir ,
                   struct dirent *** __restrict  __namelist ,
                   int (*__selector)(struct dirent  const  * ) ,
                   int (*__cmp)(struct dirent  const  ** ,
                                struct dirent  const  ** ) )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int dirfd(DIR *__dirp )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) long telldir(DIR *__dirp )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) void seekdir(DIR *__dirp , long __pos )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) void rewinddir(DIR *__dirp )  __attribute__((__nonnull__(1))) ;
extern int readdir64_r(DIR * __restrict  __dirp ,
                       struct dirent64 * __restrict  __entry ,
                       struct dirent64 ** __restrict  __result )  __attribute__((__nonnull__(1,2,3))) ;
extern int readdir_r(DIR * __restrict  __dirp ,
                     struct dirent * __restrict  __entry ,
                     struct dirent ** __restrict  __result )  __attribute__((__nonnull__(1,2,3))) ;
extern struct dirent64 *readdir64(DIR *__dirp )  __attribute__((__nonnull__(1))) ;
extern struct dirent *readdir(DIR *__dirp )  __attribute__((__nonnull__(1))) ;
extern int closedir(DIR *__dirp )  __attribute__((__nonnull__(1))) ;
extern DIR *fdopendir(int __fd ) ;
extern DIR *opendir(char const   *__name )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int utime(char const   *__file ,
                                               struct utimbuf  const  *__file_times )  __attribute__((__nonnull__(1))) ;
extern int __attribute__((__visibility__("default")))  OnUpdateBaseDir(zend_ini_entry *entry ,
                                                                       char *new_value ,
                                                                       uint new_value_length ,
                                                                       void *mh_arg1 ,
                                                                       void *mh_arg2 ,
                                                                       void *mh_arg3 ,
                                                                       int stage ) ;
extern char __attribute__((__visibility__("default")))  *php_strip_url_passwd(char *path ) ;
extern FILE __attribute__((__visibility__("default")))  *php_fopen_with_path(char const   *filename ,
                                                                             char const   *mode ,
                                                                             char const   *path ,
                                                                             char **opened_path ) ;
extern char __attribute__((__visibility__("default")))  *php_resolve_path(char const   *filename ,
                                                                          int filename_len ,
                                                                          char const   *path ) ;
extern int __attribute__((__visibility__("default")))  php_check_safe_mode_include_dir(char const   *path ) ;
extern int __attribute__((__visibility__("default")))  php_check_specific_open_basedir(char const   *basedir ,
                                                                                       char const   *path ) ;
extern int __attribute__((__visibility__("default")))  php_check_open_basedir_ex(char const   *path ,
                                                                                 int warn ) ;
extern int __attribute__((__visibility__("default")))  php_check_open_basedir(char const   *path ) ;
extern char __attribute__((__visibility__("default")))  *expand_filepath_with_mode(char const   *filepath ,
                                                                                   char *real_path ,
                                                                                   char const   *relative_to ,
                                                                                   size_t relative_to_len ,
                                                                                   int use_realpath ) ;
extern char __attribute__((__visibility__("default")))  *expand_filepath_ex(char const   *filepath ,
                                                                            char *real_path ,
                                                                            char const   *relative_to ,
                                                                            size_t relative_to_len ) ;
extern char __attribute__((__visibility__("default")))  *expand_filepath(char const   *filepath ,
                                                                         char *real_path ) ;
struct _zend_file_handle;
typedef struct _zend_file_handle zend_file_handle;
extern int __attribute__((__visibility__("default")))  php_fopen_primary_script(zend_file_handle *file_handle ) ;
extern HashTable __attribute__((__visibility__("default")))  *php_ini_get_configuration_hash(void) ;
extern void __attribute__((__visibility__("default")))  php_ini_activate_per_host_config(char const   *host ,
                                                                                         uint host_len ) ;
extern void __attribute__((__visibility__("default")))  php_ini_activate_per_dir_config(char *path ,
                                                                                        uint path_len ) ;
extern int __attribute__((__visibility__("default")))  php_ini_has_per_host_config(void) ;
extern int __attribute__((__visibility__("default")))  php_ini_has_per_dir_config(void) ;
extern void __attribute__((__visibility__("default")))  php_ini_activate_config(HashTable *source_hash ,
                                                                                int modify_type ,
                                                                                int stage ) ;
extern int __attribute__((__visibility__("default")))  php_parse_user_ini_file(char const   *dirname ,
                                                                               char *ini_filename ,
                                                                               HashTable *target_hash ) ;
extern int __attribute__((__visibility__("default")))  cfg_get_string(char const   *varname ,
                                                                      char **result ) ;
extern int __attribute__((__visibility__("default")))  cfg_get_double(char const   *varname ,
                                                                      double *result ) ;
extern int __attribute__((__visibility__("default")))  cfg_get_long(char const   *varname ,
                                                                    long *result ) ;
extern zval __attribute__((__visibility__("default")))  *cfg_get_entry(char const   *name ,
                                                                       uint name_length ) ;
extern void php_ini_register_extensions(void) ;
extern int php_shutdown_config(void) ;
extern int php_init_config(void) ;
extern void __attribute__((__visibility__("default")))  config_zval_dtor(zval *zvalue ) ;
extern int __attribute__((__visibility__("default")))  zend_parse_ini_string(char *str ,
                                                                             zend_bool unbuffered_errors ,
                                                                             int scanner_mode ,
                                                                             void (*ini_parser_cb)(zval *arg1 ,
                                                                                                   zval *arg2 ,
                                                                                                   zval *arg3 ,
                                                                                                   int callback_type ,
                                                                                                   void *arg ) ,
                                                                             void *arg ) ;
extern int __attribute__((__visibility__("default")))  zend_parse_ini_file(zend_file_handle *fh ,
                                                                           zend_bool unbuffered_errors ,
                                                                           int scanner_mode ,
                                                                           void (*ini_parser_cb)(zval *arg1 ,
                                                                                                 zval *arg2 ,
                                                                                                 zval *arg3 ,
                                                                                                 int callback_type ,
                                                                                                 void *arg ) ,
                                                                           void *arg ) ;
extern int __attribute__((__visibility__("default")))  OnUpdateStringUnempty(zend_ini_entry *entry ,
                                                                             char *new_value ,
                                                                             uint new_value_length ,
                                                                             void *mh_arg1 ,
                                                                             void *mh_arg2 ,
                                                                             void *mh_arg3 ,
                                                                             int stage ) ;
extern int __attribute__((__visibility__("default")))  OnUpdateString(zend_ini_entry *entry ,
                                                                      char *new_value ,
                                                                      uint new_value_length ,
                                                                      void *mh_arg1 ,
                                                                      void *mh_arg2 ,
                                                                      void *mh_arg3 ,
                                                                      int stage ) ;
extern int __attribute__((__visibility__("default")))  OnUpdateReal(zend_ini_entry *entry ,
                                                                    char *new_value ,
                                                                    uint new_value_length ,
                                                                    void *mh_arg1 ,
                                                                    void *mh_arg2 ,
                                                                    void *mh_arg3 ,
                                                                    int stage ) ;
extern int __attribute__((__visibility__("default")))  OnUpdateLongGEZero(zend_ini_entry *entry ,
                                                                          char *new_value ,
                                                                          uint new_value_length ,
                                                                          void *mh_arg1 ,
                                                                          void *mh_arg2 ,
                                                                          void *mh_arg3 ,
                                                                          int stage ) ;
extern int __attribute__((__visibility__("default")))  OnUpdateLong(zend_ini_entry *entry ,
                                                                    char *new_value ,
                                                                    uint new_value_length ,
                                                                    void *mh_arg1 ,
                                                                    void *mh_arg2 ,
                                                                    void *mh_arg3 ,
                                                                    int stage ) ;
extern int __attribute__((__visibility__("default")))  OnUpdateBool(zend_ini_entry *entry ,
                                                                    char *new_value ,
                                                                    uint new_value_length ,
                                                                    void *mh_arg1 ,
                                                                    void *mh_arg2 ,
                                                                    void *mh_arg3 ,
                                                                    int stage ) ;
extern void __attribute__((__visibility__("default")))  display_link_numbers(zend_ini_entry *ini_entry ,
                                                                             int type ) ;
extern void __attribute__((__visibility__("default")))  zend_ini_color_displayer_cb(zend_ini_entry *ini_entry ,
                                                                                    int type ) ;
extern void __attribute__((__visibility__("default")))  zend_ini_boolean_displayer_cb(zend_ini_entry *ini_entry ,
                                                                                      int type ) ;
extern int __attribute__((__visibility__("default")))  zend_ini_register_displayer(char *name ,
                                                                                   uint name_length ,
                                                                                   void (*displayer)(zend_ini_entry *ini_entry ,
                                                                                                     int type ) ) ;
extern char __attribute__((__visibility__("default")))  *zend_ini_string_ex(char *name ,
                                                                            uint name_length ,
                                                                            int orig ,
                                                                            zend_bool *exists ) ;
extern char __attribute__((__visibility__("default")))  *zend_ini_string(char *name ,
                                                                         uint name_length ,
                                                                         int orig ) ;
extern double __attribute__((__visibility__("default")))  zend_ini_double(char *name ,
                                                                          uint name_length ,
                                                                          int orig ) ;
extern long __attribute__((__visibility__("default")))  zend_ini_long(char *name ,
                                                                      uint name_length ,
                                                                      int orig ) ;
extern void __attribute__((__visibility__("default")))  display_ini_entries(zend_module_entry *module ) ;
extern int __attribute__((__visibility__("default")))  zend_restore_ini_entry(char *name ,
                                                                              uint name_length ,
                                                                              int stage ) ;
extern int __attribute__((__visibility__("default")))  zend_alter_ini_entry_ex(char *name ,
                                                                               uint name_length ,
                                                                               char *new_value ,
                                                                               uint new_value_length ,
                                                                               int modify_type ,
                                                                               int stage ,
                                                                               int force_change ) ;
extern int __attribute__((__visibility__("default")))  zend_alter_ini_entry(char *name ,
                                                                            uint name_length ,
                                                                            char *new_value ,
                                                                            uint new_value_length ,
                                                                            int modify_type ,
                                                                            int stage ) ;
extern void __attribute__((__visibility__("default")))  zend_ini_refresh_caches(int stage ) ;
extern void __attribute__((__visibility__("default")))  zend_unregister_ini_entries(int module_number ) ;
extern int __attribute__((__visibility__("default")))  zend_register_ini_entries(zend_ini_entry const   *ini_entry ,
                                                                                 int module_number ) ;
extern void __attribute__((__visibility__("default")))  zend_ini_sort_entries(void) ;
extern int __attribute__((__visibility__("default")))  zend_copy_ini_directives(void) ;
extern int __attribute__((__visibility__("default")))  zend_ini_deactivate(void) ;
extern int __attribute__((__visibility__("default")))  zend_ini_global_shutdown(void) ;
extern int __attribute__((__visibility__("default")))  zend_ini_shutdown(void) ;
extern int __attribute__((__visibility__("default")))  zend_ini_startup(void) ;
struct _php_core_globals;
extern struct _php_core_globals  __attribute__((__visibility__("default"))) core_globals ;
struct _php_stream_wrapper;
typedef struct _php_stream_wrapper php_stream_wrapper;
extern php_stream_wrapper __attribute__((__visibility__("default")))  php_stream_rfc2397_wrapper ;
struct _php_stream_ops;
typedef struct _php_stream_ops php_stream_ops;
extern php_stream_ops __attribute__((__visibility__("default")))  php_stream_rfc2397_ops ;
extern php_stream_ops __attribute__((__visibility__("default")))  php_stream_temp_ops ;
extern php_stream_ops __attribute__((__visibility__("default")))  php_stream_memory_ops ;
struct _php_stream;
typedef struct _php_stream php_stream;
extern php_stream __attribute__((__visibility__("default")))  *_php_stream_temp_open(int mode ,
                                                                                     size_t max_memory_usage ,
                                                                                     char *buf ,
                                                                                     size_t length ) ;
extern php_stream __attribute__((__visibility__("default")))  *_php_stream_temp_create(int mode ,
                                                                                       size_t max_memory_usage ) ;
extern char __attribute__((__visibility__("default")))  *_php_stream_memory_get_buffer(php_stream *stream ,
                                                                                       size_t *length ) ;
extern php_stream __attribute__((__visibility__("default")))  *_php_stream_memory_open(int mode ,
                                                                                       char *buf ,
                                                                                       size_t length ) ;
extern php_stream __attribute__((__visibility__("default")))  *_php_stream_memory_create(int mode ) ;
struct _php_stream_wrapper_ops;
typedef struct _php_stream_wrapper_ops php_stream_wrapper_ops;
extern php_stream_wrapper_ops *php_stream_user_wrapper_ops ;
extern HashTable __attribute__((__visibility__("default")))  *php_get_stream_filters_hash_global(void) ;
extern HashTable __attribute__((__visibility__("default")))  *_php_get_stream_filters_hash(void) ;
extern HashTable __attribute__((__visibility__("default")))  *php_stream_get_url_stream_wrappers_hash_global(void) ;
extern HashTable __attribute__((__visibility__("default")))  *_php_stream_get_url_stream_wrappers_hash(void) ;
extern int __attribute__((__visibility__("default")))  _php_stream_make_seekable(php_stream *origstream ,
                                                                                 php_stream **newstream ,
                                                                                 int flags ) ;
extern void __attribute__((__visibility__("default")))  ( /* format attribute */  php_stream_wrapper_log_error)(php_stream_wrapper *wrapper ,
                                                                                                                int options ,
                                                                                                                char const   *fmt 
                                                                                                                , ...) ;
extern char __attribute__((__visibility__("default")))  *php_stream_locate_eol(php_stream *stream ,
                                                                               char *buf ,
                                                                               size_t buf_len ) ;
extern php_stream_wrapper __attribute__((__visibility__("default")))  *php_stream_locate_url_wrapper(char const   *path ,
                                                                                                     char **path_for_open ,
                                                                                                     int options ) ;
struct _php_stream_context;
typedef struct _php_stream_context php_stream_context;
extern php_stream __attribute__((__visibility__("default")))  *_php_stream_open_wrapper_ex(char *path ,
                                                                                           char *mode ,
                                                                                           int options ,
                                                                                           char **opened_path ,
                                                                                           php_stream_context *context ) ;
extern int __attribute__((__visibility__("default")))  php_unregister_url_stream_wrapper_volatile(char *protocol ) ;
extern int __attribute__((__visibility__("default")))  php_register_url_stream_wrapper_volatile(char *protocol ,
                                                                                                php_stream_wrapper *wrapper ) ;
extern int __attribute__((__visibility__("default")))  php_unregister_url_stream_wrapper(char *protocol ) ;
extern int __attribute__((__visibility__("default")))  php_register_url_stream_wrapper(char *protocol ,
                                                                                       php_stream_wrapper *wrapper ) ;
extern int zm_deactivate_streams(int type , int module_number ) ;
extern void php_shutdown_stream_hashes(void) ;
extern int php_shutdown_stream_wrappers(int module_number ) ;
extern int php_init_stream_wrappers(int module_number ) ;
extern int __attribute__((__visibility__("default")))  _php_stream_cast(php_stream *stream ,
                                                                        int castas ,
                                                                        void **ret ,
                                                                        int show_err ) ;
typedef __off_t off_t;
extern int __attribute__((__visibility__("default")))  _php_stream_mmap_unmap_ex(php_stream *stream ,
                                                                                 off_t readden ) ;
extern int __attribute__((__visibility__("default")))  _php_stream_mmap_unmap(php_stream *stream ) ;
enum __anonenum_php_stream_mmap_operation_t_76;
typedef enum __anonenum_php_stream_mmap_operation_t_76 php_stream_mmap_operation_t;
extern char __attribute__((__visibility__("default")))  *_php_stream_mmap_range(php_stream *stream ,
                                                                                size_t offset ,
                                                                                size_t length ,
                                                                                php_stream_mmap_operation_t mode ,
                                                                                size_t *mapped_len ) ;
extern php_stream_ops __attribute__((__visibility__("default")))  php_stream_userspace_dir_ops ;
extern php_stream_ops __attribute__((__visibility__("default")))  php_stream_userspace_ops ;
extern int __attribute__((__visibility__("default")))  _php_glob_stream_get_count(php_stream *stream ,
                                                                                  int *pflags ) ;
extern char __attribute__((__visibility__("default")))  *_php_glob_stream_get_pattern(php_stream *stream ,
                                                                                      int copy ,
                                                                                      int *plen ) ;
extern char __attribute__((__visibility__("default")))  *_php_glob_stream_get_path(php_stream *stream ,
                                                                                   int copy ,
                                                                                   int *plen ) ;
extern php_stream_ops __attribute__((__visibility__("default")))  php_glob_stream_ops ;
extern php_stream_wrapper __attribute__((__visibility__("default")))  php_glob_stream_wrapper ;
extern FILE __attribute__((__visibility__("default")))  *_php_stream_open_wrapper_as_file(char *path ,
                                                                                          char *mode ,
                                                                                          int options ,
                                                                                          char **opened_path ) ;
extern php_stream __attribute__((__visibility__("default")))  *_php_stream_fopen_temporary_file(char const   *dir ,
                                                                                                char const   *pfx ,
                                                                                                char **opened_path ) ;
extern php_stream __attribute__((__visibility__("default")))  *_php_stream_fopen_tmpfile(int dummy ) ;
extern php_stream __attribute__((__visibility__("default")))  *_php_stream_fopen_from_pipe(FILE *file ,
                                                                                           char const   *mode ) ;
extern php_stream __attribute__((__visibility__("default")))  *_php_stream_fopen_from_fd(int fd ,
                                                                                         char const   *mode ,
                                                                                         char const   *persistent_id ) ;
extern php_stream __attribute__((__visibility__("default")))  *_php_stream_fopen_from_file(FILE *file ,
                                                                                           char const   *mode ) ;
extern php_stream __attribute__((__visibility__("default")))  *_php_stream_fopen_with_path(char *filename ,
                                                                                           char *mode ,
                                                                                           char *path ,
                                                                                           char **opened_path ,
                                                                                           int options ) ;
extern php_stream __attribute__((__visibility__("default")))  *_php_stream_fopen(char const   *filename ,
                                                                                 char const   *mode ,
                                                                                 char **opened_path ,
                                                                                 int options ) ;
extern php_stream_wrapper __attribute__((__visibility__("default")))  php_plain_files_wrapper ;
extern php_stream_ops __attribute__((__visibility__("default")))  php_stream_stdio_ops ;
struct timeval;
typedef php_stream *php_stream_transport_factory_func(char const   *proto ,
                                                      long protolen ,
                                                      char *resourcename ,
                                                      long resourcenamelen ,
                                                      char const   *persistent_id ,
                                                      int options , int flags ,
                                                      struct timeval *timeout ,
                                                      php_stream_context *context );
extern php_stream_transport_factory_func __attribute__((__visibility__("default")))  php_stream_generic_socket_factory ;
extern HashTable __attribute__((__visibility__("default")))  *php_stream_xport_get_hash(void) ;
extern int __attribute__((__visibility__("default")))  php_stream_xport_crypto_enable(php_stream *stream ,
                                                                                      int activate ) ;
enum __anonenum_php_stream_xport_crypt_method_t_72;
typedef enum __anonenum_php_stream_xport_crypt_method_t_72 php_stream_xport_crypt_method_t;
extern int __attribute__((__visibility__("default")))  php_stream_xport_crypto_setup(php_stream *stream ,
                                                                                     php_stream_xport_crypt_method_t crypto_method ,
                                                                                     php_stream *session_stream ) ;
enum __anonenum_stream_shutdown_t_68;
typedef enum __anonenum_stream_shutdown_t_68 stream_shutdown_t;
extern int __attribute__((__visibility__("default")))  php_stream_xport_shutdown(php_stream *stream ,
                                                                                 stream_shutdown_t how ) ;
typedef unsigned int __socklen_t;
typedef __socklen_t socklen_t;
extern int __attribute__((__visibility__("default")))  php_stream_xport_sendto(php_stream *stream ,
                                                                               char const   *buf ,
                                                                               size_t buflen ,
                                                                               long flags ,
                                                                               void *addr ,
                                                                               socklen_t addrlen ) ;
extern int __attribute__((__visibility__("default")))  php_stream_xport_recvfrom(php_stream *stream ,
                                                                                 char *buf ,
                                                                                 size_t buflen ,
                                                                                 long flags ,
                                                                                 void **addr ,
                                                                                 socklen_t *addrlen ,
                                                                                 char **textaddr ,
                                                                                 int *textaddrlen ) ;
extern int __attribute__((__visibility__("default")))  php_stream_xport_get_name(php_stream *stream ,
                                                                                 int want_peer ,
                                                                                 char **textaddr ,
                                                                                 int *textaddrlen ,
                                                                                 void **addr ,
                                                                                 socklen_t *addrlen ) ;
extern int __attribute__((__visibility__("default")))  php_stream_xport_accept(php_stream *stream ,
                                                                               php_stream **client ,
                                                                               char **textaddr ,
                                                                               int *textaddrlen ,
                                                                               void **addr ,
                                                                               socklen_t *addrlen ,
                                                                               struct timeval *timeout ,
                                                                               char **error_text ) ;
extern int __attribute__((__visibility__("default")))  php_stream_xport_listen(php_stream *stream ,
                                                                               int backlog ,
                                                                               char **error_text ) ;
extern int __attribute__((__visibility__("default")))  php_stream_xport_connect(php_stream *stream ,
                                                                                char const   *name ,
                                                                                long namelen ,
                                                                                int asynchronous ,
                                                                                struct timeval *timeout ,
                                                                                char **error_text ,
                                                                                int *error_code ) ;
extern int __attribute__((__visibility__("default")))  php_stream_xport_bind(php_stream *stream ,
                                                                             char const   *name ,
                                                                             long namelen ,
                                                                             char **error_text ) ;
extern php_stream __attribute__((__visibility__("default")))  *_php_stream_xport_create(char const   *name ,
                                                                                        long namelen ,
                                                                                        int options ,
                                                                                        int flags ,
                                                                                        char const   *persistent_id ,
                                                                                        struct timeval *timeout ,
                                                                                        php_stream_context *context ,
                                                                                        char **error_string ,
                                                                                        int *error_code ) ;
extern int __attribute__((__visibility__("default")))  php_stream_xport_unregister(char *protocol ) ;
typedef php_stream_transport_factory_func *php_stream_transport_factory;
extern int __attribute__((__visibility__("default")))  php_stream_xport_register(char *protocol ,
                                                                                 php_stream_transport_factory factory ) ;
extern  __attribute__((__nothrow__)) int isfdtype(int __fd , int __fdtype ) ;
extern  __attribute__((__nothrow__)) int sockatmark(int __fd ) ;
extern  __attribute__((__nothrow__)) int shutdown(int __fd , int __how ) ;
struct sockaddr;
extern int accept4(int __fd , struct sockaddr * __restrict  __addr ,
                   socklen_t * __restrict  __addr_len , int __flags ) ;
extern int accept(int __fd , struct sockaddr * __restrict  __addr ,
                  socklen_t * __restrict  __addr_len ) ;
extern  __attribute__((__nothrow__)) int listen(int __fd , int __n ) ;
extern  __attribute__((__nothrow__)) int setsockopt(int __fd , int __level ,
                                                    int __optname ,
                                                    void const   *__optval ,
                                                    socklen_t __optlen ) ;
extern  __attribute__((__nothrow__)) int getsockopt(int __fd , int __level ,
                                                    int __optname ,
                                                    void * __restrict  __optval ,
                                                    socklen_t * __restrict  __optlen ) ;
struct msghdr;
typedef __ssize_t ssize_t;
extern ssize_t recvmsg(int __fd , struct msghdr *__message , int __flags ) ;
extern ssize_t sendmsg(int __fd , struct msghdr  const  *__message ,
                       int __flags ) ;
extern ssize_t recvfrom(int __fd , void * __restrict  __buf , size_t __n ,
                        int __flags , struct sockaddr * __restrict  __addr ,
                        socklen_t * __restrict  __addr_len ) ;
extern ssize_t sendto(int __fd , void const   *__buf , size_t __n ,
                      int __flags ,
                      struct sockaddr  const  * __restrict  __addr ,
                      socklen_t __addr_len ) ;
extern ssize_t recv(int __fd , void *__buf , size_t __n , int __flags ) ;
extern ssize_t send(int __fd , void const   *__buf , size_t __n , int __flags ) ;
extern  __attribute__((__nothrow__)) int getpeername(int __fd ,
                                                     struct sockaddr * __restrict  __addr ,
                                                     socklen_t * __restrict  __len ) ;
extern int connect(int __fd , struct sockaddr  const  * __restrict  __addr ,
                   socklen_t __len ) ;
extern  __attribute__((__nothrow__)) int getsockname(int __fd ,
                                                     struct sockaddr * __restrict  __addr ,
                                                     socklen_t * __restrict  __len ) ;
extern  __attribute__((__nothrow__)) int bind(int __fd ,
                                              struct sockaddr  const  * __restrict  __addr ,
                                              socklen_t __len ) ;
extern  __attribute__((__nothrow__)) int socketpair(int __domain , int __type ,
                                                    int __protocol , int *__fds ) ;
extern  __attribute__((__nothrow__)) int socket(int __domain , int __type ,
                                                int __protocol ) ;
struct mmsghdr;
extern int recvmmsg(int __fd , struct mmsghdr *__vmessages ,
                    unsigned int __vlen , int __flags ,
                    struct timespec  const  *__tmo ) ;
struct cmsghdr;
struct iovec;
struct msghdr {
   void *msg_name ;
   socklen_t msg_namelen ;
   struct iovec *msg_iov ;
   size_t msg_iovlen ;
   void *msg_control ;
   size_t msg_controllen ;
   int msg_flags ;
};
struct cmsghdr {
   size_t cmsg_len ;
   int cmsg_level ;
   int cmsg_type ;
   unsigned char __cmsg_data[] ;
};
__inline extern  __attribute__((__nothrow__)) struct cmsghdr *__cmsg_nxthdr(struct msghdr *__mhdr ,
                                                                            struct cmsghdr *__cmsg ) ;
__inline extern struct cmsghdr *__cmsg_nxthdr(struct msghdr *__mhdr ,
                                              struct cmsghdr *__cmsg ) 
{ 

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1352\n");
  fflush(_coverage_fout);
  }
  if (__cmsg->cmsg_len < sizeof(struct cmsghdr )) {
    {
    fprintf(_coverage_fout, "1346\n");
    fflush(_coverage_fout);
    }
    return ((struct cmsghdr *)0);
  } else {
    {
    fprintf(_coverage_fout, "1347\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "1353\n");
  fflush(_coverage_fout);
  }
  __cmsg = (struct cmsghdr *)((unsigned char *)__cmsg + (((__cmsg->cmsg_len + sizeof(size_t )) - 1U) & ~ (sizeof(size_t ) - 1U)));
  {
  fprintf(_coverage_fout, "1354\n");
  fflush(_coverage_fout);
  }
  if ((unsigned int )((unsigned char *)(__cmsg + 1)) > (unsigned int )((unsigned char *)__mhdr->msg_control + __mhdr->msg_controllen)) {
    {
    fprintf(_coverage_fout, "1348\n");
    fflush(_coverage_fout);
    }
    return ((struct cmsghdr *)0);
  } else {
    {
    fprintf(_coverage_fout, "1351\n");
    fflush(_coverage_fout);
    }
    if ((unsigned int )((unsigned char *)__cmsg + (((__cmsg->cmsg_len + sizeof(size_t )) - 1U) & ~ (sizeof(size_t ) - 1U))) > (unsigned int )((unsigned char *)__mhdr->msg_control + __mhdr->msg_controllen)) {
      {
      fprintf(_coverage_fout, "1349\n");
      fflush(_coverage_fout);
      }
      return ((struct cmsghdr *)0);
    } else {
      {
      fprintf(_coverage_fout, "1350\n");
      fflush(_coverage_fout);
      }

    }
  }
  {
  fprintf(_coverage_fout, "1355\n");
  fflush(_coverage_fout);
  }
  return (__cmsg);
}
}
__inline extern  __attribute__((__nothrow__)) struct cmsghdr *__cmsg_nxthdr(struct msghdr *__mhdr ,
                                                                            struct cmsghdr *__cmsg ) ;
extern ssize_t pwritev64(int __fd , struct iovec  const  *__iovec ,
                         int __count , __off64_t __offset ) ;
extern ssize_t preadv64(int __fd , struct iovec  const  *__iovec , int __count ,
                        __off64_t __offset ) ;
extern ssize_t pwritev(int __fd , struct iovec  const  *__iovec , int __count ,
                       __off_t __offset ) ;
extern ssize_t preadv(int __fd , struct iovec  const  *__iovec , int __count ,
                      __off_t __offset ) ;
extern ssize_t writev(int __fd , struct iovec  const  *__iovec , int __count ) ;
extern ssize_t readv(int __fd , struct iovec  const  *__iovec , int __count ) ;
extern size_t __attribute__((__visibility__("default")))  _php_stream_passthru(php_stream *src ) ;
extern size_t __attribute__((__visibility__("default")))  _php_stream_copy_to_mem(php_stream *src ,
                                                                                  char **buf ,
                                                                                  size_t maxlen ,
                                                                                  int persistent ) ;
extern int __attribute__((__visibility__("default")))  _php_stream_copy_to_stream_ex(php_stream *src ,
                                                                                     php_stream *dest ,
                                                                                     size_t maxlen ,
                                                                                     size_t *len ) ;
extern size_t __attribute__((__visibility__("default"),
__deprecated__))  _php_stream_copy_to_stream(php_stream *src ,
                                             php_stream *dest , size_t maxlen ) ;
extern int __attribute__((__visibility__("default")))  _php_stream_truncate_set_size(php_stream *stream ,
                                                                                     size_t newsize ) ;
extern int __attribute__((__visibility__("default")))  _php_stream_set_option(php_stream *stream ,
                                                                              int option ,
                                                                              int value ,
                                                                              void *ptrparam ) ;
extern int __attribute__((__visibility__("default")))  _php_stream_scandir(char *dirname ,
                                                                           char ***namelist ,
                                                                           int flags ,
                                                                           php_stream_context *context ,
                                                                           int (*compare)(char const   **a ,
                                                                                          char const   **b ) ) ;
extern int __attribute__((__visibility__("default")))  php_stream_dirent_alphasortr(char const   **a ,
                                                                                    char const   **b ) ;
extern int __attribute__((__visibility__("default")))  php_stream_dirent_alphasort(char const   **a ,
                                                                                   char const   **b ) ;
struct _php_stream_dirent;
typedef struct _php_stream_dirent php_stream_dirent;
extern php_stream_dirent __attribute__((__visibility__("default")))  *_php_stream_readdir(php_stream *dirstream ,
                                                                                          php_stream_dirent *ent ) ;
extern php_stream __attribute__((__visibility__("default")))  *_php_stream_opendir(char *path ,
                                                                                   int options ,
                                                                                   php_stream_context *context ) ;
extern int __attribute__((__visibility__("default")))  _php_stream_rmdir(char *path ,
                                                                         int options ,
                                                                         php_stream_context *context ) ;
extern int __attribute__((__visibility__("default")))  _php_stream_mkdir(char *path ,
                                                                         int mode ,
                                                                         int options ,
                                                                         php_stream_context *context ) ;
struct _php_stream_statbuf;
extern int __attribute__((__visibility__("default")))  _php_stream_stat_path(char *path ,
                                                                             int flags ,
                                                                             php_stream_statbuf *ssb ,
                                                                             php_stream_context *context ) ;
extern int __attribute__((__visibility__("default")))  _php_stream_stat(php_stream *stream ,
                                                                        php_stream_statbuf *ssb ) ;
extern int __attribute__((__visibility__("default")))  _php_stream_puts(php_stream *stream ,
                                                                        char *buf ) ;
extern char __attribute__((__visibility__("default")))  *php_stream_get_record(php_stream *stream ,
                                                                               size_t maxlen ,
                                                                               size_t *returned_len ,
                                                                               char *delim ,
                                                                               size_t delim_len ) ;
extern char __attribute__((__visibility__("default")))  *_php_stream_get_line(php_stream *stream ,
                                                                              char *buf ,
                                                                              size_t maxlen ,
                                                                              size_t *returned_len ) ;
extern int __attribute__((__visibility__("default")))  _php_stream_flush(php_stream *stream ,
                                                                         int closing ) ;
extern int __attribute__((__visibility__("default")))  _php_stream_putc(php_stream *stream ,
                                                                        int c ) ;
extern int __attribute__((__visibility__("default")))  _php_stream_getc(php_stream *stream ) ;
extern int __attribute__((__visibility__("default")))  _php_stream_eof(php_stream *stream ) ;
extern size_t __attribute__((__visibility__("default")))  ( /* format attribute */  _php_stream_printf)(php_stream *stream ,
                                                                                                        char const   *fmt 
                                                                                                        , ...) ;
extern size_t __attribute__((__visibility__("default")))  _php_stream_write(php_stream *stream ,
                                                                            char const   *buf ,
                                                                            size_t count ) ;
extern size_t __attribute__((__visibility__("default")))  _php_stream_read(php_stream *stream ,
                                                                           char *buf ,
                                                                           size_t count ) ;
extern off_t __attribute__((__visibility__("default")))  _php_stream_tell(php_stream *stream ) ;
extern int __attribute__((__visibility__("default")))  _php_stream_seek(php_stream *stream ,
                                                                        off_t offset ,
                                                                        int whence ) ;
extern int __attribute__((__visibility__("default")))  _php_stream_free(php_stream *stream ,
                                                                        int close_options ) ;
extern int __attribute__((__visibility__("default")))  php_stream_from_persistent_id(char const   *persistent_id ,
                                                                                     php_stream **stream ) ;
extern int __attribute__((__visibility__("default")))  _php_stream_free_enclosed(php_stream *stream_enclosed ,
                                                                                 int close_options ) ;
extern php_stream __attribute__((__visibility__("default")))  *php_stream_encloses(php_stream *enclosing ,
                                                                                   php_stream *enclosed ) ;
extern php_stream __attribute__((__visibility__("default")))  *_php_stream_alloc(php_stream_ops *ops ,
                                                                                 void *abstract ,
                                                                                 char const   *persistent_id ,
                                                                                 char const   *mode ) ;
struct _php_stream_filter;
typedef struct _php_stream_filter php_stream_filter;
extern php_stream_filter __attribute__((__visibility__("default")))  *php_stream_filter_create(char const   *filtername ,
                                                                                               zval *filterparams ,
                                                                                               int persistent ) ;
struct _php_stream_filter_factory;
typedef struct _php_stream_filter_factory php_stream_filter_factory;
extern int __attribute__((__visibility__("default")))  php_stream_filter_register_factory_volatile(char const   *filterpattern ,
                                                                                                   php_stream_filter_factory *factory ) ;
extern int __attribute__((__visibility__("default")))  php_stream_filter_unregister_factory(char const   *filterpattern ) ;
extern int __attribute__((__visibility__("default")))  php_stream_filter_register_factory(char const   *filterpattern ,
                                                                                          php_stream_filter_factory *factory ) ;
struct _php_stream_filter_ops;
typedef struct _php_stream_filter_ops php_stream_filter_ops;
extern php_stream_filter __attribute__((__visibility__("default")))  *_php_stream_filter_alloc(php_stream_filter_ops *fops ,
                                                                                               void *abstract ,
                                                                                               int persistent ) ;
extern void __attribute__((__visibility__("default")))  php_stream_filter_free(php_stream_filter *filter ) ;
extern php_stream_filter __attribute__((__visibility__("default")))  *php_stream_filter_remove(php_stream_filter *filter ,
                                                                                               int call_dtor ) ;
extern int __attribute__((__visibility__("default")))  _php_stream_filter_flush(php_stream_filter *filter ,
                                                                                int finish ) ;
struct _php_stream_filter_chain;
typedef struct _php_stream_filter_chain php_stream_filter_chain;
extern int __attribute__((__visibility__("default")))  php_stream_filter_append_ex(php_stream_filter_chain *chain ,
                                                                                   php_stream_filter *filter ) ;
extern void __attribute__((__visibility__("default")))  _php_stream_filter_append(php_stream_filter_chain *chain ,
                                                                                  php_stream_filter *filter ) ;
extern int __attribute__((__visibility__("default")))  php_stream_filter_prepend_ex(php_stream_filter_chain *chain ,
                                                                                    php_stream_filter *filter ) ;
extern void __attribute__((__visibility__("default")))  _php_stream_filter_prepend(php_stream_filter_chain *chain ,
                                                                                   php_stream_filter *filter ) ;
struct _php_stream_bucket;
typedef struct _php_stream_bucket php_stream_bucket;
extern php_stream_bucket __attribute__((__visibility__("default")))  *php_stream_bucket_make_writeable(php_stream_bucket *bucket ) ;
extern void __attribute__((__visibility__("default")))  php_stream_bucket_unlink(php_stream_bucket *bucket ) ;
struct _php_stream_bucket_brigade;
typedef struct _php_stream_bucket_brigade php_stream_bucket_brigade;
extern void __attribute__((__visibility__("default")))  php_stream_bucket_append(php_stream_bucket_brigade *brigade ,
                                                                                 php_stream_bucket *bucket ) ;
extern void __attribute__((__visibility__("default")))  php_stream_bucket_prepend(php_stream_bucket_brigade *brigade ,
                                                                                  php_stream_bucket *bucket ) ;
extern void __attribute__((__visibility__("default")))  php_stream_bucket_delref(php_stream_bucket *bucket ) ;
extern int __attribute__((__visibility__("default")))  php_stream_bucket_split(php_stream_bucket *in ,
                                                                               php_stream_bucket **left ,
                                                                               php_stream_bucket **right ,
                                                                               size_t length ) ;
extern php_stream_bucket __attribute__((__visibility__("default")))  *php_stream_bucket_new(php_stream *stream ,
                                                                                            char *buf ,
                                                                                            size_t buflen ,
                                                                                            int own_buf ,
                                                                                            int buf_persistent ) ;
extern php_stream_context __attribute__((__visibility__("default")))  *php_stream_context_set(php_stream *stream ,
                                                                                              php_stream_context *context ) ;
extern void __attribute__((__visibility__("default")))  php_stream_notification_notify(php_stream_context *context ,
                                                                                       int notifycode ,
                                                                                       int severity ,
                                                                                       char *xmsg ,
                                                                                       int xcode ,
                                                                                       size_t bytes_sofar ,
                                                                                       size_t bytes_max ,
                                                                                       void *ptr ) ;
struct _php_stream_notifier;
typedef struct _php_stream_notifier php_stream_notifier;
extern void __attribute__((__visibility__("default")))  php_stream_notification_free(php_stream_notifier *notifier ) ;
extern php_stream_notifier __attribute__((__visibility__("default")))  *php_stream_notification_alloc(void) ;
extern int __attribute__((__visibility__("default")))  php_stream_context_set_option(php_stream_context *context ,
                                                                                     char const   *wrappername ,
                                                                                     char const   *optionname ,
                                                                                     zval *optionvalue ) ;
extern int __attribute__((__visibility__("default")))  php_stream_context_get_option(php_stream_context *context ,
                                                                                     char const   *wrappername ,
                                                                                     char const   *optionname ,
                                                                                     zval ***optionvalue ) ;
extern php_stream_context __attribute__((__visibility__("default")))  *php_stream_context_alloc(void) ;
extern void __attribute__((__visibility__("default")))  php_stream_context_free(php_stream_context *context ) ;
extern int __attribute__((__visibility__("default")))  php_file_le_stream_filter(void) ;
extern int __attribute__((__visibility__("default")))  php_file_le_pstream(void) ;
extern int __attribute__((__visibility__("default")))  php_file_le_stream(void) ;
struct stat64;
extern  __attribute__((__nothrow__)) int __fxstatat64(int __ver , int __fildes ,
                                                      char const   *__filename ,
                                                      struct stat64 *__stat_buf ,
                                                      int __flag )  __attribute__((__nonnull__(3,4))) ;
__inline extern  __attribute__((__nothrow__)) int fstatat64(int __fd ,
                                                            char const   * __restrict  __filename ,
                                                            struct stat64 * __restrict  __statbuf ,
                                                            int __flag )  __attribute__((__nonnull__(2,3))) ;
__inline extern int fstatat64(int __fd , char const   * __restrict  __filename ,
                              struct stat64 * __restrict  __statbuf ,
                              int __flag ) 
{ int tmp ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1344\n");
  fflush(_coverage_fout);
  }
  tmp = __fxstatat64(3, __fd, (char const   *)__filename,
                     (struct stat64 *)__statbuf, __flag);
  {
  fprintf(_coverage_fout, "1345\n");
  fflush(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) int fstatat64(int __fd ,
                                                            char const   * __restrict  __filename ,
                                                            struct stat64 * __restrict  __statbuf ,
                                                            int __flag )  __attribute__((__nonnull__(2,3))) ;
extern  __attribute__((__nothrow__)) int __fxstat64(int __ver , int __fildes ,
                                                    struct stat64 *__stat_buf )  __attribute__((__nonnull__(3))) ;
__inline extern  __attribute__((__nothrow__)) int fstat64(int __fd ,
                                                          struct stat64 *__statbuf )  __attribute__((__nonnull__(2))) ;
__inline extern int fstat64(int __fd , struct stat64 *__statbuf ) 
{ int tmp ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1342\n");
  fflush(_coverage_fout);
  }
  tmp = __fxstat64(3, __fd, __statbuf);
  {
  fprintf(_coverage_fout, "1343\n");
  fflush(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) int fstat64(int __fd ,
                                                          struct stat64 *__statbuf )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) int __lxstat64(int __ver ,
                                                    char const   *__filename ,
                                                    struct stat64 *__stat_buf )  __attribute__((__nonnull__(2,3))) ;
__inline extern  __attribute__((__nothrow__)) int lstat64(char const   * __restrict  __path ,
                                                          struct stat64 * __restrict  __statbuf )  __attribute__((__nonnull__(1,2))) ;
__inline extern int lstat64(char const   * __restrict  __path ,
                            struct stat64 * __restrict  __statbuf ) 
{ int tmp ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1340\n");
  fflush(_coverage_fout);
  }
  tmp = __lxstat64(3, (char const   *)__path, (struct stat64 *)__statbuf);
  {
  fprintf(_coverage_fout, "1341\n");
  fflush(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) int lstat64(char const   * __restrict  __path ,
                                                          struct stat64 * __restrict  __statbuf )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int __xstat64(int __ver ,
                                                   char const   *__filename ,
                                                   struct stat64 *__stat_buf )  __attribute__((__nonnull__(2,3))) ;
__inline extern  __attribute__((__nothrow__)) int stat64(char const   * __restrict  __path ,
                                                         struct stat64 * __restrict  __statbuf )  __attribute__((__nonnull__(1,2))) ;
__inline extern int stat64(char const   * __restrict  __path ,
                           struct stat64 * __restrict  __statbuf ) 
{ int tmp ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1338\n");
  fflush(_coverage_fout);
  }
  tmp = __xstat64(3, (char const   *)__path, (struct stat64 *)__statbuf);
  {
  fprintf(_coverage_fout, "1339\n");
  fflush(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) int stat64(char const   * __restrict  __path ,
                                                         struct stat64 * __restrict  __statbuf )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int __xmknodat(int __ver , int __fd ,
                                                    char const   *__path ,
                                                    __mode_t __mode ,
                                                    __dev_t *__dev )  __attribute__((__nonnull__(3,5))) ;
__inline extern  __attribute__((__nothrow__)) int mknodat(int __fd ,
                                                          char const   *__path ,
                                                          __mode_t __mode ,
                                                          __dev_t __dev )  __attribute__((__nonnull__(2))) ;
__inline extern int mknodat(int __fd , char const   *__path , __mode_t __mode ,
                            __dev_t __dev ) 
{ int tmp ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1336\n");
  fflush(_coverage_fout);
  }
  tmp = __xmknodat(1, __fd, __path, __mode, & __dev);
  {
  fprintf(_coverage_fout, "1337\n");
  fflush(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) int mknodat(int __fd ,
                                                          char const   *__path ,
                                                          __mode_t __mode ,
                                                          __dev_t __dev )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) int __xmknod(int __ver ,
                                                  char const   *__path ,
                                                  __mode_t __mode ,
                                                  __dev_t *__dev )  __attribute__((__nonnull__(2,4))) ;
__inline extern  __attribute__((__nothrow__)) int mknod(char const   *__path ,
                                                        __mode_t __mode ,
                                                        __dev_t __dev )  __attribute__((__nonnull__(1))) ;
__inline extern int mknod(char const   *__path , __mode_t __mode ,
                          __dev_t __dev ) 
{ int tmp ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1334\n");
  fflush(_coverage_fout);
  }
  tmp = __xmknod(1, __path, __mode, & __dev);
  {
  fprintf(_coverage_fout, "1335\n");
  fflush(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) int mknod(char const   *__path ,
                                                        __mode_t __mode ,
                                                        __dev_t __dev )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int __fxstatat(int __ver , int __fildes ,
                                                    char const   *__filename ,
                                                    struct stat *__stat_buf ,
                                                    int __flag )  __attribute__((__nonnull__(3,4))) ;
__inline extern  __attribute__((__nothrow__)) int fstatat(int __fd ,
                                                          char const   * __restrict  __filename ,
                                                          struct stat * __restrict  __statbuf ,
                                                          int __flag )  __attribute__((__nonnull__(2,3))) ;
__inline extern int fstatat(int __fd , char const   * __restrict  __filename ,
                            struct stat * __restrict  __statbuf , int __flag ) 
{ int tmp ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1332\n");
  fflush(_coverage_fout);
  }
  tmp = __fxstatat(3, __fd, (char const   *)__filename,
                   (struct stat *)__statbuf, __flag);
  {
  fprintf(_coverage_fout, "1333\n");
  fflush(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) int fstatat(int __fd ,
                                                          char const   * __restrict  __filename ,
                                                          struct stat * __restrict  __statbuf ,
                                                          int __flag )  __attribute__((__nonnull__(2,3))) ;
extern  __attribute__((__nothrow__)) int __fxstat(int __ver , int __fildes ,
                                                  struct stat *__stat_buf )  __attribute__((__nonnull__(3))) ;
__inline extern  __attribute__((__nothrow__)) int fstat(int __fd ,
                                                        struct stat *__statbuf )  __attribute__((__nonnull__(2))) ;
__inline extern int fstat(int __fd , struct stat *__statbuf ) 
{ int tmp ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1330\n");
  fflush(_coverage_fout);
  }
  tmp = __fxstat(3, __fd, __statbuf);
  {
  fprintf(_coverage_fout, "1331\n");
  fflush(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) int fstat(int __fd ,
                                                        struct stat *__statbuf )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) int __lxstat(int __ver ,
                                                  char const   *__filename ,
                                                  struct stat *__stat_buf )  __attribute__((__nonnull__(2,3))) ;
__inline extern  __attribute__((__nothrow__)) int lstat(char const   * __restrict  __path ,
                                                        struct stat * __restrict  __statbuf )  __attribute__((__nonnull__(1,2))) ;
__inline extern int lstat(char const   * __restrict  __path ,
                          struct stat * __restrict  __statbuf ) 
{ int tmp ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1328\n");
  fflush(_coverage_fout);
  }
  tmp = __lxstat(3, (char const   *)__path, (struct stat *)__statbuf);
  {
  fprintf(_coverage_fout, "1329\n");
  fflush(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) int lstat(char const   * __restrict  __path ,
                                                        struct stat * __restrict  __statbuf )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int __xstat(int __ver ,
                                                 char const   *__filename ,
                                                 struct stat *__stat_buf )  __attribute__((__nonnull__(2,3))) ;
__inline extern  __attribute__((__nothrow__)) int stat(char const   * __restrict  __path ,
                                                       struct stat * __restrict  __statbuf )  __attribute__((__nonnull__(1,2))) ;
__inline extern int stat(char const   * __restrict  __path ,
                         struct stat * __restrict  __statbuf ) 
{ int tmp ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1326\n");
  fflush(_coverage_fout);
  }
  tmp = __xstat(3, (char const   *)__path, (struct stat *)__statbuf);
  {
  fprintf(_coverage_fout, "1327\n");
  fflush(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) int stat(char const   * __restrict  __path ,
                                                       struct stat * __restrict  __statbuf )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int futimens(int __fd ,
                                                  struct timespec  const  *__times ) ;
extern  __attribute__((__nothrow__)) int utimensat(int __fd ,
                                                   char const   *__path ,
                                                   struct timespec  const  *__times ,
                                                   int __flags )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) int mkfifoat(int __fd ,
                                                  char const   *__path ,
                                                  __mode_t __mode )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) int mkfifo(char const   *__path ,
                                                __mode_t __mode )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int mkdirat(int __fd ,
                                                 char const   *__path ,
                                                 __mode_t __mode )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) int mkdir(char const   *__path ,
                                               __mode_t __mode )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) __mode_t getumask(void) ;
extern  __attribute__((__nothrow__)) __mode_t umask(__mode_t __mask ) ;
extern  __attribute__((__nothrow__)) int fchmodat(int __fd ,
                                                  char const   *__file ,
                                                  __mode_t __mode , int __flag )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) int fchmod(int __fd , __mode_t __mode ) ;
extern  __attribute__((__nothrow__)) int lchmod(char const   *__file ,
                                                __mode_t __mode )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int chmod(char const   *__file ,
                                               __mode_t __mode )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int futimesat(int __fd ,
                                                   char const   *__file ,
                                                   struct timeval  const  *__tvp ) ;
extern  __attribute__((__nothrow__)) int futimes(int __fd ,
                                                 struct timeval  const  *__tvp ) ;
extern  __attribute__((__nothrow__)) int lutimes(char const   *__file ,
                                                 struct timeval  const  *__tvp )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int utimes(char const   *__file ,
                                                struct timeval  const  *__tvp )  __attribute__((__nonnull__(1))) ;
struct itimerval;
enum __itimer_which;
typedef enum __itimer_which __itimer_which_t;
extern  __attribute__((__nothrow__)) int setitimer(__itimer_which_t __which ,
                                                   struct itimerval  const  * __restrict  __new ,
                                                   struct itimerval * __restrict  __old ) ;
extern  __attribute__((__nothrow__)) int getitimer(__itimer_which_t __which ,
                                                   struct itimerval *__value ) ;
extern  __attribute__((__nothrow__)) int adjtime(struct timeval  const  *__delta ,
                                                 struct timeval *__olddelta ) ;
struct timezone;
extern  __attribute__((__nothrow__)) int settimeofday(struct timeval  const  *__tv ,
                                                      struct timezone  const  *__tz )  __attribute__((__nonnull__(1))) ;
typedef struct timezone * __restrict  __timezone_ptr_t;
extern  __attribute__((__nothrow__)) int gettimeofday(struct timeval * __restrict  __tv ,
                                                      __timezone_ptr_t __tz )  __attribute__((__nonnull__(1))) ;
extern void zif_output_reset_rewrite_vars(int ht , zval *return_value ,
                                          zval **return_value_ptr ,
                                          zval *this_ptr ,
                                          int return_value_used ) ;
extern void zif_output_add_rewrite_var(int ht , zval *return_value ,
                                       zval **return_value_ptr ,
                                       zval *this_ptr , int return_value_used ) ;
extern void zif_ob_list_handlers(int ht , zval *return_value ,
                                 zval **return_value_ptr , zval *this_ptr ,
                                 int return_value_used ) ;
extern void zif_ob_implicit_flush(int ht , zval *return_value ,
                                  zval **return_value_ptr , zval *this_ptr ,
                                  int return_value_used ) ;
extern void zif_ob_get_status(int ht , zval *return_value ,
                              zval **return_value_ptr , zval *this_ptr ,
                              int return_value_used ) ;
extern void zif_ob_get_level(int ht , zval *return_value ,
                             zval **return_value_ptr , zval *this_ptr ,
                             int return_value_used ) ;
extern void zif_ob_get_length(int ht , zval *return_value ,
                              zval **return_value_ptr , zval *this_ptr ,
                              int return_value_used ) ;
extern void zif_ob_get_contents(int ht , zval *return_value ,
                                zval **return_value_ptr , zval *this_ptr ,
                                int return_value_used ) ;
extern void zif_ob_get_clean(int ht , zval *return_value ,
                             zval **return_value_ptr , zval *this_ptr ,
                             int return_value_used ) ;
extern void zif_ob_get_flush(int ht , zval *return_value ,
                             zval **return_value_ptr , zval *this_ptr ,
                             int return_value_used ) ;
extern void zif_ob_end_clean(int ht , zval *return_value ,
                             zval **return_value_ptr , zval *this_ptr ,
                             int return_value_used ) ;
extern void zif_ob_end_flush(int ht , zval *return_value ,
                             zval **return_value_ptr , zval *this_ptr ,
                             int return_value_used ) ;
extern void zif_ob_clean(int ht , zval *return_value , zval **return_value_ptr ,
                         zval *this_ptr , int return_value_used ) ;
extern void zif_ob_flush(int ht , zval *return_value , zval **return_value_ptr ,
                         zval *this_ptr , int return_value_used ) ;
extern void zif_ob_start(int ht , zval *return_value , zval **return_value_ptr ,
                         zval *this_ptr , int return_value_used ) ;
struct _php_output_handler;
extern int __attribute__((__visibility__("default")))  php_output_handler_alias_register(char const   *handler_name ,
                                                                                         size_t handler_name_len ,
                                                                                         struct _php_output_handler *(*func)(char const   *handler_name ,
                                                                                                                             size_t handler_name_len ,
                                                                                                                             size_t chunk_size ,
                                                                                                                             int flags ) ) ;
typedef struct _php_output_handler *(*php_output_handler_alias_ctor_t)(char const   *handler_name ,
                                                                       size_t handler_name_len ,
                                                                       size_t chunk_size ,
                                                                       int flags );
extern php_output_handler_alias_ctor_t __attribute__((__visibility__("default")))  *php_output_handler_alias(char const   *handler_name ,
                                                                                                             size_t handler_name_len ) ;
extern int __attribute__((__visibility__("default")))  php_output_handler_reverse_conflict_register(char const   *handler_name ,
                                                                                                    size_t handler_name_len ,
                                                                                                    int (*check_func)(char const   *handler_name ,
                                                                                                                      size_t handler_name_len ) ) ;
extern int __attribute__((__visibility__("default")))  php_output_handler_conflict_register(char const   *handler_name ,
                                                                                            size_t handler_name_len ,
                                                                                            int (*check_func)(char const   *handler_name ,
                                                                                                              size_t handler_name_len ) ) ;
extern int __attribute__((__visibility__("default")))  php_output_handler_conflict(char const   *handler_new ,
                                                                                   size_t handler_new_len ,
                                                                                   char const   *handler_set ,
                                                                                   size_t handler_set_len ) ;
typedef struct _php_output_handler php_output_handler;
extern void __attribute__((__visibility__("default")))  php_output_handler_free(php_output_handler **handler ) ;
extern void __attribute__((__visibility__("default")))  php_output_handler_dtor(php_output_handler *handler ) ;
enum _php_output_handler_hook_t;
typedef enum _php_output_handler_hook_t php_output_handler_hook_t;
extern int __attribute__((__visibility__("default")))  php_output_handler_hook(php_output_handler_hook_t type ,
                                                                               void *arg ) ;
extern int __attribute__((__visibility__("default")))  php_output_handler_started(char const   *name ,
                                                                                  size_t name_len ) ;
extern int __attribute__((__visibility__("default")))  php_output_handler_start(php_output_handler *handler ) ;
extern void __attribute__((__visibility__("default")))  php_output_handler_set_context(php_output_handler *handler ,
                                                                                       void *opaq ,
                                                                                       void (*dtor)(void * ) ) ;
struct _php_output_context;
typedef struct _php_output_context php_output_context;
extern php_output_handler __attribute__((__visibility__("default")))  *php_output_handler_create_internal(char const   *name ,
                                                                                                          size_t name_len ,
                                                                                                          int (*handler)(void **handler_context ,
                                                                                                                         php_output_context *output_context ) ,
                                                                                                          size_t chunk_size ,
                                                                                                          int flags ) ;
extern php_output_handler __attribute__((__visibility__("default")))  *php_output_handler_create_user(zval *handler ,
                                                                                                      size_t chunk_size ,
                                                                                                      int flags ) ;
extern int __attribute__((__visibility__("default")))  php_output_start_internal(char const   *name ,
                                                                                 size_t name_len ,
                                                                                 void (*output_handler)(char *output ,
                                                                                                        uint output_len ,
                                                                                                        char **handled_output ,
                                                                                                        uint *handled_output_len ,
                                                                                                        int mode ) ,
                                                                                 size_t chunk_size ,
                                                                                 int flags ) ;
extern int __attribute__((__visibility__("default")))  php_output_start_user(zval *output_handler ,
                                                                             size_t chunk_size ,
                                                                             int flags ) ;
extern int __attribute__((__visibility__("default")))  php_output_start_devnull(void) ;
extern int __attribute__((__visibility__("default")))  php_output_start_default(void) ;
extern php_output_handler __attribute__((__visibility__("default")))  *php_output_get_active_handler(void) ;
extern int __attribute__((__visibility__("default")))  php_output_get_level(void) ;
extern int __attribute__((__visibility__("default")))  php_output_get_length(zval *p ) ;
extern int __attribute__((__visibility__("default")))  php_output_get_contents(zval *p ) ;
extern void __attribute__((__visibility__("default")))  php_output_discard_all(void) ;
extern int __attribute__((__visibility__("default")))  php_output_discard(void) ;
extern void __attribute__((__visibility__("default")))  php_output_end_all(void) ;
extern int __attribute__((__visibility__("default")))  php_output_end(void) ;
extern void __attribute__((__visibility__("default")))  php_output_clean_all(void) ;
extern int __attribute__((__visibility__("default")))  php_output_clean(void) ;
extern void __attribute__((__visibility__("default")))  php_output_flush_all(void) ;
extern int __attribute__((__visibility__("default")))  php_output_flush(void) ;
extern int __attribute__((__visibility__("default")))  php_output_write(char const   *str ,
                                                                        size_t len ) ;
extern int __attribute__((__visibility__("default")))  php_output_write_unbuffered(char const   *str ,
                                                                                   size_t len ) ;
extern int __attribute__((__visibility__("default")))  php_output_get_start_lineno(void) ;
extern char const __attribute__((__visibility__("default")))  *php_output_get_start_filename(void) ;
extern void __attribute__((__visibility__("default")))  php_output_set_implicit_flush(int flush ) ;
extern int __attribute__((__visibility__("default")))  php_output_get_status(void) ;
extern void __attribute__((__visibility__("default")))  php_output_set_status(int status ) ;
extern void __attribute__((__visibility__("default")))  php_output_deactivate(void) ;
extern int __attribute__((__visibility__("default")))  php_output_activate(void) ;
extern void __attribute__((__visibility__("default")))  php_output_register_constants(void) ;
extern void __attribute__((__visibility__("default")))  php_output_shutdown(void) ;
extern void __attribute__((__visibility__("default")))  php_output_startup(void) ;
extern char const   php_output_devnull_handler_name[sizeof("null output handler")] ;
extern char const   php_output_default_handler_name[sizeof("default output handler")] ;
extern char __attribute__((__visibility__("default")))  *php_get_current_user(void) ;
extern void __attribute__((__visibility__("default")))  php_com_initialize(void) ;
extern void __attribute__((__visibility__("default")))  php_register_pre_request_shutdown(void (*func)(void * ) ,
                                                                                          void *userdata ) ;
extern int __attribute__((__visibility__("default")))  php_mergesort(void *base ,
                                                                     size_t nmemb ,
                                                                     size_t size ,
                                                                     int (*cmp)(void const   * ,
                                                                                void const   * ) ) ;
extern int __attribute__((__visibility__("default")))  php_register_internal_extensions(void) ;
extern int __attribute__((__visibility__("default")))  (*php_register_internal_extensions_func)(void) ;
extern void __attribute__((__visibility__("default")))  ( /* format attribute */  php_error_docref2)(char const   *docref ,
                                                                                                     char const   *param1 ,
                                                                                                     char const   *param2 ,
                                                                                                     int type ,
                                                                                                     char const   *format 
                                                                                                     , ...) ;
extern void __attribute__((__visibility__("default")))  ( /* format attribute */  php_error_docref1)(char const   *docref ,
                                                                                                     char const   *param1 ,
                                                                                                     int type ,
                                                                                                     char const   *format 
                                                                                                     , ...) ;
typedef __gnuc_va_list va_list;
extern void __attribute__((__visibility__("default")))  ( /* format attribute */  php_verror)(char const   *docref ,
                                                                                              char const   *params ,
                                                                                              int type ,
                                                                                              char const   *format ,
                                                                                              va_list args ) ;
__inline static void __attribute__((__deprecated__))  php_std_error_handling(void) 
{ 

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1325\n");
  fflush(_coverage_fout);
  }
  return;
}
}
__inline static void __attribute__((__deprecated__))  php_std_error_handling(void) ;
__inline static void __attribute__((__deprecated__))  php_set_error_handling(zend_error_handling_t error_handling ,
                                                                             zend_class_entry *exception_class ) 
{ 

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1323\n");
  fflush(_coverage_fout);
  }
  zend_replace_error_handling(error_handling, exception_class,
                              (zend_error_handling *)((void *)0));
  {
  fprintf(_coverage_fout, "1324\n");
  fflush(_coverage_fout);
  }
  return;
}
}
__inline static void __attribute__((__deprecated__))  php_set_error_handling(zend_error_handling_t error_handling ,
                                                                             zend_class_entry *exception_class ) ;
extern int cfgparse(void) ;
extern int ( /* format attribute */  Debug)(char *format  , ...) ;
extern void __attribute__((__visibility__("default")))  php_log_err(char *log_message ) ;
extern int __attribute__((__visibility__("default")))  php_get_module_initialized(void) ;
extern int __attribute__((__visibility__("default")))  ( /* format attribute */  php_printf)(char const   *format 
                                                                                             , ...) ;
extern int __attribute__((__visibility__("default")))  php_write(void *buf ,
                                                                 uint size ) ;
extern void phperror(char *error ) ;
extern int __attribute__((__visibility__("default")))  ( /* format attribute */  vspprintf)(char **pbuf ,
                                                                                            size_t max_len ,
                                                                                            char const   *format ,
                                                                                            va_list ap ) ;
extern int __attribute__((__visibility__("default")))  ( /* format attribute */  spprintf)(char **pbuf ,
                                                                                           size_t max_len ,
                                                                                           char const   *format 
                                                                                           , ...) ;
typedef unsigned long long u_wide_int;
extern char *ap_php_conv_p2(u_wide_int num , int nbits , char format ,
                            char *buf_end , int *len ) ;
typedef int bool_int;
typedef long long wide_int;
extern char *ap_php_conv_10(wide_int num , bool_int is_unsigned ,
                            bool_int *is_negative , char *buf_end , int *len ) ;
enum __anonenum_boolean_e_59;
typedef enum __anonenum_boolean_e_59 boolean_e;
extern char __attribute__((__visibility__("default")))  *php_conv_fp(char format ,
                                                                     double num ,
                                                                     boolean_e add_dp ,
                                                                     int precision ,
                                                                     char dec_point ,
                                                                     bool_int *is_negative ,
                                                                     char *buf ,
                                                                     int *len ) ;
extern char __attribute__((__visibility__("default")))  *php_gcvt(double value ,
                                                                  int ndigit ,
                                                                  char dec_point ,
                                                                  char exponent ,
                                                                  char *buf ) ;
extern int __attribute__((__visibility__("default")))  ( /* format attribute */  php_sprintf)(char *s ,
                                                                                              char const   *format 
                                                                                              , ...) ;
extern int __attribute__((__visibility__("default")))  ap_php_asprintf(char **buf ,
                                                                       char const   *format 
                                                                       , ...) ;
extern int __attribute__((__visibility__("default")))  ap_php_vasprintf(char **buf ,
                                                                        char const   *format ,
                                                                        va_list ap ) ;
extern int __attribute__((__visibility__("default")))  ap_php_vsnprintf(char * ,
                                                                        size_t  ,
                                                                        char const   * ,
                                                                        va_list ap ) ;
extern int __attribute__((__visibility__("default")))  ap_php_snprintf(char * ,
                                                                       size_t  ,
                                                                       char const   * 
                                                                       , ...) ;
extern int __attribute__((__visibility__("default")))  ap_php_vslprintf(char *buf ,
                                                                        size_t len ,
                                                                        char const   *format ,
                                                                        va_list ap ) ;
extern int __attribute__((__visibility__("default")))  ap_php_slprintf(char *buf ,
                                                                       size_t len ,
                                                                       char const   *format 
                                                                       , ...) ;
extern int getpw(__uid_t __uid , char *__buffer ) ;
struct passwd;
extern int fgetpwent_r(FILE * __restrict  __stream ,
                       struct passwd * __restrict  __resultbuf ,
                       char * __restrict  __buffer , size_t __buflen ,
                       struct passwd ** __restrict  __result ) ;
extern int getpwnam_r(char const   * __restrict  __name ,
                      struct passwd * __restrict  __resultbuf ,
                      char * __restrict  __buffer , size_t __buflen ,
                      struct passwd ** __restrict  __result ) ;
extern int getpwuid_r(__uid_t __uid , struct passwd * __restrict  __resultbuf ,
                      char * __restrict  __buffer , size_t __buflen ,
                      struct passwd ** __restrict  __result ) ;
extern int getpwent_r(struct passwd * __restrict  __resultbuf ,
                      char * __restrict  __buffer , size_t __buflen ,
                      struct passwd ** __restrict  __result ) ;
extern struct passwd *getpwnam(char const   *__name ) ;
extern struct passwd *getpwuid(__uid_t __uid ) ;
extern int putpwent(struct passwd  const  * __restrict  __p ,
                    FILE * __restrict  __f ) ;
extern struct passwd *fgetpwent(FILE *__stream ) ;
extern struct passwd *getpwent(void) ;
extern void endpwent(void) ;
extern void setpwent(void) ;
extern  __attribute__((__nothrow__)) void swab(void const   * __restrict  __from ,
                                               void * __restrict  __to ,
                                               ssize_t __n )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) void encrypt(char *__block , int __edflag )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) char *crypt(char const   *__key ,
                                                 char const   *__salt )  __attribute__((__nonnull__(1,2))) ;
extern int fdatasync(int __fildes ) ;
extern int lockf64(int __fd , int __cmd , __off64_t __len ) ;
extern int lockf(int __fd , int __cmd , __off_t __len ) ;
extern  __attribute__((__nothrow__)) long syscall(long __sysno  , ...) ;
typedef int __intptr_t;
typedef __intptr_t intptr_t;
extern  __attribute__((__nothrow__)) void *sbrk(intptr_t __delta ) ;
extern  __attribute__((__nothrow__)) int brk(void *__addr ) ;
extern  __attribute__((__nothrow__)) int ftruncate64(int __fd ,
                                                     __off64_t __length ) ;
extern  __attribute__((__nothrow__)) int ftruncate(int __fd , __off_t __length ) ;
extern  __attribute__((__nothrow__)) int truncate64(char const   *__file ,
                                                    __off64_t __length )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int truncate(char const   *__file ,
                                                  __off_t __length )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int getdtablesize(void) ;
extern  __attribute__((__nothrow__)) int getpagesize(void)  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) void sync(void) ;
extern long gethostid(void) ;
extern int fsync(int __fd ) ;
extern char *getpass(char const   *__prompt )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int chroot(char const   *__path )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int daemon(int __nochdir , int __noclose ) ;
extern  __attribute__((__nothrow__)) void setusershell(void) ;
extern  __attribute__((__nothrow__)) void endusershell(void) ;
extern  __attribute__((__nothrow__)) char *getusershell(void) ;
extern  __attribute__((__nothrow__)) int acct(char const   *__name ) ;
extern  __attribute__((__nothrow__)) int profil(unsigned short *__sample_buffer ,
                                                size_t __size ,
                                                size_t __offset ,
                                                unsigned int __scale )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int revoke(char const   *__file )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int vhangup(void) ;
extern  __attribute__((__nothrow__)) int setdomainname(char const   *__name ,
                                                       size_t __len )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int getdomainname(char *__name ,
                                                       size_t __len )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int sethostid(long __id ) ;
extern  __attribute__((__nothrow__)) int sethostname(char const   *__name ,
                                                     size_t __len )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int gethostname(char *__name ,
                                                     size_t __len )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int getopt(int ___argc ,
                                                char * const  *___argv ,
                                                char const   *__shortopts ) ;
extern int optopt ;
extern int opterr ;
extern int optind ;
extern char *optarg ;
extern  __attribute__((__nothrow__)) int setlogin(char const   *__name )  __attribute__((__nonnull__(1))) ;
extern int getlogin_r(char *__name , size_t __name_len )  __attribute__((__nonnull__(1))) ;
extern char *getlogin(void) ;
extern  __attribute__((__nothrow__)) int tcsetpgrp(int __fd , __pid_t __pgrp_id ) ;
extern  __attribute__((__nothrow__)) __pid_t tcgetpgrp(int __fd ) ;
extern  __attribute__((__nothrow__)) int rmdir(char const   *__path )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int unlinkat(int __fd ,
                                                  char const   *__name ,
                                                  int __flag )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) int unlink(char const   *__name )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) ssize_t readlinkat(int __fd ,
                                                        char const   * __restrict  __path ,
                                                        char * __restrict  __buf ,
                                                        size_t __len )  __attribute__((__nonnull__(2,3))) ;
extern  __attribute__((__nothrow__)) int symlinkat(char const   *__from ,
                                                   int __tofd ,
                                                   char const   *__to )  __attribute__((__nonnull__(1,3))) ;
extern  __attribute__((__nothrow__)) ssize_t readlink(char const   * __restrict  __path ,
                                                      char * __restrict  __buf ,
                                                      size_t __len )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int symlink(char const   *__from ,
                                                 char const   *__to )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int linkat(int __fromfd ,
                                                char const   *__from ,
                                                int __tofd ,
                                                char const   *__to ,
                                                int __flags )  __attribute__((__nonnull__(2,4))) ;
extern  __attribute__((__nothrow__)) int link(char const   *__from ,
                                              char const   *__to )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int ttyslot(void) ;
extern  __attribute__((__nothrow__)) int isatty(int __fd ) ;
extern  __attribute__((__nothrow__)) int ttyname_r(int __fd , char *__buf ,
                                                   size_t __buflen )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) char *ttyname(int __fd ) ;
extern  __attribute__((__nothrow__)) __pid_t vfork(void) ;
extern  __attribute__((__nothrow__)) __pid_t fork(void) ;
extern  __attribute__((__nothrow__)) int setresgid(__gid_t __rgid ,
                                                   __gid_t __egid ,
                                                   __gid_t __sgid ) ;
extern  __attribute__((__nothrow__)) int setresuid(__uid_t __ruid ,
                                                   __uid_t __euid ,
                                                   __uid_t __suid ) ;
extern  __attribute__((__nothrow__)) int getresgid(__gid_t *__rgid ,
                                                   __gid_t *__egid ,
                                                   __gid_t *__sgid ) ;
extern  __attribute__((__nothrow__)) int getresuid(__uid_t *__ruid ,
                                                   __uid_t *__euid ,
                                                   __uid_t *__suid ) ;
extern  __attribute__((__nothrow__)) int setegid(__gid_t __gid ) ;
extern  __attribute__((__nothrow__)) int setregid(__gid_t __rgid ,
                                                  __gid_t __egid ) ;
extern  __attribute__((__nothrow__)) int setgid(__gid_t __gid ) ;
extern  __attribute__((__nothrow__)) int seteuid(__uid_t __uid ) ;
extern  __attribute__((__nothrow__)) int setreuid(__uid_t __ruid ,
                                                  __uid_t __euid ) ;
extern  __attribute__((__nothrow__)) int setuid(__uid_t __uid ) ;
extern  __attribute__((__nothrow__)) int group_member(__gid_t __gid ) ;
extern  __attribute__((__nothrow__)) int getgroups(int __size , __gid_t *__list ) ;
extern  __attribute__((__nothrow__)) __gid_t getegid(void) ;
extern  __attribute__((__nothrow__)) __gid_t getgid(void) ;
extern  __attribute__((__nothrow__)) __uid_t geteuid(void) ;
extern  __attribute__((__nothrow__)) __uid_t getuid(void) ;
extern  __attribute__((__nothrow__)) __pid_t getsid(__pid_t __pid ) ;
extern  __attribute__((__nothrow__)) __pid_t setsid(void) ;
extern  __attribute__((__nothrow__)) int setpgrp(void) ;
extern  __attribute__((__nothrow__)) int setpgid(__pid_t __pid , __pid_t __pgid ) ;
extern  __attribute__((__nothrow__)) __pid_t getpgid(__pid_t __pid ) ;
extern  __attribute__((__nothrow__)) __pid_t __getpgid(__pid_t __pid ) ;
extern  __attribute__((__nothrow__)) __pid_t getpgrp(void) ;
extern  __attribute__((__nothrow__)) __pid_t getppid(void) ;
extern  __attribute__((__nothrow__)) __pid_t getpid(void) ;
extern  __attribute__((__nothrow__)) size_t confstr(int __name , char *__buf ,
                                                    size_t __len ) ;
extern  __attribute__((__nothrow__)) long sysconf(int __name ) ;
extern  __attribute__((__nothrow__)) long fpathconf(int __fd , int __name ) ;
extern  __attribute__((__nothrow__)) long pathconf(char const   *__path ,
                                                   int __name )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__noreturn__)) void _exit(int __status ) ;
extern  __attribute__((__nothrow__)) int nice(int __inc ) ;
extern  __attribute__((__nothrow__)) int execvpe(char const   *__file ,
                                                 char * const  *__argv ,
                                                 char * const  *__envp )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int execlp(char const   *__file ,
                                                char const   *__arg  , ...)  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int execvp(char const   *__file ,
                                                char * const  *__argv )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int execl(char const   *__path ,
                                               char const   *__arg  , ...)  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int execle(char const   *__path ,
                                                char const   *__arg  , ...)  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int execv(char const   *__path ,
                                               char * const  *__argv )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int fexecve(int __fd ,
                                                 char * const  *__argv ,
                                                 char * const  *__envp )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) int execve(char const   *__path ,
                                                char * const  *__argv ,
                                                char * const  *__envp )  __attribute__((__nonnull__(1,2))) ;
extern char **environ ;
extern char **__environ ;
extern  __attribute__((__nothrow__)) int dup3(int __fd , int __fd2 ,
                                              int __flags ) ;
extern  __attribute__((__nothrow__)) int dup2(int __fd , int __fd2 ) ;
extern  __attribute__((__nothrow__)) int dup(int __fd ) ;
extern  __attribute__((__nothrow__)) char *getwd(char *__buf )  __attribute__((__nonnull__(1),
__deprecated__)) ;
extern  __attribute__((__nothrow__)) char *get_current_dir_name(void) ;
extern  __attribute__((__nothrow__)) char *getcwd(char *__buf , size_t __size ) ;
extern  __attribute__((__nothrow__)) int fchdir(int __fd ) ;
extern  __attribute__((__nothrow__)) int chdir(char const   *__path )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int fchownat(int __fd ,
                                                  char const   *__file ,
                                                  __uid_t __owner ,
                                                  __gid_t __group , int __flag )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) int lchown(char const   *__file ,
                                                __uid_t __owner ,
                                                __gid_t __group )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int fchown(int __fd , __uid_t __owner ,
                                                __gid_t __group ) ;
extern  __attribute__((__nothrow__)) int chown(char const   *__file ,
                                               __uid_t __owner ,
                                               __gid_t __group )  __attribute__((__nonnull__(1))) ;
extern int pause(void) ;
typedef unsigned int __useconds_t;
extern int usleep(__useconds_t __useconds ) ;
extern  __attribute__((__nothrow__)) __useconds_t ualarm(__useconds_t __value ,
                                                         __useconds_t __interval ) ;
extern unsigned int sleep(unsigned int __seconds ) ;
extern  __attribute__((__nothrow__)) unsigned int alarm(unsigned int __seconds ) ;
extern  __attribute__((__nothrow__)) int pipe2(int *__pipedes , int __flags ) ;
extern  __attribute__((__nothrow__)) int pipe(int *__pipedes ) ;
extern ssize_t pwrite64(int __fd , void const   *__buf , size_t __n ,
                        __off64_t __offset ) ;
extern ssize_t pread64(int __fd , void *__buf , size_t __nbytes ,
                       __off64_t __offset ) ;
extern ssize_t pwrite(int __fd , void const   *__buf , size_t __n ,
                      __off_t __offset ) ;
extern ssize_t pread(int __fd , void *__buf , size_t __nbytes ,
                     __off_t __offset ) ;
extern ssize_t write(int __fd , void const   *__buf , size_t __n ) ;
extern ssize_t read(int __fd , void *__buf , size_t __nbytes ) ;
extern int close(int __fd ) ;
extern  __attribute__((__nothrow__)) __off64_t lseek64(int __fd ,
                                                       __off64_t __offset ,
                                                       int __whence ) ;
extern  __attribute__((__nothrow__)) __off_t lseek(int __fd , __off_t __offset ,
                                                   int __whence ) ;
extern  __attribute__((__nothrow__)) int faccessat(int __fd ,
                                                   char const   *__file ,
                                                   int __type , int __flag )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) int eaccess(char const   *__name ,
                                                 int __type )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int euidaccess(char const   *__name ,
                                                    int __type )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int access(char const   *__name ,
                                                int __type )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int toupper_l(int __c , __locale_t __l ) ;
extern  __attribute__((__nothrow__)) int __toupper_l(int __c , __locale_t __l ) ;
extern  __attribute__((__nothrow__)) int tolower_l(int __c , __locale_t __l ) ;
extern  __attribute__((__nothrow__)) int __tolower_l(int __c , __locale_t __l ) ;
extern  __attribute__((__nothrow__)) int isblank_l(int  , __locale_t  ) ;
extern  __attribute__((__nothrow__)) int isxdigit_l(int  , __locale_t  ) ;
extern  __attribute__((__nothrow__)) int isupper_l(int  , __locale_t  ) ;
extern  __attribute__((__nothrow__)) int isspace_l(int  , __locale_t  ) ;
extern  __attribute__((__nothrow__)) int ispunct_l(int  , __locale_t  ) ;
extern  __attribute__((__nothrow__)) int isprint_l(int  , __locale_t  ) ;
extern  __attribute__((__nothrow__)) int isgraph_l(int  , __locale_t  ) ;
extern  __attribute__((__nothrow__)) int islower_l(int  , __locale_t  ) ;
extern  __attribute__((__nothrow__)) int isdigit_l(int  , __locale_t  ) ;
extern  __attribute__((__nothrow__)) int iscntrl_l(int  , __locale_t  ) ;
extern  __attribute__((__nothrow__)) int isalpha_l(int  , __locale_t  ) ;
extern  __attribute__((__nothrow__)) int isalnum_l(int  , __locale_t  ) ;
typedef int __int32_t;
extern  __attribute__((__nothrow__)) __int32_t const   **__ctype_toupper_loc(void)  __attribute__((__const__)) ;
__inline extern  __attribute__((__nothrow__)) int toupper(int __c ) ;
__inline extern int toupper(int __c ) 
{ __int32_t const   **tmp ;
  __int32_t tmp___0 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1321\n");
  fflush(_coverage_fout);
  }
  if (__c >= -128) {
    {
    fprintf(_coverage_fout, "1319\n");
    fflush(_coverage_fout);
    }
    if (__c < 256) {
      {
      fprintf(_coverage_fout, "1316\n");
      fflush(_coverage_fout);
      }
      tmp = __ctype_toupper_loc();
      {
      fprintf(_coverage_fout, "1317\n");
      fflush(_coverage_fout);
      }
      tmp___0 = *(*tmp + __c);
    } else {
      {
      fprintf(_coverage_fout, "1318\n");
      fflush(_coverage_fout);
      }
      tmp___0 = (int const   )__c;
    }
  } else {
    {
    fprintf(_coverage_fout, "1320\n");
    fflush(_coverage_fout);
    }
    tmp___0 = (int const   )__c;
  }
  {
  fprintf(_coverage_fout, "1322\n");
  fflush(_coverage_fout);
  }
  return ((int )tmp___0);
}
}
__inline extern  __attribute__((__nothrow__)) int toupper(int __c ) ;
extern  __attribute__((__nothrow__)) __int32_t const   **__ctype_tolower_loc(void)  __attribute__((__const__)) ;
__inline extern  __attribute__((__nothrow__)) int tolower(int __c ) ;
__inline extern int tolower(int __c ) 
{ __int32_t const   **tmp ;
  __int32_t tmp___0 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1314\n");
  fflush(_coverage_fout);
  }
  if (__c >= -128) {
    {
    fprintf(_coverage_fout, "1312\n");
    fflush(_coverage_fout);
    }
    if (__c < 256) {
      {
      fprintf(_coverage_fout, "1309\n");
      fflush(_coverage_fout);
      }
      tmp = __ctype_tolower_loc();
      {
      fprintf(_coverage_fout, "1310\n");
      fflush(_coverage_fout);
      }
      tmp___0 = *(*tmp + __c);
    } else {
      {
      fprintf(_coverage_fout, "1311\n");
      fflush(_coverage_fout);
      }
      tmp___0 = (int const   )__c;
    }
  } else {
    {
    fprintf(_coverage_fout, "1313\n");
    fflush(_coverage_fout);
    }
    tmp___0 = (int const   )__c;
  }
  {
  fprintf(_coverage_fout, "1315\n");
  fflush(_coverage_fout);
  }
  return ((int )tmp___0);
}
}
__inline extern  __attribute__((__nothrow__)) int tolower(int __c ) ;
extern  __attribute__((__nothrow__)) int _tolower(int  ) ;
extern  __attribute__((__nothrow__)) int _toupper(int  ) ;
extern  __attribute__((__nothrow__)) int toascii(int __c ) ;
extern  __attribute__((__nothrow__)) int isascii(int __c ) ;
extern  __attribute__((__nothrow__)) int isctype(int __c , int __mask ) ;
extern  __attribute__((__nothrow__)) int isblank(int  ) ;
extern  __attribute__((__nothrow__)) int isxdigit(int  ) ;
extern  __attribute__((__nothrow__)) int isupper(int  ) ;
extern  __attribute__((__nothrow__)) int isspace(int  ) ;
extern  __attribute__((__nothrow__)) int ispunct(int  ) ;
extern  __attribute__((__nothrow__)) int isprint(int  ) ;
extern  __attribute__((__nothrow__)) int isgraph(int  ) ;
extern  __attribute__((__nothrow__)) int islower(int  ) ;
extern  __attribute__((__nothrow__)) int isdigit(int  ) ;
extern  __attribute__((__nothrow__)) int iscntrl(int  ) ;
extern  __attribute__((__nothrow__)) int isalpha(int  ) ;
extern  __attribute__((__nothrow__)) int isalnum(int  ) ;
extern  __attribute__((__nothrow__)) unsigned short const   **__ctype_b_loc(void)  __attribute__((__const__)) ;
extern size_t __attribute__((__visibility__("default")))  php_strlcat(char *dst ,
                                                                      char const   *src ,
                                                                      size_t siz ) ;
extern size_t __attribute__((__visibility__("default")))  php_strlcpy(char *dst ,
                                                                      char const   *src ,
                                                                      size_t siz ) ;
extern void __attribute__((__visibility__("default")))  zif_display_disabled_class(int ht ,
                                                                                   zval *return_value ,
                                                                                   zval **return_value_ptr ,
                                                                                   zval *this_ptr ,
                                                                                   int return_value_used ) ;
extern void __attribute__((__visibility__("default")))  zif_display_disabled_function(int ht ,
                                                                                      zval *return_value ,
                                                                                      zval **return_value_ptr ,
                                                                                      zval *this_ptr ,
                                                                                      int return_value_used ) ;
extern void __attribute__((__visibility__("default")))  zend_rebuild_symbol_table(void) ;
extern void __attribute__((__visibility__("default")))  zend_reset_all_cv(HashTable *symbol_table ) ;
extern int __attribute__((__visibility__("default")))  zend_delete_global_variable_ex(char const   *name ,
                                                                                      int name_len ,
                                                                                      ulong hash_value ) ;
extern void __attribute__((__visibility__("default")))  zend_delete_variable(zend_execute_data *ex ,
                                                                             HashTable *ht ,
                                                                             char const   *name ,
                                                                             int name_len ,
                                                                             ulong hash_value ) ;
extern int __attribute__((__visibility__("default")))  zend_set_hash_symbol(zval *symbol ,
                                                                            char const   *name ,
                                                                            int name_length ,
                                                                            zend_bool is_ref ,
                                                                            int num_symbol_tables 
                                                                            , ...) ;
struct _zend_fcall_info_cache;
struct _zend_fcall_info;
extern int __attribute__((__visibility__("default")))  zend_call_function(zend_fcall_info *fci ,
                                                                          zend_fcall_info_cache *fci_cache ) ;
extern int __attribute__((__visibility__("default")))  zend_fcall_info_call(zend_fcall_info *fci ,
                                                                            zend_fcall_info_cache *fcc ,
                                                                            zval **retval ,
                                                                            zval *args ) ;
extern int __attribute__((__visibility__("default")))  zend_fcall_info_argn(zend_fcall_info *fci ,
                                                                            int argc 
                                                                            , ...) ;
extern int __attribute__((__visibility__("default")))  zend_fcall_info_argv(zend_fcall_info *fci ,
                                                                            int argc ,
                                                                            va_list *argv ) ;
extern int __attribute__((__visibility__("default")))  zend_fcall_info_argp(zend_fcall_info *fci ,
                                                                            int argc ,
                                                                            zval ***argv ) ;
extern int __attribute__((__visibility__("default")))  zend_fcall_info_args(zend_fcall_info *fci ,
                                                                            zval *args ) ;
extern void __attribute__((__visibility__("default")))  zend_fcall_info_args_restore(zend_fcall_info *fci ,
                                                                                     int param_count ,
                                                                                     zval ***params ) ;
extern void __attribute__((__visibility__("default")))  zend_fcall_info_args_save(zend_fcall_info *fci ,
                                                                                  int *param_count ,
                                                                                  zval ****params ) ;
extern void __attribute__((__visibility__("default")))  zend_fcall_info_args_clear(zend_fcall_info *fci ,
                                                                                   int free_mem ) ;
extern int __attribute__((__visibility__("default")))  zend_fcall_info_init(zval *callable ,
                                                                            uint check_flags ,
                                                                            zend_fcall_info *fci ,
                                                                            zend_fcall_info_cache *fcc ,
                                                                            char **callable_name ,
                                                                            char **error ) ;
extern zend_fcall_info_cache const __attribute__((__visibility__("default")))  empty_fcall_info_cache ;
extern zend_fcall_info const __attribute__((__visibility__("default")))  empty_fcall_info ;
extern int __attribute__((__visibility__("default")))  call_user_function_ex(HashTable *function_table ,
                                                                             zval **object_pp ,
                                                                             zval *function_name ,
                                                                             zval **retval_ptr_ptr ,
                                                                             zend_uint param_count ,
                                                                             zval ***params ,
                                                                             int no_separation ,
                                                                             HashTable *symbol_table ) ;
extern int __attribute__((__visibility__("default")))  call_user_function(HashTable *function_table ,
                                                                          zval **object_pp ,
                                                                          zval *function_name ,
                                                                          zval *retval_ptr ,
                                                                          zend_uint param_count ,
                                                                          zval **params ) ;
extern int __attribute__((__visibility__("default")))  add_property_zval_ex(zval *arg ,
                                                                            char const   *key ,
                                                                            uint key_len ,
                                                                            zval *value ) ;
extern int __attribute__((__visibility__("default")))  add_property_stringl_ex(zval *arg ,
                                                                               char const   *key ,
                                                                               uint key_len ,
                                                                               char const   *str ,
                                                                               uint length ,
                                                                               int duplicate ) ;
extern int __attribute__((__visibility__("default")))  add_property_string_ex(zval *arg ,
                                                                              char const   *key ,
                                                                              uint key_len ,
                                                                              char const   *str ,
                                                                              int duplicate ) ;
extern int __attribute__((__visibility__("default")))  add_property_double_ex(zval *arg ,
                                                                              char const   *key ,
                                                                              uint key_len ,
                                                                              double d ) ;
extern int __attribute__((__visibility__("default")))  add_property_resource_ex(zval *arg ,
                                                                                char const   *key ,
                                                                                uint key_len ,
                                                                                long r ) ;
extern int __attribute__((__visibility__("default")))  add_property_bool_ex(zval *arg ,
                                                                            char const   *key ,
                                                                            uint key_len ,
                                                                            int b ) ;
extern int __attribute__((__visibility__("default")))  add_property_null_ex(zval *arg ,
                                                                            char const   *key ,
                                                                            uint key_len ) ;
extern int __attribute__((__visibility__("default")))  add_property_long_ex(zval *arg ,
                                                                            char const   *key ,
                                                                            uint key_len ,
                                                                            long l ) ;
extern int __attribute__((__visibility__("default")))  add_get_index_stringl(zval *arg ,
                                                                             ulong idx ,
                                                                             char const   *str ,
                                                                             uint length ,
                                                                             void **dest ,
                                                                             int duplicate ) ;
extern int __attribute__((__visibility__("default")))  add_get_index_string(zval *arg ,
                                                                            ulong idx ,
                                                                            char const   *str ,
                                                                            void **dest ,
                                                                            int duplicate ) ;
extern int __attribute__((__visibility__("default")))  add_get_index_double(zval *arg ,
                                                                            ulong idx ,
                                                                            double d ,
                                                                            void **dest ) ;
extern int __attribute__((__visibility__("default")))  add_get_index_long(zval *arg ,
                                                                          ulong idx ,
                                                                          long l ,
                                                                          void **dest ) ;
extern int __attribute__((__visibility__("default")))  add_get_assoc_stringl_ex(zval *arg ,
                                                                                char const   *key ,
                                                                                uint key_len ,
                                                                                char const   *str ,
                                                                                uint length ,
                                                                                void **dest ,
                                                                                int duplicate ) ;
extern int __attribute__((__visibility__("default")))  add_get_assoc_string_ex(zval *arg ,
                                                                               char const   *key ,
                                                                               uint key_len ,
                                                                               char const   *str ,
                                                                               void **dest ,
                                                                               int duplicate ) ;
extern int __attribute__((__visibility__("default")))  add_next_index_zval(zval *arg ,
                                                                           zval *value ) ;
extern int __attribute__((__visibility__("default")))  add_next_index_stringl(zval *arg ,
                                                                              char const   *str ,
                                                                              uint length ,
                                                                              int duplicate ) ;
extern int __attribute__((__visibility__("default")))  add_next_index_string(zval *arg ,
                                                                             char const   *str ,
                                                                             int duplicate ) ;
extern int __attribute__((__visibility__("default")))  add_next_index_double(zval *arg ,
                                                                             double d ) ;
extern int __attribute__((__visibility__("default")))  add_next_index_resource(zval *arg ,
                                                                               int r ) ;
extern int __attribute__((__visibility__("default")))  add_next_index_bool(zval *arg ,
                                                                           int b ) ;
extern int __attribute__((__visibility__("default")))  add_next_index_null(zval *arg ) ;
extern int __attribute__((__visibility__("default")))  add_next_index_long(zval *arg ,
                                                                           long n ) ;
extern int __attribute__((__visibility__("default")))  add_index_zval(zval *arg ,
                                                                      ulong index ,
                                                                      zval *value ) ;
extern int __attribute__((__visibility__("default")))  add_index_stringl(zval *arg ,
                                                                         ulong idx ,
                                                                         char const   *str ,
                                                                         uint length ,
                                                                         int duplicate ) ;
extern int __attribute__((__visibility__("default")))  add_index_string(zval *arg ,
                                                                        ulong idx ,
                                                                        char const   *str ,
                                                                        int duplicate ) ;
extern int __attribute__((__visibility__("default")))  add_index_double(zval *arg ,
                                                                        ulong idx ,
                                                                        double d ) ;
extern int __attribute__((__visibility__("default")))  add_index_resource(zval *arg ,
                                                                          ulong idx ,
                                                                          int r ) ;
extern int __attribute__((__visibility__("default")))  add_index_bool(zval *arg ,
                                                                      ulong idx ,
                                                                      int b ) ;
extern int __attribute__((__visibility__("default")))  add_index_null(zval *arg ,
                                                                      ulong idx ) ;
extern int __attribute__((__visibility__("default")))  add_index_long(zval *arg ,
                                                                      ulong idx ,
                                                                      long n ) ;
extern int __attribute__((__visibility__("default")))  add_assoc_zval_ex(zval *arg ,
                                                                         char const   *key ,
                                                                         uint key_len ,
                                                                         zval *value ) ;
extern int __attribute__((__visibility__("default")))  add_assoc_stringl_ex(zval *arg ,
                                                                            char const   *key ,
                                                                            uint key_len ,
                                                                            char *str ,
                                                                            uint length ,
                                                                            int duplicate ) ;
extern int __attribute__((__visibility__("default")))  add_assoc_string_ex(zval *arg ,
                                                                           char const   *key ,
                                                                           uint key_len ,
                                                                           char *str ,
                                                                           int duplicate ) ;
extern int __attribute__((__visibility__("default")))  add_assoc_double_ex(zval *arg ,
                                                                           char const   *key ,
                                                                           uint key_len ,
                                                                           double d ) ;
extern int __attribute__((__visibility__("default")))  add_assoc_resource_ex(zval *arg ,
                                                                             char const   *key ,
                                                                             uint key_len ,
                                                                             int r ) ;
extern int __attribute__((__visibility__("default")))  add_assoc_bool_ex(zval *arg ,
                                                                         char const   *key ,
                                                                         uint key_len ,
                                                                         int b ) ;
extern int __attribute__((__visibility__("default")))  add_assoc_null_ex(zval *arg ,
                                                                         char const   *key ,
                                                                         uint key_len ) ;
extern int __attribute__((__visibility__("default")))  add_assoc_long_ex(zval *arg ,
                                                                         char const   *key ,
                                                                         uint key_len ,
                                                                         long n ) ;
extern int __attribute__((__visibility__("default")))  add_assoc_function(zval *arg ,
                                                                          char const   *key ,
                                                                          void (*function_ptr)(int ht ,
                                                                                               zval *return_value ,
                                                                                               zval **return_value_ptr ,
                                                                                               zval *this_ptr ,
                                                                                               int return_value_used ) ) ;
extern void __attribute__((__visibility__("default")))  zend_merge_properties(zval *obj ,
                                                                              HashTable *properties ,
                                                                              int destroy_ht ) ;
extern int __attribute__((__visibility__("default")))  _object_and_properties_init(zval *arg ,
                                                                                   zend_class_entry *ce ,
                                                                                   HashTable *properties ) ;
extern int __attribute__((__visibility__("default")))  _object_init_ex(zval *arg ,
                                                                       zend_class_entry *ce ) ;
extern int __attribute__((__visibility__("default")))  _object_init(zval *arg ) ;
extern char __attribute__((__visibility__("default")))  *zend_get_type_by_const(int type ) ;
extern int __attribute__((__visibility__("default")))  zend_get_object_classname(zval const   *object ,
                                                                                 char const   **class_name ,
                                                                                 zend_uint *class_name_len ) ;
extern zval __attribute__((__visibility__("default")))  *zend_read_static_property(zend_class_entry *scope ,
                                                                                   char const   *name ,
                                                                                   int name_length ,
                                                                                   zend_bool silent ) ;
extern zval __attribute__((__visibility__("default")))  *zend_read_property(zend_class_entry *scope ,
                                                                            zval *object ,
                                                                            char const   *name ,
                                                                            int name_length ,
                                                                            zend_bool silent ) ;
extern int __attribute__((__visibility__("default")))  zend_update_static_property_stringl(zend_class_entry *scope ,
                                                                                           char const   *name ,
                                                                                           int name_length ,
                                                                                           char const   *value ,
                                                                                           int value_length ) ;
extern int __attribute__((__visibility__("default")))  zend_update_static_property_string(zend_class_entry *scope ,
                                                                                          char const   *name ,
                                                                                          int name_length ,
                                                                                          char const   *value ) ;
extern int __attribute__((__visibility__("default")))  zend_update_static_property_double(zend_class_entry *scope ,
                                                                                          char const   *name ,
                                                                                          int name_length ,
                                                                                          double value ) ;
extern int __attribute__((__visibility__("default")))  zend_update_static_property_long(zend_class_entry *scope ,
                                                                                        char const   *name ,
                                                                                        int name_length ,
                                                                                        long value ) ;
extern int __attribute__((__visibility__("default")))  zend_update_static_property_bool(zend_class_entry *scope ,
                                                                                        char const   *name ,
                                                                                        int name_length ,
                                                                                        long value ) ;
extern int __attribute__((__visibility__("default")))  zend_update_static_property_null(zend_class_entry *scope ,
                                                                                        char const   *name ,
                                                                                        int name_length ) ;
extern int __attribute__((__visibility__("default")))  zend_update_static_property(zend_class_entry *scope ,
                                                                                   char const   *name ,
                                                                                   int name_length ,
                                                                                   zval *value ) ;
extern void __attribute__((__visibility__("default")))  zend_update_property_stringl(zend_class_entry *scope ,
                                                                                     zval *object ,
                                                                                     char const   *name ,
                                                                                     int name_length ,
                                                                                     char const   *value ,
                                                                                     int value_length ) ;
extern void __attribute__((__visibility__("default")))  zend_update_property_string(zend_class_entry *scope ,
                                                                                    zval *object ,
                                                                                    char const   *name ,
                                                                                    int name_length ,
                                                                                    char const   *value ) ;
extern void __attribute__((__visibility__("default")))  zend_update_property_double(zend_class_entry *scope ,
                                                                                    zval *object ,
                                                                                    char const   *name ,
                                                                                    int name_length ,
                                                                                    double value ) ;
extern void __attribute__((__visibility__("default")))  zend_update_property_long(zend_class_entry *scope ,
                                                                                  zval *object ,
                                                                                  char const   *name ,
                                                                                  int name_length ,
                                                                                  long value ) ;
extern void __attribute__((__visibility__("default")))  zend_update_property_bool(zend_class_entry *scope ,
                                                                                  zval *object ,
                                                                                  char const   *name ,
                                                                                  int name_length ,
                                                                                  long value ) ;
extern void __attribute__((__visibility__("default")))  zend_update_property_null(zend_class_entry *scope ,
                                                                                  zval *object ,
                                                                                  char const   *name ,
                                                                                  int name_length ) ;
extern void __attribute__((__visibility__("default")))  zend_update_property(zend_class_entry *scope ,
                                                                             zval *object ,
                                                                             char const   *name ,
                                                                             int name_length ,
                                                                             zval *value ) ;
extern void __attribute__((__visibility__("default")))  zend_update_class_constants(zend_class_entry *class_type ) ;
extern int __attribute__((__visibility__("default")))  zend_declare_class_constant_string(zend_class_entry *ce ,
                                                                                          char const   *name ,
                                                                                          size_t name_length ,
                                                                                          char const   *value ) ;
extern int __attribute__((__visibility__("default")))  zend_declare_class_constant_stringl(zend_class_entry *ce ,
                                                                                           char const   *name ,
                                                                                           size_t name_length ,
                                                                                           char const   *value ,
                                                                                           size_t value_length ) ;
extern int __attribute__((__visibility__("default")))  zend_declare_class_constant_double(zend_class_entry *ce ,
                                                                                          char const   *name ,
                                                                                          size_t name_length ,
                                                                                          double value ) ;
extern int __attribute__((__visibility__("default")))  zend_declare_class_constant_bool(zend_class_entry *ce ,
                                                                                        char const   *name ,
                                                                                        size_t name_length ,
                                                                                        zend_bool value ) ;
extern int __attribute__((__visibility__("default")))  zend_declare_class_constant_null(zend_class_entry *ce ,
                                                                                        char const   *name ,
                                                                                        size_t name_length ) ;
extern int __attribute__((__visibility__("default")))  zend_declare_class_constant(zend_class_entry *ce ,
                                                                                   char const   *name ,
                                                                                   size_t name_length ,
                                                                                   zval *value ) ;
extern int __attribute__((__visibility__("default")))  zend_declare_property_stringl(zend_class_entry *ce ,
                                                                                     char const   *name ,
                                                                                     int name_length ,
                                                                                     char const   *value ,
                                                                                     int value_len ,
                                                                                     int access_type ) ;
extern int __attribute__((__visibility__("default")))  zend_declare_property_string(zend_class_entry *ce ,
                                                                                    char const   *name ,
                                                                                    int name_length ,
                                                                                    char const   *value ,
                                                                                    int access_type ) ;
extern int __attribute__((__visibility__("default")))  zend_declare_property_double(zend_class_entry *ce ,
                                                                                    char const   *name ,
                                                                                    int name_length ,
                                                                                    double value ,
                                                                                    int access_type ) ;
extern int __attribute__((__visibility__("default")))  zend_declare_property_long(zend_class_entry *ce ,
                                                                                  char const   *name ,
                                                                                  int name_length ,
                                                                                  long value ,
                                                                                  int access_type ) ;
extern int __attribute__((__visibility__("default")))  zend_declare_property_bool(zend_class_entry *ce ,
                                                                                  char const   *name ,
                                                                                  int name_length ,
                                                                                  long value ,
                                                                                  int access_type ) ;
extern int __attribute__((__visibility__("default")))  zend_declare_property_null(zend_class_entry *ce ,
                                                                                  char const   *name ,
                                                                                  int name_length ,
                                                                                  int access_type ) ;
extern int __attribute__((__visibility__("default")))  zend_declare_property_ex(zend_class_entry *ce ,
                                                                                char const   *name ,
                                                                                int name_length ,
                                                                                zval *property ,
                                                                                int access_type ,
                                                                                char const   *doc_comment ,
                                                                                int doc_comment_len ) ;
extern int __attribute__((__visibility__("default")))  zend_declare_property(zend_class_entry *ce ,
                                                                             char const   *name ,
                                                                             int name_length ,
                                                                             zval *property ,
                                                                             int access_type ) ;
extern int __attribute__((__visibility__("default")))  zend_get_module_started(char const   *module_name ) ;
extern char const __attribute__((__visibility__("default")))  *zend_get_module_version(char const   *module_name ) ;
extern zend_bool __attribute__((__visibility__("default")))  zend_make_callable(zval *callable ,
                                                                                char **callable_name ) ;
extern zend_bool __attribute__((__visibility__("default")))  zend_is_callable(zval *callable ,
                                                                              uint check_flags ,
                                                                              char **callable_name ) ;
extern zend_bool __attribute__((__visibility__("default")))  zend_is_callable_ex(zval *callable ,
                                                                                 zval *object_ptr ,
                                                                                 uint check_flags ,
                                                                                 char **callable_name ,
                                                                                 int *callable_name_len ,
                                                                                 zend_fcall_info_cache *fcc ,
                                                                                 char **error ) ;
extern void __attribute__((__visibility__("default")))  zend_wrong_param_count(void) ;
extern int __attribute__((__visibility__("default")))  zend_disable_class(char *class_name ,
                                                                          uint class_name_length ) ;
extern int __attribute__((__visibility__("default")))  zend_disable_function(char *function_name ,
                                                                             uint function_name_length ) ;
extern int __attribute__((__visibility__("default")))  zend_register_class_alias_ex(char const   *name ,
                                                                                    int name_len ,
                                                                                    zend_class_entry *ce ) ;
extern zend_class_entry __attribute__((__visibility__("default")))  *zend_register_internal_interface(zend_class_entry *orig_class_entry ) ;
extern zend_class_entry __attribute__((__visibility__("default")))  *zend_register_internal_class_ex(zend_class_entry *class_entry ,
                                                                                                     zend_class_entry *parent_ce ,
                                                                                                     char *parent_name ) ;
extern zend_class_entry __attribute__((__visibility__("default")))  *zend_register_internal_class(zend_class_entry *class_entry ) ;
extern void __attribute__((__visibility__("default")))  zend_check_magic_method_implementation(zend_class_entry const   *ce ,
                                                                                               zend_function const   *fptr ,
                                                                                               int error_type ) ;
extern void __attribute__((__visibility__("default")))  zend_destroy_modules(void) ;
extern void __attribute__((__visibility__("default")))  zend_collect_module_handlers(void) ;
extern int __attribute__((__visibility__("default")))  zend_startup_modules(void) ;
extern int __attribute__((__visibility__("default")))  zend_startup_module_ex(zend_module_entry *module ) ;
extern zend_module_entry __attribute__((__visibility__("default")))  *zend_register_module_ex(zend_module_entry *module ) ;
extern zend_module_entry __attribute__((__visibility__("default")))  *zend_register_internal_module(zend_module_entry *module_entry ) ;
extern int __attribute__((__visibility__("default")))  zend_startup_module(zend_module_entry *module_entry ) ;
extern void __attribute__((__visibility__("default")))  zend_unregister_functions(zend_function_entry const   *functions ,
                                                                                  int count ,
                                                                                  HashTable *function_table ) ;
extern int __attribute__((__visibility__("default")))  zend_register_functions(zend_class_entry *scope ,
                                                                               zend_function_entry const   *functions ,
                                                                               HashTable *function_table ,
                                                                               int type ) ;
extern int __attribute__((__visibility__("default")))  zend_parse_method_parameters_ex(int flags ,
                                                                                       int num_args ,
                                                                                       zval *this_ptr ,
                                                                                       char const   *type_spec 
                                                                                       , ...) ;
extern int __attribute__((__visibility__("default")))  zend_parse_method_parameters(int num_args ,
                                                                                    zval *this_ptr ,
                                                                                    char const   *type_spec 
                                                                                    , ...) ;
extern char __attribute__((__visibility__("default")))  *zend_zval_type_name(zval const   *arg ) ;
extern int __attribute__((__visibility__("default")))  zend_copy_parameters_array(int param_count ,
                                                                                  zval *argument_array ) ;
extern int __attribute__((__visibility__("default")))  _zend_get_parameters_array_ex(int param_count ,
                                                                                     zval ***argument_array ) ;
extern int __attribute__((__visibility__("default"),
__deprecated__))  zend_get_parameters_ex(int param_count  , ...) ;
extern int __attribute__((__visibility__("default")))  _zend_get_parameters_array(int ht ,
                                                                                  int param_count ,
                                                                                  zval **argument_array ) ;
extern int __attribute__((__visibility__("default")))  zend_get_parameters(int ht ,
                                                                           int param_count 
                                                                           , ...) ;
extern int zend_next_free_module(void) ;
extern int __attribute__((__visibility__("default")))  zend_do_fcall(zend_execute_data *execute_data ) ;
struct _zend_free_op;
typedef struct _zend_free_op zend_free_op;
union _temp_variable;
typedef union _temp_variable temp_variable;
union _znode_op;
extern zval __attribute__((__visibility__("default")))  **zend_get_zval_ptr_ptr(int op_type ,
                                                                                znode_op const   *node ,
                                                                                temp_variable const   *Ts ,
                                                                                zend_free_op *should_free ,
                                                                                int type ) ;
extern zval __attribute__((__visibility__("default")))  *zend_get_zval_ptr(int op_type ,
                                                                           znode_op const   *node ,
                                                                           temp_variable const   *Ts ,
                                                                           zend_free_op *should_free ,
                                                                           int type ) ;
typedef int (*user_opcode_handler_t)(zend_execute_data *execute_data );
extern user_opcode_handler_t __attribute__((__visibility__("default")))  zend_get_user_opcode_handler(zend_uchar opcode ) ;
extern int __attribute__((__visibility__("default")))  zend_set_user_opcode_handler(zend_uchar opcode ,
                                                                                    int (*handler)(zend_execute_data *execute_data ) ) ;
extern zval __attribute__((__visibility__("default")))  **zend_get_compiled_variable_value(zend_execute_data const   *execute_data_ptr ,
                                                                                           zend_uint var ) ;
extern void zend_verify_abstract_class(zend_class_entry *ce ) ;
extern zend_class_entry __attribute__((__visibility__("default")))  *zend_fetch_class_by_name(char const   *class_name ,
                                                                                              uint class_name_len ,
                                                                                              zend_literal const   *key ,
                                                                                              int fetch_type ) ;
extern zend_class_entry __attribute__((__visibility__("default")))  *zend_fetch_class(char const   *class_name ,
                                                                                      uint class_name_len ,
                                                                                      int fetch_type ) ;
extern void __attribute__((__visibility__("default")))  zend_timeout(int dummy ) ;
extern void __attribute__((__visibility__("default")))  zend_unset_timeout(void) ;
extern void __attribute__((__visibility__("default")))  zend_set_timeout(long seconds ,
                                                                         int reset_signals ) ;
extern zend_bool __attribute__((__visibility__("default")))  zend_is_executing(void) ;
extern uint __attribute__((__visibility__("default")))  zend_get_executed_lineno(void) ;
extern char const __attribute__((__visibility__("default")))  *zend_get_executed_filename(void) ;
extern char const __attribute__((__visibility__("default")))  *get_active_function_name(void) ;
extern char const __attribute__((__visibility__("default")))  *get_active_class_name(char const   **space ) ;
extern void execute_new_code(void) ;
struct _zend_ptr_stack;
__inline static void ( __attribute__((__always_inline__)) zend_ptr_stack_3_pop)(zend_ptr_stack *stack ,
                                                                                void **a ,
                                                                                void **b ,
                                                                                void **c ) ;
__inline static void ( __attribute__((__always_inline__)) zend_arg_types_stack_3_pop)(zend_ptr_stack *stack ,
                                                                                      zend_class_entry **called_scope ,
                                                                                      zval **object ,
                                                                                      zend_function **fbc ) 
{ void *a ;
  void *b ;
  void *c ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1304\n");
  fflush(_coverage_fout);
  }
  zend_ptr_stack_3_pop(stack, & a, & b, & c);
  {
  fprintf(_coverage_fout, "1305\n");
  fflush(_coverage_fout);
  }
  *called_scope = (zend_class_entry *)a;
  {
  fprintf(_coverage_fout, "1306\n");
  fflush(_coverage_fout);
  }
  *object = (zval *)b;
  {
  fprintf(_coverage_fout, "1307\n");
  fflush(_coverage_fout);
  }
  *fbc = (zend_function *)c;
  {
  fprintf(_coverage_fout, "1308\n");
  fflush(_coverage_fout);
  }
  return;
}
}
__inline static void ( __attribute__((__always_inline__)) zend_arg_types_stack_3_pop)(zend_ptr_stack *stack ,
                                                                                      zend_class_entry **called_scope ,
                                                                                      zval **object ,
                                                                                      zend_function **fbc ) ;
__inline static void ( __attribute__((__always_inline__)) zend_ptr_stack_2_pop)(zend_ptr_stack *stack ,
                                                                                void **a ,
                                                                                void **b ) ;
__inline static void ( __attribute__((__always_inline__)) zend_arg_types_stack_2_pop)(zend_ptr_stack *stack ,
                                                                                      zval **object ,
                                                                                      zend_function **fbc ) 
{ void *a ;
  void *b ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1300\n");
  fflush(_coverage_fout);
  }
  zend_ptr_stack_2_pop(stack, & a, & b);
  {
  fprintf(_coverage_fout, "1301\n");
  fflush(_coverage_fout);
  }
  *object = (zval *)a;
  {
  fprintf(_coverage_fout, "1302\n");
  fflush(_coverage_fout);
  }
  *fbc = (zend_function *)b;
  {
  fprintf(_coverage_fout, "1303\n");
  fflush(_coverage_fout);
  }
  return;
}
}
__inline static void ( __attribute__((__always_inline__)) zend_arg_types_stack_2_pop)(zend_ptr_stack *stack ,
                                                                                      zval **object ,
                                                                                      zend_function **fbc ) ;
struct _zend_function_state {
   zend_function *function ;
   void **arguments ;
};
typedef struct _zend_function_state zend_function_state;
struct _zend_execute_data {
   struct _zend_op *opline ;
   zend_function_state function_state ;
   zend_function *fbc ;
   zend_class_entry *called_scope ;
   zend_op_array *op_array ;
   zval *object ;
   union _temp_variable *Ts ;
   zval ***CVs ;
   HashTable *symbol_table ;
   struct _zend_execute_data *prev_execute_data ;
   zval *old_error_reporting ;
   zend_bool nested ;
   zval **original_return_value ;
   zend_class_entry *current_scope ;
   zend_class_entry *current_called_scope ;
   zval *current_this ;
   zval *current_object ;
};
__inline static zval **( __attribute__((__always_inline__)) zend_vm_stack_get_arg)(int requested_arg ) 
{ void **p ;
  int arg_count ;
  long tmp ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1295\n");
  fflush(_coverage_fout);
  }
  p = ((executor_globals.current_execute_data)->prev_execute_data)->function_state.arguments;
  {
  fprintf(_coverage_fout, "1296\n");
  fflush(_coverage_fout);
  }
  arg_count = (int )((unsigned long )*p);
  {
  fprintf(_coverage_fout, "1297\n");
  fflush(_coverage_fout);
  }
  tmp = __builtin_expect((long )(requested_arg > arg_count), 0L);
  {
  fprintf(_coverage_fout, "1298\n");
  fflush(_coverage_fout);
  }
  if (tmp) {
    {
    fprintf(_coverage_fout, "1293\n");
    fflush(_coverage_fout);
    }
    return ((zval **)((void *)0));
  } else {
    {
    fprintf(_coverage_fout, "1294\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "1299\n");
  fflush(_coverage_fout);
  }
  return ((((zval **)p - arg_count) + requested_arg) - 1);
}
}
__inline static zval **( __attribute__((__always_inline__)) zend_vm_stack_get_arg)(int requested_arg ) ;
__inline static void ( __attribute__((__always_inline__)) zend_vm_stack_free_int)(void *ptr ) ;
__inline static void ( __attribute__((__always_inline__)) i_zval_ptr_dtor)(zval *zval_ptr ) ;
struct _zend_vm_stack {
   void **top ;
   void **end ;
   zend_vm_stack prev ;
};
__inline static void ( __attribute__((__always_inline__)) zend_vm_stack_clear_multiple)(void) 
{ void **p ;
  int delete_count ;
  zval *q ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1288\n");
  fflush(_coverage_fout);
  }
  p = (executor_globals.argument_stack)->top - 1;
  {
  fprintf(_coverage_fout, "1289\n");
  fflush(_coverage_fout);
  }
  delete_count = (int )((unsigned long )*p);
  {
  fprintf(_coverage_fout, "1290\n");
  fflush(_coverage_fout);
  }
  while (1) {
    {
    fprintf(_coverage_fout, "1282\n");
    fflush(_coverage_fout);
    }
    delete_count --;
    {
    fprintf(_coverage_fout, "1283\n");
    fflush(_coverage_fout);
    }
    if (delete_count >= 0) {
      {
      fprintf(_coverage_fout, "1281\n");
      fflush(_coverage_fout);
      }

    } else {
      break;
    }
    {
    fprintf(_coverage_fout, "1284\n");
    fflush(_coverage_fout);
    }
    p --;
    {
    fprintf(_coverage_fout, "1285\n");
    fflush(_coverage_fout);
    }
    q = *((zval **)p);
    {
    fprintf(_coverage_fout, "1286\n");
    fflush(_coverage_fout);
    }
    *p = (void *)0;
    {
    fprintf(_coverage_fout, "1287\n");
    fflush(_coverage_fout);
    }
    i_zval_ptr_dtor(q);
  }
  {
  fprintf(_coverage_fout, "1291\n");
  fflush(_coverage_fout);
  }
  zend_vm_stack_free_int((void *)p);
  {
  fprintf(_coverage_fout, "1292\n");
  fflush(_coverage_fout);
  }
  return;
}
}
__inline static void ( __attribute__((__always_inline__)) zend_vm_stack_clear_multiple)(void) ;
__inline static void ( __attribute__((__always_inline__)) zend_vm_stack_extend)(int count ) ;
__inline static void **( __attribute__((__always_inline__)) zend_vm_stack_push_args)(int count ) 
{ zend_vm_stack p ;
  void *data ;
  zend_vm_stack r ;
  long tmp ;
  int tmp___0 ;
  void **tmp___1 ;
  long tmp___2 ;
  long tmp___3 ;
  void **tmp___4 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1275\n");
  fflush(_coverage_fout);
  }
  tmp___2 = __builtin_expect((long )((executor_globals.argument_stack)->top - (void **)((char *)executor_globals.argument_stack + (((sizeof(struct _zend_vm_stack ) + 4U) - 1U) & 4294967292U)) < count),
                             0L);
  {
  fprintf(_coverage_fout, "1276\n");
  fflush(_coverage_fout);
  }
  if (tmp___2) {
    goto _L;
  } else {
    {
    fprintf(_coverage_fout, "1273\n");
    fflush(_coverage_fout);
    }
    tmp___3 = __builtin_expect((long )((unsigned int )(executor_globals.argument_stack)->top == (unsigned int )(executor_globals.argument_stack)->end),
                               0L);
    {
    fprintf(_coverage_fout, "1274\n");
    fflush(_coverage_fout);
    }
    if (tmp___3) {
      {
      fprintf(_coverage_fout, "1264\n");
      fflush(_coverage_fout);
      }
      _L: /* CIL Label */ 
      p = executor_globals.argument_stack;
      {
      fprintf(_coverage_fout, "1265\n");
      fflush(_coverage_fout);
      }
      zend_vm_stack_extend(count + 1);
      {
      fprintf(_coverage_fout, "1266\n");
      fflush(_coverage_fout);
      }
      (executor_globals.argument_stack)->top += count;
      {
      fprintf(_coverage_fout, "1267\n");
      fflush(_coverage_fout);
      }
      *((executor_globals.argument_stack)->top) = (void *)((unsigned long )count);
      {
      fprintf(_coverage_fout, "1268\n");
      fflush(_coverage_fout);
      }
      while (1) {
        {
        fprintf(_coverage_fout, "1256\n");
        fflush(_coverage_fout);
        }
        tmp___0 = count;
        {
        fprintf(_coverage_fout, "1257\n");
        fflush(_coverage_fout);
        }
        count --;
        {
        fprintf(_coverage_fout, "1258\n");
        fflush(_coverage_fout);
        }
        if (tmp___0 > 0) {
          {
          fprintf(_coverage_fout, "1250\n");
          fflush(_coverage_fout);
          }

        } else {
          break;
        }
        {
        fprintf(_coverage_fout, "1259\n");
        fflush(_coverage_fout);
        }
        (p->top) --;
        {
        fprintf(_coverage_fout, "1260\n");
        fflush(_coverage_fout);
        }
        data = *(p->top);
        {
        fprintf(_coverage_fout, "1261\n");
        fflush(_coverage_fout);
        }
        tmp = __builtin_expect((long )((unsigned int )p->top == (unsigned int )((void **)((char *)p + (((sizeof(struct _zend_vm_stack ) + 4U) - 1U) & 4294967292U)))),
                               0L);
        {
        fprintf(_coverage_fout, "1262\n");
        fflush(_coverage_fout);
        }
        if (tmp) {
          {
          fprintf(_coverage_fout, "1251\n");
          fflush(_coverage_fout);
          }
          r = p;
          {
          fprintf(_coverage_fout, "1252\n");
          fflush(_coverage_fout);
          }
          (executor_globals.argument_stack)->prev = p->prev;
          {
          fprintf(_coverage_fout, "1253\n");
          fflush(_coverage_fout);
          }
          p = p->prev;
          {
          fprintf(_coverage_fout, "1254\n");
          fflush(_coverage_fout);
          }
          _efree((void *)r);
        } else {
          {
          fprintf(_coverage_fout, "1255\n");
          fflush(_coverage_fout);
          }

        }
        {
        fprintf(_coverage_fout, "1263\n");
        fflush(_coverage_fout);
        }
        *((void **)((char *)executor_globals.argument_stack + (((sizeof(struct _zend_vm_stack ) + 4U) - 1U) & 4294967292U)) + count) = data;
      }
      {
      fprintf(_coverage_fout, "1269\n");
      fflush(_coverage_fout);
      }
      tmp___1 = (executor_globals.argument_stack)->top;
      {
      fprintf(_coverage_fout, "1270\n");
      fflush(_coverage_fout);
      }
      ((executor_globals.argument_stack)->top) ++;
      {
      fprintf(_coverage_fout, "1271\n");
      fflush(_coverage_fout);
      }
      return (tmp___1);
    } else {
      {
      fprintf(_coverage_fout, "1272\n");
      fflush(_coverage_fout);
      }

    }
  }
  {
  fprintf(_coverage_fout, "1277\n");
  fflush(_coverage_fout);
  }
  *((executor_globals.argument_stack)->top) = (void *)((unsigned long )count);
  {
  fprintf(_coverage_fout, "1278\n");
  fflush(_coverage_fout);
  }
  tmp___4 = (executor_globals.argument_stack)->top;
  {
  fprintf(_coverage_fout, "1279\n");
  fflush(_coverage_fout);
  }
  ((executor_globals.argument_stack)->top) ++;
  {
  fprintf(_coverage_fout, "1280\n");
  fflush(_coverage_fout);
  }
  return (tmp___4);
}
}
__inline static void **( __attribute__((__always_inline__)) zend_vm_stack_push_args)(int count ) ;
__inline static void ( __attribute__((__always_inline__)) zend_vm_stack_free)(void *ptr ) 
{ zend_vm_stack p ;
  long tmp ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1247\n");
  fflush(_coverage_fout);
  }
  tmp = __builtin_expect((long )((unsigned int )((void **)((char *)executor_globals.argument_stack + (((sizeof(struct _zend_vm_stack ) + 4U) - 1U) & 4294967292U))) == (unsigned int )((void **)ptr)),
                         0L);
  {
  fprintf(_coverage_fout, "1248\n");
  fflush(_coverage_fout);
  }
  if (tmp) {
    {
    fprintf(_coverage_fout, "1240\n");
    fflush(_coverage_fout);
    }
    p = executor_globals.argument_stack;
    {
    fprintf(_coverage_fout, "1241\n");
    fflush(_coverage_fout);
    }
    executor_globals.argument_stack = p->prev;
    {
    fprintf(_coverage_fout, "1242\n");
    fflush(_coverage_fout);
    }
    _efree((void *)p);
  } else {
    {
    fprintf(_coverage_fout, "1246\n");
    fflush(_coverage_fout);
    }
    if (4U > sizeof(void *)) {
      {
      fprintf(_coverage_fout, "1243\n");
      fflush(_coverage_fout);
      }
      ptr = (void *)((char *)ptr - (((sizeof(void *) + 4U) - 1U) & 4294967292U));
      {
      fprintf(_coverage_fout, "1244\n");
      fflush(_coverage_fout);
      }
      (executor_globals.argument_stack)->top = *((void ***)ptr);
    } else {
      {
      fprintf(_coverage_fout, "1245\n");
      fflush(_coverage_fout);
      }
      (executor_globals.argument_stack)->top = (void **)ptr;
    }
  }
  {
  fprintf(_coverage_fout, "1249\n");
  fflush(_coverage_fout);
  }
  return;
}
}
__inline static void ( __attribute__((__always_inline__)) zend_vm_stack_free)(void *ptr ) ;
__inline static void ( __attribute__((__always_inline__)) zend_vm_stack_free_int)(void *ptr ) 
{ zend_vm_stack p ;
  long tmp ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1237\n");
  fflush(_coverage_fout);
  }
  tmp = __builtin_expect((long )((unsigned int )((void **)((char *)executor_globals.argument_stack + (((sizeof(struct _zend_vm_stack ) + 4U) - 1U) & 4294967292U))) == (unsigned int )((void **)ptr)),
                         0L);
  {
  fprintf(_coverage_fout, "1238\n");
  fflush(_coverage_fout);
  }
  if (tmp) {
    {
    fprintf(_coverage_fout, "1233\n");
    fflush(_coverage_fout);
    }
    p = executor_globals.argument_stack;
    {
    fprintf(_coverage_fout, "1234\n");
    fflush(_coverage_fout);
    }
    executor_globals.argument_stack = p->prev;
    {
    fprintf(_coverage_fout, "1235\n");
    fflush(_coverage_fout);
    }
    _efree((void *)p);
  } else {
    {
    fprintf(_coverage_fout, "1236\n");
    fflush(_coverage_fout);
    }
    (executor_globals.argument_stack)->top = (void **)ptr;
  }
  {
  fprintf(_coverage_fout, "1239\n");
  fflush(_coverage_fout);
  }
  return;
}
}
__inline static void *( __attribute__((__always_inline__)) zend_vm_stack_alloc)(size_t size ) 
{ void *ret ;
  int extra ;
  void **old_top ;
  long tmp ;
  long tmp___0 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1228\n");
  fflush(_coverage_fout);
  }
  size = (size + (sizeof(void *) - 1U)) / sizeof(void *);
  {
  fprintf(_coverage_fout, "1229\n");
  fflush(_coverage_fout);
  }
  if (4U > sizeof(void *)) {
    {
    fprintf(_coverage_fout, "1220\n");
    fflush(_coverage_fout);
    }
    extra = (int )((4UL - ((unsigned long )(executor_globals.argument_stack)->top & 3UL)) / (unsigned long )sizeof(void *));
    {
    fprintf(_coverage_fout, "1221\n");
    fflush(_coverage_fout);
    }
    tmp = __builtin_expect((long )((unsigned long )((size + (size_t )extra) + (((sizeof(void *) + 4U) - 1U) & 4294967292U) / sizeof(void *)) > (unsigned long )((executor_globals.argument_stack)->end - (executor_globals.argument_stack)->top)),
                           0L);
    {
    fprintf(_coverage_fout, "1222\n");
    fflush(_coverage_fout);
    }
    if (tmp) {
      {
      fprintf(_coverage_fout, "1215\n");
      fflush(_coverage_fout);
      }
      zend_vm_stack_extend((int )size);
    } else {
      {
      fprintf(_coverage_fout, "1216\n");
      fflush(_coverage_fout);
      }
      old_top = (executor_globals.argument_stack)->top;
      {
      fprintf(_coverage_fout, "1217\n");
      fflush(_coverage_fout);
      }
      (executor_globals.argument_stack)->top += extra;
      {
      fprintf(_coverage_fout, "1218\n");
      fflush(_coverage_fout);
      }
      *((executor_globals.argument_stack)->top) = (void *)old_top;
      {
      fprintf(_coverage_fout, "1219\n");
      fflush(_coverage_fout);
      }
      (executor_globals.argument_stack)->top += (((sizeof(void *) + 4U) - 1U) & 4294967292U) / sizeof(void *);
    }
  } else {
    {
    fprintf(_coverage_fout, "1227\n");
    fflush(_coverage_fout);
    }
    while (1) {
      {
      fprintf(_coverage_fout, "1225\n");
      fflush(_coverage_fout);
      }
      tmp___0 = __builtin_expect((long )((int )size > (executor_globals.argument_stack)->end - (executor_globals.argument_stack)->top),
                                 0L);
      {
      fprintf(_coverage_fout, "1226\n");
      fflush(_coverage_fout);
      }
      if (tmp___0) {
        {
        fprintf(_coverage_fout, "1223\n");
        fflush(_coverage_fout);
        }
        zend_vm_stack_extend((int )size);
      } else {
        {
        fprintf(_coverage_fout, "1224\n");
        fflush(_coverage_fout);
        }

      }
      break;
    }
  }
  {
  fprintf(_coverage_fout, "1230\n");
  fflush(_coverage_fout);
  }
  ret = (void *)(executor_globals.argument_stack)->top;
  {
  fprintf(_coverage_fout, "1231\n");
  fflush(_coverage_fout);
  }
  (executor_globals.argument_stack)->top += size;
  {
  fprintf(_coverage_fout, "1232\n");
  fflush(_coverage_fout);
  }
  return (ret);
}
}
__inline static void *( __attribute__((__always_inline__)) zend_vm_stack_alloc)(size_t size ) ;
__inline static void *( __attribute__((__always_inline__)) zend_vm_stack_pop)(void) 
{ void *el ;
  zend_vm_stack p ;
  long tmp ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1210\n");
  fflush(_coverage_fout);
  }
  ((executor_globals.argument_stack)->top) --;
  {
  fprintf(_coverage_fout, "1211\n");
  fflush(_coverage_fout);
  }
  el = *((executor_globals.argument_stack)->top);
  {
  fprintf(_coverage_fout, "1212\n");
  fflush(_coverage_fout);
  }
  tmp = __builtin_expect((long )((unsigned int )(executor_globals.argument_stack)->top == (unsigned int )((void **)((char *)executor_globals.argument_stack + (((sizeof(struct _zend_vm_stack ) + 4U) - 1U) & 4294967292U)))),
                         0L);
  {
  fprintf(_coverage_fout, "1213\n");
  fflush(_coverage_fout);
  }
  if (tmp) {
    {
    fprintf(_coverage_fout, "1206\n");
    fflush(_coverage_fout);
    }
    p = executor_globals.argument_stack;
    {
    fprintf(_coverage_fout, "1207\n");
    fflush(_coverage_fout);
    }
    executor_globals.argument_stack = p->prev;
    {
    fprintf(_coverage_fout, "1208\n");
    fflush(_coverage_fout);
    }
    _efree((void *)p);
  } else {
    {
    fprintf(_coverage_fout, "1209\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "1214\n");
  fflush(_coverage_fout);
  }
  return (el);
}
}
__inline static void *( __attribute__((__always_inline__)) zend_vm_stack_pop)(void) ;
__inline static void ( __attribute__((__always_inline__)) zend_vm_stack_push_nocheck)(void *ptr ) 
{ void **tmp ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1202\n");
  fflush(_coverage_fout);
  }
  tmp = (executor_globals.argument_stack)->top;
  {
  fprintf(_coverage_fout, "1203\n");
  fflush(_coverage_fout);
  }
  ((executor_globals.argument_stack)->top) ++;
  {
  fprintf(_coverage_fout, "1204\n");
  fflush(_coverage_fout);
  }
  *tmp = ptr;
  {
  fprintf(_coverage_fout, "1205\n");
  fflush(_coverage_fout);
  }
  return;
}
}
__inline static void ( __attribute__((__always_inline__)) zend_vm_stack_push_nocheck)(void *ptr ) ;
__inline static void ( __attribute__((__always_inline__)) zend_vm_stack_push)(void *ptr ) 
{ long tmp ;
  void **tmp___0 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1197\n");
  fflush(_coverage_fout);
  }
  while (1) {
    {
    fprintf(_coverage_fout, "1195\n");
    fflush(_coverage_fout);
    }
    tmp = __builtin_expect((long )(1 > (executor_globals.argument_stack)->end - (executor_globals.argument_stack)->top),
                           0L);
    {
    fprintf(_coverage_fout, "1196\n");
    fflush(_coverage_fout);
    }
    if (tmp) {
      {
      fprintf(_coverage_fout, "1193\n");
      fflush(_coverage_fout);
      }
      zend_vm_stack_extend(1);
    } else {
      {
      fprintf(_coverage_fout, "1194\n");
      fflush(_coverage_fout);
      }

    }
    break;
  }
  {
  fprintf(_coverage_fout, "1198\n");
  fflush(_coverage_fout);
  }
  tmp___0 = (executor_globals.argument_stack)->top;
  {
  fprintf(_coverage_fout, "1199\n");
  fflush(_coverage_fout);
  }
  ((executor_globals.argument_stack)->top) ++;
  {
  fprintf(_coverage_fout, "1200\n");
  fflush(_coverage_fout);
  }
  *tmp___0 = ptr;
  {
  fprintf(_coverage_fout, "1201\n");
  fflush(_coverage_fout);
  }
  return;
}
}
__inline static void ( __attribute__((__always_inline__)) zend_vm_stack_push)(void *ptr ) ;
__inline static void **( __attribute__((__always_inline__)) zend_vm_stack_top)(void) 
{ 

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1192\n");
  fflush(_coverage_fout);
  }
  return ((executor_globals.argument_stack)->top);
}
}
__inline static void **( __attribute__((__always_inline__)) zend_vm_stack_top)(void) ;
__inline static zend_vm_stack ( __attribute__((__always_inline__)) zend_vm_stack_new_page)(int count ) ;
__inline static void ( __attribute__((__always_inline__)) zend_vm_stack_extend)(int count ) 
{ zend_vm_stack p ;
  int tmp ;
  zend_vm_stack tmp___0 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1186\n");
  fflush(_coverage_fout);
  }
  if (count >= 16368) {
    {
    fprintf(_coverage_fout, "1184\n");
    fflush(_coverage_fout);
    }
    tmp = count;
  } else {
    {
    fprintf(_coverage_fout, "1185\n");
    fflush(_coverage_fout);
    }
    tmp = 16368;
  }
  {
  fprintf(_coverage_fout, "1187\n");
  fflush(_coverage_fout);
  }
  tmp___0 = zend_vm_stack_new_page(tmp);
  {
  fprintf(_coverage_fout, "1188\n");
  fflush(_coverage_fout);
  }
  p = tmp___0;
  {
  fprintf(_coverage_fout, "1189\n");
  fflush(_coverage_fout);
  }
  p->prev = executor_globals.argument_stack;
  {
  fprintf(_coverage_fout, "1190\n");
  fflush(_coverage_fout);
  }
  executor_globals.argument_stack = p;
  {
  fprintf(_coverage_fout, "1191\n");
  fflush(_coverage_fout);
  }
  return;
}
}
__inline static void ( __attribute__((__always_inline__)) zend_vm_stack_destroy)(void) 
{ zend_vm_stack stack ;
  zend_vm_stack p ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1181\n");
  fflush(_coverage_fout);
  }
  stack = executor_globals.argument_stack;
  {
  fprintf(_coverage_fout, "1182\n");
  fflush(_coverage_fout);
  }
  while (1) {
    {
    fprintf(_coverage_fout, "1177\n");
    fflush(_coverage_fout);
    }
    if ((unsigned int )stack != (unsigned int )((void *)0)) {
      {
      fprintf(_coverage_fout, "1176\n");
      fflush(_coverage_fout);
      }

    } else {
      break;
    }
    {
    fprintf(_coverage_fout, "1178\n");
    fflush(_coverage_fout);
    }
    p = stack->prev;
    {
    fprintf(_coverage_fout, "1179\n");
    fflush(_coverage_fout);
    }
    _efree((void *)stack);
    {
    fprintf(_coverage_fout, "1180\n");
    fflush(_coverage_fout);
    }
    stack = p;
  }
  {
  fprintf(_coverage_fout, "1183\n");
  fflush(_coverage_fout);
  }
  return;
}
}
__inline static void ( __attribute__((__always_inline__)) zend_vm_stack_destroy)(void) ;
__inline static void ( __attribute__((__always_inline__)) zend_vm_stack_init)(void) 
{ 

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1174\n");
  fflush(_coverage_fout);
  }
  executor_globals.argument_stack = zend_vm_stack_new_page(16368);
  {
  fprintf(_coverage_fout, "1175\n");
  fflush(_coverage_fout);
  }
  return;
}
}
__inline static void ( __attribute__((__always_inline__)) zend_vm_stack_init)(void) ;
__inline static zend_vm_stack ( __attribute__((__always_inline__)) zend_vm_stack_new_page)(int count ) 
{ zend_vm_stack page ;
  void __attribute__((__visibility__("default")))  *tmp ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1168\n");
  fflush(_coverage_fout);
  }
  tmp = _emalloc((((sizeof(*page) + 4U) - 1U) & 4294967292U) + sizeof(void *) * (unsigned int )count);
  {
  fprintf(_coverage_fout, "1169\n");
  fflush(_coverage_fout);
  }
  page = (struct _zend_vm_stack *)tmp;
  {
  fprintf(_coverage_fout, "1170\n");
  fflush(_coverage_fout);
  }
  page->top = (void **)((char *)page + (((sizeof(struct _zend_vm_stack ) + 4U) - 1U) & 4294967292U));
  {
  fprintf(_coverage_fout, "1171\n");
  fflush(_coverage_fout);
  }
  page->end = page->top + count;
  {
  fprintf(_coverage_fout, "1172\n");
  fflush(_coverage_fout);
  }
  page->prev = (struct _zend_vm_stack *)((void *)0);
  {
  fprintf(_coverage_fout, "1173\n");
  fflush(_coverage_fout);
  }
  return (page);
}
}
extern int __attribute__((__visibility__("default")))  zval_update_constant_ex(zval **pp ,
                                                                               void *arg ,
                                                                               zend_class_entry *scope ) ;
extern int __attribute__((__visibility__("default")))  zval_update_constant_no_inline_change(zval **pp ,
                                                                                             void *arg ) ;
extern int __attribute__((__visibility__("default")))  zval_update_constant_inline_change(zval **pp ,
                                                                                          void *arg ) ;
extern int __attribute__((__visibility__("default")))  zval_update_constant(zval **pp ,
                                                                            void *arg ) ;
extern void __attribute__((__visibility__("default")))  convert_to_boolean(zval *op ) ;
__inline static int ( __attribute__((__always_inline__)) i_zend_is_true)(zval *op ) 
{ int result ;
  int __attribute__((__visibility__("default")))  tmp ;
  int __attribute__((__visibility__("default")))  tmp___0 ;
  zval tmp___1 ;
  int tmp___2 ;
  zval *tmp___3 ;
  zval *tmp___4 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  switch ((int )op->type) {
  {
  fprintf(_coverage_fout, "1158\n");
  fflush(_coverage_fout);
  }
  case 0: 
  result = 0;
  break;
  {
  fprintf(_coverage_fout, "1159\n");
  fflush(_coverage_fout);
  }
  case 1: 
  case 3: 
  case 7: 
  if (op->value.lval) {
    {
    fprintf(_coverage_fout, "1129\n");
    fflush(_coverage_fout);
    }
    result = 1;
  } else {
    {
    fprintf(_coverage_fout, "1130\n");
    fflush(_coverage_fout);
    }
    result = 0;
  }
  break;
  {
  fprintf(_coverage_fout, "1160\n");
  fflush(_coverage_fout);
  }
  case 2: 
  if (op->value.dval) {
    {
    fprintf(_coverage_fout, "1131\n");
    fflush(_coverage_fout);
    }
    result = 1;
  } else {
    {
    fprintf(_coverage_fout, "1132\n");
    fflush(_coverage_fout);
    }
    result = 0;
  }
  break;
  {
  fprintf(_coverage_fout, "1161\n");
  fflush(_coverage_fout);
  }
  case 6: 
  if (op->value.str.len == 0) {
    {
    fprintf(_coverage_fout, "1133\n");
    fflush(_coverage_fout);
    }
    result = 0;
  } else {
    {
    fprintf(_coverage_fout, "1138\n");
    fflush(_coverage_fout);
    }
    if (op->value.str.len == 1) {
      {
      fprintf(_coverage_fout, "1136\n");
      fflush(_coverage_fout);
      }
      if ((int )*(op->value.str.val + 0) == 48) {
        {
        fprintf(_coverage_fout, "1134\n");
        fflush(_coverage_fout);
        }
        result = 0;
      } else {
        {
        fprintf(_coverage_fout, "1135\n");
        fflush(_coverage_fout);
        }
        result = 1;
      }
    } else {
      {
      fprintf(_coverage_fout, "1137\n");
      fflush(_coverage_fout);
      }
      result = 1;
    }
  }
  break;
  {
  fprintf(_coverage_fout, "1162\n");
  fflush(_coverage_fout);
  }
  case 4: 
  tmp___0 = zend_hash_num_elements((HashTable const   *)op->value.ht);
  {
  fprintf(_coverage_fout, "1163\n");
  fflush(_coverage_fout);
  }
  if (tmp___0) {
    {
    fprintf(_coverage_fout, "1139\n");
    fflush(_coverage_fout);
    }
    result = 1;
  } else {
    {
    fprintf(_coverage_fout, "1140\n");
    fflush(_coverage_fout);
    }
    result = 0;
  }
  break;
  {
  fprintf(_coverage_fout, "1164\n");
  fflush(_coverage_fout);
  }
  case 5: 
  if ((int )op->type == 5) {
    {
    fprintf(_coverage_fout, "1156\n");
    fflush(_coverage_fout);
    }
    if ((unsigned int )(op->value.obj.handlers)->get_class_entry != (unsigned int )((void *)0)) {
      {
      fprintf(_coverage_fout, "1154\n");
      fflush(_coverage_fout);
      }
      if ((op->value.obj.handlers)->cast_object) {
        {
        fprintf(_coverage_fout, "1143\n");
        fflush(_coverage_fout);
        }
        tmp___2 = (*((op->value.obj.handlers)->cast_object))(op, & tmp___1, 3);
        {
        fprintf(_coverage_fout, "1144\n");
        fflush(_coverage_fout);
        }
        if (tmp___2 == 0) {
          {
          fprintf(_coverage_fout, "1141\n");
          fflush(_coverage_fout);
          }
          result = (int )tmp___1.value.lval;
          break;
        } else {
          {
          fprintf(_coverage_fout, "1142\n");
          fflush(_coverage_fout);
          }

        }
      } else {
        {
        fprintf(_coverage_fout, "1153\n");
        fflush(_coverage_fout);
        }
        if ((op->value.obj.handlers)->get) {
          {
          fprintf(_coverage_fout, "1149\n");
          fflush(_coverage_fout);
          }
          tmp___4 = (*((op->value.obj.handlers)->get))(op);
          {
          fprintf(_coverage_fout, "1150\n");
          fflush(_coverage_fout);
          }
          tmp___3 = tmp___4;
          {
          fprintf(_coverage_fout, "1151\n");
          fflush(_coverage_fout);
          }
          if ((int )tmp___3->type != 5) {
            {
            fprintf(_coverage_fout, "1145\n");
            fflush(_coverage_fout);
            }
            convert_to_boolean(tmp___3);
            {
            fprintf(_coverage_fout, "1146\n");
            fflush(_coverage_fout);
            }
            result = (int )tmp___3->value.lval;
            {
            fprintf(_coverage_fout, "1147\n");
            fflush(_coverage_fout);
            }
            _zval_ptr_dtor(& tmp___3);
            break;
          } else {
            {
            fprintf(_coverage_fout, "1148\n");
            fflush(_coverage_fout);
            }

          }
        } else {
          {
          fprintf(_coverage_fout, "1152\n");
          fflush(_coverage_fout);
          }

        }
      }
    } else {
      {
      fprintf(_coverage_fout, "1155\n");
      fflush(_coverage_fout);
      }

    }
  } else {
    {
    fprintf(_coverage_fout, "1157\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "1165\n");
  fflush(_coverage_fout);
  }
  result = 1;
  break;
  {
  fprintf(_coverage_fout, "1166\n");
  fflush(_coverage_fout);
  }
  default: 
  result = 0;
  break;
  }
  {
  fprintf(_coverage_fout, "1167\n");
  fflush(_coverage_fout);
  }
  return (result);
}
}
__inline static int ( __attribute__((__always_inline__)) i_zend_is_true)(zval *op ) ;
__inline static void ( __attribute__((__always_inline__)) _zval_dtor)(zval *zvalue ) ;
__inline static void ( __attribute__((__always_inline__)) gc_zval_check_possible_root)(zval *z ) ;
__inline static void ( __attribute__((__always_inline__)) i_zval_ptr_dtor)(zval *zval_ptr ) 
{ zend_uint tmp ;
  zend_uint tmp___0 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1126\n");
  fflush(_coverage_fout);
  }
  tmp___0 = zval_delref_p(zval_ptr);
  {
  fprintf(_coverage_fout, "1127\n");
  fflush(_coverage_fout);
  }
  if (tmp___0) {
    {
    fprintf(_coverage_fout, "1116\n");
    fflush(_coverage_fout);
    }
    tmp = zval_refcount_p(zval_ptr);
    {
    fprintf(_coverage_fout, "1117\n");
    fflush(_coverage_fout);
    }
    if (tmp == 1U) {
      {
      fprintf(_coverage_fout, "1114\n");
      fflush(_coverage_fout);
      }
      zval_unset_isref_p(zval_ptr);
    } else {
      {
      fprintf(_coverage_fout, "1115\n");
      fflush(_coverage_fout);
      }

    }
    {
    fprintf(_coverage_fout, "1118\n");
    fflush(_coverage_fout);
    }
    gc_zval_check_possible_root(zval_ptr);
  } else {
    {
    fprintf(_coverage_fout, "1125\n");
    fflush(_coverage_fout);
    }
    if ((unsigned int )zval_ptr != (unsigned int )(& executor_globals.uninitialized_zval)) {
      {
      fprintf(_coverage_fout, "1121\n");
      fflush(_coverage_fout);
      }
      if ((gc_root_buffer *)((unsigned long )((zval_gc_info *)zval_ptr)->u.buffered & 4294967292UL)) {
        {
        fprintf(_coverage_fout, "1119\n");
        fflush(_coverage_fout);
        }
        gc_remove_zval_from_buffer(zval_ptr);
      } else {
        {
        fprintf(_coverage_fout, "1120\n");
        fflush(_coverage_fout);
        }

      }
      {
      fprintf(_coverage_fout, "1122\n");
      fflush(_coverage_fout);
      }
      _zval_dtor(zval_ptr);
      {
      fprintf(_coverage_fout, "1123\n");
      fflush(_coverage_fout);
      }
      _efree((void *)zval_ptr);
    } else {
      {
      fprintf(_coverage_fout, "1124\n");
      fflush(_coverage_fout);
      }

    }
  }
  {
  fprintf(_coverage_fout, "1128\n");
  fflush(_coverage_fout);
  }
  return;
}
}
extern int __attribute__((__visibility__("default")))  zend_verify_arg_error(int error_type ,
                                                                             zend_function const   *zf ,
                                                                             zend_uint arg_num ,
                                                                             char const   *need_msg ,
                                                                             char const   *need_kind ,
                                                                             char const   *given_msg ,
                                                                             char const   *given_kind ) ;
extern char __attribute__((__visibility__("default")))  *zend_verify_arg_class_kind(zend_arg_info const   *cur_arg_info ,
                                                                                    ulong fetch_type ,
                                                                                    char const   **class_name ,
                                                                                    zend_class_entry **pce ) ;
extern int __attribute__((__visibility__("default")))  zend_eval_stringl_ex(char *str ,
                                                                            int str_len ,
                                                                            zval *retval_ptr ,
                                                                            char *string_name ,
                                                                            int handle_exceptions ) ;
extern int __attribute__((__visibility__("default")))  zend_eval_string_ex(char *str ,
                                                                           zval *retval_ptr ,
                                                                           char *string_name ,
                                                                           int handle_exceptions ) ;
extern int __attribute__((__visibility__("default")))  zend_eval_stringl(char *str ,
                                                                         int str_len ,
                                                                         zval *retval_ptr ,
                                                                         char *string_name ) ;
extern int __attribute__((__visibility__("default")))  zend_eval_string(char *str ,
                                                                        zval *retval_ptr ,
                                                                        char *string_name ) ;
extern int __attribute__((__visibility__("default")))  zend_lookup_class_ex(char const   *name ,
                                                                            int name_length ,
                                                                            zend_literal const   *key ,
                                                                            int use_autoload ,
                                                                            zend_class_entry ***ce ) ;
extern int __attribute__((__visibility__("default")))  zend_lookup_class(char const   *name ,
                                                                         int name_length ,
                                                                         zend_class_entry ***ce ) ;
__inline static void ( __attribute__((__always_inline__)) safe_free_zval_ptr_rel)(zval *p ) 
{ 

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1112\n");
  fflush(_coverage_fout);
  }
  if ((unsigned int )p != (unsigned int )executor_globals.uninitialized_zval_ptr) {
    {
    fprintf(_coverage_fout, "1110\n");
    fflush(_coverage_fout);
    }
    while (1) {
      {
      fprintf(_coverage_fout, "1108\n");
      fflush(_coverage_fout);
      }
      if ((gc_root_buffer *)((unsigned long )((zval_gc_info *)p)->u.buffered & 4294967292UL)) {
        {
        fprintf(_coverage_fout, "1106\n");
        fflush(_coverage_fout);
        }
        gc_remove_zval_from_buffer(p);
      } else {
        {
        fprintf(_coverage_fout, "1107\n");
        fflush(_coverage_fout);
        }

      }
      {
      fprintf(_coverage_fout, "1109\n");
      fflush(_coverage_fout);
      }
      _efree((void *)p);
      break;
    }
  } else {
    {
    fprintf(_coverage_fout, "1111\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "1113\n");
  fflush(_coverage_fout);
  }
  return;
}
}
__inline static void ( __attribute__((__always_inline__)) safe_free_zval_ptr_rel)(zval *p ) ;
extern void __attribute__((__visibility__("default")))  execute_internal(zend_execute_data *execute_data_ptr ,
                                                                         int return_value_used ) ;
extern void __attribute__((__visibility__("default")))  execute(zend_op_array *op_array ) ;
extern void shutdown_destructors(void) ;
extern void shutdown_executor(void) ;
extern void init_executor(void) ;
extern void __attribute__((__visibility__("default")))  (*zend_execute_internal)(zend_execute_data *execute_data_ptr ,
                                                                                 int return_value_used ) ;
extern void __attribute__((__visibility__("default")))  (*zend_execute)(zend_op_array *op_array ) ;
extern int __attribute__((__visibility__("default")))  le_index_ptr ;
extern int __attribute__((__visibility__("default")))  zend_fetch_list_dtor_id(char *type_name ) ;
extern char const __attribute__((__visibility__("default")))  *zend_rsrc_list_get_rsrc_type(int resource ) ;
extern void __attribute__((__visibility__("default")))  *zend_fetch_resource(zval **passed_id ,
                                                                             int default_id ,
                                                                             char const   *resource_type_name ,
                                                                             int *found_resource_type ,
                                                                             int num_resource_types 
                                                                             , ...) ;
extern int __attribute__((__visibility__("default")))  zend_register_resource(zval *rsrc_result ,
                                                                              void *rsrc_pointer ,
                                                                              int rsrc_type ) ;
extern void __attribute__((__visibility__("default")))  *_zend_list_find(int id ,
                                                                         int *type ) ;
extern int __attribute__((__visibility__("default")))  _zend_list_delete(int id ) ;
extern int __attribute__((__visibility__("default")))  _zend_list_addref(int id ) ;
extern int __attribute__((__visibility__("default")))  zend_list_insert(void *ptr ,
                                                                        int type ) ;
extern void zend_destroy_rsrc_list_dtors(void) ;
extern int zend_init_rsrc_list_dtors(void) ;
extern void zend_destroy_rsrc_list(HashTable *ht ) ;
extern int zend_init_rsrc_plist(void) ;
extern int zend_init_rsrc_list(void) ;
extern void zend_clean_module_rsrc_dtors(int module_number ) ;
extern void plist_entry_destructor(void *ptr ) ;
extern void list_entry_destructor(void *ptr ) ;
struct _zend_rsrc_list_entry;
typedef struct _zend_rsrc_list_entry zend_rsrc_list_entry;
extern int __attribute__((__visibility__("default")))  zend_register_list_destructors_ex(void (*ld)(zend_rsrc_list_entry *rsrc ) ,
                                                                                         void (*pld)(zend_rsrc_list_entry *rsrc ) ,
                                                                                         char const   *type_name ,
                                                                                         int module_number ) ;
extern int __attribute__((__visibility__("default")))  zend_register_list_destructors(void (*ld)(void * ) ,
                                                                                      void (*pld)(void * ) ,
                                                                                      int module_number ) ;
extern int module_registry_unload_temp(zend_module_entry const   *module ) ;
extern int module_registry_request_startup(zend_module_entry *module ) ;
extern int module_registry_cleanup(zend_module_entry *module ) ;
extern void module_destructor(zend_module_entry *module ) ;
extern HashTable __attribute__((__visibility__("default")))  module_registry ;
extern int zend_add_literal(zend_op_array *op_array , zval const   *zv ) ;
struct _znode;
typedef struct _znode znode;
extern int zendlex(znode *zendlval ) ;
extern size_t __attribute__((__visibility__("default")))  zend_dirname(char *path ,
                                                                       size_t len ) ;
extern zend_bool __attribute__((__visibility__("default")))  zend_is_auto_global_quick(char const   *name ,
                                                                                       uint name_len ,
                                                                                       ulong hashval ) ;
extern zend_bool __attribute__((__visibility__("default")))  zend_is_auto_global(char const   *name ,
                                                                                 uint name_len ) ;
extern void __attribute__((__visibility__("default")))  zend_activate_auto_globals(void) ;
extern int __attribute__((__visibility__("default")))  zend_register_auto_global(char const   *name ,
                                                                                 uint name_len ,
                                                                                 zend_bool jit ,
                                                                                 zend_bool (*auto_global_callback)(char const   *name ,
                                                                                                                   uint name_len ) ) ;
extern int zend_get_class_fetch_type(char const   *class_name ,
                                     uint class_name_len ) ;
extern void __attribute__((__visibility__("default")))  zend_initialize_class_data(zend_class_entry *ce ,
                                                                                   zend_bool nullify_handlers ) ;
extern char __attribute__((__visibility__("default")))  *zend_make_compiled_string_description(char const   *name ) ;
extern zend_bool __attribute__((__visibility__("default")))  zend_is_compiling(void) ;
extern void zend_do_mark_last_catch(znode const   *first_catch ,
                                    znode const   *last_additional_catch ) ;
extern void zend_initialize_try_catch_element(znode const   *try_token ) ;
extern void zend_do_first_catch(znode *open_parentheses ) ;
extern zend_brk_cont_element *get_next_brk_cont_element(zend_op_array *op_array ) ;
extern int __attribute__((__visibility__("default")))  pass_two(zend_op_array *op_array ) ;
extern void print_op_array(zend_op_array *op_array , int optimizations ) ;
extern int print_class(zend_class_entry *class_entry ) ;
extern int get_next_op_number(zend_op_array *op_array ) ;
extern void init_op(zend_op *op ) ;
extern zend_op *get_next_op(zend_op_array *op_array ) ;
extern int __attribute__((__visibility__("default")))  zend_unmangle_property_name(char const   *mangled_property ,
                                                                                   int mangled_property_len ,
                                                                                   char const   **class_name ,
                                                                                   char const   **prop_name ) ;
extern void __attribute__((__visibility__("default")))  zend_mangle_property_name(char **dest ,
                                                                                  int *dest_length ,
                                                                                  char const   *src1 ,
                                                                                  int src1_length ,
                                                                                  char const   *src2 ,
                                                                                  int src2_length ,
                                                                                  int internal ) ;
extern void zend_class_add_ref(zend_class_entry **ce ) ;
extern void __attribute__((__visibility__("default")))  destroy_zend_class(zend_class_entry **pce ) ;
extern void __attribute__((__visibility__("default")))  zend_function_dtor(zend_function *function ) ;
extern void __attribute__((__visibility__("default")))  destroy_zend_function(zend_function *function ) ;
extern int __attribute__((__visibility__("default")))  zend_cleanup_function_data_full(zend_function *function ) ;
extern int __attribute__((__visibility__("default")))  zend_cleanup_function_data(zend_function *function ) ;
extern void __attribute__((__visibility__("default")))  zend_cleanup_internal_classes(void) ;
extern void __attribute__((__visibility__("default")))  zend_cleanup_internal_class_data(zend_class_entry *ce ) ;
extern int __attribute__((__visibility__("default")))  zend_cleanup_user_class_data(zend_class_entry **pce ) ;
extern int __attribute__((__visibility__("default")))  zend_cleanup_class_data(zend_class_entry **pce ) ;
extern void __attribute__((__visibility__("default")))  zend_destroy_file_handle(zend_file_handle *file_handle ) ;
extern void __attribute__((__visibility__("default")))  destroy_op_array(zend_op_array *op_array ) ;
extern void __attribute__((__visibility__("default")))  init_op_array(zend_op_array *op_array ,
                                                                      zend_uchar type ,
                                                                      int initial_ops_size ) ;
extern int __attribute__((__visibility__("default")))  open_file_for_scanning(zend_file_handle *file_handle ) ;
extern int __attribute__((__visibility__("default")))  zend_execute_scripts(int type ,
                                                                            zval **retval ,
                                                                            int file_count 
                                                                            , ...) ;
extern zend_op_array __attribute__((__visibility__("default")))  *compile_filename(int type ,
                                                                                   zval *filename ) ;
extern zend_op_array __attribute__((__visibility__("default")))  *compile_string(zval *source_string ,
                                                                                 char *filename ) ;
extern zend_op_array __attribute__((__visibility__("default")))  *compile_file(zend_file_handle *file_handle ,
                                                                               int type ) ;
extern void __attribute__((__visibility__("default")))  function_add_ref(zend_function *function ) ;
extern void zend_release_labels(void) ;
extern void zend_resolve_goto_label(zend_op_array *op_array , zend_op *opline ,
                                    int pass2 ) ;
extern void zend_do_goto(znode const   *label ) ;
extern void zend_do_label(znode *label ) ;
extern void zend_do_end_compilation(void) ;
extern void zend_do_use(znode *name , znode *new_name , int is_global ) ;
extern void zend_verify_namespace(void) ;
extern void zend_do_end_namespace(void) ;
extern void zend_do_begin_namespace(znode const   *name ,
                                    zend_bool with_brackets ) ;
extern void zend_do_build_namespace_name(znode *result , znode *prefix ,
                                         znode *name ) ;
extern void zend_do_declare_constant(znode *name , znode *value ) ;
extern void zend_do_abstract_method(znode const   *function_name ,
                                    znode *modifiers , znode const   *body ) ;
extern void zend_do_ticks(void) ;
extern void zend_do_extended_fcall_end(void) ;
extern void zend_do_extended_fcall_begin(void) ;
extern void zend_do_extended_info(void) ;
extern void zend_do_qm_false(znode *result , znode const   *false_value ,
                             znode const   *qm_token ,
                             znode const   *colon_token ) ;
extern void zend_do_qm_true(znode const   *true_value , znode *qm_token ,
                            znode *colon_token ) ;
extern void zend_do_begin_qm_op(znode const   *cond , znode *qm_token ) ;
extern void zend_do_jmp_set_else(znode *result , znode const   *false_value ,
                                 znode const   *jmp_token ,
                                 znode const   *colon_token ) ;
extern void zend_do_jmp_set(znode const   *value , znode *jmp_token ,
                            znode *colon_token ) ;
extern void zend_do_end_silence(znode const   *strudel_token ) ;
extern void zend_do_begin_silence(znode *strudel_token ) ;
extern void zend_do_exit(znode *result , znode const   *message ) ;
extern void zend_do_declare_end(znode const   *declare_token ) ;
extern void zend_do_declare_stmt(znode *var , znode *val ) ;
extern void zend_do_declare_begin(void) ;
extern void zend_do_foreach_end(znode const   *foreach_token ,
                                znode const   *as_token ) ;
extern void zend_do_foreach_cont(znode *foreach_token ,
                                 znode const   *open_brackets_token ,
                                 znode const   *as_token , znode *value ,
                                 znode *key ) ;
extern void zend_do_foreach_begin(znode *foreach_token ,
                                  znode *open_brackets_token , znode *array ,
                                  znode *as_token , int variable ) ;
extern void zend_do_instanceof(znode *result , znode const   *expr ,
                               znode const   *class_znode , int type ) ;
extern void zend_do_isset_or_isempty(int type , znode *result , znode *variable ) ;
extern void zend_do_unset(znode const   *variable ) ;
extern void zend_do_include_or_eval(int type , znode *result ,
                                    znode const   *op1 ) ;
extern void zend_do_cast(znode *result , znode const   *expr , int type ) ;
extern void zend_add_to_list(void *result , void *item ) ;
extern void zend_init_list(void *result , void *item ) ;
extern void zend_do_new_list_end(void) ;
extern void zend_do_new_list_begin(void) ;
extern void zend_do_add_list_element(znode const   *element ) ;
extern void zend_do_list_end(znode *result , znode *expr ) ;
extern void zend_do_list_init(void) ;
extern void zend_do_add_static_array_element(znode *result , znode *offset ,
                                             znode const   *expr ) ;
extern void zend_do_add_array_element(znode *result , znode const   *expr ,
                                      znode const   *offset , zend_bool is_ref ) ;
extern void zend_do_init_array(znode *result , znode const   *expr ,
                               znode const   *offset , zend_bool is_ref ) ;
extern void zend_do_shell_exec(znode *result , znode const   *cmd ) ;
extern void zend_do_fetch_constant(znode *result , znode *constant_container ,
                                   znode *constant_name , int mode ,
                                   zend_bool check_namespace ) ;
extern void zend_do_end_new_object(znode *result , znode const   *new_token ,
                                   znode const   *argument_list ) ;
extern void zend_do_begin_new_object(znode *new_token , znode *class_type ) ;
extern void zend_do_pop_object(znode *object ) ;
extern void zend_do_push_object(znode const   *object ) ;
extern void zend_do_halt_compiler_register(void) ;
extern void zend_do_fetch_property(znode *result , znode *object ,
                                   znode const   *property ) ;
extern void zend_do_declare_class_constant(znode *var_name ,
                                           znode const   *value ) ;
extern void zend_do_declare_property(znode const   *var_name ,
                                     znode const   *value ,
                                     zend_uint access_type ) ;
extern void zend_do_end_class_declaration(znode const   *class_token ,
                                          znode const   *parent_token ) ;
extern void zend_do_begin_class_declaration(znode const   *class_token ,
                                            znode *class_name ,
                                            znode const   *parent_class_name ) ;
extern void zend_do_default_before_statement(znode const   *case_list ,
                                             znode *default_token ) ;
extern void zend_do_case_after_statement(znode *result ,
                                         znode const   *case_token ) ;
extern void zend_do_case_before_statement(znode const   *case_list ,
                                          znode *case_token ,
                                          znode const   *case_expr ) ;
extern void zend_do_switch_end(znode const   *case_list ) ;
extern void zend_do_switch_cond(znode const   *cond ) ;
extern void zend_do_brk_cont(zend_uchar op , znode const   *expr ) ;
extern void zend_do_boolean_and_end(znode *result , znode const   *expr1 ,
                                    znode const   *expr2 ,
                                    znode const   *op_token ) ;
extern void zend_do_boolean_and_begin(znode *expr1 , znode *op_token ) ;
extern void zend_do_boolean_or_end(znode *result , znode const   *expr1 ,
                                   znode const   *expr2 , znode *op_token ) ;
extern void zend_do_boolean_or_begin(znode *expr1 , znode *op_token ) ;
extern void zend_do_pass_param(znode *param , zend_uchar op , int offset ) ;
extern void __attribute__((__visibility__("default")))  zend_do_delayed_early_binding(zend_op_array const   *op_array ) ;
extern void zend_do_early_binding(void) ;
extern void __attribute__((__visibility__("default")))  zend_do_inheritance(zend_class_entry *ce ,
                                                                            zend_class_entry *parent_ce ) ;
extern void init_trait_alias(znode *result , znode const   *method_name ,
                             znode const   *alias , znode const   *modifiers ) ;
extern void add_trait_alias(znode *result , znode const   *trait_alias ) ;
extern void init_trait_alias_list(znode *result , znode const   *trait_alias ) ;
extern void zend_prepare_trait_alias(znode *result , znode *method_reference ,
                                     znode *modifiers , znode *alias ) ;
extern void zend_prepare_reference(znode *result , znode *class_name ,
                                   znode *method_name ) ;
extern void zend_prepare_trait_precedence(znode *result ,
                                          znode *method_reference ,
                                          znode *trait_list ) ;
extern void __attribute__((__visibility__("default")))  zend_do_bind_traits(zend_class_entry *ce ) ;
extern void __attribute__((__visibility__("default")))  zend_do_implement_trait(zend_class_entry *ce ,
                                                                                zend_class_entry *trait ) ;
extern void zend_do_implements_trait(znode *interface_znode ) ;
extern void zend_add_trait_alias(znode *alias_znode ) ;
extern void zend_add_trait_precedence(znode *precedence_znode ) ;
extern void zend_do_implements_interface(znode *interface_znode ) ;
extern void __attribute__((__visibility__("default")))  zend_do_implement_interface(zend_class_entry *ce ,
                                                                                    zend_class_entry *iface ) ;
extern void __attribute__((__visibility__("default")))  zend_do_inherit_interfaces(zend_class_entry *ce ,
                                                                                   zend_class_entry const   *iface ) ;
extern zend_class_entry __attribute__((__visibility__("default")))  *do_bind_inherited_class(zend_op_array const   *op_array ,
                                                                                             zend_op const   *opline ,
                                                                                             HashTable *class_table ,
                                                                                             zend_class_entry *parent_ce ,
                                                                                             zend_bool compile_time ) ;
extern zend_class_entry __attribute__((__visibility__("default")))  *do_bind_class(zend_op_array const   *op_array ,
                                                                                   zend_op const   *opline ,
                                                                                   HashTable *class_table ,
                                                                                   zend_bool compile_time ) ;
extern int __attribute__((__visibility__("default")))  do_bind_function(zend_op_array const   *op_array ,
                                                                        zend_op *opline ,
                                                                        HashTable *function_table ,
                                                                        zend_bool compile_time ) ;
extern void zend_do_throw(znode const   *expr ) ;
extern void zend_do_end_catch(znode const   *try_token ) ;
extern void zend_do_begin_catch(znode *try_token , znode *catch_class ,
                                znode *catch_var , znode *first_catch ) ;
extern void zend_do_try(znode *try_token ) ;
extern void zend_do_fetch_lexical_variable(znode *varname , zend_bool is_ref ) ;
extern void zend_do_begin_lambda_function_declaration(znode *result ,
                                                      znode *function_token ,
                                                      int return_reference ,
                                                      int is_static ) ;
extern void zend_do_handle_exception(void) ;
extern void zend_do_return(znode *expr , int do_end_vparse ) ;
extern void zend_do_end_function_call(znode *function_name , znode *result ,
                                      znode const   *argument_list ,
                                      int is_method , int is_dynamic_fcall ) ;
extern int zend_do_begin_class_member_function_call(znode *class_name ,
                                                    znode *method_name ) ;
extern void zend_do_build_full_name(znode *result , znode *prefix ,
                                    znode *name , int is_class_member ) ;
extern void zend_do_fetch_class(znode *result , znode *class_name ) ;
extern void zend_do_begin_dynamic_function_call(znode *function_name ,
                                                int prefix_len ) ;
extern void zend_do_clone(znode *result , znode const   *expr ) ;
extern void zend_do_begin_method_call(znode *left_bracket ) ;
extern int zend_do_begin_function_call(znode *function_name ,
                                       zend_bool check_namespace ) ;
extern void zend_do_receive_arg(zend_uchar op , znode *varname ,
                                znode const   *offset ,
                                znode const   *initialization ,
                                znode *class_type , zend_bool pass_by_reference ) ;
extern void zend_do_end_function_declaration(znode const   *function_token ) ;
extern void zend_do_begin_function_declaration(znode *function_token ,
                                               znode *function_name ,
                                               int is_method ,
                                               int return_reference ,
                                               znode *fn_flags_znode ) ;
extern int zend_do_verify_access_types(znode const   *current_access_type ,
                                       znode const   *new_modifier ) ;
extern void zend_do_add_variable(znode *result , znode const   *op1 ,
                                 znode const   *op2 ) ;
extern void zend_do_add_string(znode *result , znode const   *op1 , znode *op2 ) ;
extern void zend_do_free(znode *op1 ) ;
extern void zend_check_writable_variable(znode const   *variable ) ;
extern void zend_do_end_variable_parse(znode *variable , int type ,
                                       int arg_offset ) ;
extern void zend_do_begin_variable_parse(void) ;
extern void zend_do_post_incdec(znode *result , znode const   *op1 ,
                                zend_uchar op ) ;
extern void zend_do_pre_incdec(znode *result , znode const   *op1 ,
                               zend_uchar op ) ;
extern void zend_do_for_end(znode const   *second_semicolon_token ) ;
extern void zend_do_for_before_statement(znode const   *cond_start ,
                                         znode const   *second_semicolon_token ) ;
extern void zend_do_for_cond(znode const   *expr ,
                             znode *second_semicolon_token ) ;
extern void zend_do_if_end(void) ;
extern void zend_do_if_after_statement(znode const   *closing_bracket_token ,
                                       unsigned char initialize ) ;
extern void zend_do_if_cond(znode const   *cond , znode *closing_bracket_token ) ;
extern void zend_do_do_while_end(znode const   *do_token ,
                                 znode const   *expr_open_bracket ,
                                 znode const   *expr ) ;
extern void zend_do_do_while_begin(void) ;
extern void zend_do_while_end(znode const   *while_token ,
                              znode const   *close_bracket_token ) ;
extern void zend_do_while_cond(znode const   *expr , znode *close_bracket_token ) ;
typedef int (*binary_op_type)(zval * , zval * , zval * );
extern binary_op_type __attribute__((__visibility__("default")))  get_binary_op(int opcode ) ;
typedef int (*unary_op_type)(zval * , zval * );
extern unary_op_type __attribute__((__visibility__("default")))  get_unary_op(int opcode ) ;
extern void zend_do_echo(znode const   *arg ) ;
extern void zend_do_print(znode *result , znode const   *arg ) ;
extern void zend_do_fetch_static_member(znode *result , znode *class_znode ) ;
extern void fetch_string_offset(znode *result , znode const   *parent ,
                                znode const   *offset ) ;
extern void fetch_array_dim(znode *result , znode const   *parent ,
                            znode const   *dim ) ;
extern void fetch_array_begin(znode *result , znode *varname , znode *first_dim ) ;
extern void zend_do_fetch_global_variable(znode *varname ,
                                          znode const   *static_assignment ,
                                          int fetch_type ) ;
extern void zend_do_fetch_static_variable(znode *varname ,
                                          znode const   *static_assignment ,
                                          int fetch_type ) ;
extern void zend_do_indirect_references(znode *result ,
                                        znode const   *num_references ,
                                        znode *variable ) ;
extern void fetch_simple_variable_ex(znode *result , znode *varname , int bp ,
                                     zend_uchar op ) ;
extern void fetch_simple_variable(znode *result , znode *varname , int bp ) ;
extern void zend_do_assign_ref(znode *result , znode const   *lvar ,
                               znode const   *rvar ) ;
extern void zend_do_assign(znode *result , znode *variable , znode *value ) ;
extern void zend_do_binary_assign_op(zend_uchar op , znode *result ,
                                     znode const   *op1 , znode const   *op2 ) ;
extern void zend_do_unary_op(zend_uchar op , znode *result , znode const   *op1 ) ;
extern void zend_do_binary_op(zend_uchar op , znode *result ,
                              znode const   *op1 , znode const   *op2 ) ;
extern char const __attribute__((__visibility__("default")))  *zend_get_compiled_variable_name(zend_op_array const   *op_array ,
                                                                                               zend_uint var ,
                                                                                               int *name_len ) ;
extern void zend_resolve_class_name(znode *class_name , ulong fetch_type ,
                                    int check_ns_name ) ;
extern void zend_resolve_non_class_name(znode *element_name ,
                                        zend_bool check_namespace ) ;
extern size_t __attribute__((__visibility__("default")))  zend_get_scanned_file_offset(void) ;
extern int __attribute__((__visibility__("default")))  zend_get_compiled_lineno(void) ;
extern char __attribute__((__visibility__("default")))  *zend_get_compiled_filename(void) ;
extern void __attribute__((__visibility__("default")))  zend_restore_compiled_filename(char *original_compiled_filename ) ;
extern char __attribute__((__visibility__("default")))  *zend_set_compiled_filename(char const   *new_compiled_filename ) ;
extern void shutdown_scanner(void) ;
extern void startup_scanner(void) ;
extern int __attribute__((__visibility__("default")))  lex_scan(zval *zendlval ) ;
extern zend_op_array __attribute__((__visibility__("default")))  *(*zend_compile_string)(zval *source_string ,
                                                                                         char *filename ) ;
extern zend_op_array __attribute__((__visibility__("default")))  *(*zend_compile_file)(zend_file_handle *file_handle ,
                                                                                       int type ) ;
extern void zend_init_compiler_context(void) ;
extern void zend_init_compiler_data_structures(void) ;
extern void shutdown_compiler(void) ;
extern void init_compiler(void) ;
extern int __attribute__((__visibility__("default")))  zend_multibyte_set_script_encoding_by_string(char const   *new_value ,
                                                                                                    size_t new_value_length ) ;
struct _zend_encoding;
typedef struct _zend_encoding zend_encoding;
extern int __attribute__((__visibility__("default")))  zend_multibyte_set_internal_encoding(zend_encoding const   *encoding ) ;
extern int __attribute__((__visibility__("default")))  zend_multibyte_set_script_encoding(zend_encoding const   **encoding_list ,
                                                                                          size_t encoding_list_size ) ;
extern zend_encoding const __attribute__((__visibility__("default")))  *zend_multibyte_get_script_encoding(void) ;
extern zend_encoding const __attribute__((__visibility__("default")))  *zend_multibyte_get_internal_encoding(void) ;
extern int __attribute__((__visibility__("default")))  zend_multibyte_parse_encoding_list(char const   *encoding_list ,
                                                                                          size_t encoding_list_len ,
                                                                                          zend_encoding const   ***return_list ,
                                                                                          size_t *return_size ,
                                                                                          int persistent ) ;
extern size_t __attribute__((__visibility__("default")))  zend_multibyte_encoding_converter(unsigned char **to ,
                                                                                            size_t *to_length ,
                                                                                            unsigned char const   *from ,
                                                                                            size_t from_length ,
                                                                                            zend_encoding const   *encoding_to ,
                                                                                            zend_encoding const   *encoding_from ) ;
extern zend_encoding const __attribute__((__visibility__("default")))  *zend_multibyte_encoding_detector(unsigned char const   *string ,
                                                                                                         size_t length ,
                                                                                                         zend_encoding const   **list ,
                                                                                                         size_t list_size ) ;
extern int __attribute__((__visibility__("default")))  zend_multibyte_check_lexer_compatibility(zend_encoding const   *encoding ) ;
extern char const __attribute__((__visibility__("default")))  *zend_multibyte_get_encoding_name(zend_encoding const   *encoding ) ;
extern zend_encoding const __attribute__((__visibility__("default")))  *zend_multibyte_fetch_encoding(char const   *name ) ;
struct _zend_multibyte_functions;
typedef struct _zend_multibyte_functions zend_multibyte_functions;
extern zend_multibyte_functions const __attribute__((__visibility__("default")))  *zend_multibyte_get_functions(void) ;
extern int __attribute__((__visibility__("default")))  zend_multibyte_set_functions(zend_multibyte_functions const   *functions ) ;
extern zend_encoding const __attribute__((__visibility__("default")))  *zend_multibyte_encoding_utf8 ;
extern zend_encoding const __attribute__((__visibility__("default")))  *zend_multibyte_encoding_utf16le ;
extern zend_encoding const __attribute__((__visibility__("default")))  *zend_multibyte_encoding_utf16be ;
extern zend_encoding const __attribute__((__visibility__("default")))  *zend_multibyte_encoding_utf32le ;
extern zend_encoding const __attribute__((__visibility__("default")))  *zend_multibyte_encoding_utf32be ;
extern fpu_control_t __fpu_control ;
extern void __attribute__((__visibility__("default")))  zend_ensure_fpu_mode(void) ;
extern void __attribute__((__visibility__("default")))  zend_shutdown_fpu(void) ;
extern void __attribute__((__visibility__("default")))  zend_init_fpu(void) ;
extern zval __attribute__((__visibility__("default")))  *zend_object_create_proxy(zval *object ,
                                                                                  zval *member ) ;
struct _zend_objects_store;
extern void __attribute__((__visibility__("default")))  zend_objects_store_free_object_storage(zend_objects_store *objects ) ;
extern void __attribute__((__visibility__("default")))  zend_object_store_ctor_failed(zval *zobject ) ;
extern void __attribute__((__visibility__("default")))  zend_object_store_set_object(zval *zobject ,
                                                                                     void *object ) ;
extern void __attribute__((__visibility__("default")))  *zend_object_store_get_object_by_handle(zend_object_handle handle ) ;
extern zend_object_value __attribute__((__visibility__("default")))  zend_objects_store_clone_obj(zval *object ) ;
extern zend_uint __attribute__((__visibility__("default")))  zend_objects_store_get_refcount(zval *object ) ;
extern void __attribute__((__visibility__("default")))  zend_objects_store_del_ref_by_handle_ex(zend_object_handle handle ,
                                                                                                zend_object_handlers const   *handlers ) ;
__inline static void ( __attribute__((__always_inline__)) zend_objects_store_del_ref_by_handle)(zend_object_handle handle ) 
{ 

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1104\n");
  fflush(_coverage_fout);
  }
  zend_objects_store_del_ref_by_handle_ex(handle,
                                          (zend_object_handlers const   *)((void *)0));
  {
  fprintf(_coverage_fout, "1105\n");
  fflush(_coverage_fout);
  }
  return;
}
}
__inline static void ( __attribute__((__always_inline__)) zend_objects_store_del_ref_by_handle)(zend_object_handle handle ) ;
extern void __attribute__((__visibility__("default")))  zend_objects_store_add_ref_by_handle(zend_object_handle handle ) ;
extern void __attribute__((__visibility__("default")))  zend_objects_store_del_ref(zval *object ) ;
extern void __attribute__((__visibility__("default")))  zend_objects_store_add_ref(zval *object ) ;
extern void __attribute__((__visibility__("default")))  zend_objects_store_destroy(zend_objects_store *objects ) ;
extern void __attribute__((__visibility__("default")))  zend_objects_store_mark_destructed(zend_objects_store *objects ) ;
extern void __attribute__((__visibility__("default")))  zend_objects_store_call_destructors(zend_objects_store *objects ) ;
extern void __attribute__((__visibility__("default")))  zend_objects_store_init(zend_objects_store *objects ,
                                                                                zend_uint init_size ) ;
extern void __attribute__((__visibility__("default")))  zend_objects_free_object_storage(zend_object *object ) ;
extern zend_object_value __attribute__((__visibility__("default")))  zend_objects_clone_obj(zval *object ) ;
extern zend_object_value __attribute__((__visibility__("default")))  zend_objects_new(zend_object **object ,
                                                                                      zend_class_entry *class_type ) ;
__inline static void *( __attribute__((__always_inline__)) zend_ptr_stack_pop)(zend_ptr_stack *stack ) 
{ 

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1101\n");
  fflush(_coverage_fout);
  }
  (stack->top) --;
  {
  fprintf(_coverage_fout, "1102\n");
  fflush(_coverage_fout);
  }
  (stack->top_element) --;
  {
  fprintf(_coverage_fout, "1103\n");
  fflush(_coverage_fout);
  }
  return (*(stack->top_element));
}
}
__inline static void *( __attribute__((__always_inline__)) zend_ptr_stack_pop)(zend_ptr_stack *stack ) ;
__inline static void *__zend_realloc(void *p , size_t len ) ;
__inline static void ( __attribute__((__always_inline__)) zend_ptr_stack_push)(zend_ptr_stack *stack ,
                                                                               void *ptr ) 
{ void *tmp ;
  void __attribute__((__visibility__("default")))  *tmp___0 ;
  void *tmp___1 ;
  void **tmp___2 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1095\n");
  fflush(_coverage_fout);
  }
  if (stack->top + 1 > stack->max) {
    {
    fprintf(_coverage_fout, "1090\n");
    fflush(_coverage_fout);
    }
    while (1) {
      {
      fprintf(_coverage_fout, "1084\n");
      fflush(_coverage_fout);
      }
      stack->max += 64;
      {
      fprintf(_coverage_fout, "1085\n");
      fflush(_coverage_fout);
      }
      if (stack->top + 1 > stack->max) {
        {
        fprintf(_coverage_fout, "1083\n");
        fflush(_coverage_fout);
        }

      } else {
        break;
      }
    }
    {
    fprintf(_coverage_fout, "1091\n");
    fflush(_coverage_fout);
    }
    if (stack->persistent) {
      {
      fprintf(_coverage_fout, "1086\n");
      fflush(_coverage_fout);
      }
      tmp = __zend_realloc((void *)stack->elements,
                           sizeof(void *) * (unsigned int )stack->max);
      {
      fprintf(_coverage_fout, "1087\n");
      fflush(_coverage_fout);
      }
      tmp___1 = tmp;
    } else {
      {
      fprintf(_coverage_fout, "1088\n");
      fflush(_coverage_fout);
      }
      tmp___0 = _erealloc((void *)stack->elements,
                          sizeof(void *) * (unsigned int )stack->max, 0);
      {
      fprintf(_coverage_fout, "1089\n");
      fflush(_coverage_fout);
      }
      tmp___1 = (void *)tmp___0;
    }
    {
    fprintf(_coverage_fout, "1092\n");
    fflush(_coverage_fout);
    }
    stack->elements = (void **)tmp___1;
    {
    fprintf(_coverage_fout, "1093\n");
    fflush(_coverage_fout);
    }
    stack->top_element = stack->elements + stack->top;
  } else {
    {
    fprintf(_coverage_fout, "1094\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "1096\n");
  fflush(_coverage_fout);
  }
  (stack->top) ++;
  {
  fprintf(_coverage_fout, "1097\n");
  fflush(_coverage_fout);
  }
  tmp___2 = stack->top_element;
  {
  fprintf(_coverage_fout, "1098\n");
  fflush(_coverage_fout);
  }
  (stack->top_element) ++;
  {
  fprintf(_coverage_fout, "1099\n");
  fflush(_coverage_fout);
  }
  *tmp___2 = ptr;
  {
  fprintf(_coverage_fout, "1100\n");
  fflush(_coverage_fout);
  }
  return;
}
}
__inline static void ( __attribute__((__always_inline__)) zend_ptr_stack_push)(zend_ptr_stack *stack ,
                                                                               void *ptr ) ;
__inline static void ( __attribute__((__always_inline__)) zend_ptr_stack_2_pop)(zend_ptr_stack *stack ,
                                                                                void **a ,
                                                                                void **b ) 
{ 

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1077\n");
  fflush(_coverage_fout);
  }
  (stack->top_element) --;
  {
  fprintf(_coverage_fout, "1078\n");
  fflush(_coverage_fout);
  }
  *a = *(stack->top_element);
  {
  fprintf(_coverage_fout, "1079\n");
  fflush(_coverage_fout);
  }
  (stack->top_element) --;
  {
  fprintf(_coverage_fout, "1080\n");
  fflush(_coverage_fout);
  }
  *b = *(stack->top_element);
  {
  fprintf(_coverage_fout, "1081\n");
  fflush(_coverage_fout);
  }
  stack->top -= 2;
  {
  fprintf(_coverage_fout, "1082\n");
  fflush(_coverage_fout);
  }
  return;
}
}
__inline static void ( __attribute__((__always_inline__)) zend_ptr_stack_3_pop)(zend_ptr_stack *stack ,
                                                                                void **a ,
                                                                                void **b ,
                                                                                void **c ) 
{ 

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1069\n");
  fflush(_coverage_fout);
  }
  (stack->top_element) --;
  {
  fprintf(_coverage_fout, "1070\n");
  fflush(_coverage_fout);
  }
  *a = *(stack->top_element);
  {
  fprintf(_coverage_fout, "1071\n");
  fflush(_coverage_fout);
  }
  (stack->top_element) --;
  {
  fprintf(_coverage_fout, "1072\n");
  fflush(_coverage_fout);
  }
  *b = *(stack->top_element);
  {
  fprintf(_coverage_fout, "1073\n");
  fflush(_coverage_fout);
  }
  (stack->top_element) --;
  {
  fprintf(_coverage_fout, "1074\n");
  fflush(_coverage_fout);
  }
  *c = *(stack->top_element);
  {
  fprintf(_coverage_fout, "1075\n");
  fflush(_coverage_fout);
  }
  stack->top -= 3;
  {
  fprintf(_coverage_fout, "1076\n");
  fflush(_coverage_fout);
  }
  return;
}
}
__inline static void ( __attribute__((__always_inline__)) zend_ptr_stack_2_push)(zend_ptr_stack *stack ,
                                                                                 void *a ,
                                                                                 void *b ) 
{ void *tmp ;
  void __attribute__((__visibility__("default")))  *tmp___0 ;
  void *tmp___1 ;
  void **tmp___2 ;
  void **tmp___3 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1060\n");
  fflush(_coverage_fout);
  }
  if (stack->top + 2 > stack->max) {
    {
    fprintf(_coverage_fout, "1055\n");
    fflush(_coverage_fout);
    }
    while (1) {
      {
      fprintf(_coverage_fout, "1049\n");
      fflush(_coverage_fout);
      }
      stack->max += 64;
      {
      fprintf(_coverage_fout, "1050\n");
      fflush(_coverage_fout);
      }
      if (stack->top + 2 > stack->max) {
        {
        fprintf(_coverage_fout, "1048\n");
        fflush(_coverage_fout);
        }

      } else {
        break;
      }
    }
    {
    fprintf(_coverage_fout, "1056\n");
    fflush(_coverage_fout);
    }
    if (stack->persistent) {
      {
      fprintf(_coverage_fout, "1051\n");
      fflush(_coverage_fout);
      }
      tmp = __zend_realloc((void *)stack->elements,
                           sizeof(void *) * (unsigned int )stack->max);
      {
      fprintf(_coverage_fout, "1052\n");
      fflush(_coverage_fout);
      }
      tmp___1 = tmp;
    } else {
      {
      fprintf(_coverage_fout, "1053\n");
      fflush(_coverage_fout);
      }
      tmp___0 = _erealloc((void *)stack->elements,
                          sizeof(void *) * (unsigned int )stack->max, 0);
      {
      fprintf(_coverage_fout, "1054\n");
      fflush(_coverage_fout);
      }
      tmp___1 = (void *)tmp___0;
    }
    {
    fprintf(_coverage_fout, "1057\n");
    fflush(_coverage_fout);
    }
    stack->elements = (void **)tmp___1;
    {
    fprintf(_coverage_fout, "1058\n");
    fflush(_coverage_fout);
    }
    stack->top_element = stack->elements + stack->top;
  } else {
    {
    fprintf(_coverage_fout, "1059\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "1061\n");
  fflush(_coverage_fout);
  }
  stack->top += 2;
  {
  fprintf(_coverage_fout, "1062\n");
  fflush(_coverage_fout);
  }
  tmp___2 = stack->top_element;
  {
  fprintf(_coverage_fout, "1063\n");
  fflush(_coverage_fout);
  }
  (stack->top_element) ++;
  {
  fprintf(_coverage_fout, "1064\n");
  fflush(_coverage_fout);
  }
  *tmp___2 = a;
  {
  fprintf(_coverage_fout, "1065\n");
  fflush(_coverage_fout);
  }
  tmp___3 = stack->top_element;
  {
  fprintf(_coverage_fout, "1066\n");
  fflush(_coverage_fout);
  }
  (stack->top_element) ++;
  {
  fprintf(_coverage_fout, "1067\n");
  fflush(_coverage_fout);
  }
  *tmp___3 = b;
  {
  fprintf(_coverage_fout, "1068\n");
  fflush(_coverage_fout);
  }
  return;
}
}
__inline static void ( __attribute__((__always_inline__)) zend_ptr_stack_2_push)(zend_ptr_stack *stack ,
                                                                                 void *a ,
                                                                                 void *b ) ;
__inline static void ( __attribute__((__always_inline__)) zend_ptr_stack_3_push)(zend_ptr_stack *stack ,
                                                                                 void *a ,
                                                                                 void *b ,
                                                                                 void *c ) 
{ void *tmp ;
  void __attribute__((__visibility__("default")))  *tmp___0 ;
  void *tmp___1 ;
  void **tmp___2 ;
  void **tmp___3 ;
  void **tmp___4 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1036\n");
  fflush(_coverage_fout);
  }
  if (stack->top + 3 > stack->max) {
    {
    fprintf(_coverage_fout, "1031\n");
    fflush(_coverage_fout);
    }
    while (1) {
      {
      fprintf(_coverage_fout, "1025\n");
      fflush(_coverage_fout);
      }
      stack->max += 64;
      {
      fprintf(_coverage_fout, "1026\n");
      fflush(_coverage_fout);
      }
      if (stack->top + 3 > stack->max) {
        {
        fprintf(_coverage_fout, "1024\n");
        fflush(_coverage_fout);
        }

      } else {
        break;
      }
    }
    {
    fprintf(_coverage_fout, "1032\n");
    fflush(_coverage_fout);
    }
    if (stack->persistent) {
      {
      fprintf(_coverage_fout, "1027\n");
      fflush(_coverage_fout);
      }
      tmp = __zend_realloc((void *)stack->elements,
                           sizeof(void *) * (unsigned int )stack->max);
      {
      fprintf(_coverage_fout, "1028\n");
      fflush(_coverage_fout);
      }
      tmp___1 = tmp;
    } else {
      {
      fprintf(_coverage_fout, "1029\n");
      fflush(_coverage_fout);
      }
      tmp___0 = _erealloc((void *)stack->elements,
                          sizeof(void *) * (unsigned int )stack->max, 0);
      {
      fprintf(_coverage_fout, "1030\n");
      fflush(_coverage_fout);
      }
      tmp___1 = (void *)tmp___0;
    }
    {
    fprintf(_coverage_fout, "1033\n");
    fflush(_coverage_fout);
    }
    stack->elements = (void **)tmp___1;
    {
    fprintf(_coverage_fout, "1034\n");
    fflush(_coverage_fout);
    }
    stack->top_element = stack->elements + stack->top;
  } else {
    {
    fprintf(_coverage_fout, "1035\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "1037\n");
  fflush(_coverage_fout);
  }
  stack->top += 3;
  {
  fprintf(_coverage_fout, "1038\n");
  fflush(_coverage_fout);
  }
  tmp___2 = stack->top_element;
  {
  fprintf(_coverage_fout, "1039\n");
  fflush(_coverage_fout);
  }
  (stack->top_element) ++;
  {
  fprintf(_coverage_fout, "1040\n");
  fflush(_coverage_fout);
  }
  *tmp___2 = a;
  {
  fprintf(_coverage_fout, "1041\n");
  fflush(_coverage_fout);
  }
  tmp___3 = stack->top_element;
  {
  fprintf(_coverage_fout, "1042\n");
  fflush(_coverage_fout);
  }
  (stack->top_element) ++;
  {
  fprintf(_coverage_fout, "1043\n");
  fflush(_coverage_fout);
  }
  *tmp___3 = b;
  {
  fprintf(_coverage_fout, "1044\n");
  fflush(_coverage_fout);
  }
  tmp___4 = stack->top_element;
  {
  fprintf(_coverage_fout, "1045\n");
  fflush(_coverage_fout);
  }
  (stack->top_element) ++;
  {
  fprintf(_coverage_fout, "1046\n");
  fflush(_coverage_fout);
  }
  *tmp___4 = c;
  {
  fprintf(_coverage_fout, "1047\n");
  fflush(_coverage_fout);
  }
  return;
}
}
__inline static void ( __attribute__((__always_inline__)) zend_ptr_stack_3_push)(zend_ptr_stack *stack ,
                                                                                 void *a ,
                                                                                 void *b ,
                                                                                 void *c ) ;
extern int __attribute__((__visibility__("default")))  zend_ptr_stack_num_elements(zend_ptr_stack *stack ) ;
extern void __attribute__((__visibility__("default")))  zend_ptr_stack_clean(zend_ptr_stack *stack ,
                                                                             void (*func)(void * ) ,
                                                                             zend_bool free_elements ) ;
extern void __attribute__((__visibility__("default")))  zend_ptr_stack_apply(zend_ptr_stack *stack ,
                                                                             void (*func)(void * ) ) ;
extern void __attribute__((__visibility__("default")))  zend_ptr_stack_destroy(zend_ptr_stack *stack ) ;
extern void __attribute__((__visibility__("default")))  zend_ptr_stack_n_pop(zend_ptr_stack *stack ,
                                                                             int count 
                                                                             , ...) ;
extern void __attribute__((__visibility__("default")))  zend_ptr_stack_n_push(zend_ptr_stack *stack ,
                                                                              int count 
                                                                              , ...) ;
extern void __attribute__((__visibility__("default")))  zend_ptr_stack_init_ex(zend_ptr_stack *stack ,
                                                                               zend_bool persistent ) ;
extern void __attribute__((__visibility__("default")))  zend_ptr_stack_init(zend_ptr_stack *stack ) ;
struct _zend_stack;
extern void __attribute__((__visibility__("default")))  zend_stack_apply_with_argument(zend_stack *stack ,
                                                                                       int type ,
                                                                                       int (*apply_function)(void *element ,
                                                                                                             void *arg ) ,
                                                                                       void *arg ) ;
extern void __attribute__((__visibility__("default")))  zend_stack_apply(zend_stack *stack ,
                                                                         int type ,
                                                                         int (*apply_function)(void *element ) ) ;
extern int __attribute__((__visibility__("default")))  zend_stack_count(zend_stack const   *stack ) ;
extern void __attribute__((__visibility__("default")))  **zend_stack_base(zend_stack const   *stack ) ;
extern int __attribute__((__visibility__("default")))  zend_stack_destroy(zend_stack *stack ) ;
extern int __attribute__((__visibility__("default")))  zend_stack_is_empty(zend_stack const   *stack ) ;
extern int __attribute__((__visibility__("default")))  zend_stack_int_top(zend_stack const   *stack ) ;
extern int __attribute__((__visibility__("default")))  zend_stack_del_top(zend_stack *stack ) ;
extern int __attribute__((__visibility__("default")))  zend_stack_top(zend_stack const   *stack ,
                                                                      void **element ) ;
extern int __attribute__((__visibility__("default")))  zend_stack_push(zend_stack *stack ,
                                                                       void const   *element ,
                                                                       int size ) ;
extern int __attribute__((__visibility__("default")))  zend_stack_init(zend_stack *stack ) ;
struct _zend_ini_scanner_globals;
typedef struct _zend_ini_scanner_globals zend_ini_scanner_globals;
extern zend_ini_scanner_globals __attribute__((__visibility__("default")))  ini_scanner_globals ;
struct _zend_php_scanner_globals;
typedef struct _zend_php_scanner_globals zend_php_scanner_globals;
extern zend_php_scanner_globals __attribute__((__visibility__("default")))  language_scanner_globals ;
extern int zendparse(void) ;
struct _zend_compiler_globals;
extern struct _zend_compiler_globals  __attribute__((__visibility__("default"))) compiler_globals ;
struct __jmp_buf_tag;
extern  __attribute__((__nothrow__,
__noreturn__)) void siglongjmp(struct __jmp_buf_tag *__env , int __val ) ;
extern  __attribute__((__nothrow__,
__noreturn__)) void _longjmp(struct __jmp_buf_tag *__env , int __val ) ;
extern  __attribute__((__nothrow__,
__noreturn__)) void longjmp(struct __jmp_buf_tag *__env , int __val ) ;
extern  __attribute__((__nothrow__)) int _setjmp(struct __jmp_buf_tag *__env ) ;
extern  __attribute__((__nothrow__)) int __sigsetjmp(struct __jmp_buf_tag *__env ,
                                                     int __savemask ) ;
extern  __attribute__((__nothrow__)) int setjmp(struct __jmp_buf_tag *__env ) ;
extern char *zend_visibility_string(zend_uint fn_flags ) ;
typedef int ( __attribute__((__fastcall__)) (*opcode_handler_t))(zend_execute_data *execute_data );
extern opcode_handler_t __attribute__((__visibility__("default")))  *zend_opcode_handlers ;
extern void __attribute__((__visibility__("default")))  zend_qsort(void *base ,
                                                                   size_t nmemb ,
                                                                   size_t siz ,
                                                                   int (*compare)(void const   * ,
                                                                                  void const   * ) ) ;
extern void __attribute__((__visibility__("default")))  zend_save_error_handling(zend_error_handling *current ) ;
extern void __attribute__((__visibility__("default")))  _zval_dtor_wrapper(zval *zvalue ) ;
extern void __attribute__((__visibility__("default")))  _zval_internal_ptr_dtor(zval **zvalue ) ;
extern void __attribute__((__visibility__("default")))  _zval_internal_dtor(zval *zvalue ) ;
extern int __attribute__((__visibility__("default")))  zend_print_variable(zval *var ) ;
struct _zend_hash_key;
typedef struct _zend_hash_key zend_hash_key;
extern int __attribute__((__visibility__("default")))  zval_copy_static_var(zval **p ,
                                                                            int num_args ,
                                                                            va_list args ,
                                                                            zend_hash_key *key ) ;
extern void __attribute__((__visibility__("default")))  _zval_copy_ctor_func(zval *zvalue ) ;
__inline static void ( __attribute__((__always_inline__)) _zval_copy_ctor)(zval *zvalue ) 
{ 

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1021\n");
  fflush(_coverage_fout);
  }
  if ((int )zvalue->type <= 3) {
    {
    fprintf(_coverage_fout, "1019\n");
    fflush(_coverage_fout);
    }
    return;
  } else {
    {
    fprintf(_coverage_fout, "1020\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "1022\n");
  fflush(_coverage_fout);
  }
  _zval_copy_ctor_func(zvalue);
  {
  fprintf(_coverage_fout, "1023\n");
  fflush(_coverage_fout);
  }
  return;
}
}
extern void __attribute__((__visibility__("default")))  _zval_dtor_func(zval *zvalue ) ;
__inline static void ( __attribute__((__always_inline__)) _zval_dtor)(zval *zvalue ) 
{ 

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1016\n");
  fflush(_coverage_fout);
  }
  if ((int )zvalue->type <= 3) {
    {
    fprintf(_coverage_fout, "1014\n");
    fflush(_coverage_fout);
    }
    return;
  } else {
    {
    fprintf(_coverage_fout, "1015\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "1017\n");
  fflush(_coverage_fout);
  }
  _zval_dtor_func(zvalue);
  {
  fprintf(_coverage_fout, "1018\n");
  fflush(_coverage_fout);
  }
  return;
}
}
extern int __attribute__((__visibility__("default")))  compare_function(zval *result ,
                                                                        zval *op1 ,
                                                                        zval *op2 ) ;
__inline static int ( __attribute__((__always_inline__)) fast_is_smaller_or_equal_function)(zval *result ,
                                                                                            zval *op1 ,
                                                                                            zval *op2 ) 
{ long tmp ;
  long tmp___0 ;
  long tmp___1 ;
  long tmp___2 ;
  long tmp___3 ;
  long tmp___4 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1010\n");
  fflush(_coverage_fout);
  }
  tmp___4 = __builtin_expect((long )((int )op1->type == 1), 1L);
  {
  fprintf(_coverage_fout, "1011\n");
  fflush(_coverage_fout);
  }
  if (tmp___4) {
    {
    fprintf(_coverage_fout, "998\n");
    fflush(_coverage_fout);
    }
    tmp___0 = __builtin_expect((long )((int )op2->type == 1), 1L);
    {
    fprintf(_coverage_fout, "999\n");
    fflush(_coverage_fout);
    }
    if (tmp___0) {
      {
      fprintf(_coverage_fout, "993\n");
      fflush(_coverage_fout);
      }
      return (op1->value.lval <= op2->value.lval);
    } else {
      {
      fprintf(_coverage_fout, "996\n");
      fflush(_coverage_fout);
      }
      tmp = __builtin_expect((long )((int )op2->type == 2), 1L);
      {
      fprintf(_coverage_fout, "997\n");
      fflush(_coverage_fout);
      }
      if (tmp) {
        {
        fprintf(_coverage_fout, "994\n");
        fflush(_coverage_fout);
        }
        return ((double )op1->value.lval <= op2->value.dval);
      } else {
        {
        fprintf(_coverage_fout, "995\n");
        fflush(_coverage_fout);
        }

      }
    }
  } else {
    {
    fprintf(_coverage_fout, "1008\n");
    fflush(_coverage_fout);
    }
    tmp___3 = __builtin_expect((long )((int )op1->type == 2), 1L);
    {
    fprintf(_coverage_fout, "1009\n");
    fflush(_coverage_fout);
    }
    if (tmp___3) {
      {
      fprintf(_coverage_fout, "1005\n");
      fflush(_coverage_fout);
      }
      tmp___2 = __builtin_expect((long )((int )op2->type == 2), 1L);
      {
      fprintf(_coverage_fout, "1006\n");
      fflush(_coverage_fout);
      }
      if (tmp___2) {
        {
        fprintf(_coverage_fout, "1000\n");
        fflush(_coverage_fout);
        }
        return (op1->value.dval <= op2->value.dval);
      } else {
        {
        fprintf(_coverage_fout, "1003\n");
        fflush(_coverage_fout);
        }
        tmp___1 = __builtin_expect((long )((int )op2->type == 1), 1L);
        {
        fprintf(_coverage_fout, "1004\n");
        fflush(_coverage_fout);
        }
        if (tmp___1) {
          {
          fprintf(_coverage_fout, "1001\n");
          fflush(_coverage_fout);
          }
          return (op1->value.dval <= (double )op2->value.lval);
        } else {
          {
          fprintf(_coverage_fout, "1002\n");
          fflush(_coverage_fout);
          }

        }
      }
    } else {
      {
      fprintf(_coverage_fout, "1007\n");
      fflush(_coverage_fout);
      }

    }
  }
  {
  fprintf(_coverage_fout, "1012\n");
  fflush(_coverage_fout);
  }
  compare_function(result, op1, op2);
  {
  fprintf(_coverage_fout, "1013\n");
  fflush(_coverage_fout);
  }
  return (result->value.lval <= 0L);
}
}
__inline static int ( __attribute__((__always_inline__)) fast_is_smaller_or_equal_function)(zval *result ,
                                                                                            zval *op1 ,
                                                                                            zval *op2 ) ;
__inline static int ( __attribute__((__always_inline__)) fast_is_smaller_function)(zval *result ,
                                                                                   zval *op1 ,
                                                                                   zval *op2 ) 
{ long tmp ;
  long tmp___0 ;
  long tmp___1 ;
  long tmp___2 ;
  long tmp___3 ;
  long tmp___4 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "989\n");
  fflush(_coverage_fout);
  }
  tmp___4 = __builtin_expect((long )((int )op1->type == 1), 1L);
  {
  fprintf(_coverage_fout, "990\n");
  fflush(_coverage_fout);
  }
  if (tmp___4) {
    {
    fprintf(_coverage_fout, "977\n");
    fflush(_coverage_fout);
    }
    tmp___0 = __builtin_expect((long )((int )op2->type == 1), 1L);
    {
    fprintf(_coverage_fout, "978\n");
    fflush(_coverage_fout);
    }
    if (tmp___0) {
      {
      fprintf(_coverage_fout, "972\n");
      fflush(_coverage_fout);
      }
      return (op1->value.lval < op2->value.lval);
    } else {
      {
      fprintf(_coverage_fout, "975\n");
      fflush(_coverage_fout);
      }
      tmp = __builtin_expect((long )((int )op2->type == 2), 1L);
      {
      fprintf(_coverage_fout, "976\n");
      fflush(_coverage_fout);
      }
      if (tmp) {
        {
        fprintf(_coverage_fout, "973\n");
        fflush(_coverage_fout);
        }
        return ((double )op1->value.lval < op2->value.dval);
      } else {
        {
        fprintf(_coverage_fout, "974\n");
        fflush(_coverage_fout);
        }

      }
    }
  } else {
    {
    fprintf(_coverage_fout, "987\n");
    fflush(_coverage_fout);
    }
    tmp___3 = __builtin_expect((long )((int )op1->type == 2), 1L);
    {
    fprintf(_coverage_fout, "988\n");
    fflush(_coverage_fout);
    }
    if (tmp___3) {
      {
      fprintf(_coverage_fout, "984\n");
      fflush(_coverage_fout);
      }
      tmp___2 = __builtin_expect((long )((int )op2->type == 2), 1L);
      {
      fprintf(_coverage_fout, "985\n");
      fflush(_coverage_fout);
      }
      if (tmp___2) {
        {
        fprintf(_coverage_fout, "979\n");
        fflush(_coverage_fout);
        }
        return (op1->value.dval < op2->value.dval);
      } else {
        {
        fprintf(_coverage_fout, "982\n");
        fflush(_coverage_fout);
        }
        tmp___1 = __builtin_expect((long )((int )op2->type == 1), 1L);
        {
        fprintf(_coverage_fout, "983\n");
        fflush(_coverage_fout);
        }
        if (tmp___1) {
          {
          fprintf(_coverage_fout, "980\n");
          fflush(_coverage_fout);
          }
          return (op1->value.dval < (double )op2->value.lval);
        } else {
          {
          fprintf(_coverage_fout, "981\n");
          fflush(_coverage_fout);
          }

        }
      }
    } else {
      {
      fprintf(_coverage_fout, "986\n");
      fflush(_coverage_fout);
      }

    }
  }
  {
  fprintf(_coverage_fout, "991\n");
  fflush(_coverage_fout);
  }
  compare_function(result, op1, op2);
  {
  fprintf(_coverage_fout, "992\n");
  fflush(_coverage_fout);
  }
  return (result->value.lval < 0L);
}
}
__inline static int ( __attribute__((__always_inline__)) fast_is_smaller_function)(zval *result ,
                                                                                   zval *op1 ,
                                                                                   zval *op2 ) ;
__inline static int ( __attribute__((__always_inline__)) fast_not_equal_function)(zval *result ,
                                                                                  zval *op1 ,
                                                                                  zval *op2 ) 
{ long tmp ;
  long tmp___0 ;
  long tmp___1 ;
  long tmp___2 ;
  long tmp___3 ;
  long tmp___4 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "968\n");
  fflush(_coverage_fout);
  }
  tmp___4 = __builtin_expect((long )((int )op1->type == 1), 1L);
  {
  fprintf(_coverage_fout, "969\n");
  fflush(_coverage_fout);
  }
  if (tmp___4) {
    {
    fprintf(_coverage_fout, "956\n");
    fflush(_coverage_fout);
    }
    tmp___0 = __builtin_expect((long )((int )op2->type == 1), 1L);
    {
    fprintf(_coverage_fout, "957\n");
    fflush(_coverage_fout);
    }
    if (tmp___0) {
      {
      fprintf(_coverage_fout, "951\n");
      fflush(_coverage_fout);
      }
      return (op1->value.lval != op2->value.lval);
    } else {
      {
      fprintf(_coverage_fout, "954\n");
      fflush(_coverage_fout);
      }
      tmp = __builtin_expect((long )((int )op2->type == 2), 1L);
      {
      fprintf(_coverage_fout, "955\n");
      fflush(_coverage_fout);
      }
      if (tmp) {
        {
        fprintf(_coverage_fout, "952\n");
        fflush(_coverage_fout);
        }
        return ((double )op1->value.lval != op2->value.dval);
      } else {
        {
        fprintf(_coverage_fout, "953\n");
        fflush(_coverage_fout);
        }

      }
    }
  } else {
    {
    fprintf(_coverage_fout, "966\n");
    fflush(_coverage_fout);
    }
    tmp___3 = __builtin_expect((long )((int )op1->type == 2), 1L);
    {
    fprintf(_coverage_fout, "967\n");
    fflush(_coverage_fout);
    }
    if (tmp___3) {
      {
      fprintf(_coverage_fout, "963\n");
      fflush(_coverage_fout);
      }
      tmp___2 = __builtin_expect((long )((int )op2->type == 2), 1L);
      {
      fprintf(_coverage_fout, "964\n");
      fflush(_coverage_fout);
      }
      if (tmp___2) {
        {
        fprintf(_coverage_fout, "958\n");
        fflush(_coverage_fout);
        }
        return (op1->value.dval != op2->value.dval);
      } else {
        {
        fprintf(_coverage_fout, "961\n");
        fflush(_coverage_fout);
        }
        tmp___1 = __builtin_expect((long )((int )op2->type == 1), 1L);
        {
        fprintf(_coverage_fout, "962\n");
        fflush(_coverage_fout);
        }
        if (tmp___1) {
          {
          fprintf(_coverage_fout, "959\n");
          fflush(_coverage_fout);
          }
          return (op1->value.dval != (double )op2->value.lval);
        } else {
          {
          fprintf(_coverage_fout, "960\n");
          fflush(_coverage_fout);
          }

        }
      }
    } else {
      {
      fprintf(_coverage_fout, "965\n");
      fflush(_coverage_fout);
      }

    }
  }
  {
  fprintf(_coverage_fout, "970\n");
  fflush(_coverage_fout);
  }
  compare_function(result, op1, op2);
  {
  fprintf(_coverage_fout, "971\n");
  fflush(_coverage_fout);
  }
  return (result->value.lval != 0L);
}
}
__inline static int ( __attribute__((__always_inline__)) fast_not_equal_function)(zval *result ,
                                                                                  zval *op1 ,
                                                                                  zval *op2 ) ;
__inline static int ( __attribute__((__always_inline__)) fast_equal_function)(zval *result ,
                                                                              zval *op1 ,
                                                                              zval *op2 ) 
{ long tmp ;
  long tmp___0 ;
  long tmp___1 ;
  long tmp___2 ;
  long tmp___3 ;
  long tmp___4 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "947\n");
  fflush(_coverage_fout);
  }
  tmp___4 = __builtin_expect((long )((int )op1->type == 1), 1L);
  {
  fprintf(_coverage_fout, "948\n");
  fflush(_coverage_fout);
  }
  if (tmp___4) {
    {
    fprintf(_coverage_fout, "935\n");
    fflush(_coverage_fout);
    }
    tmp___0 = __builtin_expect((long )((int )op2->type == 1), 1L);
    {
    fprintf(_coverage_fout, "936\n");
    fflush(_coverage_fout);
    }
    if (tmp___0) {
      {
      fprintf(_coverage_fout, "930\n");
      fflush(_coverage_fout);
      }
      return (op1->value.lval == op2->value.lval);
    } else {
      {
      fprintf(_coverage_fout, "933\n");
      fflush(_coverage_fout);
      }
      tmp = __builtin_expect((long )((int )op2->type == 2), 1L);
      {
      fprintf(_coverage_fout, "934\n");
      fflush(_coverage_fout);
      }
      if (tmp) {
        {
        fprintf(_coverage_fout, "931\n");
        fflush(_coverage_fout);
        }
        return ((double )op1->value.lval == op2->value.dval);
      } else {
        {
        fprintf(_coverage_fout, "932\n");
        fflush(_coverage_fout);
        }

      }
    }
  } else {
    {
    fprintf(_coverage_fout, "945\n");
    fflush(_coverage_fout);
    }
    tmp___3 = __builtin_expect((long )((int )op1->type == 2), 1L);
    {
    fprintf(_coverage_fout, "946\n");
    fflush(_coverage_fout);
    }
    if (tmp___3) {
      {
      fprintf(_coverage_fout, "942\n");
      fflush(_coverage_fout);
      }
      tmp___2 = __builtin_expect((long )((int )op2->type == 2), 1L);
      {
      fprintf(_coverage_fout, "943\n");
      fflush(_coverage_fout);
      }
      if (tmp___2) {
        {
        fprintf(_coverage_fout, "937\n");
        fflush(_coverage_fout);
        }
        return (op1->value.dval == op2->value.dval);
      } else {
        {
        fprintf(_coverage_fout, "940\n");
        fflush(_coverage_fout);
        }
        tmp___1 = __builtin_expect((long )((int )op2->type == 1), 1L);
        {
        fprintf(_coverage_fout, "941\n");
        fflush(_coverage_fout);
        }
        if (tmp___1) {
          {
          fprintf(_coverage_fout, "938\n");
          fflush(_coverage_fout);
          }
          return (op1->value.dval == (double )op2->value.lval);
        } else {
          {
          fprintf(_coverage_fout, "939\n");
          fflush(_coverage_fout);
          }

        }
      }
    } else {
      {
      fprintf(_coverage_fout, "944\n");
      fflush(_coverage_fout);
      }

    }
  }
  {
  fprintf(_coverage_fout, "949\n");
  fflush(_coverage_fout);
  }
  compare_function(result, op1, op2);
  {
  fprintf(_coverage_fout, "950\n");
  fflush(_coverage_fout);
  }
  return (result->value.lval == 0L);
}
}
__inline static int ( __attribute__((__always_inline__)) fast_equal_function)(zval *result ,
                                                                              zval *op1 ,
                                                                              zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  mod_function(zval *result ,
                                                                    zval *op1 ,
                                                                    zval *op2 ) ;
__inline static int ( __attribute__((__always_inline__)) fast_mod_function)(zval *result ,
                                                                            zval *op1 ,
                                                                            zval *op2 ) 
{ long tmp ;
  long tmp___0 ;
  long tmp___1 ;
  long tmp___2 ;
  int __attribute__((__visibility__("default")))  tmp___3 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "926\n");
  fflush(_coverage_fout);
  }
  tmp___2 = __builtin_expect((long )((int )op1->type == 1), 1L);
  {
  fprintf(_coverage_fout, "927\n");
  fflush(_coverage_fout);
  }
  if (tmp___2) {
    {
    fprintf(_coverage_fout, "923\n");
    fflush(_coverage_fout);
    }
    tmp___1 = __builtin_expect((long )((int )op2->type == 1), 1L);
    {
    fprintf(_coverage_fout, "924\n");
    fflush(_coverage_fout);
    }
    if (tmp___1) {
      {
      fprintf(_coverage_fout, "917\n");
      fflush(_coverage_fout);
      }
      tmp___0 = __builtin_expect((long )(op2->value.lval == 0L), 0L);
      {
      fprintf(_coverage_fout, "918\n");
      fflush(_coverage_fout);
      }
      if (tmp___0) {
        {
        fprintf(_coverage_fout, "907\n");
        fflush(_coverage_fout);
        }
        zend_error(1 << 1L, "Division by zero");
        {
        fprintf(_coverage_fout, "908\n");
        fflush(_coverage_fout);
        }
        result->value.lval = 0L;
        {
        fprintf(_coverage_fout, "909\n");
        fflush(_coverage_fout);
        }
        result->type = (unsigned char)3;
        {
        fprintf(_coverage_fout, "910\n");
        fflush(_coverage_fout);
        }
        return (-1);
      } else {
        {
        fprintf(_coverage_fout, "915\n");
        fflush(_coverage_fout);
        }
        tmp = __builtin_expect((long )(op2->value.lval == -1L), 0L);
        {
        fprintf(_coverage_fout, "916\n");
        fflush(_coverage_fout);
        }
        if (tmp) {
          {
          fprintf(_coverage_fout, "911\n");
          fflush(_coverage_fout);
          }
          result->value.lval = 0L;
          {
          fprintf(_coverage_fout, "912\n");
          fflush(_coverage_fout);
          }
          result->type = (unsigned char)1;
          {
          fprintf(_coverage_fout, "913\n");
          fflush(_coverage_fout);
          }
          return (0);
        } else {
          {
          fprintf(_coverage_fout, "914\n");
          fflush(_coverage_fout);
          }

        }
      }
      {
      fprintf(_coverage_fout, "919\n");
      fflush(_coverage_fout);
      }
      result->value.lval = op1->value.lval % op2->value.lval;
      {
      fprintf(_coverage_fout, "920\n");
      fflush(_coverage_fout);
      }
      result->type = (unsigned char)1;
      {
      fprintf(_coverage_fout, "921\n");
      fflush(_coverage_fout);
      }
      return (0);
    } else {
      {
      fprintf(_coverage_fout, "922\n");
      fflush(_coverage_fout);
      }

    }
  } else {
    {
    fprintf(_coverage_fout, "925\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "928\n");
  fflush(_coverage_fout);
  }
  tmp___3 = mod_function(result, op1, op2);
  {
  fprintf(_coverage_fout, "929\n");
  fflush(_coverage_fout);
  }
  return ((int )tmp___3);
}
}
__inline static int ( __attribute__((__always_inline__)) fast_mod_function)(zval *result ,
                                                                            zval *op1 ,
                                                                            zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  div_function(zval *result ,
                                                                    zval *op1 ,
                                                                    zval *op2 ) ;
__inline static int ( __attribute__((__always_inline__)) fast_div_function)(zval *result ,
                                                                            zval *op1 ,
                                                                            zval *op2 ) 
{ long tmp ;
  int tmp___0 ;
  long tmp___1 ;
  long tmp___2 ;
  long tmp___3 ;
  long tmp___4 ;
  long tmp___5 ;
  long tmp___6 ;
  long tmp___7 ;
  long tmp___8 ;
  long tmp___9 ;
  long tmp___10 ;
  long tmp___11 ;
  int __attribute__((__visibility__("default")))  tmp___12 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "903\n");
  fflush(_coverage_fout);
  }
  tmp___11 = __builtin_expect((long )((int )op1->type == 1), 1L);
  {
  fprintf(_coverage_fout, "904\n");
  fflush(_coverage_fout);
  }
  if (tmp___11) {
    goto _L;
  } else {
    {
    fprintf(_coverage_fout, "901\n");
    fflush(_coverage_fout);
    }
    _L: /* CIL Label */ 
    tmp___10 = __builtin_expect((long )((int )op1->type == 2), 1L);
    {
    fprintf(_coverage_fout, "902\n");
    fflush(_coverage_fout);
    }
    if (tmp___10) {
      {
      fprintf(_coverage_fout, "899\n");
      fflush(_coverage_fout);
      }

    } else {
      {
      fprintf(_coverage_fout, "900\n");
      fflush(_coverage_fout);
      }

    }
  }
  {
  fprintf(_coverage_fout, "905\n");
  fflush(_coverage_fout);
  }
  tmp___12 = div_function(result, op1, op2);
  {
  fprintf(_coverage_fout, "906\n");
  fflush(_coverage_fout);
  }
  return ((int )tmp___12);
}
}
__inline static int ( __attribute__((__always_inline__)) fast_div_function)(zval *result ,
                                                                            zval *op1 ,
                                                                            zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  mul_function(zval *result ,
                                                                    zval *op1 ,
                                                                    zval *op2 ) ;
__inline static int ( __attribute__((__always_inline__)) fast_mul_function)(zval *result ,
                                                                            zval *op1 ,
                                                                            zval *op2 ) 
{ long overflow ;
  long __tmpvar ;
  long tmp ;
  long tmp___0 ;
  long tmp___1 ;
  long tmp___2 ;
  long tmp___3 ;
  long tmp___4 ;
  int __attribute__((__visibility__("default")))  tmp___5 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "895\n");
  fflush(_coverage_fout);
  }
  tmp___4 = __builtin_expect((long )((int )op1->type == 1), 1L);
  {
  fprintf(_coverage_fout, "896\n");
  fflush(_coverage_fout);
  }
  if (tmp___4) {
    {
    fprintf(_coverage_fout, "879\n");
    fflush(_coverage_fout);
    }
    tmp___0 = __builtin_expect((long )((int )op2->type == 1), 1L);
    {
    fprintf(_coverage_fout, "880\n");
    fflush(_coverage_fout);
    }
    if (tmp___0) {
      {
      fprintf(_coverage_fout, "870\n");
      fflush(_coverage_fout);
      }
      while (1) {
        {
        fprintf(_coverage_fout, "866\n");
        fflush(_coverage_fout);
        }
        __asm__  ("imul %3,%0\n"
                  "adc $0,%1": "=r" (__tmpvar),
                  "=r" (overflow): "0" (op1->value.lval), "r" (op2->value.lval),
                  "1" (0));
        {
        fprintf(_coverage_fout, "867\n");
        fflush(_coverage_fout);
        }
        if (overflow) {
          {
          fprintf(_coverage_fout, "864\n");
          fflush(_coverage_fout);
          }
          result->value.dval = (double )op1->value.lval * (double )op2->value.lval;
        } else {
          {
          fprintf(_coverage_fout, "865\n");
          fflush(_coverage_fout);
          }
          result->value.lval = __tmpvar;
        }
        break;
      }
      {
      fprintf(_coverage_fout, "871\n");
      fflush(_coverage_fout);
      }
      if (overflow) {
        {
        fprintf(_coverage_fout, "868\n");
        fflush(_coverage_fout);
        }
        result->type = (unsigned char)2;
      } else {
        {
        fprintf(_coverage_fout, "869\n");
        fflush(_coverage_fout);
        }
        result->type = (unsigned char)1;
      }
      {
      fprintf(_coverage_fout, "872\n");
      fflush(_coverage_fout);
      }
      return (0);
    } else {
      {
      fprintf(_coverage_fout, "877\n");
      fflush(_coverage_fout);
      }
      tmp = __builtin_expect((long )((int )op2->type == 2), 1L);
      {
      fprintf(_coverage_fout, "878\n");
      fflush(_coverage_fout);
      }
      if (tmp) {
        {
        fprintf(_coverage_fout, "873\n");
        fflush(_coverage_fout);
        }
        result->value.dval = (double )op1->value.lval * op2->value.dval;
        {
        fprintf(_coverage_fout, "874\n");
        fflush(_coverage_fout);
        }
        result->type = (unsigned char)2;
        {
        fprintf(_coverage_fout, "875\n");
        fflush(_coverage_fout);
        }
        return (0);
      } else {
        {
        fprintf(_coverage_fout, "876\n");
        fflush(_coverage_fout);
        }

      }
    }
  } else {
    {
    fprintf(_coverage_fout, "893\n");
    fflush(_coverage_fout);
    }
    tmp___3 = __builtin_expect((long )((int )op1->type == 2), 1L);
    {
    fprintf(_coverage_fout, "894\n");
    fflush(_coverage_fout);
    }
    if (tmp___3) {
      {
      fprintf(_coverage_fout, "890\n");
      fflush(_coverage_fout);
      }
      tmp___2 = __builtin_expect((long )((int )op2->type == 2), 1L);
      {
      fprintf(_coverage_fout, "891\n");
      fflush(_coverage_fout);
      }
      if (tmp___2) {
        {
        fprintf(_coverage_fout, "881\n");
        fflush(_coverage_fout);
        }
        result->value.dval = op1->value.dval * op2->value.dval;
        {
        fprintf(_coverage_fout, "882\n");
        fflush(_coverage_fout);
        }
        result->type = (unsigned char)2;
        {
        fprintf(_coverage_fout, "883\n");
        fflush(_coverage_fout);
        }
        return (0);
      } else {
        {
        fprintf(_coverage_fout, "888\n");
        fflush(_coverage_fout);
        }
        tmp___1 = __builtin_expect((long )((int )op2->type == 1), 1L);
        {
        fprintf(_coverage_fout, "889\n");
        fflush(_coverage_fout);
        }
        if (tmp___1) {
          {
          fprintf(_coverage_fout, "884\n");
          fflush(_coverage_fout);
          }
          result->value.dval = op1->value.dval * (double )op2->value.lval;
          {
          fprintf(_coverage_fout, "885\n");
          fflush(_coverage_fout);
          }
          result->type = (unsigned char)2;
          {
          fprintf(_coverage_fout, "886\n");
          fflush(_coverage_fout);
          }
          return (0);
        } else {
          {
          fprintf(_coverage_fout, "887\n");
          fflush(_coverage_fout);
          }

        }
      }
    } else {
      {
      fprintf(_coverage_fout, "892\n");
      fflush(_coverage_fout);
      }

    }
  }
  {
  fprintf(_coverage_fout, "897\n");
  fflush(_coverage_fout);
  }
  tmp___5 = mul_function(result, op1, op2);
  {
  fprintf(_coverage_fout, "898\n");
  fflush(_coverage_fout);
  }
  return ((int )tmp___5);
}
}
__inline static int ( __attribute__((__always_inline__)) fast_mul_function)(zval *result ,
                                                                            zval *op1 ,
                                                                            zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  sub_function(zval *result ,
                                                                    zval *op1 ,
                                                                    zval *op2 ) ;
__inline static int ( __attribute__((__always_inline__)) fast_sub_function)(zval *result ,
                                                                            zval *op1 ,
                                                                            zval *op2 ) 
{ long tmp ;
  long tmp___0 ;
  long tmp___1 ;
  long tmp___2 ;
  long tmp___3 ;
  long tmp___4 ;
  int __attribute__((__visibility__("default")))  tmp___5 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "860\n");
  fflush(_coverage_fout);
  }
  tmp___4 = __builtin_expect((long )((int )op1->type == 1), 1L);
  {
  fprintf(_coverage_fout, "861\n");
  fflush(_coverage_fout);
  }
  if (tmp___4) {
    {
    fprintf(_coverage_fout, "844\n");
    fflush(_coverage_fout);
    }
    tmp___0 = __builtin_expect((long )((int )op2->type == 1), 1L);
    {
    fprintf(_coverage_fout, "845\n");
    fflush(_coverage_fout);
    }
    if (tmp___0) {
      {
      fprintf(_coverage_fout, "836\n");
      fflush(_coverage_fout);
      }
      __asm__  ("movl\t(%1), %%eax\n\t"
                "subl   (%2), %%eax\n\t"
                "jo     0f\n\t"
                "movl   %%eax, (%0)\n\t"
                "movb   $0x1,0xc(%0)\n\t"
                "jmp    1f\n"
                "0:\n\t"
                "fildl\t(%2)\n\t"
                "fildl\t(%1)\n\t"
                "fsubp\t%%st, %%st(1)\n\t"
                "movb   $0x2,0xc(%0)\n\t"
                "fstpl\t(%0)\n"
                "1:": : "r" (result), "r" (op1), "r" (op2): "eax");
      {
      fprintf(_coverage_fout, "837\n");
      fflush(_coverage_fout);
      }
      return (0);
    } else {
      {
      fprintf(_coverage_fout, "842\n");
      fflush(_coverage_fout);
      }
      tmp = __builtin_expect((long )((int )op2->type == 2), 1L);
      {
      fprintf(_coverage_fout, "843\n");
      fflush(_coverage_fout);
      }
      if (tmp) {
        {
        fprintf(_coverage_fout, "838\n");
        fflush(_coverage_fout);
        }
        result->value.dval = (double )op1->value.lval - op2->value.dval;
        {
        fprintf(_coverage_fout, "839\n");
        fflush(_coverage_fout);
        }
        result->type = (unsigned char)2;
        {
        fprintf(_coverage_fout, "840\n");
        fflush(_coverage_fout);
        }
        return (0);
      } else {
        {
        fprintf(_coverage_fout, "841\n");
        fflush(_coverage_fout);
        }

      }
    }
  } else {
    {
    fprintf(_coverage_fout, "858\n");
    fflush(_coverage_fout);
    }
    tmp___3 = __builtin_expect((long )((int )op1->type == 2), 1L);
    {
    fprintf(_coverage_fout, "859\n");
    fflush(_coverage_fout);
    }
    if (tmp___3) {
      {
      fprintf(_coverage_fout, "855\n");
      fflush(_coverage_fout);
      }
      tmp___2 = __builtin_expect((long )((int )op2->type == 2), 1L);
      {
      fprintf(_coverage_fout, "856\n");
      fflush(_coverage_fout);
      }
      if (tmp___2) {
        {
        fprintf(_coverage_fout, "846\n");
        fflush(_coverage_fout);
        }
        result->value.dval = op1->value.dval - op2->value.dval;
        {
        fprintf(_coverage_fout, "847\n");
        fflush(_coverage_fout);
        }
        result->type = (unsigned char)2;
        {
        fprintf(_coverage_fout, "848\n");
        fflush(_coverage_fout);
        }
        return (0);
      } else {
        {
        fprintf(_coverage_fout, "853\n");
        fflush(_coverage_fout);
        }
        tmp___1 = __builtin_expect((long )((int )op2->type == 1), 1L);
        {
        fprintf(_coverage_fout, "854\n");
        fflush(_coverage_fout);
        }
        if (tmp___1) {
          {
          fprintf(_coverage_fout, "849\n");
          fflush(_coverage_fout);
          }
          result->value.dval = op1->value.dval - (double )op2->value.lval;
          {
          fprintf(_coverage_fout, "850\n");
          fflush(_coverage_fout);
          }
          result->type = (unsigned char)2;
          {
          fprintf(_coverage_fout, "851\n");
          fflush(_coverage_fout);
          }
          return (0);
        } else {
          {
          fprintf(_coverage_fout, "852\n");
          fflush(_coverage_fout);
          }

        }
      }
    } else {
      {
      fprintf(_coverage_fout, "857\n");
      fflush(_coverage_fout);
      }

    }
  }
  {
  fprintf(_coverage_fout, "862\n");
  fflush(_coverage_fout);
  }
  tmp___5 = sub_function(result, op1, op2);
  {
  fprintf(_coverage_fout, "863\n");
  fflush(_coverage_fout);
  }
  return ((int )tmp___5);
}
}
__inline static int ( __attribute__((__always_inline__)) fast_sub_function)(zval *result ,
                                                                            zval *op1 ,
                                                                            zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  add_function(zval *result ,
                                                                    zval *op1 ,
                                                                    zval *op2 ) ;
__inline static int ( __attribute__((__always_inline__)) fast_add_function)(zval *result ,
                                                                            zval *op1 ,
                                                                            zval *op2 ) 
{ long tmp ;
  long tmp___0 ;
  long tmp___1 ;
  long tmp___2 ;
  long tmp___3 ;
  long tmp___4 ;
  int __attribute__((__visibility__("default")))  tmp___5 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "832\n");
  fflush(_coverage_fout);
  }
  tmp___4 = __builtin_expect((long )((int )op1->type == 1), 1L);
  {
  fprintf(_coverage_fout, "833\n");
  fflush(_coverage_fout);
  }
  if (tmp___4) {
    {
    fprintf(_coverage_fout, "816\n");
    fflush(_coverage_fout);
    }
    tmp___0 = __builtin_expect((long )((int )op2->type == 1), 1L);
    {
    fprintf(_coverage_fout, "817\n");
    fflush(_coverage_fout);
    }
    if (tmp___0) {
      {
      fprintf(_coverage_fout, "808\n");
      fflush(_coverage_fout);
      }
      __asm__  ("movl\t(%1), %%eax\n\t"
                "addl   (%2), %%eax\n\t"
                "jo     0f\n\t"
                "movl   %%eax, (%0)\n\t"
                "movb   $0x1,0xc(%0)\n\t"
                "jmp    1f\n"
                "0:\n\t"
                "fildl\t(%1)\n\t"
                "fildl\t(%2)\n\t"
                "faddp\t%%st, %%st(1)\n\t"
                "movb   $0x2,0xc(%0)\n\t"
                "fstpl\t(%0)\n"
                "1:": : "r" (result), "r" (op1), "r" (op2): "eax");
      {
      fprintf(_coverage_fout, "809\n");
      fflush(_coverage_fout);
      }
      return (0);
    } else {
      {
      fprintf(_coverage_fout, "814\n");
      fflush(_coverage_fout);
      }
      tmp = __builtin_expect((long )((int )op2->type == 2), 1L);
      {
      fprintf(_coverage_fout, "815\n");
      fflush(_coverage_fout);
      }
      if (tmp) {
        {
        fprintf(_coverage_fout, "810\n");
        fflush(_coverage_fout);
        }
        result->value.dval = (double )op1->value.lval + op2->value.dval;
        {
        fprintf(_coverage_fout, "811\n");
        fflush(_coverage_fout);
        }
        result->type = (unsigned char)2;
        {
        fprintf(_coverage_fout, "812\n");
        fflush(_coverage_fout);
        }
        return (0);
      } else {
        {
        fprintf(_coverage_fout, "813\n");
        fflush(_coverage_fout);
        }

      }
    }
  } else {
    {
    fprintf(_coverage_fout, "830\n");
    fflush(_coverage_fout);
    }
    tmp___3 = __builtin_expect((long )((int )op1->type == 2), 1L);
    {
    fprintf(_coverage_fout, "831\n");
    fflush(_coverage_fout);
    }
    if (tmp___3) {
      {
      fprintf(_coverage_fout, "827\n");
      fflush(_coverage_fout);
      }
      tmp___2 = __builtin_expect((long )((int )op2->type == 2), 1L);
      {
      fprintf(_coverage_fout, "828\n");
      fflush(_coverage_fout);
      }
      if (tmp___2) {
        {
        fprintf(_coverage_fout, "818\n");
        fflush(_coverage_fout);
        }
        result->value.dval = op1->value.dval + op2->value.dval;
        {
        fprintf(_coverage_fout, "819\n");
        fflush(_coverage_fout);
        }
        result->type = (unsigned char)2;
        {
        fprintf(_coverage_fout, "820\n");
        fflush(_coverage_fout);
        }
        return (0);
      } else {
        {
        fprintf(_coverage_fout, "825\n");
        fflush(_coverage_fout);
        }
        tmp___1 = __builtin_expect((long )((int )op2->type == 1), 1L);
        {
        fprintf(_coverage_fout, "826\n");
        fflush(_coverage_fout);
        }
        if (tmp___1) {
          {
          fprintf(_coverage_fout, "821\n");
          fflush(_coverage_fout);
          }
          result->value.dval = op1->value.dval + (double )op2->value.lval;
          {
          fprintf(_coverage_fout, "822\n");
          fflush(_coverage_fout);
          }
          result->type = (unsigned char)2;
          {
          fprintf(_coverage_fout, "823\n");
          fflush(_coverage_fout);
          }
          return (0);
        } else {
          {
          fprintf(_coverage_fout, "824\n");
          fflush(_coverage_fout);
          }

        }
      }
    } else {
      {
      fprintf(_coverage_fout, "829\n");
      fflush(_coverage_fout);
      }

    }
  }
  {
  fprintf(_coverage_fout, "834\n");
  fflush(_coverage_fout);
  }
  tmp___5 = add_function(result, op1, op2);
  {
  fprintf(_coverage_fout, "835\n");
  fflush(_coverage_fout);
  }
  return ((int )tmp___5);
}
}
__inline static int ( __attribute__((__always_inline__)) fast_add_function)(zval *result ,
                                                                            zval *op1 ,
                                                                            zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  decrement_function(zval *op2 ) ;
__inline static int ( __attribute__((__always_inline__)) fast_decrement_function)(zval *op1 ) 
{ long tmp ;
  int __attribute__((__visibility__("default")))  tmp___0 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "804\n");
  fflush(_coverage_fout);
  }
  tmp = __builtin_expect((long )((int )op1->type == 1), 1L);
  {
  fprintf(_coverage_fout, "805\n");
  fflush(_coverage_fout);
  }
  if (tmp) {
    {
    fprintf(_coverage_fout, "801\n");
    fflush(_coverage_fout);
    }
    __asm__  ("decl (%0)\n\t"
              "jno  0f\n\t"
              "movl $0x00200000, (%0)\n\t"
              "movl $0xc1e00000, 0x4(%0)\n\t"
              "movb $0x2,0xc(%0)\n"
              "0:": : "r" (op1));
    {
    fprintf(_coverage_fout, "802\n");
    fflush(_coverage_fout);
    }
    return (0);
  } else {
    {
    fprintf(_coverage_fout, "803\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "806\n");
  fflush(_coverage_fout);
  }
  tmp___0 = decrement_function(op1);
  {
  fprintf(_coverage_fout, "807\n");
  fflush(_coverage_fout);
  }
  return ((int )tmp___0);
}
}
__inline static int ( __attribute__((__always_inline__)) fast_decrement_function)(zval *op1 ) ;
extern int __attribute__((__visibility__("default")))  increment_function(zval *op1 ) ;
__inline static int ( __attribute__((__always_inline__)) fast_increment_function)(zval *op1 ) 
{ long tmp ;
  int __attribute__((__visibility__("default")))  tmp___0 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "797\n");
  fflush(_coverage_fout);
  }
  tmp = __builtin_expect((long )((int )op1->type == 1), 1L);
  {
  fprintf(_coverage_fout, "798\n");
  fflush(_coverage_fout);
  }
  if (tmp) {
    {
    fprintf(_coverage_fout, "794\n");
    fflush(_coverage_fout);
    }
    __asm__  ("incl (%0)\n\t"
              "jno  0f\n\t"
              "movl $0x0, (%0)\n\t"
              "movl $0x41e00000, 0x4(%0)\n\t"
              "movb $0x2,0xc(%0)\n"
              "0:": : "r" (op1));
    {
    fprintf(_coverage_fout, "795\n");
    fflush(_coverage_fout);
    }
    return (0);
  } else {
    {
    fprintf(_coverage_fout, "796\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "799\n");
  fflush(_coverage_fout);
  }
  tmp___0 = increment_function(op1);
  {
  fprintf(_coverage_fout, "800\n");
  fflush(_coverage_fout);
  }
  return ((int )tmp___0);
}
}
__inline static int ( __attribute__((__always_inline__)) fast_increment_function)(zval *op1 ) ;
extern void __attribute__((__visibility__("default")))  zend_locale_sprintf_double(zval *op ) ;
extern long __attribute__((__visibility__("default")))  zend_atol(char const   *str ,
                                                                  int str_len ) ;
extern int __attribute__((__visibility__("default")))  zend_atoi(char const   *str ,
                                                                 int str_len ) ;
extern void __attribute__((__visibility__("default")))  zend_compare_objects(zval *result ,
                                                                             zval *o1 ,
                                                                             zval *o2 ) ;
extern void __attribute__((__visibility__("default")))  zend_compare_arrays(zval *result ,
                                                                            zval *a1 ,
                                                                            zval *a2 ) ;
extern void __attribute__((__visibility__("default")))  zend_compare_symbol_tables(zval *result ,
                                                                                   HashTable *ht1 ,
                                                                                   HashTable *ht2 ) ;
extern void __attribute__((__visibility__("default")))  zendi_smart_strcmp(zval *result ,
                                                                           zval *s1 ,
                                                                           zval *s2 ) ;
extern int __attribute__((__visibility__("default")))  zend_binary_strncasecmp(char const   *s1 ,
                                                                               uint len1 ,
                                                                               char const   *s2 ,
                                                                               uint len2 ,
                                                                               uint length ) ;
extern int __attribute__((__visibility__("default")))  zend_binary_strcasecmp(char const   *s1 ,
                                                                              uint len1 ,
                                                                              char const   *s2 ,
                                                                              uint len2 ) ;
extern int __attribute__((__visibility__("default")))  zend_binary_strncmp(char const   *s1 ,
                                                                           uint len1 ,
                                                                           char const   *s2 ,
                                                                           uint len2 ,
                                                                           uint length ) ;
extern int __attribute__((__visibility__("default")))  zend_binary_strcmp(char const   *s1 ,
                                                                          uint len1 ,
                                                                          char const   *s2 ,
                                                                          uint len2 ) ;
extern int __attribute__((__visibility__("default")))  zend_binary_zval_strncasecmp(zval *s1 ,
                                                                                    zval *s2 ,
                                                                                    zval *s3 ) ;
extern int __attribute__((__visibility__("default")))  zend_binary_zval_strcasecmp(zval *s1 ,
                                                                                   zval *s2 ) ;
extern int __attribute__((__visibility__("default")))  zend_binary_zval_strncmp(zval *s1 ,
                                                                                zval *s2 ,
                                                                                zval *s3 ) ;
extern int __attribute__((__visibility__("default")))  zend_binary_zval_strcmp(zval *s1 ,
                                                                               zval *s2 ) ;
extern char __attribute__((__visibility__("default")))  *zend_str_tolower_dup(char const   *source ,
                                                                              unsigned int length ) ;
extern char __attribute__((__visibility__("default")))  *zend_str_tolower_copy(char *dest ,
                                                                               char const   *source ,
                                                                               unsigned int length ) ;
extern void __attribute__((__visibility__("default")))  zend_str_tolower(char *str ,
                                                                         unsigned int length ) ;
extern int __attribute__((__visibility__("default")))  string_locale_compare_function(zval *result ,
                                                                                      zval *op1 ,
                                                                                      zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  string_case_compare_function(zval *result ,
                                                                                    zval *op1 ,
                                                                                    zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  string_compare_function(zval *result ,
                                                                               zval *op1 ,
                                                                               zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  string_compare_function_ex(zval *result ,
                                                                                  zval *op1 ,
                                                                                  zval *op2 ,
                                                                                  zend_bool case_insensitive ) ;
extern int __attribute__((__visibility__("default")))  numeric_compare_function(zval *result ,
                                                                                zval *op1 ,
                                                                                zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  zval_is_true(zval *op ) ;
extern double __attribute__((__visibility__("default")))  zend_string_to_double(char const   *number ,
                                                                                zend_uint length ) ;
extern int __attribute__((__visibility__("default")))  add_string_to_string(zval *result ,
                                                                            zval const   *op1 ,
                                                                            zval const   *op2 ) ;
extern int __attribute__((__visibility__("default")))  add_char_to_string(zval *result ,
                                                                          zval const   *op1 ,
                                                                          zval const   *op2 ) ;
extern void __attribute__((__visibility__("default")))  multi_convert_to_string_ex(int argc 
                                                                                   , ...) ;
extern void __attribute__((__visibility__("default")))  multi_convert_to_double_ex(int argc 
                                                                                   , ...) ;
extern void __attribute__((__visibility__("default")))  multi_convert_to_long_ex(int argc 
                                                                                 , ...) ;
extern void __attribute__((__visibility__("default")))  convert_to_object(zval *op ) ;
extern void __attribute__((__visibility__("default")))  convert_to_array(zval *op ) ;
extern void __attribute__((__visibility__("default")))  convert_to_null(zval *op ) ;
extern void __attribute__((__visibility__("default")))  convert_to_long_base(zval *op ,
                                                                             int base ) ;
extern void __attribute__((__visibility__("default")))  convert_to_double(zval *op ) ;
extern void __attribute__((__visibility__("default")))  _convert_to_string(zval *op ) ;
extern void __attribute__((__visibility__("default")))  convert_scalar_to_number(zval *op ) ;
__inline static void const   *zend_memrchr(void const   *s , int c , size_t n ) 
{ register unsigned char const   *e ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "790\n");
  fflush(_coverage_fout);
  }
  if (n <= 0U) {
    {
    fprintf(_coverage_fout, "782\n");
    fflush(_coverage_fout);
    }
    return ((void const   *)((void *)0));
  } else {
    {
    fprintf(_coverage_fout, "783\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "791\n");
  fflush(_coverage_fout);
  }
  e = ((unsigned char const   *)s + n) - 1;
  {
  fprintf(_coverage_fout, "792\n");
  fflush(_coverage_fout);
  }
  while (1) {
    {
    fprintf(_coverage_fout, "787\n");
    fflush(_coverage_fout);
    }
    if ((unsigned int )e >= (unsigned int )((unsigned char const   *)s)) {
      {
      fprintf(_coverage_fout, "784\n");
      fflush(_coverage_fout);
      }

    } else {
      break;
    }
    {
    fprintf(_coverage_fout, "788\n");
    fflush(_coverage_fout);
    }
    if ((int const   )*e == (int const   )((unsigned char const   )c)) {
      {
      fprintf(_coverage_fout, "785\n");
      fflush(_coverage_fout);
      }
      return ((void const   *)e);
    } else {
      {
      fprintf(_coverage_fout, "786\n");
      fflush(_coverage_fout);
      }

    }
    {
    fprintf(_coverage_fout, "789\n");
    fflush(_coverage_fout);
    }
    e --;
  }
  {
  fprintf(_coverage_fout, "793\n");
  fflush(_coverage_fout);
  }
  return ((void const   *)((void *)0));
}
}
__inline static void const   *zend_memrchr(void const   *s , int c , size_t n ) ;
extern  __attribute__((__nothrow__)) void *memchr(void const   *__s , int __c ,
                                                  size_t __n )  __attribute__((__pure__,
__nonnull__(1))) ;
__inline static char *zend_memnstr(char *haystack , char *needle ,
                                   int needle_len , char *end ) 
{ char *p ;
  char ne ;
  void *tmp ;
  int tmp___0 ;
  void *tmp___1 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "775\n");
  fflush(_coverage_fout);
  }
  p = haystack;
  {
  fprintf(_coverage_fout, "776\n");
  fflush(_coverage_fout);
  }
  ne = *(needle + (needle_len - 1));
  {
  fprintf(_coverage_fout, "777\n");
  fflush(_coverage_fout);
  }
  if (needle_len == 1) {
    {
    fprintf(_coverage_fout, "754\n");
    fflush(_coverage_fout);
    }
    tmp = memchr((void const   *)p, (int )*needle, (unsigned int )(end - p));
    {
    fprintf(_coverage_fout, "755\n");
    fflush(_coverage_fout);
    }
    return ((char *)tmp);
  } else {
    {
    fprintf(_coverage_fout, "756\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "778\n");
  fflush(_coverage_fout);
  }
  if (needle_len > end - haystack) {
    {
    fprintf(_coverage_fout, "757\n");
    fflush(_coverage_fout);
    }
    return ((char *)((void *)0));
  } else {
    {
    fprintf(_coverage_fout, "758\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "779\n");
  fflush(_coverage_fout);
  }
  end -= needle_len;
  {
  fprintf(_coverage_fout, "780\n");
  fflush(_coverage_fout);
  }
  while (1) {
    {
    fprintf(_coverage_fout, "769\n");
    fflush(_coverage_fout);
    }
    if ((unsigned int )p <= (unsigned int )end) {
      {
      fprintf(_coverage_fout, "759\n");
      fflush(_coverage_fout);
      }

    } else {
      break;
    }
    {
    fprintf(_coverage_fout, "770\n");
    fflush(_coverage_fout);
    }
    tmp___1 = memchr((void const   *)p, (int )*needle,
                     (unsigned int )((end - p) + 1));
    {
    fprintf(_coverage_fout, "771\n");
    fflush(_coverage_fout);
    }
    p = (char *)tmp___1;
    {
    fprintf(_coverage_fout, "772\n");
    fflush(_coverage_fout);
    }
    if (p) {
      {
      fprintf(_coverage_fout, "765\n");
      fflush(_coverage_fout);
      }
      if ((int )ne == (int )*(p + (needle_len - 1))) {
        {
        fprintf(_coverage_fout, "762\n");
        fflush(_coverage_fout);
        }
        tmp___0 = memcmp((void const   *)needle, (void const   *)p,
                         (unsigned int )(needle_len - 1));
        {
        fprintf(_coverage_fout, "763\n");
        fflush(_coverage_fout);
        }
        if (tmp___0) {
          {
          fprintf(_coverage_fout, "760\n");
          fflush(_coverage_fout);
          }

        } else {
          {
          fprintf(_coverage_fout, "761\n");
          fflush(_coverage_fout);
          }
          return (p);
        }
      } else {
        {
        fprintf(_coverage_fout, "764\n");
        fflush(_coverage_fout);
        }

      }
    } else {
      {
      fprintf(_coverage_fout, "766\n");
      fflush(_coverage_fout);
      }

    }
    {
    fprintf(_coverage_fout, "773\n");
    fflush(_coverage_fout);
    }
    if ((unsigned int )p == (unsigned int )((void *)0)) {
      {
      fprintf(_coverage_fout, "767\n");
      fflush(_coverage_fout);
      }
      return ((char *)((void *)0));
    } else {
      {
      fprintf(_coverage_fout, "768\n");
      fflush(_coverage_fout);
      }

    }
    {
    fprintf(_coverage_fout, "774\n");
    fflush(_coverage_fout);
    }
    p ++;
  }
  {
  fprintf(_coverage_fout, "781\n");
  fflush(_coverage_fout);
  }
  return ((char *)((void *)0));
}
}
__inline static char *zend_memnstr(char *haystack , char *needle ,
                                   int needle_len , char *end ) ;
extern  __attribute__((__nothrow__)) long strtol(char const   * __restrict  __nptr ,
                                                 char ** __restrict  __endptr ,
                                                 int __base )  __attribute__((__nonnull__(1))) ;
extern double __attribute__((__visibility__("default")))  zend_strtod(char const   *s00 ,
                                                                      char const   **se ) ;
static char const   long_min_digits[11] ;
extern double __attribute__((__visibility__("default")))  zend_hex_strtod(char const   *str ,
                                                                          char const   **endptr ) ;
__inline static zend_uchar is_numeric_string(char const   *str , int length ,
                                             long *lval , double *dval ,
                                             int allow_errors ) 
{ char const   *ptr ;
  int base ;
  int digits ;
  int dp_or_e ;
  double local_dval ;
  zend_uchar type ;
  char const   *e ;
  char const   *tmp ;
  double __attribute__((__visibility__("default")))  tmp___0 ;
  double __attribute__((__visibility__("default")))  tmp___1 ;
  char const   *tmp___2 ;
  char const   *tmp___3 ;
  int cmp ;
  size_t __s1_len ;
  size_t __s2_len ;
  int tmp___4 ;
  int tmp___5 ;
  int tmp___6 ;
  unsigned char const   *__s2 ;
  register int __result ;
  int tmp___7 ;
  unsigned char const   *__s1 ;
  register int __result___0 ;
  int tmp___8 ;
  int tmp___9 ;
  int tmp___10 ;
  double __attribute__((__visibility__("default")))  tmp___11 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "744\n");
  fflush(_coverage_fout);
  }
  base = 10;
  {
  fprintf(_coverage_fout, "745\n");
  fflush(_coverage_fout);
  }
  digits = 0;
  {
  fprintf(_coverage_fout, "746\n");
  fflush(_coverage_fout);
  }
  dp_or_e = 0;
  {
  fprintf(_coverage_fout, "747\n");
  fflush(_coverage_fout);
  }
  if (! length) {
    {
    fprintf(_coverage_fout, "590\n");
    fflush(_coverage_fout);
    }
    return ((unsigned char)0);
  } else {
    {
    fprintf(_coverage_fout, "591\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "748\n");
  fflush(_coverage_fout);
  }
  while (1) {
    {
    fprintf(_coverage_fout, "603\n");
    fflush(_coverage_fout);
    }
    if ((int const   )*str == 32) {
      {
      fprintf(_coverage_fout, "592\n");
      fflush(_coverage_fout);
      }

    } else {
      {
      fprintf(_coverage_fout, "602\n");
      fflush(_coverage_fout);
      }
      if ((int const   )*str == 9) {
        {
        fprintf(_coverage_fout, "593\n");
        fflush(_coverage_fout);
        }

      } else {
        {
        fprintf(_coverage_fout, "601\n");
        fflush(_coverage_fout);
        }
        if ((int const   )*str == 10) {
          {
          fprintf(_coverage_fout, "594\n");
          fflush(_coverage_fout);
          }

        } else {
          {
          fprintf(_coverage_fout, "600\n");
          fflush(_coverage_fout);
          }
          if ((int const   )*str == 13) {
            {
            fprintf(_coverage_fout, "595\n");
            fflush(_coverage_fout);
            }

          } else {
            {
            fprintf(_coverage_fout, "599\n");
            fflush(_coverage_fout);
            }
            if ((int const   )*str == 11) {
              {
              fprintf(_coverage_fout, "596\n");
              fflush(_coverage_fout);
              }

            } else {
              {
              fprintf(_coverage_fout, "598\n");
              fflush(_coverage_fout);
              }
              if ((int const   )*str == 12) {
                {
                fprintf(_coverage_fout, "597\n");
                fflush(_coverage_fout);
                }

              } else {
                break;
              }
            }
          }
        }
      }
    }
    {
    fprintf(_coverage_fout, "604\n");
    fflush(_coverage_fout);
    }
    str ++;
    {
    fprintf(_coverage_fout, "605\n");
    fflush(_coverage_fout);
    }
    length --;
  }
  {
  fprintf(_coverage_fout, "749\n");
  fflush(_coverage_fout);
  }
  ptr = str;
  {
  fprintf(_coverage_fout, "750\n");
  fflush(_coverage_fout);
  }
  if ((int const   )*ptr == 45) {
    {
    fprintf(_coverage_fout, "606\n");
    fflush(_coverage_fout);
    }
    ptr ++;
  } else {
    {
    fprintf(_coverage_fout, "609\n");
    fflush(_coverage_fout);
    }
    if ((int const   )*ptr == 43) {
      {
      fprintf(_coverage_fout, "607\n");
      fflush(_coverage_fout);
      }
      ptr ++;
    } else {
      {
      fprintf(_coverage_fout, "608\n");
      fflush(_coverage_fout);
      }

    }
  }
  {
  fprintf(_coverage_fout, "751\n");
  fflush(_coverage_fout);
  }
  if ((int const   )*ptr >= 48) {
    {
    fprintf(_coverage_fout, "678\n");
    fflush(_coverage_fout);
    }
    if ((int const   )*ptr <= 57) {
      {
      fprintf(_coverage_fout, "673\n");
      fflush(_coverage_fout);
      }
      if (length > 2) {
        {
        fprintf(_coverage_fout, "618\n");
        fflush(_coverage_fout);
        }
        if ((int const   )*str == 48) {
          {
          fprintf(_coverage_fout, "616\n");
          fflush(_coverage_fout);
          }
          if ((int const   )*(str + 1) == 120) {
            {
            fprintf(_coverage_fout, "610\n");
            fflush(_coverage_fout);
            }
            base = 16;
            {
            fprintf(_coverage_fout, "611\n");
            fflush(_coverage_fout);
            }
            ptr += 2;
          } else {
            {
            fprintf(_coverage_fout, "615\n");
            fflush(_coverage_fout);
            }
            if ((int const   )*(str + 1) == 88) {
              {
              fprintf(_coverage_fout, "612\n");
              fflush(_coverage_fout);
              }
              base = 16;
              {
              fprintf(_coverage_fout, "613\n");
              fflush(_coverage_fout);
              }
              ptr += 2;
            } else {
              {
              fprintf(_coverage_fout, "614\n");
              fflush(_coverage_fout);
              }

            }
          }
        } else {
          {
          fprintf(_coverage_fout, "617\n");
          fflush(_coverage_fout);
          }

        }
      } else {
        {
        fprintf(_coverage_fout, "619\n");
        fflush(_coverage_fout);
        }

      }
      {
      fprintf(_coverage_fout, "674\n");
      fflush(_coverage_fout);
      }
      while (1) {
        {
        fprintf(_coverage_fout, "621\n");
        fflush(_coverage_fout);
        }
        if ((int const   )*ptr == 48) {
          {
          fprintf(_coverage_fout, "620\n");
          fflush(_coverage_fout);
          }

        } else {
          break;
        }
        {
        fprintf(_coverage_fout, "622\n");
        fflush(_coverage_fout);
        }
        ptr ++;
      }
      {
      fprintf(_coverage_fout, "675\n");
      fflush(_coverage_fout);
      }
      type = (unsigned char)1;
      {
      fprintf(_coverage_fout, "676\n");
      fflush(_coverage_fout);
      }
      while (1) {
        {
        fprintf(_coverage_fout, "656\n");
        fflush(_coverage_fout);
        }
        if (digits >= 11) {
          {
          fprintf(_coverage_fout, "625\n");
          fflush(_coverage_fout);
          }
          if (dval) {
            break;
          } else {
            {
            fprintf(_coverage_fout, "624\n");
            fflush(_coverage_fout);
            }
            if (allow_errors == 1) {
              break;
            } else {
              {
              fprintf(_coverage_fout, "623\n");
              fflush(_coverage_fout);
              }

            }
          }
        } else {
          {
          fprintf(_coverage_fout, "626\n");
          fflush(_coverage_fout);
          }

        }
        {
        fprintf(_coverage_fout, "657\n");
        fflush(_coverage_fout);
        }
        check_digits: 
        if ((int const   )*ptr >= 48) {
          {
          fprintf(_coverage_fout, "627\n");
          fflush(_coverage_fout);
          }
          if ((int const   )*ptr <= 57) {
            goto __Cont;
          } else {
            goto _L___3;
          }
        } else {
          {
          fprintf(_coverage_fout, "655\n");
          fflush(_coverage_fout);
          }
          _L___3: /* CIL Label */ 
          if (base == 16) {
            {
            fprintf(_coverage_fout, "631\n");
            fflush(_coverage_fout);
            }
            if ((int const   )*ptr >= 65) {
              {
              fprintf(_coverage_fout, "628\n");
              fflush(_coverage_fout);
              }
              if ((int const   )*ptr <= 70) {
                goto __Cont;
              } else {
                goto _L___2;
              }
            } else {
              {
              fprintf(_coverage_fout, "630\n");
              fflush(_coverage_fout);
              }
              _L___2: /* CIL Label */ 
              if ((int const   )*ptr >= 97) {
                {
                fprintf(_coverage_fout, "629\n");
                fflush(_coverage_fout);
                }
                if ((int const   )*ptr <= 102) {
                  goto __Cont;
                } else {
                  goto _L___1;
                }
              } else {
                goto _L___1;
              }
            }
          } else {
            {
            fprintf(_coverage_fout, "654\n");
            fflush(_coverage_fout);
            }
            _L___1: /* CIL Label */ 
            if (base == 10) {
              {
              fprintf(_coverage_fout, "652\n");
              fflush(_coverage_fout);
              }
              if ((int const   )*ptr == 46) {
                {
                fprintf(_coverage_fout, "632\n");
                fflush(_coverage_fout);
                }
                if (dp_or_e < 1) {
                  goto process_double;
                } else {
                  goto _L___0;
                }
              } else {
                {
                fprintf(_coverage_fout, "651\n");
                fflush(_coverage_fout);
                }
                _L___0: /* CIL Label */ 
                if ((int const   )*ptr == 101) {
                  goto _L;
                } else {
                  {
                  fprintf(_coverage_fout, "650\n");
                  fflush(_coverage_fout);
                  }
                  if ((int const   )*ptr == 69) {
                    {
                    fprintf(_coverage_fout, "648\n");
                    fflush(_coverage_fout);
                    }
                    _L: /* CIL Label */ 
                    if (dp_or_e < 2) {
                      {
                      fprintf(_coverage_fout, "644\n");
                      fflush(_coverage_fout);
                      }
                      e = ptr + 1;
                      {
                      fprintf(_coverage_fout, "645\n");
                      fflush(_coverage_fout);
                      }
                      if ((int const   )*e == 45) {
                        {
                        fprintf(_coverage_fout, "633\n");
                        fflush(_coverage_fout);
                        }
                        tmp = e;
                        {
                        fprintf(_coverage_fout, "634\n");
                        fflush(_coverage_fout);
                        }
                        e ++;
                        {
                        fprintf(_coverage_fout, "635\n");
                        fflush(_coverage_fout);
                        }
                        ptr = tmp;
                      } else {
                        {
                        fprintf(_coverage_fout, "640\n");
                        fflush(_coverage_fout);
                        }
                        if ((int const   )*e == 43) {
                          {
                          fprintf(_coverage_fout, "636\n");
                          fflush(_coverage_fout);
                          }
                          tmp = e;
                          {
                          fprintf(_coverage_fout, "637\n");
                          fflush(_coverage_fout);
                          }
                          e ++;
                          {
                          fprintf(_coverage_fout, "638\n");
                          fflush(_coverage_fout);
                          }
                          ptr = tmp;
                        } else {
                          {
                          fprintf(_coverage_fout, "639\n");
                          fflush(_coverage_fout);
                          }

                        }
                      }
                      {
                      fprintf(_coverage_fout, "646\n");
                      fflush(_coverage_fout);
                      }
                      if ((int const   )*e >= 48) {
                        {
                        fprintf(_coverage_fout, "642\n");
                        fflush(_coverage_fout);
                        }
                        if ((int const   )*e <= 57) {
                          goto process_double;
                        } else {
                          {
                          fprintf(_coverage_fout, "641\n");
                          fflush(_coverage_fout);
                          }

                        }
                      } else {
                        {
                        fprintf(_coverage_fout, "643\n");
                        fflush(_coverage_fout);
                        }

                      }
                    } else {
                      {
                      fprintf(_coverage_fout, "647\n");
                      fflush(_coverage_fout);
                      }

                    }
                  } else {
                    {
                    fprintf(_coverage_fout, "649\n");
                    fflush(_coverage_fout);
                    }

                  }
                }
              }
            } else {
              {
              fprintf(_coverage_fout, "653\n");
              fflush(_coverage_fout);
              }

            }
          }
        }
        break;
        {
        fprintf(_coverage_fout, "658\n");
        fflush(_coverage_fout);
        }
        __Cont: /* CIL Label */ 
        digits ++;
        {
        fprintf(_coverage_fout, "659\n");
        fflush(_coverage_fout);
        }
        ptr ++;
      }
      {
      fprintf(_coverage_fout, "677\n");
      fflush(_coverage_fout);
      }
      if (base == 10) {
        {
        fprintf(_coverage_fout, "662\n");
        fflush(_coverage_fout);
        }
        if (digits >= 11) {
          {
          fprintf(_coverage_fout, "660\n");
          fflush(_coverage_fout);
          }
          dp_or_e = -1;
          goto process_double;
        } else {
          {
          fprintf(_coverage_fout, "661\n");
          fflush(_coverage_fout);
          }

        }
      } else {
        {
        fprintf(_coverage_fout, "672\n");
        fflush(_coverage_fout);
        }
        if (digits < 8) {
          {
          fprintf(_coverage_fout, "663\n");
          fflush(_coverage_fout);
          }

        } else {
          {
          fprintf(_coverage_fout, "671\n");
          fflush(_coverage_fout);
          }
          if (digits == 8) {
            {
            fprintf(_coverage_fout, "665\n");
            fflush(_coverage_fout);
            }
            if ((int const   )*(ptr + - digits) <= 55) {
              {
              fprintf(_coverage_fout, "664\n");
              fflush(_coverage_fout);
              }

            } else {
              goto _L___4;
            }
          } else {
            {
            fprintf(_coverage_fout, "669\n");
            fflush(_coverage_fout);
            }
            _L___4: /* CIL Label */ 
            if (dval) {
              {
              fprintf(_coverage_fout, "666\n");
              fflush(_coverage_fout);
              }
              tmp___0 = zend_hex_strtod(str, & ptr);
              {
              fprintf(_coverage_fout, "667\n");
              fflush(_coverage_fout);
              }
              local_dval = (double )tmp___0;
            } else {
              {
              fprintf(_coverage_fout, "668\n");
              fflush(_coverage_fout);
              }

            }
            {
            fprintf(_coverage_fout, "670\n");
            fflush(_coverage_fout);
            }
            type = (unsigned char)2;
          }
        }
      }
    } else {
      goto _L___5;
    }
  } else {
    {
    fprintf(_coverage_fout, "697\n");
    fflush(_coverage_fout);
    }
    _L___5: /* CIL Label */ 
    if ((int const   )*ptr == 46) {
      {
      fprintf(_coverage_fout, "695\n");
      fflush(_coverage_fout);
      }
      if ((int const   )*(ptr + 1) >= 48) {
        {
        fprintf(_coverage_fout, "693\n");
        fflush(_coverage_fout);
        }
        if ((int const   )*(ptr + 1) <= 57) {
          {
          fprintf(_coverage_fout, "690\n");
          fflush(_coverage_fout);
          }
          process_double: 
          type = (unsigned char)2;
          {
          fprintf(_coverage_fout, "691\n");
          fflush(_coverage_fout);
          }
          if (dval) {
            {
            fprintf(_coverage_fout, "679\n");
            fflush(_coverage_fout);
            }
            tmp___1 = zend_strtod(str, & ptr);
            {
            fprintf(_coverage_fout, "680\n");
            fflush(_coverage_fout);
            }
            local_dval = (double )tmp___1;
          } else {
            {
            fprintf(_coverage_fout, "689\n");
            fflush(_coverage_fout);
            }
            if (allow_errors != 1) {
              {
              fprintf(_coverage_fout, "687\n");
              fflush(_coverage_fout);
              }
              if (dp_or_e != -1) {
                {
                fprintf(_coverage_fout, "683\n");
                fflush(_coverage_fout);
                }
                tmp___3 = ptr;
                {
                fprintf(_coverage_fout, "684\n");
                fflush(_coverage_fout);
                }
                ptr ++;
                {
                fprintf(_coverage_fout, "685\n");
                fflush(_coverage_fout);
                }
                if ((int const   )*tmp___3 == 46) {
                  {
                  fprintf(_coverage_fout, "681\n");
                  fflush(_coverage_fout);
                  }
                  dp_or_e = 1;
                } else {
                  {
                  fprintf(_coverage_fout, "682\n");
                  fflush(_coverage_fout);
                  }
                  dp_or_e = 2;
                }
                goto check_digits;
              } else {
                {
                fprintf(_coverage_fout, "686\n");
                fflush(_coverage_fout);
                }

              }
            } else {
              {
              fprintf(_coverage_fout, "688\n");
              fflush(_coverage_fout);
              }

            }
          }
        } else {
          {
          fprintf(_coverage_fout, "692\n");
          fflush(_coverage_fout);
          }
          return ((unsigned char)0);
        }
      } else {
        {
        fprintf(_coverage_fout, "694\n");
        fflush(_coverage_fout);
        }
        return ((unsigned char)0);
      }
    } else {
      {
      fprintf(_coverage_fout, "696\n");
      fflush(_coverage_fout);
      }
      return ((unsigned char)0);
    }
  }
  {
  fprintf(_coverage_fout, "752\n");
  fflush(_coverage_fout);
  }
  if ((unsigned int )ptr != (unsigned int )(str + length)) {
    {
    fprintf(_coverage_fout, "702\n");
    fflush(_coverage_fout);
    }
    if (! allow_errors) {
      {
      fprintf(_coverage_fout, "698\n");
      fflush(_coverage_fout);
      }
      return ((unsigned char)0);
    } else {
      {
      fprintf(_coverage_fout, "699\n");
      fflush(_coverage_fout);
      }

    }
    {
    fprintf(_coverage_fout, "703\n");
    fflush(_coverage_fout);
    }
    if (allow_errors == -1) {
      {
      fprintf(_coverage_fout, "700\n");
      fflush(_coverage_fout);
      }
      zend_error(1 << 3L, "A non well formed numeric value encountered");
    } else {
      {
      fprintf(_coverage_fout, "701\n");
      fflush(_coverage_fout);
      }

    }
  } else {
    {
    fprintf(_coverage_fout, "704\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "753\n");
  fflush(_coverage_fout);
  }
  if ((int )type == 1) {
    {
    fprintf(_coverage_fout, "737\n");
    fflush(_coverage_fout);
    }
    if (digits == 10) {
      {
      fprintf(_coverage_fout, "731\n");
      fflush(_coverage_fout);
      }
      if (0) {
        {
        fprintf(_coverage_fout, "716\n");
        fflush(_coverage_fout);
        }
        __s1_len = strlen(ptr + - digits);
        {
        fprintf(_coverage_fout, "717\n");
        fflush(_coverage_fout);
        }
        __s2_len = strlen(long_min_digits);
        {
        fprintf(_coverage_fout, "718\n");
        fflush(_coverage_fout);
        }
        if (! ((unsigned int )((void const   *)((ptr + - digits) + 1)) - (unsigned int )((void const   *)(ptr + - digits)) == 1U)) {
          goto _L___7;
        } else {
          {
          fprintf(_coverage_fout, "711\n");
          fflush(_coverage_fout);
          }
          if (__s1_len >= 4U) {
            {
            fprintf(_coverage_fout, "709\n");
            fflush(_coverage_fout);
            }
            _L___7: /* CIL Label */ 
            if (! ((unsigned int )((void const   *)(long_min_digits + 1)) - (unsigned int )((void const   *)(long_min_digits)) == 1U)) {
              {
              fprintf(_coverage_fout, "705\n");
              fflush(_coverage_fout);
              }
              tmp___10 = 1;
            } else {
              {
              fprintf(_coverage_fout, "708\n");
              fflush(_coverage_fout);
              }
              if (__s2_len >= 4U) {
                {
                fprintf(_coverage_fout, "706\n");
                fflush(_coverage_fout);
                }
                tmp___10 = 1;
              } else {
                {
                fprintf(_coverage_fout, "707\n");
                fflush(_coverage_fout);
                }
                tmp___10 = 0;
              }
            }
          } else {
            {
            fprintf(_coverage_fout, "710\n");
            fflush(_coverage_fout);
            }
            tmp___10 = 0;
          }
        }
        {
        fprintf(_coverage_fout, "719\n");
        fflush(_coverage_fout);
        }
        if (tmp___10) {
          {
          fprintf(_coverage_fout, "712\n");
          fflush(_coverage_fout);
          }
          tmp___5 = __builtin_strcmp(ptr + - digits, long_min_digits);
          {
          fprintf(_coverage_fout, "713\n");
          fflush(_coverage_fout);
          }
          tmp___9 = tmp___5;
        } else {
          {
          fprintf(_coverage_fout, "714\n");
          fflush(_coverage_fout);
          }
          tmp___8 = __builtin_strcmp(ptr + - digits, long_min_digits);
          {
          fprintf(_coverage_fout, "715\n");
          fflush(_coverage_fout);
          }
          tmp___9 = tmp___8;
        }
      } else {
        {
        fprintf(_coverage_fout, "720\n");
        fflush(_coverage_fout);
        }
        tmp___8 = __builtin_strcmp(ptr + - digits, long_min_digits);
        {
        fprintf(_coverage_fout, "721\n");
        fflush(_coverage_fout);
        }
        tmp___9 = tmp___8;
      }
      {
      fprintf(_coverage_fout, "732\n");
      fflush(_coverage_fout);
      }
      cmp = tmp___9;
      {
      fprintf(_coverage_fout, "733\n");
      fflush(_coverage_fout);
      }
      if (cmp < 0) {
        {
        fprintf(_coverage_fout, "722\n");
        fflush(_coverage_fout);
        }

      } else {
        {
        fprintf(_coverage_fout, "730\n");
        fflush(_coverage_fout);
        }
        if (cmp == 0) {
          {
          fprintf(_coverage_fout, "724\n");
          fflush(_coverage_fout);
          }
          if ((int const   )*str == 45) {
            {
            fprintf(_coverage_fout, "723\n");
            fflush(_coverage_fout);
            }

          } else {
            goto _L___8;
          }
        } else {
          {
          fprintf(_coverage_fout, "728\n");
          fflush(_coverage_fout);
          }
          _L___8: /* CIL Label */ 
          if (dval) {
            {
            fprintf(_coverage_fout, "725\n");
            fflush(_coverage_fout);
            }
            tmp___11 = zend_strtod(str, (char const   **)((void *)0));
            {
            fprintf(_coverage_fout, "726\n");
            fflush(_coverage_fout);
            }
            *dval = (double )tmp___11;
          } else {
            {
            fprintf(_coverage_fout, "727\n");
            fflush(_coverage_fout);
            }

          }
          {
          fprintf(_coverage_fout, "729\n");
          fflush(_coverage_fout);
          }
          return ((unsigned char)2);
        }
      }
    } else {
      {
      fprintf(_coverage_fout, "734\n");
      fflush(_coverage_fout);
      }

    }
    {
    fprintf(_coverage_fout, "738\n");
    fflush(_coverage_fout);
    }
    if (lval) {
      {
      fprintf(_coverage_fout, "735\n");
      fflush(_coverage_fout);
      }
      *lval = strtol((char const   */* __restrict  */)str,
                     (char **/* __restrict  */)((void *)0), base);
    } else {
      {
      fprintf(_coverage_fout, "736\n");
      fflush(_coverage_fout);
      }

    }
    {
    fprintf(_coverage_fout, "739\n");
    fflush(_coverage_fout);
    }
    return ((unsigned char)1);
  } else {
    {
    fprintf(_coverage_fout, "742\n");
    fflush(_coverage_fout);
    }
    if (dval) {
      {
      fprintf(_coverage_fout, "740\n");
      fflush(_coverage_fout);
      }
      *dval = local_dval;
    } else {
      {
      fprintf(_coverage_fout, "741\n");
      fflush(_coverage_fout);
      }

    }
    {
    fprintf(_coverage_fout, "743\n");
    fflush(_coverage_fout);
    }
    return ((unsigned char)2);
  }
}
}
__inline static zend_uchar is_numeric_string(char const   *str , int length ,
                                             long *lval , double *dval ,
                                             int allow_errors ) ;
__inline static long ( __attribute__((__always_inline__)) zend_dval_to_lval)(double d ) 
{ 

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "588\n");
  fflush(_coverage_fout);
  }
  if (d > (double )2147483647L) {
    {
    fprintf(_coverage_fout, "584\n");
    fflush(_coverage_fout);
    }
    return ((long )((unsigned long )((long long )d)));
  } else {
    {
    fprintf(_coverage_fout, "587\n");
    fflush(_coverage_fout);
    }
    if (d < (double )(-0x7FFFFFFF-1)) {
      {
      fprintf(_coverage_fout, "585\n");
      fflush(_coverage_fout);
      }
      return ((long )((unsigned long )((long long )d)));
    } else {
      {
      fprintf(_coverage_fout, "586\n");
      fflush(_coverage_fout);
      }

    }
  }
  {
  fprintf(_coverage_fout, "589\n");
  fflush(_coverage_fout);
  }
  return ((long )d);
}
}
__inline static long ( __attribute__((__always_inline__)) zend_dval_to_lval)(double d ) ;
extern zend_bool __attribute__((__visibility__("default")))  instanceof_function_ex(zend_class_entry const   *instance_ce ,
                                                                                    zend_class_entry const   *ce ,
                                                                                    zend_bool interfaces_only ) ;
extern int __attribute__((__visibility__("default")))  is_smaller_or_equal_function(zval *result ,
                                                                                    zval *op1 ,
                                                                                    zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  is_smaller_function(zval *result ,
                                                                           zval *op1 ,
                                                                           zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  is_not_equal_function(zval *result ,
                                                                             zval *op1 ,
                                                                             zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  is_not_identical_function(zval *result ,
                                                                                 zval *op1 ,
                                                                                 zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  is_identical_function(zval *result ,
                                                                             zval *op1 ,
                                                                             zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  is_equal_function(zval *result ,
                                                                         zval *op1 ,
                                                                         zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  concat_function(zval *result ,
                                                                       zval *op1 ,
                                                                       zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  shift_right_function(zval *result ,
                                                                            zval *op1 ,
                                                                            zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  shift_left_function(zval *result ,
                                                                           zval *op1 ,
                                                                           zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  bitwise_xor_function(zval *result ,
                                                                            zval *op1 ,
                                                                            zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  bitwise_and_function(zval *result ,
                                                                            zval *op1 ,
                                                                            zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  bitwise_or_function(zval *result ,
                                                                           zval *op1 ,
                                                                           zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  bitwise_not_function(zval *result ,
                                                                            zval *op1 ) ;
extern int __attribute__((__visibility__("default")))  boolean_not_function(zval *result ,
                                                                            zval *op1 ) ;
extern int __attribute__((__visibility__("default")))  boolean_xor_function(zval *result ,
                                                                            zval *op1 ,
                                                                            zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  zend_shutdown_strtod(void) ;
extern int __attribute__((__visibility__("default")))  zend_startup_strtod(void) ;
extern double __attribute__((__visibility__("default")))  zend_bin_strtod(char const   *str ,
                                                                          char const   **endptr ) ;
extern double __attribute__((__visibility__("default")))  zend_oct_strtod(char const   *str ,
                                                                          char const   **endptr ) ;
extern char __attribute__((__visibility__("default")))  *zend_dtoa(double _d ,
                                                                   int mode ,
                                                                   int ndigits ,
                                                                   int *decpt ,
                                                                   int *sign ,
                                                                   char **rve ) ;
extern void __attribute__((__visibility__("default")))  zend_freedtoa(char *s ) ;
extern  __attribute__((__nothrow__,
__noreturn__)) void __assert(char const   *__assertion , char const   *__file ,
                             int __line ) ;
extern  __attribute__((__nothrow__,
__noreturn__)) void __assert_perror_fail(int __errnum , char const   *__file ,
                                         unsigned int __line ,
                                         char const   *__function ) ;
extern  __attribute__((__nothrow__,
__noreturn__)) void __assert_fail(char const   *__assertion ,
                                  char const   *__file , unsigned int __line ,
                                  char const   *__function ) ;
extern char *program_invocation_short_name ;
extern char *program_invocation_name ;
extern  __attribute__((__nothrow__)) int *__errno_location(void)  __attribute__((__const__)) ;
struct _zend_gc_globals;
typedef struct _zend_gc_globals zend_gc_globals;
extern zend_gc_globals __attribute__((__visibility__("default")))  gc_globals ;
union __anonunion_u_45 {
   zval *pz ;
   zend_object_handlers const   *handlers ;
};
struct _gc_root_buffer {
   struct _gc_root_buffer *prev ;
   struct _gc_root_buffer *next ;
   zend_object_handle handle ;
   union __anonunion_u_45 u ;
};
struct _zend_gc_globals {
   zend_bool gc_enabled ;
   zend_bool gc_active ;
   gc_root_buffer *buf ;
   gc_root_buffer roots ;
   gc_root_buffer *unused ;
   gc_root_buffer *first_unused ;
   gc_root_buffer *last_unused ;
   zval_gc_info *zval_to_free ;
   zval_gc_info *free_list ;
   zval_gc_info *next_to_free ;
   zend_uint gc_runs ;
   zend_uint collected ;
};
__inline static void ( __attribute__((__always_inline__)) gc_remove_from_buffer)(gc_root_buffer *root ) 
{ 

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "579\n");
  fflush(_coverage_fout);
  }
  (root->next)->prev = root->prev;
  {
  fprintf(_coverage_fout, "580\n");
  fflush(_coverage_fout);
  }
  (root->prev)->next = root->next;
  {
  fprintf(_coverage_fout, "581\n");
  fflush(_coverage_fout);
  }
  root->prev = gc_globals.unused;
  {
  fprintf(_coverage_fout, "582\n");
  fflush(_coverage_fout);
  }
  gc_globals.unused = root;
  {
  fprintf(_coverage_fout, "583\n");
  fflush(_coverage_fout);
  }
  return;
}
}
__inline static void ( __attribute__((__always_inline__)) gc_remove_from_buffer)(gc_root_buffer *root ) ;
extern void __attribute__((__visibility__("default")))  gc_zval_possible_root(zval *zv ) ;
__inline static void ( __attribute__((__always_inline__)) gc_zval_check_possible_root)(zval *z ) 
{ 

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "577\n");
  fflush(_coverage_fout);
  }
  if ((int )z->type == 4) {
    {
    fprintf(_coverage_fout, "573\n");
    fflush(_coverage_fout);
    }
    gc_zval_possible_root(z);
  } else {
    {
    fprintf(_coverage_fout, "576\n");
    fflush(_coverage_fout);
    }
    if ((int )z->type == 5) {
      {
      fprintf(_coverage_fout, "574\n");
      fflush(_coverage_fout);
      }
      gc_zval_possible_root(z);
    } else {
      {
      fprintf(_coverage_fout, "575\n");
      fflush(_coverage_fout);
      }

    }
  }
  {
  fprintf(_coverage_fout, "578\n");
  fflush(_coverage_fout);
  }
  return;
}
}
extern void __attribute__((__visibility__("default")))  gc_reset(void) ;
extern void __attribute__((__visibility__("default")))  gc_init(void) ;
extern void __attribute__((__visibility__("default")))  gc_globals_dtor(void) ;
extern void __attribute__((__visibility__("default")))  gc_globals_ctor(void) ;
extern void __attribute__((__visibility__("default")))  gc_zobj_possible_root(zval *zv ) ;
extern int __attribute__((__visibility__("default")))  gc_collect_cycles(void) ;
extern int __attribute__((__visibility__("default")))  zend_get_configuration_directive(char const   *name ,
                                                                                        uint name_length ,
                                                                                        zval *contents ) ;
extern void __attribute__((__visibility__("default")))  zend_message_dispatcher(long message ,
                                                                                void const   *data ) ;
struct _zend_utility_values;
typedef struct _zend_utility_values zend_utility_values;
extern zend_utility_values __attribute__((__visibility__("default")))  zend_uv ;
extern zend_class_entry __attribute__((__visibility__("default")))  *zend_standard_class_def ;
extern void zenderror(char const   *error ) ;
extern char __attribute__((__visibility__("default")))  *(*zend_resolve_path)(char const   *filename ,
                                                                              int filename_len ) ;
extern char __attribute__((__visibility__("default")))  *(*zend_getenv)(char *name ,
                                                                        size_t name_len ) ;
extern int (*zend_vspprintf)(char **pbuf , size_t max_len ,
                             char const   *format , va_list ap ) ;
extern int __attribute__((__visibility__("default")))  (*zend_stream_open_function)(char const   *filename ,
                                                                                    zend_file_handle *handle ) ;
extern void __attribute__((__visibility__("default")))  (*zend_on_timeout)(int seconds ) ;
extern void __attribute__((__visibility__("default")))  ( /* format attribute */  (*zend_error_cb))(int type ,
                                                                                                    char const   *error_filename ,
                                                                                                    uint error_lineno ,
                                                                                                    char const   *format ,
                                                                                                    va_list args ) ;
extern void __attribute__((__visibility__("default")))  (*zend_ticks_function)(int ticks ) ;
extern void __attribute__((__visibility__("default")))  (*zend_unblock_interruptions)(void) ;
extern void __attribute__((__visibility__("default")))  (*zend_block_interruptions)(void) ;
extern FILE __attribute__((__visibility__("default")))  *(*zend_fopen)(char const   *filename ,
                                                                       char **opened_path ) ;
extern int (* __attribute__((__visibility__("default"))) zend_write)(char const   *str ,
                                                                     uint str_length ) ;
extern int __attribute__((__visibility__("default")))  ( /* format attribute */  (*zend_printf))(char const   *format 
                                                                                                 , ...) ;
extern void __attribute__((__visibility__("default")))  free_estring(char **str_p ) ;
extern void zend_post_deactivate_modules(void) ;
extern void zend_deactivate_modules(void) ;
extern void zend_activate_modules(void) ;
extern void zend_call_destructors(void) ;
extern void zend_deactivate(void) ;
extern void zend_activate(void) ;
extern void __attribute__((__visibility__("default")))  ( /* format attribute */  zend_output_debug_string)(zend_bool trigger_break ,
                                                                                                            char const   *format 
                                                                                                            , ...) ;
extern void __attribute__((__visibility__("default")))  zend_print_zval_r_ex(int (*write_func)(char const   *str ,
                                                                                               uint str_length ) ,
                                                                             zval *expr ,
                                                                             int indent ) ;
extern void __attribute__((__visibility__("default")))  zend_print_flat_zval_r(zval *expr ) ;
extern void __attribute__((__visibility__("default")))  zend_print_zval_r(zval *expr ,
                                                                          int indent ) ;
extern int __attribute__((__visibility__("default")))  zend_print_zval_ex(int (*write_func)(char const   *str ,
                                                                                            uint str_length ) ,
                                                                          zval *expr ,
                                                                          int indent ) ;
extern int __attribute__((__visibility__("default")))  zend_print_zval(zval *expr ,
                                                                       int indent ) ;
extern void __attribute__((__visibility__("default")))  zend_make_printable_zval(zval *expr ,
                                                                                 zval *expr_copy ,
                                                                                 int *use_copy ) ;
extern char __attribute__((__visibility__("default")))  *get_zend_version(void) ;
extern void __attribute__((__visibility__("default")))  _zend_bailout(char *filename ,
                                                                      uint lineno ) ;
extern void zend_set_utility_values(zend_utility_values *utility_values ) ;
extern void zend_post_startup(void) ;
extern void zend_register_standard_ini_entries(void) ;
extern void zend_shutdown(void) ;
struct _zend_utility_functions;
typedef struct _zend_utility_functions zend_utility_functions;
extern int zend_startup(zend_utility_functions *utility_functions ,
                        char **extensions ) ;
extern int __attribute__((__visibility__("default")))  zend_compare_file_handles(zend_file_handle *fh1 ,
                                                                                 zend_file_handle *fh2 ) ;
extern void __attribute__((__visibility__("default")))  zend_file_handle_dtor(zend_file_handle *fh ) ;
extern int __attribute__((__visibility__("default")))  zend_stream_fixup(zend_file_handle *file_handle ,
                                                                         char **buf ,
                                                                         size_t *len ) ;
extern int __attribute__((__visibility__("default")))  zend_stream_open(char const   *filename ,
                                                                        zend_file_handle *handle ) ;
extern void __attribute__((__visibility__("default")))  zend_register_iterator_wrapper(void) ;
extern zval __attribute__((__visibility__("default")))  *zend_iterator_wrap(zend_object_iterator *iter ) ;
enum zend_object_iterator_kind;
extern enum zend_object_iterator_kind  __attribute__((__visibility__("default"))) zend_iterator_unwrap(zval *array_ptr ,
                                                                                                       zend_object_iterator **iter ) ;
__inline static zend_bool ( __attribute__((__always_inline__)) zval_set_isref_to_p)(zval *pz ,
                                                                                    zend_bool isref ) 
{ zend_uchar tmp ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "570\n");
  fflush(_coverage_fout);
  }
  tmp = isref;
  {
  fprintf(_coverage_fout, "571\n");
  fflush(_coverage_fout);
  }
  pz->is_ref__gc = tmp;
  {
  fprintf(_coverage_fout, "572\n");
  fflush(_coverage_fout);
  }
  return (tmp);
}
}
__inline static zend_bool ( __attribute__((__always_inline__)) zval_unset_isref_p)(zval *pz ) 
{ zend_uchar tmp ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "567\n");
  fflush(_coverage_fout);
  }
  tmp = (unsigned char)0;
  {
  fprintf(_coverage_fout, "568\n");
  fflush(_coverage_fout);
  }
  pz->is_ref__gc = tmp;
  {
  fprintf(_coverage_fout, "569\n");
  fflush(_coverage_fout);
  }
  return (tmp);
}
}
__inline static zend_bool ( __attribute__((__always_inline__)) zval_set_isref_p)(zval *pz ) 
{ zend_uchar tmp ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "564\n");
  fflush(_coverage_fout);
  }
  tmp = (unsigned char)1;
  {
  fprintf(_coverage_fout, "565\n");
  fflush(_coverage_fout);
  }
  pz->is_ref__gc = tmp;
  {
  fprintf(_coverage_fout, "566\n");
  fflush(_coverage_fout);
  }
  return (tmp);
}
}
__inline static zend_bool ( __attribute__((__always_inline__)) zval_isref_p)(zval *pz ) 
{ 

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "563\n");
  fflush(_coverage_fout);
  }
  return (pz->is_ref__gc);
}
}
__inline static zend_uint ( __attribute__((__always_inline__)) zval_delref_p)(zval *pz ) 
{ 

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "561\n");
  fflush(_coverage_fout);
  }
  (pz->refcount__gc) --;
  {
  fprintf(_coverage_fout, "562\n");
  fflush(_coverage_fout);
  }
  return (pz->refcount__gc);
}
}
__inline static zend_uint ( __attribute__((__always_inline__)) zval_addref_p)(zval *pz ) 
{ 

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "559\n");
  fflush(_coverage_fout);
  }
  (pz->refcount__gc) ++;
  {
  fprintf(_coverage_fout, "560\n");
  fflush(_coverage_fout);
  }
  return (pz->refcount__gc);
}
}
__inline static zend_uint ( __attribute__((__always_inline__)) zval_set_refcount_p)(zval *pz ,
                                                                                    zend_uint rc ) 
{ zend_uint tmp ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "556\n");
  fflush(_coverage_fout);
  }
  tmp = rc;
  {
  fprintf(_coverage_fout, "557\n");
  fflush(_coverage_fout);
  }
  pz->refcount__gc = tmp;
  {
  fprintf(_coverage_fout, "558\n");
  fflush(_coverage_fout);
  }
  return (tmp);
}
}
__inline static zend_uint ( __attribute__((__always_inline__)) zval_refcount_p)(zval *pz ) 
{ 

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "555\n");
  fflush(_coverage_fout);
  }
  return (pz->refcount__gc);
}
}
extern void __attribute__((__visibility__("default")))  zend_std_call_user_call(int ht ,
                                                                                zval *return_value ,
                                                                                zval **return_value_ptr ,
                                                                                zval *this_ptr ,
                                                                                int return_value_used ) ;
extern int __attribute__((__visibility__("default")))  zend_check_property_access(zend_object *zobj ,
                                                                                  char const   *prop_info_name ,
                                                                                  int prop_info_name_len ) ;
extern int __attribute__((__visibility__("default")))  zend_check_protected(zend_class_entry *ce ,
                                                                            zend_class_entry *scope ) ;
extern int __attribute__((__visibility__("default")))  zend_check_private(union _zend_function *fbc ,
                                                                          zend_class_entry *ce ,
                                                                          char *function_name_strval ,
                                                                          int function_name_strlen ) ;
extern void __attribute__((__visibility__("default")))  zend_std_write_property(zval *object ,
                                                                                zval *member ,
                                                                                zval *value ,
                                                                                struct _zend_literal  const  *key ) ;
extern int __attribute__((__visibility__("default")))  zend_std_cast_object_tostring(zval *readobj ,
                                                                                     zval *writeobj ,
                                                                                     int type ) ;
extern HashTable __attribute__((__visibility__("default")))  *zend_std_get_debug_info(zval *object ,
                                                                                      int *is_temp ) ;
extern HashTable __attribute__((__visibility__("default")))  *zend_std_get_properties(zval *object ) ;
extern union _zend_function  __attribute__((__visibility__("default"))) *zend_std_get_constructor(zval *object ) ;
extern zend_bool __attribute__((__visibility__("default")))  zend_std_unset_static_property(zend_class_entry *ce ,
                                                                                            char const   *property_name ,
                                                                                            int property_name_len ,
                                                                                            struct _zend_literal  const  *key ) ;
extern zval __attribute__((__visibility__("default")))  **zend_std_get_static_property(zend_class_entry *ce ,
                                                                                       char const   *property_name ,
                                                                                       int property_name_len ,
                                                                                       zend_bool silent ,
                                                                                       struct _zend_literal  const  *key ) ;
extern union _zend_function  __attribute__((__visibility__("default"))) *zend_std_get_static_method(zend_class_entry *ce ,
                                                                                                    char const   *function_name_strval ,
                                                                                                    int function_name_strlen ,
                                                                                                    struct _zend_literal  const  *key ) ;
extern  __attribute__((__noreturn__)) void zend_error_noreturn(int type ,
                                                               char const   *format 
                                                               , ...) ;
struct _zend_llist_element;
typedef struct _zend_llist_element zend_llist_element;
typedef zend_llist_element *zend_llist_position;
extern void __attribute__((__visibility__("default")))  *zend_llist_get_prev_ex(zend_llist *l ,
                                                                                zend_llist_position *pos ) ;
extern void __attribute__((__visibility__("default")))  *zend_llist_get_next_ex(zend_llist *l ,
                                                                                zend_llist_position *pos ) ;
extern void __attribute__((__visibility__("default")))  *zend_llist_get_last_ex(zend_llist *l ,
                                                                                zend_llist_position *pos ) ;
extern void __attribute__((__visibility__("default")))  *zend_llist_get_first_ex(zend_llist *l ,
                                                                                 zend_llist_position *pos ) ;
extern void __attribute__((__visibility__("default")))  zend_llist_sort(zend_llist *l ,
                                                                        int (*comp_func)(zend_llist_element const   ** ,
                                                                                         zend_llist_element const   ** ) ) ;
extern int __attribute__((__visibility__("default")))  zend_llist_count(zend_llist *l ) ;
extern void __attribute__((__visibility__("default")))  zend_llist_apply_with_arguments(zend_llist *l ,
                                                                                        void (*func)(void *data ,
                                                                                                     int num_args ,
                                                                                                     va_list args ) ,
                                                                                        int num_args 
                                                                                        , ...) ;
extern void __attribute__((__visibility__("default")))  zend_llist_apply_with_argument(zend_llist *l ,
                                                                                       void (*func)(void *data ,
                                                                                                    void *arg ) ,
                                                                                       void *arg ) ;
extern void __attribute__((__visibility__("default")))  zend_llist_apply_with_del(zend_llist *l ,
                                                                                  int (*func)(void *data ) ) ;
extern void __attribute__((__visibility__("default")))  zend_llist_apply(zend_llist *l ,
                                                                         void (*func)(void * ) ) ;
extern void __attribute__((__visibility__("default")))  zend_llist_copy(zend_llist *dst ,
                                                                        zend_llist *src ) ;
extern void __attribute__((__visibility__("default")))  *zend_llist_remove_tail(zend_llist *l ) ;
extern void __attribute__((__visibility__("default")))  zend_llist_clean(zend_llist *l ) ;
extern void __attribute__((__visibility__("default")))  zend_llist_destroy(zend_llist *l ) ;
extern void __attribute__((__visibility__("default")))  zend_llist_del_element(zend_llist *l ,
                                                                               void *element ,
                                                                               int (*compare)(void *element1 ,
                                                                                              void *element2 ) ) ;
extern void __attribute__((__visibility__("default")))  zend_llist_prepend_element(zend_llist *l ,
                                                                                   void *element ) ;
extern void __attribute__((__visibility__("default")))  zend_llist_add_element(zend_llist *l ,
                                                                               void *element ) ;
extern void __attribute__((__visibility__("default")))  zend_llist_init(zend_llist *l ,
                                                                        size_t size ,
                                                                        void (*dtor)(void * ) ,
                                                                        unsigned char persistent ) ;
extern ulong __attribute__((__visibility__("default")))  zend_ts_hash_func(char *arKey ,
                                                                           uint nKeyLength ) ;
struct _zend_ts_hashtable;
typedef struct _zend_ts_hashtable TsHashTable;
extern int __attribute__((__visibility__("default")))  zend_ts_hash_rehash(TsHashTable *ht ) ;
extern int __attribute__((__visibility__("default")))  zend_ts_hash_num_elements(TsHashTable *ht ) ;
extern int __attribute__((__visibility__("default")))  zend_ts_hash_minmax(TsHashTable *ht ,
                                                                           int (*compar)(void const   * ,
                                                                                         void const   * ) ,
                                                                           int flag ,
                                                                           void **pData ) ;
extern int __attribute__((__visibility__("default")))  zend_ts_hash_compare(TsHashTable *ht1 ,
                                                                            TsHashTable *ht2 ,
                                                                            int (*compar)(void const   * ,
                                                                                          void const   * ) ,
                                                                            zend_bool ordered ) ;
extern int __attribute__((__visibility__("default")))  zend_ts_hash_sort(TsHashTable *ht ,
                                                                         void (*sort_func)(void * ,
                                                                                           size_t  ,
                                                                                           size_t  ,
                                                                                           int (*)(void const   * ,
                                                                                                   void const   * ) ) ,
                                                                         int (*compare_func)(void const   * ,
                                                                                             void const   * ) ,
                                                                         int renumber ) ;
extern void __attribute__((__visibility__("default")))  zend_ts_hash_merge_ex(TsHashTable *target ,
                                                                              TsHashTable *source ,
                                                                              void (*pCopyConstructor)(void *pElement ) ,
                                                                              uint size ,
                                                                              zend_bool (*pMergeSource)(HashTable *target_ht ,
                                                                                                        void *source_data ,
                                                                                                        zend_hash_key *hash_key ,
                                                                                                        void *pParam ) ,
                                                                              void *pParam ) ;
extern void __attribute__((__visibility__("default")))  zend_ts_hash_merge(TsHashTable *target ,
                                                                           TsHashTable *source ,
                                                                           void (*pCopyConstructor)(void *pElement ) ,
                                                                           void *tmp ,
                                                                           uint size ,
                                                                           int overwrite ) ;
extern void __attribute__((__visibility__("default")))  zend_ts_hash_copy_to_hash(HashTable *target ,
                                                                                  TsHashTable *source ,
                                                                                  void (*pCopyConstructor)(void *pElement ) ,
                                                                                  void *tmp ,
                                                                                  uint size ) ;
extern void __attribute__((__visibility__("default")))  zend_ts_hash_copy(TsHashTable *target ,
                                                                          TsHashTable *source ,
                                                                          void (*pCopyConstructor)(void *pElement ) ,
                                                                          void *tmp ,
                                                                          uint size ) ;
extern int __attribute__((__visibility__("default")))  zend_ts_hash_index_exists(TsHashTable *ht ,
                                                                                 ulong h ) ;
extern int __attribute__((__visibility__("default")))  zend_ts_hash_exists(TsHashTable *ht ,
                                                                           char *arKey ,
                                                                           uint nKeyLength ) ;
extern int __attribute__((__visibility__("default")))  zend_ts_hash_index_find(TsHashTable *ht ,
                                                                               ulong h ,
                                                                               void **pData ) ;
extern int __attribute__((__visibility__("default")))  zend_ts_hash_quick_find(TsHashTable *ht ,
                                                                               char *arKey ,
                                                                               uint nKeyLength ,
                                                                               ulong h ,
                                                                               void **pData ) ;
extern int __attribute__((__visibility__("default")))  zend_ts_hash_find(TsHashTable *ht ,
                                                                         char *arKey ,
                                                                         uint nKeyLength ,
                                                                         void **pData ) ;
extern ulong __attribute__((__visibility__("default")))  zend_ts_get_hash_value(TsHashTable *ht ,
                                                                                char *arKey ,
                                                                                uint nKeyLength ) ;
extern int __attribute__((__visibility__("default")))  zend_ts_hash_del_key_or_index(TsHashTable *ht ,
                                                                                     char *arKey ,
                                                                                     uint nKeyLength ,
                                                                                     ulong h ,
                                                                                     int flag ) ;
extern void __attribute__((__visibility__("default")))  zend_ts_hash_reverse_apply(TsHashTable *ht ,
                                                                                   int (*apply_func)(void *pDest ) ) ;
extern void __attribute__((__visibility__("default")))  zend_ts_hash_apply_with_arguments(TsHashTable *ht ,
                                                                                          int (*apply_func)(void *pDest ,
                                                                                                            int num_args ,
                                                                                                            va_list args ,
                                                                                                            zend_hash_key *hash_key ) ,
                                                                                          int  
                                                                                          , ...) ;
extern void __attribute__((__visibility__("default")))  zend_ts_hash_apply_with_argument(TsHashTable *ht ,
                                                                                         int (*apply_func)(void *pDest ,
                                                                                                           void *argument ) ,
                                                                                         void * ) ;
extern void __attribute__((__visibility__("default")))  zend_ts_hash_apply(TsHashTable *ht ,
                                                                           int (*apply_func)(void *pDest ) ) ;
extern void __attribute__((__visibility__("default")))  zend_ts_hash_graceful_destroy(TsHashTable *ht ) ;
extern int __attribute__((__visibility__("default")))  zend_ts_hash_add_empty_element(TsHashTable *ht ,
                                                                                      char *arKey ,
                                                                                      uint nKeyLength ) ;
extern int __attribute__((__visibility__("default")))  _zend_ts_hash_index_update_or_next_insert(TsHashTable *ht ,
                                                                                                 ulong h ,
                                                                                                 void *pData ,
                                                                                                 uint nDataSize ,
                                                                                                 void **pDest ,
                                                                                                 int flag ) ;
extern int __attribute__((__visibility__("default")))  _zend_ts_hash_quick_add_or_update(TsHashTable *ht ,
                                                                                         char *arKey ,
                                                                                         uint nKeyLength ,
                                                                                         ulong h ,
                                                                                         void *pData ,
                                                                                         uint nDataSize ,
                                                                                         void **pDest ,
                                                                                         int flag ) ;
extern int __attribute__((__visibility__("default")))  _zend_ts_hash_add_or_update(TsHashTable *ht ,
                                                                                   char *arKey ,
                                                                                   uint nKeyLength ,
                                                                                   void *pData ,
                                                                                   uint nDataSize ,
                                                                                   void **pDest ,
                                                                                   int flag ) ;
extern void __attribute__((__visibility__("default")))  zend_ts_hash_clean(TsHashTable *ht ) ;
extern void __attribute__((__visibility__("default")))  zend_ts_hash_destroy(TsHashTable *ht ) ;
extern int __attribute__((__visibility__("default")))  _zend_ts_hash_init_ex(TsHashTable *ht ,
                                                                             uint nSize ,
                                                                             ulong (*pHashFunction)(char const   *arKey ,
                                                                                                    uint nKeyLength ) ,
                                                                             void (*pDestructor)(void *pDest ) ,
                                                                             zend_bool persistent ,
                                                                             zend_bool bApplyProtection ) ;
extern int __attribute__((__visibility__("default")))  _zend_ts_hash_init(TsHashTable *ht ,
                                                                          uint nSize ,
                                                                          ulong (*pHashFunction)(char const   *arKey ,
                                                                                                 uint nKeyLength ) ,
                                                                          void (*pDestructor)(void *pDest ) ,
                                                                          zend_bool persistent ) ;
extern int __attribute__((__visibility__("default")))  zend_hash_update_current_key_ex(HashTable *ht ,
                                                                                       int key_type ,
                                                                                       char const   *str_index ,
                                                                                       uint str_length ,
                                                                                       ulong num_index ,
                                                                                       int mode ,
                                                                                       HashPosition *pos ) ;
__inline static int zend_symtable_update_current_key_ex(HashTable *ht ,
                                                        char const   *arKey ,
                                                        uint nKeyLength ,
                                                        int mode ,
                                                        HashPosition *pos ) 
{ ulong idx ;
  register char const   *tmp ;
  char const   *end ;
  int __attribute__((__visibility__("default")))  tmp___0 ;
  int __attribute__((__visibility__("default")))  tmp___1 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "552\n");
  fflush(_coverage_fout);
  }
  while (1) {
    {
    fprintf(_coverage_fout, "551\n");
    fflush(_coverage_fout);
    }
    while (1) {
      {
      fprintf(_coverage_fout, "548\n");
      fflush(_coverage_fout);
      }
      tmp = arKey;
      {
      fprintf(_coverage_fout, "549\n");
      fflush(_coverage_fout);
      }
      if ((int const   )*tmp == 45) {
        {
        fprintf(_coverage_fout, "516\n");
        fflush(_coverage_fout);
        }
        tmp ++;
      } else {
        {
        fprintf(_coverage_fout, "517\n");
        fflush(_coverage_fout);
        }

      }
      {
      fprintf(_coverage_fout, "550\n");
      fflush(_coverage_fout);
      }
      if ((int const   )*tmp >= 48) {
        {
        fprintf(_coverage_fout, "546\n");
        fflush(_coverage_fout);
        }
        if ((int const   )*tmp <= 57) {
          {
          fprintf(_coverage_fout, "540\n");
          fflush(_coverage_fout);
          }
          end = (arKey + nKeyLength) - 1;
          {
          fprintf(_coverage_fout, "541\n");
          fflush(_coverage_fout);
          }
          if ((int const   )*end != 0) {
            break;
          } else {
            {
            fprintf(_coverage_fout, "524\n");
            fflush(_coverage_fout);
            }
            if ((int const   )*tmp == 48) {
              {
              fprintf(_coverage_fout, "518\n");
              fflush(_coverage_fout);
              }
              if (nKeyLength > 2U) {
                break;
              } else {
                goto _L;
              }
            } else {
              {
              fprintf(_coverage_fout, "523\n");
              fflush(_coverage_fout);
              }
              _L: /* CIL Label */ 
              if (end - tmp > 10) {
                break;
              } else {
                {
                fprintf(_coverage_fout, "522\n");
                fflush(_coverage_fout);
                }
                if (end - tmp == 10) {
                  {
                  fprintf(_coverage_fout, "520\n");
                  fflush(_coverage_fout);
                  }
                  if ((int const   )*tmp > 50) {
                    break;
                  } else {
                    {
                    fprintf(_coverage_fout, "519\n");
                    fflush(_coverage_fout);
                    }

                  }
                } else {
                  {
                  fprintf(_coverage_fout, "521\n");
                  fflush(_coverage_fout);
                  }

                }
              }
            }
          }
          {
          fprintf(_coverage_fout, "542\n");
          fflush(_coverage_fout);
          }
          idx = (unsigned long )((int const   )*tmp - 48);
          {
          fprintf(_coverage_fout, "543\n");
          fflush(_coverage_fout);
          }
          while (1) {
            {
            fprintf(_coverage_fout, "528\n");
            fflush(_coverage_fout);
            }
            tmp ++;
            {
            fprintf(_coverage_fout, "529\n");
            fflush(_coverage_fout);
            }
            if ((unsigned int )tmp != (unsigned int )end) {
              {
              fprintf(_coverage_fout, "527\n");
              fflush(_coverage_fout);
              }
              if ((int const   )*tmp >= 48) {
                {
                fprintf(_coverage_fout, "526\n");
                fflush(_coverage_fout);
                }
                if ((int const   )*tmp <= 57) {
                  {
                  fprintf(_coverage_fout, "525\n");
                  fflush(_coverage_fout);
                  }

                } else {
                  break;
                }
              } else {
                break;
              }
            } else {
              break;
            }
            {
            fprintf(_coverage_fout, "530\n");
            fflush(_coverage_fout);
            }
            idx = idx * 10UL + (ulong )((int const   )*tmp - 48);
          }
          {
          fprintf(_coverage_fout, "544\n");
          fflush(_coverage_fout);
          }
          if ((unsigned int )tmp == (unsigned int )end) {
            {
            fprintf(_coverage_fout, "536\n");
            fflush(_coverage_fout);
            }
            if ((int const   )*arKey == 45) {
              {
              fprintf(_coverage_fout, "532\n");
              fflush(_coverage_fout);
              }
              if (idx - 1UL > 2147483647UL) {
                break;
              } else {
                {
                fprintf(_coverage_fout, "531\n");
                fflush(_coverage_fout);
                }

              }
              {
              fprintf(_coverage_fout, "533\n");
              fflush(_coverage_fout);
              }
              idx = (unsigned long )(- ((long )idx));
            } else {
              {
              fprintf(_coverage_fout, "535\n");
              fflush(_coverage_fout);
              }
              if (idx > 2147483647UL) {
                break;
              } else {
                {
                fprintf(_coverage_fout, "534\n");
                fflush(_coverage_fout);
                }

              }
            }
            {
            fprintf(_coverage_fout, "537\n");
            fflush(_coverage_fout);
            }
            tmp___0 = zend_hash_update_current_key_ex(ht, 2,
                                                      (char const   *)((void *)0),
                                                      0U, idx, mode, pos);
            {
            fprintf(_coverage_fout, "538\n");
            fflush(_coverage_fout);
            }
            return ((int )tmp___0);
          } else {
            {
            fprintf(_coverage_fout, "539\n");
            fflush(_coverage_fout);
            }

          }
        } else {
          {
          fprintf(_coverage_fout, "545\n");
          fflush(_coverage_fout);
          }

        }
      } else {
        {
        fprintf(_coverage_fout, "547\n");
        fflush(_coverage_fout);
        }

      }
      break;
    }
    break;
  }
  {
  fprintf(_coverage_fout, "553\n");
  fflush(_coverage_fout);
  }
  tmp___1 = zend_hash_update_current_key_ex(ht, 1, arKey, nKeyLength, 0UL, mode,
                                            pos);
  {
  fprintf(_coverage_fout, "554\n");
  fflush(_coverage_fout);
  }
  return ((int )tmp___1);
}
}
__inline static int zend_symtable_update_current_key_ex(HashTable *ht ,
                                                        char const   *arKey ,
                                                        uint nKeyLength ,
                                                        int mode ,
                                                        HashPosition *pos ) ;
extern int __attribute__((__visibility__("default")))  zend_hash_exists(HashTable const   *ht ,
                                                                        char const   *arKey ,
                                                                        uint nKeyLength ) ;
__inline static int zend_symtable_exists(HashTable *ht , char const   *arKey ,
                                         uint nKeyLength ) 
{ ulong idx ;
  register char const   *tmp ;
  char const   *end ;
  int __attribute__((__visibility__("default")))  tmp___0 ;
  int __attribute__((__visibility__("default")))  tmp___1 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "513\n");
  fflush(_coverage_fout);
  }
  while (1) {
    {
    fprintf(_coverage_fout, "512\n");
    fflush(_coverage_fout);
    }
    while (1) {
      {
      fprintf(_coverage_fout, "509\n");
      fflush(_coverage_fout);
      }
      tmp = arKey;
      {
      fprintf(_coverage_fout, "510\n");
      fflush(_coverage_fout);
      }
      if ((int const   )*tmp == 45) {
        {
        fprintf(_coverage_fout, "477\n");
        fflush(_coverage_fout);
        }
        tmp ++;
      } else {
        {
        fprintf(_coverage_fout, "478\n");
        fflush(_coverage_fout);
        }

      }
      {
      fprintf(_coverage_fout, "511\n");
      fflush(_coverage_fout);
      }
      if ((int const   )*tmp >= 48) {
        {
        fprintf(_coverage_fout, "507\n");
        fflush(_coverage_fout);
        }
        if ((int const   )*tmp <= 57) {
          {
          fprintf(_coverage_fout, "501\n");
          fflush(_coverage_fout);
          }
          end = (arKey + nKeyLength) - 1;
          {
          fprintf(_coverage_fout, "502\n");
          fflush(_coverage_fout);
          }
          if ((int const   )*end != 0) {
            break;
          } else {
            {
            fprintf(_coverage_fout, "485\n");
            fflush(_coverage_fout);
            }
            if ((int const   )*tmp == 48) {
              {
              fprintf(_coverage_fout, "479\n");
              fflush(_coverage_fout);
              }
              if (nKeyLength > 2U) {
                break;
              } else {
                goto _L;
              }
            } else {
              {
              fprintf(_coverage_fout, "484\n");
              fflush(_coverage_fout);
              }
              _L: /* CIL Label */ 
              if (end - tmp > 10) {
                break;
              } else {
                {
                fprintf(_coverage_fout, "483\n");
                fflush(_coverage_fout);
                }
                if (end - tmp == 10) {
                  {
                  fprintf(_coverage_fout, "481\n");
                  fflush(_coverage_fout);
                  }
                  if ((int const   )*tmp > 50) {
                    break;
                  } else {
                    {
                    fprintf(_coverage_fout, "480\n");
                    fflush(_coverage_fout);
                    }

                  }
                } else {
                  {
                  fprintf(_coverage_fout, "482\n");
                  fflush(_coverage_fout);
                  }

                }
              }
            }
          }
          {
          fprintf(_coverage_fout, "503\n");
          fflush(_coverage_fout);
          }
          idx = (unsigned long )((int const   )*tmp - 48);
          {
          fprintf(_coverage_fout, "504\n");
          fflush(_coverage_fout);
          }
          while (1) {
            {
            fprintf(_coverage_fout, "489\n");
            fflush(_coverage_fout);
            }
            tmp ++;
            {
            fprintf(_coverage_fout, "490\n");
            fflush(_coverage_fout);
            }
            if ((unsigned int )tmp != (unsigned int )end) {
              {
              fprintf(_coverage_fout, "488\n");
              fflush(_coverage_fout);
              }
              if ((int const   )*tmp >= 48) {
                {
                fprintf(_coverage_fout, "487\n");
                fflush(_coverage_fout);
                }
                if ((int const   )*tmp <= 57) {
                  {
                  fprintf(_coverage_fout, "486\n");
                  fflush(_coverage_fout);
                  }

                } else {
                  break;
                }
              } else {
                break;
              }
            } else {
              break;
            }
            {
            fprintf(_coverage_fout, "491\n");
            fflush(_coverage_fout);
            }
            idx = idx * 10UL + (ulong )((int const   )*tmp - 48);
          }
          {
          fprintf(_coverage_fout, "505\n");
          fflush(_coverage_fout);
          }
          if ((unsigned int )tmp == (unsigned int )end) {
            {
            fprintf(_coverage_fout, "497\n");
            fflush(_coverage_fout);
            }
            if ((int const   )*arKey == 45) {
              {
              fprintf(_coverage_fout, "493\n");
              fflush(_coverage_fout);
              }
              if (idx - 1UL > 2147483647UL) {
                break;
              } else {
                {
                fprintf(_coverage_fout, "492\n");
                fflush(_coverage_fout);
                }

              }
              {
              fprintf(_coverage_fout, "494\n");
              fflush(_coverage_fout);
              }
              idx = (unsigned long )(- ((long )idx));
            } else {
              {
              fprintf(_coverage_fout, "496\n");
              fflush(_coverage_fout);
              }
              if (idx > 2147483647UL) {
                break;
              } else {
                {
                fprintf(_coverage_fout, "495\n");
                fflush(_coverage_fout);
                }

              }
            }
            {
            fprintf(_coverage_fout, "498\n");
            fflush(_coverage_fout);
            }
            tmp___0 = zend_hash_index_exists((HashTable const   *)ht, idx);
            {
            fprintf(_coverage_fout, "499\n");
            fflush(_coverage_fout);
            }
            return ((int )tmp___0);
          } else {
            {
            fprintf(_coverage_fout, "500\n");
            fflush(_coverage_fout);
            }

          }
        } else {
          {
          fprintf(_coverage_fout, "506\n");
          fflush(_coverage_fout);
          }

        }
      } else {
        {
        fprintf(_coverage_fout, "508\n");
        fflush(_coverage_fout);
        }

      }
      break;
    }
    break;
  }
  {
  fprintf(_coverage_fout, "514\n");
  fflush(_coverage_fout);
  }
  tmp___1 = zend_hash_exists((HashTable const   *)ht, arKey, nKeyLength);
  {
  fprintf(_coverage_fout, "515\n");
  fflush(_coverage_fout);
  }
  return ((int )tmp___1);
}
}
__inline static int zend_symtable_find(HashTable *ht , char const   *arKey ,
                                       uint nKeyLength , void **pData ) 
{ ulong idx ;
  register char const   *tmp ;
  char const   *end ;
  int __attribute__((__visibility__("default")))  tmp___0 ;
  int __attribute__((__visibility__("default")))  tmp___1 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "474\n");
  fflush(_coverage_fout);
  }
  while (1) {
    {
    fprintf(_coverage_fout, "473\n");
    fflush(_coverage_fout);
    }
    while (1) {
      {
      fprintf(_coverage_fout, "470\n");
      fflush(_coverage_fout);
      }
      tmp = arKey;
      {
      fprintf(_coverage_fout, "471\n");
      fflush(_coverage_fout);
      }
      if ((int const   )*tmp == 45) {
        {
        fprintf(_coverage_fout, "438\n");
        fflush(_coverage_fout);
        }
        tmp ++;
      } else {
        {
        fprintf(_coverage_fout, "439\n");
        fflush(_coverage_fout);
        }

      }
      {
      fprintf(_coverage_fout, "472\n");
      fflush(_coverage_fout);
      }
      if ((int const   )*tmp >= 48) {
        {
        fprintf(_coverage_fout, "468\n");
        fflush(_coverage_fout);
        }
        if ((int const   )*tmp <= 57) {
          {
          fprintf(_coverage_fout, "462\n");
          fflush(_coverage_fout);
          }
          end = (arKey + nKeyLength) - 1;
          {
          fprintf(_coverage_fout, "463\n");
          fflush(_coverage_fout);
          }
          if ((int const   )*end != 0) {
            break;
          } else {
            {
            fprintf(_coverage_fout, "446\n");
            fflush(_coverage_fout);
            }
            if ((int const   )*tmp == 48) {
              {
              fprintf(_coverage_fout, "440\n");
              fflush(_coverage_fout);
              }
              if (nKeyLength > 2U) {
                break;
              } else {
                goto _L;
              }
            } else {
              {
              fprintf(_coverage_fout, "445\n");
              fflush(_coverage_fout);
              }
              _L: /* CIL Label */ 
              if (end - tmp > 10) {
                break;
              } else {
                {
                fprintf(_coverage_fout, "444\n");
                fflush(_coverage_fout);
                }
                if (end - tmp == 10) {
                  {
                  fprintf(_coverage_fout, "442\n");
                  fflush(_coverage_fout);
                  }
                  if ((int const   )*tmp > 50) {
                    break;
                  } else {
                    {
                    fprintf(_coverage_fout, "441\n");
                    fflush(_coverage_fout);
                    }

                  }
                } else {
                  {
                  fprintf(_coverage_fout, "443\n");
                  fflush(_coverage_fout);
                  }

                }
              }
            }
          }
          {
          fprintf(_coverage_fout, "464\n");
          fflush(_coverage_fout);
          }
          idx = (unsigned long )((int const   )*tmp - 48);
          {
          fprintf(_coverage_fout, "465\n");
          fflush(_coverage_fout);
          }
          while (1) {
            {
            fprintf(_coverage_fout, "450\n");
            fflush(_coverage_fout);
            }
            tmp ++;
            {
            fprintf(_coverage_fout, "451\n");
            fflush(_coverage_fout);
            }
            if ((unsigned int )tmp != (unsigned int )end) {
              {
              fprintf(_coverage_fout, "449\n");
              fflush(_coverage_fout);
              }
              if ((int const   )*tmp >= 48) {
                {
                fprintf(_coverage_fout, "448\n");
                fflush(_coverage_fout);
                }
                if ((int const   )*tmp <= 57) {
                  {
                  fprintf(_coverage_fout, "447\n");
                  fflush(_coverage_fout);
                  }

                } else {
                  break;
                }
              } else {
                break;
              }
            } else {
              break;
            }
            {
            fprintf(_coverage_fout, "452\n");
            fflush(_coverage_fout);
            }
            idx = idx * 10UL + (ulong )((int const   )*tmp - 48);
          }
          {
          fprintf(_coverage_fout, "466\n");
          fflush(_coverage_fout);
          }
          if ((unsigned int )tmp == (unsigned int )end) {
            {
            fprintf(_coverage_fout, "458\n");
            fflush(_coverage_fout);
            }
            if ((int const   )*arKey == 45) {
              {
              fprintf(_coverage_fout, "454\n");
              fflush(_coverage_fout);
              }
              if (idx - 1UL > 2147483647UL) {
                break;
              } else {
                {
                fprintf(_coverage_fout, "453\n");
                fflush(_coverage_fout);
                }

              }
              {
              fprintf(_coverage_fout, "455\n");
              fflush(_coverage_fout);
              }
              idx = (unsigned long )(- ((long )idx));
            } else {
              {
              fprintf(_coverage_fout, "457\n");
              fflush(_coverage_fout);
              }
              if (idx > 2147483647UL) {
                break;
              } else {
                {
                fprintf(_coverage_fout, "456\n");
                fflush(_coverage_fout);
                }

              }
            }
            {
            fprintf(_coverage_fout, "459\n");
            fflush(_coverage_fout);
            }
            tmp___0 = zend_hash_index_find((HashTable const   *)ht, idx, pData);
            {
            fprintf(_coverage_fout, "460\n");
            fflush(_coverage_fout);
            }
            return ((int )tmp___0);
          } else {
            {
            fprintf(_coverage_fout, "461\n");
            fflush(_coverage_fout);
            }

          }
        } else {
          {
          fprintf(_coverage_fout, "467\n");
          fflush(_coverage_fout);
          }

        }
      } else {
        {
        fprintf(_coverage_fout, "469\n");
        fflush(_coverage_fout);
        }

      }
      break;
    }
    break;
  }
  {
  fprintf(_coverage_fout, "475\n");
  fflush(_coverage_fout);
  }
  tmp___1 = zend_hash_find((HashTable const   *)ht, arKey, nKeyLength, pData);
  {
  fprintf(_coverage_fout, "476\n");
  fflush(_coverage_fout);
  }
  return ((int )tmp___1);
}
}
__inline static int zend_symtable_del(HashTable *ht , char const   *arKey ,
                                      uint nKeyLength ) 
{ ulong idx ;
  register char const   *tmp ;
  char const   *end ;
  int __attribute__((__visibility__("default")))  tmp___0 ;
  int __attribute__((__visibility__("default")))  tmp___1 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "435\n");
  fflush(_coverage_fout);
  }
  while (1) {
    {
    fprintf(_coverage_fout, "434\n");
    fflush(_coverage_fout);
    }
    while (1) {
      {
      fprintf(_coverage_fout, "431\n");
      fflush(_coverage_fout);
      }
      tmp = arKey;
      {
      fprintf(_coverage_fout, "432\n");
      fflush(_coverage_fout);
      }
      if ((int const   )*tmp == 45) {
        {
        fprintf(_coverage_fout, "399\n");
        fflush(_coverage_fout);
        }
        tmp ++;
      } else {
        {
        fprintf(_coverage_fout, "400\n");
        fflush(_coverage_fout);
        }

      }
      {
      fprintf(_coverage_fout, "433\n");
      fflush(_coverage_fout);
      }
      if ((int const   )*tmp >= 48) {
        {
        fprintf(_coverage_fout, "429\n");
        fflush(_coverage_fout);
        }
        if ((int const   )*tmp <= 57) {
          {
          fprintf(_coverage_fout, "423\n");
          fflush(_coverage_fout);
          }
          end = (arKey + nKeyLength) - 1;
          {
          fprintf(_coverage_fout, "424\n");
          fflush(_coverage_fout);
          }
          if ((int const   )*end != 0) {
            break;
          } else {
            {
            fprintf(_coverage_fout, "407\n");
            fflush(_coverage_fout);
            }
            if ((int const   )*tmp == 48) {
              {
              fprintf(_coverage_fout, "401\n");
              fflush(_coverage_fout);
              }
              if (nKeyLength > 2U) {
                break;
              } else {
                goto _L;
              }
            } else {
              {
              fprintf(_coverage_fout, "406\n");
              fflush(_coverage_fout);
              }
              _L: /* CIL Label */ 
              if (end - tmp > 10) {
                break;
              } else {
                {
                fprintf(_coverage_fout, "405\n");
                fflush(_coverage_fout);
                }
                if (end - tmp == 10) {
                  {
                  fprintf(_coverage_fout, "403\n");
                  fflush(_coverage_fout);
                  }
                  if ((int const   )*tmp > 50) {
                    break;
                  } else {
                    {
                    fprintf(_coverage_fout, "402\n");
                    fflush(_coverage_fout);
                    }

                  }
                } else {
                  {
                  fprintf(_coverage_fout, "404\n");
                  fflush(_coverage_fout);
                  }

                }
              }
            }
          }
          {
          fprintf(_coverage_fout, "425\n");
          fflush(_coverage_fout);
          }
          idx = (unsigned long )((int const   )*tmp - 48);
          {
          fprintf(_coverage_fout, "426\n");
          fflush(_coverage_fout);
          }
          while (1) {
            {
            fprintf(_coverage_fout, "411\n");
            fflush(_coverage_fout);
            }
            tmp ++;
            {
            fprintf(_coverage_fout, "412\n");
            fflush(_coverage_fout);
            }
            if ((unsigned int )tmp != (unsigned int )end) {
              {
              fprintf(_coverage_fout, "410\n");
              fflush(_coverage_fout);
              }
              if ((int const   )*tmp >= 48) {
                {
                fprintf(_coverage_fout, "409\n");
                fflush(_coverage_fout);
                }
                if ((int const   )*tmp <= 57) {
                  {
                  fprintf(_coverage_fout, "408\n");
                  fflush(_coverage_fout);
                  }

                } else {
                  break;
                }
              } else {
                break;
              }
            } else {
              break;
            }
            {
            fprintf(_coverage_fout, "413\n");
            fflush(_coverage_fout);
            }
            idx = idx * 10UL + (ulong )((int const   )*tmp - 48);
          }
          {
          fprintf(_coverage_fout, "427\n");
          fflush(_coverage_fout);
          }
          if ((unsigned int )tmp == (unsigned int )end) {
            {
            fprintf(_coverage_fout, "419\n");
            fflush(_coverage_fout);
            }
            if ((int const   )*arKey == 45) {
              {
              fprintf(_coverage_fout, "415\n");
              fflush(_coverage_fout);
              }
              if (idx - 1UL > 2147483647UL) {
                break;
              } else {
                {
                fprintf(_coverage_fout, "414\n");
                fflush(_coverage_fout);
                }

              }
              {
              fprintf(_coverage_fout, "416\n");
              fflush(_coverage_fout);
              }
              idx = (unsigned long )(- ((long )idx));
            } else {
              {
              fprintf(_coverage_fout, "418\n");
              fflush(_coverage_fout);
              }
              if (idx > 2147483647UL) {
                break;
              } else {
                {
                fprintf(_coverage_fout, "417\n");
                fflush(_coverage_fout);
                }

              }
            }
            {
            fprintf(_coverage_fout, "420\n");
            fflush(_coverage_fout);
            }
            tmp___0 = zend_hash_del_key_or_index(ht,
                                                 (char const   *)((void *)0),
                                                 0U, idx, 1);
            {
            fprintf(_coverage_fout, "421\n");
            fflush(_coverage_fout);
            }
            return ((int )tmp___0);
          } else {
            {
            fprintf(_coverage_fout, "422\n");
            fflush(_coverage_fout);
            }

          }
        } else {
          {
          fprintf(_coverage_fout, "428\n");
          fflush(_coverage_fout);
          }

        }
      } else {
        {
        fprintf(_coverage_fout, "430\n");
        fflush(_coverage_fout);
        }

      }
      break;
    }
    break;
  }
  {
  fprintf(_coverage_fout, "436\n");
  fflush(_coverage_fout);
  }
  tmp___1 = zend_hash_del_key_or_index(ht, arKey, nKeyLength, 0UL, 0);
  {
  fprintf(_coverage_fout, "437\n");
  fflush(_coverage_fout);
  }
  return ((int )tmp___1);
}
}
extern int __attribute__((__visibility__("default")))  _zend_hash_add_or_update(HashTable *ht ,
                                                                                char const   *arKey ,
                                                                                uint nKeyLength ,
                                                                                void *pData ,
                                                                                uint nDataSize ,
                                                                                void **pDest ,
                                                                                int flag ) ;
__inline static int zend_symtable_update(HashTable *ht , char const   *arKey ,
                                         uint nKeyLength , void *pData ,
                                         uint nDataSize , void **pDest ) 
{ ulong idx ;
  register char const   *tmp ;
  char const   *end ;
  int __attribute__((__visibility__("default")))  tmp___0 ;
  int __attribute__((__visibility__("default")))  tmp___1 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "396\n");
  fflush(_coverage_fout);
  }
  while (1) {
    {
    fprintf(_coverage_fout, "395\n");
    fflush(_coverage_fout);
    }
    while (1) {
      {
      fprintf(_coverage_fout, "392\n");
      fflush(_coverage_fout);
      }
      tmp = arKey;
      {
      fprintf(_coverage_fout, "393\n");
      fflush(_coverage_fout);
      }
      if ((int const   )*tmp == 45) {
        {
        fprintf(_coverage_fout, "360\n");
        fflush(_coverage_fout);
        }
        tmp ++;
      } else {
        {
        fprintf(_coverage_fout, "361\n");
        fflush(_coverage_fout);
        }

      }
      {
      fprintf(_coverage_fout, "394\n");
      fflush(_coverage_fout);
      }
      if ((int const   )*tmp >= 48) {
        {
        fprintf(_coverage_fout, "390\n");
        fflush(_coverage_fout);
        }
        if ((int const   )*tmp <= 57) {
          {
          fprintf(_coverage_fout, "384\n");
          fflush(_coverage_fout);
          }
          end = (arKey + nKeyLength) - 1;
          {
          fprintf(_coverage_fout, "385\n");
          fflush(_coverage_fout);
          }
          if ((int const   )*end != 0) {
            break;
          } else {
            {
            fprintf(_coverage_fout, "368\n");
            fflush(_coverage_fout);
            }
            if ((int const   )*tmp == 48) {
              {
              fprintf(_coverage_fout, "362\n");
              fflush(_coverage_fout);
              }
              if (nKeyLength > 2U) {
                break;
              } else {
                goto _L;
              }
            } else {
              {
              fprintf(_coverage_fout, "367\n");
              fflush(_coverage_fout);
              }
              _L: /* CIL Label */ 
              if (end - tmp > 10) {
                break;
              } else {
                {
                fprintf(_coverage_fout, "366\n");
                fflush(_coverage_fout);
                }
                if (end - tmp == 10) {
                  {
                  fprintf(_coverage_fout, "364\n");
                  fflush(_coverage_fout);
                  }
                  if ((int const   )*tmp > 50) {
                    break;
                  } else {
                    {
                    fprintf(_coverage_fout, "363\n");
                    fflush(_coverage_fout);
                    }

                  }
                } else {
                  {
                  fprintf(_coverage_fout, "365\n");
                  fflush(_coverage_fout);
                  }

                }
              }
            }
          }
          {
          fprintf(_coverage_fout, "386\n");
          fflush(_coverage_fout);
          }
          idx = (unsigned long )((int const   )*tmp - 48);
          {
          fprintf(_coverage_fout, "387\n");
          fflush(_coverage_fout);
          }
          while (1) {
            {
            fprintf(_coverage_fout, "372\n");
            fflush(_coverage_fout);
            }
            tmp ++;
            {
            fprintf(_coverage_fout, "373\n");
            fflush(_coverage_fout);
            }
            if ((unsigned int )tmp != (unsigned int )end) {
              {
              fprintf(_coverage_fout, "371\n");
              fflush(_coverage_fout);
              }
              if ((int const   )*tmp >= 48) {
                {
                fprintf(_coverage_fout, "370\n");
                fflush(_coverage_fout);
                }
                if ((int const   )*tmp <= 57) {
                  {
                  fprintf(_coverage_fout, "369\n");
                  fflush(_coverage_fout);
                  }

                } else {
                  break;
                }
              } else {
                break;
              }
            } else {
              break;
            }
            {
            fprintf(_coverage_fout, "374\n");
            fflush(_coverage_fout);
            }
            idx = idx * 10UL + (ulong )((int const   )*tmp - 48);
          }
          {
          fprintf(_coverage_fout, "388\n");
          fflush(_coverage_fout);
          }
          if ((unsigned int )tmp == (unsigned int )end) {
            {
            fprintf(_coverage_fout, "380\n");
            fflush(_coverage_fout);
            }
            if ((int const   )*arKey == 45) {
              {
              fprintf(_coverage_fout, "376\n");
              fflush(_coverage_fout);
              }
              if (idx - 1UL > 2147483647UL) {
                break;
              } else {
                {
                fprintf(_coverage_fout, "375\n");
                fflush(_coverage_fout);
                }

              }
              {
              fprintf(_coverage_fout, "377\n");
              fflush(_coverage_fout);
              }
              idx = (unsigned long )(- ((long )idx));
            } else {
              {
              fprintf(_coverage_fout, "379\n");
              fflush(_coverage_fout);
              }
              if (idx > 2147483647UL) {
                break;
              } else {
                {
                fprintf(_coverage_fout, "378\n");
                fflush(_coverage_fout);
                }

              }
            }
            {
            fprintf(_coverage_fout, "381\n");
            fflush(_coverage_fout);
            }
            tmp___0 = _zend_hash_index_update_or_next_insert(ht, idx, pData,
                                                             nDataSize, pDest, 1);
            {
            fprintf(_coverage_fout, "382\n");
            fflush(_coverage_fout);
            }
            return ((int )tmp___0);
          } else {
            {
            fprintf(_coverage_fout, "383\n");
            fflush(_coverage_fout);
            }

          }
        } else {
          {
          fprintf(_coverage_fout, "389\n");
          fflush(_coverage_fout);
          }

        }
      } else {
        {
        fprintf(_coverage_fout, "391\n");
        fflush(_coverage_fout);
        }

      }
      break;
    }
    break;
  }
  {
  fprintf(_coverage_fout, "397\n");
  fflush(_coverage_fout);
  }
  tmp___1 = _zend_hash_add_or_update(ht, arKey, nKeyLength, pData, nDataSize,
                                     pDest, 1);
  {
  fprintf(_coverage_fout, "398\n");
  fflush(_coverage_fout);
  }
  return ((int )tmp___1);
}
}
extern ulong __attribute__((__visibility__("default")))  zend_hash_func(char const   *arKey ,
                                                                        uint nKeyLength ) ;
__inline static ulong zend_inline_hash_func(char const   *arKey ,
                                            uint nKeyLength ) 
{ register ulong hash ;
  char const   *tmp ;
  char const   *tmp___0 ;
  char const   *tmp___1 ;
  char const   *tmp___2 ;
  char const   *tmp___3 ;
  char const   *tmp___4 ;
  char const   *tmp___5 ;
  char const   *tmp___6 ;
  char const   *tmp___7 ;
  char const   *tmp___8 ;
  char const   *tmp___9 ;
  char const   *tmp___10 ;
  char const   *tmp___11 ;
  char const   *tmp___12 ;
  char const   *tmp___13 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "357\n");
  fflush(_coverage_fout);
  }
  hash = (ulong )5381;
  {
  fprintf(_coverage_fout, "358\n");
  fflush(_coverage_fout);
  }
  while (1) {
    {
    fprintf(_coverage_fout, "310\n");
    fflush(_coverage_fout);
    }
    if (nKeyLength >= 8U) {
      {
      fprintf(_coverage_fout, "309\n");
      fflush(_coverage_fout);
      }

    } else {
      break;
    }
    {
    fprintf(_coverage_fout, "311\n");
    fflush(_coverage_fout);
    }
    tmp = arKey;
    {
    fprintf(_coverage_fout, "312\n");
    fflush(_coverage_fout);
    }
    arKey ++;
    {
    fprintf(_coverage_fout, "313\n");
    fflush(_coverage_fout);
    }
    hash = ((hash << 5) + hash) + (ulong )*tmp;
    {
    fprintf(_coverage_fout, "314\n");
    fflush(_coverage_fout);
    }
    tmp___0 = arKey;
    {
    fprintf(_coverage_fout, "315\n");
    fflush(_coverage_fout);
    }
    arKey ++;
    {
    fprintf(_coverage_fout, "316\n");
    fflush(_coverage_fout);
    }
    hash = ((hash << 5) + hash) + (ulong )*tmp___0;
    {
    fprintf(_coverage_fout, "317\n");
    fflush(_coverage_fout);
    }
    tmp___1 = arKey;
    {
    fprintf(_coverage_fout, "318\n");
    fflush(_coverage_fout);
    }
    arKey ++;
    {
    fprintf(_coverage_fout, "319\n");
    fflush(_coverage_fout);
    }
    hash = ((hash << 5) + hash) + (ulong )*tmp___1;
    {
    fprintf(_coverage_fout, "320\n");
    fflush(_coverage_fout);
    }
    tmp___2 = arKey;
    {
    fprintf(_coverage_fout, "321\n");
    fflush(_coverage_fout);
    }
    arKey ++;
    {
    fprintf(_coverage_fout, "322\n");
    fflush(_coverage_fout);
    }
    hash = ((hash << 5) + hash) + (ulong )*tmp___2;
    {
    fprintf(_coverage_fout, "323\n");
    fflush(_coverage_fout);
    }
    tmp___3 = arKey;
    {
    fprintf(_coverage_fout, "324\n");
    fflush(_coverage_fout);
    }
    arKey ++;
    {
    fprintf(_coverage_fout, "325\n");
    fflush(_coverage_fout);
    }
    hash = ((hash << 5) + hash) + (ulong )*tmp___3;
    {
    fprintf(_coverage_fout, "326\n");
    fflush(_coverage_fout);
    }
    tmp___4 = arKey;
    {
    fprintf(_coverage_fout, "327\n");
    fflush(_coverage_fout);
    }
    arKey ++;
    {
    fprintf(_coverage_fout, "328\n");
    fflush(_coverage_fout);
    }
    hash = ((hash << 5) + hash) + (ulong )*tmp___4;
    {
    fprintf(_coverage_fout, "329\n");
    fflush(_coverage_fout);
    }
    tmp___5 = arKey;
    {
    fprintf(_coverage_fout, "330\n");
    fflush(_coverage_fout);
    }
    arKey ++;
    {
    fprintf(_coverage_fout, "331\n");
    fflush(_coverage_fout);
    }
    hash = ((hash << 5) + hash) + (ulong )*tmp___5;
    {
    fprintf(_coverage_fout, "332\n");
    fflush(_coverage_fout);
    }
    tmp___6 = arKey;
    {
    fprintf(_coverage_fout, "333\n");
    fflush(_coverage_fout);
    }
    arKey ++;
    {
    fprintf(_coverage_fout, "334\n");
    fflush(_coverage_fout);
    }
    hash = ((hash << 5) + hash) + (ulong )*tmp___6;
    {
    fprintf(_coverage_fout, "335\n");
    fflush(_coverage_fout);
    }
    nKeyLength -= 8U;
  }
  switch ((int )nKeyLength) {
  {
  fprintf(_coverage_fout, "336\n");
  fflush(_coverage_fout);
  }
  case 7: 
  tmp___7 = arKey;
  {
  fprintf(_coverage_fout, "337\n");
  fflush(_coverage_fout);
  }
  arKey ++;
  {
  fprintf(_coverage_fout, "338\n");
  fflush(_coverage_fout);
  }
  hash = ((hash << 5) + hash) + (ulong )*tmp___7;
  {
  fprintf(_coverage_fout, "339\n");
  fflush(_coverage_fout);
  }
  case 6: 
  tmp___8 = arKey;
  {
  fprintf(_coverage_fout, "340\n");
  fflush(_coverage_fout);
  }
  arKey ++;
  {
  fprintf(_coverage_fout, "341\n");
  fflush(_coverage_fout);
  }
  hash = ((hash << 5) + hash) + (ulong )*tmp___8;
  {
  fprintf(_coverage_fout, "342\n");
  fflush(_coverage_fout);
  }
  case 5: 
  tmp___9 = arKey;
  {
  fprintf(_coverage_fout, "343\n");
  fflush(_coverage_fout);
  }
  arKey ++;
  {
  fprintf(_coverage_fout, "344\n");
  fflush(_coverage_fout);
  }
  hash = ((hash << 5) + hash) + (ulong )*tmp___9;
  {
  fprintf(_coverage_fout, "345\n");
  fflush(_coverage_fout);
  }
  case 4: 
  tmp___10 = arKey;
  {
  fprintf(_coverage_fout, "346\n");
  fflush(_coverage_fout);
  }
  arKey ++;
  {
  fprintf(_coverage_fout, "347\n");
  fflush(_coverage_fout);
  }
  hash = ((hash << 5) + hash) + (ulong )*tmp___10;
  {
  fprintf(_coverage_fout, "348\n");
  fflush(_coverage_fout);
  }
  case 3: 
  tmp___11 = arKey;
  {
  fprintf(_coverage_fout, "349\n");
  fflush(_coverage_fout);
  }
  arKey ++;
  {
  fprintf(_coverage_fout, "350\n");
  fflush(_coverage_fout);
  }
  hash = ((hash << 5) + hash) + (ulong )*tmp___11;
  {
  fprintf(_coverage_fout, "351\n");
  fflush(_coverage_fout);
  }
  case 2: 
  tmp___12 = arKey;
  {
  fprintf(_coverage_fout, "352\n");
  fflush(_coverage_fout);
  }
  arKey ++;
  {
  fprintf(_coverage_fout, "353\n");
  fflush(_coverage_fout);
  }
  hash = ((hash << 5) + hash) + (ulong )*tmp___12;
  {
  fprintf(_coverage_fout, "354\n");
  fflush(_coverage_fout);
  }
  case 1: 
  tmp___13 = arKey;
  {
  fprintf(_coverage_fout, "355\n");
  fflush(_coverage_fout);
  }
  arKey ++;
  {
  fprintf(_coverage_fout, "356\n");
  fflush(_coverage_fout);
  }
  hash = ((hash << 5) + hash) + (ulong )*tmp___13;
  break;
  case 0: 
  break;
  }
  {
  fprintf(_coverage_fout, "359\n");
  fflush(_coverage_fout);
  }
  return (hash);
}
}
__inline static ulong zend_inline_hash_func(char const   *arKey ,
                                            uint nKeyLength ) ;
extern int __attribute__((__visibility__("default")))  zend_hash_rehash(HashTable *ht ) ;
extern int __attribute__((__visibility__("default")))  zend_hash_minmax(HashTable const   *ht ,
                                                                        int (*compar)(void const   * ,
                                                                                      void const   * ) ,
                                                                        int flag ,
                                                                        void **pData ) ;
extern int __attribute__((__visibility__("default")))  zend_hash_compare(HashTable *ht1 ,
                                                                         HashTable *ht2 ,
                                                                         int (*compar)(void const   * ,
                                                                                       void const   * ) ,
                                                                         zend_bool ordered ) ;
extern int __attribute__((__visibility__("default")))  zend_hash_sort(HashTable *ht ,
                                                                      void (*sort_func)(void * ,
                                                                                        size_t  ,
                                                                                        size_t  ,
                                                                                        int (*)(void const   * ,
                                                                                                void const   * ) ) ,
                                                                      int (*compare_func)(void const   * ,
                                                                                          void const   * ) ,
                                                                      int renumber ) ;
extern void __attribute__((__visibility__("default")))  zend_hash_merge_ex(HashTable *target ,
                                                                           HashTable *source ,
                                                                           void (*pCopyConstructor)(void *pElement ) ,
                                                                           uint size ,
                                                                           zend_bool (*pMergeSource)(HashTable *target_ht ,
                                                                                                     void *source_data ,
                                                                                                     zend_hash_key *hash_key ,
                                                                                                     void *pParam ) ,
                                                                           void *pParam ) ;
extern void __attribute__((__visibility__("default")))  _zend_hash_merge(HashTable *target ,
                                                                         HashTable *source ,
                                                                         void (*pCopyConstructor)(void *pElement ) ,
                                                                         void *tmp ,
                                                                         uint size ,
                                                                         int overwrite ) ;
struct _HashPointer;
typedef struct _HashPointer HashPointer;
extern int __attribute__((__visibility__("default")))  zend_hash_set_pointer(HashTable *ht ,
                                                                             HashPointer const   *ptr ) ;
extern int __attribute__((__visibility__("default")))  zend_hash_get_pointer(HashTable const   *ht ,
                                                                             HashPointer *ptr ) ;
extern void __attribute__((__visibility__("default")))  zend_hash_internal_pointer_end_ex(HashTable *ht ,
                                                                                          HashPosition *pos ) ;
extern int __attribute__((__visibility__("default")))  zend_hash_move_backwards_ex(HashTable *ht ,
                                                                                   HashPosition *pos ) ;
extern ulong __attribute__((__visibility__("default")))  zend_hash_next_free_element(HashTable const   *ht ) ;
extern int __attribute__((__visibility__("default")))  zend_hash_quick_exists(HashTable const   *ht ,
                                                                              char const   *arKey ,
                                                                              uint nKeyLength ,
                                                                              ulong h ) ;
extern int __attribute__((__visibility__("default")))  zend_hash_quick_find(HashTable const   *ht ,
                                                                            char const   *arKey ,
                                                                            uint nKeyLength ,
                                                                            ulong h ,
                                                                            void **pData ) ;
extern ulong __attribute__((__visibility__("default")))  zend_get_hash_value(char const   *arKey ,
                                                                             uint nKeyLength ) ;
extern void __attribute__((__visibility__("default")))  zend_hash_reverse_apply(HashTable *ht ,
                                                                                int (*apply_func)(void *pDest ) ) ;
extern void __attribute__((__visibility__("default")))  zend_hash_apply_with_arguments(HashTable *ht ,
                                                                                       int (*apply_func)(void *pDest ,
                                                                                                         int num_args ,
                                                                                                         va_list args ,
                                                                                                         zend_hash_key *hash_key ) ,
                                                                                       int  
                                                                                       , ...) ;
extern void __attribute__((__visibility__("default")))  zend_hash_apply_with_argument(HashTable *ht ,
                                                                                      int (*apply_func)(void *pDest ,
                                                                                                        void *argument ) ,
                                                                                      void * ) ;
extern void __attribute__((__visibility__("default")))  zend_hash_apply(HashTable *ht ,
                                                                        int (*apply_func)(void *pDest ) ) ;
extern void __attribute__((__visibility__("default")))  zend_hash_graceful_reverse_destroy(HashTable *ht ) ;
extern void __attribute__((__visibility__("default")))  zend_hash_graceful_destroy(HashTable *ht ) ;
extern int __attribute__((__visibility__("default")))  zend_hash_add_empty_element(HashTable *ht ,
                                                                                   char const   *arKey ,
                                                                                   uint nKeyLength ) ;
extern int __attribute__((__visibility__("default")))  _zend_hash_quick_add_or_update(HashTable *ht ,
                                                                                      char const   *arKey ,
                                                                                      uint nKeyLength ,
                                                                                      ulong h ,
                                                                                      void *pData ,
                                                                                      uint nDataSize ,
                                                                                      void **pDest ,
                                                                                      int flag ) ;
extern int __attribute__((__visibility__("default")))  _zend_hash_init_ex(HashTable *ht ,
                                                                          uint nSize ,
                                                                          ulong (*pHashFunction)(char const   *arKey ,
                                                                                                 uint nKeyLength ) ,
                                                                          void (*pDestructor)(void *pDest ) ,
                                                                          zend_bool persistent ,
                                                                          zend_bool bApplyProtection ) ;
static char const   long_min_digits[11]  = 
  {      (char const   )'2',      (char const   )'1',      (char const   )'4',      (char const   )'7', 
        (char const   )'4',      (char const   )'8',      (char const   )'3',      (char const   )'6', 
        (char const   )'4',      (char const   )'8',      (char const   )'\000'};
extern void zend_interned_strings_dtor(void) ;
extern void zend_interned_strings_init(void) ;
extern void __attribute__((__visibility__("default")))  (*zend_interned_strings_restore)(void) ;
extern void __attribute__((__visibility__("default")))  (*zend_interned_strings_snapshot)(void) ;
extern char const __attribute__((__visibility__("default")))  *(*zend_new_interned_string)(char const   *str ,
                                                                                           int len ,
                                                                                           int free_src ) ;
struct _zend_mm_heap;
typedef struct _zend_mm_heap zend_mm_heap;
extern void __attribute__((__visibility__("default")))  zend_mm_set_custom_handlers(zend_mm_heap *heap ,
                                                                                    void *(*_malloc)(size_t  ) ,
                                                                                    void (*_free)(void * ) ,
                                                                                    void *(*_realloc)(void * ,
                                                                                                      size_t  ) ) ;
struct _zend_mm_storage;
typedef struct _zend_mm_storage zend_mm_storage;
extern zend_mm_storage __attribute__((__visibility__("default")))  *zend_mm_get_storage(zend_mm_heap *heap ) ;
extern zend_mm_heap __attribute__((__visibility__("default")))  *zend_mm_set_heap(zend_mm_heap *new_heap ) ;
struct _zend_mm_mem_handlers;
typedef struct _zend_mm_mem_handlers zend_mm_mem_handlers;
extern zend_mm_heap __attribute__((__visibility__("default")))  *zend_mm_startup_ex(zend_mm_mem_handlers const   *handlers ,
                                                                                    size_t block_size ,
                                                                                    size_t reserve_size ,
                                                                                    int internal ,
                                                                                    void *params ) ;
extern size_t __attribute__((__visibility__("default")))  _zend_mm_block_size(zend_mm_heap *heap ,
                                                                              void *p ) ;
extern void __attribute__((__visibility__("default")))  *_zend_mm_realloc(zend_mm_heap *heap ,
                                                                          void *p ,
                                                                          size_t size ) ;
extern void __attribute__((__visibility__("default")))  _zend_mm_free(zend_mm_heap *heap ,
                                                                      void *p ) ;
extern void __attribute__((__visibility__("default")))  *_zend_mm_alloc(zend_mm_heap *heap ,
                                                                        size_t size )  __attribute__((__malloc__)) ;
extern void __attribute__((__visibility__("default")))  zend_mm_shutdown(zend_mm_heap *heap ,
                                                                         int full_shutdown ,
                                                                         int silent ) ;
extern zend_mm_heap __attribute__((__visibility__("default")))  *zend_mm_startup(void) ;
extern size_t __attribute__((__visibility__("default")))  zend_memory_peak_usage(int real_usage ) ;
extern size_t __attribute__((__visibility__("default")))  zend_memory_usage(int real_usage ) ;
extern int __attribute__((__visibility__("default")))  is_zend_mm(void) ;
extern void __attribute__((__visibility__("default")))  shutdown_memory_manager(int silent ,
                                                                                int full_shutdown ) ;
extern void __attribute__((__visibility__("default")))  start_memory_manager(void) ;
extern int __attribute__((__visibility__("default")))  zend_set_memory_limit(size_t memory_limit ) ;
extern  __attribute__((__nothrow__, __noreturn__)) void exit(int __status ) ;
extern struct _IO_FILE *stderr ;
extern  __attribute__((__nothrow__)) void *realloc(void *__ptr , size_t __size )  __attribute__((__warn_unused_result__)) ;
__inline static void *__zend_realloc(void *p , size_t len ) 
{ 

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "305\n");
  fflush(_coverage_fout);
  }
  p = realloc(p, len);
  {
  fprintf(_coverage_fout, "306\n");
  fflush(_coverage_fout);
  }
  if (p) {
    {
    fprintf(_coverage_fout, "303\n");
    fflush(_coverage_fout);
    }
    return (p);
  } else {
    {
    fprintf(_coverage_fout, "304\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "307\n");
  fflush(_coverage_fout);
  }
  fprintf((FILE */* __restrict  */)stderr,
          (char const   */* __restrict  */)"Out of memory\n");
  {
  fprintf(_coverage_fout, "308\n");
  fflush(_coverage_fout);
  }
  exit(1);
}
}
extern void __attribute__((__visibility__("default")))  *_safe_malloc(size_t nmemb ,
                                                                      size_t size ,
                                                                      size_t offset )  __attribute__((__malloc__)) ;
__inline static void *__zend_calloc(size_t nmemb , size_t len ) 
{ void *tmp ;
  void __attribute__((__visibility__("default")))  *tmp___0 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "299\n");
  fflush(_coverage_fout);
  }
  tmp___0 = _safe_malloc(nmemb, len, 0U);
  {
  fprintf(_coverage_fout, "300\n");
  fflush(_coverage_fout);
  }
  tmp = (void *)tmp___0;
  {
  fprintf(_coverage_fout, "301\n");
  fflush(_coverage_fout);
  }
  memset(tmp, 0, nmemb * len);
  {
  fprintf(_coverage_fout, "302\n");
  fflush(_coverage_fout);
  }
  return (tmp);
}
}
__inline static void *__zend_calloc(size_t nmemb , size_t len ) ;
extern  __attribute__((__nothrow__)) void *malloc(size_t __size )  __attribute__((__malloc__)) ;
__inline static void *__zend_malloc(size_t len ) 
{ void *tmp ;
  void *tmp___0 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "294\n");
  fflush(_coverage_fout);
  }
  tmp___0 = malloc(len);
  {
  fprintf(_coverage_fout, "295\n");
  fflush(_coverage_fout);
  }
  tmp = tmp___0;
  {
  fprintf(_coverage_fout, "296\n");
  fflush(_coverage_fout);
  }
  if (tmp) {
    {
    fprintf(_coverage_fout, "292\n");
    fflush(_coverage_fout);
    }
    return (tmp);
  } else {
    {
    fprintf(_coverage_fout, "293\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "297\n");
  fflush(_coverage_fout);
  }
  fprintf((FILE */* __restrict  */)stderr,
          (char const   */* __restrict  */)"Out of memory\n");
  {
  fprintf(_coverage_fout, "298\n");
  fflush(_coverage_fout);
  }
  exit(1);
}
}
__inline static void *__zend_malloc(size_t len ) ;
extern size_t __attribute__((__visibility__("default")))  _zend_mem_block_size(void *ptr ) ;
extern char __attribute__((__visibility__("default")))  *_estrdup(char const   *s )  __attribute__((__malloc__)) ;
extern void __attribute__((__visibility__("default")))  *_safe_realloc(void *ptr ,
                                                                       size_t nmemb ,
                                                                       size_t size ,
                                                                       size_t offset ) ;
extern void __attribute__((__visibility__("default")))  *_safe_erealloc(void *ptr ,
                                                                        size_t nmemb ,
                                                                        size_t size ,
                                                                        size_t offset ) ;
extern void __attribute__((__visibility__("default")))  *_safe_emalloc(size_t nmemb ,
                                                                       size_t size ,
                                                                       size_t offset )  __attribute__((__malloc__)) ;
extern char __attribute__((__visibility__("default")))  *zend_strndup(char const   *s ,
                                                                      unsigned int length )  __attribute__((__malloc__)) ;
extern  __attribute__((__nothrow__)) int dlinfo(void * __restrict  __handle ,
                                                int __request ,
                                                void * __restrict  __arg )  __attribute__((__nonnull__(1,3))) ;
struct __anonstruct_Dl_info_34;
typedef struct __anonstruct_Dl_info_34 Dl_info;
extern  __attribute__((__nothrow__)) int dladdr1(void const   *__address ,
                                                 Dl_info *__info ,
                                                 void **__extra_info ,
                                                 int __flags )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) int dladdr(void const   *__address ,
                                                Dl_info *__info )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) char *dlerror(void) ;
extern  __attribute__((__nothrow__)) void *dlvsym(void * __restrict  __handle ,
                                                  char const   * __restrict  __name ,
                                                  char const   * __restrict  __version )  __attribute__((__nonnull__(2,3))) ;
typedef long Lmid_t;
extern  __attribute__((__nothrow__)) void *dlmopen(Lmid_t __nsid ,
                                                   char const   *__file ,
                                                   int __mode ) ;
extern  __attribute__((__nothrow__)) void *dlsym(void * __restrict  __handle ,
                                                 char const   * __restrict  __name )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) int dlclose(void *__handle )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) void *dlopen(char const   *__file ,
                                                  int __mode ) ;
extern  __attribute__((__nothrow__)) void _dl_mcount_wrapper_check(void *__selfpc ) ;
typedef void _IO_lock_t;
struct _IO_marker;
struct _IO_FILE {
   int _flags ;
   char *_IO_read_ptr ;
   char *_IO_read_end ;
   char *_IO_read_base ;
   char *_IO_write_base ;
   char *_IO_write_ptr ;
   char *_IO_write_end ;
   char *_IO_buf_base ;
   char *_IO_buf_end ;
   char *_IO_save_base ;
   char *_IO_backup_base ;
   char *_IO_save_end ;
   struct _IO_marker *_markers ;
   struct _IO_FILE *_chain ;
   int _fileno ;
   int _flags2 ;
   __off_t _old_offset ;
   unsigned short _cur_column ;
   signed char _vtable_offset ;
   char _shortbuf[1] ;
   _IO_lock_t *_lock ;
   __off64_t _offset ;
   void *__pad1 ;
   void *__pad2 ;
   void *__pad3 ;
   void *__pad4 ;
   size_t __pad5 ;
   int _mode ;
   char _unused2[(15U * sizeof(int ) - 4U * sizeof(void *)) - sizeof(size_t )] ;
};
__inline extern  __attribute__((__nothrow__)) int ferror_unlocked(FILE *__stream ) ;
__inline extern int ferror_unlocked(FILE *__stream ) 
{ 

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "291\n");
  fflush(_coverage_fout);
  }
  return ((__stream->_flags & 0x20) != 0);
}
}
__inline extern  __attribute__((__nothrow__)) int ferror_unlocked(FILE *__stream ) ;
__inline extern  __attribute__((__nothrow__)) int feof_unlocked(FILE *__stream ) ;
__inline extern int feof_unlocked(FILE *__stream ) 
{ 

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "290\n");
  fflush(_coverage_fout);
  }
  return ((__stream->_flags & 0x10) != 0);
}
}
__inline extern  __attribute__((__nothrow__)) int feof_unlocked(FILE *__stream ) ;
extern __ssize_t __getdelim(char ** __restrict  __lineptr ,
                            size_t * __restrict  __n , int __delimiter ,
                            FILE * __restrict  __stream ) ;
__inline extern __ssize_t getline(char ** __restrict  __lineptr ,
                                  size_t * __restrict  __n ,
                                  FILE * __restrict  __stream ) 
{ __ssize_t tmp ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "288\n");
  fflush(_coverage_fout);
  }
  tmp = __getdelim(__lineptr, __n, '\n', __stream);
  {
  fprintf(_coverage_fout, "289\n");
  fflush(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern __ssize_t getline(char ** __restrict  __lineptr ,
                                  size_t * __restrict  __n ,
                                  FILE * __restrict  __stream ) ;
extern struct _IO_FILE *stdout ;
typedef struct _IO_FILE _IO_FILE;
extern int __overflow(_IO_FILE * , int  ) ;
__inline extern int putchar_unlocked(int __c ) 
{ long tmp ;
  int tmp___0 ;
  char *tmp___1 ;
  char tmp___2 ;
  int tmp___3 ;
  long tmp___4 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "285\n");
  fflush(_coverage_fout);
  }
  tmp___4 = __builtin_expect((long )((unsigned int )stdout->_IO_write_ptr >= (unsigned int )stdout->_IO_write_end),
                             0L);
  {
  fprintf(_coverage_fout, "286\n");
  fflush(_coverage_fout);
  }
  if (tmp___4) {
    {
    fprintf(_coverage_fout, "278\n");
    fflush(_coverage_fout);
    }
    tmp___0 = __overflow(stdout, (int )((unsigned char )__c));
    {
    fprintf(_coverage_fout, "279\n");
    fflush(_coverage_fout);
    }
    tmp___3 = tmp___0;
  } else {
    {
    fprintf(_coverage_fout, "280\n");
    fflush(_coverage_fout);
    }
    tmp___1 = stdout->_IO_write_ptr;
    {
    fprintf(_coverage_fout, "281\n");
    fflush(_coverage_fout);
    }
    (stdout->_IO_write_ptr) ++;
    {
    fprintf(_coverage_fout, "282\n");
    fflush(_coverage_fout);
    }
    tmp___2 = (char )__c;
    {
    fprintf(_coverage_fout, "283\n");
    fflush(_coverage_fout);
    }
    *tmp___1 = tmp___2;
    {
    fprintf(_coverage_fout, "284\n");
    fflush(_coverage_fout);
    }
    tmp___3 = (int )((unsigned char )tmp___2);
  }
  {
  fprintf(_coverage_fout, "287\n");
  fflush(_coverage_fout);
  }
  return (tmp___3);
}
}
__inline extern int putchar_unlocked(int __c ) ;
__inline extern int putc_unlocked(int __c , FILE *__stream ) 
{ long tmp ;
  int tmp___0 ;
  char *tmp___1 ;
  char tmp___2 ;
  int tmp___3 ;
  long tmp___4 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "275\n");
  fflush(_coverage_fout);
  }
  tmp___4 = __builtin_expect((long )((unsigned int )__stream->_IO_write_ptr >= (unsigned int )__stream->_IO_write_end),
                             0L);
  {
  fprintf(_coverage_fout, "276\n");
  fflush(_coverage_fout);
  }
  if (tmp___4) {
    {
    fprintf(_coverage_fout, "268\n");
    fflush(_coverage_fout);
    }
    tmp___0 = __overflow(__stream, (int )((unsigned char )__c));
    {
    fprintf(_coverage_fout, "269\n");
    fflush(_coverage_fout);
    }
    tmp___3 = tmp___0;
  } else {
    {
    fprintf(_coverage_fout, "270\n");
    fflush(_coverage_fout);
    }
    tmp___1 = __stream->_IO_write_ptr;
    {
    fprintf(_coverage_fout, "271\n");
    fflush(_coverage_fout);
    }
    (__stream->_IO_write_ptr) ++;
    {
    fprintf(_coverage_fout, "272\n");
    fflush(_coverage_fout);
    }
    tmp___2 = (char )__c;
    {
    fprintf(_coverage_fout, "273\n");
    fflush(_coverage_fout);
    }
    *tmp___1 = tmp___2;
    {
    fprintf(_coverage_fout, "274\n");
    fflush(_coverage_fout);
    }
    tmp___3 = (int )((unsigned char )tmp___2);
  }
  {
  fprintf(_coverage_fout, "277\n");
  fflush(_coverage_fout);
  }
  return (tmp___3);
}
}
__inline extern int putc_unlocked(int __c , FILE *__stream ) ;
__inline extern int fputc_unlocked(int __c , FILE *__stream ) 
{ long tmp ;
  int tmp___0 ;
  char *tmp___1 ;
  char tmp___2 ;
  int tmp___3 ;
  long tmp___4 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "265\n");
  fflush(_coverage_fout);
  }
  tmp___4 = __builtin_expect((long )((unsigned int )__stream->_IO_write_ptr >= (unsigned int )__stream->_IO_write_end),
                             0L);
  {
  fprintf(_coverage_fout, "266\n");
  fflush(_coverage_fout);
  }
  if (tmp___4) {
    {
    fprintf(_coverage_fout, "258\n");
    fflush(_coverage_fout);
    }
    tmp___0 = __overflow(__stream, (int )((unsigned char )__c));
    {
    fprintf(_coverage_fout, "259\n");
    fflush(_coverage_fout);
    }
    tmp___3 = tmp___0;
  } else {
    {
    fprintf(_coverage_fout, "260\n");
    fflush(_coverage_fout);
    }
    tmp___1 = __stream->_IO_write_ptr;
    {
    fprintf(_coverage_fout, "261\n");
    fflush(_coverage_fout);
    }
    (__stream->_IO_write_ptr) ++;
    {
    fprintf(_coverage_fout, "262\n");
    fflush(_coverage_fout);
    }
    tmp___2 = (char )__c;
    {
    fprintf(_coverage_fout, "263\n");
    fflush(_coverage_fout);
    }
    *tmp___1 = tmp___2;
    {
    fprintf(_coverage_fout, "264\n");
    fflush(_coverage_fout);
    }
    tmp___3 = (int )((unsigned char )tmp___2);
  }
  {
  fprintf(_coverage_fout, "267\n");
  fflush(_coverage_fout);
  }
  return (tmp___3);
}
}
__inline extern int fputc_unlocked(int __c , FILE *__stream ) ;
extern int _IO_putc(int __c , _IO_FILE *__fp ) ;
__inline extern int putchar(int __c ) 
{ int tmp ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "256\n");
  fflush(_coverage_fout);
  }
  tmp = _IO_putc(__c, stdout);
  {
  fprintf(_coverage_fout, "257\n");
  fflush(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern int putchar(int __c ) ;
extern struct _IO_FILE *stdin ;
extern int __uflow(_IO_FILE * ) ;
__inline extern int getchar_unlocked(void) 
{ long tmp ;
  int tmp___0 ;
  char *tmp___1 ;
  int tmp___2 ;
  long tmp___3 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "253\n");
  fflush(_coverage_fout);
  }
  tmp___3 = __builtin_expect((long )((unsigned int )stdin->_IO_read_ptr >= (unsigned int )stdin->_IO_read_end),
                             0L);
  {
  fprintf(_coverage_fout, "254\n");
  fflush(_coverage_fout);
  }
  if (tmp___3) {
    {
    fprintf(_coverage_fout, "248\n");
    fflush(_coverage_fout);
    }
    tmp___0 = __uflow(stdin);
    {
    fprintf(_coverage_fout, "249\n");
    fflush(_coverage_fout);
    }
    tmp___2 = tmp___0;
  } else {
    {
    fprintf(_coverage_fout, "250\n");
    fflush(_coverage_fout);
    }
    tmp___1 = stdin->_IO_read_ptr;
    {
    fprintf(_coverage_fout, "251\n");
    fflush(_coverage_fout);
    }
    (stdin->_IO_read_ptr) ++;
    {
    fprintf(_coverage_fout, "252\n");
    fflush(_coverage_fout);
    }
    tmp___2 = (int )*((unsigned char *)tmp___1);
  }
  {
  fprintf(_coverage_fout, "255\n");
  fflush(_coverage_fout);
  }
  return (tmp___2);
}
}
__inline extern int getchar_unlocked(void) ;
__inline extern int getc_unlocked(FILE *__fp ) 
{ long tmp ;
  int tmp___0 ;
  char *tmp___1 ;
  int tmp___2 ;
  long tmp___3 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "245\n");
  fflush(_coverage_fout);
  }
  tmp___3 = __builtin_expect((long )((unsigned int )__fp->_IO_read_ptr >= (unsigned int )__fp->_IO_read_end),
                             0L);
  {
  fprintf(_coverage_fout, "246\n");
  fflush(_coverage_fout);
  }
  if (tmp___3) {
    {
    fprintf(_coverage_fout, "240\n");
    fflush(_coverage_fout);
    }
    tmp___0 = __uflow(__fp);
    {
    fprintf(_coverage_fout, "241\n");
    fflush(_coverage_fout);
    }
    tmp___2 = tmp___0;
  } else {
    {
    fprintf(_coverage_fout, "242\n");
    fflush(_coverage_fout);
    }
    tmp___1 = __fp->_IO_read_ptr;
    {
    fprintf(_coverage_fout, "243\n");
    fflush(_coverage_fout);
    }
    (__fp->_IO_read_ptr) ++;
    {
    fprintf(_coverage_fout, "244\n");
    fflush(_coverage_fout);
    }
    tmp___2 = (int )*((unsigned char *)tmp___1);
  }
  {
  fprintf(_coverage_fout, "247\n");
  fflush(_coverage_fout);
  }
  return (tmp___2);
}
}
__inline extern int getc_unlocked(FILE *__fp ) ;
__inline extern int fgetc_unlocked(FILE *__fp ) 
{ long tmp ;
  int tmp___0 ;
  char *tmp___1 ;
  int tmp___2 ;
  long tmp___3 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "237\n");
  fflush(_coverage_fout);
  }
  tmp___3 = __builtin_expect((long )((unsigned int )__fp->_IO_read_ptr >= (unsigned int )__fp->_IO_read_end),
                             0L);
  {
  fprintf(_coverage_fout, "238\n");
  fflush(_coverage_fout);
  }
  if (tmp___3) {
    {
    fprintf(_coverage_fout, "232\n");
    fflush(_coverage_fout);
    }
    tmp___0 = __uflow(__fp);
    {
    fprintf(_coverage_fout, "233\n");
    fflush(_coverage_fout);
    }
    tmp___2 = tmp___0;
  } else {
    {
    fprintf(_coverage_fout, "234\n");
    fflush(_coverage_fout);
    }
    tmp___1 = __fp->_IO_read_ptr;
    {
    fprintf(_coverage_fout, "235\n");
    fflush(_coverage_fout);
    }
    (__fp->_IO_read_ptr) ++;
    {
    fprintf(_coverage_fout, "236\n");
    fflush(_coverage_fout);
    }
    tmp___2 = (int )*((unsigned char *)tmp___1);
  }
  {
  fprintf(_coverage_fout, "239\n");
  fflush(_coverage_fout);
  }
  return (tmp___2);
}
}
__inline extern int fgetc_unlocked(FILE *__fp ) ;
extern int _IO_getc(_IO_FILE *__fp ) ;
__inline extern int getchar(void) 
{ int tmp ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "230\n");
  fflush(_coverage_fout);
  }
  tmp = _IO_getc(stdin);
  {
  fprintf(_coverage_fout, "231\n");
  fflush(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern int getchar(void) ;
extern int vfprintf(FILE * __restrict  __s ,
                    char const   * __restrict  __format , __gnuc_va_list __arg ) ;
__inline extern int vprintf(char const   * __restrict  __fmt ,
                            __gnuc_va_list __arg ) 
{ int tmp ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "228\n");
  fflush(_coverage_fout);
  }
  tmp = vfprintf((FILE */* __restrict  */)stdout, __fmt, __arg);
  {
  fprintf(_coverage_fout, "229\n");
  fflush(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern int vprintf(char const   * __restrict  __fmt ,
                            __gnuc_va_list __arg ) ;
extern  __attribute__((__nothrow__)) void funlockfile(FILE *__stream ) ;
extern  __attribute__((__nothrow__)) int ftrylockfile(FILE *__stream ) ;
extern  __attribute__((__nothrow__)) void flockfile(FILE *__stream ) ;
struct obstack;
extern  __attribute__((__nothrow__)) int ( /* format attribute */  obstack_vprintf)(struct obstack * __restrict  __obstack ,
                                                                                    char const   * __restrict  __format ,
                                                                                    __gnuc_va_list __args ) ;
extern  __attribute__((__nothrow__)) int ( /* format attribute */  obstack_printf)(struct obstack * __restrict  __obstack ,
                                                                                   char const   * __restrict  __format 
                                                                                   , ...) ;
extern char *cuserid(char *__s ) ;
extern  __attribute__((__nothrow__)) char *ctermid(char *__s ) ;
extern int pclose(FILE *__stream ) ;
extern FILE *popen(char const   *__command , char const   *__modes ) ;
extern  __attribute__((__nothrow__)) int fileno_unlocked(FILE *__stream ) ;
extern  __attribute__((__nothrow__)) int fileno(FILE *__stream ) ;
extern char const   * const  _sys_errlist[] ;
extern int _sys_nerr ;
extern char const   * const  sys_errlist[] ;
extern int sys_nerr ;
extern void perror(char const   *__s ) ;
extern  __attribute__((__nothrow__)) void clearerr_unlocked(FILE *__stream ) ;
extern  __attribute__((__nothrow__)) int ferror(FILE *__stream ) ;
extern  __attribute__((__nothrow__)) int feof(FILE *__stream ) ;
extern  __attribute__((__nothrow__)) void clearerr(FILE *__stream ) ;
struct __anonstruct__G_fpos64_t_32;
typedef struct __anonstruct__G_fpos64_t_32 _G_fpos64_t;
typedef _G_fpos64_t fpos64_t;
extern int fsetpos64(FILE *__stream , fpos64_t const   *__pos ) ;
extern int fgetpos64(FILE * __restrict  __stream , fpos64_t * __restrict  __pos ) ;
extern __off64_t ftello64(FILE *__stream ) ;
extern int fseeko64(FILE *__stream , __off64_t __off , int __whence ) ;
struct __anonstruct__G_fpos_t_31;
typedef struct __anonstruct__G_fpos_t_31 _G_fpos_t;
typedef _G_fpos_t fpos_t;
extern int fsetpos(FILE *__stream , fpos_t const   *__pos ) ;
extern int fgetpos(FILE * __restrict  __stream , fpos_t * __restrict  __pos ) ;
extern __off_t ftello(FILE *__stream ) ;
extern int fseeko(FILE *__stream , __off_t __off , int __whence ) ;
extern void rewind(FILE *__stream ) ;
extern long ftell(FILE *__stream ) ;
extern int fseek(FILE *__stream , long __off , int __whence ) ;
extern size_t fwrite_unlocked(void const   * __restrict  __ptr , size_t __size ,
                              size_t __n , FILE * __restrict  __stream ) ;
extern size_t fread_unlocked(void * __restrict  __ptr , size_t __size ,
                             size_t __n , FILE * __restrict  __stream ) ;
extern int fputs_unlocked(char const   * __restrict  __s ,
                          FILE * __restrict  __stream ) ;
extern size_t fwrite(void const   * __restrict  __ptr , size_t __size ,
                     size_t __n , FILE * __restrict  __s ) ;
extern size_t fread(void * __restrict  __ptr , size_t __size , size_t __n ,
                    FILE * __restrict  __stream ) ;
extern int ungetc(int __c , FILE *__stream ) ;
extern int puts(char const   *__s ) ;
extern int fputs(char const   * __restrict  __s , FILE * __restrict  __stream ) ;
extern __ssize_t getdelim(char ** __restrict  __lineptr ,
                          size_t * __restrict  __n , int __delimiter ,
                          FILE * __restrict  __stream ) ;
extern char *fgets_unlocked(char * __restrict  __s , int __n ,
                            FILE * __restrict  __stream ) ;
extern char *gets(char *__s ) ;
extern char *fgets(char * __restrict  __s , int __n ,
                   FILE * __restrict  __stream ) ;
extern int putw(int __w , FILE *__stream ) ;
extern int getw(FILE *__stream ) ;
extern int putc(int __c , FILE *__stream ) ;
extern int fputc(int __c , FILE *__stream ) ;
extern int getc(FILE *__stream ) ;
extern int fgetc(FILE *__stream ) ;
extern  __attribute__((__nothrow__)) int ( /* format attribute */  vsscanf)(char const   * __restrict  __s ,
                                                                            char const   * __restrict  __format ,
                                                                            __gnuc_va_list __arg ) ;
extern int ( /* format attribute */  vscanf)(char const   * __restrict  __format ,
                                             __gnuc_va_list __arg ) ;
extern int ( /* format attribute */  vfscanf)(FILE * __restrict  __s ,
                                              char const   * __restrict  __format ,
                                              __gnuc_va_list __arg ) ;
extern  __attribute__((__nothrow__)) int sscanf(char const   * __restrict  __s ,
                                                char const   * __restrict  __format 
                                                , ...) ;
extern int scanf(char const   * __restrict  __format  , ...) ;
extern int fscanf(FILE * __restrict  __stream ,
                  char const   * __restrict  __format  , ...) ;
extern int ( /* format attribute */  dprintf)(int __fd ,
                                              char const   * __restrict  __fmt 
                                              , ...) ;
extern int ( /* format attribute */  vdprintf)(int __fd ,
                                               char const   * __restrict  __fmt ,
                                               __gnuc_va_list __arg ) ;
extern  __attribute__((__nothrow__)) int ( /* format attribute */  asprintf)(char ** __restrict  __ptr ,
                                                                             char const   * __restrict  __fmt 
                                                                             , ...) ;
extern  __attribute__((__nothrow__)) int ( /* format attribute */  __asprintf)(char ** __restrict  __ptr ,
                                                                               char const   * __restrict  __fmt 
                                                                               , ...) ;
extern  __attribute__((__nothrow__)) int ( /* format attribute */  vasprintf)(char ** __restrict  __ptr ,
                                                                              char const   * __restrict  __f ,
                                                                              __gnuc_va_list __arg ) ;
extern  __attribute__((__nothrow__)) int ( /* format attribute */  vsnprintf)(char * __restrict  __s ,
                                                                              size_t __maxlen ,
                                                                              char const   * __restrict  __format ,
                                                                              __gnuc_va_list __arg ) ;
extern  __attribute__((__nothrow__)) int ( /* format attribute */  snprintf)(char * __restrict  __s ,
                                                                             size_t __maxlen ,
                                                                             char const   * __restrict  __format 
                                                                             , ...) ;
extern  __attribute__((__nothrow__)) int vsprintf(char * __restrict  __s ,
                                                  char const   * __restrict  __format ,
                                                  __gnuc_va_list __arg ) ;
extern  __attribute__((__nothrow__)) int sprintf(char * __restrict  __s ,
                                                 char const   * __restrict  __format 
                                                 , ...) ;
extern int printf(char const   * __restrict  __format  , ...) ;
extern  __attribute__((__nothrow__)) void setlinebuf(FILE *__stream ) ;
extern  __attribute__((__nothrow__)) void setbuffer(FILE * __restrict  __stream ,
                                                    char * __restrict  __buf ,
                                                    size_t __size ) ;
extern  __attribute__((__nothrow__)) int setvbuf(FILE * __restrict  __stream ,
                                                 char * __restrict  __buf ,
                                                 int __modes , size_t __n ) ;
extern  __attribute__((__nothrow__)) void setbuf(FILE * __restrict  __stream ,
                                                 char * __restrict  __buf ) ;
extern  __attribute__((__nothrow__)) FILE *open_memstream(char **__bufloc ,
                                                          size_t *__sizeloc ) ;
extern  __attribute__((__nothrow__)) FILE *fmemopen(void *__s , size_t __len ,
                                                    char const   *__modes ) ;
struct __anonstruct__IO_cookie_io_functions_t_33;
typedef struct __anonstruct__IO_cookie_io_functions_t_33 _IO_cookie_io_functions_t;
extern  __attribute__((__nothrow__)) FILE *fopencookie(void * __restrict  __magic_cookie ,
                                                       char const   * __restrict  __modes ,
                                                       _IO_cookie_io_functions_t __io_funcs ) ;
extern  __attribute__((__nothrow__)) FILE *fdopen(int __fd ,
                                                  char const   *__modes ) ;
extern FILE *freopen64(char const   * __restrict  __filename ,
                       char const   * __restrict  __modes ,
                       FILE * __restrict  __stream ) ;
extern FILE *fopen64(char const   * __restrict  __filename ,
                     char const   * __restrict  __modes ) ;
extern FILE *freopen(char const   * __restrict  __filename ,
                     char const   * __restrict  __modes ,
                     FILE * __restrict  __stream ) ;
extern int fcloseall(void) ;
extern int fflush_unlocked(FILE *__stream ) ;
extern int fclose(FILE *__stream ) ;
extern  __attribute__((__nothrow__)) char *tempnam(char const   *__dir ,
                                                   char const   *__pfx )  __attribute__((__malloc__)) ;
extern  __attribute__((__nothrow__)) char *tmpnam_r(char *__s ) ;
extern  __attribute__((__nothrow__)) char *tmpnam(char *__s ) ;
extern FILE *tmpfile64(void) ;
extern FILE *tmpfile(void) ;
extern  __attribute__((__nothrow__)) int renameat(int __oldfd ,
                                                  char const   *__old ,
                                                  int __newfd ,
                                                  char const   *__new ) ;
extern  __attribute__((__nothrow__)) int rename(char const   *__old ,
                                                char const   *__new ) ;
extern  __attribute__((__nothrow__)) int remove(char const   *__filename ) ;
extern  __attribute__((__nothrow__)) void _IO_free_backup_area(_IO_FILE * ) ;
extern __off64_t _IO_seekpos(_IO_FILE * , __off64_t  , int  ) ;
extern __off64_t _IO_seekoff(_IO_FILE * , __off64_t  , int  , int  ) ;
extern size_t _IO_sgetn(_IO_FILE * , void * , size_t  ) ;
extern __ssize_t _IO_padn(_IO_FILE * , int  , __ssize_t  ) ;
extern int _IO_vfprintf(_IO_FILE * __restrict   , char const   * __restrict   ,
                        __gnuc_va_list  ) ;
extern int _IO_vfscanf(_IO_FILE * __restrict   , char const   * __restrict   ,
                       __gnuc_va_list  , int * __restrict   ) ;
extern  __attribute__((__nothrow__)) int _IO_ftrylockfile(_IO_FILE * ) ;
extern  __attribute__((__nothrow__)) void _IO_funlockfile(_IO_FILE * ) ;
extern  __attribute__((__nothrow__)) void _IO_flockfile(_IO_FILE * ) ;
extern int _IO_peekc_locked(_IO_FILE *__fp ) ;
extern  __attribute__((__nothrow__)) int _IO_ferror(_IO_FILE *__fp ) ;
extern  __attribute__((__nothrow__)) int _IO_feof(_IO_FILE *__fp ) ;
extern int __underflow(_IO_FILE * ) ;
struct _IO_cookie_file;
extern void _IO_cookie_init(struct _IO_cookie_file *__cfile , int __read_write ,
                            void *__cookie , _IO_cookie_io_functions_t __fns ) ;
struct _IO_FILE_plus;
extern struct _IO_FILE_plus _IO_2_1_stderr_ ;
extern struct _IO_FILE_plus _IO_2_1_stdout_ ;
extern struct _IO_FILE_plus _IO_2_1_stdin_ ;
union __anonunion_28 {
   double __d ;
   int __i[2] ;
};
__inline extern  __attribute__((__nothrow__)) int __finite(double __x )  __attribute__((__const__)) ;
__inline extern int __finite(double __x ) 
{ union __anonunion_28 __constr_expr_0 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "226\n");
  fflush(_coverage_fout);
  }
  __constr_expr_0.__d = __x;
  {
  fprintf(_coverage_fout, "227\n");
  fflush(_coverage_fout);
  }
  return ((int )((((unsigned int )__constr_expr_0.__i[1] | 0x800fffffu) + 1U) >> 31));
}
}
__inline extern  __attribute__((__nothrow__)) int __finite(double __x )  __attribute__((__const__)) ;
__inline extern  __attribute__((__nothrow__)) long long llrintl(long double __x ) ;
__inline extern long long llrintl(long double __x ) 
{ long long __llrintres ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "224\n");
  fflush(_coverage_fout);
  }
  __asm__  volatile   ("fistpll %0": "=m" (__llrintres): "t" (__x): "st");
  {
  fprintf(_coverage_fout, "225\n");
  fflush(_coverage_fout);
  }
  return (__llrintres);
}
}
__inline extern  __attribute__((__nothrow__)) long long llrintl(long double __x ) ;
__inline extern  __attribute__((__nothrow__)) long long llrint(double __x ) ;
__inline extern long long llrint(double __x ) 
{ long long __llrintres ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "222\n");
  fflush(_coverage_fout);
  }
  __asm__  volatile   ("fistpll %0": "=m" (__llrintres): "t" (__x): "st");
  {
  fprintf(_coverage_fout, "223\n");
  fflush(_coverage_fout);
  }
  return (__llrintres);
}
}
__inline extern  __attribute__((__nothrow__)) long long llrint(double __x ) ;
__inline extern  __attribute__((__nothrow__)) long long llrintf(float __x ) ;
__inline extern long long llrintf(float __x ) 
{ long long __llrintres ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "220\n");
  fflush(_coverage_fout);
  }
  __asm__  volatile   ("fistpll %0": "=m" (__llrintres): "t" (__x): "st");
  {
  fprintf(_coverage_fout, "221\n");
  fflush(_coverage_fout);
  }
  return (__llrintres);
}
}
__inline extern  __attribute__((__nothrow__)) long long llrintf(float __x ) ;
__inline extern  __attribute__((__nothrow__)) long lrintl(long double __x ) ;
__inline extern long lrintl(long double __x ) 
{ long __lrintres ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "218\n");
  fflush(_coverage_fout);
  }
  __asm__  volatile   ("fistpl %0": "=m" (__lrintres): "t" (__x): "st");
  {
  fprintf(_coverage_fout, "219\n");
  fflush(_coverage_fout);
  }
  return (__lrintres);
}
}
__inline extern  __attribute__((__nothrow__)) long lrintl(long double __x ) ;
__inline extern  __attribute__((__nothrow__)) long lrint(double __x ) ;
__inline extern long lrint(double __x ) 
{ long __lrintres ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "216\n");
  fflush(_coverage_fout);
  }
  __asm__  volatile   ("fistpl %0": "=m" (__lrintres): "t" (__x): "st");
  {
  fprintf(_coverage_fout, "217\n");
  fflush(_coverage_fout);
  }
  return (__lrintres);
}
}
__inline extern  __attribute__((__nothrow__)) long lrint(double __x ) ;
__inline extern  __attribute__((__nothrow__)) long lrintf(float __x ) ;
__inline extern long lrintf(float __x ) 
{ long __lrintres ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "214\n");
  fflush(_coverage_fout);
  }
  __asm__  volatile   ("fistpl %0": "=m" (__lrintres): "t" (__x): "st");
  {
  fprintf(_coverage_fout, "215\n");
  fflush(_coverage_fout);
  }
  return (__lrintres);
}
}
__inline extern  __attribute__((__nothrow__)) long lrintf(float __x ) ;
__inline extern  __attribute__((__nothrow__)) long double ceill(long double __x )  __attribute__((__const__)) ;
__inline extern long double ceill(long double __x ) 
{ register long double __value ;
  register int __ignore ;
  unsigned short __cw ;
  unsigned short __cwtmp ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "212\n");
  fflush(_coverage_fout);
  }
  __asm__  volatile   ("fnstcw %3\n\t"
                       "movzwl %3, %1\n\t"
                       "andl $0xf3ff, %1\n\t"
                       "orl $0x0800, %1\n\t"
                       "movw %w1, %2\n\t"
                       "fldcw %2\n\t"
                       "frndint\n\t"
                       "fldcw %3": "=t" (__value), "=&q" (__ignore),
                       "=m" (__cwtmp), "=m" (__cw): "0" (__x));
  {
  fprintf(_coverage_fout, "213\n");
  fflush(_coverage_fout);
  }
  return (__value);
}
}
__inline extern  __attribute__((__nothrow__)) long double ceill(long double __x )  __attribute__((__const__)) ;
__inline extern  __attribute__((__nothrow__)) float ceilf(float __x )  __attribute__((__const__)) ;
__inline extern float ceilf(float __x ) 
{ register long double __value ;
  register int __ignore ;
  unsigned short __cw ;
  unsigned short __cwtmp ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "210\n");
  fflush(_coverage_fout);
  }
  __asm__  volatile   ("fnstcw %3\n\t"
                       "movzwl %3, %1\n\t"
                       "andl $0xf3ff, %1\n\t"
                       "orl $0x0800, %1\n\t"
                       "movw %w1, %2\n\t"
                       "fldcw %2\n\t"
                       "frndint\n\t"
                       "fldcw %3": "=t" (__value), "=&q" (__ignore),
                       "=m" (__cwtmp), "=m" (__cw): "0" (__x));
  {
  fprintf(_coverage_fout, "211\n");
  fflush(_coverage_fout);
  }
  return ((float )__value);
}
}
__inline extern  __attribute__((__nothrow__)) float ceilf(float __x )  __attribute__((__const__)) ;
__inline extern  __attribute__((__nothrow__)) double ceil(double __x )  __attribute__((__const__)) ;
__inline extern double ceil(double __x ) 
{ register long double __value ;
  register int __ignore ;
  unsigned short __cw ;
  unsigned short __cwtmp ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "208\n");
  fflush(_coverage_fout);
  }
  __asm__  volatile   ("fnstcw %3\n\t"
                       "movzwl %3, %1\n\t"
                       "andl $0xf3ff, %1\n\t"
                       "orl $0x0800, %1\n\t"
                       "movw %w1, %2\n\t"
                       "fldcw %2\n\t"
                       "frndint\n\t"
                       "fldcw %3": "=t" (__value), "=&q" (__ignore),
                       "=m" (__cwtmp), "=m" (__cw): "0" (__x));
  {
  fprintf(_coverage_fout, "209\n");
  fflush(_coverage_fout);
  }
  return ((double )__value);
}
}
__inline extern  __attribute__((__nothrow__)) double ceil(double __x )  __attribute__((__const__)) ;
__inline extern  __attribute__((__nothrow__)) long double floorl(long double __x )  __attribute__((__const__)) ;
__inline extern long double floorl(long double __x ) 
{ register long double __value ;
  register int __ignore ;
  unsigned short __cw ;
  unsigned short __cwtmp ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "206\n");
  fflush(_coverage_fout);
  }
  __asm__  volatile   ("fnstcw %3\n\t"
                       "movzwl %3, %1\n\t"
                       "andl $0xf3ff, %1\n\t"
                       "orl $0x0400, %1\n\t"
                       "movw %w1, %2\n\t"
                       "fldcw %2\n\t"
                       "frndint\n\t"
                       "fldcw %3": "=t" (__value), "=&q" (__ignore),
                       "=m" (__cwtmp), "=m" (__cw): "0" (__x));
  {
  fprintf(_coverage_fout, "207\n");
  fflush(_coverage_fout);
  }
  return (__value);
}
}
__inline extern  __attribute__((__nothrow__)) long double floorl(long double __x )  __attribute__((__const__)) ;
__inline extern  __attribute__((__nothrow__)) float floorf(float __x )  __attribute__((__const__)) ;
__inline extern float floorf(float __x ) 
{ register long double __value ;
  register int __ignore ;
  unsigned short __cw ;
  unsigned short __cwtmp ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "204\n");
  fflush(_coverage_fout);
  }
  __asm__  volatile   ("fnstcw %3\n\t"
                       "movzwl %3, %1\n\t"
                       "andl $0xf3ff, %1\n\t"
                       "orl $0x0400, %1\n\t"
                       "movw %w1, %2\n\t"
                       "fldcw %2\n\t"
                       "frndint\n\t"
                       "fldcw %3": "=t" (__value), "=&q" (__ignore),
                       "=m" (__cwtmp), "=m" (__cw): "0" (__x));
  {
  fprintf(_coverage_fout, "205\n");
  fflush(_coverage_fout);
  }
  return ((float )__value);
}
}
__inline extern  __attribute__((__nothrow__)) float floorf(float __x )  __attribute__((__const__)) ;
__inline extern  __attribute__((__nothrow__)) double floor(double __x )  __attribute__((__const__)) ;
__inline extern double floor(double __x ) 
{ register long double __value ;
  register int __ignore ;
  unsigned short __cw ;
  unsigned short __cwtmp ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "202\n");
  fflush(_coverage_fout);
  }
  __asm__  volatile   ("fnstcw %3\n\t"
                       "movzwl %3, %1\n\t"
                       "andl $0xf3ff, %1\n\t"
                       "orl $0x0400, %1\n\t"
                       "movw %w1, %2\n\t"
                       "fldcw %2\n\t"
                       "frndint\n\t"
                       "fldcw %3": "=t" (__value), "=&q" (__ignore),
                       "=m" (__cwtmp), "=m" (__cw): "0" (__x));
  {
  fprintf(_coverage_fout, "203\n");
  fflush(_coverage_fout);
  }
  return ((double )__value);
}
}
__inline extern  __attribute__((__nothrow__)) double floor(double __x )  __attribute__((__const__)) ;
union __anonunion___n_27 {
   long double __xld ;
   unsigned int __xi[3] ;
};
__inline extern  __attribute__((__nothrow__)) long double __sgn1l(long double __x ) ;
__inline extern long double __sgn1l(long double __x ) 
{ union __anonunion___n_27 __n ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "197\n");
  fflush(_coverage_fout);
  }
  __n.__xld = __x;
  {
  fprintf(_coverage_fout, "198\n");
  fflush(_coverage_fout);
  }
  __n.__xi[2] = (__n.__xi[2] & 32768U) | 16383U;
  {
  fprintf(_coverage_fout, "199\n");
  fflush(_coverage_fout);
  }
  __n.__xi[1] = 0x80000000;
  {
  fprintf(_coverage_fout, "200\n");
  fflush(_coverage_fout);
  }
  __n.__xi[0] = 0U;
  {
  fprintf(_coverage_fout, "201\n");
  fflush(_coverage_fout);
  }
  return (__n.__xld);
}
}
__inline extern  __attribute__((__nothrow__)) long double __sgn1l(long double __x ) ;
__inline extern  __attribute__((__nothrow__)) long double __fabsl(long double __x )  __attribute__((__const__)) ;
__inline extern long double __fabsl(long double __x ) 
{ long double tmp ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "195\n");
  fflush(_coverage_fout);
  }
  tmp = __builtin_fabsl(__x);
  {
  fprintf(_coverage_fout, "196\n");
  fflush(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) long double __fabsl(long double __x )  __attribute__((__const__)) ;
__inline extern  __attribute__((__nothrow__)) long double fabsl(long double __x )  __attribute__((__const__)) ;
__inline extern long double fabsl(long double __x ) 
{ long double tmp ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "193\n");
  fflush(_coverage_fout);
  }
  tmp = __builtin_fabsl(__x);
  {
  fprintf(_coverage_fout, "194\n");
  fflush(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) long double fabsl(long double __x )  __attribute__((__const__)) ;
__inline extern  __attribute__((__nothrow__)) float fabsf(float __x )  __attribute__((__const__)) ;
__inline extern float fabsf(float __x ) 
{ float tmp ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "191\n");
  fflush(_coverage_fout);
  }
  tmp = __builtin_fabsf(__x);
  {
  fprintf(_coverage_fout, "192\n");
  fflush(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) float fabsf(float __x )  __attribute__((__const__)) ;
__inline extern  __attribute__((__nothrow__)) double fabs(double __x )  __attribute__((__const__)) ;
__inline extern double fabs(double __x ) 
{ double tmp ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "189\n");
  fflush(_coverage_fout);
  }
  tmp = __builtin_fabs(__x);
  {
  fprintf(_coverage_fout, "190\n");
  fflush(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) double fabs(double __x )  __attribute__((__const__)) ;
__inline extern  __attribute__((__nothrow__)) long double __atan2l(long double __y ,
                                                                   long double __x ) ;
__inline extern long double __atan2l(long double __y , long double __x ) 
{ long double tmp ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "187\n");
  fflush(_coverage_fout);
  }
  tmp = __builtin_atan2l(__y, __x);
  {
  fprintf(_coverage_fout, "188\n");
  fflush(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) long double __atan2l(long double __y ,
                                                                   long double __x ) ;
__inline extern  __attribute__((__nothrow__)) long double __sgnl(long double __x ) ;
__inline extern long double __sgnl(long double __x ) 
{ double tmp ;
  double tmp___0 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "185\n");
  fflush(_coverage_fout);
  }
  if (__x == (long double )0.0) {
    {
    fprintf(_coverage_fout, "180\n");
    fflush(_coverage_fout);
    }
    tmp___0 = 0.0;
  } else {
    {
    fprintf(_coverage_fout, "183\n");
    fflush(_coverage_fout);
    }
    if (__x > (long double )0.0) {
      {
      fprintf(_coverage_fout, "181\n");
      fflush(_coverage_fout);
      }
      tmp = 1.0;
    } else {
      {
      fprintf(_coverage_fout, "182\n");
      fflush(_coverage_fout);
      }
      tmp = - 1.0;
    }
    {
    fprintf(_coverage_fout, "184\n");
    fflush(_coverage_fout);
    }
    tmp___0 = tmp;
  }
  {
  fprintf(_coverage_fout, "186\n");
  fflush(_coverage_fout);
  }
  return ((long double )tmp___0);
}
}
__inline extern  __attribute__((__nothrow__)) long double __sgnl(long double __x ) ;
__inline extern  __attribute__((__nothrow__)) float __sgnf(float __x ) ;
__inline extern float __sgnf(float __x ) 
{ double tmp ;
  double tmp___0 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "178\n");
  fflush(_coverage_fout);
  }
  if ((double )__x == 0.0) {
    {
    fprintf(_coverage_fout, "173\n");
    fflush(_coverage_fout);
    }
    tmp___0 = 0.0;
  } else {
    {
    fprintf(_coverage_fout, "176\n");
    fflush(_coverage_fout);
    }
    if ((double )__x > 0.0) {
      {
      fprintf(_coverage_fout, "174\n");
      fflush(_coverage_fout);
      }
      tmp = 1.0;
    } else {
      {
      fprintf(_coverage_fout, "175\n");
      fflush(_coverage_fout);
      }
      tmp = - 1.0;
    }
    {
    fprintf(_coverage_fout, "177\n");
    fflush(_coverage_fout);
    }
    tmp___0 = tmp;
  }
  {
  fprintf(_coverage_fout, "179\n");
  fflush(_coverage_fout);
  }
  return ((float )tmp___0);
}
}
__inline extern  __attribute__((__nothrow__)) float __sgnf(float __x ) ;
__inline extern  __attribute__((__nothrow__)) double __sgn(double __x ) ;
__inline extern double __sgn(double __x ) 
{ double tmp ;
  double tmp___0 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "171\n");
  fflush(_coverage_fout);
  }
  if (__x == 0.0) {
    {
    fprintf(_coverage_fout, "166\n");
    fflush(_coverage_fout);
    }
    tmp___0 = 0.0;
  } else {
    {
    fprintf(_coverage_fout, "169\n");
    fflush(_coverage_fout);
    }
    if (__x > 0.0) {
      {
      fprintf(_coverage_fout, "167\n");
      fflush(_coverage_fout);
      }
      tmp = 1.0;
    } else {
      {
      fprintf(_coverage_fout, "168\n");
      fflush(_coverage_fout);
      }
      tmp = - 1.0;
    }
    {
    fprintf(_coverage_fout, "170\n");
    fflush(_coverage_fout);
    }
    tmp___0 = tmp;
  }
  {
  fprintf(_coverage_fout, "172\n");
  fflush(_coverage_fout);
  }
  return (tmp___0);
}
}
__inline extern  __attribute__((__nothrow__)) double __sgn(double __x ) ;
union __anonunion___u_26 {
   long double __l ;
   int __i[3] ;
};
__inline extern  __attribute__((__nothrow__)) int __signbitl(long double __x )  __attribute__((__const__)) ;
__inline extern int __signbitl(long double __x ) 
{ union __anonunion___u_26 __u ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "164\n");
  fflush(_coverage_fout);
  }
  __u.__l = __x;
  {
  fprintf(_coverage_fout, "165\n");
  fflush(_coverage_fout);
  }
  return ((__u.__i[2] & 0x8000) != 0);
}
}
__inline extern  __attribute__((__nothrow__)) int __signbitl(long double __x )  __attribute__((__const__)) ;
union __anonunion___u_25 {
   double __d ;
   int __i[2] ;
};
__inline extern  __attribute__((__nothrow__)) int __signbit(double __x )  __attribute__((__const__)) ;
__inline extern int __signbit(double __x ) 
{ union __anonunion___u_25 __u ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "162\n");
  fflush(_coverage_fout);
  }
  __u.__d = __x;
  {
  fprintf(_coverage_fout, "163\n");
  fflush(_coverage_fout);
  }
  return (__u.__i[1] < 0);
}
}
__inline extern  __attribute__((__nothrow__)) int __signbit(double __x )  __attribute__((__const__)) ;
union __anonunion___u_24 {
   float __f ;
   int __i ;
};
__inline extern  __attribute__((__nothrow__)) int __signbitf(float __x )  __attribute__((__const__)) ;
__inline extern int __signbitf(float __x ) 
{ union __anonunion___u_24 __u ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "160\n");
  fflush(_coverage_fout);
  }
  __u.__f = __x;
  {
  fprintf(_coverage_fout, "161\n");
  fflush(_coverage_fout);
  }
  return (__u.__i < 0);
}
}
__inline extern  __attribute__((__nothrow__)) int __signbitf(float __x )  __attribute__((__const__)) ;
struct exception;
extern int matherr(struct exception *__exc ) ;
enum __anonenum__LIB_VERSION_TYPE_23;
typedef enum __anonenum__LIB_VERSION_TYPE_23 _LIB_VERSION_TYPE;
extern _LIB_VERSION_TYPE _LIB_VERSION ;
extern int signgam ;
extern  __attribute__((__nothrow__)) long double __scalbl(long double __x ,
                                                          long double __n ) ;
extern  __attribute__((__nothrow__)) long double scalbl(long double __x ,
                                                        long double __n ) ;
extern  __attribute__((__nothrow__)) long double __fmal(long double __x ,
                                                        long double __y ,
                                                        long double __z ) ;
extern  __attribute__((__nothrow__)) long double fmal(long double __x ,
                                                      long double __y ,
                                                      long double __z ) ;
extern  __attribute__((__nothrow__)) int __fpclassifyl(long double __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double __fminl(long double __x ,
                                                         long double __y ) ;
extern  __attribute__((__nothrow__)) long double fminl(long double __x ,
                                                       long double __y ) ;
extern  __attribute__((__nothrow__)) long double __fmaxl(long double __x ,
                                                         long double __y ) ;
extern  __attribute__((__nothrow__)) long double fmaxl(long double __x ,
                                                       long double __y ) ;
extern  __attribute__((__nothrow__)) long double __fdiml(long double __x ,
                                                         long double __y ) ;
extern  __attribute__((__nothrow__)) long double fdiml(long double __x ,
                                                       long double __y ) ;
extern  __attribute__((__nothrow__)) long long __llroundl(long double __x ) ;
extern  __attribute__((__nothrow__)) long long llroundl(long double __x ) ;
extern  __attribute__((__nothrow__)) long __lroundl(long double __x ) ;
extern  __attribute__((__nothrow__)) long lroundl(long double __x ) ;
extern  __attribute__((__nothrow__)) long long __llrintl(long double __x ) ;
extern  __attribute__((__nothrow__)) long __lrintl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __remquol(long double __x ,
                                                           long double __y ,
                                                           int *__quo ) ;
extern  __attribute__((__nothrow__)) long double remquol(long double __x ,
                                                         long double __y ,
                                                         int *__quo ) ;
extern  __attribute__((__nothrow__)) long double __truncl(long double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double truncl(long double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double __roundl(long double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double roundl(long double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double __nearbyintl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double nearbyintl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __scalblnl(long double __x ,
                                                            long __n ) ;
extern  __attribute__((__nothrow__)) long double scalblnl(long double __x ,
                                                          long __n ) ;
extern  __attribute__((__nothrow__)) int __ilogbl(long double __x ) ;
extern  __attribute__((__nothrow__)) int ilogbl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __scalbnl(long double __x ,
                                                           int __n ) ;
extern  __attribute__((__nothrow__)) long double scalbnl(long double __x ,
                                                         int __n ) ;
extern  __attribute__((__nothrow__)) long double __remainderl(long double __x ,
                                                              long double __y ) ;
extern  __attribute__((__nothrow__)) long double remainderl(long double __x ,
                                                            long double __y ) ;
extern  __attribute__((__nothrow__)) long double __nexttowardl(long double __x ,
                                                               long double __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double nexttowardl(long double __x ,
                                                             long double __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double __nextafterl(long double __x ,
                                                              long double __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double nextafterl(long double __x ,
                                                            long double __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double __rintl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double rintl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __lgammal_r(long double  ,
                                                             int *__signgamp ) ;
extern  __attribute__((__nothrow__)) long double lgammal_r(long double  ,
                                                           int *__signgamp ) ;
extern  __attribute__((__nothrow__)) long double __gammal(long double  ) ;
extern  __attribute__((__nothrow__)) long double gammal(long double  ) ;
extern  __attribute__((__nothrow__)) long double __tgammal(long double  ) ;
extern  __attribute__((__nothrow__)) long double tgammal(long double  ) ;
extern  __attribute__((__nothrow__)) long double __lgammal(long double  ) ;
extern  __attribute__((__nothrow__)) long double lgammal(long double  ) ;
extern  __attribute__((__nothrow__)) long double __erfcl(long double  ) ;
extern  __attribute__((__nothrow__)) long double erfcl(long double  ) ;
extern  __attribute__((__nothrow__)) long double __erfl(long double  ) ;
extern  __attribute__((__nothrow__)) long double erfl(long double  ) ;
extern  __attribute__((__nothrow__)) long double __ynl(int  , long double  ) ;
extern  __attribute__((__nothrow__)) long double ynl(int  , long double  ) ;
extern  __attribute__((__nothrow__)) long double __y1l(long double  ) ;
extern  __attribute__((__nothrow__)) long double y1l(long double  ) ;
extern  __attribute__((__nothrow__)) long double __y0l(long double  ) ;
extern  __attribute__((__nothrow__)) long double y0l(long double  ) ;
extern  __attribute__((__nothrow__)) long double __jnl(int  , long double  ) ;
extern  __attribute__((__nothrow__)) long double jnl(int  , long double  ) ;
extern  __attribute__((__nothrow__)) long double __j1l(long double  ) ;
extern  __attribute__((__nothrow__)) long double j1l(long double  ) ;
extern  __attribute__((__nothrow__)) long double __j0l(long double  ) ;
extern  __attribute__((__nothrow__)) long double j0l(long double  ) ;
extern  __attribute__((__nothrow__)) int isnanl(long double __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int __isnanl(long double __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double __nanl(char const   *__tagb )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double nanl(char const   *__tagb )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double __copysignl(long double __x ,
                                                             long double __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double copysignl(long double __x ,
                                                           long double __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double __significandl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double significandl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __dreml(long double __x ,
                                                         long double __y ) ;
extern  __attribute__((__nothrow__)) long double dreml(long double __x ,
                                                       long double __y ) ;
extern  __attribute__((__nothrow__)) int finitel(long double __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int isinfl(long double __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int __finitel(long double __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int __isinfl(long double __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double __fmodl(long double __x ,
                                                         long double __y ) ;
extern  __attribute__((__nothrow__)) long double fmodl(long double __x ,
                                                       long double __y ) ;
extern  __attribute__((__nothrow__)) long double __floorl(long double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double __ceill(long double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double __cbrtl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double cbrtl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __hypotl(long double __x ,
                                                          long double __y ) ;
extern  __attribute__((__nothrow__)) long double hypotl(long double __x ,
                                                        long double __y ) ;
extern  __attribute__((__nothrow__)) long double __sqrtl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double sqrtl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __powl(long double __x ,
                                                        long double __y ) ;
extern  __attribute__((__nothrow__)) long double powl(long double __x ,
                                                      long double __y ) ;
extern  __attribute__((__nothrow__)) long double __log2l(long double __x ) ;
extern  __attribute__((__nothrow__)) long double log2l(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __exp2l(long double __x ) ;
extern  __attribute__((__nothrow__)) long double exp2l(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __logbl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double logbl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __log1pl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double log1pl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __expm1l(long double __x ) ;
extern  __attribute__((__nothrow__)) long double expm1l(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __pow10l(long double __x ) ;
extern  __attribute__((__nothrow__)) long double pow10l(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __exp10l(long double __x ) ;
extern  __attribute__((__nothrow__)) long double exp10l(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __modfl(long double __x ,
                                                         long double *__iptr ) ;
extern  __attribute__((__nothrow__)) long double modfl(long double __x ,
                                                       long double *__iptr ) ;
extern  __attribute__((__nothrow__)) long double __log10l(long double __x ) ;
extern  __attribute__((__nothrow__)) long double log10l(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __logl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double logl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __ldexpl(long double __x ,
                                                          int __exponent ) ;
extern  __attribute__((__nothrow__)) long double ldexpl(long double __x ,
                                                        int __exponent ) ;
extern  __attribute__((__nothrow__)) long double __frexpl(long double __x ,
                                                          int *__exponent ) ;
extern  __attribute__((__nothrow__)) long double frexpl(long double __x ,
                                                        int *__exponent ) ;
extern  __attribute__((__nothrow__)) long double __expl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double expl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __atanhl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double atanhl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __asinhl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double asinhl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __acoshl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double acoshl(long double __x ) ;
extern  __attribute__((__nothrow__)) void __sincosl(long double __x ,
                                                    long double *__sinx ,
                                                    long double *__cosx ) ;
extern  __attribute__((__nothrow__)) void sincosl(long double __x ,
                                                  long double *__sinx ,
                                                  long double *__cosx ) ;
extern  __attribute__((__nothrow__)) long double __tanhl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double tanhl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __sinhl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double sinhl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __coshl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double coshl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __tanl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double tanl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __sinl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double sinl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __cosl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double cosl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double atan2l(long double __y ,
                                                        long double __x ) ;
extern  __attribute__((__nothrow__)) long double __atanl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double atanl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __asinl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double asinl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __acosl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double acosl(long double __x ) ;
extern  __attribute__((__nothrow__)) float __scalbf(float __x , float __n ) ;
extern  __attribute__((__nothrow__)) float scalbf(float __x , float __n ) ;
extern  __attribute__((__nothrow__)) float __fmaf(float __x , float __y ,
                                                  float __z ) ;
extern  __attribute__((__nothrow__)) float fmaf(float __x , float __y ,
                                                float __z ) ;
extern  __attribute__((__nothrow__)) int __fpclassifyf(float __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float __fminf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) float fminf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) float __fmaxf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) float fmaxf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) float __fdimf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) float fdimf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) long long __llroundf(float __x ) ;
extern  __attribute__((__nothrow__)) long long llroundf(float __x ) ;
extern  __attribute__((__nothrow__)) long __lroundf(float __x ) ;
extern  __attribute__((__nothrow__)) long lroundf(float __x ) ;
extern  __attribute__((__nothrow__)) long long __llrintf(float __x ) ;
extern  __attribute__((__nothrow__)) long __lrintf(float __x ) ;
extern  __attribute__((__nothrow__)) float __remquof(float __x , float __y ,
                                                     int *__quo ) ;
extern  __attribute__((__nothrow__)) float remquof(float __x , float __y ,
                                                   int *__quo ) ;
extern  __attribute__((__nothrow__)) float __truncf(float __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float truncf(float __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float __roundf(float __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float roundf(float __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float __nearbyintf(float __x ) ;
extern  __attribute__((__nothrow__)) float nearbyintf(float __x ) ;
extern  __attribute__((__nothrow__)) float __scalblnf(float __x , long __n ) ;
extern  __attribute__((__nothrow__)) float scalblnf(float __x , long __n ) ;
extern  __attribute__((__nothrow__)) int __ilogbf(float __x ) ;
extern  __attribute__((__nothrow__)) int ilogbf(float __x ) ;
extern  __attribute__((__nothrow__)) float __scalbnf(float __x , int __n ) ;
extern  __attribute__((__nothrow__)) float scalbnf(float __x , int __n ) ;
extern  __attribute__((__nothrow__)) float __remainderf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) float remainderf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) float __nexttowardf(float __x ,
                                                         long double __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float nexttowardf(float __x ,
                                                       long double __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float __nextafterf(float __x , float __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float nextafterf(float __x , float __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float __rintf(float __x ) ;
extern  __attribute__((__nothrow__)) float rintf(float __x ) ;
extern  __attribute__((__nothrow__)) float __lgammaf_r(float  , int *__signgamp ) ;
extern  __attribute__((__nothrow__)) float lgammaf_r(float  , int *__signgamp ) ;
extern  __attribute__((__nothrow__)) float __gammaf(float  ) ;
extern  __attribute__((__nothrow__)) float gammaf(float  ) ;
extern  __attribute__((__nothrow__)) float __tgammaf(float  ) ;
extern  __attribute__((__nothrow__)) float tgammaf(float  ) ;
extern  __attribute__((__nothrow__)) float __lgammaf(float  ) ;
extern  __attribute__((__nothrow__)) float lgammaf(float  ) ;
extern  __attribute__((__nothrow__)) float __erfcf(float  ) ;
extern  __attribute__((__nothrow__)) float erfcf(float  ) ;
extern  __attribute__((__nothrow__)) float __erff(float  ) ;
extern  __attribute__((__nothrow__)) float erff(float  ) ;
extern  __attribute__((__nothrow__)) float __ynf(int  , float  ) ;
extern  __attribute__((__nothrow__)) float ynf(int  , float  ) ;
extern  __attribute__((__nothrow__)) float __y1f(float  ) ;
extern  __attribute__((__nothrow__)) float y1f(float  ) ;
extern  __attribute__((__nothrow__)) float __y0f(float  ) ;
extern  __attribute__((__nothrow__)) float y0f(float  ) ;
extern  __attribute__((__nothrow__)) float __jnf(int  , float  ) ;
extern  __attribute__((__nothrow__)) float jnf(int  , float  ) ;
extern  __attribute__((__nothrow__)) float __j1f(float  ) ;
extern  __attribute__((__nothrow__)) float j1f(float  ) ;
extern  __attribute__((__nothrow__)) float __j0f(float  ) ;
extern  __attribute__((__nothrow__)) float j0f(float  ) ;
extern  __attribute__((__nothrow__)) int isnanf(float __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int __isnanf(float __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float __nanf(char const   *__tagb )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float nanf(char const   *__tagb )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float __copysignf(float __x , float __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float copysignf(float __x , float __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float __significandf(float __x ) ;
extern  __attribute__((__nothrow__)) float significandf(float __x ) ;
extern  __attribute__((__nothrow__)) float __dremf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) float dremf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) int finitef(float __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int isinff(float __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int __finitef(float __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int __isinff(float __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float __fmodf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) float fmodf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) float __floorf(float __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float __fabsf(float __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float __ceilf(float __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float __cbrtf(float __x ) ;
extern  __attribute__((__nothrow__)) float cbrtf(float __x ) ;
extern  __attribute__((__nothrow__)) float __hypotf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) float hypotf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) float __sqrtf(float __x ) ;
extern  __attribute__((__nothrow__)) float sqrtf(float __x ) ;
extern  __attribute__((__nothrow__)) float __powf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) float powf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) float __log2f(float __x ) ;
extern  __attribute__((__nothrow__)) float log2f(float __x ) ;
extern  __attribute__((__nothrow__)) float __exp2f(float __x ) ;
extern  __attribute__((__nothrow__)) float exp2f(float __x ) ;
extern  __attribute__((__nothrow__)) float __logbf(float __x ) ;
extern  __attribute__((__nothrow__)) float logbf(float __x ) ;
extern  __attribute__((__nothrow__)) float __log1pf(float __x ) ;
extern  __attribute__((__nothrow__)) float log1pf(float __x ) ;
extern  __attribute__((__nothrow__)) float __expm1f(float __x ) ;
extern  __attribute__((__nothrow__)) float expm1f(float __x ) ;
extern  __attribute__((__nothrow__)) float __pow10f(float __x ) ;
extern  __attribute__((__nothrow__)) float pow10f(float __x ) ;
extern  __attribute__((__nothrow__)) float __exp10f(float __x ) ;
extern  __attribute__((__nothrow__)) float exp10f(float __x ) ;
extern  __attribute__((__nothrow__)) float __modff(float __x , float *__iptr ) ;
extern  __attribute__((__nothrow__)) float modff(float __x , float *__iptr ) ;
extern  __attribute__((__nothrow__)) float __log10f(float __x ) ;
extern  __attribute__((__nothrow__)) float log10f(float __x ) ;
extern  __attribute__((__nothrow__)) float __logf(float __x ) ;
extern  __attribute__((__nothrow__)) float logf(float __x ) ;
extern  __attribute__((__nothrow__)) float __ldexpf(float __x , int __exponent ) ;
extern  __attribute__((__nothrow__)) float ldexpf(float __x , int __exponent ) ;
extern  __attribute__((__nothrow__)) float __frexpf(float __x , int *__exponent ) ;
extern  __attribute__((__nothrow__)) float frexpf(float __x , int *__exponent ) ;
extern  __attribute__((__nothrow__)) float __expf(float __x ) ;
extern  __attribute__((__nothrow__)) float expf(float __x ) ;
extern  __attribute__((__nothrow__)) float __atanhf(float __x ) ;
extern  __attribute__((__nothrow__)) float atanhf(float __x ) ;
extern  __attribute__((__nothrow__)) float __asinhf(float __x ) ;
extern  __attribute__((__nothrow__)) float asinhf(float __x ) ;
extern  __attribute__((__nothrow__)) float __acoshf(float __x ) ;
extern  __attribute__((__nothrow__)) float acoshf(float __x ) ;
extern  __attribute__((__nothrow__)) void __sincosf(float __x , float *__sinx ,
                                                    float *__cosx ) ;
extern  __attribute__((__nothrow__)) void sincosf(float __x , float *__sinx ,
                                                  float *__cosx ) ;
extern  __attribute__((__nothrow__)) float __tanhf(float __x ) ;
extern  __attribute__((__nothrow__)) float tanhf(float __x ) ;
extern  __attribute__((__nothrow__)) float __sinhf(float __x ) ;
extern  __attribute__((__nothrow__)) float sinhf(float __x ) ;
extern  __attribute__((__nothrow__)) float __coshf(float __x ) ;
extern  __attribute__((__nothrow__)) float coshf(float __x ) ;
extern  __attribute__((__nothrow__)) float __tanf(float __x ) ;
extern  __attribute__((__nothrow__)) float tanf(float __x ) ;
extern  __attribute__((__nothrow__)) float __sinf(float __x ) ;
extern  __attribute__((__nothrow__)) float sinf(float __x ) ;
extern  __attribute__((__nothrow__)) float __cosf(float __x ) ;
extern  __attribute__((__nothrow__)) float cosf(float __x ) ;
extern  __attribute__((__nothrow__)) float __atan2f(float __y , float __x ) ;
extern  __attribute__((__nothrow__)) float atan2f(float __y , float __x ) ;
extern  __attribute__((__nothrow__)) float __atanf(float __x ) ;
extern  __attribute__((__nothrow__)) float atanf(float __x ) ;
extern  __attribute__((__nothrow__)) float __asinf(float __x ) ;
extern  __attribute__((__nothrow__)) float asinf(float __x ) ;
extern  __attribute__((__nothrow__)) float __acosf(float __x ) ;
extern  __attribute__((__nothrow__)) float acosf(float __x ) ;
extern  __attribute__((__nothrow__)) double __scalb(double __x , double __n ) ;
extern  __attribute__((__nothrow__)) double scalb(double __x , double __n ) ;
extern  __attribute__((__nothrow__)) double __fma(double __x , double __y ,
                                                  double __z ) ;
extern  __attribute__((__nothrow__)) double fma(double __x , double __y ,
                                                double __z ) ;
extern  __attribute__((__nothrow__)) int __fpclassify(double __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double __fmin(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) double fmin(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) double __fmax(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) double fmax(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) double __fdim(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) double fdim(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) long long __llround(double __x ) ;
extern  __attribute__((__nothrow__)) long long llround(double __x ) ;
extern  __attribute__((__nothrow__)) long __lround(double __x ) ;
extern  __attribute__((__nothrow__)) long lround(double __x ) ;
extern  __attribute__((__nothrow__)) long long __llrint(double __x ) ;
extern  __attribute__((__nothrow__)) long __lrint(double __x ) ;
extern  __attribute__((__nothrow__)) double __remquo(double __x , double __y ,
                                                     int *__quo ) ;
extern  __attribute__((__nothrow__)) double remquo(double __x , double __y ,
                                                   int *__quo ) ;
extern  __attribute__((__nothrow__)) double __trunc(double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double trunc(double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double __round(double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double round(double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double __nearbyint(double __x ) ;
extern  __attribute__((__nothrow__)) double nearbyint(double __x ) ;
extern  __attribute__((__nothrow__)) double __scalbln(double __x , long __n ) ;
extern  __attribute__((__nothrow__)) double scalbln(double __x , long __n ) ;
extern  __attribute__((__nothrow__)) int __ilogb(double __x ) ;
extern  __attribute__((__nothrow__)) int ilogb(double __x ) ;
extern  __attribute__((__nothrow__)) double __scalbn(double __x , int __n ) ;
extern  __attribute__((__nothrow__)) double scalbn(double __x , int __n ) ;
extern  __attribute__((__nothrow__)) double __remainder(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) double remainder(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) double __nexttoward(double __x ,
                                                         long double __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double nexttoward(double __x ,
                                                       long double __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double __nextafter(double __x , double __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double nextafter(double __x , double __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double __rint(double __x ) ;
extern  __attribute__((__nothrow__)) double rint(double __x ) ;
extern  __attribute__((__nothrow__)) double __lgamma_r(double  ,
                                                       int *__signgamp ) ;
extern  __attribute__((__nothrow__)) double lgamma_r(double  , int *__signgamp ) ;
extern  __attribute__((__nothrow__)) double __gamma(double  ) ;
extern  __attribute__((__nothrow__)) double gamma(double  ) ;
extern  __attribute__((__nothrow__)) double __tgamma(double  ) ;
extern  __attribute__((__nothrow__)) double tgamma(double  ) ;
extern  __attribute__((__nothrow__)) double __lgamma(double  ) ;
extern  __attribute__((__nothrow__)) double lgamma(double  ) ;
extern  __attribute__((__nothrow__)) double __erfc(double  ) ;
extern  __attribute__((__nothrow__)) double erfc(double  ) ;
extern  __attribute__((__nothrow__)) double __erf(double  ) ;
extern  __attribute__((__nothrow__)) double erf(double  ) ;
extern  __attribute__((__nothrow__)) double __yn(int  , double  ) ;
extern  __attribute__((__nothrow__)) double yn(int  , double  ) ;
extern  __attribute__((__nothrow__)) double __y1(double  ) ;
extern  __attribute__((__nothrow__)) double y1(double  ) ;
extern  __attribute__((__nothrow__)) double __y0(double  ) ;
extern  __attribute__((__nothrow__)) double y0(double  ) ;
extern  __attribute__((__nothrow__)) double __jn(int  , double  ) ;
extern  __attribute__((__nothrow__)) double jn(int  , double  ) ;
extern  __attribute__((__nothrow__)) double __j1(double  ) ;
extern  __attribute__((__nothrow__)) double j1(double  ) ;
extern  __attribute__((__nothrow__)) double __j0(double  ) ;
extern  __attribute__((__nothrow__)) double j0(double  ) ;
extern  __attribute__((__nothrow__)) int isnan(double __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int __isnan(double __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double __nan(char const   *__tagb )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double nan(char const   *__tagb )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double __copysign(double __x , double __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double copysign(double __x , double __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double __significand(double __x ) ;
extern  __attribute__((__nothrow__)) double significand(double __x ) ;
extern  __attribute__((__nothrow__)) double __drem(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) double drem(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) int finite(double __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int isinf(double __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int __isinf(double __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double __fmod(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) double fmod(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) double __floor(double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double __fabs(double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double __ceil(double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double __cbrt(double __x ) ;
extern  __attribute__((__nothrow__)) double cbrt(double __x ) ;
extern  __attribute__((__nothrow__)) double __hypot(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) double hypot(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) double __sqrt(double __x ) ;
extern  __attribute__((__nothrow__)) double sqrt(double __x ) ;
extern  __attribute__((__nothrow__)) double __pow(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) double pow(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) double __log2(double __x ) ;
extern  __attribute__((__nothrow__)) double log2(double __x ) ;
extern  __attribute__((__nothrow__)) double __exp2(double __x ) ;
extern  __attribute__((__nothrow__)) double exp2(double __x ) ;
extern  __attribute__((__nothrow__)) double __logb(double __x ) ;
extern  __attribute__((__nothrow__)) double logb(double __x ) ;
extern  __attribute__((__nothrow__)) double __log1p(double __x ) ;
extern  __attribute__((__nothrow__)) double log1p(double __x ) ;
extern  __attribute__((__nothrow__)) double __expm1(double __x ) ;
extern  __attribute__((__nothrow__)) double expm1(double __x ) ;
extern  __attribute__((__nothrow__)) double __pow10(double __x ) ;
extern  __attribute__((__nothrow__)) double pow10(double __x ) ;
extern  __attribute__((__nothrow__)) double __exp10(double __x ) ;
extern  __attribute__((__nothrow__)) double exp10(double __x ) ;
extern  __attribute__((__nothrow__)) double __modf(double __x , double *__iptr ) ;
extern  __attribute__((__nothrow__)) double modf(double __x , double *__iptr ) ;
extern  __attribute__((__nothrow__)) double __log10(double __x ) ;
extern  __attribute__((__nothrow__)) double log10(double __x ) ;
extern  __attribute__((__nothrow__)) double __log(double __x ) ;
extern  __attribute__((__nothrow__)) double log(double __x ) ;
extern  __attribute__((__nothrow__)) double __ldexp(double __x , int __exponent ) ;
extern  __attribute__((__nothrow__)) double ldexp(double __x , int __exponent ) ;
extern  __attribute__((__nothrow__)) double __frexp(double __x ,
                                                    int *__exponent ) ;
extern  __attribute__((__nothrow__)) double frexp(double __x , int *__exponent ) ;
extern  __attribute__((__nothrow__)) double __exp(double __x ) ;
extern  __attribute__((__nothrow__)) double exp(double __x ) ;
extern  __attribute__((__nothrow__)) double __atanh(double __x ) ;
extern  __attribute__((__nothrow__)) double atanh(double __x ) ;
extern  __attribute__((__nothrow__)) double __asinh(double __x ) ;
extern  __attribute__((__nothrow__)) double asinh(double __x ) ;
extern  __attribute__((__nothrow__)) double __acosh(double __x ) ;
extern  __attribute__((__nothrow__)) double acosh(double __x ) ;
extern  __attribute__((__nothrow__)) void __sincos(double __x , double *__sinx ,
                                                   double *__cosx ) ;
extern  __attribute__((__nothrow__)) void sincos(double __x , double *__sinx ,
                                                 double *__cosx ) ;
extern  __attribute__((__nothrow__)) double __tanh(double __x ) ;
extern  __attribute__((__nothrow__)) double tanh(double __x ) ;
extern  __attribute__((__nothrow__)) double __sinh(double __x ) ;
extern  __attribute__((__nothrow__)) double sinh(double __x ) ;
extern  __attribute__((__nothrow__)) double __cosh(double __x ) ;
extern  __attribute__((__nothrow__)) double cosh(double __x ) ;
extern  __attribute__((__nothrow__)) double __tan(double __x ) ;
extern  __attribute__((__nothrow__)) double tan(double __x ) ;
extern  __attribute__((__nothrow__)) double __sin(double __x ) ;
extern  __attribute__((__nothrow__)) double sin(double __x ) ;
extern  __attribute__((__nothrow__)) double __cos(double __x ) ;
extern  __attribute__((__nothrow__)) double cos(double __x ) ;
extern  __attribute__((__nothrow__)) double __atan2(double __y , double __x ) ;
extern  __attribute__((__nothrow__)) double atan2(double __y , double __x ) ;
extern  __attribute__((__nothrow__)) double __atan(double __x ) ;
extern  __attribute__((__nothrow__)) double atan(double __x ) ;
extern  __attribute__((__nothrow__)) double __asin(double __x ) ;
extern  __attribute__((__nothrow__)) double asin(double __x ) ;
extern  __attribute__((__nothrow__)) double __acos(double __x ) ;
extern  __attribute__((__nothrow__)) double acos(double __x ) ;
extern  __attribute__((__nothrow__)) char *__strndup(char const   *__string ,
                                                     size_t __n )  __attribute__((__malloc__)) ;
extern  __attribute__((__nothrow__)) char *__strdup(char const   *__string )  __attribute__((__malloc__)) ;
__inline extern char *__strsep_3c(char **__s , char __reject1 , char __reject2 ,
                                  char __reject3 ) 
{ register char *__retval ;
  register char *__cp ;
  char *tmp ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "157\n");
  fflush(_coverage_fout);
  }
  __retval = *__s;
  {
  fprintf(_coverage_fout, "158\n");
  fflush(_coverage_fout);
  }
  if ((unsigned int )__retval != (unsigned int )((void *)0)) {
    {
    fprintf(_coverage_fout, "153\n");
    fflush(_coverage_fout);
    }
    __cp = __retval;
    {
    fprintf(_coverage_fout, "154\n");
    fflush(_coverage_fout);
    }
    while (1) {
      {
      fprintf(_coverage_fout, "150\n");
      fflush(_coverage_fout);
      }
      if ((int )*__cp == 0) {
        {
        fprintf(_coverage_fout, "136\n");
        fflush(_coverage_fout);
        }
        __cp = (char *)((void *)0);
        break;
      } else {
        {
        fprintf(_coverage_fout, "137\n");
        fflush(_coverage_fout);
        }

      }
      {
      fprintf(_coverage_fout, "151\n");
      fflush(_coverage_fout);
      }
      if ((int )*__cp == (int )__reject1) {
        {
        fprintf(_coverage_fout, "138\n");
        fflush(_coverage_fout);
        }
        tmp = __cp;
        {
        fprintf(_coverage_fout, "139\n");
        fflush(_coverage_fout);
        }
        __cp ++;
        {
        fprintf(_coverage_fout, "140\n");
        fflush(_coverage_fout);
        }
        *tmp = (char )'\000';
        break;
      } else {
        {
        fprintf(_coverage_fout, "149\n");
        fflush(_coverage_fout);
        }
        if ((int )*__cp == (int )__reject2) {
          {
          fprintf(_coverage_fout, "141\n");
          fflush(_coverage_fout);
          }
          tmp = __cp;
          {
          fprintf(_coverage_fout, "142\n");
          fflush(_coverage_fout);
          }
          __cp ++;
          {
          fprintf(_coverage_fout, "143\n");
          fflush(_coverage_fout);
          }
          *tmp = (char )'\000';
          break;
        } else {
          {
          fprintf(_coverage_fout, "148\n");
          fflush(_coverage_fout);
          }
          if ((int )*__cp == (int )__reject3) {
            {
            fprintf(_coverage_fout, "144\n");
            fflush(_coverage_fout);
            }
            tmp = __cp;
            {
            fprintf(_coverage_fout, "145\n");
            fflush(_coverage_fout);
            }
            __cp ++;
            {
            fprintf(_coverage_fout, "146\n");
            fflush(_coverage_fout);
            }
            *tmp = (char )'\000';
            break;
          } else {
            {
            fprintf(_coverage_fout, "147\n");
            fflush(_coverage_fout);
            }

          }
        }
      }
      {
      fprintf(_coverage_fout, "152\n");
      fflush(_coverage_fout);
      }
      __cp ++;
    }
    {
    fprintf(_coverage_fout, "155\n");
    fflush(_coverage_fout);
    }
    *__s = __cp;
  } else {
    {
    fprintf(_coverage_fout, "156\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "159\n");
  fflush(_coverage_fout);
  }
  return (__retval);
}
}
__inline extern char *__strsep_3c(char **__s , char __reject1 , char __reject2 ,
                                  char __reject3 ) ;
__inline extern char *__strsep_2c(char **__s , char __reject1 , char __reject2 ) 
{ register char *__retval ;
  register char *__cp ;
  char *tmp ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "133\n");
  fflush(_coverage_fout);
  }
  __retval = *__s;
  {
  fprintf(_coverage_fout, "134\n");
  fflush(_coverage_fout);
  }
  if ((unsigned int )__retval != (unsigned int )((void *)0)) {
    {
    fprintf(_coverage_fout, "129\n");
    fflush(_coverage_fout);
    }
    __cp = __retval;
    {
    fprintf(_coverage_fout, "130\n");
    fflush(_coverage_fout);
    }
    while (1) {
      {
      fprintf(_coverage_fout, "126\n");
      fflush(_coverage_fout);
      }
      if ((int )*__cp == 0) {
        {
        fprintf(_coverage_fout, "116\n");
        fflush(_coverage_fout);
        }
        __cp = (char *)((void *)0);
        break;
      } else {
        {
        fprintf(_coverage_fout, "117\n");
        fflush(_coverage_fout);
        }

      }
      {
      fprintf(_coverage_fout, "127\n");
      fflush(_coverage_fout);
      }
      if ((int )*__cp == (int )__reject1) {
        {
        fprintf(_coverage_fout, "118\n");
        fflush(_coverage_fout);
        }
        tmp = __cp;
        {
        fprintf(_coverage_fout, "119\n");
        fflush(_coverage_fout);
        }
        __cp ++;
        {
        fprintf(_coverage_fout, "120\n");
        fflush(_coverage_fout);
        }
        *tmp = (char )'\000';
        break;
      } else {
        {
        fprintf(_coverage_fout, "125\n");
        fflush(_coverage_fout);
        }
        if ((int )*__cp == (int )__reject2) {
          {
          fprintf(_coverage_fout, "121\n");
          fflush(_coverage_fout);
          }
          tmp = __cp;
          {
          fprintf(_coverage_fout, "122\n");
          fflush(_coverage_fout);
          }
          __cp ++;
          {
          fprintf(_coverage_fout, "123\n");
          fflush(_coverage_fout);
          }
          *tmp = (char )'\000';
          break;
        } else {
          {
          fprintf(_coverage_fout, "124\n");
          fflush(_coverage_fout);
          }

        }
      }
      {
      fprintf(_coverage_fout, "128\n");
      fflush(_coverage_fout);
      }
      __cp ++;
    }
    {
    fprintf(_coverage_fout, "131\n");
    fflush(_coverage_fout);
    }
    *__s = __cp;
  } else {
    {
    fprintf(_coverage_fout, "132\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "135\n");
  fflush(_coverage_fout);
  }
  return (__retval);
}
}
__inline extern char *__strsep_2c(char **__s , char __reject1 , char __reject2 ) ;
__inline extern char *__strsep_1c(char **__s , char __reject ) 
{ register char *__retval ;
  char *tmp ;
  char *tmp___0 ;
  void *tmp___1 ;
  char *tmp___2 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "113\n");
  fflush(_coverage_fout);
  }
  __retval = *__s;
  {
  fprintf(_coverage_fout, "114\n");
  fflush(_coverage_fout);
  }
  if ((unsigned int )__retval != (unsigned int )((void *)0)) {
    {
    fprintf(_coverage_fout, "108\n");
    fflush(_coverage_fout);
    }
    tmp___2 = __builtin_strchr(__retval, (int )__reject);
    {
    fprintf(_coverage_fout, "109\n");
    fflush(_coverage_fout);
    }
    tmp___0 = tmp___2;
    {
    fprintf(_coverage_fout, "110\n");
    fflush(_coverage_fout);
    }
    *__s = tmp___0;
    {
    fprintf(_coverage_fout, "111\n");
    fflush(_coverage_fout);
    }
    if ((unsigned int )tmp___0 != (unsigned int )((void *)0)) {
      {
      fprintf(_coverage_fout, "104\n");
      fflush(_coverage_fout);
      }
      tmp = *__s;
      {
      fprintf(_coverage_fout, "105\n");
      fflush(_coverage_fout);
      }
      (*__s) ++;
      {
      fprintf(_coverage_fout, "106\n");
      fflush(_coverage_fout);
      }
      *tmp = (char )'\000';
    } else {
      {
      fprintf(_coverage_fout, "107\n");
      fflush(_coverage_fout);
      }

    }
  } else {
    {
    fprintf(_coverage_fout, "112\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "115\n");
  fflush(_coverage_fout);
  }
  return (__retval);
}
}
__inline extern char *__strsep_1c(char **__s , char __reject ) ;
extern char *__strsep_g(char **__stringp , char const   *__delim ) ;
__inline extern char *__strtok_r_1c(char *__s , char __sep , char **__nextp ) 
{ char *__result ;
  char *tmp ;
  char *tmp___0 ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "98\n");
  fflush(_coverage_fout);
  }
  if ((unsigned int )__s == (unsigned int )((void *)0)) {
    {
    fprintf(_coverage_fout, "81\n");
    fflush(_coverage_fout);
    }
    __s = *__nextp;
  } else {
    {
    fprintf(_coverage_fout, "82\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "99\n");
  fflush(_coverage_fout);
  }
  while (1) {
    {
    fprintf(_coverage_fout, "84\n");
    fflush(_coverage_fout);
    }
    if ((int )*__s == (int )__sep) {
      {
      fprintf(_coverage_fout, "83\n");
      fflush(_coverage_fout);
      }

    } else {
      break;
    }
    {
    fprintf(_coverage_fout, "85\n");
    fflush(_coverage_fout);
    }
    __s ++;
  }
  {
  fprintf(_coverage_fout, "100\n");
  fflush(_coverage_fout);
  }
  __result = (char *)((void *)0);
  {
  fprintf(_coverage_fout, "101\n");
  fflush(_coverage_fout);
  }
  if ((int )*__s != 0) {
    {
    fprintf(_coverage_fout, "93\n");
    fflush(_coverage_fout);
    }
    tmp = __s;
    {
    fprintf(_coverage_fout, "94\n");
    fflush(_coverage_fout);
    }
    __s ++;
    {
    fprintf(_coverage_fout, "95\n");
    fflush(_coverage_fout);
    }
    __result = tmp;
    {
    fprintf(_coverage_fout, "96\n");
    fflush(_coverage_fout);
    }
    while (1) {
      {
      fprintf(_coverage_fout, "89\n");
      fflush(_coverage_fout);
      }
      if ((int )*__s != 0) {
        {
        fprintf(_coverage_fout, "86\n");
        fflush(_coverage_fout);
        }

      } else {
        break;
      }
      {
      fprintf(_coverage_fout, "90\n");
      fflush(_coverage_fout);
      }
      tmp___0 = __s;
      {
      fprintf(_coverage_fout, "91\n");
      fflush(_coverage_fout);
      }
      __s ++;
      {
      fprintf(_coverage_fout, "92\n");
      fflush(_coverage_fout);
      }
      if ((int )*tmp___0 == (int )__sep) {
        {
        fprintf(_coverage_fout, "87\n");
        fflush(_coverage_fout);
        }
        *(__s + -1) = (char )'\000';
        break;
      } else {
        {
        fprintf(_coverage_fout, "88\n");
        fflush(_coverage_fout);
        }

      }
    }
  } else {
    {
    fprintf(_coverage_fout, "97\n");
    fflush(_coverage_fout);
    }

  }
  {
  fprintf(_coverage_fout, "102\n");
  fflush(_coverage_fout);
  }
  *__nextp = __s;
  {
  fprintf(_coverage_fout, "103\n");
  fflush(_coverage_fout);
  }
  return (__result);
}
}
__inline extern char *__strtok_r_1c(char *__s , char __sep , char **__nextp ) ;
__inline extern char *__strpbrk_c3(char const   *__s , int __accept1 ,
                                   int __accept2 , int __accept3 ) 
{ char *tmp ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "78\n");
  fflush(_coverage_fout);
  }
  while (1) {
    {
    fprintf(_coverage_fout, "74\n");
    fflush(_coverage_fout);
    }
    if ((int const   )*__s != 0) {
      {
      fprintf(_coverage_fout, "73\n");
      fflush(_coverage_fout);
      }
      if ((int const   )*__s != (int const   )__accept1) {
        {
        fprintf(_coverage_fout, "72\n");
        fflush(_coverage_fout);
        }
        if ((int const   )*__s != (int const   )__accept2) {
          {
          fprintf(_coverage_fout, "71\n");
          fflush(_coverage_fout);
          }
          if ((int const   )*__s != (int const   )__accept3) {
            {
            fprintf(_coverage_fout, "70\n");
            fflush(_coverage_fout);
            }

          } else {
            break;
          }
        } else {
          break;
        }
      } else {
        break;
      }
    } else {
      break;
    }
    {
    fprintf(_coverage_fout, "75\n");
    fflush(_coverage_fout);
    }
    __s ++;
  }
  {
  fprintf(_coverage_fout, "79\n");
  fflush(_coverage_fout);
  }
  if ((int const   )*__s == 0) {
    {
    fprintf(_coverage_fout, "76\n");
    fflush(_coverage_fout);
    }
    tmp = (char *)((void *)0);
  } else {
    {
    fprintf(_coverage_fout, "77\n");
    fflush(_coverage_fout);
    }
    tmp = (char *)((unsigned int )__s);
  }
  {
  fprintf(_coverage_fout, "80\n");
  fflush(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern char *__strpbrk_c3(char const   *__s , int __accept1 ,
                                   int __accept2 , int __accept3 ) ;
__inline extern char *__strpbrk_c2(char const   *__s , int __accept1 ,
                                   int __accept2 ) 
{ char *tmp ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "67\n");
  fflush(_coverage_fout);
  }
  while (1) {
    {
    fprintf(_coverage_fout, "63\n");
    fflush(_coverage_fout);
    }
    if ((int const   )*__s != 0) {
      {
      fprintf(_coverage_fout, "62\n");
      fflush(_coverage_fout);
      }
      if ((int const   )*__s != (int const   )__accept1) {
        {
        fprintf(_coverage_fout, "61\n");
        fflush(_coverage_fout);
        }
        if ((int const   )*__s != (int const   )__accept2) {
          {
          fprintf(_coverage_fout, "60\n");
          fflush(_coverage_fout);
          }

        } else {
          break;
        }
      } else {
        break;
      }
    } else {
      break;
    }
    {
    fprintf(_coverage_fout, "64\n");
    fflush(_coverage_fout);
    }
    __s ++;
  }
  {
  fprintf(_coverage_fout, "68\n");
  fflush(_coverage_fout);
  }
  if ((int const   )*__s == 0) {
    {
    fprintf(_coverage_fout, "65\n");
    fflush(_coverage_fout);
    }
    tmp = (char *)((void *)0);
  } else {
    {
    fprintf(_coverage_fout, "66\n");
    fflush(_coverage_fout);
    }
    tmp = (char *)((unsigned int )__s);
  }
  {
  fprintf(_coverage_fout, "69\n");
  fflush(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern char *__strpbrk_c2(char const   *__s , int __accept1 ,
                                   int __accept2 ) ;
__inline extern size_t __strspn_c3(char const   *__s , int __accept1 ,
                                   int __accept2 , int __accept3 ) 
{ register size_t __result ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "57\n");
  fflush(_coverage_fout);
  }
  __result = (size_t )0;
  {
  fprintf(_coverage_fout, "58\n");
  fflush(_coverage_fout);
  }
  while (1) {
    {
    fprintf(_coverage_fout, "55\n");
    fflush(_coverage_fout);
    }
    if ((int const   )*(__s + __result) == (int const   )__accept1) {
      {
      fprintf(_coverage_fout, "50\n");
      fflush(_coverage_fout);
      }

    } else {
      {
      fprintf(_coverage_fout, "54\n");
      fflush(_coverage_fout);
      }
      if ((int const   )*(__s + __result) == (int const   )__accept2) {
        {
        fprintf(_coverage_fout, "51\n");
        fflush(_coverage_fout);
        }

      } else {
        {
        fprintf(_coverage_fout, "53\n");
        fflush(_coverage_fout);
        }
        if ((int const   )*(__s + __result) == (int const   )__accept3) {
          {
          fprintf(_coverage_fout, "52\n");
          fflush(_coverage_fout);
          }

        } else {
          break;
        }
      }
    }
    {
    fprintf(_coverage_fout, "56\n");
    fflush(_coverage_fout);
    }
    __result ++;
  }
  {
  fprintf(_coverage_fout, "59\n");
  fflush(_coverage_fout);
  }
  return (__result);
}
}
__inline extern size_t __strspn_c3(char const   *__s , int __accept1 ,
                                   int __accept2 , int __accept3 ) ;
__inline extern size_t __strspn_c2(char const   *__s , int __accept1 ,
                                   int __accept2 ) 
{ register size_t __result ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "47\n");
  fflush(_coverage_fout);
  }
  __result = (size_t )0;
  {
  fprintf(_coverage_fout, "48\n");
  fflush(_coverage_fout);
  }
  while (1) {
    {
    fprintf(_coverage_fout, "45\n");
    fflush(_coverage_fout);
    }
    if ((int const   )*(__s + __result) == (int const   )__accept1) {
      {
      fprintf(_coverage_fout, "42\n");
      fflush(_coverage_fout);
      }

    } else {
      {
      fprintf(_coverage_fout, "44\n");
      fflush(_coverage_fout);
      }
      if ((int const   )*(__s + __result) == (int const   )__accept2) {
        {
        fprintf(_coverage_fout, "43\n");
        fflush(_coverage_fout);
        }

      } else {
        break;
      }
    }
    {
    fprintf(_coverage_fout, "46\n");
    fflush(_coverage_fout);
    }
    __result ++;
  }
  {
  fprintf(_coverage_fout, "49\n");
  fflush(_coverage_fout);
  }
  return (__result);
}
}
__inline extern size_t __strspn_c2(char const   *__s , int __accept1 ,
                                   int __accept2 ) ;
__inline extern size_t __strspn_c1(char const   *__s , int __accept ) 
{ register size_t __result ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "39\n");
  fflush(_coverage_fout);
  }
  __result = (size_t )0;
  {
  fprintf(_coverage_fout, "40\n");
  fflush(_coverage_fout);
  }
  while (1) {
    {
    fprintf(_coverage_fout, "37\n");
    fflush(_coverage_fout);
    }
    if ((int const   )*(__s + __result) == (int const   )__accept) {
      {
      fprintf(_coverage_fout, "36\n");
      fflush(_coverage_fout);
      }

    } else {
      break;
    }
    {
    fprintf(_coverage_fout, "38\n");
    fflush(_coverage_fout);
    }
    __result ++;
  }
  {
  fprintf(_coverage_fout, "41\n");
  fflush(_coverage_fout);
  }
  return (__result);
}
}
__inline extern size_t __strspn_c1(char const   *__s , int __accept ) ;
__inline extern size_t __strcspn_c3(char const   *__s , int __reject1 ,
                                    int __reject2 , int __reject3 ) 
{ register size_t __result ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "33\n");
  fflush(_coverage_fout);
  }
  __result = (size_t )0;
  {
  fprintf(_coverage_fout, "34\n");
  fflush(_coverage_fout);
  }
  while (1) {
    {
    fprintf(_coverage_fout, "31\n");
    fflush(_coverage_fout);
    }
    if ((int const   )*(__s + __result) != 0) {
      {
      fprintf(_coverage_fout, "30\n");
      fflush(_coverage_fout);
      }
      if ((int const   )*(__s + __result) != (int const   )__reject1) {
        {
        fprintf(_coverage_fout, "29\n");
        fflush(_coverage_fout);
        }
        if ((int const   )*(__s + __result) != (int const   )__reject2) {
          {
          fprintf(_coverage_fout, "28\n");
          fflush(_coverage_fout);
          }
          if ((int const   )*(__s + __result) != (int const   )__reject3) {
            {
            fprintf(_coverage_fout, "27\n");
            fflush(_coverage_fout);
            }

          } else {
            break;
          }
        } else {
          break;
        }
      } else {
        break;
      }
    } else {
      break;
    }
    {
    fprintf(_coverage_fout, "32\n");
    fflush(_coverage_fout);
    }
    __result ++;
  }
  {
  fprintf(_coverage_fout, "35\n");
  fflush(_coverage_fout);
  }
  return (__result);
}
}
__inline extern size_t __strcspn_c3(char const   *__s , int __reject1 ,
                                    int __reject2 , int __reject3 ) ;
__inline extern size_t __strcspn_c2(char const   *__s , int __reject1 ,
                                    int __reject2 ) 
{ register size_t __result ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "24\n");
  fflush(_coverage_fout);
  }
  __result = (size_t )0;
  {
  fprintf(_coverage_fout, "25\n");
  fflush(_coverage_fout);
  }
  while (1) {
    {
    fprintf(_coverage_fout, "22\n");
    fflush(_coverage_fout);
    }
    if ((int const   )*(__s + __result) != 0) {
      {
      fprintf(_coverage_fout, "21\n");
      fflush(_coverage_fout);
      }
      if ((int const   )*(__s + __result) != (int const   )__reject1) {
        {
        fprintf(_coverage_fout, "20\n");
        fflush(_coverage_fout);
        }
        if ((int const   )*(__s + __result) != (int const   )__reject2) {
          {
          fprintf(_coverage_fout, "19\n");
          fflush(_coverage_fout);
          }

        } else {
          break;
        }
      } else {
        break;
      }
    } else {
      break;
    }
    {
    fprintf(_coverage_fout, "23\n");
    fflush(_coverage_fout);
    }
    __result ++;
  }
  {
  fprintf(_coverage_fout, "26\n");
  fflush(_coverage_fout);
  }
  return (__result);
}
}
__inline extern size_t __strcspn_c2(char const   *__s , int __reject1 ,
                                    int __reject2 ) ;
__inline extern size_t __strcspn_c1(char const   *__s , int __reject ) 
{ register size_t __result ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "16\n");
  fflush(_coverage_fout);
  }
  __result = (size_t )0;
  {
  fprintf(_coverage_fout, "17\n");
  fflush(_coverage_fout);
  }
  while (1) {
    {
    fprintf(_coverage_fout, "14\n");
    fflush(_coverage_fout);
    }
    if ((int const   )*(__s + __result) != 0) {
      {
      fprintf(_coverage_fout, "13\n");
      fflush(_coverage_fout);
      }
      if ((int const   )*(__s + __result) != (int const   )__reject) {
        {
        fprintf(_coverage_fout, "12\n");
        fflush(_coverage_fout);
        }

      } else {
        break;
      }
    } else {
      break;
    }
    {
    fprintf(_coverage_fout, "15\n");
    fflush(_coverage_fout);
    }
    __result ++;
  }
  {
  fprintf(_coverage_fout, "18\n");
  fflush(_coverage_fout);
  }
  return (__result);
}
}
__inline extern size_t __strcspn_c1(char const   *__s , int __reject ) ;
extern void *__rawmemchr(void const   *__s , int __c ) ;
extern  __attribute__((__nothrow__)) char *basename(char const   *__filename )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) void *memfrob(void *__s , size_t __n )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) char *strfry(char *__string )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int strverscmp(char const   *__s1 ,
                                                    char const   *__s2 )  __attribute__((__pure__,
__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) char *stpncpy(char * __restrict  __dest ,
                                                   char const   * __restrict  __src ,
                                                   size_t __n )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) char *__stpncpy(char * __restrict  __dest ,
                                                     char const   * __restrict  __src ,
                                                     size_t __n )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) char *stpcpy(char * __restrict  __dest ,
                                                  char const   * __restrict  __src )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) char *__stpcpy(char * __restrict  __dest ,
                                                    char const   * __restrict  __src )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) char *strsignal(int __sig ) ;
extern  __attribute__((__nothrow__)) char *strsep(char ** __restrict  __stringp ,
                                                  char const   * __restrict  __delim )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int strncasecmp_l(char const   *__s1 ,
                                                       char const   *__s2 ,
                                                       size_t __n ,
                                                       __locale_t __loc )  __attribute__((__pure__,
__nonnull__(1,2,4))) ;
extern  __attribute__((__nothrow__)) int strcasecmp_l(char const   *__s1 ,
                                                      char const   *__s2 ,
                                                      __locale_t __loc )  __attribute__((__pure__,
__nonnull__(1,2,3))) ;
extern  __attribute__((__nothrow__)) int strncasecmp(char const   *__s1 ,
                                                     char const   *__s2 ,
                                                     size_t __n )  __attribute__((__pure__,
__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int strcasecmp(char const   *__s1 ,
                                                    char const   *__s2 )  __attribute__((__pure__,
__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int ffsll(long long __ll )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int ffsl(long __l )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int ffs(int __i )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) char *rindex(char const   *__s , int __c )  __attribute__((__pure__,
__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) char *index(char const   *__s , int __c )  __attribute__((__pure__,
__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int bcmp(void const   *__s1 ,
                                              void const   *__s2 , size_t __n )  __attribute__((__pure__,
__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) void bzero(void *__s , size_t __n )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) void bcopy(void const   *__src ,
                                                void *__dest , size_t __n )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) void __bzero(void *__s , size_t __n )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) char *strerror_l(int __errnum ,
                                                      __locale_t __l ) ;
extern  __attribute__((__nothrow__)) char *strerror_r(int __errnum ,
                                                      char *__buf ,
                                                      size_t __buflen )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) char *strerror(int __errnum ) ;
extern  __attribute__((__nothrow__)) size_t strnlen(char const   *__string ,
                                                    size_t __maxlen )  __attribute__((__pure__,
__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) void *mempcpy(void * __restrict  __dest ,
                                                   void const   * __restrict  __src ,
                                                   size_t __n )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) void *__mempcpy(void * __restrict  __dest ,
                                                     void const   * __restrict  __src ,
                                                     size_t __n )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) void *memmem(void const   *__haystack ,
                                                  size_t __haystacklen ,
                                                  void const   *__needle ,
                                                  size_t __needlelen )  __attribute__((__pure__,
__nonnull__(1,3))) ;
extern  __attribute__((__nothrow__)) char *strcasestr(char const   *__haystack ,
                                                      char const   *__needle )  __attribute__((__pure__,
__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) char *strtok_r(char * __restrict  __s ,
                                                    char const   * __restrict  __delim ,
                                                    char ** __restrict  __save_ptr )  __attribute__((__nonnull__(2,3))) ;
extern  __attribute__((__nothrow__)) char *__strtok_r(char * __restrict  __s ,
                                                      char const   * __restrict  __delim ,
                                                      char ** __restrict  __save_ptr )  __attribute__((__nonnull__(2,3))) ;
extern  __attribute__((__nothrow__)) char *strtok(char * __restrict  __s ,
                                                  char const   * __restrict  __delim )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) char *strstr(char const   *__haystack ,
                                                  char const   *__needle )  __attribute__((__pure__,
__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) char *strpbrk(char const   *__s ,
                                                   char const   *__accept )  __attribute__((__pure__,
__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) size_t strspn(char const   *__s ,
                                                   char const   *__accept )  __attribute__((__pure__,
__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) size_t strcspn(char const   *__s ,
                                                    char const   *__reject )  __attribute__((__pure__,
__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) char *strchrnul(char const   *__s ,
                                                     int __c )  __attribute__((__pure__,
__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) char *strrchr(char const   *__s , int __c )  __attribute__((__pure__,
__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) char *strchr(char const   *__s , int __c )  __attribute__((__pure__,
__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) char *strndup(char const   *__string ,
                                                   size_t __n )  __attribute__((__nonnull__(1),
__malloc__)) ;
extern  __attribute__((__nothrow__)) char *strdup(char const   *__s )  __attribute__((__nonnull__(1),
__malloc__)) ;
extern  __attribute__((__nothrow__)) size_t strxfrm_l(char *__dest ,
                                                      char const   *__src ,
                                                      size_t __n ,
                                                      __locale_t __l )  __attribute__((__nonnull__(2,4))) ;
extern  __attribute__((__nothrow__)) int strcoll_l(char const   *__s1 ,
                                                   char const   *__s2 ,
                                                   __locale_t __l )  __attribute__((__pure__,
__nonnull__(1,2,3))) ;
extern  __attribute__((__nothrow__)) size_t strxfrm(char * __restrict  __dest ,
                                                    char const   * __restrict  __src ,
                                                    size_t __n )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) int strcoll(char const   *__s1 ,
                                                 char const   *__s2 )  __attribute__((__pure__,
__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int strncmp(char const   *__s1 ,
                                                 char const   *__s2 ,
                                                 size_t __n )  __attribute__((__pure__,
__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int strcmp(char const   *__s1 ,
                                                char const   *__s2 )  __attribute__((__pure__,
__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) char *strncat(char * __restrict  __dest ,
                                                   char const   * __restrict  __src ,
                                                   size_t __n )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) char *strcat(char * __restrict  __dest ,
                                                  char const   * __restrict  __src )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) char *strncpy(char * __restrict  __dest ,
                                                   char const   * __restrict  __src ,
                                                   size_t __n )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) char *strcpy(char * __restrict  __dest ,
                                                  char const   * __restrict  __src )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) void *memrchr(void const   *__s , int __c ,
                                                   size_t __n )  __attribute__((__pure__,
__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) void *rawmemchr(void const   *__s ,
                                                     int __c )  __attribute__((__pure__,
__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) void *memccpy(void * __restrict  __dest ,
                                                   void const   * __restrict  __src ,
                                                   int __c , size_t __n )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) void *memmove(void *__dest ,
                                                   void const   *__src ,
                                                   size_t __n )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int getloadavg(double *__loadavg ,
                                                    int __nelem )  __attribute__((__nonnull__(1))) ;
extern int getpt(void) ;
extern  __attribute__((__nothrow__)) int ptsname_r(int __fd , char *__buf ,
                                                   size_t __buflen )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) char *ptsname(int __fd ) ;
extern  __attribute__((__nothrow__)) int unlockpt(int __fd ) ;
extern  __attribute__((__nothrow__)) int grantpt(int __fd ) ;
extern int posix_openpt(int __oflag ) ;
extern  __attribute__((__nothrow__)) void setkey(char const   *__key )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int getsubopt(char ** __restrict  __optionp ,
                                                   char * const  * __restrict  __tokens ,
                                                   char ** __restrict  __valuep )  __attribute__((__nonnull__(1,2,3))) ;
extern  __attribute__((__nothrow__)) int rpmatch(char const   *__response )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) size_t wcstombs(char * __restrict  __s ,
                                                     wchar_t const   * __restrict  __pwcs ,
                                                     size_t __n ) ;
extern  __attribute__((__nothrow__)) size_t mbstowcs(wchar_t * __restrict  __pwcs ,
                                                     char const   * __restrict  __s ,
                                                     size_t __n ) ;
extern  __attribute__((__nothrow__)) int wctomb(char *__s , wchar_t __wchar ) ;
extern  __attribute__((__nothrow__)) int mbtowc(wchar_t * __restrict  __pwc ,
                                                char const   * __restrict  __s ,
                                                size_t __n ) ;
extern  __attribute__((__nothrow__)) int mblen(char const   *__s , size_t __n ) ;
extern  __attribute__((__nothrow__)) int qfcvt_r(long double __value ,
                                                 int __ndigit ,
                                                 int * __restrict  __decpt ,
                                                 int * __restrict  __sign ,
                                                 char * __restrict  __buf ,
                                                 size_t __len )  __attribute__((__nonnull__(3,4,5))) ;
extern  __attribute__((__nothrow__)) int qecvt_r(long double __value ,
                                                 int __ndigit ,
                                                 int * __restrict  __decpt ,
                                                 int * __restrict  __sign ,
                                                 char * __restrict  __buf ,
                                                 size_t __len )  __attribute__((__nonnull__(3,4,5))) ;
extern  __attribute__((__nothrow__)) int fcvt_r(double __value , int __ndigit ,
                                                int * __restrict  __decpt ,
                                                int * __restrict  __sign ,
                                                char * __restrict  __buf ,
                                                size_t __len )  __attribute__((__nonnull__(3,4,5))) ;
extern  __attribute__((__nothrow__)) int ecvt_r(double __value , int __ndigit ,
                                                int * __restrict  __decpt ,
                                                int * __restrict  __sign ,
                                                char * __restrict  __buf ,
                                                size_t __len )  __attribute__((__nonnull__(3,4,5))) ;
extern  __attribute__((__nothrow__)) char *qgcvt(long double __value ,
                                                 int __ndigit , char *__buf )  __attribute__((__nonnull__(3))) ;
extern  __attribute__((__nothrow__)) char *qfcvt(long double __value ,
                                                 int __ndigit ,
                                                 int * __restrict  __decpt ,
                                                 int * __restrict  __sign )  __attribute__((__nonnull__(3,4))) ;
extern  __attribute__((__nothrow__)) char *qecvt(long double __value ,
                                                 int __ndigit ,
                                                 int * __restrict  __decpt ,
                                                 int * __restrict  __sign )  __attribute__((__nonnull__(3,4))) ;
extern  __attribute__((__nothrow__)) char *gcvt(double __value , int __ndigit ,
                                                char *__buf )  __attribute__((__nonnull__(3))) ;
extern  __attribute__((__nothrow__)) char *fcvt(double __value , int __ndigit ,
                                                int * __restrict  __decpt ,
                                                int * __restrict  __sign )  __attribute__((__nonnull__(3,4))) ;
extern  __attribute__((__nothrow__)) char *ecvt(double __value , int __ndigit ,
                                                int * __restrict  __decpt ,
                                                int * __restrict  __sign )  __attribute__((__nonnull__(3,4))) ;
struct __anonstruct_lldiv_t_6;
typedef struct __anonstruct_lldiv_t_6 lldiv_t;
extern  __attribute__((__nothrow__)) lldiv_t lldiv(long long __numer ,
                                                   long long __denom )  __attribute__((__const__)) ;
struct __anonstruct_ldiv_t_5;
typedef struct __anonstruct_ldiv_t_5 ldiv_t;
extern  __attribute__((__nothrow__)) ldiv_t ldiv(long __numer , long __denom )  __attribute__((__const__)) ;
struct __anonstruct_div_t_4;
typedef struct __anonstruct_div_t_4 div_t;
extern  __attribute__((__nothrow__)) div_t div(int __numer , int __denom )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long long llabs(long long __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long labs(long __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int abs(int __x )  __attribute__((__const__)) ;
extern void qsort_r(void *__base , size_t __nmemb , size_t __size ,
                    int (*__compar)(void const   * , void const   * , void * ) ,
                    void *__arg )  __attribute__((__nonnull__(1,4))) ;
extern void qsort(void *__base , size_t __nmemb , size_t __size ,
                  int (*__compar)(void const   * , void const   * ) )  __attribute__((__nonnull__(1,4))) ;
extern void *bsearch(void const   *__key , void const   *__base ,
                     size_t __nmemb , size_t __size ,
                     int (*__compar)(void const   * , void const   * ) )  __attribute__((__nonnull__(1,2,5))) ;
extern  __attribute__((__nothrow__)) char *realpath(char const   * __restrict  __name ,
                                                    char * __restrict  __resolved ) ;
extern  __attribute__((__nothrow__)) char *canonicalize_file_name(char const   *__name )  __attribute__((__nonnull__(1))) ;
extern int system(char const   *__command ) ;
extern int mkostemps64(char *__template , int __suffixlen , int __flags )  __attribute__((__nonnull__(1))) ;
extern int mkostemps(char *__template , int __suffixlen , int __flags )  __attribute__((__nonnull__(1))) ;
extern int mkostemp64(char *__template , int __flags )  __attribute__((__nonnull__(1))) ;
extern int mkostemp(char *__template , int __flags )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) char *mkdtemp(char *__template )  __attribute__((__nonnull__(1))) ;
extern int mkstemps64(char *__template , int __suffixlen )  __attribute__((__nonnull__(1))) ;
extern int mkstemps(char *__template , int __suffixlen )  __attribute__((__nonnull__(1))) ;
extern int mkstemp64(char *__template )  __attribute__((__nonnull__(1))) ;
extern int mkstemp(char *__template )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) char *mktemp(char *__template )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int clearenv(void) ;
extern  __attribute__((__nothrow__)) int unsetenv(char const   *__name )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int setenv(char const   *__name ,
                                                char const   *__value ,
                                                int __replace )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) int putenv(char *__string )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) char *__secure_getenv(char const   *__name )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) char *getenv(char const   *__name )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__, __noreturn__)) void _Exit(int __status ) ;
extern  __attribute__((__nothrow__,
__noreturn__)) void quick_exit(int __status ) ;
extern  __attribute__((__nothrow__)) int on_exit(void (*__func)(int __status ,
                                                                void *__arg ) ,
                                                 void *__arg )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int at_quick_exit(void (*__func)(void) )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int atexit(void (*__func)(void) )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__, __noreturn__)) void abort(void) ;
extern  __attribute__((__nothrow__)) int posix_memalign(void **__memptr ,
                                                        size_t __alignment ,
                                                        size_t __size )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) void *valloc(size_t __size )  __attribute__((__malloc__)) ;
extern  __attribute__((__nothrow__)) void *alloca(size_t __size ) ;
extern  __attribute__((__nothrow__)) void cfree(void *__ptr ) ;
extern  __attribute__((__nothrow__)) void free(void *__ptr ) ;
extern  __attribute__((__nothrow__)) void *calloc(size_t __nmemb ,
                                                  size_t __size )  __attribute__((__malloc__)) ;
struct drand48_data;
extern  __attribute__((__nothrow__)) int lcong48_r(unsigned short *__param ,
                                                   struct drand48_data *__buffer )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int seed48_r(unsigned short *__seed16v ,
                                                  struct drand48_data *__buffer )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int srand48_r(long __seedval ,
                                                   struct drand48_data *__buffer )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) int jrand48_r(unsigned short *__xsubi ,
                                                   struct drand48_data * __restrict  __buffer ,
                                                   long * __restrict  __result )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int mrand48_r(struct drand48_data * __restrict  __buffer ,
                                                   long * __restrict  __result )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int nrand48_r(unsigned short *__xsubi ,
                                                   struct drand48_data * __restrict  __buffer ,
                                                   long * __restrict  __result )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int lrand48_r(struct drand48_data * __restrict  __buffer ,
                                                   long * __restrict  __result )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int erand48_r(unsigned short *__xsubi ,
                                                   struct drand48_data * __restrict  __buffer ,
                                                   double * __restrict  __result )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int drand48_r(struct drand48_data * __restrict  __buffer ,
                                                   double * __restrict  __result )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) void lcong48(unsigned short *__param )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) unsigned short *seed48(unsigned short *__seed16v )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) void srand48(long __seedval ) ;
extern  __attribute__((__nothrow__)) long jrand48(unsigned short *__xsubi )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) long mrand48(void) ;
extern  __attribute__((__nothrow__)) long nrand48(unsigned short *__xsubi )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) long lrand48(void) ;
extern  __attribute__((__nothrow__)) double erand48(unsigned short *__xsubi )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) double drand48(void) ;
extern  __attribute__((__nothrow__)) int rand_r(unsigned int *__seed ) ;
extern  __attribute__((__nothrow__)) void srand(unsigned int __seed ) ;
extern  __attribute__((__nothrow__)) int rand(void) ;
struct random_data;
extern  __attribute__((__nothrow__)) int setstate_r(char * __restrict  __statebuf ,
                                                    struct random_data * __restrict  __buf )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int initstate_r(unsigned int __seed ,
                                                     char * __restrict  __statebuf ,
                                                     size_t __statelen ,
                                                     struct random_data * __restrict  __buf )  __attribute__((__nonnull__(2,4))) ;
extern  __attribute__((__nothrow__)) int srandom_r(unsigned int __seed ,
                                                   struct random_data *__buf )  __attribute__((__nonnull__(2))) ;
typedef int int32_t;
extern  __attribute__((__nothrow__)) int random_r(struct random_data * __restrict  __buf ,
                                                  int32_t * __restrict  __result )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) char *setstate(char *__statebuf )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) char *initstate(unsigned int __seed ,
                                                     char *__statebuf ,
                                                     size_t __statelen )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) void srandom(unsigned int __seed ) ;
extern  __attribute__((__nothrow__)) long random(void) ;
__inline extern  __attribute__((__nothrow__)) unsigned long long gnu_dev_makedev(unsigned int __major ,
                                                                                 unsigned int __minor ) ;
__inline extern unsigned long long gnu_dev_makedev(unsigned int __major ,
                                                   unsigned int __minor ) 
{ 

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "11\n");
  fflush(_coverage_fout);
  }
  return (((unsigned long long )((__minor & 255U) | ((__major & 4095U) << 8)) | ((unsigned long long )(__minor & 4294967040U) << 12)) | ((unsigned long long )(__major & 4294963200U) << 32));
}
}
__inline extern  __attribute__((__nothrow__)) unsigned long long gnu_dev_makedev(unsigned int __major ,
                                                                                 unsigned int __minor ) ;
__inline extern  __attribute__((__nothrow__)) unsigned int gnu_dev_minor(unsigned long long __dev ) ;
__inline extern unsigned int gnu_dev_minor(unsigned long long __dev ) 
{ 

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "10\n");
  fflush(_coverage_fout);
  }
  return ((unsigned int )((__dev & 255ULL) | (unsigned long long )((unsigned int )(__dev >> 12) & 4294967040U)));
}
}
__inline extern  __attribute__((__nothrow__)) unsigned int gnu_dev_minor(unsigned long long __dev ) ;
__inline extern  __attribute__((__nothrow__)) unsigned int gnu_dev_major(unsigned long long __dev ) ;
__inline extern unsigned int gnu_dev_major(unsigned long long __dev ) 
{ 

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "9\n");
  fflush(_coverage_fout);
  }
  return ((unsigned int )(((__dev >> 8) & 4095ULL) | (unsigned long long )((unsigned int )(__dev >> 32) & 4294963200U)));
}
}
__inline extern  __attribute__((__nothrow__)) unsigned int gnu_dev_major(unsigned long long __dev ) ;
struct __anonstruct___sigset_t_8;
struct __anonstruct_fd_set_9;
typedef struct __anonstruct_fd_set_9 fd_set;
extern int pselect(int __nfds , fd_set * __restrict  __readfds ,
                   fd_set * __restrict  __writefds ,
                   fd_set * __restrict  __exceptfds ,
                   struct timespec  const  * __restrict  __timeout ,
                   __sigset_t const   * __restrict  __sigmask ) ;
extern int select(int __nfds , fd_set * __restrict  __readfds ,
                  fd_set * __restrict  __writefds ,
                  fd_set * __restrict  __exceptfds ,
                  struct timeval * __restrict  __timeout ) ;
extern  __attribute__((__nothrow__)) long a64l(char const   *__s )  __attribute__((__pure__,
__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) char *l64a(long __n ) ;
extern  __attribute__((__nothrow__)) long long strtoll(char const   * __restrict  __nptr ,
                                                       char ** __restrict  __endptr ,
                                                       int __base )  __attribute__((__nonnull__(1))) ;
__inline extern  __attribute__((__nothrow__)) long long atoll(char const   *__nptr )  __attribute__((__pure__,
__nonnull__(1))) ;
__inline extern long long atoll(char const   *__nptr ) 
{ long long tmp ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "7\n");
  fflush(_coverage_fout);
  }
  tmp = strtoll((char const   */* __restrict  */)__nptr,
                (char **/* __restrict  */)((char **)((void *)0)), 10);
  {
  fprintf(_coverage_fout, "8\n");
  fflush(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) long long atoll(char const   *__nptr )  __attribute__((__pure__,
__nonnull__(1))) ;
__inline extern  __attribute__((__nothrow__)) long atol(char const   *__nptr )  __attribute__((__pure__,
__nonnull__(1))) ;
__inline extern long atol(char const   *__nptr ) 
{ long tmp ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "5\n");
  fflush(_coverage_fout);
  }
  tmp = strtol((char const   */* __restrict  */)__nptr,
               (char **/* __restrict  */)((char **)((void *)0)), 10);
  {
  fprintf(_coverage_fout, "6\n");
  fflush(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) long atol(char const   *__nptr )  __attribute__((__pure__,
__nonnull__(1))) ;
__inline extern  __attribute__((__nothrow__)) int atoi(char const   *__nptr )  __attribute__((__pure__,
__nonnull__(1))) ;
__inline extern int atoi(char const   *__nptr ) 
{ long tmp ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "3\n");
  fflush(_coverage_fout);
  }
  tmp = strtol((char const   */* __restrict  */)__nptr,
               (char **/* __restrict  */)((char **)((void *)0)), 10);
  {
  fprintf(_coverage_fout, "4\n");
  fflush(_coverage_fout);
  }
  return ((int )tmp);
}
}
__inline extern  __attribute__((__nothrow__)) int atoi(char const   *__nptr )  __attribute__((__pure__,
__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) double strtod(char const   * __restrict  __nptr ,
                                                   char ** __restrict  __endptr )  __attribute__((__nonnull__(1))) ;
__inline extern  __attribute__((__nothrow__)) double atof(char const   *__nptr )  __attribute__((__pure__,
__nonnull__(1))) ;
__inline extern double atof(char const   *__nptr ) 
{ double tmp ;

  {
  {
  if (_coverage_fout == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-08-d54e6ce832-23e65a9dcc/coverage/coverage.path",
                           "wb");
    }
  }
  }
  {
  fprintf(_coverage_fout, "1\n");
  fflush(_coverage_fout);
  }
  tmp = strtod((char const   */* __restrict  */)__nptr,
               (char **/* __restrict  */)((char **)((void *)0)));
  {
  fprintf(_coverage_fout, "2\n");
  fflush(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) double atof(char const   *__nptr )  __attribute__((__pure__,
__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) long double strtold_l(char const   * __restrict  __nptr ,
                                                           char ** __restrict  __endptr ,
                                                           __locale_t __loc )  __attribute__((__nonnull__(1,3))) ;
extern  __attribute__((__nothrow__)) float strtof_l(char const   * __restrict  __nptr ,
                                                    char ** __restrict  __endptr ,
                                                    __locale_t __loc )  __attribute__((__nonnull__(1,3))) ;
extern  __attribute__((__nothrow__)) double strtod_l(char const   * __restrict  __nptr ,
                                                     char ** __restrict  __endptr ,
                                                     __locale_t __loc )  __attribute__((__nonnull__(1,3))) ;
extern  __attribute__((__nothrow__)) unsigned long long strtoull_l(char const   * __restrict  __nptr ,
                                                                   char ** __restrict  __endptr ,
                                                                   int __base ,
                                                                   __locale_t __loc )  __attribute__((__nonnull__(1,4))) ;
extern  __attribute__((__nothrow__)) long long strtoll_l(char const   * __restrict  __nptr ,
                                                         char ** __restrict  __endptr ,
                                                         int __base ,
                                                         __locale_t __loc )  __attribute__((__nonnull__(1,4))) ;
extern  __attribute__((__nothrow__)) unsigned long strtoul_l(char const   * __restrict  __nptr ,
                                                             char ** __restrict  __endptr ,
                                                             int __base ,
                                                             __locale_t __loc )  __attribute__((__nonnull__(1,4))) ;
extern  __attribute__((__nothrow__)) long strtol_l(char const   * __restrict  __nptr ,
                                                   char ** __restrict  __endptr ,
                                                   int __base ,
                                                   __locale_t __loc )  __attribute__((__nonnull__(1,4))) ;
extern  __attribute__((__nothrow__)) unsigned long long strtoull(char const   * __restrict  __nptr ,
                                                                 char ** __restrict  __endptr ,
                                                                 int __base )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) unsigned long long strtouq(char const   * __restrict  __nptr ,
                                                                char ** __restrict  __endptr ,
                                                                int __base )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) long long strtoq(char const   * __restrict  __nptr ,
                                                      char ** __restrict  __endptr ,
                                                      int __base )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) unsigned long strtoul(char const   * __restrict  __nptr ,
                                                           char ** __restrict  __endptr ,
                                                           int __base )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) long double strtold(char const   * __restrict  __nptr ,
                                                         char ** __restrict  __endptr )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) float strtof(char const   * __restrict  __nptr ,
                                                  char ** __restrict  __endptr )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) size_t __ctype_get_mb_cur_max(void) ;
typedef int (*spl_iterator_apply_func_t)(zend_object_iterator *iter ,
                                         void *puser );
struct _spl_dual_it_object;
typedef struct _spl_dual_it_object spl_dual_it_object;
struct _spl_cbfilter_it_intern;
typedef struct _spl_cbfilter_it_intern _spl_cbfilter_it_intern;
enum __anonenum_regex_mode_91 {
    REGIT_MODE_MATCH = 0,
    REGIT_MODE_GET_MATCH = 1,
    REGIT_MODE_ALL_MATCHES = 2,
    REGIT_MODE_SPLIT = 3,
    REGIT_MODE_REPLACE = 4,
    REGIT_MODE_MAX = 5
} ;
typedef enum __anonenum_regex_mode_91 regex_mode;
struct __anonstruct_regex_98 {
   int use_flags ;
   long flags ;
   regex_mode mode ;
   long preg_flags ;
   pcre_cache_entry *pce ;
   char *regex ;
   uint regex_len ;
};
struct __anonstruct_append_97 {
   zval *zarrayit ;
   zend_object_iterator *iterator ;
};
struct __anonstruct_caching_96 {
   long flags ;
   zval *zstr ;
   zval *zchildren ;
   zval *zcache ;
};
struct __anonstruct_limit_95 {
   long offset ;
   long count ;
};
union __anonunion_u_94 {
   struct __anonstruct_limit_95 limit ;
   struct __anonstruct_caching_96 caching ;
   struct __anonstruct_append_97 append ;
   struct __anonstruct_regex_98 regex ;
   _spl_cbfilter_it_intern *cbfilter ;
};
enum __anonenum_dual_it_type_87 {
    DIT_Default = 0,
    DIT_FilterIterator = 0,
    DIT_RecursiveFilterIterator = 0,
    DIT_ParentIterator = 0,
    DIT_LimitIterator = 1,
    DIT_CachingIterator = 2,
    DIT_RecursiveCachingIterator = 3,
    DIT_IteratorIterator = 4,
    DIT_NoRewindIterator = 5,
    DIT_InfiniteIterator = 6,
    DIT_AppendIterator = 7,
    DIT_RegexIterator = 8,
    DIT_RecursiveRegexIterator = 9,
    DIT_CallbackFilterIterator = 10,
    DIT_RecursiveCallbackFilterIterator = 11,
    DIT_Unknown = -1
} ;
typedef enum __anonenum_dual_it_type_87 dual_it_type;
struct __anonstruct_current_93 {
   zval *data ;
   char *str_key ;
   uint str_key_len ;
   ulong int_key ;
   int key_type ;
   int pos ;
};
struct __anonstruct_inner_92 {
   zval *zobject ;
   zend_class_entry *ce ;
   zend_object *object ;
   zend_object_iterator *iterator ;
};
struct _spl_dual_it_object {
   zend_object std ;
   struct __anonstruct_inner_92 inner ;
   struct __anonstruct_current_93 current ;
   dual_it_type dit_type ;
   union __anonunion_u_94 u ;
};
union __anonunion_u_94;
struct __anonstruct_regex_98;
struct __anonstruct_append_97;
struct __anonstruct_caching_96;
struct __anonstruct_limit_95;
struct __anonstruct_current_93;
struct __anonstruct_inner_92;
struct _spl_cbfilter_it_intern {
   zend_fcall_info fci ;
   zend_fcall_info_cache fcc ;
};
enum __anonenum_regex_mode_91;
enum __anonenum_90 {
    REGIT_USE_KEY = 1,
    REGIT_INVERTED = 2
} ;
enum __anonenum_90;
enum __anonenum_89 {
    CIT_CALL_TOSTRING = 1,
    CIT_TOSTRING_USE_KEY = 2,
    CIT_TOSTRING_USE_CURRENT = 4,
    CIT_TOSTRING_USE_INNER = 8,
    CIT_CATCH_GET_CHILD = 16,
    CIT_FULL_CACHE = 256,
    CIT_PUBLIC = 65535,
    CIT_VALID = 65536,
    CIT_HAS_CHILDREN = 131072
} ;
enum __anonenum_89;
enum __anonenum_recursive_it_it_type_88;
typedef enum __anonenum_recursive_it_it_type_88 recursive_it_it_type;
enum __anonenum_recursive_it_it_type_88 {
    RIT_Default = 0,
    RIT_RecursiveIteratorIterator = 0,
    RIT_RecursiveTreeIterator = 1,
    RIT_Unknow = -1
} ;
enum __anonenum_dual_it_type_87;
struct _zend_pcre_globals;
typedef struct _zend_pcre_globals zend_pcre_globals;
struct _zend_pcre_globals {
   HashTable pcre_cache ;
   long backtrack_limit ;
   long recursion_limit ;
   int error_code ;
};
struct __anonstruct_pcre_cache_entry_86 {
   pcre *re ;
   pcre_extra *extra ;
   int preg_options ;
   char *locale ;
   unsigned char const   *tables ;
   int compile_options ;
   int refcount ;
};
struct lconv {
   char *decimal_point ;
   char *thousands_sep ;
   char *grouping ;
   char *int_curr_symbol ;
   char *currency_symbol ;
   char *mon_decimal_point ;
   char *mon_thousands_sep ;
   char *mon_grouping ;
   char *positive_sign ;
   char *negative_sign ;
   char int_frac_digits ;
   char frac_digits ;
   char p_cs_precedes ;
   char p_sep_by_space ;
   char n_cs_precedes ;
   char n_sep_by_space ;
   char p_sign_posn ;
   char n_sign_posn ;
   char int_p_cs_precedes ;
   char int_p_sep_by_space ;
   char int_n_cs_precedes ;
   char int_n_sep_by_space ;
   char int_p_sign_posn ;
   char int_n_sign_posn ;
};
enum __anonenum_85 {
    __LC_CTYPE = 0,
    __LC_NUMERIC = 1,
    __LC_TIME = 2,
    __LC_COLLATE = 3,
    __LC_MONETARY = 4,
    __LC_MESSAGES = 5,
    __LC_ALL = 6,
    __LC_PAPER = 7,
    __LC_NAME = 8,
    __LC_ADDRESS = 9,
    __LC_TELEPHONE = 10,
    __LC_MEASUREMENT = 11,
    __LC_IDENTIFICATION = 12
} ;
enum __anonenum_85;
struct pcre_callout_block {
   int version ;
   int callout_number ;
   int *offset_vector ;
   char const   *subject ;
   int subject_length ;
   int start_match ;
   int current_position ;
   int capture_top ;
   int capture_last ;
   void *callout_data ;
   int pattern_position ;
   int next_item_length ;
};
struct pcre_extra {
   unsigned long flags ;
   void *study_data ;
   unsigned long match_limit ;
   void *callout_data ;
   unsigned char const   *tables ;
   unsigned long match_limit_recursion ;
   unsigned char **mark ;
};
typedef zend_object_value (*create_object_func_t)(zend_class_entry *class_type );
struct _zend_spl_globals {
   char *autoload_extensions ;
   HashTable *autoload_functions ;
   int autoload_running ;
   int autoload_extensions_len ;
   intptr_t hash_mask_handle ;
   intptr_t hash_mask_handlers ;
   int hash_mask_init ;
};
typedef unsigned long long uintmax_t;
typedef long long intmax_t;
typedef unsigned int uintptr_t;
typedef unsigned long long uint_fast64_t;
typedef unsigned int uint_fast32_t;
typedef unsigned int uint_fast16_t;
typedef unsigned char uint_fast8_t;
typedef long long int_fast64_t;
typedef int int_fast32_t;
typedef int int_fast16_t;
typedef signed char int_fast8_t;
typedef unsigned long long uint_least64_t;
typedef unsigned int uint_least32_t;
typedef unsigned short uint_least16_t;
typedef unsigned char uint_least8_t;
typedef long long int_least64_t;
typedef int int_least32_t;
typedef short int_least16_t;
typedef signed char int_least8_t;
typedef unsigned long long uint64_t;
typedef unsigned int uint32_t;
typedef unsigned short uint16_t;
typedef unsigned char uint8_t;
struct _zend_user_iterator;
struct _php_shutdown_function_entry {
   zval **arguments ;
   int arg_count ;
};
struct __anonstruct_putenv_entry_84;
typedef struct __anonstruct_putenv_entry_84 putenv_entry;
struct __anonstruct_putenv_entry_84 {
   char *putenv_string ;
   char *previous_value ;
   char *key ;
   int key_len ;
};
struct __anonstruct_unserialize_83;
struct __anonstruct_serialize_82;
typedef int php_int32;
struct __anonstruct_url_adapt_state_ex_t_81;
struct _zend_syntax_highlighter_ini {
   char *highlight_html ;
   char *highlight_comment ;
   char *highlight_default ;
   char *highlight_string ;
   char *highlight_keyword ;
};
struct itimerspec {
   struct timespec it_interval ;
   struct timespec it_value ;
};
struct tm {
   int tm_sec ;
   int tm_min ;
   int tm_hour ;
   int tm_mday ;
   int tm_mon ;
   int tm_year ;
   int tm_wday ;
   int tm_yday ;
   int tm_isdst ;
   long tm_gmtoff ;
   char const   *tm_zone ;
};
struct _zend_constant {
   zval value ;
   int flags ;
   char *name ;
   uint name_len ;
   int module_number ;
};
struct _cwd_state {
   char *cwd ;
   int cwd_length ;
};
struct _virtual_cwd_globals {
   cwd_state cwd ;
   long realpath_cache_size ;
   long realpath_cache_size_limit ;
   long realpath_cache_ttl ;
   realpath_cache_bucket *realpath_cache[1024] ;
};
struct _realpath_cache_bucket {
   unsigned long key ;
   char *path ;
   int path_len ;
   char *realpath ;
   int realpath_len ;
   int is_dir ;
   time_t expires ;
   struct _realpath_cache_bucket *next ;
};
typedef int (*verify_path_func)(cwd_state const   * );
enum __anonenum_79 {
    DT_UNKNOWN = 0,
    DT_FIFO = 1,
    DT_CHR = 2,
    DT_DIR = 4,
    DT_BLK = 6,
    DT_REG = 8,
    DT_LNK = 10,
    DT_SOCK = 12,
    DT_WHT = 14
} ;
enum __anonenum_79;
typedef __u_quad_t __ino64_t;
struct dirent64 {
   __ino64_t d_ino ;
   __off64_t d_off ;
   unsigned short d_reclen ;
   unsigned char d_type ;
   char d_name[256] ;
};
struct dirent {
   __ino_t d_ino ;
   __off_t d_off ;
   unsigned short d_reclen ;
   unsigned char d_type ;
   char d_name[256] ;
};
struct utimbuf {
   __time_t actime ;
   __time_t modtime ;
};
struct _zend_ini_parser_param;
typedef struct _zend_ini_parser_param zend_ini_parser_param;
struct _zend_ini_parser_param {
   void (*ini_parser_cb)(zval *arg1 , zval *arg2 , zval *arg3 ,
                         int callback_type , void *arg ) ;
   void *arg ;
};
typedef void (*zend_ini_parser_cb_t)(zval *arg1 , zval *arg2 , zval *arg3 ,
                                     int callback_type , void *arg );
struct _zend_ini_entry {
   int module_number ;
   int modifiable ;
   char *name ;
   uint name_length ;
   int (*on_modify)(zend_ini_entry *entry , char *new_value ,
                    uint new_value_length , void *mh_arg1 , void *mh_arg2 ,
                    void *mh_arg3 , int stage ) ;
   void *mh_arg1 ;
   void *mh_arg2 ;
   void *mh_arg3 ;
   char *value ;
   uint value_length ;
   char *orig_value ;
   uint orig_value_length ;
   int orig_modifiable ;
   int modified ;
   void (*displayer)(zend_ini_entry *ini_entry , int type ) ;
};
struct _zend_llist {
   zend_llist_element *head ;
   zend_llist_element *tail ;
   size_t count ;
   size_t size ;
   void (*dtor)(void * ) ;
   unsigned char persistent ;
   zend_llist_element *traverse_ptr ;
};
struct _arg_separators {
   char *output ;
   char *input ;
};
typedef struct _arg_separators arg_separators;
struct _php_core_globals {
   zend_bool implicit_flush ;
   long output_buffering ;
   zend_bool sql_safe_mode ;
   zend_bool enable_dl ;
   char *output_handler ;
   char *unserialize_callback_func ;
   long serialize_precision ;
   long memory_limit ;
   long max_input_time ;
   zend_bool track_errors ;
   zend_bool display_errors ;
   zend_bool display_startup_errors ;
   zend_bool log_errors ;
   long log_errors_max_len ;
   zend_bool ignore_repeated_errors ;
   zend_bool ignore_repeated_source ;
   zend_bool report_memleaks ;
   char *error_log ;
   char *doc_root ;
   char *user_dir ;
   char *include_path ;
   char *open_basedir ;
   char *extension_dir ;
   char *php_binary ;
   char *upload_tmp_dir ;
   long upload_max_filesize ;
   char *error_append_string ;
   char *error_prepend_string ;
   char *auto_prepend_file ;
   char *auto_append_file ;
   arg_separators arg_separator ;
   char *variables_order ;
   HashTable rfc1867_protected_variables ;
   short connection_status ;
   short ignore_user_abort ;
   unsigned char header_is_being_sent ;
   zend_llist tick_functions ;
   zval *http_globals[6] ;
   zend_bool expose_php ;
   zend_bool register_argc_argv ;
   zend_bool auto_globals_jit ;
   char *docref_root ;
   char *docref_ext ;
   zend_bool html_errors ;
   zend_bool xmlrpc_errors ;
   long xmlrpc_error_number ;
   zend_bool activated_auto_globals[8] ;
   zend_bool modules_activated ;
   zend_bool file_uploads ;
   zend_bool during_request_startup ;
   zend_bool allow_url_fopen ;
   zend_bool enable_post_data_reading ;
   zend_bool always_populate_raw_post_data ;
   zend_bool report_zend_debug ;
   int last_error_type ;
   char *last_error_message ;
   char *last_error_file ;
   int last_error_lineno ;
   char *disable_functions ;
   char *disable_classes ;
   zend_bool allow_url_include ;
   zend_bool exit_on_timeout ;
   long max_input_nesting_level ;
   long max_input_vars ;
   zend_bool in_user_include ;
   char *user_ini_filename ;
   long user_ini_cache_ttl ;
   char *request_order ;
   zend_bool mail_x_header ;
   char *mail_log ;
   zend_bool in_error_log ;
};
struct _arg_separators;
struct _php_tick_function_entry;
typedef struct _php_core_globals php_core_globals;
struct __anonstruct_php_stream_mmap_range_78;
typedef struct __anonstruct_php_stream_mmap_range_78 php_stream_mmap_range;
enum __anonenum_php_stream_mmap_access_t_77 {
    PHP_STREAM_MAP_MODE_READONLY = 0,
    PHP_STREAM_MAP_MODE_READWRITE = 1,
    PHP_STREAM_MAP_MODE_SHARED_READONLY = 2,
    PHP_STREAM_MAP_MODE_SHARED_READWRITE = 3
} ;
typedef enum __anonenum_php_stream_mmap_access_t_77 php_stream_mmap_access_t;
struct __anonstruct_php_stream_mmap_range_78 {
   size_t offset ;
   size_t length ;
   php_stream_mmap_access_t mode ;
   char *mapped ;
};
enum __anonenum_php_stream_mmap_access_t_77;
enum __anonenum_php_stream_mmap_operation_t_76 {
    PHP_STREAM_MMAP_SUPPORTED = 0,
    PHP_STREAM_MMAP_MAP_RANGE = 1,
    PHP_STREAM_MMAP_UNMAP = 2
} ;
struct _php_stream_xport_crypto_param;
typedef struct _php_stream_xport_crypto_param php_stream_xport_crypto_param;
struct __anonstruct_outputs_75 {
   int returncode ;
};
enum __anonenum_php_stream_xport_crypt_method_t_72 {
    STREAM_CRYPTO_METHOD_SSLv2_CLIENT = 0,
    STREAM_CRYPTO_METHOD_SSLv3_CLIENT = 1,
    STREAM_CRYPTO_METHOD_SSLv23_CLIENT = 2,
    STREAM_CRYPTO_METHOD_TLS_CLIENT = 3,
    STREAM_CRYPTO_METHOD_SSLv2_SERVER = 4,
    STREAM_CRYPTO_METHOD_SSLv3_SERVER = 5,
    STREAM_CRYPTO_METHOD_SSLv23_SERVER = 6,
    STREAM_CRYPTO_METHOD_TLS_SERVER = 7
} ;
struct __anonstruct_inputs_74 {
   int activate ;
   php_stream_xport_crypt_method_t method ;
   php_stream *session ;
};
enum __anonenum_op_73 {
    STREAM_XPORT_CRYPTO_OP_SETUP = 0,
    STREAM_XPORT_CRYPTO_OP_ENABLE = 1
} ;
struct _php_stream_xport_crypto_param {
   enum __anonenum_op_73 op ;
   struct __anonstruct_inputs_74 inputs ;
   struct __anonstruct_outputs_75 outputs ;
};
struct __anonstruct_outputs_75;
struct __anonstruct_inputs_74;
enum __anonenum_op_73;
struct _php_stream_xport_param;
typedef struct _php_stream_xport_param php_stream_xport_param;
struct __anonstruct_outputs_71 {
   php_stream *client ;
   int returncode ;
   struct sockaddr *addr ;
   socklen_t addrlen ;
   char *textaddr ;
   long textaddrlen ;
   char *error_text ;
   int error_code ;
};
struct __anonstruct_inputs_70 {
   char *name ;
   long namelen ;
   int backlog ;
   struct timeval *timeout ;
   struct sockaddr *addr ;
   socklen_t addrlen ;
   char *buf ;
   size_t buflen ;
   long flags ;
};
enum __anonenum_op_69 {
    STREAM_XPORT_OP_BIND = 0,
    STREAM_XPORT_OP_CONNECT = 1,
    STREAM_XPORT_OP_LISTEN = 2,
    STREAM_XPORT_OP_ACCEPT = 3,
    STREAM_XPORT_OP_CONNECT_ASYNC = 4,
    STREAM_XPORT_OP_GET_NAME = 5,
    STREAM_XPORT_OP_GET_PEER_NAME = 6,
    STREAM_XPORT_OP_RECV = 7,
    STREAM_XPORT_OP_SEND = 8,
    STREAM_XPORT_OP_SHUTDOWN = 9
} ;
struct _php_stream_xport_param {
   enum __anonenum_op_69 op ;
   unsigned int want_addr : 1 ;
   unsigned int want_textaddr : 1 ;
   unsigned int want_errortext : 1 ;
   unsigned int how : 2 ;
   struct __anonstruct_inputs_70 inputs ;
   struct __anonstruct_outputs_71 outputs ;
};
struct __anonstruct_outputs_71;
struct __anonstruct_inputs_70;
enum __anonenum_op_69;
enum __anonenum_stream_shutdown_t_68 {
    STREAM_SHUT_RD = 0,
    STREAM_SHUT_WR = 1,
    STREAM_SHUT_RDWR = 2
} ;
enum php_stream_xport_send_recv_flags {
    STREAM_OOB = 1,
    STREAM_PEEK = 2
} ;
enum php_stream_xport_send_recv_flags;
union __anonunion___CONST_SOCKADDR_ARG_67;
typedef union __anonunion___CONST_SOCKADDR_ARG_67  __attribute__((__transparent_union__)) __CONST_SOCKADDR_ARG;
struct sockaddr_x25;
struct sockaddr_un;
struct sockaddr_ns;
struct sockaddr_iso;
struct sockaddr_ipx;
struct sockaddr_inarp;
struct sockaddr_in6;
struct sockaddr_in;
struct sockaddr_eon;
struct sockaddr_dl;
struct sockaddr_ax25;
struct sockaddr_at;
union __anonunion___CONST_SOCKADDR_ARG_67 {
   struct sockaddr  const  * __restrict  __sockaddr__ ;
   struct sockaddr_at  const  * __restrict  __sockaddr_at__ ;
   struct sockaddr_ax25  const  * __restrict  __sockaddr_ax25__ ;
   struct sockaddr_dl  const  * __restrict  __sockaddr_dl__ ;
   struct sockaddr_eon  const  * __restrict  __sockaddr_eon__ ;
   struct sockaddr_in  const  * __restrict  __sockaddr_in__ ;
   struct sockaddr_in6  const  * __restrict  __sockaddr_in6__ ;
   struct sockaddr_inarp  const  * __restrict  __sockaddr_inarp__ ;
   struct sockaddr_ipx  const  * __restrict  __sockaddr_ipx__ ;
   struct sockaddr_iso  const  * __restrict  __sockaddr_iso__ ;
   struct sockaddr_ns  const  * __restrict  __sockaddr_ns__ ;
   struct sockaddr_un  const  * __restrict  __sockaddr_un__ ;
   struct sockaddr_x25  const  * __restrict  __sockaddr_x25__ ;
};
union __anonunion___SOCKADDR_ARG_66;
typedef union __anonunion___SOCKADDR_ARG_66  __attribute__((__transparent_union__)) __SOCKADDR_ARG;
union __anonunion___SOCKADDR_ARG_66 {
   struct sockaddr * __restrict  __sockaddr__ ;
   struct sockaddr_at * __restrict  __sockaddr_at__ ;
   struct sockaddr_ax25 * __restrict  __sockaddr_ax25__ ;
   struct sockaddr_dl * __restrict  __sockaddr_dl__ ;
   struct sockaddr_eon * __restrict  __sockaddr_eon__ ;
   struct sockaddr_in * __restrict  __sockaddr_in__ ;
   struct sockaddr_in6 * __restrict  __sockaddr_in6__ ;
   struct sockaddr_inarp * __restrict  __sockaddr_inarp__ ;
   struct sockaddr_ipx * __restrict  __sockaddr_ipx__ ;
   struct sockaddr_iso * __restrict  __sockaddr_iso__ ;
   struct sockaddr_ns * __restrict  __sockaddr_ns__ ;
   struct sockaddr_un * __restrict  __sockaddr_un__ ;
   struct sockaddr_x25 * __restrict  __sockaddr_x25__ ;
};
enum __anonenum_65 {
    SHUT_RD = 0,
    SHUT_WR = 1,
    SHUT_RDWR = 2
} ;
enum __anonenum_65;
struct osockaddr {
   unsigned short sa_family ;
   unsigned char sa_data[14] ;
};
struct osockaddr;
struct linger {
   int l_onoff ;
   int l_linger ;
};
struct linger;
struct ucred {
   pid_t pid ;
   uid_t uid ;
   gid_t gid ;
};
struct ucred;
enum __anonenum_64 {
    SCM_RIGHTS = 1,
    SCM_CREDENTIALS = 2
} ;
enum __anonenum_64;
struct mmsghdr {
   struct msghdr msg_hdr ;
   unsigned int msg_len ;
};
enum __anonenum_63 {
    MSG_OOB = 1,
    MSG_PEEK = 2,
    MSG_DONTROUTE = 4,
    MSG_TRYHARD = 4,
    MSG_CTRUNC = 8,
    MSG_PROXY = 16,
    MSG_TRUNC = 32,
    MSG_DONTWAIT = 64,
    MSG_EOR = 128,
    MSG_WAITALL = 256,
    MSG_FIN = 512,
    MSG_SYN = 1024,
    MSG_CONFIRM = 2048,
    MSG_RST = 4096,
    MSG_ERRQUEUE = 8192,
    MSG_NOSIGNAL = 16384,
    MSG_MORE = 32768,
    MSG_WAITFORONE = 65536,
    MSG_CMSG_CLOEXEC = 1073741824
} ;
enum __anonenum_63;
typedef unsigned short sa_family_t;
struct sockaddr_storage {
   sa_family_t ss_family ;
   unsigned long __ss_align ;
   char __ss_padding[128U - 2U * sizeof(unsigned long )] ;
};
struct sockaddr_storage;
struct sockaddr {
   sa_family_t sa_family ;
   char sa_data[14] ;
};
enum __socket_type {
    SOCK_STREAM = 1,
    SOCK_DGRAM = 2,
    SOCK_RAW = 3,
    SOCK_RDM = 4,
    SOCK_SEQPACKET = 5,
    SOCK_DCCP = 6,
    SOCK_PACKET = 10,
    SOCK_CLOEXEC = 524288,
    SOCK_NONBLOCK = 2048
} ;
enum __socket_type;
struct iovec {
   void *iov_base ;
   size_t iov_len ;
};
struct _php_stream_filter_chain {
   php_stream_filter *head ;
   php_stream_filter *tail ;
   php_stream *stream ;
};
struct _php_stream {
   php_stream_ops *ops ;
   void *abstract ;
   php_stream_filter_chain readfilters ;
   php_stream_filter_chain writefilters ;
   php_stream_wrapper *wrapper ;
   void *wrapperthis ;
   zval *wrapperdata ;
   int fgetss_state ;
   int is_persistent ;
   char mode[16] ;
   int rsrc_id ;
   int in_free ;
   int fclose_stdiocast ;
   FILE *stdiocast ;
   char *orig_path ;
   php_stream_context *context ;
   int flags ;
   off_t position ;
   unsigned char *readbuf ;
   size_t readbuflen ;
   off_t readpos ;
   off_t writepos ;
   size_t chunk_size ;
   int eof ;
   struct _php_stream *enclosing_stream ;
};
struct _php_stream_wrapper {
   php_stream_wrapper_ops *wops ;
   void *abstract ;
   int is_url ;
};
struct _php_stream_wrapper_ops {
   php_stream *(*stream_opener)(php_stream_wrapper *wrapper , char *filename ,
                                char *mode , int options , char **opened_path ,
                                php_stream_context *context ) ;
   int (*stream_closer)(php_stream_wrapper *wrapper , php_stream *stream ) ;
   int (*stream_stat)(php_stream_wrapper *wrapper , php_stream *stream ,
                      php_stream_statbuf *ssb ) ;
   int (*url_stat)(php_stream_wrapper *wrapper , char *url , int flags ,
                   php_stream_statbuf *ssb , php_stream_context *context ) ;
   php_stream *(*dir_opener)(php_stream_wrapper *wrapper , char *filename ,
                             char *mode , int options , char **opened_path ,
                             php_stream_context *context ) ;
   char const   *label ;
   int (*unlink)(php_stream_wrapper *wrapper , char *url , int options ,
                 php_stream_context *context ) ;
   int (*rename)(php_stream_wrapper *wrapper , char *url_from , char *url_to ,
                 int options , php_stream_context *context ) ;
   int (*stream_mkdir)(php_stream_wrapper *wrapper , char *url , int mode ,
                       int options , php_stream_context *context ) ;
   int (*stream_rmdir)(php_stream_wrapper *wrapper , char *url , int options ,
                       php_stream_context *context ) ;
   int (*stream_metadata)(php_stream_wrapper *wrapper , char *url ,
                          int options , void *value ,
                          php_stream_context *context ) ;
};
struct _php_stream_ops {
   size_t (*write)(php_stream *stream , char const   *buf , size_t count ) ;
   size_t (*read)(php_stream *stream , char *buf , size_t count ) ;
   int (*close)(php_stream *stream , int close_handle ) ;
   int (*flush)(php_stream *stream ) ;
   char const   *label ;
   int (*seek)(php_stream *stream , off_t offset , int whence ,
               off_t *newoffset ) ;
   int (*cast)(php_stream *stream , int castas , void **ret ) ;
   int (*stat)(php_stream *stream , php_stream_statbuf *ssb ) ;
   int (*set_option)(php_stream *stream , int option , int value ,
                     void *ptrparam ) ;
};
struct _php_stream_dirent {
   char d_name[4096] ;
};
struct _php_stream_filter_factory {
   php_stream_filter *(*create_filter)(char const   *filtername ,
                                       zval *filterparams , int persistent ) ;
};
struct _php_stream_bucket_brigade {
   php_stream_bucket *head ;
   php_stream_bucket *tail ;
};
struct _php_stream_filter {
   php_stream_filter_ops *fops ;
   void *abstract ;
   php_stream_filter *next ;
   php_stream_filter *prev ;
   int is_persistent ;
   php_stream_filter_chain *chain ;
   php_stream_bucket_brigade buffer ;
   int rsrc_id ;
};
enum __anonenum_php_stream_filter_status_t_62 {
    PSFS_ERR_FATAL = 0,
    PSFS_FEED_ME = 1,
    PSFS_PASS_ON = 2
} ;
typedef enum __anonenum_php_stream_filter_status_t_62 php_stream_filter_status_t;
struct _php_stream_filter_ops {
   php_stream_filter_status_t (*filter)(php_stream *stream ,
                                        php_stream_filter *thisfilter ,
                                        php_stream_bucket_brigade *buckets_in ,
                                        php_stream_bucket_brigade *buckets_out ,
                                        size_t *bytes_consumed , int flags ) ;
   void (*dtor)(php_stream_filter *thisfilter ) ;
   char const   *label ;
};
enum __anonenum_php_stream_filter_status_t_62;
struct _php_stream_bucket {
   php_stream_bucket *next ;
   php_stream_bucket *prev ;
   php_stream_bucket_brigade *brigade ;
   char *buf ;
   size_t buflen ;
   int own_buf ;
   int is_persistent ;
   int refcount ;
};
struct _php_stream_context {
   php_stream_notifier *notifier ;
   zval *options ;
   int rsrc_id ;
};
struct _php_stream_notifier {
   void (*func)(php_stream_context *context , int notifycode , int severity ,
                char *xmsg , int xcode , size_t bytes_sofar , size_t bytes_max ,
                void *ptr ) ;
   void (*dtor)(php_stream_notifier *notifier ) ;
   void *ptr ;
   int mask ;
   size_t progress ;
   size_t progress_max ;
};
typedef void (*php_stream_notification_func)(php_stream_context *context ,
                                             int notifycode , int severity ,
                                             char *xmsg , int xcode ,
                                             size_t bytes_sofar ,
                                             size_t bytes_max , void *ptr );
typedef __quad_t __blkcnt64_t;
struct stat64 {
   __dev_t st_dev ;
   unsigned int __pad1 ;
   __ino_t __st_ino ;
   __mode_t st_mode ;
   __nlink_t st_nlink ;
   __uid_t st_uid ;
   __gid_t st_gid ;
   __dev_t st_rdev ;
   unsigned int __pad2 ;
   __off64_t st_size ;
   __blksize_t st_blksize ;
   __blkcnt64_t st_blocks ;
   struct timespec st_atim ;
   struct timespec st_mtim ;
   struct timespec st_ctim ;
   __ino64_t st_ino ;
};
typedef long __suseconds_t;
struct timeval {
   __time_t tv_sec ;
   __suseconds_t tv_usec ;
};
struct itimerval {
   struct timeval it_interval ;
   struct timeval it_value ;
};
enum __itimer_which {
    ITIMER_REAL = 0,
    ITIMER_VIRTUAL = 1,
    ITIMER_PROF = 2
} ;
struct timezone {
   int tz_minuteswest ;
   int tz_dsttime ;
};
struct _zend_output_globals;
typedef struct _zend_output_globals zend_output_globals;
struct _zend_output_globals {
   int flags ;
   zend_stack handlers ;
   php_output_handler *active ;
   php_output_handler *running ;
   char const   *output_start_filename ;
   int output_start_lineno ;
};
struct _php_output_handler_user_func_t;
typedef struct _php_output_handler_user_func_t php_output_handler_user_func_t;
union __anonunion_func_61 {
   php_output_handler_user_func_t *user ;
   int (*internal)(void **handler_context , php_output_context *output_context ) ;
};
struct _php_output_buffer {
   char *data ;
   size_t size ;
   size_t used ;
   uint free : 1 ;
   uint _res : 31 ;
};
typedef struct _php_output_buffer php_output_buffer;
struct _php_output_handler {
   char *name ;
   size_t name_len ;
   int flags ;
   int level ;
   size_t size ;
   php_output_buffer buffer ;
   void *opaq ;
   void (*dtor)(void *opaq ) ;
   union __anonunion_func_61 func ;
};
union __anonunion_func_61;
struct _php_output_handler_user_func_t {
   zend_fcall_info fci ;
   zend_fcall_info_cache fcc ;
   zval *zoh ;
};
typedef int (*php_output_handler_conflict_check_t)(char const   *handler_name ,
                                                   size_t handler_name_len );
typedef void (*php_output_handler_context_dtor_t)(void *opaq );
typedef int (*php_output_handler_context_func_t)(void **handler_context ,
                                                 php_output_context *output_context );
typedef void (*php_output_handler_func_t)(char *output , uint output_len ,
                                          char **handled_output ,
                                          uint *handled_output_len , int mode );
struct _php_output_context {
   int op ;
   php_output_buffer in ;
   php_output_buffer out ;
};
struct _php_output_buffer;
enum _php_output_handler_hook_t {
    PHP_OUTPUT_HANDLER_HOOK_GET_OPAQ = 0,
    PHP_OUTPUT_HANDLER_HOOK_GET_FLAGS = 1,
    PHP_OUTPUT_HANDLER_HOOK_GET_LEVEL = 2,
    PHP_OUTPUT_HANDLER_HOOK_IMMUTABLE = 3,
    PHP_OUTPUT_HANDLER_HOOK_DISABLE = 4,
    PHP_OUTPUT_HANDLER_HOOK_LAST = 5
} ;
enum _php_output_handler_status_t;
typedef enum _php_output_handler_status_t php_output_handler_status_t;
enum _php_output_handler_status_t {
    PHP_OUTPUT_HANDLER_FAILURE = 0,
    PHP_OUTPUT_HANDLER_SUCCESS = 1,
    PHP_OUTPUT_HANDLER_NO_DATA = 2
} ;
enum __anonenum_length_modifier_e_60;
typedef enum __anonenum_length_modifier_e_60 length_modifier_e;
enum __anonenum_length_modifier_e_60 {
    LM_STD = 0,
    LM_INTMAX_T = 1,
    LM_PTRDIFF_T = 2,
    LM_LONG_LONG = 3,
    LM_SIZE_T = 4,
    LM_LONG = 5,
    LM_LONG_DOUBLE = 6
} ;
enum __anonenum_boolean_e_59 {
    NO = 0,
    YES = 1
} ;
struct passwd {
   char *pw_name ;
   char *pw_passwd ;
   __uid_t pw_uid ;
   __gid_t pw_gid ;
   char *pw_gecos ;
   char *pw_dir ;
   char *pw_shell ;
};
enum __anonenum_58 {
    _CS_PATH = 0,
    _CS_V6_WIDTH_RESTRICTED_ENVS = 1,
    _CS_GNU_LIBC_VERSION = 2,
    _CS_GNU_LIBPTHREAD_VERSION = 3,
    _CS_V5_WIDTH_RESTRICTED_ENVS = 4,
    _CS_V7_WIDTH_RESTRICTED_ENVS = 5,
    _CS_LFS_CFLAGS = 1000,
    _CS_LFS_LDFLAGS = 1001,
    _CS_LFS_LIBS = 1002,
    _CS_LFS_LINTFLAGS = 1003,
    _CS_LFS64_CFLAGS = 1004,
    _CS_LFS64_LDFLAGS = 1005,
    _CS_LFS64_LIBS = 1006,
    _CS_LFS64_LINTFLAGS = 1007,
    _CS_XBS5_ILP32_OFF32_CFLAGS = 1100,
    _CS_XBS5_ILP32_OFF32_LDFLAGS = 1101,
    _CS_XBS5_ILP32_OFF32_LIBS = 1102,
    _CS_XBS5_ILP32_OFF32_LINTFLAGS = 1103,
    _CS_XBS5_ILP32_OFFBIG_CFLAGS = 1104,
    _CS_XBS5_ILP32_OFFBIG_LDFLAGS = 1105,
    _CS_XBS5_ILP32_OFFBIG_LIBS = 1106,
    _CS_XBS5_ILP32_OFFBIG_LINTFLAGS = 1107,
    _CS_XBS5_LP64_OFF64_CFLAGS = 1108,
    _CS_XBS5_LP64_OFF64_LDFLAGS = 1109,
    _CS_XBS5_LP64_OFF64_LIBS = 1110,
    _CS_XBS5_LP64_OFF64_LINTFLAGS = 1111,
    _CS_XBS5_LPBIG_OFFBIG_CFLAGS = 1112,
    _CS_XBS5_LPBIG_OFFBIG_LDFLAGS = 1113,
    _CS_XBS5_LPBIG_OFFBIG_LIBS = 1114,
    _CS_XBS5_LPBIG_OFFBIG_LINTFLAGS = 1115,
    _CS_POSIX_V6_ILP32_OFF32_CFLAGS = 1116,
    _CS_POSIX_V6_ILP32_OFF32_LDFLAGS = 1117,
    _CS_POSIX_V6_ILP32_OFF32_LIBS = 1118,
    _CS_POSIX_V6_ILP32_OFF32_LINTFLAGS = 1119,
    _CS_POSIX_V6_ILP32_OFFBIG_CFLAGS = 1120,
    _CS_POSIX_V6_ILP32_OFFBIG_LDFLAGS = 1121,
    _CS_POSIX_V6_ILP32_OFFBIG_LIBS = 1122,
    _CS_POSIX_V6_ILP32_OFFBIG_LINTFLAGS = 1123,
    _CS_POSIX_V6_LP64_OFF64_CFLAGS = 1124,
    _CS_POSIX_V6_LP64_OFF64_LDFLAGS = 1125,
    _CS_POSIX_V6_LP64_OFF64_LIBS = 1126,
    _CS_POSIX_V6_LP64_OFF64_LINTFLAGS = 1127,
    _CS_POSIX_V6_LPBIG_OFFBIG_CFLAGS = 1128,
    _CS_POSIX_V6_LPBIG_OFFBIG_LDFLAGS = 1129,
    _CS_POSIX_V6_LPBIG_OFFBIG_LIBS = 1130,
    _CS_POSIX_V6_LPBIG_OFFBIG_LINTFLAGS = 1131,
    _CS_POSIX_V7_ILP32_OFF32_CFLAGS = 1132,
    _CS_POSIX_V7_ILP32_OFF32_LDFLAGS = 1133,
    _CS_POSIX_V7_ILP32_OFF32_LIBS = 1134,
    _CS_POSIX_V7_ILP32_OFF32_LINTFLAGS = 1135,
    _CS_POSIX_V7_ILP32_OFFBIG_CFLAGS = 1136,
    _CS_POSIX_V7_ILP32_OFFBIG_LDFLAGS = 1137,
    _CS_POSIX_V7_ILP32_OFFBIG_LIBS = 1138,
    _CS_POSIX_V7_ILP32_OFFBIG_LINTFLAGS = 1139,
    _CS_POSIX_V7_LP64_OFF64_CFLAGS = 1140,
    _CS_POSIX_V7_LP64_OFF64_LDFLAGS = 1141,
    _CS_POSIX_V7_LP64_OFF64_LIBS = 1142,
    _CS_POSIX_V7_LP64_OFF64_LINTFLAGS = 1143,
    _CS_POSIX_V7_LPBIG_OFFBIG_CFLAGS = 1144,
    _CS_POSIX_V7_LPBIG_OFFBIG_LDFLAGS = 1145,
    _CS_POSIX_V7_LPBIG_OFFBIG_LIBS = 1146,
    _CS_POSIX_V7_LPBIG_OFFBIG_LINTFLAGS = 1147,
    _CS_V6_ENV = 1148,
    _CS_V7_ENV = 1149
} ;
enum __anonenum_58;
enum __anonenum_57 {
    _SC_ARG_MAX = 0,
    _SC_CHILD_MAX = 1,
    _SC_CLK_TCK = 2,
    _SC_NGROUPS_MAX = 3,
    _SC_OPEN_MAX = 4,
    _SC_STREAM_MAX = 5,
    _SC_TZNAME_MAX = 6,
    _SC_JOB_CONTROL = 7,
    _SC_SAVED_IDS = 8,
    _SC_REALTIME_SIGNALS = 9,
    _SC_PRIORITY_SCHEDULING = 10,
    _SC_TIMERS = 11,
    _SC_ASYNCHRONOUS_IO = 12,
    _SC_PRIORITIZED_IO = 13,
    _SC_SYNCHRONIZED_IO = 14,
    _SC_FSYNC = 15,
    _SC_MAPPED_FILES = 16,
    _SC_MEMLOCK = 17,
    _SC_MEMLOCK_RANGE = 18,
    _SC_MEMORY_PROTECTION = 19,
    _SC_MESSAGE_PASSING = 20,
    _SC_SEMAPHORES = 21,
    _SC_SHARED_MEMORY_OBJECTS = 22,
    _SC_AIO_LISTIO_MAX = 23,
    _SC_AIO_MAX = 24,
    _SC_AIO_PRIO_DELTA_MAX = 25,
    _SC_DELAYTIMER_MAX = 26,
    _SC_MQ_OPEN_MAX = 27,
    _SC_MQ_PRIO_MAX = 28,
    _SC_VERSION = 29,
    _SC_PAGESIZE = 30,
    _SC_RTSIG_MAX = 31,
    _SC_SEM_NSEMS_MAX = 32,
    _SC_SEM_VALUE_MAX = 33,
    _SC_SIGQUEUE_MAX = 34,
    _SC_TIMER_MAX = 35,
    _SC_BC_BASE_MAX = 36,
    _SC_BC_DIM_MAX = 37,
    _SC_BC_SCALE_MAX = 38,
    _SC_BC_STRING_MAX = 39,
    _SC_COLL_WEIGHTS_MAX = 40,
    _SC_EQUIV_CLASS_MAX = 41,
    _SC_EXPR_NEST_MAX = 42,
    _SC_LINE_MAX = 43,
    _SC_RE_DUP_MAX = 44,
    _SC_CHARCLASS_NAME_MAX = 45,
    _SC_2_VERSION = 46,
    _SC_2_C_BIND = 47,
    _SC_2_C_DEV = 48,
    _SC_2_FORT_DEV = 49,
    _SC_2_FORT_RUN = 50,
    _SC_2_SW_DEV = 51,
    _SC_2_LOCALEDEF = 52,
    _SC_PII = 53,
    _SC_PII_XTI = 54,
    _SC_PII_SOCKET = 55,
    _SC_PII_INTERNET = 56,
    _SC_PII_OSI = 57,
    _SC_POLL = 58,
    _SC_SELECT = 59,
    _SC_UIO_MAXIOV = 60,
    _SC_IOV_MAX = 60,
    _SC_PII_INTERNET_STREAM = 61,
    _SC_PII_INTERNET_DGRAM = 62,
    _SC_PII_OSI_COTS = 63,
    _SC_PII_OSI_CLTS = 64,
    _SC_PII_OSI_M = 65,
    _SC_T_IOV_MAX = 66,
    _SC_THREADS = 67,
    _SC_THREAD_SAFE_FUNCTIONS = 68,
    _SC_GETGR_R_SIZE_MAX = 69,
    _SC_GETPW_R_SIZE_MAX = 70,
    _SC_LOGIN_NAME_MAX = 71,
    _SC_TTY_NAME_MAX = 72,
    _SC_THREAD_DESTRUCTOR_ITERATIONS = 73,
    _SC_THREAD_KEYS_MAX = 74,
    _SC_THREAD_STACK_MIN = 75,
    _SC_THREAD_THREADS_MAX = 76,
    _SC_THREAD_ATTR_STACKADDR = 77,
    _SC_THREAD_ATTR_STACKSIZE = 78,
    _SC_THREAD_PRIORITY_SCHEDULING = 79,
    _SC_THREAD_PRIO_INHERIT = 80,
    _SC_THREAD_PRIO_PROTECT = 81,
    _SC_THREAD_PROCESS_SHARED = 82,
    _SC_NPROCESSORS_CONF = 83,
    _SC_NPROCESSORS_ONLN = 84,
    _SC_PHYS_PAGES = 85,
    _SC_AVPHYS_PAGES = 86,
    _SC_ATEXIT_MAX = 87,
    _SC_PASS_MAX = 88,
    _SC_XOPEN_VERSION = 89,
    _SC_XOPEN_XCU_VERSION = 90,
    _SC_XOPEN_UNIX = 91,
    _SC_XOPEN_CRYPT = 92,
    _SC_XOPEN_ENH_I18N = 93,
    _SC_XOPEN_SHM = 94,
    _SC_2_CHAR_TERM = 95,
    _SC_2_C_VERSION = 96,
    _SC_2_UPE = 97,
    _SC_XOPEN_XPG2 = 98,
    _SC_XOPEN_XPG3 = 99,
    _SC_XOPEN_XPG4 = 100,
    _SC_CHAR_BIT = 101,
    _SC_CHAR_MAX = 102,
    _SC_CHAR_MIN = 103,
    _SC_INT_MAX = 104,
    _SC_INT_MIN = 105,
    _SC_LONG_BIT = 106,
    _SC_WORD_BIT = 107,
    _SC_MB_LEN_MAX = 108,
    _SC_NZERO = 109,
    _SC_SSIZE_MAX = 110,
    _SC_SCHAR_MAX = 111,
    _SC_SCHAR_MIN = 112,
    _SC_SHRT_MAX = 113,
    _SC_SHRT_MIN = 114,
    _SC_UCHAR_MAX = 115,
    _SC_UINT_MAX = 116,
    _SC_ULONG_MAX = 117,
    _SC_USHRT_MAX = 118,
    _SC_NL_ARGMAX = 119,
    _SC_NL_LANGMAX = 120,
    _SC_NL_MSGMAX = 121,
    _SC_NL_NMAX = 122,
    _SC_NL_SETMAX = 123,
    _SC_NL_TEXTMAX = 124,
    _SC_XBS5_ILP32_OFF32 = 125,
    _SC_XBS5_ILP32_OFFBIG = 126,
    _SC_XBS5_LP64_OFF64 = 127,
    _SC_XBS5_LPBIG_OFFBIG = 128,
    _SC_XOPEN_LEGACY = 129,
    _SC_XOPEN_REALTIME = 130,
    _SC_XOPEN_REALTIME_THREADS = 131,
    _SC_ADVISORY_INFO = 132,
    _SC_BARRIERS = 133,
    _SC_BASE = 134,
    _SC_C_LANG_SUPPORT = 135,
    _SC_C_LANG_SUPPORT_R = 136,
    _SC_CLOCK_SELECTION = 137,
    _SC_CPUTIME = 138,
    _SC_THREAD_CPUTIME = 139,
    _SC_DEVICE_IO = 140,
    _SC_DEVICE_SPECIFIC = 141,
    _SC_DEVICE_SPECIFIC_R = 142,
    _SC_FD_MGMT = 143,
    _SC_FIFO = 144,
    _SC_PIPE = 145,
    _SC_FILE_ATTRIBUTES = 146,
    _SC_FILE_LOCKING = 147,
    _SC_FILE_SYSTEM = 148,
    _SC_MONOTONIC_CLOCK = 149,
    _SC_MULTI_PROCESS = 150,
    _SC_SINGLE_PROCESS = 151,
    _SC_NETWORKING = 152,
    _SC_READER_WRITER_LOCKS = 153,
    _SC_SPIN_LOCKS = 154,
    _SC_REGEXP = 155,
    _SC_REGEX_VERSION = 156,
    _SC_SHELL = 157,
    _SC_SIGNALS = 158,
    _SC_SPAWN = 159,
    _SC_SPORADIC_SERVER = 160,
    _SC_THREAD_SPORADIC_SERVER = 161,
    _SC_SYSTEM_DATABASE = 162,
    _SC_SYSTEM_DATABASE_R = 163,
    _SC_TIMEOUTS = 164,
    _SC_TYPED_MEMORY_OBJECTS = 165,
    _SC_USER_GROUPS = 166,
    _SC_USER_GROUPS_R = 167,
    _SC_2_PBS = 168,
    _SC_2_PBS_ACCOUNTING = 169,
    _SC_2_PBS_LOCATE = 170,
    _SC_2_PBS_MESSAGE = 171,
    _SC_2_PBS_TRACK = 172,
    _SC_SYMLOOP_MAX = 173,
    _SC_STREAMS = 174,
    _SC_2_PBS_CHECKPOINT = 175,
    _SC_V6_ILP32_OFF32 = 176,
    _SC_V6_ILP32_OFFBIG = 177,
    _SC_V6_LP64_OFF64 = 178,
    _SC_V6_LPBIG_OFFBIG = 179,
    _SC_HOST_NAME_MAX = 180,
    _SC_TRACE = 181,
    _SC_TRACE_EVENT_FILTER = 182,
    _SC_TRACE_INHERIT = 183,
    _SC_TRACE_LOG = 184,
    _SC_LEVEL1_ICACHE_SIZE = 185,
    _SC_LEVEL1_ICACHE_ASSOC = 186,
    _SC_LEVEL1_ICACHE_LINESIZE = 187,
    _SC_LEVEL1_DCACHE_SIZE = 188,
    _SC_LEVEL1_DCACHE_ASSOC = 189,
    _SC_LEVEL1_DCACHE_LINESIZE = 190,
    _SC_LEVEL2_CACHE_SIZE = 191,
    _SC_LEVEL2_CACHE_ASSOC = 192,
    _SC_LEVEL2_CACHE_LINESIZE = 193,
    _SC_LEVEL3_CACHE_SIZE = 194,
    _SC_LEVEL3_CACHE_ASSOC = 195,
    _SC_LEVEL3_CACHE_LINESIZE = 196,
    _SC_LEVEL4_CACHE_SIZE = 197,
    _SC_LEVEL4_CACHE_ASSOC = 198,
    _SC_LEVEL4_CACHE_LINESIZE = 199,
    _SC_IPV6 = 235,
    _SC_RAW_SOCKETS = 236,
    _SC_V7_ILP32_OFF32 = 237,
    _SC_V7_ILP32_OFFBIG = 238,
    _SC_V7_LP64_OFF64 = 239,
    _SC_V7_LPBIG_OFFBIG = 240,
    _SC_SS_REPL_MAX = 241,
    _SC_TRACE_EVENT_NAME_MAX = 242,
    _SC_TRACE_NAME_MAX = 243,
    _SC_TRACE_SYS_MAX = 244,
    _SC_TRACE_USER_EVENT_MAX = 245,
    _SC_XOPEN_STREAMS = 246,
    _SC_THREAD_ROBUST_PRIO_INHERIT = 247,
    _SC_THREAD_ROBUST_PRIO_PROTECT = 248
} ;
enum __anonenum_57;
enum __anonenum_56 {
    _PC_LINK_MAX = 0,
    _PC_MAX_CANON = 1,
    _PC_MAX_INPUT = 2,
    _PC_NAME_MAX = 3,
    _PC_PATH_MAX = 4,
    _PC_PIPE_BUF = 5,
    _PC_CHOWN_RESTRICTED = 6,
    _PC_NO_TRUNC = 7,
    _PC_VDISABLE = 8,
    _PC_SYNC_IO = 9,
    _PC_ASYNC_IO = 10,
    _PC_PRIO_IO = 11,
    _PC_SOCK_MAXBUF = 12,
    _PC_FILESIZEBITS = 13,
    _PC_REC_INCR_XFER_SIZE = 14,
    _PC_REC_MAX_XFER_SIZE = 15,
    _PC_REC_MIN_XFER_SIZE = 16,
    _PC_REC_XFER_ALIGN = 17,
    _PC_ALLOC_SIZE_MIN = 18,
    _PC_SYMLINK_MAX = 19,
    _PC_2_SYMLINKS = 20
} ;
enum __anonenum_56;
enum __anonenum_55 {
    _ISupper = 256,
    _ISlower = 512,
    _ISalpha = 1024,
    _ISdigit = 2048,
    _ISxdigit = 4096,
    _ISspace = 8192,
    _ISprint = 16384,
    _ISgraph = 32768,
    _ISblank = 1,
    _IScntrl = 2,
    _ISpunct = 4,
    _ISalnum = 8
} ;
enum __anonenum_55;
struct _zend_free_op {
   zval *var ;
};
struct _HashPointer {
   HashPosition pos ;
   ulong h ;
};
struct __anonstruct_fe_54 {
   zval **ptr_ptr ;
   zval *ptr ;
   HashPointer fe_pos ;
};
struct __anonstruct_str_offset_53 {
   zval **ptr_ptr ;
   zval *str ;
   zend_uint offset ;
};
struct __anonstruct_var_52 {
   zval **ptr_ptr ;
   zval *ptr ;
   zend_bool fcall_returned_reference ;
};
union _temp_variable {
   zval tmp_var ;
   struct __anonstruct_var_52 var ;
   struct __anonstruct_str_offset_53 str_offset ;
   struct __anonstruct_fe_54 fe ;
   zend_class_entry *class_entry ;
};
struct __anonstruct_fe_54;
struct __anonstruct_str_offset_53;
struct __anonstruct_var_52;
struct _zend_rsrc_list_dtors_entry;
typedef struct _zend_rsrc_list_dtors_entry zend_rsrc_list_dtors_entry;
struct _zend_rsrc_list_dtors_entry {
   void (*list_dtor)(void * ) ;
   void (*plist_dtor)(void * ) ;
   void (*list_dtor_ex)(zend_rsrc_list_entry *rsrc ) ;
   void (*plist_dtor_ex)(zend_rsrc_list_entry *rsrc ) ;
   char const   *type_name ;
   int module_number ;
   int resource_id ;
   unsigned char type ;
};
typedef void (*rsrc_dtor_func_t)(zend_rsrc_list_entry *rsrc );
struct _zend_rsrc_list_entry {
   void *ptr ;
   int type ;
   int refcount ;
};
struct _zend_module_dep {
   char const   *name ;
   char const   *rel ;
   char const   *version ;
   unsigned char type ;
};
struct _zend_module_dep;
struct _zend_module_entry {
   unsigned short size ;
   unsigned int zend_api ;
   unsigned char zend_debug ;
   unsigned char zts ;
   struct _zend_ini_entry  const  *ini_entry ;
   struct _zend_module_dep  const  *deps ;
   char const   *name ;
   struct _zend_function_entry  const  *functions ;
   int (*module_startup_func)(int type , int module_number ) ;
   int (*module_shutdown_func)(int type , int module_number ) ;
   int (*request_startup_func)(int type , int module_number ) ;
   int (*request_shutdown_func)(int type , int module_number ) ;
   void (*info_func)(zend_module_entry *zend_module ) ;
   char const   *version ;
   size_t globals_size ;
   void *globals_ptr ;
   void (*globals_ctor)(void *global ) ;
   void (*globals_dtor)(void *global ) ;
   int (*post_deactivate_func)(void) ;
   int module_started ;
   unsigned char type ;
   void *handle ;
   int module_number ;
   char const   *build_id ;
};
typedef struct _zend_module_dep zend_module_dep;
struct _zend_auto_global;
typedef struct _zend_auto_global zend_auto_global;
struct _zend_auto_global {
   char const   *name ;
   uint name_len ;
   zend_bool (*auto_global_callback)(char const   *name , uint name_len ) ;
   zend_bool jit ;
   zend_bool armed ;
};
typedef zend_bool (*zend_auto_global_callback)(char const   *name ,
                                               uint name_len );
struct _zend_php_scanner_globals {
   zend_file_handle *yy_in ;
   zend_file_handle *yy_out ;
   unsigned int yy_leng ;
   unsigned char *yy_start ;
   unsigned char *yy_text ;
   unsigned char *yy_cursor ;
   unsigned char *yy_marker ;
   unsigned char *yy_limit ;
   int yy_state ;
   zend_stack state_stack ;
   unsigned char *script_org ;
   size_t script_org_size ;
   unsigned char *script_filtered ;
   size_t script_filtered_size ;
   size_t (*input_filter)(unsigned char **str , size_t *str_length ,
                          unsigned char const   *buf , size_t length ) ;
   size_t (*output_filter)(unsigned char **str , size_t *str_length ,
                           unsigned char const   *buf , size_t length ) ;
   zend_encoding const   *script_encoding ;
};
struct _zend_ini_scanner_globals {
   zend_file_handle *yy_in ;
   zend_file_handle *yy_out ;
   unsigned int yy_leng ;
   unsigned char *yy_start ;
   unsigned char *yy_text ;
   unsigned char *yy_cursor ;
   unsigned char *yy_marker ;
   unsigned char *yy_limit ;
   int yy_state ;
   zend_stack state_stack ;
   char *filename ;
   int lineno ;
   int scanner_mode ;
};
struct _zend_compiler_context {
   zend_uint opcodes_size ;
   int vars_size ;
   int literals_size ;
   int current_brk_cont ;
   int backpatch_count ;
   HashTable *labels ;
};
typedef struct _zend_compiler_context zend_compiler_context;
union __anonunion_u_49 {
   znode_op op ;
   zval constant ;
   zend_op_array *op_array ;
};
struct _znode {
   int op_type ;
   union __anonunion_u_49 u ;
   zend_uint EA ;
};
struct _zend_declarables {
   zval ticks ;
};
typedef struct _zend_declarables zend_declarables;
struct _zend_compiler_globals {
   zend_stack bp_stack ;
   zend_stack switch_cond_stack ;
   zend_stack foreach_copy_stack ;
   zend_stack object_stack ;
   zend_stack declare_stack ;
   zend_class_entry *active_class_entry ;
   zend_llist list_llist ;
   zend_llist dimension_llist ;
   zend_stack list_stack ;
   zend_stack function_call_stack ;
   char *compiled_filename ;
   int zend_lineno ;
   char *heredoc ;
   int heredoc_len ;
   zend_op_array *active_op_array ;
   HashTable *function_table ;
   HashTable *class_table ;
   HashTable filenames_table ;
   HashTable *auto_globals ;
   zend_bool parse_error ;
   zend_bool in_compilation ;
   zend_bool short_tags ;
   zend_bool asp_tags ;
   zend_declarables declarables ;
   zend_bool unclean_shutdown ;
   zend_bool ini_parser_unbuffered_errors ;
   zend_llist open_files ;
   long catch_begin ;
   struct _zend_ini_parser_param *ini_parser_param ;
   int interactive ;
   zend_uint start_lineno ;
   zend_bool increment_lineno ;
   znode implementing_class ;
   zend_uint access_type ;
   char *doc_comment ;
   zend_uint doc_comment_len ;
   zend_uint compiler_options ;
   zval *current_namespace ;
   HashTable *current_import ;
   zend_bool in_namespace ;
   zend_bool has_bracketed_namespaces ;
   zend_compiler_context context ;
   zend_stack context_stack ;
   char *interned_strings_start ;
   char *interned_strings_end ;
   char *interned_strings_top ;
   char *interned_strings_snapshot_top ;
   HashTable interned_strings ;
   zend_encoding const   **script_encoding_list ;
   size_t script_encoding_list_size ;
   zend_bool multibyte ;
   zend_bool detect_unicode ;
   zend_bool encoding_declared ;
};
struct _zend_declarables;
struct _zend_multibyte_functions {
   char const   *provider_name ;
   zend_encoding const   *(*encoding_fetcher)(char const   *encoding_name ) ;
   char const   *(*encoding_name_getter)(zend_encoding const   *encoding ) ;
   int (*lexer_compatibility_checker)(zend_encoding const   *encoding ) ;
   zend_encoding const   *(*encoding_detector)(unsigned char const   *string ,
                                               size_t length ,
                                               zend_encoding const   **list ,
                                               size_t list_size ) ;
   size_t (*encoding_converter)(unsigned char **to , size_t *to_length ,
                                unsigned char const   *from ,
                                size_t from_length ,
                                zend_encoding const   *encoding_to ,
                                zend_encoding const   *encoding_from ) ;
   int (*encoding_list_parser)(char const   *encoding_list ,
                               size_t encoding_list_len ,
                               zend_encoding const   ***return_list ,
                               size_t *return_size , int persistent ) ;
   zend_encoding const   *(*internal_encoding_getter)(void) ;
   int (*internal_encoding_setter)(zend_encoding const   *encoding ) ;
};
typedef int (*zend_encoding_internal_encoding_setter)(zend_encoding const   *encoding );
typedef zend_encoding const   *(*zend_encoding_internal_encoding_getter)(void);
typedef int (*zend_encoding_list_parser)(char const   *encoding_list ,
                                         size_t encoding_list_len ,
                                         zend_encoding const   ***return_list ,
                                         size_t *return_size , int persistent );
typedef size_t (*zend_encoding_converter)(unsigned char **to ,
                                          size_t *to_length ,
                                          unsigned char const   *from ,
                                          size_t from_length ,
                                          zend_encoding const   *encoding_to ,
                                          zend_encoding const   *encoding_from );
typedef zend_encoding const   *(*zend_encoding_detector)(unsigned char const   *string ,
                                                         size_t length ,
                                                         zend_encoding const   **list ,
                                                         size_t list_size );
typedef int (*zend_encoding_lexer_compatibility_checker)(zend_encoding const   *encoding );
typedef char const   *(*zend_encoding_name_getter)(zend_encoding const   *encoding );
typedef zend_encoding const   *(*zend_encoding_fetcher)(char const   *encoding_name );
typedef size_t (*zend_encoding_filter)(unsigned char **str ,
                                       size_t *str_length ,
                                       unsigned char const   *buf ,
                                       size_t length );
struct __anonstruct_free_list_51 {
   int next ;
};
struct _store_object {
   void *object ;
   void (*dtor)(void *object , zend_object_handle handle ) ;
   void (*free_storage)(void *object ) ;
   void (*clone)(void *object , void **object_clone ) ;
   zend_object_handlers const   *handlers ;
   zend_uint refcount ;
   gc_root_buffer *buffered ;
};
union _store_bucket {
   struct _store_object obj ;
   struct __anonstruct_free_list_51 free_list ;
};
struct _zend_object_store_bucket {
   zend_bool destructor_called ;
   zend_bool valid ;
   union _store_bucket bucket ;
};
union _store_bucket;
struct __anonstruct_free_list_51;
struct _store_object;
typedef void (*zend_objects_store_clone_t)(void *object , void **object_clone );
typedef void (*zend_objects_free_object_storage_t)(void *object );
typedef void (*zend_objects_store_dtor_t)(void *object ,
                                          zend_object_handle handle );
typedef struct _zend_compiler_globals zend_compiler_globals;
typedef struct __jmp_buf_tag sigjmp_buf[1];
struct _list_llist_element;
typedef struct _list_llist_element list_llist_element;
struct _list_llist_element {
   znode var ;
   zend_llist dimensions ;
   znode value ;
};
struct _zend_switch_entry;
typedef struct _zend_switch_entry zend_switch_entry;
struct _zend_switch_entry {
   znode cond ;
   int default_case ;
   int control_var ;
};
struct _zend_function_state;
struct __anonstruct_common_50;
struct _zend_internal_function;
struct _zend_compiled_variable {
   char const   *name ;
   int name_len ;
   ulong hash_value ;
};
struct _zend_internal_function_info;
typedef struct _zend_internal_function_info zend_internal_function_info;
struct _zend_internal_function_info {
   char const   *_name ;
   zend_uint _name_len ;
   char const   *_class_name ;
   zend_uint required_num_args ;
   zend_uchar _type_hint ;
   zend_bool return_reference ;
   zend_bool pass_rest_by_reference ;
};
struct _zend_try_catch_element {
   zend_uint try_op ;
   zend_uint catch_op ;
};
struct _zend_label;
typedef struct _zend_label zend_label;
struct _zend_label {
   int brk_cont ;
   zend_uint opline_num ;
};
struct _zend_brk_cont_element {
   int start ;
   int cont ;
   int brk ;
   int parent ;
};
union __anonunion_u_49;
struct _zend_literal {
   zval constant ;
   zend_ulong hash_value ;
   zend_uint cache_slot ;
};
struct _zend_compiler_context;
typedef int error_t;
union __anonunion_u_46;
union __anonunion_u_45;
typedef int (*zend_write_func_t)(char const   *str , uint str_length );
struct _zend_utility_values {
   char *import_use_extension ;
   uint import_use_extension_length ;
   zend_bool html_errors ;
};
struct _zend_utility_functions {
   void ( /* format attribute */  (*error_function))(int type ,
                                                     char const   *error_filename ,
                                                     uint error_lineno ,
                                                     char const   *format ,
                                                     va_list args ) ;
   int ( /* format attribute */  (*printf_function))(char const   *format  , ...) ;
   int (*write_function)(char const   *str , uint str_length ) ;
   FILE *(*fopen_function)(char const   *filename , char **opened_path ) ;
   void (*message_handler)(long message , void const   *data ) ;
   void (*block_interruptions)(void) ;
   void (*unblock_interruptions)(void) ;
   int (*get_configuration_directive)(char const   *name , uint name_length ,
                                      zval *contents ) ;
   void (*ticks_function)(int ticks ) ;
   void (*on_timeout)(int seconds ) ;
   int (*stream_open_function)(char const   *filename ,
                               zend_file_handle *handle ) ;
   int (*vspprintf_function)(char **pbuf , size_t max_len ,
                             char const   *format , va_list ap ) ;
   char *(*getenv_function)(char *name , size_t name_len ) ;
   char *(*resolve_path_function)(char const   *filename , int filename_len ) ;
};
struct _zend_mmap {
   size_t len ;
   size_t pos ;
   void *map ;
   char *buf ;
   void *old_handle ;
   void (*old_closer)(void *handle ) ;
};
typedef struct _zend_mmap zend_mmap;
struct _zend_stream {
   void *handle ;
   int isatty ;
   zend_mmap mmap ;
   size_t (*reader)(void *handle , char *buf , size_t len ) ;
   size_t (*fsizer)(void *handle ) ;
   void (*closer)(void *handle ) ;
};
typedef struct _zend_stream zend_stream;
union __anonunion_handle_44 {
   int fd ;
   FILE *fp ;
   zend_stream stream ;
};
enum __anonenum_zend_stream_type_43 {
    ZEND_HANDLE_FILENAME = 0,
    ZEND_HANDLE_FD = 1,
    ZEND_HANDLE_FP = 2,
    ZEND_HANDLE_STREAM = 3,
    ZEND_HANDLE_MAPPED = 4
} ;
typedef enum __anonenum_zend_stream_type_43 zend_stream_type;
struct _zend_file_handle {
   zend_stream_type type ;
   char const   *filename ;
   char *opened_path ;
   union __anonunion_handle_44 handle ;
   zend_bool free_filename ;
};
union __anonunion_handle_44;
struct _zend_stream;
struct _zend_mmap;
enum __anonenum_zend_stream_type_43;
typedef void (*zend_stream_closer_t)(void *handle );
typedef size_t (*zend_stream_reader_t)(void *handle , char *buf , size_t len );
typedef size_t (*zend_stream_fsizer_t)(void *handle );
union __anonunion_info_40;
struct __anonstruct_internal_42;
struct __anonstruct_user_41;
struct _zend_trait_method_reference;
typedef struct _zend_trait_method_reference zend_trait_method_reference;
struct _zend_trait_alias {
   zend_trait_method_reference *trait_method ;
   char const   *alias ;
   unsigned int alias_len ;
   zend_uint modifiers ;
   union _zend_function *function ;
};
struct _zend_trait_precedence {
   zend_trait_method_reference *trait_method ;
   zend_class_entry **exclude_from_classes ;
   union _zend_function *function ;
};
struct _zend_trait_method_reference {
   char const   *method_name ;
   unsigned int mname_len ;
   zend_class_entry *ce ;
   char const   *class_name ;
   unsigned int cname_len ;
};
enum zend_object_iterator_kind {
    ZEND_ITER_INVALID = 0,
    ZEND_ITER_PLAIN_ARRAY = 1,
    ZEND_ITER_PLAIN_OBJECT = 2,
    ZEND_ITER_OBJECT = 3
} ;
struct _zend_class_iterator_funcs;
union _zvalue_value;
struct __anonstruct_str_39;
typedef HashTable *(*zend_object_get_gc_t)(zval *object , zval ***table ,
                                           int *n );
typedef int (*zend_object_get_closure_t)(zval *obj , zend_class_entry **ce_ptr ,
                                         union _zend_function **fptr_ptr ,
                                         zval **zobj_ptr );
typedef int (*zend_object_count_elements_t)(zval *object , long *count );
typedef int (*zend_object_cast_t)(zval *readobj , zval *retval , int type );
typedef int (*zend_object_compare_t)(zval *object1 , zval *object2 );
typedef int (*zend_object_get_class_name_t)(zval const   *object ,
                                            char const   **class_name ,
                                            zend_uint *class_name_len ,
                                            int parent );
typedef zend_class_entry *(*zend_object_get_class_entry_t)(zval const   *object );
typedef zend_object_value (*zend_object_clone_obj_t)(zval *object );
typedef void (*zend_object_delete_obj_t)(zval *object );
typedef void (*zend_object_del_ref_t)(zval *object );
typedef void (*zend_object_add_ref_t)(zval *object );
typedef union _zend_function *(*zend_object_get_constructor_t)(zval *object );
typedef union _zend_function *(*zend_object_get_method_t)(zval **object_ptr ,
                                                          char *method ,
                                                          int method_len ,
                                                          struct _zend_literal  const  *key );
typedef int (*zend_object_call_method_t)(char const   *method , int ht ,
                                         zval *return_value ,
                                         zval **return_value_ptr ,
                                         zval *this_ptr , int return_value_used );
typedef HashTable *(*zend_object_get_debug_info_t)(zval *object , int *is_temp );
typedef HashTable *(*zend_object_get_properties_t)(zval *object );
typedef void (*zend_object_unset_dimension_t)(zval *object , zval *offset );
typedef void (*zend_object_unset_property_t)(zval *object , zval *member ,
                                             struct _zend_literal  const  *key );
typedef int (*zend_object_has_dimension_t)(zval *object , zval *member ,
                                           int check_empty );
typedef int (*zend_object_has_property_t)(zval *object , zval *member ,
                                          int has_set_exists ,
                                          struct _zend_literal  const  *key );
typedef zval *(*zend_object_get_t)(zval *object );
typedef void (*zend_object_set_t)(zval **object , zval *value );
typedef zval **(*zend_object_get_property_ptr_ptr_t)(zval *object ,
                                                     zval *member ,
                                                     struct _zend_literal  const  *key );
typedef void (*zend_object_write_dimension_t)(zval *object , zval *offset ,
                                              zval *value );
typedef void (*zend_object_write_property_t)(zval *object , zval *member ,
                                             zval *value ,
                                             struct _zend_literal  const  *key );
typedef zval *(*zend_object_read_dimension_t)(zval *object , zval *offset ,
                                              int type );
typedef zval *(*zend_object_read_property_t)(zval *object , zval *member ,
                                             int type ,
                                             struct _zend_literal  const  *key );
struct _zend_guard;
typedef struct _zend_guard zend_guard;
struct _zend_guard {
   zend_bool in_get ;
   zend_bool in_set ;
   zend_bool in_unset ;
   zend_bool in_isset ;
   zend_bool dummy ;
};
typedef void (*llist_apply_func_t)(void * );
typedef void (*llist_apply_with_arg_func_t)(void *data , void *arg );
typedef void (*llist_apply_with_args_func_t)(void *data , int num_args ,
                                             va_list args );
typedef int (*llist_compare_func_t)(zend_llist_element const   ** ,
                                    zend_llist_element const   ** );
typedef void (*llist_dtor_func_t)(void * );
struct _zend_llist_element {
   struct _zend_llist_element *next ;
   struct _zend_llist_element *prev ;
   char data[1] ;
};
struct _zend_ts_hashtable {
   HashTable hash ;
   zend_uint reader ;
};
typedef int (*apply_func_args_t)(void *pDest , int num_args , va_list args ,
                                 zend_hash_key *hash_key );
typedef int (*apply_func_arg_t)(void *pDest , void *argument );
typedef int (*apply_func_t)(void *pDest );
typedef zend_bool (*merge_checker_func_t)(HashTable *target_ht ,
                                          void *source_data ,
                                          zend_hash_key *hash_key ,
                                          void *pParam );
struct _zend_hash_key {
   char const   *arKey ;
   uint nKeyLength ;
   ulong h ;
};
typedef void (*copy_ctor_param_func_t)(void *pElement , void *pParam );
typedef void (*copy_ctor_func_t)(void *pElement );
typedef void (*dtor_func_t)(void *pDest );
typedef void (*sort_func_t)(void * , size_t  , size_t  ,
                            int (*)(void const   * , void const   * ) );
typedef int (*compare_func_t)(void const   * , void const   * );
typedef ulong (*hash_func_t)(char const   *arKey , uint nKeyLength );
typedef unsigned long zend_uintptr_t;
typedef long zend_intptr_t;
typedef unsigned long long zend_ulong64;
typedef long long zend_long64;
typedef unsigned short zend_ushort;
struct _zend_mm_storage {
   zend_mm_mem_handlers const   *handlers ;
   void *data ;
};
struct _zend_mm_segment;
typedef struct _zend_mm_segment zend_mm_segment;
struct _zend_mm_mem_handlers {
   char const   *name ;
   zend_mm_storage *(*init)(void *params ) ;
   void (*dtor)(zend_mm_storage *storage ) ;
   void (*compact)(zend_mm_storage *storage ) ;
   zend_mm_segment *(*_alloc)(zend_mm_storage *storage , size_t size ) ;
   zend_mm_segment *(*_realloc)(zend_mm_storage *storage ,
                                zend_mm_segment *ptr , size_t size ) ;
   void (*_free)(zend_mm_storage *storage , zend_mm_segment *ptr ) ;
};
struct _zend_mm_segment {
   size_t size ;
   struct _zend_mm_segment *next_segment ;
};
struct _zend_leak_info;
typedef struct _zend_leak_info zend_leak_info;
struct _zend_leak_info {
   void *addr ;
   size_t size ;
   char const   *filename ;
   uint lineno ;
   char const   *orig_filename ;
   uint orig_lineno ;
};
typedef unsigned long tsrm_uintptr_t;
typedef long tsrm_intptr_t;
struct __anonstruct_Dl_serinfo_38;
typedef struct __anonstruct_Dl_serinfo_38 Dl_serinfo;
struct __anonstruct_Dl_serpath_37 {
   char *dls_name ;
   unsigned int dls_flags ;
};
typedef struct __anonstruct_Dl_serpath_37 Dl_serpath;
struct __anonstruct_Dl_serinfo_38 {
   size_t dls_size ;
   unsigned int dls_cnt ;
   Dl_serpath dls_serpath[1] ;
};
struct __anonstruct_Dl_serpath_37;
enum __anonenum_36 {
    RTLD_DI_LMID = 1,
    RTLD_DI_LINKMAP = 2,
    RTLD_DI_CONFIGADDR = 3,
    RTLD_DI_SERINFO = 4,
    RTLD_DI_SERINFOSIZE = 5,
    RTLD_DI_ORIGIN = 6,
    RTLD_DI_PROFILENAME = 7,
    RTLD_DI_PROFILEOUT = 8,
    RTLD_DI_TLS_MODID = 9,
    RTLD_DI_TLS_DATA = 10,
    RTLD_DI_MAX = 10
} ;
enum __anonenum_36;
enum __anonenum_35 {
    RTLD_DL_SYMENT = 1,
    RTLD_DL_LINKMAP = 2
} ;
enum __anonenum_35;
struct __anonstruct_Dl_info_34 {
   char const   *dli_fname ;
   void *dli_fbase ;
   char const   *dli_sname ;
   void *dli_saddr ;
};
typedef _IO_cookie_io_functions_t cookie_io_functions_t;
typedef int __io_close_fn(void *__cookie );
typedef int __io_seek_fn(void *__cookie , __off64_t *__pos , int __w );
typedef __ssize_t __io_write_fn(void *__cookie , char const   *__buf ,
                                size_t __n );
typedef __ssize_t __io_read_fn(void *__cookie , char *__buf , size_t __nbytes );
struct __anonstruct__IO_cookie_io_functions_t_33 {
   __io_read_fn *read ;
   __io_write_fn *write ;
   __io_seek_fn *seek ;
   __io_close_fn *close ;
};
typedef __io_close_fn cookie_close_function_t;
typedef __io_seek_fn cookie_seek_function_t;
typedef __io_write_fn cookie_write_function_t;
typedef __io_read_fn cookie_read_function_t;
enum __codecvt_result {
    __codecvt_ok = 0,
    __codecvt_partial = 1,
    __codecvt_error = 2,
    __codecvt_noconv = 3
} ;
enum __codecvt_result;
struct _IO_marker {
   struct _IO_marker *_next ;
   struct _IO_FILE *_sbuf ;
   int _pos ;
};
struct _IO_jump_t;
typedef unsigned int _G_uint32_t;
typedef unsigned short _G_uint16_t;
typedef int _G_int32_t;
typedef short _G_int16_t;
union __anonunion___value_30 {
   unsigned int __wch ;
   char __wchb[4] ;
};
struct __anonstruct___mbstate_t_29 {
   int __count ;
   union __anonunion___value_30 __value ;
};
struct __anonstruct__G_fpos64_t_32 {
   __off64_t __pos ;
   __mbstate_t __state ;
};
struct __anonstruct__G_fpos_t_31 {
   __off_t __pos ;
   __mbstate_t __state ;
};
union __anonunion___value_30;
union __anonunion_28;
union __anonunion___n_27;
union __anonunion___u_26;
union __anonunion___u_25;
union __anonunion___u_24;
struct exception {
   int type ;
   char *name ;
   double arg1 ;
   double arg2 ;
   double retval ;
};
enum __anonenum__LIB_VERSION_TYPE_23 {
    _IEEE_ = -1,
    _SVID_ = 0,
    _XOPEN_ = 1,
    _POSIX_ = 2,
    _ISOC_ = 3
} ;
enum __anonenum_22 {
    FP_NAN = 0,
    FP_INFINITE = 1,
    FP_ZERO = 2,
    FP_SUBNORMAL = 3,
    FP_NORMAL = 4
} ;
enum __anonenum_22;
typedef long double double_t;
typedef long double float_t;
typedef int (*__compar_d_fn_t)(void const   * , void const   * , void * );
typedef int (*comparison_fn_t)(void const   * , void const   * );
typedef int (*__compar_fn_t)(void const   * , void const   * );
struct drand48_data {
   unsigned short __x[3] ;
   unsigned short __old_x[3] ;
   unsigned short __c ;
   unsigned short __init ;
   unsigned long long __a ;
};
struct random_data {
   int32_t *fptr ;
   int32_t *rptr ;
   int32_t *state ;
   int rand_type ;
   int rand_deg ;
   int rand_sep ;
   int32_t *end_ptr ;
};
union __anonunion_pthread_barrierattr_t_21;
typedef union __anonunion_pthread_barrierattr_t_21 pthread_barrierattr_t;
union __anonunion_pthread_barrierattr_t_21 {
   char __size[4] ;
   int __align ;
};
union __anonunion_pthread_barrier_t_20;
typedef union __anonunion_pthread_barrier_t_20 pthread_barrier_t;
union __anonunion_pthread_barrier_t_20 {
   char __size[20] ;
   long __align ;
};
typedef int volatile   pthread_spinlock_t;
union __anonunion_pthread_rwlockattr_t_19;
typedef union __anonunion_pthread_rwlockattr_t_19 pthread_rwlockattr_t;
union __anonunion_pthread_rwlockattr_t_19 {
   char __size[8] ;
   long __align ;
};
union __anonunion_pthread_rwlock_t_17;
typedef union __anonunion_pthread_rwlock_t_17 pthread_rwlock_t;
struct __anonstruct___data_18 {
   int __lock ;
   unsigned int __nr_readers ;
   unsigned int __readers_wakeup ;
   unsigned int __writer_wakeup ;
   unsigned int __nr_readers_queued ;
   unsigned int __nr_writers_queued ;
   unsigned char __flags ;
   unsigned char __shared ;
   unsigned char __pad1 ;
   unsigned char __pad2 ;
   int __writer ;
};
union __anonunion_pthread_rwlock_t_17 {
   struct __anonstruct___data_18 __data ;
   char __size[32] ;
   long __align ;
};
struct __anonstruct___data_18;
typedef int pthread_once_t;
typedef unsigned int pthread_key_t;
union __anonunion_pthread_condattr_t_16;
typedef union __anonunion_pthread_condattr_t_16 pthread_condattr_t;
union __anonunion_pthread_condattr_t_16 {
   char __size[4] ;
   long __align ;
};
union __anonunion_pthread_cond_t_14;
typedef union __anonunion_pthread_cond_t_14 pthread_cond_t;
struct __anonstruct___data_15 {
   int __lock ;
   unsigned int __futex ;
   unsigned long long __total_seq ;
   unsigned long long __wakeup_seq ;
   unsigned long long __woken_seq ;
   void *__mutex ;
   unsigned int __nwaiters ;
   unsigned int __broadcast_seq ;
};
union __anonunion_pthread_cond_t_14 {
   struct __anonstruct___data_15 __data ;
   char __size[48] ;
   long long __align ;
};
struct __anonstruct___data_15;
union __anonunion_pthread_mutexattr_t_13;
typedef union __anonunion_pthread_mutexattr_t_13 pthread_mutexattr_t;
union __anonunion_pthread_mutexattr_t_13 {
   char __size[4] ;
   long __align ;
};
union __anonunion_pthread_mutex_t_11;
typedef union __anonunion_pthread_mutex_t_11 pthread_mutex_t;
struct __pthread_internal_slist {
   struct __pthread_internal_slist *__next ;
};
typedef struct __pthread_internal_slist __pthread_slist_t;
union __anonunion____missing_field_name_12 {
   int __spins ;
   __pthread_slist_t __list ;
};
struct __pthread_mutex_s {
   int __lock ;
   unsigned int __count ;
   int __owner ;
   int __kind ;
   unsigned int __nusers ;
   union __anonunion____missing_field_name_12 __annonCompField1 ;
};
union __anonunion_pthread_mutex_t_11 {
   struct __pthread_mutex_s __data ;
   char __size[24] ;
   long __align ;
};
struct __pthread_mutex_s;
union __anonunion____missing_field_name_12;
struct __pthread_internal_slist;
union __anonunion_pthread_attr_t_10;
typedef union __anonunion_pthread_attr_t_10 pthread_attr_t;
union __anonunion_pthread_attr_t_10 {
   char __size[36] ;
   long __align ;
};
typedef unsigned long pthread_t;
typedef __u_quad_t __fsfilcnt64_t;
typedef __fsfilcnt64_t fsfilcnt64_t;
typedef __u_quad_t __fsblkcnt64_t;
typedef __fsblkcnt64_t fsblkcnt64_t;
typedef __blkcnt64_t blkcnt64_t;
typedef unsigned long __fsfilcnt_t;
typedef __fsfilcnt_t fsfilcnt_t;
typedef unsigned long __fsblkcnt_t;
typedef __fsblkcnt_t fsblkcnt_t;
typedef __blkcnt_t blkcnt_t;
typedef __blksize_t blksize_t;
typedef long __fd_mask;
typedef __fd_mask fd_mask;
struct __anonstruct_fd_set_9 {
   __fd_mask fds_bits[1024 / (8 * (int )sizeof(__fd_mask ))] ;
};
typedef __sigset_t sigset_t;
typedef int __sig_atomic_t;
typedef int register_t;
typedef unsigned long long u_int64_t;
typedef unsigned int u_int32_t;
typedef unsigned short u_int16_t;
typedef unsigned char u_int8_t;
typedef long long int64_t;
typedef short int16_t;
typedef signed char int8_t;
typedef unsigned short ushort;
typedef __suseconds_t suseconds_t;
typedef __useconds_t useconds_t;
typedef int __key_t;
typedef __key_t key_t;
typedef char *__caddr_t;
typedef __caddr_t caddr_t;
typedef int __daddr_t;
typedef __daddr_t daddr_t;
typedef unsigned int __id_t;
typedef __id_t id_t;
typedef __off64_t off64_t;
typedef __nlink_t nlink_t;
typedef __dev_t dev_t;
typedef __ino64_t ino64_t;
typedef __ino_t ino_t;
typedef __off64_t __loff_t;
typedef __loff_t loff_t;
struct __anonstruct___fsid_t_7;
typedef struct __anonstruct___fsid_t_7 __fsid_t;
typedef __fsid_t fsid_t;
typedef __u_quad_t u_quad_t;
typedef __quad_t quad_t;
typedef unsigned long __u_long;
typedef __u_long u_long;
typedef unsigned int __u_int;
typedef __u_int u_int;
typedef unsigned short __u_short;
typedef __u_short u_short;
typedef unsigned char __u_char;
typedef __u_char u_char;
typedef __quad_t *__qaddr_t;
typedef long __swblk_t;
typedef __u_quad_t __rlim64_t;
typedef unsigned long __rlim_t;
struct __anonstruct___fsid_t_7 {
   int __val[2] ;
};
typedef unsigned long long __uint64_t;
typedef long long __int64_t;
typedef unsigned int __uint32_t;
typedef unsigned short __uint16_t;
typedef short __int16_t;
typedef unsigned char __uint8_t;
typedef signed char __int8_t;
typedef __locale_t locale_t;
struct __locale_data;
struct __locale_struct {
   struct __locale_data *__locales[13] ;
   unsigned short const   *__ctype_b ;
   int const   *__ctype_tolower ;
   int const   *__ctype_toupper ;
   char const   *__names[13] ;
};
struct __anonstruct_lldiv_t_6 {
   long long quot ;
   long long rem ;
};
struct __anonstruct_ldiv_t_5 {
   long quot ;
   long rem ;
};
struct __anonstruct_div_t_4 {
   int quot ;
   int rem ;
};
union __anonunion___WAIT_STATUS_3;
typedef union __anonunion___WAIT_STATUS_3  __attribute__((__transparent_union__)) __WAIT_STATUS;
union wait;
union __anonunion___WAIT_STATUS_3 {
   union wait *__uptr ;
   int *__iptr ;
};
struct __anonstruct___wait_stopped_2 {
   unsigned int __w_stopval : 8 ;
   unsigned int __w_stopsig : 8 ;
   unsigned int  : 16 ;
};
struct __anonstruct___wait_terminated_1 {
   unsigned int __w_termsig : 7 ;
   unsigned int __w_coredump : 1 ;
   unsigned int __w_retcode : 8 ;
   unsigned int  : 16 ;
};
union wait {
   int w_status ;
   struct __anonstruct___wait_terminated_1 __wait_terminated ;
   struct __anonstruct___wait_stopped_2 __wait_stopped ;
};
struct __anonstruct___wait_stopped_2;
struct __anonstruct___wait_terminated_1;
struct _IO_FILE *_coverage_fout  ;
